#include <windows.h>
#include <shellapi.h>
#include <stdio.h>
#include <time.h>
#include <conio.h>

#define PDF_STATIC // Enable static binding when compiling the project with the workspace dynapdf_static.
#include "../../../include/C_CPP/dynapdf.h"
#include "../util/absolute_path.h" // Helper function to create an absoulte file path

/*
   Note that the dynapdf.dll must be copied into the output directory or into a
   Windwos search path (e.g. %WINDOWS%/System32) before the application can be executed!
*/

// Error callback function.
SI32 PDF_CALL PDFError(const void* Data, SI32 ErrCode, const char* ErrMessage, SI32 ErrType)
{
   printf("%s\n", ErrMessage);
   return 0; // any other return value breaks processing!
}

int main(int argc, char* argv[])
{
   time_t t;
   struct tm* now;
   SI32 f, act, keyStrokeAct, keyStrokeAct2, formatAct, formatAct2;
   char filePath[512];
   char buf[64];
   char alertScript[] =
      "if (app.viewerVersion >= 7.0 && app.viewerVersion < 8.0)\n"
      "{\n"
      "app.alert(\"Due to bugs in Acrobat 7.0 character spacing does not work"
      " in text fields. We use the flag ffComb instead which is supported since"
      " Acrobat 6.0. However, if the field's format is controled by a JavaScript"
      " Action, we cannot set the flag because Acrobat 6.0 is then unable"
      " to display the field's text. Acrobat 7.0 has problems to place the"
      " the cursor in such fields but the final format will be correctly applied."
      " The problem is that if the form should work with Acrobat 6.0 we cannot set"
      " the flag ffComb to the date fields. It is also impossible to get the same"
      " result as with character spacing with the flag ffComb. So, what can we do to make the"
      " form compatible to all Acrobat versions? I really don't know! The easiest way"
      " would be to avoid the usage of such date formats.\");\n"
      "}";

   void* pdf = pdfNewPDF();
   if (!pdf) return 2; // Out of memory?
   // Error messages and warnings are passed to the callback function.
   pdfSetOnErrorProc(pdf, NULL, PDFError);
   pdfCreateNewPDF(pdf, NULL); // The output file is created later
   pdfSetDocInfo(pdf, diCreator, "C++ Example Project");
   pdfSetDocInfo(pdf, diTitle, "How to create an Interactive Form?");
   pdfSetDocInfo(pdf, diAuthor, "Jens Boschulte");
   pdfSetViewerPreferences(pdf, vpDisplayDocTitle, avNone);
   // Conversion of pages to templates is normally not required. Templates are required if
   // the page should be scaled or used multiple times in a document, e.g. as a page background.
   // See help file for further information.
   pdfSetImportFlags(pdf, ifImportAll | ifImportAsPage);
   MakeAbsolutePath(argv, "examples/test_files/taxform.pdf", filePath);
   if (pdfOpenImportFile(pdf, filePath, ptOpen, NULL) < 0)
   {
      pdfDeletePDF(pdf);
      printf("Input file \"%s\" not found!\n", filePath);
      _getch();
      return 1;
   }
   pdfImportPDFFile(pdf, 1, 1.0, 1.0);

   pdfEditPage(pdf, 1);

   pdfSetCheckBoxChar(pdf, ccCross4);
   pdfSetLineWidth(pdf, 0.0);
   pdfSetFieldBorderColor(pdf, NO_COLOR);

   f = pdfCreateRadioButton(pdf, "AntragsArt", "EinkommensSteuerErklrung", false, -1, 70, 746.5, 13.0, 13.0);
   pdfCreateCheckBox(pdf, NULL, "Sparzulage", false, f, 70, 722.5, 13.0, 13.0);
   pdfCreateCheckBox(pdf, NULL, "VerlustErklrung", false, f, 249.5, 722.5, 13.0, 13.0);

   pdfSetFont(pdf, "Courier", fsNone, 12.0, true, cp1252);
   pdfCreateTextField(pdf, "Finanzamt", -1, false, 0, 70.0, 698.0, 358.0, 14.0);
   pdfCreateTextField(pdf, "Steuernummer", -1, false, 14, 70.0, 673.5, 150.0, 14.0);
   pdfCreateTextField(pdf, "Finanzamt_alt", -1, false, 0, 272.0, 673.5, 156.0, 14.0);

   pdfCreateCheckBox(pdf, "Rckerstattung", "Ja", true, -1, 429, 675.0, 13.0, 13.0);
   pdfCreateTextField(pdf, "Telefon", -1, false, 0, 285.5, 650.0, 142.5, 14.0);
/*
   Acrobat 7.0 ignores the character spacing, the only way to get the same result is to set the Comb flag.
   See help file for further information.
*/
   pdfSetCharacterSpacing(pdf, 7.2);
   f = pdfCreateTextField(pdf, "Name", -1, false, 25, 70.0, 626.0, 358.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   f = pdfCreateTextField(pdf, "Vorname", -1, false, 25, 70.0, 602.0, 358.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   f = pdfCreateTextField(pdf, "Geburtsdatum", -1, false, 10, 70.0, 578.0, 113.0, 14.0);
   // Create a custom date format action -> input example: 13.05.1971 or 13-05-1971 or 13/05/1971
   // Two actions are required, one for the event onKeyStroke and one for the event onFormat.
   keyStrokeAct = pdfCreateJSAction(pdf, "AFDate_KeystrokeEx(\"ddmmyyyy\");");
   formatAct = pdfCreateJSAction(pdf, "AFDate_FormatEx(\"ddmmyyyy\");");

   // Add both actions to the field
   pdfAddActionToObj(pdf, otField, oeOnKeyStroke, keyStrokeAct, f);
   pdfAddActionToObj(pdf, otField, oeOnFormat, formatAct, f);

   pdfSetCharacterSpacing(pdf, 0.0);
   f = pdfCreateComboBox(pdf, "Religion", false, -1, 185.0, 578.0, 71.0, 14.0);
   pdfAddValToChoiceField(pdf, f, "keine", "keine", true);
   pdfAddValToChoiceField(pdf, f, "Evang", "Evang.", false);
   pdfAddValToChoiceField(pdf, f, "Kath", "Kath.", false);
   pdfAddValToChoiceField(pdf, f, "andere", "andere", false);

   pdfCreateTextField(pdf, "Beruf", -1, false, 0, 257.0, 578.0, 171.0, 14.0);

   pdfSetCharacterSpacing(pdf, 7.2);
   f = pdfCreateTextField(pdf, "Strasse", -1, false, 25, 70.0, 554.0, 358.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   f = pdfCreateTextField(pdf, "PLZ", -1, false, 5, 70.0, 530.0, 70.5, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   pdfSetNumberFormat(pdf, f, dsNoneDot, 0, nsRed, NULL, false);

   f = pdfCreateTextField(pdf, "Wohnort", -1, false, 20, 142.5, 530.0, 285.5, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);

   pdfSetCharacterSpacing(pdf, 0.0);
   pdfSetFont(pdf, "Courier", fsNone, 11.0, true, cp1252);

   f = pdfCreateTextField(pdf, "VerheiratetSeit", -1, false, 10, 70.0, 506.0, 70.5, 14.0);
   // We need another date format here. The date should be delimited with a dot.
   keyStrokeAct2 = pdfCreateJSAction(pdf, "AFDate_KeystrokeEx(\"dd.mm.yyyy\");");
   formatAct2 = pdfCreateJSAction(pdf, "AFDate_FormatEx(\"dd.mm.yyyy\");");
   pdfAddActionToObj(pdf, otField, oeOnKeyStroke, keyStrokeAct2, f);
   pdfAddActionToObj(pdf, otField, oeOnFormat, formatAct2, f);

   f = pdfCreateTextField(pdf, "VerwitwetSeit", -1, false, 10, 142.0, 506.0, 71.0, 14.0);
   pdfAddActionToObj(pdf, otField, oeOnKeyStroke, keyStrokeAct2, f);
   pdfAddActionToObj(pdf, otField, oeOnFormat, formatAct2, f);

   f = pdfCreateTextField(pdf, "GeschiedenSeit", -1, false, 10, 214.0, 506.0, 71.0, 14.0);
   pdfAddActionToObj(pdf, otField, oeOnKeyStroke, keyStrokeAct2, f);
   pdfAddActionToObj(pdf, otField, oeOnFormat, formatAct2, f);

   f = pdfCreateTextField(pdf, "GetrenntSeit", -1, false, 10, 286.0, 506.0, 142.0, 14.0);
   pdfAddActionToObj(pdf, otField, oeOnKeyStroke, keyStrokeAct2, f);
   pdfAddActionToObj(pdf, otField, oeOnFormat, formatAct2, f);

   pdfSetCharacterSpacing(pdf, 7.2);
   pdfSetFont(pdf, "Courier", fsNone, 12.0, true, cp1252);
   f = pdfCreateTextField(pdf, "EhefrauVorname", -1, false, 25, 70.0, 458.0, 358.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   f = pdfCreateTextField(pdf, "EhefrauName", -1, false, 25, 70.0, 434.0, 358.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);

   f = pdfCreateTextField(pdf, "EhefrauGeburtdatum", -1, false, 10, 70.0, 410.0, 114.0, 14.0);
   // Add both actions to the field
   pdfAddActionToObj(pdf, otField, oeOnKeyStroke, keyStrokeAct, f);
   pdfAddActionToObj(pdf, otField, oeOnFormat, formatAct, f);

   pdfSetCharacterSpacing(pdf, 0.0);
   f = pdfCreateComboBox(pdf, "EhefrauReligion", false, -1, 185.0, 410.0, 71.0, 14.0);
   pdfAddValToChoiceField(pdf, f, "keine", "keine", true);
   pdfAddValToChoiceField(pdf, f, "Evang", "Evang.", false);
   pdfAddValToChoiceField(pdf, f, "Kath", "Kath.", false);
   pdfAddValToChoiceField(pdf, f, "andere", "andere", false);

   pdfCreateTextField(pdf, "EhefrauBeruf", -1, false, 0, 257.0, 410.0, 171.0, 14.0);
   pdfCreateTextField(pdf, "EhefrauStrasse", -1, false, 49, 70.0, 386.0, 358.0, 14.0);

   f = pdfCreateRadioButton(pdf, "Veranlagung", "Zusammen", false, -1, 70.0, 363.0, 13.0, 13.0);
   pdfCreateCheckBox(pdf, NULL, "Getrennt", false, f, 141.5, 363.0, 13.0, 13.0);
   pdfCreateCheckBox(pdf, NULL, "Eheschliessung", false, f, 213.5, 363.0, 13.0, 13.0);

   f = pdfCreateRadioButton(pdf, "Guetergemeinschaft", "Ja", false, -1, 357.0, 363.0, 13.0, 13.0);
   pdfCreateCheckBox(pdf, NULL, "Nein", false, f, 393.0, 363.0, 13.0, 13.0);

   pdfSetCharacterSpacing(pdf, 7.2);
   f = pdfCreateTextField(pdf, "Konto", -1, false, 10, 70.0, 314.0, 142.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   pdfSetNumberFormat(pdf, f, dsNoneDot, 0, nsRed, NULL, false);
   f = pdfCreateTextField(pdf, "BLZ", -1, false, 8, 286.0, 314.0, 113.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   pdfSetNumberFormat(pdf, f, dsNoneDot, 0, nsRed, NULL, false);

   f = pdfCreateTextField(pdf, "Bank", -1, false, 25, 70.0, 290.0, 358.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);

   pdfCreateCheckBox(pdf, "AbwInhaber", "Ja", false, -1, 70.0, 267.0, 13.0, 13.0);
   f = pdfCreateTextField(pdf, "AbwKontoInhaber", -1, false, 21, 128.0, 266.5, 300.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);

   f = pdfCreateTextField(pdf, "AbwEmpfName", -1, false, 25, 70.0, 218.0, 358.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   f = pdfCreateTextField(pdf, "AbwEmpfVorname", -1, false, 25, 70.0, 194.0, 358.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   f = pdfCreateTextField(pdf, "AbwEmpfStrasse", -1, false, 25, 70.0, 170.0, 358.0, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   f = pdfCreateTextField(pdf, "AbwEmpfPLZ", -1, false, 5, 70.0, 146.5, 70.5, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);
   pdfSetNumberFormat(pdf, f, dsNoneDot, 0, nsRed, NULL, false);

   f = pdfCreateTextField(pdf, "AbwEmpfWohnort", -1, false, 20, 142.5, 146.5, 285.5, 14.0);
   pdfSetFieldFlags(pdf, f, ffComb, false);

   // Add a reset button to the form
   pdfSetFieldBorderColor(pdf, PDF_BLACK);
   pdfSetFieldBackColor(pdf, PDF_MOGREEN);
   pdfSetLineWidth(pdf, 1.0);
   pdfSetFont(pdf, "Helvetica", fsBold, 12.0, true, cp1252);
   f = pdfCreateButton(pdf, "Reset", "Zurcksetzen", -1, 440.0, 730.0, 120.0, 25.0);
   pdfSetFieldBorderStyle(pdf, f, bsBevelled);
   act = pdfCreateResetAction(pdf);
   pdfAddActionToObj(pdf, otField, oeOnMouseUp, act, f);

   time(&t);
   now = gmtime(&t);
   sprintf(buf, "%02d.%02d.%d", now->tm_mday, now->tm_mon + 1, now->tm_year + 1900);

   pdfSetCharacterSpacing(pdf, 0.0);
   pdfSetFont(pdf, "Courier", fsNone, 12.0, true, cp1252);
   pdfWriteText(pdf, 53.0, 48.0, buf);

   pdfSetFillColor(pdf, PDF_RGB(0xFF, 0x66, 0x66));
   pdfSetFont(pdf, "Helvetica", fsBold, 22.0, false, cp1252);
   pdfWriteText(pdf, 340.0, 70.0, "www.dynaforms.de");

   pdfSetLineWidth(pdf, 0.0);
   pdfSetLinkHighlightMode(pdf, hmPush);
   pdfSetAnnotFlags(pdf, afReadOnly);
   pdfWebLink(pdf, 340.0, 64.0, 204.0, 22.0, "http://www.dynaforms.de");

   pdfAddJavaScript(pdf, "VersionCheck", alertScript);

   pdfEndPage(pdf);

   // No fatal error occurred?
   if (pdfHaveOpenDoc(pdf))
   {
      // We write the output file into the current directory.
      GetCurrentDirectory(511, filePath);
      strcat(filePath, "\\out.pdf");
      // OK, now we can open the output file.
      if (!pdfOpenOutputFile(pdf, filePath))
      {
         pdfDeletePDF(pdf);
         _getch();
         return -1;
      }
      if (pdfCloseFile(pdf))
      {
         printf("PDF file \"%s\" successfully created!\n", filePath);
         ShellExecute(0, "open", filePath, NULL, NULL, SW_SHOWMAXIMIZED);
      }
   }
   pdfDeletePDF(pdf);
   _getch();
   return 0;
}



