#include <stdio.h>
#include <conio.h>
#include <windows.h>

#define PDF_STATIC // Enable static binding when compiling the project with the workspace dynapdf_static.
#include "../../../../include/C_CPP/dynapdf.h"
#include "../../util/absolute_path.h" // Helper function to find the test files

using namespace DynaPDF;

class CHandleList
{
  public:
   CHandleList(void) :
      m_Capacity(0),
      m_Count(0),
      m_Handles(NULL)
   {}
   ~CHandleList(void)
   {
      if (m_Handles) free(m_Handles);
   }
   SI32 Add(SI32 Handle)
   {
      if (m_Count == m_Capacity)
      {
         m_Capacity += 50;
         SI32* tmp = (SI32*)realloc(m_Handles, sizeof(SI32) * m_Capacity);
         if (!tmp)
         {
            m_Capacity -= 50;
            return -1;
         }
         m_Handles = tmp;
      }
      m_Handles[m_Count] = Handle;
      return m_Count++;
   }
   SI32 Find(SI32 Handle)
   {
      SI32 i = 0;
      SI32 e = m_Count -1;
      while (i <= e)
      {
         if (m_Handles[i] == Handle) return i;
         if (m_Handles[e] == Handle) return e;
         ++i;
         --e;
      }
      return -1;
   }
  private:
   UI32  m_Capacity;
   UI32  m_Count;
   SI32* m_Handles;
};

class CImageList
{
  public:
   CImageList(void) :
      m_Capacity(0),
      m_Count(0),
      m_Handles(NULL)
   {}
   ~CImageList(void)
   {
      if (m_Handles) free(m_Handles);
   }
   SI32 Add(const void* Ptr)
   {
      if (m_Count == m_Capacity)
      {
         m_Capacity += 50;
         const void** tmp = (const void**)realloc(m_Handles, sizeof(SI32) * m_Capacity);
         if (!tmp)
         {
            m_Capacity -= 50;
            return -1;
         }
         m_Handles = tmp;
      }
      m_Handles[m_Count] = Ptr;
      return m_Count++;
   }
   SI32 Find(const void* Ptr)
   {
      SI32 i = 0;
      SI32 e = m_Count -1;
      while (i <= e)
      {
         if (m_Handles[i] == Ptr) return i;
         if (m_Handles[e] == Ptr) return e;
         ++i;
         --e;
      }
      return -1;
   }
  private:
   UI32         m_Capacity;
   UI32         m_Count;
   const void** m_Handles;
};

CHandleList m_Handles; // Template handles
CImageList  m_Images;  // Processed images
static SI32 m_Counter = 0;

SI32 PDF_CALL PDFError(const void* Data, SI32 ErrCode, const char* ErrMessage, SI32 ErrType)
{
   printf("%s\n", ErrMessage);
   return 0; // We try to continue
}

SI32 PDF_CALL parseBeginTemplate(const void* Data, const BYTE* Object, SI32 Handle, struct TPDFRect* BBox, struct TCTM* Matrix)
{
   // Already handled?
   if (m_Handles.Find(Handle) > -1) return  1; // Skip the template
   if (m_Handles.Add(Handle)  <  0) return -1; // Break processing if we have no more memory
   return 0;
}
/*
   Use this version if you want to create separate image files for each image.
   In this case remove the pdfCreateImage() and pdfCloseImage() calls from the
   main function.
*/
/*
SI32 PDF_CALL parseInsertImage(const void* Data, TPDFImage* Image)
{
   char path[512];
   if (!Image->InlineImage)
   {
      if (m_Images.Find(Image->ObjectPtr) > -1) return 0;  // Already handled?
      if (m_Images.Add(Image->ObjectPtr)  <  0) return -1; // Out of memory?
   }
   if (Image->Filter == dfDCTDecode) // JPEG
   {
      ++m_Counter;
      sprintf(path, "c:/Temp/img%.4d.jpg", m_Counter);
      FILE* f = fopen(path, "wb");
      fwrite(Image->Buffer, 1, Image->BufSize, f);
      fclose(f);
   }else if (Image->Filter == dfJPXDecode) // JPEG 2000
   {
      ++m_Counter;
      sprintf(path, "c:/Temp/img%.4d.jp2", m_Counter);
      FILE* f = fopen(path, "wb");
      fwrite(Image->Buffer, 1, Image->BufSize, f);
      fclose(f);
   }else if (Image->Filter == dfNone) // Anything else
   {
      ++m_Counter;
      sprintf(path, "c:/Temp/img%.4d.tif", m_Counter);
      // Other image formats can be created too. Take a look at TImageFormat in dynapdf.h.
      pdfCreateImage(Data, path, ifmTIFF);
      // Note that Flate compression is no standard filter; Photoshop does not support this filter.
      pdfAddImage(Data, cfLZW, icNone, Image);
      pdfCloseImage(Data);
   }
   return 0;
}
*/

SI32 PDF_CALL parseInsertImage(const void* Data, TPDFImage* Image)
{
   if (!Image->InlineImage)
   {
      if (m_Images.Find(Image->ObjectPtr) > -1) return 0;  // Already handled?
      if (m_Images.Add(Image->ObjectPtr)  <  0) return -1; // Out of memory?
   }
   // If an image cannot be decompressed due to an unsupported filter or if the decoder
   // failed to decompress it then we can get a compressed image here.
   if (Image->Filter != dfNone) return 0;
   // Note that Flate compression is no standard filter; Photoshop does not support this filter.
   if (Image->BitsPerPixel == 1)
      pdfAddImage(Data, cfCCITT4, icNone, Image);
   else
      pdfAddImage(Data, cfLZW, icNone, Image);
   return 0;
}

int main(int argc, char* argv[])
{
   char filePath[512];
   UI32 timeStart = GetTickCount();

   void* pdf = pdfNewPDF();
   pdfSetOnErrorProc(pdf, NULL, PDFError);
   pdfCreateNewPDF(pdf, NULL); // We don't create a PDF file

   // Skip anything that is not required
   pdfSetImportFlags(pdf, ifContentOnly | ifImportAsPage);
   // From which PDF file do you want to extract the images?
   MakeAbsolutePath(argv, "dynapdf_help.pdf", filePath);
   pdfOpenImportFile(pdf, filePath, ptOpen, NULL);

   // Comment this out if you want to extract the images from specific pages only
   pdfImportPDFFile(pdf, 1, 1.0, 1.0);

   SI32 i, pageCount = pdfGetPageCount(pdf);

   pdfCloseImportFile(pdf);

   TPDFParseInterface stack;
   memset(&stack, 0, sizeof(stack));
   // Set the required callback functions
   stack.BeginTemplate = parseBeginTemplate;
   stack.InsertImage   = parseInsertImage;

   // We create a multi-page TIFF in this example
   GetCurrentDirectory(511, filePath);
   strcat(filePath, "\\out.tif");
   pdfCreateImage(pdf, filePath, ifmTIFF);

   for (i = 1; i <= pageCount; i++)
   {
      pdfEditPage(pdf, i);
      /*
         If you want to convert the images into a specific color space then set
         one of the folowing flags (see also TParseFlags in dynapdf.h):

            pfDitherImagesToBW // Floyd-Steinberg dithering.
            pfConvImagesToGray
            pfConvImagesToRGB
            pfConvImagesToCMYK

         Only one color space conversion flag must be set at time.
      */
      pdfParseContent(pdf, pdf, &stack, pfDecomprAllImages);
      pdfEndPage(pdf);
   }
   pdfCloseImage(pdf);

   // Delete the PDF instance. This frees all resources.
   pdfDeletePDF(pdf);

   timeStart = GetTickCount() - timeStart;
   printf("Processing time: %d ms\n", timeStart);

   printf("Images have been extracted to %s\n", filePath);
   ShellExecute(0, "open", filePath, NULL, NULL, SW_SHOWMAXIMIZED);

   _getch();
   return 0;
}


