#include <windows.h>
#include <shellapi.h>
#include <conio.h>

#define PDF_STATIC // Enable static binding when compiling the project with the workspace dynapdf_static.
#include "../../../include/C_CPP/dynapdf.h"
#include "../util/absolute_path.h" // Helper function to create an absoulte file path
#include "../util/pdf_edit_text.h"

using namespace DynaPDF;

/*
   Note that the dynapdf.dll must be copied into the output directory or into a
   Windwos search path (e.g. %WINDOWS%/System32) before the application can be executed!
*/

// Error callback function.
// If the function name should not appear at the beginning of the error message then set
// the flag emNoFuncNames (pdf.SetErrorMode(DynaPDF.TErrMode.emNoFuncNames);).
SI32 PDF_CALL PDFError(const void* Data, SI32 ErrCode, const char* ErrMessage, SI32 ErrType)
{
   // We ignore warnings like "Font not found!" since this case is handled in the function CPDFEditText.SetFont().
   if (ErrType & E_WARNING) return 0;
   printf("%s\n", ErrMessage);
   return 0;
}

int main(int argc, char* argv[])
{
   char filePath[512];
   void* pdf = pdfNewPDF();
   if (!pdf) return 2; // Out of memory?

   UI32 timeStart = GetTickCount();

   pdfSetOnErrorProc(pdf, NULL, PDFError);
   pdfCreateNewPDF(pdf, NULL); // The output file is opened later

   // We avoid the conversion of pages to templates since this is not required in this example.
   pdfSetImportFlags(pdf, ifImportAll | ifImportAsPage);
   MakeAbsolutePath(argv, "examples/test_files/taxform.pdf", filePath);
   if (pdfOpenImportFile(pdf, filePath, ptOpen, NULL))
   {
      pdfDeletePDF(pdf);
      printf("Input file \"%s\" not found!\n", filePath);
      _getch();
      return 3;
   }
   pdfImportPDFFile(pdf, 1, 1.0, 1.0);
   pdfCloseImportFile(pdf);

   CPDFEditText parser(pdf);
   try
   {
      SI32 found;
      SI32 pageCount = pdfGetPageCount(pdf);
      for (SI32 i = 1; i <= pageCount; i++)
      {
         // DynaPDF error messages are already handled in the error callback function!
         if (!pdfEditPage(pdf, i)) throw -1;
         /*
            The function FindPattern() searches for a string and stores all records in which it
            was found in the array CPDFEditText.m_Records. Each record in the array represents
            exactly one position on the page where the string was found. The coordinates of
            text records are not stored at this time but it should be no problem to extend
            the class if necessary.

            The search run is case-sensitive. If you want to change the text search algorithm
            you must modify the protected functions CPDFEditText.FindPattern() and
            CPDFEditText.FindEndPattern().
         */
         found = parser.FindPattern(L"2002");
         /*
            ReplacePattern() replaces the text with a new one or deletes it if the new string
            is an empty string. The new text is written in red color so that you can better
            find it. Take a look into the function CPDFEditText.WriteText() if you want to
            write the text in the orignal color...
         */
         parser.ReplacePattern(L"2010");

         printf("Replaced text on page %d %d times!\n", i, found);
         pdfEndPage(pdf);
      }
      // No fatal error occurred?
      if (pdfHaveOpenDoc(pdf))
      {
         // We write the output file into the current directory.
         GetCurrentDirectory(511, filePath);
         strcat(filePath, "\\out.pdf");
         // OK, now we can open the output file.
         if (!pdfOpenOutputFile(pdf, filePath))
         {
            pdfDeletePDF(pdf);
            _getch();
            return -1;
         }
         if (pdfCloseFile(pdf))
         {
            timeStart = GetTickCount() - timeStart;
            printf("Processing time: %d ms\n", timeStart);
            printf("PDF file \"%s\" successfully created!\n", filePath);
            ShellExecute(0, "open", filePath, NULL, NULL, SW_SHOWMAXIMIZED);
         }
      }
   }catch(char* Message)
   {
      if (Message) printf("%s\n", Message);
      pdfDeletePDF(pdf);
      _getch();
      return -1;
   }catch(...)
   {
      pdfDeletePDF(pdf);
      _getch();
      return -1;
   }
   pdfDeletePDF(pdf);

   _getch();
   return 0;
}
