#include <windows.h>
#include <shellapi.h>
#include <stdio.h>
#include <time.h>
#include <conio.h>

#define PDF_STATIC // Enable static binding when compiling the project with the workspace dynapdf_static.
#include "../../../include/C_CPP/dynapdf.h"
#include "../util/absolute_path.h" // Helper function to find the test files

#include "pdf_signature.h" // Helper class to sign the PDF file

using namespace DynaPDF;

SI32 PDF_CALL PDFError(const void* Data, SI32 ErrCode, const char* ErrMessage, SI32 ErrType)
{
   printf("%s\n", ErrMessage);
   return 0;
}

int main(int argc, char* argv[])
{
   SI32 sigField, sh;
   char filePath[512];
   void* pdf = pdfNewPDF();
   if (!pdf) return -2; // Out of memory?

   pdfSetOnErrorProc(pdf, NULL, PDFError);
   pdfCreateNewPDF(pdf, NULL); // The output file is opened later

   pdfSetDocInfo(pdf, diCreator, "C Test application");
   pdfSetDocInfo(pdf, diTitle, "Custom digital signature appearance");

   pdfAppend(pdf);
   pdfSetFont(pdf, "Arial", fsNone, 12.0, true, cp1252);
   pdfWriteFText(pdf, taJustify, "This file was digitally signed with the Windows CryptAPI "
   "using a PKCS#12 certificate. It is also possible to use an arbitrary certificate of "
   "the user's certificate store. This could be a hardware or software certificate. The "
   "CryptAPI function calls are in both cases the same. Nothing special must be considered.\n\n"
   "The appearance of the signature field was created with normal DynaPDF functions. "
   "However, it would also be possible to import a PDF page, an EMF file, or an image "
   "into the appearance template. "
   "When creating an individual signature appearance make sure to place the "
   "validation icon properly with PlaceSigFieldValidateIcon(). The appearance of "
   "the validation icon depends on the Acrobat version with which the file is "
   "opened. However, the unscaled size of that icon is always 100.0 x 100.0 Units. "
   "It can be scaled to every size you want but it is usually best to preserve the "
   "aspect ratio and the icon must be placed fully inside the appearance template.");

   /* ---------------------- Signature field appearance ---------------------- */

   sigField = pdfCreateSigField(pdf, "Signature", -1, 200.0, 500.0, 200.0, 80.0);
   pdfSetFieldColor(pdf, sigField, fcBorderColor, csDeviceRGB, NO_COLOR);
   // Place the validation icon on the left side of the signature field.
   pdfPlaceSigFieldValidateIcon(pdf, sigField, 0.0, 15.0, 50.0, 50.0);
   /*
      This function creates a template into which you can draw whatever you want. The template
      is already opened when calling the function; it must be closed when finish with EndTemplate().
      An appearance template of a signature field is reserved for this field. It must not be placed
      on another page!

      In addition, it makes no sense to create an appearance template when the file is not digitally
      signed later. Empty signature fields cannot contain a user defined appearance.
   */
   pdfCreateSigFieldAP(pdf, sigField);

   pdfSaveGraphicState(pdf);
   pdfRectangle(pdf, 0.0, 0.0, 200.0, 80.0, fmNoFill);
   pdfClipPath(pdf, cmWinding, fmNoFill);
   sh = pdfCreateAxialShading(pdf, 0.0, 0.0, 200.0, 0.0, 0.5, PDF_RGB(120, 120, 220), PDF_WHITE, true, true);
   pdfApplyShading(pdf, sh);
   pdfRestoreGraphicState(pdf);

   pdfSaveGraphicState(pdf);
   pdfEllipse(pdf, 50.5, 1.0, 148.5, 78.0, fmNoFill);
   pdfClipPath(pdf, cmWinding, fmNoFill);
   sh = pdfCreateAxialShading(pdf, 0.0, 0.0, 0.0, 78.0, 2.0, PDF_WHITE, PDF_RGB(120, 120, 220), true, true);
   pdfApplyShading(pdf, sh);
   pdfRestoreGraphicState(pdf);

   pdfSetFont(pdf, "Arial", fsBold | fsUnderlined, 11.0, true, cp1252);
   pdfSetFillColor(pdf, PDF_RGB(120, 120, 220));
   pdfWriteFTextEx(pdf, 50.0, 60.0, 150.0, -1.0, taCenter, "Digitally signed by:");
   pdfSetFont(pdf, "Arial", fsBold|fsItalic, 18.0, true, cp1252);
   pdfSetFillColor(pdf, PDF_RGB(100, 100, 200));
   pdfWriteFTextEx(pdf, 50.0, 45.0, 150.0, -1.0, taCenter, "DynaPDF 2.5");

   pdfEndTemplate(pdf); // Close the appearance template.

   /* ------------------------------------------------------------------------ */

   pdfEndPage(pdf);

   // No fatal error occurred?
   if (pdfHaveOpenDoc(pdf))
   {
      char certFile[512];
      // We write the output file into the current directory.
      GetCurrentDirectory(511, filePath);
      strcat(filePath, "\\out.pdf");

      MakeAbsolutePath(argv, "examples/test_files/test_cert.pfx", certFile);

      // Now we can sign the PDF file
      CWinCrypt crypt;
      // In this example we load a PFX certificate file
      if (!crypt.LoadCertFromFile(certFile, L"test")) goto err;
      if (!crypt.OpenCert(0)) goto err;
      if (!crypt.CloseAndSignPDFFile(pdf, filePath, false)) goto err;
      crypt.CleanUp();
      /*
         // You can also use an arbitrary certificate of a certificate store.
         if (!crypt.OpenStore(L"MY")) goto err;
         if (!crypt.SelectCertificateFromStore()) goto err; // This function displays a dialog to select the certificate
         if (!crypt.CloseAndSignPDFFile(pdf, filePath, false)) goto err;
         crypt.CleanUp();
      */
      printf("PDF file %s successfully created!\n", filePath);
      ShellExecute(0, "open", filePath, NULL, NULL, SW_SHOWMAXIMIZED);
   }
   pdfDeletePDF(pdf);
   _getch();
   return 0;

err:
   pdfDeletePDF(pdf);
   _getch();
   return -2;
}
