#include <windows.h>
#include <shellapi.h>
#include <conio.h>

#define PDF_STATIC // Enable static binding when compiling the project with the workspace dynapdf_static.
#include "../../../include/C_CPP/dynapdf.h"
#include "../util/absolute_path.h" // Helper function to create an absoulte file path
#include "pdf_text_extract.h"

using namespace DynaPDF;

/*
   Note that the dynapdf.dll must be copied into the output directory or into a
   Windwos search path (e.g. %WINDOWS%/System32) before the application can be executed!
*/

// Error callback function.
// If the function name should not appear at the beginning of the error message then set
// the flag emNoFuncNames (pdf.SetErrorMode(DynaPDF.TErrMode.emNoFuncNames);).
SI32 PDF_CALL PDFError(const void* Data, SI32 ErrCode, const char* ErrMessage, SI32 ErrType)
{
   printf("%s\n", ErrMessage);
   return 0;
}

int main(int argc, char* argv[])
{
   void* pdf = pdfNewPDF();
   if (!pdf) return 2; // Out of memory?

   UI32 timeStart = GetTickCount();

   char filePath[512];
   pdfSetOnErrorProc(pdf, NULL, PDFError);
   pdfCreateNewPDF(pdf, NULL); // We create no PDF file in this example!

   // External cmaps should always be loaded when extracting text from PDF files.
   // See the description of GetPageText() for further information.
   MakeAbsolutePath(argv, "Resource/CMap/", filePath);
   pdfSetCMapDir(pdf, filePath, true);

   // We avoid the conversion of pages to templates
   pdfSetImportFlags(pdf, ifImportAll | ifImportAsPage);
   MakeAbsolutePath(argv, "dynapdf_help.pdf", filePath);
   if (pdfOpenImportFile(pdf, filePath, ptOpen, NULL))
   {
      pdfDeletePDF(pdf);
      printf("Input file \"%s\" not found!\n", filePath);
      _getch();
      return 3;
   }
   pdfImportPDFFile(pdf, 1, 1.0, 1.0);
   pdfCloseImportFile(pdf);

   // We flatten markup annotations and form fields so that we can extract the text from these objects too.
   pdfFlattenAnnots(pdf, affMarkupAnnots);
   pdfFlattenForm(pdf);

   // We write the output file into the current directory.
   GetCurrentDirectory(511, filePath);
   strcat(filePath, "\\out.txt");

   CTextExtraction parser(pdf);
   try
   {
      SI32 i, pageCount;
      // Open the output file
      parser.Open(filePath);
      pageCount = pdfGetPageCount(pdf);
      for (i = 1; i <= pageCount; i++)
      {
         // DynaPDF error messages are already handled in the error callback function!
         if (!pdfEditPage(pdf, i)) throw -1;
         parser.WritePageIdentifier(L"%%----------------------- Page %d -----------------------------\r\n", i);
         parser.ParsePage();
         pdfEndPage(pdf);
      }
      parser.Close();
   }catch(char* Message)
   {
      printf("%s\n", Message);
      pdfDeletePDF(pdf);
      printf("Unknown error!\n");
      _getch();
      return -1;
   }catch(...)
   {
      pdfDeletePDF(pdf);
      _getch();
      return -1;
   }
   pdfDeletePDF(pdf);

   timeStart = GetTickCount() - timeStart;
   printf("Processing time: %d ms\nText successfully extracted to:\n%s\n", timeStart, filePath);

   ShellExecute(0, "open", filePath, NULL, NULL, SW_SHOWMAXIMIZED);
   _getch();
   return 0;
}
