program edit_text;

{$APPTYPE CONSOLE}

uses
  Windows,
  SysUtils,
  ShellAPI,
  Classes,
  pdf_edit_text in '..\util\pdf_edit_text.pas',
  dynapdf in '..\..\..\include\Delphi\dynapdf.pas';

{
   Note that the dynapdf.dll must be copied into the output directory or into a
   Windwos search path (e.g. %WINDOWS%/System32) before the application can be executed!
}

// Error callback function.
// If the function name should not appear at the beginning of the error message set
// the flag emNoFuncNames (pdf.SetErrorMode(emNoFuncNames);). 
function ErrProc(const Data: Pointer; ErrCode: Integer; const ErrMessage: PChar; ErrType: Integer): Integer; stdcall;
begin
   Result := 0; // We ignore warnings and errors
   // Ignore warnings like 'Font not found!'.
   if ErrType and E_WARNING <> 0 then Exit;
   Writeln(ErrMessage);
end;

procedure ReplaceText();
var i, found: Integer; parser: CPDFEditText; pdf: TPDF; outFile: String;
begin
   pdf    := nil;
   parser := nil;
   try
      pdf     := TPDF.Create;
      parser  := CPDFEditText.Create(pdf);
      pdf.CreateNewPDFA(''); // The output file is opened later
      pdf.SetOnErrorProc(nil, @ErrProc);
      // Avoid the conversion of pages to templates
      pdf.SetImportFlags(ifImportAll or ifImportAsPage);
      if pdf.OpenImportFileA('../../test_files/taxform.pdf', ptOpen, '') < 0 then begin
         pdf.Free;
         parser.Free;
         Writeln('Input file "../../test_files/taxform.pdf" not found!');
         ReadLn;
         Exit;
      end;
      pdf.ImportPDFFile(1, 1.0, 1.0);
      pdf.CloseImportFile;

      // Note that page numbering starts at 1!
      for i := 1 to pdf.GetPageCount do begin
         pdf.EditPage(i); // Open the page
         {
            The function FindPattern() searches for a string and stores all records in which it
            was found in the array CPDFEditText.m_Records. Each record in the array represents
            exactly one position on the page where the string was found. The coordinates of
            text records are not stored at this time but it should be no problem to extend
            the class if necessary.

            The search run is case-sensitive. If you want to change the text search algorithm
            you must modify the protected functions CPDFEditText.FindPattern() and
            CPDFEditText.FindEndPattern().
         }
         found := parser.FindText('2002');
         {
            ReplacePattern() replaces the text with a new one or deletes it if the new string
            is an empty string. The new text is written in red color so that you can better
            find it. Take a look into the function CPDFEditText.WriteText() if you want to
            write the text in the orignal color...
         }
         if found > 0 then parser.ReplacePattern('2010');
         Writeln(Format('Replaced text on page %d %d times!', [i, found]));
         pdf.EndPage;    // Close the page
      end;
      parser.Free;
      parser := nil;
      // No fatal error occurred?
      if pdf.HaveOpenDoc then begin
         // We write the file into the application directory.
         GetDir(0, outFile);
         outFile := outFile + '\out.pdf';
         if not pdf.OpenOutputFileA(outFile) then begin
            pdf.Free;
            Readln;
            Exit;
         end;
      end;
      if pdf.CloseFile then begin
         Writeln(Format('PDF file "%s" successfully created!', [outFile]));
         ShellExecute(0, PChar('open'), PChar(outFile), nil, nil, SW_SHOWMAXIMIZED);
      end;
   except
      on E: Exception do begin
         // DynaPDF error messages are already handled in the error callback function.
         // We display the error message only if a non-DynaPDF error occurred.
         if E.Message <> '-1' then Writeln(E.Message);
      end;
   end;
   if parser <> nil then parser.Free;
   if pdf <> nil then pdf.Free;
   Readln;
end;

begin
   ReplaceText;
end.
 