Module Module1
   ' Error callback function.
   ' If the function name should not appear at the beginning of the error message then set
   ' the flag emNoFuncNames (pdf.SetErrorMode(CPDF.TErrMode.emNoFuncNames)).
   Private Function PDFError(ByVal Data As IntPtr, ByVal ErrCode As Integer, ByVal ErrMessage As IntPtr, ByVal ErrType As Integer) As Integer
      Console.WriteLine("{0}" + Chr(10), System.Runtime.InteropServices.Marshal.PtrToStringAnsi(ErrMessage))
      Return 0 ' We try to continue if an error occurrs. Any other return value breaks processing.
   End Function

   Sub Main()
      Try
         Dim pdf As CPDF = New CPDF()
         pdf.SetErrorMode(CPDF.TErrMode.emNoFuncNames)
         ' You can either use events or declare a callback function.
         pdf.SetOnErrorProc(AddressOf PDFError)
         pdf.CreateNewPDFA(Nothing) ' We open the output file later if no error occurrs.
         pdf.SetDocInfoA(CPDF.TDocumentInfo.diAuthor, "Jens Boschulte")
         pdf.SetDocInfoA(CPDF.TDocumentInfo.diCreator, "VB .Net test application")
         pdf.SetDocInfoA(CPDF.TDocumentInfo.diTitle, "How to create an Interactive Form?")
         ' Conversion of pages to templates is normally not required. Templates are required if
         ' the page should be scaled or used multiple times in a document, e.g. as a page background.
         ' See help file for further information.
         pdf.SetImportFlags(CPDF.TImportFlags.ifImportAll Or CPDF.TImportFlags.ifImportAsPage)
         If (pdf.OpenImportFileA("../../../test_files/taxform.pdf", CPDF.TPwdType.ptOpen, Nothing) < 0) Then
            Console.Write("Input file ""../../../test_files/taxform.pdf"" not found!" + Chr(10))
            Console.Read()
            Exit Sub
         End If
         pdf.ImportPDFFile(1, 1.0, 1.0)
         pdf.CloseImportFile()

         ' Transparent field background
         pdf.SetFieldBackColor(CPDF.NO_COLOR)

         pdf.EditPage(1)

         pdf.SetCheckBoxChar(CPDF.TCheckBoxChar.ccCross4)
         pdf.SetLineWidth(0.0)
         pdf.SetFieldBorderColor(CPDF.NO_COLOR)

         Dim f As Integer
         f = pdf.CreateRadioButton("AntragsArt", "EinkommensSteuerErklrung", False, -1, 70, 746.5, 13.0, 13.0)
         pdf.CreateCheckBox(Nothing, "Sparzulage", False, f, 70, 722.5, 13.0, 13.0)
         pdf.CreateCheckBox(Nothing, "VerlustErklrung", False, f, 249.5, 722.5, 13.0, 13.0)

         pdf.SetFontA("Courier", CPDF.TFStyle.fsNone, 12.0, True, CPDF.TCodepage.cp1252)
         pdf.CreateTextField("Finanzamt", -1, False, 0, 70.0, 698.0, 358.0, 14.0)
         pdf.CreateTextField("Steuernummer", -1, False, 14, 70.0, 673.5, 150.0, 14.0)
         pdf.CreateTextField("Finanzamt_alt", -1, False, 0, 272.0, 673.5, 156.0, 14.0)

         pdf.CreateCheckBox("Rckerstattung", "Ja", True, -1, 429, 675.0, 13.0, 13.0)
         pdf.CreateTextField("Telefon", -1, False, 0, 285.5, 650.0, 142.5, 14.0)

         ' Acrobat 7.0 ignores the character spacing, the only way to get the same result is to set the Comb flag.
         ' See help file for further information.

         pdf.SetCharacterSpacing(7.2)
         f = pdf.CreateTextField("Name", -1, False, 25, 70.0, 626.0, 358.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         f = pdf.CreateTextField("Vorname", -1, False, 25, 70.0, 602.0, 358.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         f = pdf.CreateTextField("Geburtsdatum", -1, False, 10, 70.0, 578.0, 113.0, 14.0)
         ' Create a custom date format action -> input example: 13.05.1971 or 13-05-1971 or 13/05/1971
         ' Two actions are required, one for the event onKeyStroke and one for the event onFormat.
         Dim keyStrokeAct, keyStrokeAct2, formatAct, formatAct2 As Integer
         keyStrokeAct = pdf.CreateJSActionA("AFDate_KeystrokeEx(""ddmmyyyy"")")
         formatAct = pdf.CreateJSActionA("AFDate_FormatEx(""ddmmyyyy"")")

         ' Add both actions to the field
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct, f)
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnFormat, formatAct, f)

         pdf.SetCharacterSpacing(0.0)
         f = pdf.CreateComboBox("Religion", False, -1, 185.0, 578.0, 71.0, 14.0)
         pdf.AddValToChoiceFieldA(f, "keine", "keine", True)
         pdf.AddValToChoiceFieldA(f, "Evang", "Evang.", False)
         pdf.AddValToChoiceFieldA(f, "Kath", "Kath.", False)
         pdf.AddValToChoiceFieldA(f, "andere", "andere", False)

         pdf.CreateTextField("Beruf", -1, False, 0, 257.0, 578.0, 171.0, 14.0)

         pdf.SetCharacterSpacing(7.2)
         f = pdf.CreateTextField("Strasse", -1, False, 25, 70.0, 554.0, 358.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         f = pdf.CreateTextField("PLZ", -1, False, 5, 70.0, 530.0, 70.5, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         pdf.SetNumberFormat(f, CPDF.TDecSeparator.dsNoneDot, 0, CPDF.TNegativeStyle.nsRed, Nothing, False)

         f = pdf.CreateTextField("Wohnort", -1, False, 20, 142.5, 530.0, 285.5, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)

         pdf.SetCharacterSpacing(0.0)
         pdf.SetFontA("Courier", CPDF.TFStyle.fsNone, 11.0, True, CPDF.TCodepage.cp1252)

         f = pdf.CreateTextField("VerheiratetSeit", -1, False, 10, 70.0, 506.0, 70.5, 14.0)
         ' We need another date format here. The date should be delimited with a dot.
         keyStrokeAct2 = pdf.CreateJSActionA("AFDate_KeystrokeEx(""dd.mm.yyyy"")")
         formatAct2 = pdf.CreateJSActionA("AFDate_FormatEx(""dd.mm.yyyy"")")
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct2, f)
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnFormat, formatAct2, f)

         f = pdf.CreateTextField("VerwitwetSeit", -1, False, 10, 142.0, 506.0, 71.0, 14.0)
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct2, f)
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnFormat, formatAct2, f)

         f = pdf.CreateTextField("GeschiedenSeit", -1, False, 10, 214.0, 506.0, 71.0, 14.0)
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct2, f)
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnFormat, formatAct2, f)

         f = pdf.CreateTextField("GetrenntSeit", -1, False, 10, 286.0, 506.0, 142.0, 14.0)
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct2, f)
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnFormat, formatAct2, f)

         pdf.SetCharacterSpacing(7.2)
         pdf.SetFontA("Courier", CPDF.TFStyle.fsNone, 12.0, True, CPDF.TCodepage.cp1252)
         f = pdf.CreateTextField("EhefrauVorname", -1, False, 25, 70.0, 458.0, 358.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         f = pdf.CreateTextField("EhefrauName", -1, False, 25, 70.0, 434.0, 358.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)

         f = pdf.CreateTextField("EhefrauGeburtdatum", -1, False, 10, 70.0, 410.0, 114.0, 14.0)
         ' Add both actions to the field
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct, f)
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnFormat, formatAct, f)

         pdf.SetCharacterSpacing(0.0)
         f = pdf.CreateComboBox("EhefrauReligion", False, -1, 185.0, 410.0, 71.0, 14.0)
         pdf.AddValToChoiceFieldA(f, "keine", "keine", True)
         pdf.AddValToChoiceFieldA(f, "Evang", "Evang.", False)
         pdf.AddValToChoiceFieldA(f, "Kath", "Kath.", False)
         pdf.AddValToChoiceFieldA(f, "andere", "andere", False)

         pdf.CreateTextField("EhefrauBeruf", -1, False, 0, 257.0, 410.0, 171.0, 14.0)
         pdf.CreateTextField("EhefrauStrasse", -1, False, 49, 70.0, 386.0, 358.0, 14.0)

         f = pdf.CreateRadioButton("Veranlagung", "Zusammen", False, -1, 70.0, 363.0, 13.0, 13.0)
         pdf.CreateCheckBox(Nothing, "Getrennt", False, f, 141.5, 363.0, 13.0, 13.0)
         pdf.CreateCheckBox(Nothing, "Eheschliessung", False, f, 213.5, 363.0, 13.0, 13.0)

         f = pdf.CreateRadioButton("Guetergemeinschaft", "Ja", False, -1, 357.0, 363.0, 13.0, 13.0)
         pdf.CreateCheckBox(Nothing, "Nein", False, f, 393.0, 363.0, 13.0, 13.0)

         pdf.SetCharacterSpacing(7.2)
         f = pdf.CreateTextField("Konto", -1, False, 10, 70.0, 314.0, 142.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         pdf.SetNumberFormat(f, CPDF.TDecSeparator.dsNoneDot, 0, CPDF.TNegativeStyle.nsRed, Nothing, False)
         f = pdf.CreateTextField("BLZ", -1, False, 8, 286.0, 314.0, 113.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         pdf.SetNumberFormat(f, CPDF.TDecSeparator.dsNoneDot, 0, CPDF.TNegativeStyle.nsRed, Nothing, False)

         f = pdf.CreateTextField("Bank", -1, False, 25, 70.0, 290.0, 358.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)

         pdf.CreateCheckBox("AbwInhaber", "Ja", False, -1, 70.0, 267.0, 13.0, 13.0)
         f = pdf.CreateTextField("AbwKontoInhaber", -1, False, 21, 128.0, 266.5, 300.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)

         f = pdf.CreateTextField("AbwEmpfName", -1, False, 25, 70.0, 218.0, 358.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         f = pdf.CreateTextField("AbwEmpfVorname", -1, False, 25, 70.0, 194.0, 358.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         f = pdf.CreateTextField("AbwEmpfStrasse", -1, False, 25, 70.0, 170.0, 358.0, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         f = pdf.CreateTextField("AbwEmpfPLZ", -1, False, 5, 70.0, 146.5, 70.5, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)
         pdf.SetNumberFormat(f, CPDF.TDecSeparator.dsNoneDot, 0, CPDF.TNegativeStyle.nsRed, Nothing, False)

         f = pdf.CreateTextField("AbwEmpfWohnort", -1, False, 20, 142.5, 146.5, 285.5, 14.0)
         pdf.SetFieldFlags(f, CPDF.TFieldFlags.ffComb, False)

         ' Add a reset button to the form
         pdf.SetFieldBorderColor(CPDF.PDF_BLACK)
         pdf.SetFieldBackColor(CPDF.PDF_MOGREEN)
         pdf.SetLineWidth(1.0)
         pdf.SetFontA("Helvetica", CPDF.TFStyle.fsBold, 12.0, True, CPDF.TCodepage.cp1252)
         f = pdf.CreateButtonA("Reset", "Zurcksetzen", -1, 440.0, 730.0, 120.0, 25.0)
         pdf.SetFieldBorderStyle(f, CPDF.TBorderStyle.bsBevelled)
         Dim act As Integer
         act = pdf.CreateResetAction()
         pdf.AddActionToObj(CPDF.TObjType.otField, CPDF.TObjEvent.oeOnMouseUp, act, f)

         pdf.SetCharacterSpacing(0.0)
         pdf.SetFontA("Courier", CPDF.TFStyle.fsNone, 12.0, True, CPDF.TCodepage.cp1252)
         pdf.WriteTextA(53.0, 48.0, DateTime.Now.ToString())

         pdf.SetFillColor(RGB(&HFF, &H66, &H66))
         pdf.SetFontA("Helvetica", CPDF.TFStyle.fsBold, 22.0, False, CPDF.TCodepage.cp1252)
         pdf.WriteTextA(340.0, 70.0, "www.dynaforms.de")

         pdf.SetLineWidth(0.0)
         pdf.SetLinkHighlightMode(CPDF.THighlightMode.hmPush)
         pdf.SetAnnotFlags(CPDF.TAnnotFlags.afReadOnly)
         pdf.WebLinkA(340.0, 64.0, 204.0, 22.0, "http://www.dynaforms.de")

         Dim alertScript As String
         alertScript = New String("if (app.viewerVersion >= 7.0 && app.viewerVersion < 8.0)" + Chr(10) & _
            "{" + Chr(10) & _
            "app.alert(""Due to bugs in Acrobat 7.0 character spacing does not work" & _
            " in text fields. We use the flag ffComb instead which is supported since" & _
            " Acrobat 6.0. However, if the field's format is controled by a JavaScript" & _
            " Action, we cannot set the flag because Acrobat 6.0 is then unable" & _
            " to display the field's text. Acrobat 7.0 has problems to place the" & _
            " the cursor in such fields but the final format will be correctly applied." & _
            " The problem is that if the form should work with Acrobat 6.0 we cannot set" & _
            " the flag ffComb to the date fields. It is also impossible to get the same" & _
            " result as with character spacing with the flag ffComb. So, what can we do to make the" & _
            " form compatible to all Acrobat versions? I really don't know! The easiest way" & _
            " would be to avoid the usage of such date formats." & _
            " However, the bug is already solved in Acrobat 8!"")" + Chr(10) & _
            "}")
         pdf.AddJavaScriptA("VersionCheck", alertScript)

         pdf.EndPage()

         ' No fatal error occurred?
         If pdf.HaveOpenDoc() Then
            ' We write the output file into the current directory.
            Dim filePath As String = System.IO.Directory.GetCurrentDirectory()
            filePath += "\out.pdf"
            ' OK, now we can open the output file.
            If Not pdf.OpenOutputFileA(filePath) Then Return
            If pdf.CloseFile() Then
               Console.Write("PDF file ""{0}"" successfully created!" + Chr(10), filePath)
               Dim p As System.Diagnostics.Process = New System.Diagnostics.Process()
               p.StartInfo.FileName = filePath
               p.Start()
            End If
         End If
      Catch e As Exception
         Console.WriteLine(e.Message)
      End Try
      Console.Read()
   End Sub

End Module
