Module Module1
   ' Error callback function.
   ' If the function name should not appear at the beginning of the error message then set
   ' the flag emNoFuncNames (pdf.SetErrorMode(CPDF.TErrMode.emNoFuncNames)).
   Private Function PDFError(ByVal Data As IntPtr, ByVal ErrCode As Integer, ByVal ErrMessage As IntPtr, ByVal ErrType As Integer) As Integer
      Console.Write("{0} + chr(10)", System.Runtime.InteropServices.Marshal.PtrToStringAnsi(ErrMessage))
      Return 0 ' We try to continue if an error occurrs. Any other return value breaks processing.
   End Function

   Sub Main()
      Try
         Dim pdf As CPDF = New CPDF()
         ' You can either use events or declare a callback function.
         pdf.SetOnErrorProc(AddressOf PDFError)
         pdf.CreateNewPDFA(Nothing) ' The ouput file is opened later

         pdf.SetDocInfoA(CPDF.TDocumentInfo.diAuthor, "Jens Boschulte")
         pdf.SetDocInfoA(CPDF.TDocumentInfo.diCreator, "VB .Net test application")
         pdf.SetDocInfoA(CPDF.TDocumentInfo.diTitle, "How to edit PDF files")
         pdf.SetViewerPreferences(CPDF.TViewerPreference.vpDisplayDocTitle, CPDF.TViewPrefAddVal.avNone)

         ' Conversion of pages to templates is normally not required. Templates are required if
         ' the page should be scaled or used multiple times in a document, e.g. as a page background.
         ' See help file for further information.
         pdf.SetImportFlags(CPDF.TImportFlags.ifImportAll Or CPDF.TImportFlags.ifImportAsPage)
         If (pdf.OpenImportFileA("../../../test_files/taxform.pdf", CPDF.TPwdType.ptOpen, Nothing) < 0) Then
            Console.Write("Input file ""../../../test_files/taxform.pdf"" not found!" + Chr(10))
            Console.Read()
            Exit Sub
         End If
         pdf.ImportPDFFile(1, 1.0, 1.0)
         pdf.CloseImportFile()

         pdf.EditPage(1)
         ' We use the 14 standard fonts only because they are always available. However, in a real
         ' project you should use fonts which can be embedded.
         pdf.SetFontA("Courier", CPDF.TFStyle.fsBold, 14.0, False, CPDF.TCodepage.cp1252)

         pdf.WriteTextA(72.5, 748.5, "X")
         pdf.WriteTextA(74.0, 701.0, "Musterstadt")
         pdf.WriteTextA(74.0, 677.0, "252/1062/3323")

         pdf.BeginContinueText(74.0, 628.0)
         pdf.SetLeading(24.0)
         pdf.SetCharacterSpacing(5.8)
         pdf.AddContinueTextA("Mustermann")
         pdf.AddContinueTextA("Hermann")
         pdf.AddContinueTextA("22021963keineKaufmann")
         pdf.AddContinueTextA("Musterstrae 145")
         pdf.AddContinueTextA("12345Musterstadt")
         pdf.SetCharacterSpacing(0.0)
         pdf.SetFontA("Courier", CPDF.TFStyle.fsBold, 10.0, False, CPDF.TCodepage.cp1252)
         pdf.SetLeading(47.5)
         pdf.AddContinueTextA("04.05.1994")
         pdf.SetFontA("Courier", CPDF.TFStyle.fsBold, 14.0, False, CPDF.TCodepage.cp1252)
         pdf.SetCharacterSpacing(5.8)
         pdf.AddContinueTextA("Sabine")
         pdf.AddContinueTextA("18121966 ev  Hausfrau")
         pdf.EndContinueText()

         pdf.WriteTextA(72.5, 365.0, "X")
         pdf.WriteTextA(396.0, 365.0, "X")

         pdf.BeginContinueText(74.0, 316.0)
         pdf.SetLeading(24.0)
         pdf.AddContinueTextA("2346256780     76834560")
         pdf.AddContinueTextA("Sparkasse Musterstadt")
         pdf.EndContinueText()

         pdf.WriteTextA(72.5, 269.0, "X")
         pdf.SetCharacterSpacing(0.0)
         pdf.SetFontA("Courier", CPDF.TFStyle.fsNone, 10.0, False, CPDF.TCodepage.cp1252)

         pdf.WriteTextA(53.0, 48.0, DateTime.Now.ToString())
         pdf.SetFillColor(RGB(&HFF, &H66, &H66))
         pdf.SetFontA("Helvetica", CPDF.TFStyle.fsBold, 22.0, False, CPDF.TCodepage.cp1252)
         pdf.WriteTextA(340, 70, "www.dynaforms.de")
         pdf.SetLineWidth(0.0)
         pdf.SetLinkHighlightMode(CPDF.THighlightMode.hmPush)
         pdf.SetAnnotFlags(CPDF.TAnnotFlags.afReadOnly)
         pdf.WebLinkA(340, 64, 204, 22, "http:'www.dynaforms.de")
         pdf.EndPage()

         ' No fatal error occurred?
         If pdf.HaveOpenDoc() Then
            ' We write the output file into the current directory.
            Dim filePath As String = System.IO.Directory.GetCurrentDirectory()
            filePath += "\out.pdf"
            ' OK, now we can open the output file.
            If Not pdf.OpenOutputFileA(filePath) Then Return
            If pdf.CloseFile() Then
               Console.Write("PDF file ""{0}"" successfully created!" + Chr(10), filePath)
               Dim p As System.Diagnostics.Process = New System.Diagnostics.Process()
               p.StartInfo.FileName = filePath
               p.Start()
            End If
         End If
      Catch e As Exception
         Console.Write(e.Message + Chr(10))
      End Try
      Console.Read()
   End Sub

End Module
