using System;
using System.IO;
using DynaPDF;

namespace acroform
{
	class AcroForm
	{
      /*
         Note that the dynapdf.dll must be copied into the output directory or into a
         Windwos search path (e.g. %WINDOWS%/System32) before the application can be executed!
      */

      // Error callback function.
      // If the function name should not appear at the beginning of the error message then set
      // the flag emNoFuncNames (pdf.SetErrorMode(DynaPDF.TErrMode.emNoFuncNames);).
      static int PDFError(IntPtr Data, int ErrCode, IntPtr ErrMessage, int ErrType)
      {
         Console.Write("{0}\n", System.Runtime.InteropServices.Marshal.PtrToStringAnsi(ErrMessage));
         return 0; // We try to continue if an error occurrs. Any other return value breaks processing.
      }

		[STAThread]
		static void Main(string[] args)
		{
		   try
		   {
            CPDF pdf = new CPDF();
            // Error messages and warnings are passed to the callback function.
            pdf.SetOnErrorProc(IntPtr.Zero, new DynaPDF.TErrorProc(PDFError));
            pdf.CreateNewPDF(null); // We open the output file later if no error occurrs.
            pdf.SetDocInfoA(DynaPDF.TDocumentInfo.diAuthor, "Jens Boschulte");
            pdf.SetDocInfoA(DynaPDF.TDocumentInfo.diCreator, "C# test application");
            pdf.SetDocInfoA(DynaPDF.TDocumentInfo.diTitle, "How to create an Interactive Form?");
            // Conversion of pages to templates is normally not required. Templates are required if
            // the page should be scaled or used multiple times in a document, e.g. as a page background.
            // See help file for further information.
            pdf.SetImportFlags(DynaPDF.TImportFlags.ifImportAll | DynaPDF.TImportFlags.ifImportAsPage);
            if (pdf.OpenImportFileA("../../../../test_files/taxform.pdf", DynaPDF.TPwdType.ptOpen, null) < 0)
            {
               Console.Write("Input file \"../../../../test_files/taxform.pdf\" not found!\n");
               Console.Read();
               return;
            }
            pdf.ImportPDFFile(1, 1.0, 1.0);
            pdf.CloseImportFile();

            pdf.EditPage(1);

            pdf.SetCheckBoxChar(DynaPDF.TCheckBoxChar.ccCross4);
            pdf.SetLineWidth(0.0);
            pdf.SetFieldBorderColor(CPDF.NO_COLOR);

            int f = pdf.CreateRadioButton("AntragsArt", "EinkommensSteuerErklrung", false, -1, 70, 746.5, 13.0, 13.0);
            pdf.CreateCheckBox(null, "Sparzulage", false, f, 70, 722.5, 13.0, 13.0);
            pdf.CreateCheckBox(null, "VerlustErklrung", false, f, 249.5, 722.5, 13.0, 13.0);

            pdf.SetFont("Courier", DynaPDF.TFStyle.fsNone, 12.0, true, DynaPDF.TCodepage.cp1252);
            pdf.CreateTextField("Finanzamt", -1, false, 0, 70.0, 698.0, 358.0, 14.0);
            pdf.CreateTextField("Steuernummer", -1, false, 14, 70.0, 673.5, 150.0, 14.0);
            pdf.CreateTextField("Finanzamt_alt", -1, false, 0, 272.0, 673.5, 156.0, 14.0);

            pdf.CreateCheckBox("Rckerstattung", "Ja", true, -1, 429, 675.0, 13.0, 13.0);
            pdf.CreateTextField("Telefon", -1, false, 0, 285.5, 650.0, 142.5, 14.0);
            
         /*
            Acrobat 7.0 ignores the character spacing, the only way to get the same result is to set the Comb flag.
            See help file for further information.
         */
            pdf.SetCharacterSpacing( 7.2);
            f = pdf.CreateTextField("Name", -1, false, 25, 70.0, 626.0, 358.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            f = pdf.CreateTextField("Vorname", -1, false, 25, 70.0, 602.0, 358.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            f = pdf.CreateTextField("Geburtsdatum", -1, false, 10, 70.0, 578.0, 113.0, 14.0);
            // Create a custom date format action -> input example: 13.05.1971 or 13-05-1971 or 13/05/1971
            // Two actions are required, one for the event onKeyStroke and one for the event onFormat.
            int keyStrokeAct = pdf.CreateJSActionA("AFDate_KeystrokeEx(\"ddmmyyyy\");");
            int formatAct = pdf.CreateJSActionA("AFDate_FormatEx(\"ddmmyyyy\");");

            // Add both actions to the field
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct, f);
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnFormat, formatAct, f);

            pdf.SetCharacterSpacing(0.0);
            f = pdf.CreateComboBox("Religion", false, -1, 185.0, 578.0, 71.0, 14.0);
            pdf.AddValToChoiceFieldA(f, "keine", "keine", true);
            pdf.AddValToChoiceFieldA(f, "Evang", "Evang.", false);
            pdf.AddValToChoiceFieldA(f, "Kath", "Kath.", false);
            pdf.AddValToChoiceFieldA(f, "andere", "andere", false);

            pdf.CreateTextField("Beruf", -1, false, 0, 257.0, 578.0, 171.0, 14.0);

            pdf.SetCharacterSpacing(7.2);
            f = pdf.CreateTextField("Strasse", -1, false, 25, 70.0, 554.0, 358.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            f = pdf.CreateTextField("PLZ", -1, false, 5, 70.0, 530.0, 70.5, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            pdf.SetNumberFormat(f, DynaPDF.TDecSeparator.dsNoneDot, 0, DynaPDF.TNegativeStyle.nsRed, null, false);

            f = pdf.CreateTextField("Wohnort", -1, false, 20, 142.5, 530.0, 285.5, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);

            pdf.SetCharacterSpacing(0.0);
            pdf.SetFont("Courier", DynaPDF.TFStyle.fsNone, 11.0, true, DynaPDF.TCodepage.cp1252);

            f = pdf.CreateTextField("VerheiratetSeit", -1, false, 10, 70.0, 506.0, 70.5, 14.0);
            // We need another date format here. The date should be delimited with a dot.
            int keyStrokeAct2 = pdf.CreateJSActionA("AFDate_KeystrokeEx(\"dd.mm.yyyy\");");
            int formatAct2 = pdf.CreateJSActionA("AFDate_FormatEx(\"dd.mm.yyyy\");");
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct2, f);
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnFormat, formatAct2, f);

            f = pdf.CreateTextField("VerwitwetSeit", -1, false, 10, 142.0, 506.0, 71.0, 14.0);
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct2, f);
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnFormat, formatAct2, f);

            f = pdf.CreateTextField("GeschiedenSeit", -1, false, 10, 214.0, 506.0, 71.0, 14.0);
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct2, f);
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnFormat, formatAct2, f);

            f = pdf.CreateTextField("GetrenntSeit", -1, false, 10, 286.0, 506.0, 142.0, 14.0);
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct2, f);
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnFormat, formatAct2, f);

            pdf.SetCharacterSpacing(7.2);
            pdf.SetFont("Courier", DynaPDF.TFStyle.fsNone, 12.0, true, DynaPDF.TCodepage.cp1252);
            f = pdf.CreateTextField("EhefrauVorname", -1, false, 25, 70.0, 458.0, 358.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            f = pdf.CreateTextField("EhefrauName", -1, false, 25, 70.0, 434.0, 358.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);

            f = pdf.CreateTextField("EhefrauGeburtdatum", -1, false, 10, 70.0, 410.0, 114.0, 14.0);
            // Add both actions to the field
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnKeyStroke, keyStrokeAct, f);
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnFormat, formatAct, f);

            pdf.SetCharacterSpacing(0.0);
            f = pdf.CreateComboBox("EhefrauReligion", false, -1, 185.0, 410.0, 71.0, 14.0);
            pdf.AddValToChoiceFieldA(f, "keine", "keine", true);
            pdf.AddValToChoiceFieldA(f, "Evang", "Evang.", false);
            pdf.AddValToChoiceFieldA(f, "Kath", "Kath.", false);
            pdf.AddValToChoiceFieldA(f, "andere", "andere", false);

            pdf.CreateTextField("EhefrauBeruf", -1, false, 0, 257.0, 410.0, 171.0, 14.0);
            pdf.CreateTextField("EhefrauStrasse", -1, false, 49, 70.0, 386.0, 358.0, 14.0);

            f = pdf.CreateRadioButton("Veranlagung", "Zusammen", false, -1, 70.0, 363.0, 13.0, 13.0);
            pdf.CreateCheckBox(null, "Getrennt", false, f, 141.5, 363.0, 13.0, 13.0);
            pdf.CreateCheckBox(null, "Eheschliessung", false, f, 213.5, 363.0, 13.0, 13.0);

            f = pdf.CreateRadioButton("Guetergemeinschaft", "Ja", false, -1, 357.0, 363.0, 13.0, 13.0);
            pdf.CreateCheckBox(null, "Nein", false, f, 393.0, 363.0, 13.0, 13.0);

            pdf.SetCharacterSpacing(7.2);
            f = pdf.CreateTextField("Konto", -1, false, 10, 70.0, 314.0, 142.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            pdf.SetNumberFormat(f, TDecSeparator.dsNoneDot, 0, DynaPDF.TNegativeStyle.nsRed, null, false);
            f = pdf.CreateTextField("BLZ", -1, false, 8, 286.0, 314.0, 113.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            pdf.SetNumberFormat(f, DynaPDF.TDecSeparator.dsNoneDot, 0, DynaPDF.TNegativeStyle.nsRed, null, false);

            f = pdf.CreateTextField("Bank", -1, false, 25, 70.0, 290.0, 358.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);

            pdf.CreateCheckBox("AbwInhaber", "Ja", false, -1, 70.0, 267.0, 13.0, 13.0);
            f = pdf.CreateTextField("AbwKontoInhaber", -1, false, 21, 128.0, 266.5, 300.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);

            f = pdf.CreateTextField("AbwEmpfName", -1, false, 25, 70.0, 218.0, 358.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            f = pdf.CreateTextField("AbwEmpfVorname", -1, false, 25, 70.0, 194.0, 358.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            f = pdf.CreateTextField("AbwEmpfStrasse", -1, false, 25, 70.0, 170.0, 358.0, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            f = pdf.CreateTextField("AbwEmpfPLZ", -1, false, 5, 70.0, 146.5, 70.5, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);
            pdf.SetNumberFormat(f, DynaPDF.TDecSeparator.dsNoneDot, 0, DynaPDF.TNegativeStyle.nsRed, null, false);

            f = pdf.CreateTextField("AbwEmpfWohnort", -1, false, 20, 142.5, 146.5, 285.5, 14.0);
            pdf.SetFieldFlags(f, DynaPDF.TFieldFlags.ffComb, false);

            // Add a reset button to the form
            pdf.SetFieldBorderColor(CPDF.PDF_BLACK);
            pdf.SetFieldBackColor(CPDF.PDF_MOGREEN);
            pdf.SetLineWidth(1.0);
            pdf.SetFont("Helvetica", DynaPDF.TFStyle.fsBold, 12.0, true, DynaPDF.TCodepage.cp1252);
            f = pdf.CreateButtonA("Reset", "Zurcksetzen", -1, 440.0, 730.0, 120.0, 25.0);
            pdf.SetFieldBorderStyle(f, DynaPDF.TBorderStyle.bsBevelled);
            int act = pdf.CreateResetAction();
            pdf.AddActionToObj(DynaPDF.TObjType.otField, DynaPDF.TObjEvent.oeOnMouseUp, act, f);

            pdf.SetCharacterSpacing(0.0);
            pdf.SetFont("Courier", DynaPDF.TFStyle.fsNone, 12.0, true, DynaPDF.TCodepage.cp1252);
            pdf.WriteTextA(53.0, 48.0, DateTime.Now.ToString());

            pdf.SetFillColor(pdf.RGB(0xFF, 0x66, 0x66));
            pdf.SetFont("Helvetica", DynaPDF.TFStyle.fsBold, 22.0, false, DynaPDF.TCodepage.cp1252);
            pdf.WriteTextA(340.0, 70.0, "www.dynaforms.de");

            pdf.SetLineWidth(0.0);
            pdf.SetLinkHighlightMode(DynaPDF.THighlightMode.hmPush);
            pdf.SetAnnotFlags(DynaPDF.TAnnotFlags.afReadOnly);
            pdf.WebLinkA(340.0, 64.0, 204.0, 22.0, "http://www.dynaforms.de");

            String alertScript = "if (app.viewerVersion >= 7.0 && app.viewerVersion < 8.0)\n" +
               "{\n" +
               "app.alert(\"Due to bugs in Acrobat 7.0 character spacing does not work" +
               " in text fields. We use the flag ffComb instead which is supported since" +
               " Acrobat 6.0. However, if the field's format is controled by a JavaScript" +
               " Action, we cannot set the flag because Acrobat 6.0 is then unable" +
               " to display the field's text. Acrobat 7.0 has problems to place the" +
               " the cursor in such fields but the final format will be correctly applied." +
               " The problem is that if the form should work with Acrobat 6.0 we cannot set" +
               " the flag ffComb to the date fields. It is also impossible to get the same" +
               " result as with character spacing with the flag ffComb. So, what can we do to make the" +
               " form compatible to all Acrobat versions? I really don't know! The easiest way" +
               " would be to avoid the usage of such date formats." +
               " However, the bug is already solved in Acrobat 8!\");\n" +
               "}";
            pdf.AddJavaScriptA("VersionCheck", alertScript);

            pdf.EndPage();

            // No fatal error occurred?
            if (pdf.HaveOpenDoc())
            {
               // We write the output file into the current directory.
               String filePath = Directory.GetCurrentDirectory();
               filePath += "\\out.pdf";
               // OK, now we can open the output file.
               if (!pdf.OpenOutputFile(filePath)) return;
               if (pdf.CloseFile())
               {
                  Console.Write("PDF file \"{0}\" successfully created!\n", filePath);
                  System.Diagnostics.Process p = new System.Diagnostics.Process();
                  p.StartInfo.FileName = filePath;
                  p.Start();
               }
            }
            pdf = null;
         }catch(Exception e)
         {
            Console.Write(e.Message);
         }
         Console.Read();
		}
	}
}
