using System;
using System.IO;
using DynaPDF;
using PDFUtils;

namespace edit_text
{
	class Edit_Text
	{
      /*
         Note that the dynapdf.dll must be copied into the output directory or into a
         Windwos search path (e.g. %WINDOWS%/System32) before the application can be executed!
      */

      // Error callback function.
      // If the function name should not appear at the beginning of the error message set
      // the flag emNoFuncNames (pdf.SetErrorMode(DynaPDF.TErrMode.emNoFuncNames);).
      static int PDFError(IntPtr Data, int ErrCode, IntPtr ErrMessage, int ErrType)
      {
         // We ignore warnings like "Font not found!" since this case is handled in the function CPDFEditText.SetFont().
         if ((ErrType & (uint)TErrType.E_WARNING) != 0) return 0;
         Console.WriteLine("{0}", System.Runtime.InteropServices.Marshal.PtrToStringAnsi(ErrMessage));
         return 0; // We try to continue if an error occurrs. Any other return value breaks processing.
      }

		[STAThread]
		static void Main(string[] args)
		{
         try
         {
            CPDF pdf = new CPDF();
            CPDFEditText parser = new CPDFEditText(pdf);
            // Error messages and warnings are passed to the callback function.
            pdf.SetOnErrorProc(IntPtr.Zero, new DynaPDF.TErrorProc(PDFError));
            pdf.CreateNewPDF(null); // We open the output file later if no error occurrs.

            // We import the contents only, without conversion of pages to templates
            pdf.SetImportFlags(DynaPDF.TImportFlags.ifContentOnly | DynaPDF.TImportFlags.ifImportAsPage);
            if (pdf.OpenImportFileA("../../../../test_files/taxform.pdf", DynaPDF.TPwdType.ptOpen, null) < 0)
            {
               Console.Write("Input file \"../../../../test_files/taxform.pdf\" not found!\n");
               Console.Read();
               return;
            }
            pdf.ImportPDFFile(1, 1.0, 1.0);
            pdf.CloseImportFile();

            int found;
            int pageCount = pdf.GetPageCount();
            for (int i = 1; i <= pageCount; i++)
            {
               if (!pdf.EditPage(i)) break;
               /*
                  The function FindPattern() searches for a string and stores all records in which it
                  was found in the array CPDFEditText.m_Records. Each record in the array represents
                  exactly one position on the page where the string was found. The coordinates of
                  text records are not stored at this time but it should be no problem to extend
                  the class if necessary.

                  The search run is case-sensitive. If you want to change the text search algorithm
                  you must modify the protected functions CPDFEditText.FindPattern() and
                  CPDFEditText.FindEndPattern().
               */
               found = parser.FindPattern("2002");
               /*
                  ReplacePattern() replaces the text with a new one or deletes it if the new string
                  is an empty string. The new text is written in red color so that you can better
                  find it. Take a look into the function CPDFEditText.WriteText() if you want to
                  write the text in the orignal color...
               */
               if (found > 0) parser.ReplacePattern("2010");
               Console.Write("Replaced text on page {0} {1} times!\n", i, found);
               pdf.EndPage();
            }
            // No fatal error occurred?
            if (pdf.HaveOpenDoc())
            {
               // We write the output file into the current directory.
               String filePath = Directory.GetCurrentDirectory();
               filePath += "\\out.pdf";
               // OK, now we can open the output file.
               if (!pdf.OpenOutputFile(filePath)) return;
               if (pdf.CloseFile())
               {
                  Console.Write("PDF file \"{0}\" successfully created!\n", filePath);
                  System.Diagnostics.Process p = new System.Diagnostics.Process();
                  p.StartInfo.FileName = filePath;
                  p.Start();
               }
            }
            pdf = null;
         }
         catch(Exception e)
         {
            // DynaPDF error messages are already handled!
            if (e.Message != "-1") Console.Write(e.Message);
         }
         Console.Read();
		}
	}
}
