/*---------------------------------------------------------+
| DynaPDF 2.5                                              |
+----------------------------------------------------------+
| Copyright (C) 2003-2011                                  |
| Jens Boschulte, DynaForms GmbH.                          |
| All rights reserved.                                     |
+----------------------------------------------------------+
| Please report errors or other potential problems to      |
| support@dynaforms.com.                                   |
| The actual version is available at www.dynaforms.com.    |
+----------------------------------------------------------|
|                Creation date: 17.10.2011                 |
+----------------------------------------------------------*/

#ifndef dynapdfH
#define dynapdfH

#define DYNAPDF_VERSIONSTRING "2.5.4.567"

#define DYNAPDF_REVISION "$Rev: 124 $"

#if defined(__BORLANDC__)
   /*
      Treat enums as int because an enum is defined as 16 bit short by default in
      Borland's C++ Builder. DynaPDF was compiled with MS Visual C++ 6.0 under
      Windows or GCC 3.2 or higher for other operating systems. These compilers
      treat enums as 32 bit integer.
   */
   #pragma option push -b
#endif

/* ------------------------------------------ IMPORTANT --------------------------------------------- */

// When compiling DynaPDF as Windows DLL the makro PDF_INTERN and DRV_GLOBAL must be defined in your DynaPDF
// project settings. But this makro must NOT be defined in your application project that includes this interface!!!

// If you want to compile the library and bind it statically with VC C++, declare PDF_STATIC in your
// project settings and include the following workspaces to your main worksapce:

   // dynapdf_static.dsp  -> main project for DynaPDF
   // aicrypt/aicrypt.dsp -> AiCrypto library
   // gpc/gpc.dsp         -> Generic Polygon Clipper
   // jasper/jasper.dsp   -> Jasper library
   // jpeg/jpeg.dsp       -> JPEG library
   // png/png.dsp         -> PNG library
   // tiff/tiff.dsp       -> Tiff library
   // zlib/zlib.dsp       -> Zlib library

// The workspace dynapdf.dsp depends on the other workspaces listed above. Set the dependencies under the menu
// Project/Dependencies. Your project depends on dynapdf_static.dsp, so set this dependency too. At least, you
// must add the library path ..\Debug or ..\Release and the gdi32.lib to your project, finished!

// An easier way is to use the workspace dynapdf_static.dsw. It contains already all necessary workspaces and
// settings which must be used to compile DynaPDF. Take a look into this workspace or use it directly if possible.

/* ---------------------------------------- START INTERFACE ----------------------------------------- */

// PDF_DEBUG is used internally by DynaForms, there is no need to use this flag.

//#define PDF_DEBUG

// DynaPDF supports certain compiler settings to enable or disable image libraries or other features.
// All supported configuration macros can be found in the file /main/drv_type.h.

#if !defined(_WINDOWS) && (defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(WIN64) || defined(_WIN64) || defined(WINDOWS))
   #define _WINDOWS
#endif

#if (!defined(USE_CLASS_CPDF) && !defined(PDF_STATIC))
   #if defined(_WINDOWS) && defined(__cplusplus)
      #if defined(PDF_INTERN)
         #define PDF_EXP extern "C"
      #else
         #define PDF_EXP extern "C" __declspec(dllimport)
      #endif
   #elif defined(_WINDOWS)
      #define PDF_EXP extern __declspec(dllimport)
   #elif ((defined(__IBMC__) || defined(__IBMCPP__)) && defined(__DLL__) && defined(OS2))
      #define PDF_EXP _Export
   #elif __cplusplus
      #if defined(__GNUC__) && (__GNUC__ >= 4) // the visibility attribute is supported since GCC 4.0
         #define PDF_EXP extern "C" __attribute ((visibility ("default")))
      #else
         #define PDF_EXP extern "C"
      #endif
   #else
      #if defined(__GNUC__) && (__GNUC__ >= 4) // the visibility attribute is supported since GCC 4.0
         #define PDF_EXP __attribute ((visibility ("default")))
      #else
         #define PDF_EXP
      #endif
   #endif
#else // static binding
   #ifdef __cplusplus
      #define PDF_EXP extern "C"
   #else
      #define PDF_EXP extern
   #endif
#endif

#if defined(_WINDOWS) && !defined(PDF_NO_STDCALL)
   #define PDF_CALL __stdcall
#else
   #define PDF_CALL
#endif

#ifdef __cplusplus
   #define ADDR &
#else
   #define ADDR *
#endif

#if defined(DRV_GLOBAL)
   #include "../main/drv_type.h"
   #ifdef __cplusplus
      using namespace DOCDRV;
   #endif
#else
   #ifndef _WINDOWS
      #include "drv_conf.h"

      #ifdef MAC_OS_X
         // We must not depend on the configure script when building a universal library!
         #if defined(__BIG_ENDIAN__)
            #define DRV_BIG_ENDIAN 1
         #elif defined(__LITTLE_ENDIAN__)
            #define DRV_BIG_ENDIAN 0
         #else
            #error "Cannot determine endian configuration!"
         #endif
      #elif (defined(WORDS_BIGENDIAN) && WORDS_BIGENDIAN == 1)
         #define DRV_BIG_ENDIAN 1
      #else
         #define DRV_BIG_ENDIAN 0
      #endif

      #if (defined(HAVE_PTHREAD_H) && HAVE_PTHREAD_H == 1)
         #define DRV_LOCK_CRITICAL_SECTIONS 1
      #else
         #define DRV_LOCK_CRITICAL_SECTIONS 0
      #endif
      #define VS_2005_OR_HIGHER 0
   #else
      #if defined(_MSC_VER) && (_MSC_VER >= 1400)
         #define VS_2005_OR_HIGHER 1
      #else
         #define VS_2005_OR_HIGHER 0
      #endif
      #define DRV_BIG_ENDIAN 0
      #define DRV_LOCK_CRITICAL_SECTIONS 1
      #define SIZEOF_WCHAR_T 2
   #endif

   #ifdef _WINDOWS
      #if defined(WIN64) || defined(_WIN64)
         #if !defined(SI32)
            typedef int SI32;
         #endif
         #if !defined(UI32)
            typedef unsigned int UI32;
         #endif
      #else
         #if !defined(SI32)
            typedef long SI32;
         #endif
         #if !defined(UI32)
            typedef unsigned long UI32;
         #endif
      #endif
   #elif defined(MAC_OS_X)
      #if !defined(SI32)
         typedef int SI32;
      #endif
      #if !defined(UI32)
         typedef unsigned int UI32;
      #endif
   #else
      #if (SIZEOF_INT == 4)
         #if !defined(SI32)
            typedef int SI32;
         #endif
         #if !defined(UI32)
            typedef unsigned int UI32;
         #endif
      #elif (SIZEOF_LONG == 4)
         #if !defined(SI32)
            typedef long SI32;
         #endif
         #if !defined(UI32)
            typedef unsigned long UI32;
         #endif
      #else
         #error "Only 32 bit and 64 bit targets are supported!"
      #endif
   #endif

   typedef SI32           LBOOL;
   typedef unsigned char  BYTE;
   typedef signed short   SI16;
   typedef unsigned short UI16;
   typedef double         FFLOAT; /* Obsolete, this typedef is not longer used by DynaPDF */
   #define DRV_FLOAT_PRECISION 5

   /* --------------------------------------------- Low level file handling ----------------------------------------- */

   #define FM_CREATE          "w+b"
   #define FM_OPEN_READ       "rb"
   #define FM_OPEN_WRITE      "r+b"
   #define FM_OPEN_READ_WRITE "r+b"

   #if defined(_WINDOWS) && !defined(__BORLANDC__)
      #define SM_DENY_NONE       _SH_DENYNO
      #define SM_DENY_READ       _SH_DENYRD
      #define SM_DENY_WRITE      _SH_DENYWR
      #define SM_DENY_READ_WRITE _SH_DENYRW
   #elif defined(_WINDOWS)
      #define SM_DENY_NONE       SH_DENYNO
      #define SM_DENY_READ       SH_DENYRD
      #define SM_DENY_WRITE      SH_DENYWR
      #define SM_DENY_READ_WRITE SH_DENYRW
   #else
      #define SM_DENY_NONE       0
      #define SM_DENY_READ       0
      #define SM_DENY_WRITE      0
      #define SM_DENY_READ_WRITE 0
   #endif

   /* ----------------------------------------------- Default color values ------------------------------------------ */

   // basic RGB colors
   #define PDF_AQUA     0x00FFFF00
   #define PDF_BLACK    0x00000000
   #define PDF_BLUE     0x00FF0000
   #define PDF_CREAM    0x00F0FBFF
   #define PDF_DKGRAY   0x00808080
   #define PDF_FUCHSIA  0x00FF00FF
   #define PDF_GRAY     0x00808080
   #define PDF_GREEN    0x00008000
   #define PDF_LIME     0x0000FF00
   #define PDF_LTGRAY   0x00C0C0C0
   #define PDF_MAROON   0x00000080
   #define PDF_MEDGRAY  0x00A4A0A0
   #define PDF_MOGREEN  0x00C0DCC0
   #define PDF_NAVY     0x00800000
   #define PDF_OLIVE    0x00008080
   #define PDF_PURPLE   0x00800080
   #define PDF_RED      0x000000FF
   #define PDF_SILVER   0x00C0C0C0
   #define PDF_SKYBLUE  0x00F0CAA6
   #define PDF_WHITE    0x00FFFFFF
   #define PDF_TEAL     0x00808000
   #define PDF_YELLOW   0x0000FFFF

   // This value can be used for form fields, annotations and standard patterns to determine that the specific color is not set.
   // For example, when a form field should have a transparent background, set the field background color
   // to NO_COLOR. The background appears then transparent.
   #define NO_COLOR     0xFFFFFFF1

   /* ------------------------------------------------ Callback functions ------------------------------------------- */

   typedef enum
   {
      ftMMType1  = 0, // Multiple Master
      ftTrueType = 1, // TrueType font
      ftType0    = 2, // CID font -> Check the font file type to determine the font sub type.
      ftType1    = 3, // Type1 font
      ftType3    = 4  // Type3 font
   }TFontType;

   typedef enum
   {
      fbtTrueType, // TrueType, TrueType Collections, or OpenType fonts with TrueType outlines
      fbtType1,    // Type1 font
      fbtOpenType, // OpenType font with Postscript outlines
      fbtStdFont,  // PDF Standard font
      fbtDisabled  // This value can be used to disable a specific font format. See SetFontSearchOrder() for further information.
   }TFontBaseType;

   typedef enum
   {
      ffsType1C        = 0, // CFF based Type1 font
      ffsCIDFontType0C = 1, // CFF based Type1 CID font
      ffsOpenType      = 2, // TrueType based OpenType font
      ffsOpenTypeC     = 3, // CFF based OpenType font
      ffsCIDFontType2  = 4, // TrueType based CID Font
      ffsNoSubtype     = 9  // The font file is in the format of FontType
   }TFontFileSubtype;

   typedef enum
   {
      ictGray = 0,
      ictRGB  = 1,
      ictCMYK = 2,
      ictLab  = 3
   }TICCProfileType;

   // Data is always a user defined pointer which is passed unchanged to the callback funtion.
   typedef SI32 PDF_CALL TErrorProc(const void* Data, SI32 ErrCode, const char* ErrMessage, SI32 ErrType);
   typedef SI32 PDF_CALL TEnumFontProc(const void* Data, const UI16* FamilyName, const char* PostScriptName, SI32 Style);
   typedef SI32 PDF_CALL TEnumFontProc2(const void* Data, const void* PDFFont, TFontType Type, const char* BaseFont, const char* FontName, LBOOL Embedded, LBOOL IsFormFont, UI32 Flags);
   typedef SI32 PDF_CALL TEnumFontProcEx(const void* Data, const UI16* FamilyName, const char* PostScriptName, SI32 Style, TFontBaseType BaseType, LBOOL Embeddable, const char* FilePath);
   typedef SI32 PDF_CALL TOnFontNotFoundProc(const void* Data, const void* PDFFont, const char* FontName, SI32 Style, SI32 StdFontIndex, LBOOL IsSymbolFont);
   //typedef SI32 PDF_CALL TOnNeedPassword(const void* Data, TPwdType ADDR Type, char* ADDR Password);
   typedef SI32 PDF_CALL TOnReplaceICCProfile(const void* Data, TICCProfileType Type, SI32 ColorSpace);

   /* -------------------------------------- Error types, specific error values ------------------------------------- */

   typedef SI32 TErrMode;
   #define emIgnoreAll    0x00000000
   #define emSyntaxError  0x00000001
   #define emValueError   0x00000002
   #define emWarning      0x00000004
   #define emFileError    0x00000008
   #define emFontError    0x00000010
   #define emAllErrors    0x0000FFFF
   #define emNoFuncNames  0x10000000 // Do not print function names in error messages

   // Error types.
   // Note that error codes are negative; to determine the error type use the absolute value:
   // if (-errCode & E_FATAL_ERROR) printf("A fatal error occurred!");
   #define E_WARNING      0x02000000
   #define E_SYNTAX_ERROR 0x04000000
   #define E_VALUE_ERROR  0x08000000
   #define E_FONT_ERROR   0x10000000
   #define E_FATAL_ERROR  0x20000000
   #define E_FILE_ERROR   0x40000000

   // Error codes are defined in ../main/drv_errcode.h
#endif

/*
   Prior versions of DynaPDF used the names LONG and ULONG for 32 bit integer types. These names are
   already defined under Windows. As long as no 64 bit Windows is used the names do not conflict
   with existing declarations. However, 64 bit Windows operating systems declare the types LONG and
   ULONG correctly as 64 bit integer types but we still need 32 bit integer types.

   To solve this issue, the types LONG and ULONG were renamed in DynaPDF 2.0.18.222. You can still use the
   the old names but if you plan to use DynaPDF under 64 bit Windows then it is a good idea to use either
   a predifined 32 bit integer type or the abbrevations SI32 or UI32 respectively.
*/
#if !defined(WIN64) && !defined(_WIN64)
   #if !defined(SI16)
      typedef SI16 SHORT;
   #endif
   #if !defined(UI16)
      typedef UI16 USHORT;
   #endif
   #if !defined(LONG)
      typedef SI32 LONG;
   #endif
   #if !defined(ULONG)
      typedef UI32 ULONG;
   #endif
#endif

// The macro ToUTF16 can be used to convert UTF-32 literal strings or UTF32 Unicode strings which use
// the data type wchar_t.
// Example: pdfWiteTextW(50.0, 50.0, ToUTF16(L"This is a literal string")); ToUTF16 is only requiered for
// Linux and UNIX operating systems.

#ifdef _WINDOWS
   #define ToUTF16(IPDF, s)((s))
#else
   #if (SIZEOF_WCHAR_T == 4)
      #define ToUTF16(IPDF, s)(pdfUTF32ToUTF16((IPDF), (UI32*)(s)))
   #else
      #define ToUTF16(IPDF, s)((s))
   #endif
#endif

#if (DRV_BIG_ENDIAN == 1)
   #define pdf_MakeWord(a, b)((UI16)(((BYTE)(b))|((UI16)((BYTE)(a)))<<8))
   #define pdf_MakeLong(a, b)((UI32)(((UI16)(b))|((UI32)((UI16)(a)))<<16))
#else
   #define pdf_MakeWord(a, b)((UI16)(((BYTE)(a))|((UI16)((BYTE)(b)))<<8))
   #define pdf_MakeLong(a, b)((UI32)(((UI16)(a))|((UI32)((UI16)(b)))<<16))
#endif

#define PDF_MAX_INT 0x7FFFFFFF

/* ----------------------------------------- Start namespace DynaPDF -------------------------------------------- */

#ifdef __cplusplus
namespace DynaPDF
{
#endif

/* -------------------------------------------- Callback functions ---------------------------------------------- */

#define PDF_MAX_LIST_COUNT 6    // Maximum count of nested list levels (WriteFText())
#define DEFAULT_LIST_CHAR  159  // Default character index of the font Wingdings-Regular to create a list (see WriteFText())
#define PDF_LIST_SEP_WIDTH 10.0 // Default width between the list symbol and text (see WriteFText() in the help file for further information)
#define PDF_LIST_FONT      "Wingdings-Regular" // Default font to display the list symbol -> can be changed with SetListFont()

/* ------------------------------------- Callback function for WriteFText ---------------------------------------- */

// The TOnPageBreakProc function is called by WriteFText when the rectangle was filled with text, or when a
// manual page break was inserted to the text and more text is available for output.
// You must set the text rectangle with SetTextRect to continue the text on a new position or simply call
// EndPage() and then Append() or EditPage() to continue the text on an arbitrary new page. You can make some
// changes inside the callback function such as setting another font, font color or other thinks.

// When the callback function returns a value greater or equal zero, processing continues on the new
// position or page that was set inside the callback function.

// The text alignment can also be changed with the following return values:

#define NEW_ALIGN_LEFT    1
#define NEW_ALIGN_RIGHT   2
#define NEW_ALIGN_CENTER  3
#define NEW_ALIGN_JUSTIFY 4

// When the return value of the callback function is one of the values listed above, then the alignment will
// be changed before the remaining text will be processed. A return value of 0 determines that processing should
// be continued without changing the alignment.

// A negative return value breaks the processing and WriteFText returns emmidiatly with a warning. Note that
// processing stops entirely when warnings should be handled as fatal errors (see SetErrorMode in the help
// file for further information).

// NOTE: The coordinate system is always set to bottom up when the callback function is called, independent
// of the coordinate system that was set before WriteFText was called. WriteFText restores the coordinate system
// and all other temporarily changed variables when the function is leaved. The last text position is
// also returned in bottom up coordinates by GetLastTextPos(). LastPosY is the position of the last baseline.

// You can change the coordinate system inside the callback function when needed, DynaPDF checks the state
// before processing continues.

// PageBreak is != 0 when a page break tag in the formatted text occurred.

typedef SI32 PDF_CALL TOnPageBreakProc(const void* Data, double LastPosX, double LastPosY, LBOOL PageBreak);

/* ------------------------------- Callback functions to control a progress bar ---------------------------------- */

typedef SI32 TProgType;
#define ptImportPage 0 // Start page import
#define ptWritePage  1 // Start writing a page to file or buffer

// TInitProgressProc is called before TProgressProc is called the first time.
typedef void PDF_CALL TInitProgressProc(const void* Data, TProgType ProgType, SI32 MaxCount);
typedef SI32 PDF_CALL TProgressProc(const void* Data, SI32 ActivePage);

#define ENEED_PWD        -(0x000000B2 | E_FILE_ERROR)
#define EWRONG_OPEN_PWD  -(0x000000B3 | E_FILE_ERROR)
#define EWRONG_OWNER_PWD -(0x000000B4 | E_FILE_ERROR)
#define EWRONG_PWD       -(0x000000B5 | E_FILE_ERROR)

// Returns true if a wrong password was passed to OpenImportFile() or OpenImportBuffer().
// ErrCode must be the return value of OpenImportFile() or OpenImportBuffer().
#define IsWrongPwd(ErrCode)((ErrCode)==ENEED_PWD||(ErrCode)==EWRONG_OPEN_PWD||(ErrCode)==EWRONG_OWNER_PWD||(ErrCode)==EWRONG_PWD)
#define PDF_WRONG_PWD IsWrongPwd // Obsolete

// recommended color macros
#define PDF_RGB(r,g,b) ((UI32)(((BYTE)(r)|((UI16)((BYTE)(g))<<8))|(((UI32)(BYTE)(b))<<16)))

#if (DRV_BIG_ENDIAN == 1)
   #define PDF_CMYK(c,m,y,k) ((UI32)((((BYTE)(c)|((UI16)((BYTE)(m))<<8))|(((UI32)(BYTE)(y))<<16))|(((UI16)(BYTE)(k))<<24)))
#else
   #define PDF_CMYK(c,m,y,k) ((UI32)((((BYTE)(k)|((UI16)((BYTE)(y))<<8))|(((UI32)(BYTE)(m))<<16))|(((UI16)(BYTE)(c))<<24)))
#endif

// Macro to extract the width class from a TFStyle variable
#define WidthFromStyle(s)(((s) & ~0xFFF2D00F) >> 8)
// Macro to extract the font weight from a TFStyle variable
#define WeightFromStyle(s)(((s) & ~0xC00FFFFF) >> 20)
// Macros to convert a width class or font weight to a style constant
#define WidthToStyle(w)((w) << 8)
#define WeightToStyle(w)((w) << 20)

#define PDF_TABLEN 3

#ifndef __cplusplus
   typedef enum
   {
      false = 0,
      true  = 1
   }bool;
#endif

typedef enum
{
   at3D_AppDefault,
   at3D_PageOpen,    // The annotaiton should be activated when the page is opened.
   at3D_PageVisible, // The annotaiton should be activated when the page becomes visible.
   at3D_Explicit     // The annotation should remain inactive until explicitly activated by a script or action (default).
}T3DActivationType;

typedef enum
{
   dt3D_AppDefault,
   dt3D_PageClosed,    // The annotaiton should be deactivated as soon as the page is closed.
   dt3D_PageInvisible, // The annotaiton should be deactivated as soon as the page becomes invisible (default).
   dt3D_Explicit       // The annotation should remain active until explicitly deactivated by a script or action.
}T3DDeActivateType;

typedef enum
{
   di3D_AppDefault,
   di3D_UnInstantiated, // The annotation will be uninstantiated (default)
   di3D_Instantiated,   // The annotation is left instantiated
   di3D_Live            // If the 3D artwork contains an animation then it will stay live
}T3DDeActInstance;

typedef enum
{
   it3D_AppDefault,
   it3D_Instantiated, // The annotation will be instantiated but animations are disabled.
   it3D_Live          // The annotation will be instantiated and animations are enabled (default).
}T3DInstanceType;

typedef enum
{
   lsArtwork, // Lights from file
   lsBlue,
   lsCAD,
   lsCube,
   lsDay,
   lsHard,
   lsHeadlamp,
   lsNight,
   lsNoLights,
   lsPrimary,
   lsRed,
   lsWhite,
   lsNotSet
}T3DLightingSheme;

typedef enum
{
   naDefault,
   naFirst,
   naLast,
   naNext,
   naPrevious
}T3DNamedAction;

typedef enum
{
   pt3DOrthographic,
   pt3DPerspective
}T3DProjType;

typedef enum
{
   rmBoundingBox,
   rmHiddenWireframe,
   rmIllustration,
   rmShadedIllustration,
   rmShadedVertices,
   rmShadedWireframe,
   rmSolid,
   rmSolidOutline,
   rmSolidWireframe,
   rmTransparent,
   rmTranspBBox,
   rmTranspBBoxOutline,
   rmTranspWireframe,
   rmVertices,
   rmWireframe,
   rmNotSet
}T3DRenderingMode;

typedef enum
{
   st3DValue,
   st3DWidth,
   st3DHeight,
   st3DMin,
   st3DMax
}T3DScaleType;

typedef enum
{
   atGoTo,
   atGoToR,
   atHide,
   atImportData,
   atJavaScript,
   atLaunch,
   atMovie,
   atNamed,
   atRendition,
   atReset,
   atSetOCGState,
   atSound,
   atSubmit,
   atThread,
   atTransition,
   atURI,
   atGoTo3DView,
   atGoToE       // PDF 1.6 Like atGoToR but refers to an embedded PDF file.
}TActionType;

typedef UI32 TAnnotFlags;
#define afNone       0x00000000
#define afInvisible  0x00000001
#define afHidden     0x00000002
#define afPrint      0x00000004
#define afNoZoom     0x00000008
#define afNoRotate   0x00000010
#define afNoView     0x00000020
#define afReadOnly   0x00000040

/*
   By default all annotations which have an appearance stream and which have the print flag set are flattened.
   All annotations are deleted when the function returns with the exception of file attachment annotations.
   If you want to flatten the view state then set the flag affUseViewState.
*/
typedef UI32 TAnnotFlattenFlags;
#define affNone         0x00000000 // Printable annotations independent of the type
#define affUseViewState 0x00000001 // If set, annotations which are visible in a viewer become flattened.
#define affMarkupAnnots 0x00000002 // If set, markup annotations are flattened only. Link, Sound, or FileAttach
                                   // annotations are no markup annotations. These types will be left intact.

typedef enum
{
   aiComment,
   aiHelp,
   aiInsert,
   aiKey,
   aiNewParagraph,
   aiNote,
   aiParagraph
}TAnnotIcon;

typedef enum
{
   asAuthor,
   asContent,
   asName,
   asSubject
}TAnnotString;

typedef enum
{
   atCaret,
   atCircle,
   atFileLink,
   atFreeText,
   atHighlight, // Highlight annotation
   atInk,
   atLine,
   atPageLink,
   atPolygon,
   atPolyLine,
   atPopUp,
   atSquare,
   atSquiggly,  // Highlight annotation
   atStamp,
   atStrikeOut, // Highlight annotation
   atText,
   atUnderline, // Highlight annotation
   atWebLink,
   atWidget,
   at3D,
   atSoundAnnot,
   atFileAttach,
   atRedact,
   atWatermark,
   atUnknown    // Unknown annotation type
}TAnnotType;

typedef enum
{
   bmNotSet     = 0,
   bmNormal     = 1,
   bmColor      = 2,
   bmColorBurn  = 3,
   bmColorDodge = 4,
   bmDarken     = 5,
   bmDifference = 6,
   bmExclusion  = 7,
   bmHardLight  = 8,
   bmHue        = 9,
   bmLighten    = 10,
   bmLuminosity = 11,
   bmMultiply   = 12,
   bmOverlay    = 13,
   bmSaturation = 14,
   bmScreen     = 15,
   bmSoftLight  = 16
}TBlendMode;

typedef enum
{
   bsSolid,
   bsBevelled,
   bsInset,
   bsUnderline,
   bsDashed
}TBorderStyle;

typedef enum
{
   bsUp,
   bsDown,
   bsRollOver
}TButtonState;

typedef enum
{
   cbUnknown,
   cbChecked,
   cbUnChecked
}TCheckBoxState;

typedef enum
{
   ccfBW_To_Gray  = 0, // Default, RGB Black and White set with rg or RG inline operators are converted to gray
   ccfRGB_To_Gray = 1  // If set, inline color operators rg and RG are converted to gray
}TColorConvFlags;

#if !defined(DRV_GLOBAL)

   typedef enum
   {
      cmEvenOdd = 0,
      cmWinding = 1
   }TClippingMode;

   typedef enum
   {
      /*---------------------------------------- 8 bit code pages --------------------------------------*/
      cp1250,
      cp1251,
      cp1252,
      cp1253,
      cp1254,
      cp1255,
      cp1256,
      cp1257,
      cp1258,
      cp8859_2,
      cp8859_3,
      cp8859_4,
      cp8859_5,
      cp8859_6,
      cp8859_7,
      cp8859_8,
      cp8859_9,
      cp8859_10,
      cp8859_13,
      cp8859_14,
      cp8859_15,
      cp8859_16,
      cpSymbol,
      cp437,
      cp737,
      cp775,
      cp850,
      cp852,
      cp855,
      cp857,
      cp860,
      cp861,
      cp862,
      cp863,
      cp864,
      cp865,
      cp866,
      cp869,
      cp874,

      /* ----------------------------------------- Unicode (UCS-2) -------------------------------------*/

      cpUnicode,

      /*-------------------------------- CJK encodings for use with Unicode fonts ----------------------*/
      // The following endcodings requires CJK or ISO-2022 input character sequences which are converted to Unicode
      // by DynaPDF. It is not possible to use these encodings with CJK fonts. If a CJK font should be used, choose
      // one of the native CJK character sets listed at the end this block. DynaPDF does not support a conversion
      // algorithm to convert ISO-2022 character sequences to CJK code points. If you needed such a mapping, please
      // send a mail to support@dynaforms.com.

      // UCS-2 does not support all available CJK code points. However, surrogates are still accessible if the used
      // TrueType font contains a cmap format 12. Another way is to use a CJK font with a native CJK charset.

      cpCJK_Big5_Uni,    // Big5 plus HKSCS extension.
      cpCJK_EUC_JP_Uni,  // EUC-JP.
      cpCJK_EUC_KR_Uni,  // EUC-KR.
      cpCJK_EUC_TW_Uni,  // CNS-11643-1992 (Planes 1-15).
      cpCJK_GBK_Uni,     // GBK is the Microsoft code page 936 (GB2312, EUC-CN plus GBK extension).
      cpCJK_GB12345_Uni, // GB-12345-1990 (Traditional Chinese form of GB-2312).
      cpCJK_HZ_Uni,      // Mixed ASCII / GB-2312 encoding
      cpCJK_2022_CN_Uni, // ISO-2022-CN-EXT (GB-2312 plus ISO-11643 Planes 1-7).
      cpCJK_2022_JP_Uni, // ISO-2022-JP.
      cpCJK_2022_KR_Uni, // ISO-2022-KR.
      cpCJK_646_CN_Uni,  // ISO-646-CN (GB-1988-80).
      cpCJK_646_JP_Uni,  // ISO-646-JP (JIS_C6220-1969-RO).
      cpCJK_IR_165_Uni,  // ISO-IR-165 (extended version of GB-2312).
      cpCJK_932_Uni,     // Microsoft extended version of SHIFT_JIS.
      cpCJK_949_Uni,     // EUC-KR extended with UHC (Unified Hangul Codes).
      cpCJK_950_Uni,     // Microsoft extended version of Big5.
      cpCJK_JOHAB_Uni,   // JOHAB.

      /*---------------------------------- Native CJK charsets (CJK fonts only) ------------------------*/

      // When using a CJK font, DynaPDF creates dynamically a "To Unicode CMAP" to enable cut & paste
      // operations in Adobe's Acrobat. If a CJK sequence has no Unicode code point, the CJK code is used instead.
      // In this case, cut & paste works for all other characters, but the unmapped character appears as arbitrary
      // wrong character when the string is paste to another document. A wrongly mapped character in the
      // "To Unicode CMAP" has no effect in viewing or printing the document.

      // The comments describe which mapping tables are available for a character set. Note these are character sets
      // and no code pages.

      cpShiftJIS,     // ShiftJIS charset plus code page 932 extension.
      cpBig5,         // Big5 plus HKSCS extension.
      cpGB2312,       // GB2312 charset plus GBK and cp936 extension.
      cpWansung,      // Wansung
      cpJohab,        // Johab
      cpMacRoman,     // Mac Roman
      cpAdobeStd,     // This is an encoding for Type1 fonts. It should normally not be used.
      cpInternal,     // Internal -> not usable
      cpGlyphIndexes, // Can be used with TrueType and OpenType fonts only. DynaPDF creates a reverse mapping so that copy & paste will work.
      cpPDFDocEnc,    // Internal -> not usable. Used for form fonts. This is a superset of the code page 1252 and MacRoman.
      cpExtCMap,      // Internal -> not usable. This code page is set when a font was loaded with an external cmap.
      cpDingbats      // Internal -> Special encoding for ZapfDingbats
   }TCodepage;

   typedef enum
   {
      cmFill       = 0,
      cmStroke     = 1,
      cmFillStroke = 2
   }TColorMode;

   typedef enum
   {
      cfFlate    = 0, // PDF or TIFF output
      cfJPEG     = 1, // PDF, JPEG, or TIFF output
      cfCCITT3   = 2, // PDF or TIFF output -> B&W CCITT Fax G3 compression -> fast but less compression ratio
      cfCCITT4   = 3, // PDF or TIFF output -> B&W CCITT Fax G4 compression -> slower but higher compression ratio
      cfLZW      = 4, // TIFF or GIF output -> Very fast but less compression ratios than flate
      cfReserved = 5, // Reserved for future extensions.
      cfFlateBW  = 6, // TIFF, PNG, or BMP output -> Dithered black & white output. The resulting image will be
                      // compressed with Flate or left uncompressed if the output image format is a bitmap. If
                      // you want to use CCITT Fax 4 compression (TIFF only) set the flag icUseCCITT4 in the
                      // AddImage() function call. Note that this filter is not supported for PDF creation!
      cfJP2K     = 7  // PDF or JPEG2000 output
   }TCompressionFilter;

   typedef enum
   {
      clNone    = 0,
      clDefault = 1,
      clFastest = 2,
      clMax     = 3
   }TCompressionLevel;

   struct TCTM
   {
      double a;
      double b;
      double c;
      double d;
      double x;
      double y;
   };

   typedef enum
   {
      diAuthor   = 0,
      diCreator  = 1,
      diKeywords = 2,
      diProducer = 3,
      diSubject  = 4,
      diTitle    = 5,
      diCompany  = 6,
      diPDFX_Ver = 7, // GetInDocInfo() only -> The PDF/X version is set by SetPDFVersion()!
      diCustom   = 8  // User defined key
   }TDocumentInfo;

   typedef enum
   {
      ddCounterClockwise = 0,
      ddClockwise        = 1
   }TDrawDirection;

   typedef enum
   {
      dmNormal         = 0,
      dmStroke         = 1,
      dmFillStroke     = 2,
      dmInvisible      = 3,
      dmFillClip       = 4,
      dmStrokeClip     = 5,
      dmFillStrokeClip = 6,
      dmClipping       = 7,
      dmNone           = -1 // Internal, do not use this mode!!!
   }TDrawMode;

   typedef enum
   {
      esDeviceRGB  = 0,  // Device color space
      esDeviceCMYK = 1,  // Device color space
      esDeviceGray = 2,  // Device color space
      esCalGray    = 3,  // CIE-based color space
      esCalRGB     = 4,  // CIE-based color space
      esLab        = 5,  // CIE-based color space
      esICCBased   = 6,  // ICC-based color space -> contains an ICC profile
      esPattern    = 7,  // Special color space
      esIndexed    = 8,  // Special color space
      esSeparation = 9,  // Special color space
      esDeviceN    = 10, // Special color space
      esNChannel   = 11, // Special color space
      esInitSpace  = -1  // Internal -> This value must not be used!
   }TExtColorSpace;

   // If the font selection mode is set to smPostScriptName the font styles fsItalic or fsBold are ignored.
   // The PostScriptName is already a unique font name incl. style information.
   typedef enum
   {
      smFamilyName     = 0,
      smPostScriptName = 1
   }TFontSelMode;

   /*
      The font search run works as follows:

         - DynaPDF tries always to find the exact weight, if it cannot be found then a font with
           the next smaller weight is selected (if available).
         - Italic styles can always be emulated but it is not possible to emulate thinner weights or
           regular styles with an italic font.
         - If the specified weight is larger as the font weight the remaining weight will be emulated
           if the difference to the requested weight is larger than 200.
           With SetFontWeight() it is possible to control whether a missing weight should be emulated.
           If FontWeight is smaller or equal to the requested font weight then emulation will be disabled.
   */
   /*
      TFStyle is a bitmask that is defined as follows:

         - Bits 0..7   // Style bits fsItalic, fsUnderlined, fsStriked
         - Bits 8..19  // Width class -> Defined for future use.
         - Bits 20..31 // Font Weight

      - A width class can be converted to a style constant by multiplying it with 256 (width << 8).
      - A font weight can be converted to a style constant by multiplying it with 1048576 (weight << 20).
      - Additional attributes can be added with a binary or operator (e.g. style |= fsItalic).
      - Only one width class and one font weight can be set at time.

      - The macro WidthFromStyle() extracts the width class.
      - The macro WeightFromStyle() extracts the font weight.

      The following macros extract the width class or font weight from a style variable:

         UI16 widthClass  = WidthFromStyle(style);
         UI16 weightClass = WeightFromStyle(style);
   */
   #if defined(__BORLANDC__)
      typedef enum
      {
         fsNone           = 0x00000000, // Regular weight (400)
         fsItalic         = 0x00000001,
         fsUnderlined     = 0x00000004,
         fsStriked        = 0x00000008,
         fsVerticalMode   = 0x00000010, // Not considered at this time
         // Width class
         fsUltraCondensed = 0x00000100, // 1
         fsExtraCondensed = 0x00000200, // 2
         fsCondensed      = 0x00000300, // 3
         fsSemiCondensed  = 0x00000400, // 4
         fsNormal         = 0x00000500, // 5
         fsSemiExpanded   = 0x00000600, // 6
         fsExpanded       = 0x00000700, // 7
         fsExtraExpanded  = 0x00000800, // 8
         fsUltraExpanded  = 0x00000900, // 9
         // Weight class
         fsThin           = 0x06400000, // 100
         fsExtraLight     = 0x0c800000, // 200
         fsLight          = 0x12C00000, // 300
         fsRegular        = 0x19000000, // 400 -> Same as fsNone
         fsMedium         = 0x1F400000, // 500
         fsDemiBold       = 0x25800000, // 600
         fsBold           = 0x2BC00000, // 700 -> The old constant 2 is still supported to preserve backward compatibility
         fsExtraBold      = 0x32000000, // 800
         fsBlack          = 0x38400000, // 900
         fsUltraBlack     = 0x3E800000  // 1000
      }TFStyle;
   #else
      typedef SI32 TFStyle;
      #define fsNone           0x00000000 // Regular weight (400)
      #define fsItalic         0x00000001
      #define fsUnderlined     0x00000004
      #define fsStriked        0x00000008
      #define fsVerticalMode   0x00000010 // Not considered at this time
      // Width Class
      #define fsUltraCondensed 0x00000100 // 1
      #define fsExtraCondensed 0x00000200 // 2
      #define fsCondensed      0x00000300 // 3
      #define fsSemiCondensed  0x00000400 // 4
      #define fsNormal         0x00000500 // 5
      #define fsSemiExpanded   0x00000600 // 6
      #define fsExpanded       0x00000700 // 7
      #define fsExtraExpanded  0x00000800 // 8
      #define fsUltraExpanded  0x00000900 // 9
      // Weight Class
      #define fsThin           0x06400000 // 100
      #define fsExtraLight     0x0C800000 // 200
      #define fsLight          0x12C00000 // 300
      #define fsRegular        0x19000000 // 400 -> Same as fsNone
      #define fsMedium         0x1F400000 // 500
      #define fsDemiBold       0x25800000 // 600
      #define fsBold           0x2BC00000 // 700 -> The old constant 2 is still supported to preserve backward compatibility
      #define fsExtraBold      0x32000000 // 800
      #define fsBlack          0x38400000 // 900
      #define fsUltraBlack     0x3E800000 // 1000
   #endif

   typedef UI32 TGStateFlags;
   #define gfCompatible         0x00000000 // Compatible graphics state to earlier DynaPDF versions -> default
   #define gfRestorePageCoords  0x00000001 // Restore the coordinate system with the graphics state (the value of PageCoords, see SetPageCoords())
   #define gfRealTopDownCoords  0x00000002 // If set, the page coordinate system is not reset to bottom-up when transforming
                                           // the coordinate system. However, real top-down coordinates require a large internal
                                           // overhead and where never fully implemented. The usage of this flag should be avoided
                                           // if possible.
   #define gfNativeBlackWhite   0x00000004 // Do not convert RGB black or white to DeviceGray
   #define gfUseImageColorSpace 0x00000008 // If set, the active color space is ignored when inserting an image. The color space is taken
                                           // from the image file instead.
   #define gfIgnoreICCProfiles  0x00000010 // Meaningful only if the flag gfUseImageColorSpace is set. If set, an embedded profile is not used to
                                           // create an ICCBased color space for the image. The image is inserted in the corresponding device
                                           // color space instead.

   /*
      TIFF is the only format that supports different compression filters. The Filter parameter of the function
      AddImage() is ignored if the image format supports only one specific compression filter.
      Note that images are automatically converted to the nearest supported color space if the image format does
      not support the color space of the image.
   */
   typedef enum
   {
      ifmTIFF,     // DeviceRGB, DeviceCMYK, DeviceGray, Black & White -> CCITT Fax Group 3/4, JPEG, Flate, LZW.
      ifmJPEG,     // DeviceRGB, DeviceCMYK, DeviceGray    -> JPEG compression.
      ifmPNG,      // DeviceGray, DeviceRGB, Black & White -> Flate compression.
      ifmReserved, // Reserved for future extensions.
      ifmBMP,      // DeviceGray, DeviceRGB, Black & White -> Uncompressed.
      ifmJPC       // DeviceRGB, DeviceCMYK, DeviceGray    -> JPEG2000 compression.
   }TImageFormat;

   typedef enum
   {
      kl40bit    = 0, // RC4 Encryption -> Acrobat 3 or higher
      kl128bit   = 1, // RC4 Encryption -> Acrobat 5 or higher
      kl128bitEx = 2, // RC4 Encryption -> Acrobat 6 or higher
      klAES128   = 3, // AES Encryption -> Acrobat 7 or higher
      klAES256   = 4  // AES Encryption -> Acrobat 9 or higher
   }TKeyLen;

   typedef enum
   {
      csButtCap    = 0,
      csRoundCap   = 1,
      csSquareCap  = 2
   }TLineCapStyle;

   typedef enum
   {
      jsMiterJoin = 0,
      jsRoundJoin = 1,
      jsBevelJoin = 2
   }TLineJoinStyle;

   typedef UI32 TMetaFlags;
   #define mfDefault           0x00000000 // Default conversion
   #define mfDebug             0x00000001 // Write debug information into the content stream
   #define mfShowBounds        0x00000002 // Show the bounding boxes of text strings
   #define mfNoTextScaling     0x00000004 // Do not scale text
   #define mfClipView          0x00000008 // Draw the file into a clipping rectangle
   #define mfUseRclBounds      0x00000010 // Use rclBounds instead of rclFrame
   #define mfNoClippingRgn     0x00000040 // Disables SelectClippingRegion, IntersectClipRect, and SelectClipPath
   #define mfNoFontEmbedding   0x00000080 // Do not embed fonts -> Fonts should be embedded!!!
   #define mfNoImages          0x00000100 // Ignore image records
   #define mfNoStdPatterns     0x00000200 // Ignore standard patterns
   #define mfNoBmpPatterns     0x00000400 // Ignore bitmap patterns
   #define mfNoText            0x00000800 // Ignore text records
   #define mfUseUnicode        0x00001000 // Ignore ANSI_CHARSET
   #define mfUseTextScaling    0x00004000 // Scale text instead of using the intercharacter spacing array
   #define mfNoUnicode         0x00008000 // Avoid usage of Unicode fonts -> recommended to enable PDF 1.2 compability
   #define mfFullScale         0x00010000 // Recommended if 32 bit coordinates are used
   #define mfUseRclFrame       0x00020000 // This flag should be set if the rclFrame rectangle is not properly set
   #define mfDefBkModeTransp   0x00040000 // Initialize the background mode to transparent (SetBkMode() overrides this state).
   #define mfApplyBidiAlgo     0x00080000 // Apply the bidirectional algorithm on Unicode strings
   #define mfGDIFontSelection  0x00100000 // Use the GDI to select fonts
   #define mfRclFrameEx        0x00200000 // If set, and if the rclBounds rectangle is larger than rclFrame, the function
                                          // extends the output rectangle according to rclBounds and uses the resulting
                                          // bounding box to calculate the image size (rclBounds represents the unscaled
                                          // image size). This is probably the correct way to calculate the image size.
                                          // However, to preserve backward compatibility the default calculation cannot
                                          // be changed.
   #define mfNoTextClipping    0x00400000 // If set, the ETO_CLIPPED flag in text records is ignored.
   #define mfSrcCopy_Only      0x00800000 // If set, images which use a ROP code other than SRCCOPY are ignored. This is useful when processing Excel 2007 spool files.
   #define mfClipRclBounds     0x01000000 // If set, the graphic is drawn into a clipping path with the size of rclBounds.
                                          // This flag is useful if the graphic contains content outside of its bounding box.
   #define mfDisableRasterEMF  0x02000000 // If set, EMF files which use unsupported ROP codes are not rastered.

   // Obsolete flags -> these flags are ignored, do no longer use them!
   #define mfUseSpacingArray   0x00000020 // enabled by default -> can be disabled with mfUseTextScaling
   #define mfIntersectClipRect 0x00002000 // enabled by default -> can be disabled with mfNoClippingRgn

   typedef enum
   {
      orDownLeft = 0,
      orTopLeft  = 1
   }TOrigin;

   typedef enum
   {
      pbArtBox     = 0,
      pbBleedBox   = 1,
      pbCropBox    = 2,
      pbTrimBox    = 3,
      pbMediaBox   = 4
   }TPageBoundary;

   typedef enum
   {
      pcBottomUp = 0,
      pcTopDown  = 1
   }TPageCoord;

   typedef enum
   {
      pfDIN_A3     = 0,
      pfDIN_A4     = 1,
      pfDIN_A5     = 2,
      pfDIN_B4     = 3,
      pfDIN_B5     = 4,
      pfDIN_B6     = 5,
      pfDIN_C3     = 6,
      pfDIN_C4     = 7,
      pfDIN_C5     = 8,
      pfDIN_C6     = 9,
      pfDIN_C65    = 10,
      pfDIN_DL     = 11,
      pfDIN_E4     = 12,
      pfDIN_E5     = 13,
      pfDIN_E6     = 14,
      pfDIN_E65    = 15,
      pfDIN_M5     = 16,
      pfDIN_M65    = 17,
      pfUS_Legal   = 18,
      pfUS_Letter  = 19
   }TPageFormat;

   typedef enum
   {
      fmFillNoClose            = 0,
      fmStrokeNoClose          = 1,
      fmFillStrokeNoClose      = 2,
      fmFill                   = 3,
      fmStroke                 = 4,
      fmFillStroke             = 5,
      fmFillEvOdd              = 6,
      fmFillStrokeEvOdd        = 7,
      fmFillEvOddNoClose       = 8,
      fmFillStrokeEvOddNoClose = 9,
      fmNoFill                 = 10,
      fmClose                  = 11  // Close the path, but do not fill or stroke it. More vertices are required or stroke the path in a separate step.
   }TPathFillMode;

   typedef enum
   {
      ptColored        = 0,
      ptUnColored      = 1,
      ptShadingPattern = 2
   }TPatternType;

   /*
      Notice:
      When using a bidirectional 8 bit code page the bidi algorithm is applied by default in Left to Right mode
      also if the bidi mode is set to bmNone (default). This mode produces identical results in comparison to
      applications like Edit or WordPad.

      The Right to Left mode is available in applications which use Microsoft's Uniscribe, e.g. BabelPad or MS Word.
      This mode works very well with the Reference Bidi Algorithm which is used by DynaPDF but Uniscribe's Left to Right
      mode produces different results in comparison to the Reference Bidi Algorithm. Because the bidi algorithm that is
      used in Uniscribe is not published it is practically impossible to get the same result in Left to Right mode
      without using this library.
   */

   typedef enum
   {
      bmLeftToRight = 0, // Apply the bidi algorithm on Unicode strings in Left to Right layout.
      bmRightToLeft = 1, // Apply the bidi algorithm on Unicode strings in Right to Left layout.
      bmNone        = 2  // Default -> do not apply the bidi algorithm
   }TPDFBidiMode;

   typedef enum
   {
      csDeviceRGB  = 0,
      csDeviceCMYK = 1,
      csDeviceGray = 2
   }TPDFColorSpace;

   struct TPDFRect
   {
      double Left;
      double Bottom;
      double Right;
      double Top;
   };

   struct TRectL
   {
      SI32 Left;
      SI32 Top;
      SI32 Right;
      SI32 Bottom;
   };

   typedef SI32 TRestrictions;
   #define rsDenyNothing      0x00000000
   #define rsDenyAll          0x00000F3C
   #define rsPrint            0x00000004
   #define rsModify           0x00000008
   #define rsCopyObj          0x00000010
   #define rsAddObj           0x00000020
   /* 128/256 bit encryption only -> these flags are ignored if 40 bit encryption is used */
   #define rsFillInFormFields 0x00000100
   #define rsExtractObj       0x00000200
   #define rsAssemble         0x00000400
   #define rsPrintHighRes     0x00000800
   #define rsExlMetadata      0x00001000 // PDF 1.5 Exclude metadata streams -> 128/256 bit encryption bit only.
   #define rsEmbFilesOnly     0x00002000 // PDF 1.6 Encrypt embedded files only -> Requires AES encryption.

   typedef enum
   {
      spHorizontal, /* ----- */
      spVertical,   /* ||||| */
      spRDiagonal,  /* \\\\\ */
      spLDiagonal,  /* ///// */
      spCross,      /* +++++ */
      spDiaCross    /* xxxxx */
   }TStdPattern;

   typedef enum
   {
      ttConstSpacing     = 0,
      ttNoDistortion     = 1,
      ttFastConstSpacing = 2
   }TTilingType;

   typedef enum
   {
      ptOpen  = 0,
      ptOwner = 1
   }TPwdType;

#endif

   /* The first part of the following flags specifying how the destination page will be zoomed.                     */
   /* Zero values determines that the paramter is not specified (null).                                             */
   /* Example: SetBookmarkDest(bmk, ptXY_Zoom, 50, 700, 0, 0); -> Go to position (X, Y), use the actual zoom factor.*/

   /* The second part of the name (after the underscore "_"), defines what kind of parameter                        */
   /* must be passed to the function. If no such value is defined, all four numeric parameters are ignored.         */

   /* The 'B' stands for bounding box. For example, if the page has a crop box the page will be                     */
   /* zoomed to the width or heigth of the crop box (or both). If the page has no crop box, the media box is used.  */
   /* See help file for further information. */

typedef enum
{
   dtXY_Zoom,     /* three parameters (a, b, c) -> (X, Y, Zoom)                    */
   dtFit,         /* no parameters                                                 */
   dtFitH_Top,    /* one parameter    (a)                                          */
   dtFitV_Left,   /* one parameter    (a)                                          */
   dtFit_Rect,    /* four parameters  (a, b, c, d) -> (left, bottom, right, top)   */
   dtFitB,        /* no parameters                                                 */
   dtFitBH_Top,   /* one parameter    (a)                                          */
   dtFitBV_Left   /* one parameter    (a)                                          */
}TDestType;

struct TBookmark
{
   UI32            Color;
   SI32            DestPage;
   struct TPDFRect DestPos;
   TDestType       DestType;
   LBOOL           Open;
   SI32            Parent;
   TFStyle         Style;
   const void*     Title;
   UI32            TitleLen;
   LBOOL           Unicode;
};

typedef enum
{
   ccCheck,
   ccCircle,
   ccCross1,
   ccCross2,
   ccCross3,
   ccCross4,
   ccDiamond,
   ccSquare,
   ccStar
}TCheckBoxChar;

typedef UI32 TCheckOptions;
#define coDefault                  0x00000FFF
#define coEmbedSubsets             0x00000001
#define coDeleteTransferFuncs      0x00000002
#define coDeleteMultiMediaContents 0x00000004
#define coDeleteActionsAndScripts  0x00000008
#define coDeleteInvRenderingIntent 0x00000010
#define coFlattenFormFields        0x00000020
#define coReplaceV4ICCProfiles     0x00000040
#define coDeleteEmbeddedFiles      0x00000080
#define coDeleteOPIComments        0x00000100
#define coDeleteSignatures         0x00000200
#define coReplCCITTFaxWithFlate    0x00010000
#define coNoFontEmbedding          0x10000000 // If this flag is set no PDF/A file will be produced!

// The data for user defined columns is stored in collection items.
typedef enum
{
   cisCreationDate,  // Data comes from the embedded file
   cisDescription,   // Data comes from the embedded file
   cisFileName,      // Data comes from the embedded file
   cisModDate,       // Data comes from the embedded file
   cisSize,          // Data comes from the embedded file
   cisCustomDate,    // User defined date.
   cisCustomNumber,  // User defined number.
   cisCustomString,  // User defined string.
   cisCompressedSize // Data comes from the embedded file
}TColColumnType;

typedef enum
{
   civNotSet,
   civDetails,
   civTile,
   civHidden
}TColView;

/*
   Please note that images must be decompressed if one of the following flags are set. Parsing gray or color images
   is in most cases not useful and you should not parse such images if it is not really required.
*/
typedef UI32 TCompBBoxFlags;
#define cbfNone              0x00000000
#define cbfIgnoreWhiteAreas  0x00000001 // Ignore white vector graphics or text.
#define cbfParse1BitImages   0x00000002 // Find the visible area in 1 bit images. This is the most important case
                                        // since scanned faxes are usually 1 bit images.
#define cbfParseGrayImages   0x00000004 // Find the visible area in gray images.
#define cbfParseColorImages  0x00000008 // Find the visible area in color images. This is usually not required
                                        // and slow downs processing a lot.
#define cbfParseAllImages    0x0000000E // Find the visible area in all images.

typedef enum
{
   ctPDFA_1b_2005, // Convert the file to PDF/A if possible
   ctNormalize     // Check the file for errors, rebuild all embedded fonts plus options
}TConformanceType;

typedef enum
{
   dfNone            = 0,
   dfASCII85Decode   = 1, // No parameters
   dfASCIIHexDecode  = 2, // No parameters
   dfCCITTFaxDecode  = 3, // Optional Parameters
   dfDCTDecode       = 4, // Optional Parameters
   dfFlateDecode     = 5, // Optional Parameters
   dfJBIG2Decode     = 6, // Optional Parameters
   dfJPXDecode       = 7, // No parameters
   dfLZWDecode       = 8, // Optional Parameters
   dfRunLengthDecode = 9, // No parameters
   dfCrypt           = 10 // Internal, cannot occur -> Already handled by the security handler.
}TDecodeFilter;

typedef enum
{  /* thousand separator, decimal separator */
   dsCommaDot,
   dsNoneDot,
   dsDotComma,
   dsNoneComma
}TDecSeparator;

typedef enum
{
   dpmNone,          // Default
   dpmSimplex,
   dpmFlipShortEdge,
   dpmFlipLongEdge
}TDuplexMode;

typedef enum
{
   fcBackColor   = 0,
   fcBorderColor = 1,
   fcTextColor   = 2
}TFieldColor, TAnnotColor;

typedef UI32 TFieldFlags;
#define ffReadOnly         0x00000001
#define ffRequired         0x00000002
#define ffNoExport         0x00000004

#define ffInvisible        0x00000008
#define ffHidden           0x00000010
#define ffPrint            0x00000020
#define ffNoZoom           0x00000040
#define ffNoRotate         0x00000080
#define ffNoView           0x00000100

#define ffMultiline        0x00001000  /* Text fields only */
#define ffPassword         0x00002000  /* Text fields only */
#define ffNoToggleToOff    0x00004000  /* Radio buttons    */
#define ffRadioIsUnion     0x02000000  /* PDF-1.5 check boxes of a radio button field only */
#define ffCommitOnSelCh    0x04000000  /* PDF-1.5 combo boxes, list boxes */

#define ffEdit             0x00040000  /* Combo boxes only */
#define ffSorted           0x00080000  /* Use this flag to change the sort flag of combo boxes or list boxes */
#define ffFileSelect       0x00100000  /* PDF 1.4 Text fields only */
#define ffMultiSelect      0x00200000  /* PDF 1.4 List boxes only  */
#define ffDoNotSpellCheck  0x00400000  /* PDF 1.4 Text fields, combo boxes. If the field is a combo box, this flag is meaningful only if ffEdit is also set. */
#define ffDoNotScroll      0x00800000  /* PDF 1.4 Text fields only */
#define ffComb             0x01000000  /* PDF 1.5 Text fields only */

typedef enum
{
   ftButton    = 0,
   ftCheckBox  = 1,
   ftRadioBtn  = 2,
   ftComboBox  = 3,
   ftListBox   = 4,
   ftText      = 5,
   ftSignature = 6,
   ftGroup     = 7 // this is not a real field type, it is just an array of fields
}TFieldType;

typedef enum
{
   faiGraph,
   faiPaperClip,
   faiPushPin,
   faiTag
}TFileAttachIcon;

typedef enum
{
   foOpen,
   foPrint
}TFileOP;

typedef enum
{
   htDetached, // CloseAndSignFileExt() returns the byte ranges of the finish PDF buffer to create a detached signature
   htSHA1      // CloseAndSignFileExt() returns the SHA1 hash of the PDF file so that it can be signed
}THashType;

typedef enum
{
   hmNone     = 0,
   hmInvert   = 1,
   hmOutline  = 2,
   hmPush     = 3,
   hmPushUpd  = 4  // Update appearance stream on changes
}THighlightMode;

typedef enum
{
   icNone      = 0, // Default
   icUseCCITT4 = 1  // Use CCITT Fax 4 compression instead of Flate for dithered images.
}TImageConversionFlags;

typedef UI32 TImportFlags;
#define ifImportAll        0x0FFFFFFE // default
#define ifContentOnly      0x00000000
// If this flag is set, only interactive objects are imported if any, Otherwise only empty pages are imported.
// This flag can be used to copy an Interactive Form, bookmarks or annotations to another PDF file.
#define ifNoContent        0x00000001
// The imported page is not converted to a template if ifImportAsPage is set.
// Note that this flag can cause resource conflicts. Use this flag carefully!
#define ifImportAsPage     0x80000000
// base objects
#define ifCatalogAction    0x00000002 // Open action, Catalog actions
#define ifPageActions      0x00000004
#define ifBookmarks        0x00000008
#define ifArticles         0x00000010
#define ifPageLabels       0x00000020
#define ifThumbs           0x00000040
#define ifTranspGroups     0x00000080 // This flag is no longer considered.
#define ifSeparationInfo   0x00000100
#define ifBoxColorInfo     0x00000200
#define ifStructureTree    0x00000400
#define ifTransition       0x00000800
#define ifSearchIndex      0x00001000
#define ifJavaScript       0x00002000 // Global JavaScripts
#define ifJSActions        0x00004000 // JavaScript actions
#define ifDocInfo          0x00008000 // Document info entries
#define ifEmbeddedFiles    0x00200000 // File attachments
#define ifFileCollections  0x00400000 // File collections (PDF 1.7)
// Annotations -> Only the most important annotation types can be selected directly.
// Note that all annotation types can be deleted with DeleteAnnotation.
#define ifAllAnnots        0x009F0000
   #define ifFreeText      0x00010000
   #define ifTextAnnot     0x00020000
   #define ifLink          0x00040000
   #define ifStamp         0x00080000
   #define if3DAnnot       0x00100000
   #define ifOtherAnnots   0x00800000
// Interactive Form Fields are annotations too but we handle this type separately!
#define ifFormFields       0x01000000
/* -------------------- Special flags -------------------- */
#define ifPrepareForPDFA   0x10000000 // Replace LZW compression with Flate, set the Interpolate key of images to false, do not import embedded files.
#define ifEnumFonts        0x20000000 // Import fonts for EnumDocFonts(). The document must be deleted when this flag is set!!!
#define ifAllPageObjects   0x40000000 // Import links when using ImportPageEx() within an open page. The entire document should be imported in this case.

typedef UI32 TImportFlags2;
#define if2MergeLayers     0x00000001 // If set, layers with identical name are merged. If this flag is absent DynaPDF
                                      // imports such layers separately so that each layer refers still to the pages
                                      // where it was orignally used.
#define if2Normalize       0x00000002 // Replace LZW compression with Flate, apply limit checks, repair errors if possible

typedef enum
{
   naFirstPage,
   naLastPage,
   naNextPage,
   naPrevPage,
   naGoBack,
   naOpenDlg,
   naPrintDlg,
   naGeneralInfo,
   naFontsInfo,
   naSaveAs,
   naSecurityInfo,
   naFitPage,
   naFullScreen,
   naDeletePages,
   naQuit
}TNamedAction;

typedef enum
{
   nsMinusBlack,
   nsRed,
   nsParensBlack,
   nsParensRed
}TNegativeStyle;

typedef enum
{ /* All actions which should be applied to an event (except OnMouseUp) must be a JavaScript action! */
   oeNoEvent,           /* Internal use only -> DO NOT USE THIS VALUE!!! */
   oeOnOpen,            /* Catalog, Pages */
   oeOnClose,           /* Pages only */
   oeOnMouseUp,         /* All fields, page link annotations, bookmarks */
   oeOnMouseEnter,      /* Form fields only */
   oeOnMouseExit,       /* Form fields only */
   oeOnMouseDown,       /* Form fields only */
   oeOnFocus,           /* Form fields only */
   oeOnBlur,            /* Form fields only */
   oeOnKeyStroke,       /* Text fields only */
   oeOnFormat,          /* Text fields only */
   oeOnCalc,            /* Text fields, combo boxes, list boxes */
   oeOnValidate,        /* All form fields, except buttons  */
   oeOnPageVisible,     /* PDF 1.5 -> Form fields only */
   oeOnPageInVisible,   /* PDF 1.5 -> Form fields only */
   oeOnPageOpen,        /* PDF 1.5 -> Form fields only */
   oeOnPageClose,       /* PDF 1.5 -> Form fields only */
   oeOnBeforeClosing,   /* PDF 1.4 -> Catalog only */
   oeOnBeforeSaving,    /* PDF 1.4 -> Catalog only */
   oeOnAfterSaving,     /* PDF 1.4 -> Catalog only */
   oeOnBeforePrinting,  /* PDF 1.4 -> Catalog only */
   oeOnAfterPrinting    /* PDF 1.4 -> Catalog only */
}TObjEvent;

typedef enum
{
   otAction,
   otAnnotation,
   otBookmark,
   otCatalog,    // PDF 1.4
   otField,
   otPage,
   otPageLink
}TObjType;

typedef enum
{
   ooAnnotation,
   ooField,
   ooImage,
   ooTemplate
}TOCObject;

typedef enum
{
   oiDesign = 2,
   oiView   = 4, // Default
   oiAll    = 8,
   oiEmpty  = 16 // Internal
}TOCGIntent;

typedef enum
{
   ovAllOff,
   ovAllOn,
   ovAnyOff,
   ovAnyOn,
   ovNotSet  // Internal
}TOCVisibility;

typedef enum
{
   plfDecimalArabic,    // 1,2,3,4...
   plfUppercaseRoman,   // I,II,III,IV...
   plfLowercaseRoman,   // i,ii,iii,iv...
   plfUppercaseLetters, // A,B,C,D...
   plfLowercaseLetters, // a,b,c,d...
   plfNone
}TPageLabelFormat;

typedef enum
{
   plSinglePage     = 0,
   plOneColumn      = 1,
   plTwoColumnLeft  = 2,
   plTwoColumnRight = 3,
   plTwoPageLeft    = 4, // PDF 1.5
   plTwoPageRight   = 5, // PDF 1.5
   plDefault        = 6  // Use viewer's default settings
}TPageLayout;

typedef enum
{
   pmUseNone        = 0,
   pmUseOutlines    = 1,
   pmUseThumbs      = 2,
   pmFullScreen     = 3,
   pmUseOC          = 4, // PDF 1.5
   pmUseAttachments = 5  // PDF 1.6
}TPageMode;

typedef UI32 TParseFlags;
#define pfNone             0x00000000
#define pfDecomprAllImages 0x00000002 // This flag causes that all image formats will be decompressed
                                      // with exception of JBIG2 compressed images. If this flag is
                                      // not set, images which are already stored in a valid image file
                                      // format are returned as is. This is the case for Gray and RGB JPEG
                                      // images and for JPEG2000 images. If you want to extract the images
                                      // of a PDF file this flag should NOT be set!

#define pfNoJPXDecode      0x00000004 // Meaningful only if the flag pfDecomprAllImages is set. If set,
                                      // JPEG2000 images are returned as is so that you can use your own
                                      // library to decompress such images since the the entire JPEG2000
                                      // codec is still marked as experimental. If we find an alternative
                                      // to the currently used Jasper library then we will replace it
                                      // immediatly with another one...

// The following flags are ignored if an image is not decompressed. Note that only one flag must be set
// at time. If no color space conversion flag is set images are returned in their native or alternate
// device color space. Note that these flags do not convert colors of vector graphics and so on.
// Use the function ConvColor() to convert colors of the graphics state into a device space.
#define pfDitherImagesToBW 0x00000008 // Floyd-Steinberg dithering.
#define pfConvImagesToGray 0x00000010
#define pfConvImagesToRGB  0x00000020
#define pfConvImagesToCMYK 0x00000040
#define pfImageInfoOnly    0x00000080 // If set, images are not decompressed. This flag is useful if you want
                                      // to enumerate the images of a PDF file or if you want to determine how
                                      // many images are stored in it.
                                      // Note that images can be compressed with multiple filters. The member
                                      // Filter of the structure TPDFImage contains only the last filter with
                                      // which the image was compressed. There is no indication whether multiple
                                      // decode filters are required to decompress the image buffer. So, it
                                      // makes no sense to set this flag if you want to try to decompress the
                                      // image buffer manually with your own decode filters.

// The tags have the same meaning as the corresponding HTML tags.
// See PDF Reference for further information.
typedef enum
{
   btArt,
   btArtifact,
   btAnnot,
   btBibEntry,      // BibEntry -> Bibliography entry
   btBlockQuote,
   btCaption,
   btCode,
   btDiv,
   btDocument,
   btFigure,
   btForm,
   btFormula,
   btH,
   btH1,
   btH2,
   btH3,
   btH4,
   btH5,
   btH6,
   btIndex,
   btLink,
   btList,          // L
   btListElem,      // LI
   btListText,      // LBody
   btNote,
   btP,
   btPart,
   btQuote,
   btReference,
   btSection,       // Sect
   btSpan,
   btTable,
   btTableDataCell, // TD
   btTableHeader,   // TH
   btTableRow,      // TR
   btTOC,
   btTOCEntry       // TOCI
}TPDFBaseTag;

typedef enum
{
   dfMM_D            = 0,
   dfM_D_YY          = 1,
   dfMM_DD_YY        = 2,
   dfMM_YY           = 3,
   dfD_MMM           = 4,
   dfD_MMM_YY        = 5,
   dfDD_MMM_YY       = 6,
   dfYY_MM_DD        = 7,
   dfMMM_YY          = 8,
   dfMMMM_YY         = 9,
   dfMMM_D_YYYY      = 10,
   dfMMMM_D_YYYY     = 11,
   dfM_D_YY_H_MM_TT  = 12,
   dfM_D_YY_HH_MM    = 13,
   /* time formats */
   df24HR_MM         = 14,
   df12HR_MM         = 15,
   df24HR_MM_SS      = 16,
   df12HR_MM_SS      = 17
}TPDFDateTime;

typedef enum
{
   pvPDF_1_0     = 0,
   pvPDF_1_1     = 1,
   pvPDF_1_2     = 2,
   pvPDF_1_3     = 3,
   pvPDF_1_4     = 4,
   pvPDF_1_5     = 5,
   pvPDF_1_6     = 6,
   pvPDF_1_7     = 7,
   pvPDF_1_8     = 8,
   pvPDF_1_9     = 9,  // reserved for future use
   pvPDFX1a_2001 = 10, // PDF/X-1a:2001
   pvPDFX1a_2003 = 11, // PDF/X-1a:2002
   pvPDFX3_2002  = 12, // PDF/X-3:2002
   pvPDFX3_2003  = 13, // PDF/X-3:2003
   pvPDFA_2005   = 14  // PDF/A-1b 2005
}TPDFVersion;

typedef enum
{
   psAppDefault, // Default
   psNone
}TPrintScaling;

typedef enum
{
   rifByteAligned  = 0x1000,
   rifRGBData      = 0x2000,
   rifCMYKData     = 0x4000
}TRawImageFlags;

typedef enum
{
   riAbsoluteColorimetric = 0,
   riPerceptual           = 1,
   riRelativeColorimetric = 2,
   riSaturation           = 3,
   riNone                 = 4 // Internal
}TRenderingIntent;

typedef enum
{
  rsApproved,
  rsAsIs,
  rsConfidential,
  rsDepartmental,
  rsDraft,
  rsExperimental,
  rsExpired,
  rsFinal,
  rsForComment,
  rsForPublicRelease,
  rsNotApproved,
  rsNotForPublicRelease,
  rsSold,
  rsTopSecred,
  rsUserDefined
}TRubberStamp;

typedef enum
{
   stUnknown            = 0, // cannot occur -> internal use
   stFunctionBased      = 1,
   stAxial              = 2,
   stRadial             = 3,
   stFreeFormGouraud    = 4,
   stLatticeFormGouraud = 5,
   stCoonsPatch         = 6,
   stTensorProduct      = 7
}TShadingType;

typedef enum
{
   taLeft      = 0,
   taCenter    = 1,
   taRight     = 2,
   taJustify   = 3,
   taPlainText = 0x10000000 // If this flag is set alignment and command tags are interpreted as plain text.
                            // See WriteFText() in the help file for further information.
}TTextAlign;

typedef SI32 TSubmitFlags;
#define sfNone             0x00000000 // The default export format is FDF
#define sfExlude           0x00000001 // If set, the fields in a submit form action are excluded if any
#define sfInclNoValFields  0x00000002
#define sfHTML             0x00000004
#define sfGetMethod        0x00000008
#define sfSubmCoords       0x00000010
#define sfXML              0x00000024
#define sfInclAppSaves     0x00000040 // Requires the full version of Adobe's Acrobat
#define sfInclAnnots       0x00000080
#define sfPDF              0x00000100 // Submit the entire PDF file -> requires the full version of Adobe's Acrobat
#define sfCanonicalFormat  0x00000200
#define sfExlNonUserAnnots 0x00000400
#define sfExlFKey          0x00000800
#define sfEmbedForm        0x00002000 // PDF 1.5 embed the entire form into a file stream inside the FDF file -> requires the full version of Adobe's Acrobat

typedef enum
{
   toRow       = 0,
   toColumn    = 1,
   toStructure = 2,
   toNone      = 3
}TTabOrder;

typedef SI32 TViewerPreference;
#define vpUseNone                0x00000000
#define vpHideToolBar            0x00000001
#define vpHideMenuBar            0x00000002
#define vpHideWindowUI           0x00000004
#define vpFitWindow              0x00000008
#define vpCenterWindow           0x00000010
#define vpDisplayDocTitle        0x00000020
#define vpNonFullScrPageMode     0x00000040
#define vpDirection              0x00000080
#define vpViewArea               0x00000100
#define vpViewClip               0x00000200
#define vpPrintArea              0x00000400
#define vpPrintClip              0x00000800

typedef SI32 TViewPrefAddVal;
#define avNone                   0x00000000
#define avNonFullScrUseNone      0x00000001
#define avNonFullScrUseOutlines  0x00000002
#define avNonFullScrUseThumbs    0x00000004
#define avNonFullScrUseOC        0x00000400 // PDF 1.6
#define avDirectionL2R           0x00000008
#define avDirectionR2L           0x00000010
#define avViewPrintArtBox        0x00000020
#define avViewPrintBleedBox      0x00000040
#define avViewPrintCropBox       0x00000080
#define avViewPrintMediaBox      0x00000100
#define avViewPrintTrimBox       0x00000200

// Use these masks to determine which value is defined.
#define AV_NON_FULL_SRC_MASK     0x00000005
#define AV_DIRECTION_MASK        0x00000018
#define AV_VIEW_PRINT_MASK       0x000003E0

struct TPDFAnnotation
{
   TAnnotType      Type;
   LBOOL           Deleted;
   struct TPDFRect BBox;
   double          BorderWidth;
   UI32            BorderColor;
   TBorderStyle    BorderStyle;
   UI32            BackColor;
   UI32            Handle;
   char*           AuthorA;
   UI16*           AuthorW;
   char*           ContentA;
   UI16*           ContentW;
   char*           NameA;
   UI16*           NameW;
   char*           SubjectA;
   UI16*           SubjectW;
   UI32            PageNum;
   THighlightMode  HighlightMode;
};

struct TPDFAnnotationEx
{
   TAnnotType      Type;
   LBOOL           Deleted;
   struct TPDFRect BBox;
   float           BorderWidth;
   UI32            BorderColor;
   TBorderStyle    BorderStyle;
   UI32            BackColor;
   UI32            Handle;
   char*           AuthorA;
   UI16*           AuthorW;
   char*           ContentA;
   UI16*           ContentW;
   char*           NameA;
   UI16*           NameW;
   char*           SubjectA;
   UI16*           SubjectW;
   UI32            PageNum;
   THighlightMode  HighlightMode;
   // Page link annotations only
   SI32            DestPage;
   struct TPDFRect DestPos;
   TDestType       DestType;
   // File link or web link annotations
   char*           DestFile;
   // The Icon type depends on the annotation type. If the annotation type is atText the Icon
   // is of type TAnnotIcon. If the annotation type is atFileAttach then it is of type
   // TFileAttachIcon. If the annotation type is atStamp then the Icon is the stamp type (TRubberStamp).
   // For any other annotation type this value is not set (-1).
   SI32            Icon;
   char*           StampName;    // Set only, if Icon == rsUserDefined
   UI32            AnnotFlags;   // See TAnnotFlags for available flags
   char*           CreateDate;   // Creation Date -> Optional
   char*           ModDate;      // Modification Date -> Optional
   LBOOL           Grouped;      // (Reply type) Meaningful only if Parent != -1 and Type != atPopUp. If true,
                                 // the annotation is part of an annotation group. Properties like Content, CreateDate,
                                 // ModDate, BackColor, Subject, and Open must be taken from the parent annotation.
   LBOOL           Open;         // Meaningful only for annotations which have a corresponding PopUp annotation.
   SI32            Parent;       // Parent annotation handle of a PopUp Annotation or the parent annotation if
                                 // this annotation represents a state of a base annotation. In this case,
                                 // the annotation type is always atText and only the following members should
                                 // be considered:
                                 //    State      // The current state
                                 //    StateModel // Marked, Review, and so on
                                 //    CreateDate // Creation Date
                                 //    ModDate    // Modification Date
                                 //    Author     // The user who has set the state
                                 //    Content    // Not displayed in Adobe's Acrobat...
                                 //    Subject    // Not displayed in Adobe's Acrobat...
                                 // The PopUp annotation of a text annotation which represent an Annotation State
                                 // must be ignored.
   SI32            PopUp;        // Handle of the corresponding PopUp annotation if any.
   char*           State;        // The state of the annotation.
   char*           StateModel;   // The state model (Marked, Review, and so on).
   SI32            EmbeddedFile; // FileAttach annotations only. A handle of an embedded file -> GetEmbeddedFile().
   char*           Subtype;      // Set only, if Type == atUnknownAnnot
   /* Reserved fields for future extensions*/
   void*           Reserved01;
   void*           Reserved02;
   void*           Reserved03;
   void*           Reserved04;
   void*           Reserved05;
   void*           Reserved06;
   void*           Reserved07;
   void*           Reserved08;
   void*           Reserved09;
   void*           Reserved10;
   void*           Reserved11;
   void*           Reserved12;
   void*           Reserved13;
   void*           Reserved14;
   void*           Reserved15;
   void*           Reserved16;
   void*           Reserved17;
   void*           Reserved18;
   void*           Reserved19;
   void*           Reserved20;
   void*           Reserved21;
   void*           Reserved22;
   void*           Reserved23;
   void*           Reserved24;
   void*           Reserved25;
   void*           Reserved26;
   void*           Reserved27;
};

struct TPDFChoiceValue
{
   UI32  StructSize; // Must be set to sizeof(TPDFChoiceValue)
   char* ExpValueA;
   UI16* ExpValueW;
   UI32  ExpValueLen;
   char* ValueA;
   UI16* ValueW;
   UI32  ValueLen;
   LBOOL Selected;
};

/*
   The structure contains several duplicate fields because CMap files contain usually a DSC comment
   section which provides Postscript specific initialization code. With exception of DSCResName the
   strings in the DSC section should not differ from their CMap counterparts. The Identity mapping
   of a character collection should contain the DSC comment "%%BeginResource: CMap (Identity)".
   Otherwise the string should be set to the CMap name.
*/
struct TPDFCMap
{
   UI32  StructSize;     // Must be set to sizeof(TPDFCMap) before calling GetCMap()!
   char* BaseCMap;       // If set, this base cmap is required when loading the cmap.
   UI32  CIDCount;       // 0 if not set.
   char* CMapName;       // The CMap name.
   UI32  CMapType;       // Should be 1!
   float CMapVersion;    // The CMap version.
   char* DSCBaseCMap;    // DSC comment.
   float DSCCMapVersion; // DSC comment.
   char* DSCResName;     // DSC comment. If the CMap uses an Identity mapping this string should be set to Identity.
   char* DSCTitle;       // DSC comment -> DSC CMap name + Registry + Ordering + Supplement, e.g. "GB-EUC-H Adobe GB1 0"
   char* FileNameA;      // The file name and CMap name should be identical!
   UI16* FileNameW;      // The file name and CMap name should be identical!
   char* FilePathA;      // The Ansi string is set if the Ansi version of SetCMapDir() was used.
   UI16* FilePathW;      // The Unicode string is set if the Unicode version of SetCMapDir() was used.
   char* Ordering;       // CIDSystemInfo -> The Character Collection, e.g. Japan1.
   char* Registry;       // CIDSystemInfo -> The registrant of the Character Collection is usually Adobe.
   UI32  Supplement;     // CIDSystemInfo -> The Supplement number should be supported in the used PDF Version.
   UI32  WritingMode;    // 0 == Horizontal, 1 == Vertical
};

struct TPDFColorSpaceObj
{
   TExtColorSpace Type;
   TExtColorSpace Alternate;        // Alternate color space or base space of an Indexed or Pattern color space.
   void*          IAlternate;       // Only set if the color space contains an alternate or base color space -> GetColorSpaceObjEx().
   BYTE*          Buffer;           // Contains either an ICC profile or the color table of an Indexed color space.
   UI32           BufSize;          // Buffer length in bytes.
   float*         BlackPoint;       // CIE blackpoint. If set, the array contains exactly 3 values.
   float*         WhitePoint;       // CIE whitepoint. If set, the array contains exactly 3 values.
   float*         Gamma;            // If set, one value per component.
   float*         Range;            // min/max for each component or for the .a and .b components of a Lab color space.
   float*         Matrix;           // XYZ matrix. If set, the array contains exactly 9 values.
   UI32           NumInComponents;  // Number of input components.
   UI32           NumOutComponents; // Number of output components.
   UI32           NumColors;        // HiVal + 1 as specified in the color space. Indexed color space only.
   BYTE*          Colorants[32];    // UTF-8 Colorant names (Separation, DeviceN, and NChannel only).
   UI32           ColorantsCount;   // The number of colorants in the array.
   BYTE*          Metadata;         // Optional XMP metadata stream -> ICCBased only.
   UI32           MetadataSize;     // Metadata length in bytes.
   void*          IFunction;        // Pointer to function object -> Separation, DeviceN, and NChannel only.
   void*          IAttributes;      // Optional attributes of DeviceN or NChannel color spaces -> GetDeviceNAttributes().
   void*          IColorSpaceObj;   // Pointer of the corresponding color space object
   void*          Reserved01;
   void*          Reserved02;
   void*          Reserved03;
   void*          Reserved04;
   void*          Reserved05;
   void*          Reserved06;
   void*          Reserved07;
   void*          Reserved08;
   void*          Reserved09;
};

struct TDeviceNAttributes
{
   void* IProcessColorSpace;    // Pointer to process color space or NULL -> GetColorSpaceEx().
   BYTE* ProcessColorants[8];   // UTF-8 Process colorant names
   UI32  ProcessColorantsCount; // Number of process colorants in the array or zero if not set.
   void* Separations[32];       // Optional pointers to Separation color spaces -> GetColorSpaceEx().
   UI32  SeparationsCount;      // Number of color spaces in the array.
   void* IMixingHints;          // Optional mixing hints. There is no API function at this time to access mixing hints.
   void* Reserved01;
   void* Reserved02;
   void* Reserved03;
   void* Reserved04;
};

// It is not possible to set all available graphic state parameters with DynaPDF, such as black generation functions,
// halftones and so on. The structure will be extended when further objects are supported.
struct TPDFExtGState
{
   UI32             AutoStrokeAdjust; // PDF_MAX_INT if not set
   TBlendMode       BlendMode;        // Default bmNotSet
   float            FlatnessTol;      // -1.0 if not set
   UI32             OverPrintFill;    // PDF_MAX_INT if not set
   UI32             OverPrintStroke;  // PDF_MAX_INT if not set
   UI32             OverPrintMode;    // PDF_MAX_INT if not set
   TRenderingIntent RenderingIntent;  // riNone if not set
   float            SmoothnessTol;    // -1.0 if not set
   float            FillAlpha;        // -1.0 if not set
   float            StrokeAlpha;      // -1.0 if not set
   UI32             AlphaIsShape;     // PDF_MAX_INT if not set
   UI32             TextKnockout;     // PDF_MAX_INT if not set
   void*            Reserved1;
   void*            Reserved2;
   void*            Reserved3;
   void*            Reserved4;
   void*            Reserved5;
   void*            Reserved6;
   void*            Reserved7;
   void*            Reserved8;
   void*            Reserved9;
};

// Extended graphics state dictionary
struct TPDFExtGState2
{
   LBOOL            AlphaIsShape;       // PDF_MAX_INT if not set
   LBOOL            AutoStrokeAdjust;   // PDF_MAX_INT if not set
   void*            BlackGen;           // Function handle or NULL -> GetFunction()
   void*            BlackGen2;          // Function handle or NULL -> GetFunction()
   TBlendMode*      BlendMode;          // Array of blend modes
   UI32             BlendModeCount;     // Number of blend modes
   float            FillAlpha;          // -1.0 if not set
   float            FlatnessTol;        // -1.0 if not set
   void*            Halftone;           // Halftone handle or NULL -> GetHalftoneDict()
   LBOOL            OverPrintFill;      // PDF_MAX_INT if not set
   LBOOL            OverPrintStroke;    // PDF_MAX_INT if not set
   LBOOL            OverPrintMode;      // PDF_MAX_INT if not set
   TRenderingIntent RenderingIntent;    // riNone if not set
   float            SmoothnessTol;      // -1.0 if not set
   void*            SoftMask;           // Soft mask handle or NULL
   float            StrokeAlpha;        // -1.0 if not set
   LBOOL            TextKnockout;       // PDF_MAX_INT if not set
   void*            TransferFunc;       // Array of functions -> GetFunction()
   UI32             TransferFuncCount;  // Number of transfer functions
   void*            TransferFunc2;      // Array of functions -> GetFunction()
   UI32             TransferFunc2Count; // Number of transfer functions
   void*            UnderColorRem;      // Function handle or NULL -> GetFunction()
   void*            UnderColorRem2;     // Function handle or NULL -> GetFunction()
   void*            Reserved1;
   void*            Reserved2;
   void*            Reserved3;
   void*            Reserved4;
};

struct TPDFField
{
   SI32   FieldType;
   LBOOL  Deleted;
   struct TPDFRect BBox;
   SI32   Handle;
   char*  FieldName;
/*
   The field name is normally an ANSI string. However, Adobe's Designer uses Unicode strings as field names.
   To determine whether the string is an ANSI or Unicode string proceed as follows:
   if (f.FieldNameLen > 0)
   {
      // The field name is always null-terminated so that this action is safe.
      if (f.FieldName && strlen(f.FieldName) != f.FieldNameLen)
      {
         // The field name is an Unicode string!
         fieldNameW = (UI16*)f.FieldName;
         ...
      }
   }
*/
   UI32   FieldNameLen;
   UI32   BackCS;
   UI32   TextCS;
   UI32   BackColor;
   UI32   BorderColor;
   UI32   TextColor;
   LBOOL  Checked;
   SI32   Parent;
   UI32   KidCount;
   char*  Font;
   double FontSize;
   void*  Value;
   LBOOL  UniVal;
   UI32   ValLen;
   void*  ToolTip;
   LBOOL  UniToolTip;
   UI32   ToolTipLen;
};

struct TPDFFieldEx
{
   UI32            StructSize;    // Must be set to sizeof(TPDFFieldEx)
   LBOOL           Deleted;       // If true, the field was marked as deleted by DeleteField()
   struct TPDFRect BBox;          // Bounding box of the field in bottom-up coordinates
   TFieldType      FieldType;     // Field type
   TFieldType      GroupType;     // If GroupType != FieldType the field is a terminal field of a field group
   UI32            Handle;        // Field handle
   UI32            BackColor;     // Background color
   TExtColorSpace  BackColorSP;   // Color space of the background color
   UI32            BorderColor;   // Border color
   TExtColorSpace  BorderColorSP; // Color space of the border color
   TBorderStyle    BorderStyle;   // Border style
   float           BorderWidth;   // Border width
   float           CharSpacing;   // Text fields only
   LBOOL           Checked;       // Check boxes only
   UI32            CheckBoxChar;  // ZapfDingbats character that is used to display the on state
   TCheckBoxState  DefState;      // Check boxes only
   const char*     DefValueA;     // Optional default value
   const UI16*     DefValueW;     // Optional default value
   const void*     IEditFont;     // Pointer to default editing font
   const char*     EditFont;      // Postscript name of the editing font
   UI32            ExpValCount;   // Combo and list boxes only. The values can be accessed with GetFieldExpValueEx()
   const char*     ExpValueA;     // Check boxes only
   const UI16*     ExpValueW;     // Check boxes only
   TFieldFlags     FieldFlags;    // Field flags
   const void*     IFieldFont;    // Pointer to the font that is used by the field
   const char*     FieldFont;     // Postscript name of the font
   UI32            Reserved;      // Reserved field to avoid alignment errors
   double          FontSize;      // Font size. 0.0 means auto font size
   const char*     FieldNameA;    // Note that children of a field group or radio button have no name
   const UI16*     FieldNameW;    // Note that children of a field group or radio button have no name
   THighlightMode  HighlightMode; // Highlight mode
   LBOOL           IsCalcField;   // If true, the OnCalc event of the field is connected with a JavaScript action
   const char*     MapNameA;      // Optional unique mapping name of the field
   const UI16*     MapNameW;      // Optional unique mapping name of the field
   UI32            MaxLen;        // Text fields only -> zero means not restricted
   const void**    Kids;          // Array of child fields -> GetFieldEx2()
   UI32            KidCount;      // Number of fields in the array
   void*           Parent;        // Pointer to parent field or NULL
   SI32            PageNum;       // Page on which the field is used or -1
   SI32            Rotate;        // Rotation angle in degrees
   TTextAlign      TextAlign;     // Text fields only
   UI32            TextColor;     // Text color
   TExtColorSpace  TextColorSP;   // Color space of the field's text
   float           TextScaling;   // Text fields only
   const char*     ToolTipA;      // Optional tool tip
   const UI16*     ToolTipW;      // Optional tool tip
   const char*     UniqueNameA;   // Optional unique name (NM key)
   const UI16*     UniqueNameW;   // Optional unique name (NM key)
   const char*     ValueA;        // Field value
   const UI16*     ValueW;        // Field value
   float           WordSpacing;   // Text fields only
};

/*
   All strings are null-terminated (except the checksum). If a string is returned in Unicode
   format then make a typecast to (UI16*) to get the right value. Note that most values are
   optional they can be NULL.
*/
struct TPDFFileSpec
{
   char* Buffer;      // Buffer of an embedded file
   UI32  BufSize;     // Buffer size in bytes.
   LBOOL Compressed;  // Should be false if Decompress was true in the GetEmbeddedFile() call, otherwise usually true.
                      // DynaPDF decompresses Flate encoded streams only. Other filters can occur but this is very unusual.
   void* ColItem;     // If != NULL the embedded file contains a collection item with user defined data. This entry
                      // can occur in PDF Collections only (PDF 1.7). See "PDF Collections" in the help file for further
                      // information.
   char* Name;        // Name of the file specification in the name tree. This value is always present.
   LBOOL NameUnicode; // Is Name in Unicode format?
   char* FileName;    // File name as 7 bit ASCII string.
   LBOOL IsURL;       // If true, FileName contains a URL.
   char* UF;          // PDF 1.7. Same as FileName but Unicode is allowed.
   LBOOL UFUnicode;   // Is UF in Unicode format?
   char* Desc;        // Description
   LBOOL DescUnicode; // Is Desc in Unicode format?
   UI32  FileSize;    // Size of the decompressed stream or zero if not known. Note: this is either the Size key of
                      // the Params dictionary if present or the DL key in the file stream. Whether this value is
                      // correct depends on the file creator! The parameter is definitely correct if the file was
                      // decompressed.
   char* MIMEType;    // MIME media type name as defined in Internet RFC 2046.
   char* CreateDate;  // Creation date as string. See help file "The standard date format".
   char* ModDate;     // Modification date as string. See help file "The standard date format".
   char* CheckSum;    // 16 byte MD5 digest. Note that this is a binary string. It is exactly 16 bytes long if set!
};

struct TPDFFontObj
{
   float            Ascent;        // Ascent
   char*            BaseFont;      // PostScript Name or Family Name
   float            CapHeight;     // Cap height
   float            Descent;       // Descent
   UI16*            Encoding;      // Unicode mapping 0..255 -> not set if a CID font is selected
   UI32             FirstChar;     // First char
   UI32             Flags;         // Font flags -> font descriptor
   char*            FontFamily;    // Optional Font Family (Family Name)
   LBOOL            FontFamilyUni; // Is FontFamily in Unicode format?
   char*            FontName;      // Font name -> font descriptor
   TFontType        FontType;      // If ftType0 the font is a CID font. The Encoding is not set in this case.
   float            ItalicAngle;   // Italic angle
   UI32             LastChar;      // Last char
   float            SpaceWidth;    // Space width in font units. A default value is set if the font contains no space character.
   float*           Widths;        // Glyph widths -> 0..WidthsCount -1
   UI32             WidthsCount;   // Number of widths in the array
   float            XHeight;       // x-height
   float            DefWidth;      // Default character width -> CID fonts only
   char*            FontFile;      // Font file buffer -> only imported fonts are returned.
   UI32             Length1;       // Length of the clear text portion of the Type1 font, or the length of the entire font program if FontType != ffType1.
   UI32             Length2;       // Length of the encrypted portion of the Type1 font program (Type1 fonts only).
   UI32             Length3;       // Length of the fixed-content portion of the Type1 font program or zero if not present.
   TFontFileSubtype FontFileType;  // See description in the help file for further information.
};

struct TPDFImage
{
   BYTE*            Buffer;           // Each scanline is aligned to a full byte.
   UI32             BufSize;          // The size of the image buffer in bytes.
   TDecodeFilter    Filter;           // Required decode filter if the image is compressed.
                                      // Possible values are dfDCTDecode (JPEG), dfJPXDecode (JPEG2000),
                                      // and dfJBIG2Decode. Other filters are already removed by DynaPDF since
                                      // a conversion to a native file format is then always required.
   TDecodeFilter    OrgFilter;        // The image was compressed with this filter in the PDF file. This info is
                                      // useful to determine which compression filter should be used when creating
                                      // a new image file from the image buffer.
   BYTE*            JBIG2Globals;     // Optional global page 0 segment (dfJBIG2Decode filter only).
   UI32             JBIG2GlobalsSize; // The size of the bit stream in bytes.
   UI32             BitsPerPixel;     // Bit depth of the image buffer. Possible values are 1, 2, 4, 8, 24, 32, and 64.
   TExtColorSpace   ColorSpace;       // The color space refers either to the image buffer or to the color table if set.
                                      // Note that 1 bit images can occur with and without a color table.
   UI32             NumComponents;    // The number of components stored in the image buffer.
   LBOOL            MinIsWhite;       // If true, the colors of 1 bit images are reversed.
   const void*      IColorSpaceObj;   // Pointer to the original color space.
   BYTE*            ColorTable;       // The color table or NULL.
   UI32             ColorCount;       // The number of colors in the color table.
   UI32             Width;            // Image width in pixel.
   UI32             Height;           // Image height in pixel.
   UI32             ScanLineLength;   // The length of a scanline in bytes.
   LBOOL            InlineImage;      // If true, the image is an inline image.
   LBOOL            Interpolate;      // If true, image interpolation should be performed.
   LBOOL            Transparent;      // The meaning is different depending on the bit depth and whether a color
                                      // table is available. If the image is a 1 bit image and if no color table is available,
                                      // black pixels must be drawn with the current fill color.
                                      // If the image contains a color table ColorMask contains the range of indexes
                                      // in the form min/max index which appears transparent. If no color table is
                                      // present ColorMask contains the transparent ranges in the form min/max for
                                      // each component.
   BYTE*            ColorMask;        // The array contains ranges in the form min/max (2 values per component) for each
                                      // component before decoding.
   const void*      IMaskImage;       // If set, a 1 bit image is used as a transparency mask. Call GetImage() to decode the image.
   const void*      ISoftMask;        // If set, a grayscale image is used as alpha channel. Call GetImage() to decode the image.
   float*           Decode;           // If set, samples must be decoded. The array contains 2 * NumComponents values.
                                      // The decode array is never set if the image is returned decompressed since
                                      // it is already applied during decompression.
   TRenderingIntent Intent;           // Default riNone.
   UI32             SMaskInData;      // JPXDecode only, PDF_MAX_INT if not set. See PDF Reference for further information.
   const void*      OCG;              // Pointer to Optional Content Group if any.
   BYTE*            Metadata;         // Optional XML Metadata stream.
   UI32             MetadataSize;     // Length of Metadata in bytes.
   const void*      ObjectPtr;        // Internal pointer to the image class.
   float            ResolutionX;      // Image resolution on the x-axis.
   float            ResolutionY;      // Image resolution on the y-axis.
   // Reserved fields for future extensions
   UI32 Reserved1;
   UI32 Reserved2;
   UI32 Reserved3;
   UI32 Reserved4;
   UI32 Reserved5;
   UI32 Reserved6;
   UI32 Reserved7;
   UI32 Reserved8;
   UI32 Reserved9;
   UI32 Reserved10;
   UI32 Reserved11;
   UI32 Reserved12;
   UI32 Reserved13;
   UI32 Reserved14;
   UI32 Reserved15;
   UI32 Reserved16;
   UI32 Reserved17;
   UI32 Reserved18;
};

// Named Destination
struct TPDFNamedDest
{
   char*           NameA;
   UI16*           NameW;
   UI32            NameLen;
   SI32            DestPage;
   struct TPDFRect DestPos;
   TDestType       DestType;
};

struct TPDFPageLabel
{
   UI32             StartRange;   // Number of the first page in the range. If no further label follows, the last
                                  // page in the range is pdfGetPageCount(). The first page is denoted by 1.
   TPageLabelFormat Format;       // Number format to be used.
   SI32             FirstPageNum; // First page number to be displayed in the page label. Subsequent pages are
                                  // numbered sequentially from this value.
   char*            Prefix;       // Optional prefix
   UI32             PrefixLen;    // Length of the prefix in characters
   LBOOL            PrefixUni;    // If true, the prefix is a Unicode string
};

struct TPDFPrintSettings
{
   TDuplexMode   DuplexMode;
   SI32          NumCopies;         // -1 means not set. Values larger than 5 are ignored in viewer applications.
   SI32          PickTrayByPDFSize; // -1 means not set. 0 == false, 1 == true.
   UI32*         PrintRanges;       // If set, the array contains PrintRangesCount * 2 values. Each pair consists
                                    // of the first and last pages in the sub-range. The first page in the PDF file
                                    // is denoted by 0.
   UI32          PrintRangesCount;  // Number of ranges available in PrintRanges.
   TPrintScaling PrintScaling;      // dpmNone means not set.
   /* Reserved fields for future extensions*/
   UI32 Reserved0;
   UI32 Reserved1;
   UI32 Reserved2;
   UI32 Reserved3;
   UI32 Reserved4;
   UI32 Reserved5;
   UI32 Reserved6;
   UI32 Reserved7;
   UI32 Reserved8;
   UI32 Reserved9;
};

struct TTextRecordA
{
   float Advance; // Negative values move the cursor to the right, positive to the left. The value is measured in text space!
   BYTE* Text;    // Raw text (not null-terminated)
   UI32  Length;  // Raw text length in bytes
};

struct TTextRecordW
{
   float Advance; // Negative values move the cursor to the right, positive to the left. The value is measured in text space!
   UI16* Text;    // Already translated Unicode string (not null-terminated)
   UI32  Length;  // Length in characters
   float Width;   // String width measured in text space
};

typedef SI32 PDF_CALL TApplyPattern(const void* Data, const BYTE* Object, TPatternType Type, void* Pattern);
typedef SI32 PDF_CALL TBeginPattern(const void* Data, LBOOL Fill, SI32 Handle, TPatternType Type, struct TPDFRect* BBox, struct TCTM* Matrix, double* XStep, double* YStep);
typedef SI32 PDF_CALL TBeginTemplate(const void* Data, const BYTE* Object, SI32 Handle, struct TPDFRect* BBox, struct TCTM* Matrix);
typedef SI32 PDF_CALL TBezierTo1(const void* Data, const BYTE* Object, double x1, double y1, double x3, double y3);
typedef SI32 PDF_CALL TBezierTo2(const void* Data, const BYTE* Object, double x2, double y2, double x3, double y3);
typedef SI32 PDF_CALL TBezierTo3(const void* Data, const BYTE* Object, double x1, double y1, double x2, double y2, double x3, double y3);
typedef SI32 PDF_CALL TClipPath(const void* Data, const BYTE* Object, LBOOL EvenOdd, TPathFillMode Mode);
typedef SI32 PDF_CALL TClosePath(const void* Data, const BYTE* Object, TPathFillMode Mode);
typedef SI32 PDF_CALL TDrawShading(const void* Data, const BYTE* Object, TShadingType Type, void* Shading);
typedef void PDF_CALL TEndPattern(const void* Data);
typedef void PDF_CALL TEndTemplate(const void* Data);
typedef SI32 PDF_CALL TInsertImage(const void* Data, struct TPDFImage* Image);
typedef SI32 PDF_CALL TLineTo(const void* Data, const BYTE* Object, double x, double y);
typedef SI32 PDF_CALL TMoveTo(const void* Data, const BYTE* Object, double x ,double y);
typedef void PDF_CALL TMulMatrix(const void* Data, const BYTE* Object, struct TCTM* Matrix);
typedef SI32 PDF_CALL TRectangle(const void* Data, const BYTE* Object, double x, double y, double w, double h);
typedef SI32 PDF_CALL TRestoreGraphicState(const void* Data);
typedef SI32 PDF_CALL TSaveGraphicState(const void* Data);
typedef void PDF_CALL TSetCharSpacing(const void* Data, const BYTE* Object, double Value);
typedef void PDF_CALL TSetExtGState(const void* Data, const BYTE* Object, const struct TPDFExtGState2* GS);
typedef void PDF_CALL TSetFillColor(const void* Data, const BYTE* Object, double* Color, UI32 NumComps, TExtColorSpace CS, void* IColorSpace);
typedef void PDF_CALL TSetFont(const void* Data, const BYTE* Object, TFontType Type, LBOOL Embedded, const char* FontName, TFStyle Style, double FontSize, const void* Font);
typedef void PDF_CALL TSetLeading(const void* Data, const BYTE* Object, double Value);
typedef void PDF_CALL TSetLineCapStyle(const void* Data, const BYTE* Object, TLineCapStyle Style);
typedef void PDF_CALL TSetLineDashPattern(const void* Data, const BYTE* Object, double* Dash, UI32 NumValues, UI32 Phase);
typedef void PDF_CALL TSetLineJoinStyle(const void* Data, const BYTE* Object, TLineJoinStyle Style);
typedef void PDF_CALL TSetLineWidth(const void* Data, const BYTE* Object, double Value);
typedef void PDF_CALL TSetMiterLimit(const void* Data, const BYTE* Object, double Value);
typedef void PDF_CALL TSetStrokeColor(const void* Data, const BYTE* Object, double* Color, UI32 NumComps, TExtColorSpace CS, void* IColorSpace);
typedef void PDF_CALL TSetTextDrawMode(const void* Data, const BYTE* Object, TDrawMode Mode);
typedef void PDF_CALL TSetTextScale(const void* Data, const BYTE* Object, double Value);
typedef void PDF_CALL TSetWordSpacing(const void* Data, const BYTE* Object, double Value);
typedef SI32 PDF_CALL TShowTextArrayA(const void* Data, const BYTE* Object, struct TCTM* Matrix, const struct TTextRecordA* Source, UI32 Count, double Width);
typedef SI32 PDF_CALL TShowTextArrayW(const void* Data, const struct TTextRecordA* Source, struct TCTM* Matrix, const struct TTextRecordW* Kerning, UI32 Count, double Width, LBOOL Decoded);

// Unnecessary callback functions can be set to NULL.
struct TPDFParseInterface
{
   TApplyPattern*        ApplyPattern;
   TBeginPattern*        BeginPattern;
   TBeginTemplate*       BeginTemplate;
   TBezierTo1*           BezierTo1;
   TBezierTo2*           BezierTo2;
   TBezierTo3*           BezierTo3;
   TClipPath*            ClipPath;
   TClosePath*           ClosePath;
   TDrawShading*         DrawShading;
   TEndPattern*          EndPattern;
   TEndTemplate*         EndTemplate;
   TLineTo*              LineTo;
   TMoveTo*              MoveTo;
   TMulMatrix*           MulMatrix;
   TRectangle*           Rectangle;
   TRestoreGraphicState* RestoreGraphicState;
   TSaveGraphicState*    SaveGraphicState;
   TSetCharSpacing*      SetCharSpacing;
   TSetExtGState*        SetExtGState;
   TSetFillColor*        SetFillColor;
   TSetFont*             SetFont;
   TSetLeading*          SetLeading;
   TSetLineCapStyle*     SetLineCapStyle;
   TSetLineDashPattern*  SetLineDashPattern;
   TSetLineJoinStyle*    SetLineJoinStyle;
   TSetLineWidth*        SetLineWidth;
   TSetMiterLimit*       SetMiterLimit;
   TSetStrokeColor*      SetStrokeColor;
   TSetTextDrawMode*     SetTextDrawMode;
   TSetTextScale*        SetTextScale;
   TSetWordSpacing*      SetWordSpacing;
   void*                 Reserved001;    // The old callabck functions TShowTextA and TShowTextW are no longer defined
   void*                 Reserved002;    // The old callabck functions TShowTextA and TShowTextW are no longer defined
   TShowTextArrayW*      ShowTextArrayW;
   TInsertImage*         InsertImage;
   TShowTextArrayA*      ShowTextArrayA;
   /* These variables must be initialized with NULL! */
   void*                 Reserved01;
   void*                 Reserved02;
   void*                 Reserved03;
   void*                 Reserved04;
   void*                 Reserved05;
   void*                 Reserved06;
   void*                 Reserved07;
   void*                 Reserved08;
   void*                 Reserved09;
   void*                 Reserved10;
   void*                 Reserved11;
   void*                 Reserved12;
   void*                 Reserved13;
   void*                 Reserved14;
   void*                 Reserved15;
   void*                 Reserved16;
   void*                 Reserved17;
   void*                 Reserved18;
   void*                 Reserved19;
   void*                 Reserved20;
   void*                 Reserved21;
   void*                 Reserved22;
   void*                 Reserved23;
   void*                 Reserved24;
   void*                 Reserved25;
   void*                 Reserved26;
   void*                 Reserved27;
};

struct TPDFSigParms
{
   UI32          StructSize;   // Must be set to sizeof(TSignParms)
   UI32          PKCS7ObjLen;  // The maximum length of the signed PKCS#7 object
   THashType     HashType;     // If set to htDetached, the bytes ranges of the PDF file will be returned.
   const BYTE*   Range1;       // Out -> Contains either the hash or the first byte range to create a detached signature
   UI32          Range1Len;    // Out -> Length of the buffer
   const BYTE*   Range2;       // Out -> Set only if HashType == htDetached
   UI32          Range2Len;    // Out -> Length of the buffer
   const char*   ContactInfoA; // Optional, e.g. an email address
   const UI16*   ContactInfoW; // Optional, e.g. an email address
   const char*   LocationA;    // Optional location of the signer
   const UI16*   LocationW;    // Optional location of the signer
   const char*   ReasonA;      // Optional reason why the file was signed
   const UI16*   ReasonW;      // Optional reason why the file was signed
   const char*   SignerA;      // Optional, the issuer of the certificate takes precedence
   const UI16*   SignerW;      // Optional, the issuer of the certificate takes precedence
   LBOOL         Encrypt;      // If true, the file will be encrypted
   // These members will be ignored if Encrypt is set to false
   const char*   OpenPwd;      // Open password
   const char*   OwnerPwd;     // Owner password to change the security settings
   TKeyLen       KeyLen;       // Key length to be used to encrypt the file
   TRestrictions Restrict;     // What should be restricted?
};

/*
   ---------------------------------------- NOTICE: ----------------------------------------

   ------------------ DO NOT CHANGE A MEMBER OF THIS STRUCTURE AT RUNTIME! ------------------

   The only members which can be changed at runtime are ConvColors, DestSpace, and DeleteKerningAt.
   All other members are read-only!

   -----------------------------------------------------------------------------------------
*/
struct TPDFStack
{
   struct TCTM          ctm;          // Pre-multiplied global transformation matrix
   struct TCTM          tm;           // Pre-multiplied text transformation matrix
   double               x;            // Unused -> always 0.0
   double               y;            // Unused -> always 0.0
   double               FontSize;     // Font size
   double               CharSP;       // Character spacing
   double               WordSP;       // Word spacing
   double               HScale;       // Horizontal text scaling
   double               TextRise;     // Text rise -> always 0.0 because it is already included in the text transformation matrix
   double               Leading;      // Leading
   double               LineWidth;    // Line width
   TDrawMode            DrawMode;     // Text draw mode
   TPDFColorSpace       FillCS;       // Fill color space
   TPDFColorSpace       StrokeCS;     // Stroke color space
   UI32                 FillColor;    // Fill color
   UI32                 StrokeColor;  // Stroke color
   void*                BaseObject;   // Internal
   LBOOL                CIDFont;      // If true, ReplacePageText() can only be used to delete a string
   char*                Text;         // Raw text without kerning space
   UI32                 TextLen;      // Raw text length
   struct TTextRecordA* RawKern;      // Raw kerning array
   struct TTextRecordW* Kerning;      // Already translated Unicode kerning array
   UI32                 KerningCount; // Number of kerning records
   float                TextWidth;    // The width of the entire text record measured in text space
   void*                IFont;        // Font object used to print the string -> fntGetFont() can be used to return the font properties
   LBOOL                Embedded;     // If true, the font is embedded
   float                SpaceWidth;   // Measured in text space
   /*
      These members can be modified after the structure has been initialized with InitStack().
      If the destination color space should be DeviceCMYK initialize FillColor and StrokeColor
      with PDF_CMYK(0,0,0,255); which represents black.
   */
   LBOOL                ConvColors;   // If set to true (default), all colors are converted to the specified destination color space
   TPDFColorSpace       DestSpace;    // Destination color space -> default == csDeviceRGB

   // This member can be used in combination with ReplacePageText() to preserve a number
   // of kerning records from deletion. All records above this value will be deleted.
   // Take a look into the file examples/util/pdf_edit_text.cpp to determine how this member
   // can be used.
   UI32                 DeleteKerningAt;
   UI32                 FontFlags;    // PDF font flags
   // ------------------------------- Reserved fields -------------------------------
   SI32                 Reserved1;
   SI32                 Reserved2;
   SI32                 Reserved3;
   SI32                 Reserved4;
   SI32                 Reserved5;
   SI32                 Reserved6;
   SI32                 Reserved7;
   SI32                 Reserved8;
   SI32                 Reserved9;
   SI32                 Reserved10;
   SI32                 Reserved11;
   void*                ContentPtr;
};

#if !defined(PDF_USE_CLASS) // Disable all exported functions to enable class access

// Debug stuff -> these functions do nothing in a release configuration!
PDF_EXP unsigned int PDF_CALL pdfGetAllocCount(void);
PDF_EXP unsigned int PDF_CALL pdfGetDenyAllocAt(void);
PDF_EXP unsigned int PDF_CALL pdfGetDenyCount(void);
PDF_EXP void         PDF_CALL pdfSetDenyAllocAt(unsigned int Count);

/*
   Due to the complexity of font handling a complete font API will be available later in a separate library.
*/

PDF_EXP LBOOL  PDF_CALL fntBuildFamilyNameAndStyle(const void* IFont, char Name[128], TFStyle ADDR Style);
PDF_EXP LBOOL  PDF_CALL fntGetFont(const void* IFont, struct TPDFFontObj* F);
PDF_EXP double PDF_CALL fntGetSpaceWidth(const void* IFont, double FontSize);
PDF_EXP double PDF_CALL fntGetTextWidth(const void* IFont, const BYTE* Text, UI32 Len, float CharSpacing, float WordSpacing, float TextScale);
PDF_EXP UI32   PDF_CALL fntTranslateRawCode(const void* IFont, const BYTE* Text, UI32 Len, double ADDR Width, UI16 OutText[32], SI32 ADDR OutLen, LBOOL ADDR Decoded, float CharSpacing, float WordSpacing, float TextScale);
PDF_EXP SI32   PDF_CALL fntTranslateString(struct TPDFStack* Stack, UI16* Buffer, UI32 Size, UI32 Flags);
PDF_EXP SI32   PDF_CALL fntTranslateString2(const void* IFont, const BYTE* Text, UI32 Len, UI16* OutText, UI32 Size, UI32 Flags);


PDF_EXP LBOOL  PDF_CALL pdfAddObjectToLayer(const void* IPDF, UI32 OCG, TOCObject ObjType, UI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfBeginLayer(const void* IPDF, UI32 OCG);
PDF_EXP SI32   PDF_CALL pdfCreateOCGA(const void* IPDF, const char* Name, LBOOL DisplayInUI, LBOOL Visible, TOCGIntent Intent);
PDF_EXP SI32   PDF_CALL pdfCreateOCGW(const void* IPDF, const UI16* Name, LBOOL DisplayInUI, LBOOL Visible, TOCGIntent Intent);
PDF_EXP SI32   PDF_CALL pdfCreateOCMD(const void* IPDF, TOCVisibility Visibility, UI32* OCGs, UI32 Count);
PDF_EXP SI32   PDF_CALL pdfCreateSetOCGStateAction(const void* IPDF, UI32* On, UI32 OnCount, UI32* Off, UI32 OffCount, UI32* Toggle, UI32 ToggleCount, LBOOL PreserveRB);
PDF_EXP LBOOL  PDF_CALL pdfEndLayer(const void* IPDF);

PDF_EXP LBOOL  PDF_CALL pdfGetFieldChoiceValue(const void* IPDF, UI32 AField, UI32 ValIndex, struct TPDFChoiceValue* Value);


PDF_EXP LBOOL  PDF_CALL pdfAddActionToObj(const void* IPDF, TObjType ObjType, TObjEvent Event, UI32 ActHandle, UI32 ObjHandle);
PDF_EXP SI32   PDF_CALL pdfAddArticle(const void* IPDF, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfAddBookmarkA(const void* IPDF, const char* Title, SI32 Parent, UI32 DestPage, LBOOL Open);
PDF_EXP SI32   PDF_CALL pdfAddBookmarkW(const void* IPDF, const UI16* Title, SI32 Parent, UI32 DestPage, LBOOL Open);
PDF_EXP SI32   PDF_CALL pdfAddBookmarkExA(const void* IPDF, const char* Title, SI32 Parent, UI32 NamedDest, LBOOL Open);
PDF_EXP SI32   PDF_CALL pdfAddBookmarkExW(const void* IPDF, const UI16* Title, SI32 Parent, UI32 NamedDest, LBOOL Open);
PDF_EXP LBOOL  PDF_CALL pdfAddButtonImage(const void* IPDF, UI32 BtnHandle, TButtonState State, const char* Caption, const char* ImgFile);
PDF_EXP LBOOL  PDF_CALL pdfAddButtonImageEx(const void* IPDF, UI32 BtnHandle, TButtonState State, const char* Caption, void* hBitmap);
PDF_EXP LBOOL  PDF_CALL pdfAddContinueTextA(const void* IPDF, const char* AText);
PDF_EXP LBOOL  PDF_CALL pdfAddContinueTextW(const void* IPDF, const UI16* AText);
PDF_EXP LBOOL  PDF_CALL pdfAddDeviceNProcessColorants(const void* IPDF, UI32 DeviceNCS, const char** Colorants, UI32 NumColorants, TExtColorSpace ProcessCS, SI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfAddDeviceNSeparations(const void* IPDF, UI32 DeviceNCS, const char** Colorants, UI32* SeparationCS, UI32 NumColorants);
PDF_EXP LBOOL  PDF_CALL pdfAddFieldToFormAction(const void* IPDF, UI32 Action, UI32 AField, LBOOL Include);
PDF_EXP LBOOL  PDF_CALL pdfAddFieldToHideAction(const void* IPDF, UI32 HideAct, UI32 AField);
PDF_EXP LBOOL  PDF_CALL pdfAddFileCommentA(const void* IPDF, const char* AText);
PDF_EXP LBOOL  PDF_CALL pdfAddFileCommentW(const void* IPDF, const UI16* AText);
PDF_EXP SI32   PDF_CALL pdfAddFontSearchPathA(const void* IPDF, const char* APath, LBOOL Recursive);
PDF_EXP SI32   PDF_CALL pdfAddFontSearchPathW(const void* IPDF, const UI16* APath, LBOOL Recursive);
PDF_EXP LBOOL  PDF_CALL pdfAddImage(const void* IPDF, TCompressionFilter Filter, TImageConversionFlags Flags, struct TPDFImage* Image);
PDF_EXP SI32   PDF_CALL pdfAddJavaScriptA(const void* IPDF, const char* Name, const char* Script);
PDF_EXP SI32   PDF_CALL pdfAddJavaScriptW(const void* IPDF, const char* Name, const UI16* Script);
PDF_EXP SI32   PDF_CALL pdfAddPageLabelA(const void* IPDF, UI32 StartRange, TPageLabelFormat Format, const char* Prefix, SI32 FirstPageNum);
PDF_EXP SI32   PDF_CALL pdfAddPageLabelW(const void* IPDF, UI32 StartRange, TPageLabelFormat Format, const UI16* Prefix, SI32 FirstPageNum);
PDF_EXP SI32   PDF_CALL pdfAddRenderingIntentA(const void* IPDF, const char* ICCFile);
PDF_EXP SI32   PDF_CALL pdfAddRenderingIntentW(const void* IPDF, const UI16* ICCFile);
PDF_EXP SI32   PDF_CALL pdfAddRenderingIntentEx(const void* IPDF, const void* Buffer, UI32 BufSize);
PDF_EXP LBOOL  PDF_CALL pdfAddValToChoiceFieldA(const void* IPDF, UI32 Field, const char* ExpValue, const char* Value, LBOOL Selected);
PDF_EXP LBOOL  PDF_CALL pdfAddValToChoiceFieldW(const void* IPDF, UI32 Field, const char* ExpValue, const UI16* Value, LBOOL Selected);
PDF_EXP LBOOL  PDF_CALL pdfAppend(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfApplyPattern(const void* IPDF, SI32 PattHandle, TColorMode ColorMode, UI32 Color);
PDF_EXP LBOOL  PDF_CALL pdfApplyShading(const void* IPDF, SI32 ShadHandle);
PDF_EXP SI32   PDF_CALL pdfAttachFileA(const void* IPDF, const char* FilePath, const char* Description, LBOOL Compress);
PDF_EXP SI32   PDF_CALL pdfAttachFileW(const void* IPDF, const UI16* FilePath, const UI16* Description, LBOOL Compress);
PDF_EXP SI32   PDF_CALL pdfAttachFileExA(const void* IPDF, const void* Buffer, UI32 BufSize, const char* FileName, const char* Description, LBOOL Compress);
PDF_EXP SI32   PDF_CALL pdfAttachFileExW(const void* IPDF, const void* Buffer, UI32 BufSize, const UI16* FileName, const UI16* Description, LBOOL Compress);
PDF_EXP LBOOL  PDF_CALL pdfAutoTemplate(const void* IPDF, UI32 Templ, double PosX, double PosY, double Width, double Height);
PDF_EXP LBOOL  PDF_CALL pdfBeginClipPath(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfBeginContinueText(const void* IPDF, double PosX, double PosY);
PDF_EXP SI32   PDF_CALL pdfBeginPattern(const void* IPDF, TPatternType PatternType, TTilingType TilingType, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfBeginTemplate(const void* IPDF, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfBeginTemplateEx(const void* IPDF, struct TPDFRect* BBox, struct TCTM* Matrix);
PDF_EXP LBOOL  PDF_CALL pdfBezier_1_2_3(const void* IPDF, double x1, double y1, double x2, double y2, double x3, double y3);
PDF_EXP LBOOL  PDF_CALL pdfBezier_1_3(const void* IPDF, double x1, double y1, double x3, double y3);
PDF_EXP LBOOL  PDF_CALL pdfBezier_2_3(const void* IPDF, double x2, double y2, double x3, double y3);
PDF_EXP double PDF_CALL pdfCalcWidthHeight(const void* IPDF, double OrgWidth, double OrgHeight, double ScaledWidth, double ScaledHeight);
PDF_EXP LBOOL  PDF_CALL pdfChangeAnnotPos(const void* IPDF, UI32 Handle, double PosX, double PosY, double Width, double Height);
PDF_EXP LBOOL  PDF_CALL pdfChangeAnnotNameA(const void* IPDF, UI32 Handle, const char* Name);
PDF_EXP LBOOL  PDF_CALL pdfChangeAnnotNameW(const void* IPDF, UI32 Handle, const UI16* Name);
PDF_EXP LBOOL  PDF_CALL pdfChangeBookmarkA(const void* IPDF, SI32 ABmk, const char* Title, SI32 Parent, UI32 DestPage, LBOOL Open);
PDF_EXP LBOOL  PDF_CALL pdfChangeBookmarkW(const void* IPDF, SI32 ABmk, const UI16* Title, SI32 Parent, UI32 DestPage, LBOOL Open);
PDF_EXP LBOOL  PDF_CALL pdfChangeFont(const void* IPDF, SI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfChangeFontSize(const void* IPDF, double Size);
PDF_EXP LBOOL  PDF_CALL pdfChangeFontStyle(const void* IPDF, TFStyle Style);
PDF_EXP LBOOL  PDF_CALL pdfChangeFontStyleEx(const void* IPDF, TFStyle Style);
PDF_EXP LBOOL  PDF_CALL pdfChangeJavaScriptA(const void* IPDF, UI32 Handle, const char* NewScript);
PDF_EXP LBOOL  PDF_CALL pdfChangeJavaScriptW(const void* IPDF, UI32 Handle, const UI16* NewScript);
PDF_EXP LBOOL  PDF_CALL pdfChangeJavaScriptActionA(const void* IPDF, UI32 Handle, const char* NewScript);
PDF_EXP LBOOL  PDF_CALL pdfChangeJavaScriptActionW(const void* IPDF, UI32 Handle, const UI16* NewScript);
PDF_EXP LBOOL  PDF_CALL pdfChangeJavaScriptNameA(const void* IPDF, UI32 Handle, const char* Name);
PDF_EXP LBOOL  PDF_CALL pdfChangeJavaScriptNameW(const void* IPDF, UI32 Handle, const UI16* Name);
PDF_EXP LBOOL  PDF_CALL pdfChangeLinkAnnot(const void* IPDF, UI32 Handle, const char* URL);
PDF_EXP SI32   PDF_CALL pdfCheckAndRepairFonts(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfCheckCollection(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfCheckConformance(const void* IPDF, TConformanceType Type, TCheckOptions Options, const void* UserData, TOnFontNotFoundProc* OnFontNotFound, TOnReplaceICCProfile* OnReplaceICCProfile);
PDF_EXP SI32   PDF_CALL pdfCheckFieldNames(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfClearAutoTemplates(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfClearHostFonts(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfClipPath(const void* IPDF, TClippingMode ClipMode, TPathFillMode FillMode);
PDF_EXP LBOOL  PDF_CALL pdfCloseAndSignFile(const void* IPDF, const char* CertFile, const char* Password, const char* Reason, const char* Location);
PDF_EXP LBOOL  PDF_CALL pdfCloseAndSignFileEx(const void* IPDF, const char* OpenPwd, const char* OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict, const char* CertFile, const char* Password, const char* Reason, const char* Location);
PDF_EXP LBOOL  PDF_CALL pdfCloseAndSignFileExt(const void* IPDF, struct TPDFSigParms* SigParms);
PDF_EXP LBOOL  PDF_CALL pdfCloseFile(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfCloseFileEx(const void* IPDF, const char* OpenPwd, const char* OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict);
PDF_EXP LBOOL  PDF_CALL pdfCloseImage(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfCloseImportFile(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfClosePath(const void* IPDF, TPathFillMode FillMode);
PDF_EXP LBOOL  PDF_CALL pdfCloseTag(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfComputeBBox(const void* IPDF, struct TPDFRect ADDR BBox, TCompBBoxFlags Flags);
PDF_EXP UI32   PDF_CALL pdfConvColor(const double* Color, UI32 NumComps, TExtColorSpace SourceCS, const void* IColorSpace, TExtColorSpace DestCS);
PDF_EXP LBOOL  PDF_CALL pdfConvertColors(const void* IPDF, TColorConvFlags Flags, const void* Reserved);
PDF_EXP UI16*  PDF_CALL pdfConvToUnicode(const void* IPDF, const char* AString, TCodepage CP);
PDF_EXP SI32   PDF_CALL pdfCreate3DAnnot(const void* IPDF, double PosX, double PosY, double Width, double Height, const char* Author, const char* Name, const char* U3DFile, const char* Image);
PDF_EXP LBOOL  PDF_CALL pdfCreate3DBackground(const void* IPDF, const void* IView, UI32 BackColor);
PDF_EXP SI32   PDF_CALL pdfCreate3DGotoViewAction(const void* IPDF, UI32 Base3DAnnot, const void* IView, T3DNamedAction Named);
PDF_EXP LBOOL  PDF_CALL pdfCreate3DProjection(const void* IPDF, const void* IView, T3DProjType ProjType, T3DScaleType ScaleType, double Diameter, double FOV);
PDF_EXP void*  PDF_CALL pdfCreate3DView(const void* IPDF, UI32 Base3DAnnot, const char* Name, LBOOL SetAsDefault, double* Matrix, double CamDistance, T3DRenderingMode RM, T3DLightingSheme LS);
PDF_EXP SI32   PDF_CALL pdfCreateAnnotAP(const void* IPDF, UI32 Annot);
PDF_EXP SI32   PDF_CALL pdfCreateArticleThreadA(const void* IPDF, const char* ThreadName);
PDF_EXP SI32   PDF_CALL pdfCreateArticleThreadW(const void* IPDF, const UI16* ThreadName);
PDF_EXP SI32   PDF_CALL pdfCreateAxialShading(const void* IPDF, double sX, double sY, double eX, double eY, double SCenter, UI32 SColor, UI32 EColor, LBOOL Extend1, LBOOL Extend2);
PDF_EXP SI32   PDF_CALL pdfCreateButtonA(const void* IPDF, const char* Name, const char* Caption, SI32 Parent, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfCreateButtonW(const void* IPDF, const char* Name, const UI16* Caption, SI32 Parent, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfCreateCheckBox(const void* IPDF, const char* Name, const char* ExpValue, LBOOL Checked, SI32 Parent, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfCreateCIEColorSpace(const void* IPDF, TExtColorSpace CS, float* WhitePoint, float* BlackPoint, float* Gamma, float* Matrix);
PDF_EXP LBOOL  PDF_CALL pdfCreateColItemDate(const void* IPDF, UI32 EmbFile, const char* Key, long Date, const char* Prefix);
PDF_EXP LBOOL  PDF_CALL pdfCreateColItemNumber(const void* IPDF, UI32 EmbFile, const char* Key, double Value, const char* Prefix);
PDF_EXP LBOOL  PDF_CALL pdfCreateColItemStringA(const void* IPDF, UI32 EmbFile, const char* Key, const char* Value, const char* Prefix);
PDF_EXP LBOOL  PDF_CALL pdfCreateColItemStringW(const void* IPDF, UI32 EmbFile, const char* Key, const UI16* Value, const UI16* Prefix);
PDF_EXP LBOOL  PDF_CALL pdfCreateCollection(const void* IPDF, TColView View);
PDF_EXP SI32   PDF_CALL pdfCreateCollectionFieldA(const void* IPDF, TColColumnType ColType, SI32 Column, const char* Name, const char* Key, LBOOL Visible, LBOOL Editable);
PDF_EXP SI32   PDF_CALL pdfCreateCollectionFieldW(const void* IPDF, TColColumnType ColType, SI32 Column, const UI16* Name, const char* Key, LBOOL Visible, LBOOL Editable);
PDF_EXP SI32   PDF_CALL pdfCreateComboBox(const void* IPDF, const char* Name, LBOOL Sort, SI32 Parent, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfCreateDeviceNColorSpace(const void* IPDF, const char** Colorants, UI32 NumColorants, const char* PostScriptFunc, TExtColorSpace Alternate, SI32 Handle);
PDF_EXP SI32   PDF_CALL pdfCreateExtGState(const void* IPDF, struct TPDFExtGState* GS);
PDF_EXP SI32   PDF_CALL pdfCreateGoToAction(const void* IPDF, TDestType DestType, UI32 PageNum, double a, double b, double c, double d);
PDF_EXP SI32   PDF_CALL pdfCreateGoToActionEx(const void* IPDF, UI32 NamedDest);
PDF_EXP SI32   PDF_CALL pdfCreateGoToRAction(const void* IPDF, const char* FileName, UI32 PageNum);
PDF_EXP SI32   PDF_CALL pdfCreateGoToRActionExA(const void* IPDF, const char* FileName, char* DestName, LBOOL NewWindow);
PDF_EXP SI32   PDF_CALL pdfCreateGoToRActionExW(const void* IPDF, const char* FileName, UI16* DestName, LBOOL NewWindow);
PDF_EXP SI32   PDF_CALL pdfCreateGroupField(const void* IPDF, const char* Name, SI32 Parent);
PDF_EXP SI32   PDF_CALL pdfCreateHideAction(const void* IPDF, UI32 AField, LBOOL Hide);
PDF_EXP SI32   PDF_CALL pdfCreateICCBasedColorSpaceA(const void* IPDF, const char* ICCProfile);
PDF_EXP SI32   PDF_CALL pdfCreateICCBasedColorSpaceW(const void* IPDF, const UI16* ICCProfile);
PDF_EXP LBOOL  PDF_CALL pdfCreateImageA(const void* IPDF, const char* FileName, TImageFormat Format);
PDF_EXP LBOOL  PDF_CALL pdfCreateImageW(const void* IPDF, const UI16* FileName, TImageFormat Format);
PDF_EXP SI32   PDF_CALL pdfCreateImpDataActionA(const void* IPDF, const char* DataFile);
PDF_EXP SI32   PDF_CALL pdfCreateImpDataActionW(const void* IPDF, const UI16* DataFile);
PDF_EXP SI32   PDF_CALL pdfCreateIndexedColorSpace(const void* IPDF, TExtColorSpace Base, SI32 Handle, const void* ColorTable, UI32 NumColors);
PDF_EXP SI32   PDF_CALL pdfCreateJSActionA(const void* IPDF, const char* Script);
PDF_EXP SI32   PDF_CALL pdfCreateJSActionW(const void* IPDF, const UI16* Script);
PDF_EXP SI32   PDF_CALL pdfCreateLaunchAction(const void* IPDF, TFileOP OP, const char* FileName, const char* DefDir, const char* Param, LBOOL NewWindow);
PDF_EXP SI32   PDF_CALL pdfCreateListBox(const void* IPDF, const char* Name, LBOOL Sort, SI32 Parent, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfCreateNamedAction(const void* IPDF, TNamedAction Action);
PDF_EXP SI32   PDF_CALL pdfCreateNamedDestA(const void* IPDF, const char* Name, UI32 DestPage, TDestType DestType, double a, double b, double c, double d);
PDF_EXP SI32   PDF_CALL pdfCreateNamedDestW(const void* IPDF, const UI16* Name, UI32 DestPage, TDestType DestType, double a, double b, double c, double d);
PDF_EXP LBOOL  PDF_CALL pdfCreateNewPDFA(const void* IPDF, const char* OutPDF);
PDF_EXP LBOOL  PDF_CALL pdfCreateNewPDFW(const void* IPDF, const UI16* OutPDF);
PDF_EXP SI32   PDF_CALL pdfCreateRadialShading(const void* IPDF, double sX, double sY, double R1, double eX, double eY, double R2, double SCenter, UI32 SColor, UI32 EColor, LBOOL Extend1, LBOOL Extend2);
PDF_EXP SI32   PDF_CALL pdfCreateRadioButton(const void* IPDF, const char* Name, const char* ExpValue, SI32 Checked, SI32 Parent, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfCreateResetAction(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfCreateSeparationCS(const void* IPDF, const char* Colorant, TExtColorSpace Alternate, SI32 Handle, UI32 Color);
PDF_EXP SI32   PDF_CALL pdfCreateSigField(const void* IPDF, const char* Name, SI32 Parent, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfCreateSigFieldAP(const void* IPDF, UI32 SigField);
PDF_EXP SI32   PDF_CALL pdfCreateStdPattern(const void* IPDF, TStdPattern Pattern, double LineWidth, double Distance, UI32 LineColor, UI32 BackColor);
PDF_EXP LBOOL  PDF_CALL pdfCreateStructureTree(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfCreateSubmitAction(const void* IPDF, TSubmitFlags Flags, const char* URL);
PDF_EXP SI32   PDF_CALL pdfCreateTextField(const void* IPDF, const char* Name, SI32 Parent, LBOOL Multiline, SI32 MaxLen, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfCreateURIAction(const void* IPDF, const char* URL);
PDF_EXP SI32   PDF_CALL pdfDecryptPDFA(const void* IPDF, const char* FileName, TPwdType PwdType, const char* Password);
PDF_EXP SI32   PDF_CALL pdfDecryptPDFW(const void* IPDF, const UI16* FileName, TPwdType PwdType, const char* Password);
PDF_EXP void   PDF_CALL pdfDeleteAcroForm(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfDeleteActionFromObj(const void* IPDF, TObjType ObjType, UI32 ActHandle, UI32 ObjHandle);
PDF_EXP LBOOL  PDF_CALL pdfDeleteActionFromObjEx(const void* IPDF, TObjType ObjType, UI32 ObjHandle, UI32 ActIndex);
PDF_EXP LBOOL  PDF_CALL pdfDeleteAnnotation(const void* IPDF, UI32 Handle);
PDF_EXP SI32   PDF_CALL pdfDeleteBookmark(const void* IPDF, UI32 ABmk);
PDF_EXP LBOOL  PDF_CALL pdfDeleteEmbeddedFile(const void* IPDF, UI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfDeleteField(const void* IPDF, UI32 AField);
PDF_EXP LBOOL  PDF_CALL pdfDeleteFieldEx(const void* IPDF, const char* Name);
PDF_EXP void   PDF_CALL pdfDeleteJavaScripts(const void* IPDF, LBOOL DelJavaScriptActions);
PDF_EXP LBOOL  PDF_CALL pdfDeletePage(const void* IPDF, UI32 PageNum);
PDF_EXP void   PDF_CALL pdfDeletePageLabels(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfDeletePDF(void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfDeleteSeparationInfo(const void* IPDF, LBOOL AllPages);
PDF_EXP LBOOL  PDF_CALL pdfDeleteTemplate(const void* IPDF, UI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfDeleteTemplateEx(const void* IPDF, UI32 Index);
PDF_EXP void   PDF_CALL pdfDeleteXFAForm(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfDrawArc(const void* IPDF, double PosX, double PosY, double Radius, double StartAngle, double EndAngle);
PDF_EXP LBOOL  PDF_CALL pdfDrawArcEx(const void* IPDF, double PosX, double PosY, double Width, double Height, double StartAngle, double EndAngle);
PDF_EXP LBOOL  PDF_CALL pdfDrawChord(const void* IPDF, double PosX, double PosY, double Width, double Height, double StartAngle, double EndAngle, TPathFillMode FillMode);
PDF_EXP LBOOL  PDF_CALL pdfDrawCircle(const void* IPDF, double PosX, double PosY, double Radius, TPathFillMode FillMode);
PDF_EXP LBOOL  PDF_CALL pdfDrawPie(const void* IPDF, double PosX, double PosY, double Width, double Height, double StartAngle, double EndAngle, TPathFillMode FillMode);
PDF_EXP LBOOL  PDF_CALL pdfEditPage(const void* IPDF, SI32 PageNum);
PDF_EXP LBOOL  PDF_CALL pdfEditTemplate(const void* IPDF, UI32 Index);
PDF_EXP LBOOL  PDF_CALL pdfEditTemplate2(const void* IPDF, UI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfEllipse(const void* IPDF, double PosX, double PosY, double Width, double Height, TPathFillMode FillMode);
PDF_EXP SI32   PDF_CALL pdfEncryptPDFA(const void* IPDF, const char* FileName, const char* OpenPwd, const char* OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict);
PDF_EXP SI32   PDF_CALL pdfEncryptPDFW(const void* IPDF, const UI16* FileName, const char* OpenPwd, const char* OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict);
PDF_EXP LBOOL  PDF_CALL pdfEndContinueText(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfEndPage(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfEndPattern(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfEndTemplate(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfEnumDocFonts(const void* IPDF, const void* Data, TEnumFontProc2* EnumProc);
PDF_EXP SI32   PDF_CALL pdfEnumHostFonts(const void* IPDF, const void* Data, TEnumFontProc* EnumProc);
PDF_EXP SI32   PDF_CALL pdfEnumHostFontsEx(const void* IPDF, const void* Data, TEnumFontProcEx* EnumProc);
PDF_EXP LBOOL  PDF_CALL pdfExchangeBookmarks(const void* IPDF, SI32 Bmk1, SI32 Bmk2);
PDF_EXP LBOOL  PDF_CALL pdfExchangePages(const void* IPDF, UI32 First, UI32 Second);
PDF_EXP SI32   PDF_CALL pdfFileAttachAnnotA(const void* IPDF, double PosX, double PosY, TFileAttachIcon Icon, const char* Author, const char* Desc, const char* AFile, LBOOL Compress);
PDF_EXP SI32   PDF_CALL pdfFileAttachAnnotW(const void* IPDF, double PosX, double PosY, TFileAttachIcon Icon, const UI16* Author, const UI16* Desc, const UI16* AFile, LBOOL Compress);
PDF_EXP SI32   PDF_CALL pdfFileLinkA(const void* IPDF, double PosX, double PosY, double Width, double Height, const char* AFilePath);
PDF_EXP SI32   PDF_CALL pdfFileLinkW(const void* IPDF, double PosX, double PosY, double Width, double Height, const UI16* AFilePath);
PDF_EXP SI32   PDF_CALL pdfFindBookmarkA(const void* IPDF, SI32 DestPage, const char* Title);
PDF_EXP SI32   PDF_CALL pdfFindBookmarkW(const void* IPDF, SI32 DestPage, const UI16* Title);
PDF_EXP SI32   PDF_CALL pdfFindFieldA(const void* IPDF, const char* Name);
PDF_EXP SI32   PDF_CALL pdfFindFieldW(const void* IPDF, const UI16* Name);
PDF_EXP SI32   PDF_CALL pdfFindLinkAnnot(const void* IPDF, const char* URL);
PDF_EXP SI32   PDF_CALL pdfFindNextBookmark(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfFinishSignature(const void* IPDF, const void* PKCS7Obj, UI32 Length);
PDF_EXP SI32   PDF_CALL pdfFlattenAnnots(const void* IPDF, TAnnotFlattenFlags Flags);
PDF_EXP LBOOL  PDF_CALL pdfFlattenForm(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfFlushPageContent(const void* IPDF, struct TPDFStack* Stack);
PDF_EXP void   PDF_CALL pdfFreeImageBuffer(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfFreePDF(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfFreeTextAnnotA(const void* IPDF, double PosX, double PosY, double Width, double Height, const char* Author, const char* AText, TTextAlign Align);
PDF_EXP SI32   PDF_CALL pdfFreeTextAnnotW(const void* IPDF, double PosX, double PosY, double Width, double Height, const UI16* Author, const UI16* AText, TTextAlign Align);
PDF_EXP LBOOL  PDF_CALL pdfFreeUniBuf(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetActionCount(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetActionType(const void* IPDF, UI32 ActHandle);
PDF_EXP SI32   PDF_CALL pdfGetActionTypeEx(const void* IPDF, TObjType ObjType, UI32 ObjHandle, UI32 ActIndex);
PDF_EXP SI32   PDF_CALL pdfGetActiveFont(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetAllocBy(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetAnnot(const void* IPDF, UI32 Handle, struct TPDFAnnotation ADDR Annot);
PDF_EXP LBOOL  PDF_CALL pdfGetAnnotBBox(const void* IPDF, UI32 Handle, struct TPDFRect ADDR BBox);
PDF_EXP SI32   PDF_CALL pdfGetAnnotCount(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetAnnotEx(const void* IPDF, UI32 Handle, struct TPDFAnnotationEx ADDR Annot);
PDF_EXP SI32   PDF_CALL pdfGetAnnotFlags(const void* IPDF);
PDF_EXP char*  PDF_CALL pdfGetAnnotLink(const void* IPDF, UI32 Handle);
PDF_EXP SI32   PDF_CALL pdfGetAnnotType(const void* IPDF, UI32 Handle);
PDF_EXP double PDF_CALL pdfGetAscent(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetBBox(const void* IPDF, TPageBoundary Boundary, struct TPDFRect ADDR BBox);
PDF_EXP SI32   PDF_CALL pdfGetBidiMode(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetBookmark(const void* IPDF, SI32 Handle, struct TBookmark ADDR Bmk);
PDF_EXP SI32   PDF_CALL pdfGetBookmarkCount(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetBorderStyle(const void* IPDF);
PDF_EXP char*  PDF_CALL pdfGetBuffer(const void* IPDF, UI32 ADDR BufSize);
PDF_EXP double PDF_CALL pdfGetCapHeight(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetCharacterSpacing(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetCheckBoxChar(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetCheckBoxCharEx(const void* IPDF, UI32 AField);
PDF_EXP SI32   PDF_CALL pdfGetCheckBoxDefState(const void* IPDF, UI32 AField);
PDF_EXP LBOOL  PDF_CALL pdfGetCMap(const void* IPDF, UI32 Index, struct TPDFCMap* CMap);
PDF_EXP SI32   PDF_CALL pdfGetCMapCount(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetColorSpace(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetColorSpaceCount(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetColorSpaceObj(const void* IPDF, UI32 Handle, struct TPDFColorSpaceObj* CS);
PDF_EXP LBOOL  PDF_CALL pdfGetColorSpaceObjEx(const void* IColorSpace, struct TPDFColorSpaceObj* CS);
PDF_EXP SI32   PDF_CALL pdfGetCompressionFilter(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetCompressionLevel(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetContent(const void* IPDF, char* ADDR Buffer);
PDF_EXP SI32   PDF_CALL pdfGetDefBitsPerPixel(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetDescent(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetDeviceNAttributes(void* IAttributes, struct TDeviceNAttributes* Attributes);
PDF_EXP SI32   PDF_CALL pdfGetDocInfo(const void* IPDF, TDocumentInfo DInfo, UI16* ADDR Value);
PDF_EXP SI32   PDF_CALL pdfGetDocInfoCount(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetDocInfoEx(const void* IPDF, UI32 Index, TDocumentInfo ADDR DInfo, char* ADDR Key, char* ADDR Value, LBOOL ADDR Unicode);
PDF_EXP SI32   PDF_CALL pdfGetDrawDirection(const void* IPDF);
PDF_EXP char*  PDF_CALL pdfGetDynaPDFVersion(void);
PDF_EXP LBOOL  PDF_CALL pdfGetEmbeddedFile(const void* IPDF, UI32 Handle, struct TPDFFileSpec* FileSpec, LBOOL Decompress);
PDF_EXP SI32   PDF_CALL pdfGetEmbeddedFileCount(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetEMFPatternDistance(const void* IPDF);
PDF_EXP char*  PDF_CALL pdfGetErrorMessage(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetErrorMode(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetField(const void* IPDF, UI32 Handle, struct TPDFField* Field);
PDF_EXP UI32   PDF_CALL pdfGetFieldBackColor(const void* IPDF);
PDF_EXP UI32   PDF_CALL pdfGetFieldBorderColor(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetFieldBorderStyle(const void* IPDF, UI32 AField);
PDF_EXP double PDF_CALL pdfGetFieldBorderWidth(const void* IPDF, UI32 AField);
PDF_EXP LBOOL  PDF_CALL pdfGetFieldColor(const void* IPDF, UI32 AField, TFieldColor ColorType, SI32 ADDR ColorSpace, UI32 ADDR Color);
PDF_EXP SI32   PDF_CALL pdfGetFieldCount(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetFieldEx(const void* IPDF, UI32 Handle, struct TPDFFieldEx* Field);
PDF_EXP LBOOL  PDF_CALL pdfGetFieldEx2(const void* IField, struct TPDFFieldEx* Field);
PDF_EXP SI32   PDF_CALL pdfGetFieldExpValCount(const void* IPDF, UI32 AField);
PDF_EXP SI32   PDF_CALL pdfGetFieldExpValue(const void* IPDF, UI32 AField, char* ADDR Value);
PDF_EXP LBOOL  PDF_CALL pdfGetFieldExpValueEx(const void* IPDF, UI32 AField, UI32 ValIndex, char* ADDR Value, char* ADDR ExpValue, LBOOL ADDR Selected);
PDF_EXP SI32   PDF_CALL pdfGetFieldFlags(const void* IPDF, UI32 AField);
PDF_EXP SI32   PDF_CALL pdfGetFieldGroupType(const void* IPDF, UI32 AField);
PDF_EXP SI32   PDF_CALL pdfGetFieldHighlightMode(const void* IPDF, UI32 AField);
PDF_EXP SI32   PDF_CALL pdfGetFieldIndex(const void* IPDF, UI32 AField);
PDF_EXP SI32   PDF_CALL pdfGetFieldMapName(const void* IPDF, UI32 AField, void* ADDR Value, LBOOL ADDR Unicode);
PDF_EXP SI32   PDF_CALL pdfGetFieldName(const void* IPDF, UI32 AField, char* ADDR Name);
PDF_EXP SI32   PDF_CALL pdfGetFieldOrientation(const void* IPDF, UI32 AField);
PDF_EXP SI32   PDF_CALL pdfGetFieldTextAlign(const void* IPDF, UI32 AField);
PDF_EXP UI32   PDF_CALL pdfGetFieldTextColor(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetFieldToolTip(const void* IPDF, UI32 AField, void* ADDR Value, LBOOL ADDR Unicode);
PDF_EXP SI32   PDF_CALL pdfGetFieldType(const void* IPDF, UI32 AField);
PDF_EXP UI32   PDF_CALL pdfGetFillColor(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetFontCount(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetFontEx(const void* IPDF, UI32 Handle, struct TPDFFontObj* F);
PDF_EXP SI32   PDF_CALL pdfGetFontOrigin(const void* IPDF);
PDF_EXP void   PDF_CALL pdfGetFontSearchOrder(const void* IPDF, TFontBaseType Order[4]);
PDF_EXP SI32   PDF_CALL pdfGetFontSelMode(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetFontWeight(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetFTextHeightA(const void* IPDF, TTextAlign Align, const char* AText);
PDF_EXP double PDF_CALL pdfGetFTextHeightW(const void* IPDF, TTextAlign Align, const UI16* AText);
PDF_EXP double PDF_CALL pdfGetFTextHeightExA(const void* IPDF, double Width, TTextAlign Align, const char* AText);
PDF_EXP double PDF_CALL pdfGetFTextHeightExW(const void* IPDF, double Width, TTextAlign Align, const UI16* AText);
PDF_EXP UI32   PDF_CALL pdfGetGStateFlags(const void* IPDF);
PDF_EXP UI32   PDF_CALL pdfGetIconColor(const void* IPDF);
PDF_EXP char*  PDF_CALL pdfGetImageBuffer(const void* IPDF, UI32 ADDR BufSize);
PDF_EXP SI32   PDF_CALL pdfGetImageCount(const void* IPDF, const char* FileName);
PDF_EXP SI32   PDF_CALL pdfGetImageCountEx(const void* IPDF, const void* Buffer, UI32 BufSize);
PDF_EXP SI32   PDF_CALL pdfGetImageHeight(const void* IPDF, UI32 Handle);
PDF_EXP SI32   PDF_CALL pdfGetImageWidth(const void* IPDF, UI32 Handle);
PDF_EXP UI32   PDF_CALL pdfGetImportFlags(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetInBBox(const void* IPDF, UI32 PageNum, TPageBoundary Boundary, struct TPDFRect ADDR BBox);
PDF_EXP SI32   PDF_CALL pdfGetInDocInfo(const void* IPDF, TDocumentInfo DInfo, UI16* ADDR Value);
PDF_EXP SI32   PDF_CALL pdfGetInDocInfoCount(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetInDocInfoEx(const void* IPDF, UI32 Index, TDocumentInfo ADDR DInfo, char* ADDR Key, char* ADDR Value, LBOOL ADDR Unicode);
PDF_EXP SI32   PDF_CALL pdfGetInEncryptionFlags(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetInIsCollection(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetInIsEncrypted(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetInIsSigned(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetInIsTrapped(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetInIsXFAForm(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetInNamedDest(const void* IPDF, UI32 Index, struct TPDFNamedDest* Dest);
PDF_EXP SI32   PDF_CALL pdfGetInNamedDestCount(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetInOrientation(const void* IPDF, SI32 PageNum);
PDF_EXP SI32   PDF_CALL pdfGetInPageCount(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetInPDFVersion(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetInPrintSettings(const void* IPDF, struct TPDFPrintSettings* Settings);
PDF_EXP LBOOL  PDF_CALL pdfGetIsFixedPitch(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetItalicAngle(const void* IPDF);
PDF_EXP char*  PDF_CALL pdfGetJavaScript(const void* IPDF, UI32 Handle, UI32 ADDR Len, LBOOL ADDR Unicode);
PDF_EXP char*  PDF_CALL pdfGetJavaScriptAction(const void* IPDF, UI32 Handle, UI32 ADDR Len, LBOOL ADDR Unicode);
PDF_EXP char*  PDF_CALL pdfGetJavaScriptAction2(const void* IPDF, TObjType ObjType, UI32 ObjHandle, UI32 ActIndex, UI32 ADDR Len, LBOOL ADDR Unicode, TObjEvent ADDR Event);
PDF_EXP SI32   PDF_CALL pdfGetJavaScriptCount(const void* IPDF);
PDF_EXP char*  PDF_CALL pdfGetJavaScriptEx(const void* IPDF, const char* Name, UI32 ADDR Len, LBOOL ADDR Unicode);
PDF_EXP char*  PDF_CALL pdfGetJavaScriptName(const void* IPDF, UI32 Handle, UI32 ADDR Len, LBOOL ADDR Unicode);
PDF_EXP SI32   PDF_CALL pdfGetJPEGQuality(const void* IPDF);
PDF_EXP char*  PDF_CALL pdfGetLanguage(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetLastTextPosX(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetLastTextPosY(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetLeading(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetLineCapStyle(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetLineJoinStyle(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetLineWidth(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetLinkHighlightMode(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetLogMetafileSize(const void* IPDF, const char* FileName, struct TRectL* R);
PDF_EXP LBOOL  PDF_CALL pdfGetLogMetafileSizeEx(const void* IPDF, const void* Buffer, UI32 BufSize, struct TRectL* R);
PDF_EXP LBOOL  PDF_CALL pdfGetMatrix(const void* IPDF, struct TCTM* M);
PDF_EXP SI32   PDF_CALL pdfGetMaxFieldLen(const void* IPDF, UI32 TxtField);
PDF_EXP UI32   PDF_CALL pdfGetMetaConvFlags(const void* IPDF);
PDF_EXP UI32*  PDF_CALL pdfGetMissingGlyphs(const void* IPDF, UI32 ADDR Count);
PDF_EXP double PDF_CALL pdfGetMiterLimit(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetNeedAppearance(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetObjActionCount(const void* IPDF, TObjType ObjType, UI32 ObjHandle);
PDF_EXP double PDF_CALL pdfGetOpacity(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetOrientation(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetPageAnnot(const void* IPDF, UI32 Index, struct TPDFAnnotation ADDR Annot);
PDF_EXP SI32   PDF_CALL pdfGetPageAnnotCount(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetPageAnnotEx(const void* IPDF, UI32 Index, struct TPDFAnnotationEx ADDR Annot);
PDF_EXP SI32   PDF_CALL pdfGetPageCoords(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetPageCount(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetPageField(const void* IPDF, UI32 Index, struct TPDFField* Field);
PDF_EXP SI32   PDF_CALL pdfGetPageFieldCount(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetPageFieldEx(const void* IPDF, UI32 Index, struct TPDFFieldEx* Field);
PDF_EXP double PDF_CALL pdfGetPageHeight(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetPageLabel(const void* IPDF, UI32 Index, struct TPDFPageLabel* Label);
PDF_EXP SI32   PDF_CALL pdfGetPageLabelCount(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetPageLayout(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetPageMode(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetPageNum(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetPageText(const void* IPDF, struct TPDFStack* Stack);
PDF_EXP double PDF_CALL pdfGetPageWidth(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetPDFVersion(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetPrintSettings(const void* IPDF, struct TPDFPrintSettings* Settings);
PDF_EXP SI32   PDF_CALL pdfGetResolution(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetSaveNewImageFormat(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetSeparationInfo(const void* IPDF, char* ADDR Colorant, TExtColorSpace ADDR CS);
PDF_EXP UI32   PDF_CALL pdfGetStrokeColor(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetTabLen(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetTemplCount(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetTemplHandle(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetTemplHeight(const void* IPDF, SI32 Handle);
PDF_EXP double PDF_CALL pdfGetTemplWidth(const void* IPDF, SI32 Handle);
PDF_EXP SI32   PDF_CALL pdfGetTextDrawMode(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetTextFieldValue(const void* IPDF, UI32 AField, char* ADDR Value, LBOOL ADDR ValIsUnicode, char* ADDR DefValue, LBOOL ADDR DefValIsUnicode);
PDF_EXP LBOOL  PDF_CALL pdfGetTextRect(const void* IPDF, double ADDR PosX, double ADDR PosY, double ADDR Width, double ADDR Height);
PDF_EXP double PDF_CALL pdfGetTextRise(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetTextScaling(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetTextWidthA(const void* IPDF, const char* AText);
PDF_EXP double PDF_CALL pdfGetTextWidthW(const void* IPDF, const UI16* AText);
PDF_EXP double PDF_CALL pdfGetTextWidthExA(const void* IPDF, const char* AText, UI32 Len);
PDF_EXP double PDF_CALL pdfGetTextWidthExW(const void* IPDF, const UI16* AText, UI32 Len);
PDF_EXP UI32   PDF_CALL pdfGetTransparentColor(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetTrapped(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetUseExactPwd(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetUseGlobalImpFiles(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfGetUserRights(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetUseStdFonts(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetUseSystemFonts(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetUseTransparency(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetUseVisibleCoords(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfGetViewerPrefrences(const void* IPDF, SI32 ADDR Preference, SI32 ADDR AddVal);
PDF_EXP LBOOL  PDF_CALL pdfGetWMFDefExtent(const void* IPDF, UI32 ADDR Width, UI32 ADDR Height);
PDF_EXP SI32   PDF_CALL pdfGetWMFPixelPerInch(const void* IPDF);
PDF_EXP double PDF_CALL pdfGetWordSpacing(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfHaveOpenDoc(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfHighlightAnnotA(const void* IPDF, TAnnotType SubType, double PosX, double PosY, double Width, double Height, UI32 Color, const char* Author, const char* Subject, const char* Comment);
PDF_EXP SI32   PDF_CALL pdfHighlightAnnotW(const void* IPDF, TAnnotType SubType, double PosX, double PosY, double Width, double Height, UI32 Color, const UI16* Author, const UI16* Subject, const UI16* Comment);
PDF_EXP SI32   PDF_CALL pdfImportBookmarks(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfImportCatalogObjects(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfImportDocInfo(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfImportPage(const void* IPDF, UI32 PageNum);
PDF_EXP SI32   PDF_CALL pdfImportPageEx(const void* IPDF, UI32 PageNum, double ScaleX, double ScaleY);
PDF_EXP SI32   PDF_CALL pdfImportPDFFile(const void* IPDF, UI32 DestPage, double ScaleX, double ScaleY);
PDF_EXP LBOOL  PDF_CALL pdfInitExtGState(struct TPDFExtGState* GS);
PDF_EXP LBOOL  PDF_CALL pdfInitStack(const void* IPDF, struct TPDFStack* Stack);
PDF_EXP SI32   PDF_CALL pdfInsertBMPFromBuffer(const void* IPDF, double PosX, double PosY, double ScaleWidth, double ScaleHeight, const void* Buffer);
PDF_EXP SI32   PDF_CALL pdfInsertBMPFromHandle(const void* IPDF, double PosX, double PosY, double ScaleWidth, double ScaleHeight, void* hBitmap);
PDF_EXP SI32   PDF_CALL pdfInsertImage(const void* IPDF, double PosX, double PosY, double ScaleWidth, double ScaleHeight, const char* AFile);
PDF_EXP SI32   PDF_CALL pdfInsertImageEx(const void* IPDF, double PosX, double PosY, double ScaleWidth, double ScaleHeight, const char* AFile, UI32 Index);
PDF_EXP SI32   PDF_CALL pdfInsertImageFromBuffer(const void* IPDF, double PosX, double PosY, double ScaleWidth, double ScaleHeight, const void* Buffer, UI32 BufSize, UI32 Index);
PDF_EXP LBOOL  PDF_CALL pdfInsertMetafile(const void* IPDF, const char* FileName, double PosX, double PosY, double Width, double Height);
PDF_EXP LBOOL  PDF_CALL pdfInsertMetafileEx(const void* IPDF, const void* Buffer, UI32 BufSize, double PosX, double PosY, double Width, double Height);
PDF_EXP LBOOL  PDF_CALL pdfInsertMetafileExt(const void* IPDF, const char* FileName, struct TRectL* View, double PosX, double PosY, double Width, double Height);
PDF_EXP LBOOL  PDF_CALL pdfInsertMetafileExtEx(const void* IPDF, const void* Buffer, UI32 BufSize, struct TRectL* View, double PosX, double PosY, double Width, double Height);
PDF_EXP LBOOL  PDF_CALL pdfInsertMetafileFromHandle(const void* IPDF, const void* hEnhMetafile, double PosX, double PosY, double Width, double Height);
PDF_EXP LBOOL  PDF_CALL pdfInsertMetafileFromHandleEx(const void* IPDF, const void* hEnhMetafile, struct TRectL* View, double PosX, double PosY, double Width, double Height);
PDF_EXP SI32   PDF_CALL pdfInsertRawImage(const void* IPDF, const void* Buffer, UI32 BitsPerPixel, UI32 ColorCount, SI32 ImgWidth, SI32 ImgHeight, double PosX, double PosY, double ScaleWidth, double ScaleHeight);
PDF_EXP SI32   PDF_CALL pdfIsBidiText(const void* IPDF, const UI16* AText);
PDF_EXP LBOOL  PDF_CALL pdfIsColorPage(const void* IPDF, LBOOL GrayIsColor);
PDF_EXP LBOOL  PDF_CALL pdfIsEmptyPage(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfLineTo(const void* IPDF, double PosX, double PosY);
PDF_EXP SI32   PDF_CALL pdfLoadCMap(const void* IPDF, const char* CMapName, LBOOL Embed);
PDF_EXP LBOOL  PDF_CALL pdfLoadFDFDataA(const void* IPDF, const char* FileName, const char* Password, UI32 Flags);
PDF_EXP LBOOL  PDF_CALL pdfLoadFDFDataW(const void* IPDF, const UI16* FileName, const char* Password, UI32 Flags);
PDF_EXP LBOOL  PDF_CALL pdfLoadFDFDataEx(const void* IPDF, const void* Buffer, UI32 BufSize, const char* Password, UI32 Flags);
PDF_EXP SI32   PDF_CALL pdfLoadFont(const void* IPDF, const void* Buffer, UI32 BufSize, TFStyle Style, double Size, LBOOL Embed, TCodepage CP);
PDF_EXP LBOOL  PDF_CALL pdfMovePage(const void* IPDF, UI32 Source, UI32 Dest);
PDF_EXP LBOOL  PDF_CALL pdfMoveTo(const void* IPDF, double PosX, double PosY);
PDF_EXP LBOOL  PDF_CALL pdfMultiplyMatrix(struct TCTM* M1, struct TCTM* M2, struct TCTM* NewMatrix);
PDF_EXP void*  PDF_CALL pdfNewPDF(void);
PDF_EXP SI32   PDF_CALL pdfOpenImportBuffer(const void* IPDF, const void* Buffer, UI32 BufSize, TPwdType PwdType, const char* Password);
PDF_EXP SI32   PDF_CALL pdfOpenImportFileA(const void* IPDF, const char* FileName, TPwdType PwdType, const char* Password);
PDF_EXP SI32   PDF_CALL pdfOpenImportFileW(const void* IPDF, const UI16* FileName, TPwdType PwdType, const char* Password);
PDF_EXP LBOOL  PDF_CALL pdfOpenOutputFileA(const void* IPDF, const char* OutPDF);
PDF_EXP LBOOL  PDF_CALL pdfOpenOutputFileW(const void* IPDF, const UI16* OutPDF);
PDF_EXP LBOOL  PDF_CALL pdfOpenTagA(const void* IPDF, TPDFBaseTag Tag, const char* Lang, const char* AltText, const char* Expansion);
PDF_EXP LBOOL  PDF_CALL pdfOpenTagW(const void* IPDF, TPDFBaseTag Tag, const char* Lang, const UI16* AltText, const UI16* Expansion);
PDF_EXP SI32   PDF_CALL pdfPageLink(const void* IPDF, double PosX, double PosY, double Width, double Height, UI32 DestPage);
PDF_EXP SI32   PDF_CALL pdfPageLink2(const void* IPDF, double PosX, double PosY, double Width, double Height, UI32 NamedDest);
PDF_EXP SI32   PDF_CALL pdfPageLinkEx(const void* IPDF, double PosX, double PosY, double Width, double Height, TDestType DestType, UI32 DestPage, double a, double b, double c, double d);
PDF_EXP LBOOL  PDF_CALL pdfParseContent(const void* IPDF, const void* Data, struct TPDFParseInterface* Stack, TParseFlags Flags);
PDF_EXP LBOOL  PDF_CALL pdfPlaceImage(const void* IPDF, SI32 ImgHandle, double PosX, double PosY, double ScaleWidth, double ScaleHeight);
PDF_EXP LBOOL  PDF_CALL pdfPlaceSigFieldValidateIcon(const void* IPDF, UI32 SigField, double PosX, double PosY, double Width, double Height);
PDF_EXP LBOOL  PDF_CALL pdfPlaceTemplate(const void* IPDF, SI32 TmplHandle, double PosX, double PosY, double ScaleWidth, double ScaleHeight);
PDF_EXP LBOOL  PDF_CALL pdfPlaceTemplateEx(const void* IPDF, SI32 TmplHandle, double PosX, double PosY, double ScaleWidth, double ScaleHeight);
PDF_EXP LBOOL  PDF_CALL pdfPlaceTemplByMatrix(const void* IPDF, SI32 TmplHandle);
PDF_EXP LBOOL  PDF_CALL pdfReadImageFormat(const void* IPDF, const char* FileName, UI32 ADDR Width, UI32 ADDR Height, SI32 ADDR BitsPerPixel, SI32 ADDR UseZip);
PDF_EXP LBOOL  PDF_CALL pdfReadImageFormat2(const void* IPDF, const char* FileName, UI32 Index, UI32 ADDR Width, UI32 ADDR Height, SI32 ADDR BitsPerPixel, SI32 ADDR UseZip);
PDF_EXP LBOOL  PDF_CALL pdfReadImageFormatEx(const void* IPDF, void* hBitmap, UI32 ADDR Width, UI32 ADDR Height, SI32 ADDR BitsPerPixel, SI32 ADDR UseZip);
PDF_EXP LBOOL  PDF_CALL pdfReadImageFormatFromBuffer(const void* IPDF, const void* Buffer, UI32 BufSize, UI32 Index, UI32 ADDR Width, UI32 ADDR Height, SI32 ADDR BitsPerPixel, SI32 ADDR UseZip);
PDF_EXP LBOOL  PDF_CALL pdfReadImageResolution(const void* IPDF, const char* FileName, UI32 Index, UI32 ADDR ResX, UI32 ADDR ResY);
PDF_EXP LBOOL  PDF_CALL pdfReadImageResolutionEx(const void* IPDF, const void* Buffer, UI32 BufSize, UI32 Index, UI32 ADDR ResX, UI32 ADDR ResY);
PDF_EXP LBOOL  PDF_CALL pdfRectangle(const void* IPDF, double PosX, double PosY, double Width, double Height, TPathFillMode FillMode);
PDF_EXP SI32   PDF_CALL pdfReEncryptPDFA(const void* IPDF, const char* FileName, TPwdType PwdType, const char* InPwd, const char* NewOpenPwd, const char* NewOwnerPwd, TKeyLen NewKeyLen, TRestrictions Restrict);
PDF_EXP SI32   PDF_CALL pdfReEncryptPDFW(const void* IPDF, const UI16* FileName, TPwdType PwdType, const char* InPwd, const char* NewOpenPwd, const char* NewOwnerPwd, TKeyLen NewKeyLen, TRestrictions Restrict);
PDF_EXP SI32   PDF_CALL pdfRenameSpotColor(const void* IPDF, const char* Colorant, const char* NewName);
PDF_EXP SI32   PDF_CALL pdfReplaceFontA(const void* IPDF, const void* PDFFont, const char* Name, TFStyle Style, LBOOL NameIsFamilyName);
PDF_EXP SI32   PDF_CALL pdfReplaceFontW(const void* IPDF, const void* PDFFont, const UI16* Name, TFStyle Style, LBOOL NameIsFamilyName);
PDF_EXP SI32   PDF_CALL pdfReplaceFontExA(const void* IPDF, const void* PDFFont, const char* FontFile, LBOOL Embed);
PDF_EXP SI32   PDF_CALL pdfReplaceFontExW(const void* IPDF, const void* PDFFont, const UI16* FontFile, LBOOL Embed);
PDF_EXP SI32   PDF_CALL pdfReplaceICCProfileA(const void* IPDF, UI32 ColorSpace, const char* ICCFile);
PDF_EXP SI32   PDF_CALL pdfReplaceICCProfileW(const void* IPDF, UI32 ColorSpace, const UI16* ICCFile);
PDF_EXP LBOOL  PDF_CALL pdfReplacePageTextA(const void* IPDF, const char* NewText, struct TPDFStack* Stack);
PDF_EXP LBOOL  PDF_CALL pdfReplacePageTextExA(const void* IPDF, const char* NewText, struct TPDFStack* Stack);
PDF_EXP LBOOL  PDF_CALL pdfReplacePageTextExW(const void* IPDF, const UI16* NewText, struct TPDFStack* Stack);
PDF_EXP LBOOL  PDF_CALL pdfResetLineDashPattern(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfRestoreGraphicState(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfRotateCoords(const void* IPDF, double alpha, double OriginX, double OriginY);
PDF_EXP LBOOL  PDF_CALL pdfRoundRect(const void* IPDF, double PosX, double PosY, double Width, double Height, double Radius, TPathFillMode FillMode);
PDF_EXP LBOOL  PDF_CALL pdfRoundRectEx(const void* IPDF, double PosX, double PosY, double Width, double Height, double rWidth, double rHeight, TPathFillMode FillMode);
PDF_EXP LBOOL  PDF_CALL pdfSaveGraphicState(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfScaleCoords(const void* IPDF, double sx, double sy);
PDF_EXP LBOOL  PDF_CALL pdfSelfTest(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfSet3DAnnotProps(const void* IPDF, UI32 Handle, T3DActivationType ActType, T3DDeActivateType DeActType, T3DInstanceType InstType, T3DDeActInstance DeInstType, LBOOL DisplToolbar, LBOOL DisplModelTree);
PDF_EXP LBOOL  PDF_CALL pdfSet3DAnnotScriptA(const void* IPDF, UI32 Annot, const char* Value, UI32 Len);
PDF_EXP SI32   PDF_CALL pdfSetAllocBy(const void* IPDF, SI32 Value);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotBorderStyle(const void* IPDF, UI32 Handle, TBorderStyle Style);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotBorderWidth(const void* IPDF, UI32 Handle, double LineWidth);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotColor(const void* IPDF, UI32 Handle, TAnnotColor ColorType, TPDFColorSpace CS, UI32 Color);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotFlags(const void* IPDF, TAnnotFlags Flags);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotFlagsEx(const void* IPDF, UI32 Handle, SI32 Flags);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotHighlightMode(const void* IPDF, UI32 Handle, THighlightMode Mode);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotIcon(const void* IPDF, UI32 Handle, TAnnotIcon Icon);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotOpenState(const void* IPDF, UI32 Handle, LBOOL Open);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotStringA(const void* IPDF, UI32 Handle, TAnnotString StringType, const char* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotStringW(const void* IPDF, UI32 Handle, TAnnotString StringType, const UI16* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotSubjectA(const void* IPDF, UI32 Handle, const char* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetAnnotSubjectW(const void* IPDF, UI32 Handle, const UI16* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetBBox(const void* IPDF, TPageBoundary Boundary, double LeftX, double LeftY, double RightX, double RightY);
PDF_EXP LBOOL  PDF_CALL pdfSetBidiMode(const void* IPDF, TPDFBidiMode Mode);
PDF_EXP LBOOL  PDF_CALL pdfSetBookmarkDest(const void* IPDF, SI32 ABmk, TDestType DestType, double a, double b, double c, double d);
PDF_EXP LBOOL  PDF_CALL pdfSetBookmarkStyle(const void* IPDF, SI32 ABmk, TFStyle Style, UI32 RGBColor);
PDF_EXP LBOOL  PDF_CALL pdfSetBorderStyle(const void* IPDF, TBorderStyle Style);
PDF_EXP LBOOL  PDF_CALL pdfSetCharacterSpacing(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetCheckBoxChar(const void* IPDF, TCheckBoxChar CheckBoxChar);
PDF_EXP LBOOL  PDF_CALL pdfSetCheckBoxDefState(const void* IPDF, UI32 AField, LBOOL Checked);
PDF_EXP LBOOL  PDF_CALL pdfSetCheckBoxState(const void* IPDF, UI32 AField, LBOOL Checked);
PDF_EXP SI32   PDF_CALL pdfSetCIDFontA(const void* IPDF, UI32 CMapHandle, const char* Name, TFStyle Style, double Size, LBOOL Embed);
PDF_EXP SI32   PDF_CALL pdfSetCIDFontW(const void* IPDF, UI32 CMapHandle, const UI16* Name, TFStyle Style, double Size, LBOOL Embed);
PDF_EXP SI32   PDF_CALL pdfSetCMapDirA(const void* IPDF, const char* Path, LBOOL Recursive);
PDF_EXP SI32   PDF_CALL pdfSetCMapDirW(const void* IPDF, const UI16* Path, LBOOL Recursive);
PDF_EXP LBOOL  PDF_CALL pdfSetColDefFile(const void* IPDF, UI32 EmbFile);
PDF_EXP LBOOL  PDF_CALL pdfSetColSortField(const void* IPDF, UI32 ColField, LBOOL AscendingOrder);
PDF_EXP LBOOL  PDF_CALL pdfSetColors(const void* IPDF, UI32 Color);
PDF_EXP LBOOL  PDF_CALL pdfSetColorSpace(const void* IPDF, TPDFColorSpace ColorSpace);
PDF_EXP LBOOL  PDF_CALL pdfSetCompressionFilter(const void* IPDF, TCompressionFilter Filter);
PDF_EXP LBOOL  PDF_CALL pdfSetCompressionLevel(const void* IPDF, TCompressionLevel CompressLevel);
PDF_EXP LBOOL  PDF_CALL pdfSetContent(const void* IPDF, const char* Buffer, UI32 BufSize);
PDF_EXP LBOOL  PDF_CALL pdfSetDateTimeFormat(const void* IPDF, UI32 TxtField, TPDFDateTime Fmt);
PDF_EXP LBOOL  PDF_CALL pdfSetDefBitsPerPixel(const void* IPDF, SI32 Value);
PDF_EXP LBOOL  PDF_CALL pdfSetDocInfoA(const void* IPDF, TDocumentInfo DInfo, const char* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetDocInfoW(const void* IPDF, TDocumentInfo DInfo, const UI16* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetDocInfoExA(const void* IPDF, TDocumentInfo DInfo, const char* Key, const char* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetDocInfoExW(const void* IPDF, TDocumentInfo DInfo, const char* Key, const UI16* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetDrawDirection(const void* IPDF, TDrawDirection Direction);
PDF_EXP LBOOL  PDF_CALL pdfSetEMFFrameDPI(const void* IPDF, UI32 DPIX, UI32 DPIY);
PDF_EXP LBOOL  PDF_CALL pdfSetEMFPatternDistance(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetExtColorSpace(const void* IPDF, UI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfSetExtGState(const void* IPDF, UI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfSetErrorMode(const void* IPDF, TErrMode ErrMode);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldBackColor(const void* IPDF, UI32 AColor);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldBBox(const void* IPDF, UI32 AField, struct TPDFRect* BBox);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldBorderColor(const void* IPDF, UI32 AColor);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldBorderWidth(const void* IPDF, UI32 AField, double LineWidth);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldColor(const void* IPDF, UI32 AField, TFieldColor ColorType, TPDFColorSpace CS, UI32 Color);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldBorderStyle(const void* IPDF, UI32 AField, TBorderStyle Style);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldExpValue(const void* IPDF, UI32 AField, UI32 ValIndex, const char* Value, const char* ExpValue, LBOOL Selected);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldFlags(const void* IPDF, UI32 AField, TFieldFlags Flags, LBOOL Reset);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldFontSize(const void* IPDF, UI32 AField, double FontSize);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldHighlightMode(const void* IPDF, UI32 AField, THighlightMode Mode);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldIndex(const void* IPDF, UI32 AField, UI32 Index);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldMapNameA(const void* IPDF, UI32 AField, const char* Name);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldMapNameW(const void* IPDF, UI32 AField, const UI16* Name);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldName(const void* IPDF, UI32 AField, const char* NewName);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldOrientation(const void* IPDF, UI32 AField, SI32 Value);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldTextAlign(const void* IPDF, UI32 AField, TTextAlign Align);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldTextColor(const void* IPDF, UI32 Color);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldToolTipA(const void* IPDF, UI32 AField, const char* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetFieldToolTipW(const void* IPDF, UI32 AField, const UI16* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetFillColor(const void* IPDF, UI32 Color);
PDF_EXP LBOOL  PDF_CALL pdfSetFillColorEx(const void* IPDF, const BYTE* Color, UI32 NumComponents);
PDF_EXP LBOOL  PDF_CALL pdfSetFillColorSpace(const void* IPDF, TPDFColorSpace CS);
PDF_EXP SI32   PDF_CALL pdfSetFontA(const void* IPDF, const char* Name, TFStyle Style, double Size, LBOOL Embed, TCodepage CP);
PDF_EXP SI32   PDF_CALL pdfSetFontW(const void* IPDF, const UI16* Name, TFStyle Style, double Size, LBOOL Embed, TCodepage CP);
PDF_EXP SI32   PDF_CALL pdfSetFontExA(const void* IPDF, const char* Name, TFStyle Style, double Size, LBOOL Embed, TCodepage CP);
PDF_EXP SI32   PDF_CALL pdfSetFontExW(const void* IPDF, const UI16* Name, TFStyle Style, double Size, LBOOL Embed, TCodepage CP);
PDF_EXP LBOOL  PDF_CALL pdfSetFontOrigin(const void* IPDF, TOrigin Origin);
PDF_EXP void   PDF_CALL pdfSetFontSearchOrder(const void* IPDF, TFontBaseType Order[4]);
PDF_EXP LBOOL  PDF_CALL pdfSetFontSelMode(const void* IPDF, TFontSelMode Mode);
PDF_EXP LBOOL  PDF_CALL pdfSetFontWeight(const void* IPDF, SI32 Weight);
PDF_EXP void   PDF_CALL pdfSetGStateFlags(const void* IPDF, TGStateFlags Flags, LBOOL Reset);
PDF_EXP LBOOL  PDF_CALL pdfSetIconColor(const void* IPDF, UI32 Color);
PDF_EXP LBOOL  PDF_CALL pdfSetImportFlags(const void* IPDF, TImportFlags Flags);
PDF_EXP LBOOL  PDF_CALL pdfSetImportFlags2(const void* IPDF, TImportFlags2 Flags);
PDF_EXP LBOOL  PDF_CALL pdfSetItalicAngle(const void* IPDF, double Angle);
PDF_EXP LBOOL  PDF_CALL pdfSetJPEGQuality(const void* IPDF, SI32 Value);
PDF_EXP LBOOL  PDF_CALL pdfSetLanguage(const void* IPDF, const char* ISOTag);
PDF_EXP LBOOL  PDF_CALL pdfSetLeading(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetLicenseKey(const void* IPDF, const char* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetLineCapStyle(const void* IPDF, TLineCapStyle Style);
PDF_EXP LBOOL  PDF_CALL pdfSetLineDashPattern(const void* IPDF, const char* Dash, SI32 Phase);
PDF_EXP LBOOL  PDF_CALL pdfSetLineDashPatternEx(const void* IPDF, const double* Dash, UI32 NumValues, SI32 Phase);
PDF_EXP LBOOL  PDF_CALL pdfSetLineJoinStyle(const void* IPDF, TLineJoinStyle Style);
PDF_EXP LBOOL  PDF_CALL pdfSetLineWidth(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetLinkHighlightMode(const void* IPDF, THighlightMode Mode);
PDF_EXP LBOOL  PDF_CALL pdfSetListFont(const void* IPDF, UI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfSetMatrix(const void* IPDF, struct TCTM* Matrix);
PDF_EXP LBOOL  PDF_CALL pdfSetMaxFieldLen(const void* IPDF, UI32 TxtField, SI32 MaxLen);
PDF_EXP LBOOL  PDF_CALL pdfSetMetaConvFlags(const void* IPDF, TMetaFlags Flags);
PDF_EXP LBOOL  PDF_CALL pdfSetMiterLimit(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetNeedAppearance(const void* IPDF, LBOOL Value);
PDF_EXP LBOOL  PDF_CALL pdfSetNumberFormat(const void* IPDF, UI32 TxtField, TDecSeparator Sep, UI32 DecPlaces, TNegativeStyle NegStyle, const char* CurrStr, LBOOL Prepend);
PDF_EXP LBOOL  PDF_CALL pdfSetOnErrorProc(const void* IPDF, const void* Data, TErrorProc* ErrProc);
PDF_EXP LBOOL  PDF_CALL pdfSetOnPageBreakProc(const void* IPDF, const void* Data, TOnPageBreakProc* OnBreakProc);
PDF_EXP LBOOL  PDF_CALL pdfSetOpacity(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetOrientation(const void* IPDF, SI32 Value);
PDF_EXP LBOOL  PDF_CALL pdfSetOrientationEx(const void* IPDF, SI32 Value);
PDF_EXP LBOOL  PDF_CALL pdfSetPageCoords(const void* IPDF, TPageCoord PageCoords);
PDF_EXP LBOOL  PDF_CALL pdfSetPageFormat(const void* IPDF, TPageFormat Value);
PDF_EXP LBOOL  PDF_CALL pdfSetPageHeight(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetPageLayout(const void* IPDF, TPageLayout Layout);
PDF_EXP LBOOL  PDF_CALL pdfSetPageMode(const void* IPDF, TPageMode Mode);
PDF_EXP LBOOL  PDF_CALL pdfSetPageWidth(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetPDFVersion(const void* IPDF, TPDFVersion Version);
PDF_EXP LBOOL  PDF_CALL pdfSetPrintSettings(const void* IPDF, TDuplexMode Mode, SI32 PickTrayByPDFSize, UI32 NumCopies, TPrintScaling PrintScaling, UI32* PrintRanges, UI32 NumRanges);
PDF_EXP LBOOL  PDF_CALL pdfSetProgressProc(const void* IPDF, const void* Data, TInitProgressProc* InitProgress, TProgressProc* Progress);
PDF_EXP LBOOL  PDF_CALL pdfSetResolution(const void* IPDF, UI32 Value);
PDF_EXP LBOOL  PDF_CALL pdfSetSaveNewImageFormat(const void* IPDF, LBOOL Value);
PDF_EXP LBOOL  PDF_CALL pdfSetSeparationInfo(const void* IPDF, UI32 Handle);
PDF_EXP LBOOL  PDF_CALL pdfSetStrokeColor(const void* IPDF, UI32 Color);
PDF_EXP LBOOL  PDF_CALL pdfSetStrokeColorEx(const void* IPDF, const BYTE* Color, UI32 NumComponents);
PDF_EXP LBOOL  PDF_CALL pdfSetStrokeColorSpace(const void* IPDF, TPDFColorSpace CS);
PDF_EXP LBOOL  PDF_CALL pdfSetTabLen(const void* IPDF, SI32 TabLen);
PDF_EXP LBOOL  PDF_CALL pdfSetTextDrawMode(const void* IPDF, TDrawMode Mode);
PDF_EXP LBOOL  PDF_CALL pdfSetTextFieldValueA(const void* IPDF, UI32 Field, const char* Value, const char* DefValue, TTextAlign Align);
PDF_EXP LBOOL  PDF_CALL pdfSetTextFieldValueW(const void* IPDF, UI32 Field, const UI16* Value, const UI16* DefValue, TTextAlign Align);
PDF_EXP LBOOL  PDF_CALL pdfSetTextFieldValueExA(const void* IPDF, UI32 Field, const char* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetTextFieldValueExW(const void* IPDF, UI32 Field, const UI16* Value);
PDF_EXP LBOOL  PDF_CALL pdfSetTextRect(const void* IPDF, double PosX, double PosY, double Width, double Height);
PDF_EXP LBOOL  PDF_CALL pdfSetTextRise(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetTextScaling(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSetTransparentColor(const void* IPDF, UI32 AColor);
PDF_EXP void   PDF_CALL pdfSetTrapped(const void* IPDF, LBOOL Value);
PDF_EXP LBOOL  PDF_CALL pdfSetUseExactPwd(const void* IPDF, LBOOL Value);
PDF_EXP LBOOL  PDF_CALL pdfSetUseGlobalImpFiles(const void* IPDF, LBOOL Value);
PDF_EXP LBOOL  PDF_CALL pdfSetUseStdFonts(const void* IPDF, LBOOL Value);
PDF_EXP LBOOL  PDF_CALL pdfSetUseSwapFile(const void* IPDF, LBOOL SwapContents, UI32 SwapLimit);
PDF_EXP LBOOL  PDF_CALL pdfSetUseSwapFileEx(const void* IPDF, LBOOL SwapContents, UI32 SwapLimit, const char* SwapDir);
PDF_EXP LBOOL  PDF_CALL pdfSetUseSystemFonts(const void* IPDF, LBOOL Value);
PDF_EXP LBOOL  PDF_CALL pdfSetUseTransparency(const void* IPDF, LBOOL Value);
PDF_EXP LBOOL  PDF_CALL pdfSetUseVisibleCoords(const void* IPDF, LBOOL Value);
PDF_EXP LBOOL  PDF_CALL pdfSetViewerPreferences(const void* IPDF, TViewerPreference Value, TViewPrefAddVal AddVal);
PDF_EXP LBOOL  PDF_CALL pdfSetWMFDefExtent(const void* IPDF, UI32 Width, UI32 Height);
PDF_EXP LBOOL  PDF_CALL pdfSetWMFPixelPerInch(const void* IPDF, UI32 Value);
PDF_EXP LBOOL  PDF_CALL pdfSetWordSpacing(const void* IPDF, double Value);
PDF_EXP LBOOL  PDF_CALL pdfSkewCoords(const void* IPDF, double alpha, double beta, double OriginX, double OriginY);
PDF_EXP LBOOL  PDF_CALL pdfSortFieldsByIndex(const void* IPDF);
PDF_EXP LBOOL  PDF_CALL pdfSortFieldsByName(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfStampAnnotA(const void* IPDF, TRubberStamp SubType, double PosX, double PosY, double Width, double Height, const char* Author, const char* Subject, const char* Comment);
PDF_EXP SI32   PDF_CALL pdfStampAnnotW(const void* IPDF, TRubberStamp SubType, double PosX, double PosY, double Width, double Height, const UI16* Author, const UI16* Subject, const UI16* Comment);
PDF_EXP SI32   PDF_CALL pdfStrLenA(const char* AStr);
PDF_EXP SI32   PDF_CALL pdfStrLenW(const UI16* AStr);
PDF_EXP LBOOL  PDF_CALL pdfStrokePath(const void* IPDF);
PDF_EXP SI32   PDF_CALL pdfTextAnnotA(const void* IPDF, double PosX, double PosY, double Width, double Height, const char* Author, const char* Text, TAnnotIcon Icon, LBOOL Open);
PDF_EXP SI32   PDF_CALL pdfTextAnnotW(const void* IPDF, double PosX, double PosY, double Width, double Height, const UI16* Author, const UI16* Text, TAnnotIcon Icon, LBOOL Open);
PDF_EXP LBOOL  PDF_CALL pdfTranslateCoords(const void* IPDF, double OriginX, double OriginY);
PDF_EXP LBOOL  PDF_CALL pdfTriangle(const void* IPDF, double x1, double y1, double x2, double y2, double x3, double y3, TPathFillMode FillMode);
PDF_EXP UI32*  PDF_CALL pdfUTF16ToUTF32(const void* IPDF, const UI16* Source);
PDF_EXP UI32*  PDF_CALL pdfUTF16ToUTF32Ex(const void* IPDF, const UI16* Source, UI32 ADDR Len);
PDF_EXP UI16*  PDF_CALL pdfUTF32ToUTF16(const void* IPDF, const UI32* Source);
PDF_EXP UI16*  PDF_CALL pdfUTF32ToUTF16Ex(const void* IPDF, const UI32* Source, UI32 ADDR Len);
PDF_EXP SI32   PDF_CALL pdfWebLinkA(const void* IPDF, double PosX, double PosY, double Width, double Height, const char* URL);
PDF_EXP SI32   PDF_CALL pdfWebLinkW(const void* IPDF, double PosX, double PosY, double Width, double Height, const UI16* URL);
PDF_EXP LBOOL  PDF_CALL pdfWriteAngleTextA(const void* IPDF, const char* AText, double Angle, double PosX, double PosY, double Radius, double YOrigin);
PDF_EXP LBOOL  PDF_CALL pdfWriteAngleTextW(const void* IPDF, const UI16* AText, double Angle, double PosX, double PosY, double Radius, double YOrigin);
PDF_EXP LBOOL  PDF_CALL pdfWriteFTextA(const void* IPDF, TTextAlign Align, const char* AText);
PDF_EXP LBOOL  PDF_CALL pdfWriteFTextW(const void* IPDF, TTextAlign Align, const UI16* AText);
PDF_EXP LBOOL  PDF_CALL pdfWriteFTextExA(const void* IPDF, double PosX, double PosY, double Width, double Height, TTextAlign Align, const char* AText);
PDF_EXP LBOOL  PDF_CALL pdfWriteFTextExW(const void* IPDF, double PosX, double PosY, double Width, double Height, TTextAlign Align, const UI16* AText);
PDF_EXP LBOOL  PDF_CALL pdfWriteTextA(const void* IPDF, double PosX, double PosY, const char* AText);
PDF_EXP LBOOL  PDF_CALL pdfWriteTextW(const void* IPDF, double PosX, double PosY, const UI16* AText);
PDF_EXP LBOOL  PDF_CALL pdfWriteTextExA(const void* IPDF, double PosX, double PosY, const char* AText, UI32 Len);
PDF_EXP LBOOL  PDF_CALL pdfWriteTextExW(const void* IPDF, double PosX, double PosY, const UI16* AText, UI32 Len);
PDF_EXP LBOOL  PDF_CALL pdfWriteTextMatrixA(const void* IPDF, struct TCTM* M, const char* AText);
PDF_EXP LBOOL  PDF_CALL pdfWriteTextMatrixW(const void* IPDF, struct TCTM* M, const UI16* AText);
PDF_EXP LBOOL  PDF_CALL pdfWriteTextMatrixExA(const void* IPDF, struct TCTM* M, const char* AText, UI32 Len);
PDF_EXP LBOOL  PDF_CALL pdfWriteTextMatrixExW(const void* IPDF, struct TCTM* M, const UI16* AText, UI32 Len);

#ifdef UNICODE
   #define pdfAddBookmark              pdfAddBookmarkW
   #define pdfAddBookmarkEx            pdfAddBookmarkExW
   #define pdfAddContinueText          pdfAddContinueTextW
   #define pdfAddFontSearchPath        pdfAddFontSearchPathW
   #define pdfAddFileComment           pdfAddFileCommentW
   #define pdfAddJavaScript            pdfAddJavaScriptW
   #define pdfAddPageLabel             pdfAddPageLabelW
   #define pdfAddRenderingIntent       pdfAddRenderingIntentW
   #define pdfAddValToChoiceField      pdfAddValToChoiceFieldW
   #define pdfAttachFile               pdfAttachFileW
   #define pdfAttachFileEx             pdfAttachFileExW
   #define pdfChangeAnnotName          pdfChangeAnnotNameW
   #define pdfChangeBookmark           pdfChangeBookmarkW
   #define pdfChangeJavaScript         pdfChangeJavaScriptW
   #define pdfChangeJavaScriptAction   pdfChangeJavaScriptActionW
   #define pdfChangeJavaScriptName     pdfChangeJavaScriptNameW
   #define pdfCreateArticleThread      pdfCreateArticleThreadW
   #define pdfCreateButton             pdfCreateButtonW
   #define pdfCreateCollectionField    pdfCreateCollectionFieldW
   #define pdfCreateColItemString      pdfCreateColItemStringW
   #define pdfCreateGoToRActionEx      pdfCreateGoToRActionExW
   #define pdfCreateICCBasedColorSpace pdfCreateICCBasedColorSpaceW
   #define pdfCreateImage              pdfCreateImageW
   #define pdfCreateImpDataAction      pdfCreateImpDataActionW
   #define pdfCreateJSAction           pdfCreateJSActionW
   #define pdfCreateNamedDest          pdfCreateNamedDestW
   #define pdfCreateNewPDF             pdfCreateNewPDFW
   #define pdfCreateOCG                pdfCreateOCGW
   #define pdfDecryptPDF               pdfDecryptPDFW
   #define pdfEncryptPDF               pdfEncryptPDFW
   #define pdfFileAttachAnnot          pdfFileAttachAnnotW
   #define pdfFileLink                 pdfFileLinkW
   #define pdfFindField                pdfFindFieldW
   #define pdfFindBookmark             pdfFindBookmarkW
   #define pdfFreeTextAnnot            pdfFreeTextAnnotW
   #define pdfGetFTextHeight           pdfGetFTextHeightW
   #define pdfGetFTextHeightEx         pdfGetFTextHeightExW
   #define pdfGetTextWidthEx           pdfGetTextWidthExW
   #define pdfGetTextWidth             pdfGetTextWidthW
   #define pdfHighlightAnnot           pdfHighlightAnnotW
   #define pdfLoadFDFData              pdfLoadFDFDataW
   #define pdfOpenImportFile           pdfOpenImportFileW
   #define pdfOpenOutputFile           pdfOpenOutputFileW
   #define pdfOpenTag                  pdfOpenTagW
   #define pdfReEncryptPDF             pdfReEncryptPDFW
   #define pdfReplaceFont              pdfReplaceFontW
   #define pdfReplaceFontEx            pdfReplaceFontExW
   #define pdfReplaceICCProfile        pdfReplaceICCProfileW
   #define pdfReplacePageTextEx        pdfReplacePageTextExW
   #define pdfSetAnnotString           pdfSetAnnotStringW
   #define pdfSetAnnotSubject          pdfSetAnnotSubjectW
   #define pdfSetCIDFont               pdfSetCIDFontW
   #define pdfSetCMapDir               pdfSetCMapDirW
   #define pdfSetDocInfo               pdfSetDocInfoW
   #define pdfSetDocInfoEx             pdfSetDocInfoExW
   #define pdfSetFieldMapName          pdfSetFieldMapNameW
   #define pdfSetFieldToolTip          pdfSetFieldToolTipW
   #define pdfSetFont                  pdfSetFontW
   #define pdfSetFontEx                pdfSetFontExW
   #define pdfSetTextFieldValue        pdfSetTextFieldValueW
   #define pdfSetTextFieldValueEx      pdfSetTextFieldValueExW
   #define pdfStampAnnot               pdfStampAnnotW
   #define pdfTextAnnot                pdfTextAnnotW
   #define pdfWebLink                  pdfWebLinkW
   #define pdfWriteAngleText           pdfWriteAngleTextW
   #define pdfWriteFText               pdfWriteFTextW
   #define pdfWriteFTextEx             pdfWriteFTextExW
   #define pdfWriteText                pdfWriteTextW
   #define pdfWriteTextEx              pdfWriteTextExW
   #define pdfWriteTextMatrix          pdfWriteTextMatrixW
   #define pdfWriteTextMatrixEx        pdfWriteTextMatrixExW
#else
   #define pdfAddBookmark              pdfAddBookmarkA
   #define pdfAddBookmarkEx            pdfAddBookmarkExA
   #define pdfAddContinueText          pdfAddContinueTextA
   #define pdfAddFileComment           pdfAddFileCommentA
   #define pdfAddFontSearchPath        pdfAddFontSearchPathA
   #define pdfAddJavaScript            pdfAddJavaScriptA
   #define pdfAddPageLabel             pdfAddPageLabelA
   #define pdfAddRenderingIntent       pdfAddRenderingIntentA
   #define pdfAddValToChoiceField      pdfAddValToChoiceFieldA
   #define pdfAttachFile               pdfAttachFileA
   #define pdfAttachFileEx             pdfAttachFileExA
   #define pdfChangeAnnotName          pdfChangeAnnotNameA
   #define pdfChangeBookmark           pdfChangeBookmarkA
   #define pdfChangeJavaScript         pdfChangeJavaScriptA
   #define pdfChangeJavaScriptAction   pdfChangeJavaScriptActionA
   #define pdfChangeJavaScriptName     pdfChangeJavaScriptNameA
   #define pdfCreateArticleThread      pdfCreateArticleThreadA
   #define pdfCreateButton             pdfCreateButtonA
   #define pdfCreateCollectionField    pdfCreateCollectionFieldA
   #define pdfCreateColItemString      pdfCreateColItemStringA
   #define pdfCreateGoToRActionEx      pdfCreateGoToRActionExA
   #define pdfCreateICCBasedColorSpace pdfCreateICCBasedColorSpaceA
   #define pdfCreateImage              pdfCreateImageA
   #define pdfCreateImpDataAction      pdfCreateImpDataActionA
   #define pdfCreateJSAction           pdfCreateJSActionA
   #define pdfCreateNamedDest          pdfCreateNamedDestA
   #define pdfCreateNewPDF             pdfCreateNewPDFA
   #define pdfCreateOCG                pdfCreateOCGA
   #define pdfDecryptPDF               pdfDecryptPDFA
   #define pdfEncryptPDF               pdfEncryptPDFA
   #define pdfFileAttachAnnot          pdfFileAttachAnnotA
   #define pdfFileLink                 pdfFileLinkA
   #define pdfFindField                pdfFindFieldA
   #define pdfFindBookmark             pdfFindBookmarkA
   #define pdfFreeTextAnnot            pdfFreeTextAnnotA
   #define pdfGetFTextHeight           pdfGetFTextHeightA
   #define pdfGetFTextHeightEx         pdfGetFTextHeightExA
   #define pdfGetTextWidthEx           pdfGetTextWidthExA
   #define pdfGetTextWidth             pdfGetTextWidthA
   #define pdfHighlightAnnot           pdfHighlightAnnotA
   #define pdfLoadFDFData              pdfLoadFDFDataA
   #define pdfOpenImportFile           pdfOpenImportFileA
   #define pdfOpenOutputFile           pdfOpenOutputFileA
   #define pdfOpenTag                  pdfOpenTagA
   #define pdfReEncryptPDF             pdfReEncryptPDFA
   #define pdfReplaceFont              pdfReplaceFontA
   #define pdfReplaceFontEx            pdfReplaceFontExA
   #define pdfReplaceICCProfile        pdfReplaceICCProfileA
   #define pdfReplacePageText          pdfReplacePageTextA
   #define pdfReplacePageTextEx        pdfReplacePageTextExA
   #define pdfSet3DAnnotScript         pdfSet3DAnnotScriptA
   #define pdfSetAnnotString           pdfSetAnnotStringA
   #define pdfSetAnnotSubject          pdfSetAnnotSubjectA
   #define pdfSetCIDFont               pdfSetCIDFontA
   #define pdfSetCMapDir               pdfSetCMapDirA
   #define pdfSetDocInfo               pdfSetDocInfoA
   #define pdfSetDocInfoEx             pdfSetDocInfoExA
   #define pdfSetFieldMapName          pdfSetFieldMapNameA
   #define pdfSetFieldToolTip          pdfSetFieldToolTipA
   #define pdfSetFont                  pdfSetFontA
   #define pdfSetFontEx                pdfSetFontExA
   #define pdfSetTextFieldValue        pdfSetTextFieldValueA
   #define pdfSetTextFieldValueEx      pdfSetTextFieldValueExA
   #define pdfStampAnnot               pdfStampAnnotA
   #define pdfTextAnnot                pdfTextAnnotA
   #define pdfWebLink                  pdfWebLinkA
   #define pdfWriteAngleText           pdfWriteAngleTextA
   #define pdfWriteFText               pdfWriteFTextA
   #define pdfWriteFTextEx             pdfWriteFTextExA
   #define pdfWriteText                pdfWriteTextA
   #define pdfWriteTextEx              pdfWriteTextExA
   #define pdfWriteTextMatrix          pdfWriteTextMatrixA
   #define pdfWriteTextMatrixEx        pdfWriteTextMatrixExA
#endif

#endif // end PDF_USE_CLASS

#ifdef __cplusplus
} // end namespace
#endif

#if defined(__BORLANDC__)
   #pragma option pop
#endif

#endif

