Attribute VB_Name = "DynaPDFInt"
Option Explicit

' Specific function which can be used to copy string values from DynaPDF pointers to VB strings
Public Declare Function pdfCopyMem Lib "dynapdf.dll" (ByVal Source As Long, ByVal Dest As String, ByVal sLen As Long) As Long
Public Declare Function pdfCopyMem2 Lib "dynapdf.dll" Alias "pdfCopyMem" (ByVal Source As Long, ByRef Dest As Any, ByVal sLen As Long) As Long
Public Declare Function pdfCopyMem3 Lib "dynapdf.dll" Alias "pdfCopyMem" (ByVal Source As Long, ByVal Dest As Long, ByVal sLen As Long) As Long
Public Declare Function pdfStrLenA Lib "dynapdf.dll" (ByVal AStr As Long) As Long
Public Declare Function pdfStrLenW Lib "dynapdf.dll" (ByVal AStr As Long) As Long

Private Declare Sub pdfCopyPtr Lib "dynapdf.dll" (ByVal Source As Long, ByRef Dest As Any)

' Error types
Public Const E_WARNING As Long = &H2000000
Public Const E_SYNTAX_ERROR As Long = &H4000000
Public Const E_VALUE_ERROR As Long = &H8000000
Public Const E_FONT_ERROR As Long = &H10000000
Public Const E_FATAL_ERROR As Long = &H20000000
Public Const E_FILE_ERROR As Long = &H40000000

' Specific error codes to determine whether the supplied to decrypt the input file password was wrong
Public Const ENEED_PWD As Long = -(&HB2 Or E_FILE_ERROR)
Public Const EWRONG_OPEN_PWD As Long = -(&HB3 Or E_FILE_ERROR)
Public Const EWRONG_OWNER_PWD As Long = -(&HB4 Or E_FILE_ERROR)
Public Const EWRONG_PWD As Long = -(&HB5 Or E_FILE_ERROR)

Public Const NO_COLOR As Long = &HFFFFFFF1 ' Transparent color used by annotaions and form fields

' Specific return values of the OnPageBreak callback function
Public Const NEW_ALIGN_LEFT As Long = 1
Public Const NEW_ALIGN_RIGHT As Long = 2
Public Const NEW_ALIGN_CENTER As Long = 3
Public Const NEW_ALIGN_JUSTIFY As Long = 4

' Use these masks to determine which viewer preference values are defined.
Public Const AV_NON_FULL_SRC_MASK As Long = &H5
Public Const AV_DIRECTION_MASK As Long = &H18
Public Const AV_VIEW_PRINT_MASK As Long = &H3E0

Enum TAnnotIcon
   aiComment
   aiHelp
   aiInsert
   aiKey
   aiNewParagraph
   aiNote
   aiParagraph
End Enum

Enum TAnnotType
   atCaret
   atCircle
   atFileLink
   atFreeText
   atHighlight
   atInk
   atLine
   atPageLink
   atPolygon
   atPolyLine
   atPopUp
   atSquare
   atSquiggly
   atStamp
   atStrikeOut
   atText
   atUnderline
   atWebLink
   atWidget
   at3D
   atSoundAnnot
   atFileAttach
   atRedact
   atWatermark
   atUnknown    ' Unknown annotation type
End Enum

Enum TBlendMode
   bmNotSet
   bmNormal
   bmColor
   bmColorBurn
   bmColorDodge
   bmDarken
   bmDifference
   bmExclusion
   bmHardLight
   bmHue
   bmLighten
   bmLuminosity
   bmMultiply
   bmOverlay
   bmSaturation
   bmScreen
   bmSoftLight
End Enum

Enum TBorderStyle
   bsSolid
   bsBevelled
   bsInset
   bsUnderline
   bsDashed
End Enum

Enum TCheckBoxState
   cbUnknown
   cbChecked
   cbUnChecked
End Enum

Enum TPDFColorSpace
   csDeviceRGB
   csDeviceCMYK
   csDeviceGray
End Enum

Enum TDestType
   dtXY_Zoom       ' three parameters (a, b, c) -> (X, Y, Zoom)
   dtFit           ' no parameters
   dtFitH_Top      ' one parameter    (a)
   dtFitV_Left     ' one parameter    (a)
   dtFit_Rect      ' four parameters  (a, b, c, d) -> (left, bottom, right, top)
   dtFitB          ' no parameter
   dtFitBH_Top     ' one parameter    (a)
   dtFitBV_Left    ' one parameter    (a)
End Enum

Enum TDrawMode
  dmNormal
  dmStroke
  dmFillStroke
  dmInvisible
  dmFillClip
  dmStrokeClip
  dmFillStrokeClip
  dmClipping
End Enum

Enum TDuplexMode
   dpmNone          ' Default
   dpmSimplex
   dpmFlipShortEdge
   dpmFlipLongEdge
End Enum

Enum TExtColorSpace
   esDeviceRGB   ' Device color space
   esDeviceCMYK  ' Device color space
   esDeviceGray  ' Device color space
   esCalGray     ' CIE-based color space
   esCalRGB      ' CIE-based color space
   esLab         ' CIE-based color space
   esICCBased    ' CIE-based color space -> contains an ICC profile
   esPattern     ' Special color space
   esIndexed     ' Special color space
   esSeparation  ' Special color space
   esDeviceN     ' Special color space
   esNChannel    ' Special color space
End Enum

Enum TFieldFlags
   ffReadOnly = &H1
   ffRequired = &H2
   ffNoExport = &H4

   ffInvisible = &H8
   ffHidden = &H10
   ffPrint = &H20
   ffNoZoom = &H40
   ffNoRotate = &H80
   ffNoView = &H100

   ffMultiline = &H1000         ' Text fields only
   ffPassword = &H2000          ' Text fields only
   ffNoToggleToOff = &H4000     ' Radio buttons
   ffRadioIsUnion = &H2000000   ' PDF-1.5 radio buttons only
   ffCommitOnSelCh = &H4000000  ' PDF-1.5 combo boxes, list boxes

   ffEdit = &H40000             ' Combo boxes only
   ffSorted = &H80000           ' Do not use this flag CreateComboBox or CreateListBox set this flag automatically
   ffFileSelect = &H100000      ' PDF 1.4 Text fields only
   ffMultiSelect = &H200000     ' PDF 1.4 List boxes only
   ffDoNotSpellCheck = &H400000 ' PDF 1.4 Text fields, combo boxes. If the field is a combo box, this flag is meaningful only if ffEdit is also set.
   ffDoNotScroll = &H800000     ' PDF 1.4 Text fields only
   ffComb = &H1000000
End Enum

Enum TFieldType
   ftButton
   ftCheckBox
   ftRadioBtn
   ftComboBox
   ftListBox
   ftText
   ftSignature
   ftGroup     ' this is not a real field type, it is just an array of fields
End Enum

Enum TFontBaseType
   fbtTrueType ' TrueType, TrueType Collections, or OpenType fonts with TrueType outlines
   fbtType1    ' Type1 font
   fbtOpenType ' OpenType font with Postscript outlines
   fbtStdFont  ' PDF Standard font
   fbtDisabled ' This value can be used to disable a specific font format. See SetFontSearchOrder() for further information.
End Enum

Enum TFontFileSubtype
   ffsType1C        ' CFF based Type1 font
   ffsCIDFontType0C ' CFF based Type1 CID font
   ffsOpenType      ' TrueType based OpenType font
   ffsOpenTypeC     ' CFF based OpenType font
   ffsCIDFontType2  ' TrueType based CID Font
   ffsReserved1
   ffsReserved2
   ffsReserved3
   ffsReserved4
End Enum

Enum TFontType
   ftMMType1
   ftTrueType
   ftType0    ' Check the font file type to determine the font sub type
   ftType1
   ftType3
End Enum

'The font search run works as follows:

'   - DynaPDF tries always to find the exact weight, if it cannot be found then a font with
'     the next smaller weight is selected (if available).
'   - Italic styles can always be emulated but it is not possible to emulate thinner weights or
'     regular styles with an italic font.
'   - If the specified weight is larger as the font weight the remaining weight will be emulated
'     if the difference to the requested weight is larger than 200.
'     With SetFontWeight() it is possible to control whether a missing weight should be emulated.
'     If FontWeight is smaller or equal to the requested font weight then emulation will be disabled.

'TFStyle is a bitmask that is defined as follows:

'   - Bits 0..7   // Style bits fsItalic, fsUnderlined, fsStriked
'   - Bits 8..19  // Width class -> Defined for future use.
'   - Bits 20..31 // Font Weight

'- A width class can be converted to a style constant by multiplying it with 256 (width shl 8).
'- A font weight can be converted to a style constant by multiplying it with 1048576 (weight shl 20).
'- Additional attributes can be added with a binary or operator (e.g. style or fsItalic).
'- Only one width class and one font weight can be set at time.

'- WidthFromStyle() extracts the width class.
'- WeightFromStyle() extracts the font weight.

'The following functions extract the width class or font weight from a style variable:

'   widthClass  = WidthFromStyle(style);
'   weightClass = WeightFromStyle(style);

Enum TFStyle
   fsNone = &H0                  ' Regular weight (400)
   fsItalic = &H1
   fsUnderlined = &H4
   fsStriked = &H8
   fsVerticalMode = &H10         ' Not considered at this time
   ' Width class
   fsUltraCondensed = &H100      ' 1
   fsExtraCondensed = &H200      ' 2
   fsCondensed = &H300           ' 3
   fsSemiCondensed = &H400       ' 4
   fsNormal = &H500              ' 5
   fsSemiExpanded = &H600        ' 6
   fsExpanded = &H700            ' 7
   fsExtraExpanded = &H800       ' 8
   fsUltraExpanded = &H900       ' 9
   ' Weight class
   fsThin = &H6400000            ' 100
   fsExtraLight = &HC800000      ' 200
   fsLight = &H12C00000          ' 300
   fsRegular = &H19000000        ' 400 -> Same as fsNone
   fsMedium = &H1F400000         ' 500
   fsDemiBold = &H25800000       ' 600
   fsBold = &H2BC00000           ' 700 -> The old constant 2 is still supported to preserve backward compatibility
   fsExtraBold = &H32000000      ' 800
   fsBlack = &H38400000          ' 900
   fsUltraBlack = &H3E800000     ' 1000
End Enum

Enum THashType
   htDetached ' CloseAndSignFileExt() returns the byte ranges of the finish PDF buffer to create a detached signature
   htSHA1     ' CloseAndSignFileExt() returns the SHA1 hash of the PDF file so that it can be signed
End Enum

Enum THighlightMode
  hmNone
  hmInvert
  hmOutline
  hmPush
  hmPushUpd       ' Update appereance stream on changes
End Enum

Enum TKeyLen
   kl40bit     ' RC4 Encryption -> Acrobat 3 or higher
   kl128bit    ' RC4 Encryption -> Acrobat 5 or higher
   kl128bitEx  ' RC4 Encryption -> Acrobat 6 or higher
   klAES128    ' AES Encryption -> Acrobat 7 or higher
   klAES256    ' AES Encryption -> Acrobat 9 or higher
End Enum

Enum TNewAlign
   naUnchanged = 0
   naLeft = 1
   naCenter = 2
   naRight = 3
   naJustify = 4
End Enum

Enum TPageLabelFormat
   plfDecimalArabic    ' 1,2,3,4...
   plfUppercaseRoman   ' I,II,III,IV...
   plfLowercaseRoman   ' i,ii,iii,iv...
   plfUppercaseLetters ' A,B,C,D...
   plfLowercaseLetters ' a,b,c,d...
   plfNone
End Enum

Enum TRenderingIntent
   riAbsoluteColorimetric
   riPerceptual
   riRelativeColorimetric
   riSaturation
   riNone
End Enum

Enum TRestrictions
   rsDenyNothing = 0
   rsDenyAll = 3900
   rsPrint = 4
   rsModify = 8
   rsCopyObj = 16
   rsAddObj = 32
   ' 128/256 bit encryption only -> these flags are ignored if 40 bit encryption is used
   rsFillInFormFields = 256
   rsExtractObj = 512
   rsAssemble = 1024
   rsPrintHighRes = 2048
   rsExlMetadata = 4096    ' PDF 1.5 Exclude metadata streams -> 128/256 bit encryption bit only.
   rsEmbFilesOnly = &H2000 ' PDF 1.6 Encrypt embedded files only -> Requires AES encryption.
End Enum

Enum TRubberStamp
  rsApproved
  rsAsIs
  rsConfidential
  rsDepartmental
  rsDraft
  rsExperimental
  rsExpired
  rsFinal
  rsForComment
  rsForPublicRelease
  rsNotApproved
  rsNotForPublicRelease
  rsSold
  rsTopSecred
  rsUserDefined
End Enum

Enum TTextAlign
   taLeft = 0
   taCenter = 1
   taRight = 2
   taJustify = 3
   taPlainText = &H10000000 ' If this flag is set alignment and command tags are interpreted as plain text.
                            ' See WriteFText() in the help file for further information.
End Enum

Public Type TCTM
   a As Double
   b As Double
   c As Double
   d As Double
   x As Double
   y As Double
End Type

Private Type TTextRecordA
   Advance As Single ' Negative values move the cursor to right, positive to left. The value is measured in text space!
   Text As Long      ' Source string (not null-terminated)
   Length As Long    ' Length in characters
End Type

Public Type TTextRecordW
   Advance As Single ' Negative values move the cursor to right, positive to left. The value is measured in text space!
   Text As String    ' Already translated Unicode string (not null-terminated)
   Width As Single   ' String width measured in text space
End Type

Public Type TPDFRect
   rLeft As Double
   rBottom As Double
   rRight As Double
   rTop As Double
End Type

Public Type TPDFAnnotation
   AnnotType As TAnnotType
   Deleted As Boolean
   BBox As TPDFRect
   BorderWidth As Double
   BorderColor As Long
   BorderStyle As TBorderStyle
   BackColor As Long
   Handle As Long
   Author As String
   Content As String
   Name As String
   Subject As String
   PageNum  As Long
   HighlightMode As THighlightMode
End Type

Public Type TPDFAnnotationEx
   AnnotType As TAnnotType
   Deleted As Boolean
   BBox As TPDFRect
   BorderWidth As Single
   BorderColor As Long
   BorderStyle As TBorderStyle
   BackColor As Long
   Handle As Long
   Author As String
   Content As String
   Name As String
   Subject As String
   PageNum As Long
   HighlightMode As THighlightMode
   ' Page link annotations only
   DestPage As Long
   DestPos As TPDFRect
   DestType As TDestType
   DestFile As String    ' File link or web link annotations
   Icon As Long          ' The Icon type depends on the annotation type. If the annotation type is atText then the Icon
                         ' is of type TAnnotIcon. If the annotation type is atFileAttach then it is of type
                         ' TFileAttachIcon. If the annotation type is atStamp then the Icon is the stamp type (TRubberStamp).
                         ' For any other annotation type this value is not set (-1).
   StampName As String   ' Set only, if Icon == rsUserDefined
   AnnotFlags As Long    ' See TAnnotFlags for available flags
   CreateDate As String  ' Creation Date -> Optional
   ModDate As String     ' Modification Date -> Optional
   Grouped As Boolean    ' (Reply type) Meaningful only if Parent != -1 and Type != atPopUp. If true,
                         ' the annotation is part of an annotation group. Properties like Content, CreateDate,
                         ' ModDate, BackColor, Subject, and Open must be taken from the parent annotation.
   Open As Boolean       ' Meaningful only for annotations which have a corresponding PopUp annotation.
   Parent As Long        ' Parent annotation handle of a PopUp Annotation or the parent annotation if
                         ' this annotation represents a state of a base annotation. In this case,
                         ' the annotation type is always atText and only the following members should
                         ' be considered:
                         '    State      // The current state
                         '    StateModel // Marked, Review, and so on
                         '    CreateDate // Creation Date
                         '    ModDate    // Modification Date
                         '    Author     // The user who has set the state
                         '    Content    // Not displayed in Adobe's Acrobat...
                         '    Subject    // Not displayed in Adobe's Acrobat...
                         ' The PopUp annotation of a text annotation which represent an Annotation State
                         ' must be ignored.
   PopUp As Long         ' Handle of the corresponding PopUp annotation if any.
   State As String       ' The state of the annotation.
   StateModel As String  ' The state model (Marked, Review, and so on).
   EmbeddedFile As Long  ' FileAttach annotations only. A handle of an embedded file -> GetEmbeddedFile().
   Subtype As String     ' Set only, if Type = atUnknownAnnot
End Type

Public Type TBookmark
   Color As Long
   DestPage As Long
   DestPos As TPDFRect
   DestType As TDestType
   DoOpen As Boolean
   Parent As Long
   Style As Long
   Title As String
End Type

Public Type TPDFChoiceValue
   ExpValue As String
   Value As String
   Selected As Boolean
End Type

Public Type TPDFCMap
   StructSize As Long       ' Must be set to sizeof(TPDFCMap) before calling GetCMap()!
   BaseCMap As String       ' If set, this base cmap is required when loading the cmap.
   CIDCount As Long         ' 0 if not set.
   CMapName As String       ' The CMap name.
   CMapType As Long         ' Should be 1!
   CMapVersion As Single    ' The CMap version.
   DSCBaseCMap As String    ' DSC comment.
   DSCCMapVersion As Single ' DSC comment.
   DSCResName As String     ' DSC comment. If the CMap uses an Identity mapping this string should be set to Identity.
   DSCTitle As String       ' DSC comment -> DSC CMap name + Registry + Ordering + Supplement, e.g. "GB-EUC-H Adobe GB1 0"
   FileName As String       ' The file name and CMap name should be identical!
   FilePath As String       ' Path to the CMap file.
   Ordering As String       ' CIDSystemInfo -> The Character Collection, e.g. Japan1.
   Registry As String       ' CIDSystemInfo -> The registrant of the Character Collection is usually Adobe.
   Supplement As Long       ' CIDSystemInfo -> The Supplement number should be supported in the used PDF Version.
   WritingMode As Long      ' 0 == Horizontal, 1 == Vertical
End Type

Public Type TPDFColorSpaceObj
   ColorSpaceType As TExtColorSpace
   Alternate As TExtColorSpace      ' Alternate color space or base space of an Indexed or Pattern color space.
   IAlternate As Long               ' Only set if the color space contains an alternate or base color space -> GetColorSpaceObjEx().
   Buffer As Long                   ' Contains either an ICC profile or the color table of an Indexed color space.
   BufSize As Long                  ' Buffer length in bytes.
   BlackPoint() As Single           ' CIE blackpoint. If set, the array contains exactly 3 values.
   WhitePoint() As Single           ' CIE whitepoint. If set, the array contains exactly 3 values.
   Gamma() As Single                ' If set, one value per component.
   Range() As Single                ' The allowed range of input values (min/max for each component).
   Matrix() As Single               ' XYZ matrix. If set, the array contains exactly 9 values.
   NumInComponents As Long          ' Number of input components.
   NumOutComponents As Long         ' Number of output components.
   NumColors As Long                ' HiVal + 1 as specified in the color space. Indexed color space only.
   Colorants(31) As String          ' Colorant names (Separation, DeviceN, and NChannel only).
   ColorantsCount As Long           ' The number of colorants in the array.
   Metadata As Long                 ' Optional XMP metadata stream -> ICCBased only.
   MetadataSize As Long             ' Metadata length in bytes.
   IFunction As Long                ' Pointer to function object -> Separation, DeviceN, and NChannel only.
   IAttributes As Long              ' Optional attributes of DeviceN or NChannel color spaces -> GetNChannelAttributes().
   IColorSpaceObj As Long           ' Pointer of the corresponding color space object
End Type

Public Type TDeviceNAttributes
   IProcessColorSpace As Long    ' Pointer to process color space or NULL -> GetColorSpaceEx().
   ProcessColorants(7) As String ' Does a process color space with more than 8 components exist? 6 components
                                 ' is the maximum so far I know. However, 8 components should be large enough for
                                 ' the next years...
   ProcessColorantsCount As Long ' Number of process colorants in the array or zero if not set.
   Separations(31) As Long       ' Optional pointers to Separation color spaces -> GetColorSpaceEx().
   SeparationsCount As Long      ' Number of color spaces in the array.
   IMixingHints As Long          ' Optional mixing hints. There is no API function at this time to access mixing hints.
End Type

' It is not possible to set all available graphic state parameters with DynaPDF, such as black generation functions,
' halftones and so on. The structure will be extended when further objects are supported.
Public Type TPDFExtGState
   AutoStrokeAdjust As Long            ' PDF_MAX_INT if not set
   BlendMode As TBlendMode             ' Default bmNotSet
   FlatnessTol As Single               ' -1.0 if not set
   OverPrintFill As Long               ' PDF_MAX_INT if not set
   OverPrintStroke As Long             ' PDF_MAX_INT if not set
   OverPrintMode As Long               ' PDF_MAX_INT if not set
   RenderingIntent As TRenderingIntent ' riNone if not set
   SmoothnessTol As Single             ' -1.0 if not set
   FillAlpha As Single                 ' -1.0 if not set
   StrokeAlpha As Single               ' -1.0 if not set
   AlphaIsShape As Long                ' PDF_MAX_INT if not set
   TextKnockout As Long                ' PDF_MAX_INT if not set
   Reserved1 As Long
   Reserved2 As Long
   Reserved3 As Long
   Reserved4 As Long
   Reserved5 As Long
   Reserved6 As Long
   Reserved7 As Long
   Reserved8 As Long
   Reserved9 As Long
End Type

Public Type TPDFField
   BBox As TPDFRect
   FieldType As TFieldType
   Deleted As Boolean
   Handle As Long
   FieldName As String
   BackCS As TPDFColorSpace
   TextCS As TPDFColorSpace
   BackColor As Long
   BorderColor As Long
   TextColor As Long
   Checked As Boolean
   Parent As Long
   KidCount As Long
   FontName As String
   FontSize As Double
   Value As String
   ToolTip As String
End Type

Public Type TPDFFieldEx
   Deleted As Boolean              ' If true, the field was marked as deleted by DeleteField()
   BBox As TPDFRect                ' Bounding box of the field in bottom-up coordinates
   FieldType As TFieldType         ' Field type
   GroupType As TFieldType         ' If GroupType != FieldType the field is a terminal field of a field group
   Handle As Long                  ' Field handle
   BackColor As Long               ' Background color
   BackColorSP As TExtColorSpace   ' Color space of the background color
   BorderColor As Long             ' Border color
   BorderColorSP As TExtColorSpace ' Color space of the border color
   BorderStyle As TBorderStyle     ' Border style
   BorderWidth As Single           ' Border width
   CharSpacing As Single           ' Text fields only
   Checked As Boolean              ' Check boxes only
   CheckBoxChar As Long            ' ZapfDingbats character that is used to display the on state
   DefState As TCheckBoxState      ' Check boxes only
   DefValue As String              ' Optional default value
   IEditFont As Long               ' Pointer to default editing font
   EditFont As String              ' Postscript name of the editing font
   ExpValCount As Long             ' Combo and list boxes only. The values can be accessed with GetFieldExpValueEx()
   ExpValue As String              ' Check boxes only
   FieldFlags As TFieldFlags       ' Field flags
   IFieldFont As Long              ' Pointer to the font that is used by the field
   FieldFont As String             ' Postscript name of the font
   FontSize As Double              ' Font size. 0.0 means auto font size
   FieldName As String             ' Note that children of a field group or radio button have no name
   HighlightMode As THighlightMode ' Highlight mode
   IsCalcField As Boolean          ' If true, the OnCalc event of the field is connected with a JavaScript action
   MapName As String               ' Optional unique mapping name of the field
   MaxLen As Long                  ' Text fields only -> zero means not restricted
   Kids() As Long                  ' Array of child fields -> GetFieldEx2()
   KidCount As Long                ' Number of pointers in the array
   Parent As Long                  ' Pointer to parent field or NULL
   PageNum As Long                 ' Page on which the field is used or -1
   Rotate As Long                  ' Rotation angle in degrees
   TextAlign As TTextAlign         ' Text fields only
   TextColor As Long               ' Text color
   TextColorSP As TExtColorSpace   ' Color space of the field's text
   TextScaling As Single           ' Text fields only
   ToolTip As String               ' Optional tool tip
   UniqueName As String            ' Optional unique name (NM key)
   Value As String                 ' Field value
   WordSpacing As Single           ' Text fields only
End Type

Public Type TPDFFileSpec
   Buffer() As Byte       ' Buffer of an embedded file
   BufSize As Long        ' Buffer size in bytes.
   Compressed As Boolean  ' Should be false if Decompress was true in the GetEmbeddedFile() call, otherwise usually true.
                          ' DynaPDF decompresses Flate encoded streams only. Other filters can occur but this is very unusual.
   ColItem As Long        ' If != NULL the embedded file contains a collection item with user defined data. This entry
                          ' can occur in PDF Collections only (PDF 1.7). See "PDF Collections" in the help file for further
                          ' information.
   Name As String         ' Name of the file specification in the name tree. This value is always present.
   FileName As String     ' File name as 7 bit ASCII string.
   IsURL As Boolean       ' If true, FileName contains a URL.
   UF As String           ' PDF 1.7. Same as FileName but Unicode is allowed.
   Desc As String         ' Description
   FileSize As Long       ' Size of the decompressed stream or zero if not known. Note: this is either the Size key of
                          ' the Params dictionary if present or the DL key in the file stream. Whether this value is
                          ' correct depends on the file creator! The parameter is definitely correct if the file was
                          ' decompressed.
   MIMEType As String     ' MIME media type name as defined in Internet RFC 2046.
   CreateDate As String   ' Creation date as string. See help file "The standard date format".
   ModDate As String      ' Modification date as string. See help file "The standard date format".
   CheckSum() As Byte     ' 16 byte MD5 digest. Note that this is a binary string. It is exactly 16 bytes long if set!
End Type

Public Type TPDFFontObj
   Ascent As Single      ' Ascent
   BaseFont As String    ' PostScript Name or Family Name
   CapHeight As Single   ' Cap height
   Descent As Single     ' Descent
   Encoding As String    ' Unicode mapping 0..255 -> not set if a CID font is selected
   FirstChar As Long     ' First char
   Flags As Long         ' Font flags -> font descriptor
   FontFamily As String  ' Optional Font Family (Family Name)
   FontName As String    ' Font name -> font descriptor
   FontType As TFontType ' If ftType0 the font is a CID font. The Encoding is not set in this case.
   ItalicAngle As Single ' Italic angle
   LastChar As Long      ' Last char
   spaceWidth As Single  ' Space width in font units. A default value is set if the font contains no space character.
   Widths() As Single    ' Glyph widths -> 0..WidthsCount -1
   XHeight As Single     ' x-height
   DefWidth As Single    ' Default character widths -> CID fonts only
   FontFile As Long      ' Font file buffer -> only imported fonts are returned.
   Length1 As Long       ' Length of the clear text portion of the Type1 font, or the length of the entire font program if FontType != ffType1.
   Length2 As Long       ' Length of the encrypted portion of the Type1 font program (Type1 fonts only).
   Length3 As Long       ' Length of the fixed-content portion of the Type1 font program or zero if not present.
   FontFileType As TFontFileSubtype  ' See description in the help file for further information.
End Type

Public Type TPDFNamedDest
   Name As String
   DestPage As Long
   DestPos As TPDFRect
   DestType As TDestType
End Type

Public Type TPDFPageLabel
   StartRange As Long         ' Number of the first page in the range. If no further label follows, the last
                              ' page in the range is pdfGetPageCount(). The first page is denoted by 1.
   Format As TPageLabelFormat ' Number format to be used.
   FirstPageNum As Long       ' First page number to be displayed in the page label. Subsequent pages are
                              ' numbered sequentially from this value.
   Prefix As String           ' Optional prefix
End Type

Public Enum TPrintScaling
   psAppDefault ' Default
   psNone
End Enum

Public Type TPDFPrintSettings
   DuplexMode As TDuplexMode
   NumCopies As Long             ' -1 means not set. Values larger than 5 are ignored in viewer applications.
   PickTrayByPDFSize As Long     ' -1 means not set. 0 == false, 1 == true.
   PrintRanges() As Long         ' If set, the array contains PrintRangesCount * 2 values. Each pair consists
                                 ' of the first and last pages in the sub-range. The first page in the PDF file
                                 ' is denoted by 0.
   PrintScaling As TPrintScaling ' dpmNone means not set.
End Type

Public Type TPDFSigParms
   PKCS7ObjLen As Integer    ' The maximum length of the signed PKCS#7 object
   HashType As THashType     ' If set to htDetached, the bytes ranges of the PDF file will be returned.
   Range1() As Byte          ' Out -> Contains either the hash or the first byte range to create a detached signature
   Range2() As Byte          ' Out -> Set only if HashType == htDetached
   ContactInfo As String     ' Optional, e.g. an email address_
   Location As String        ' Optional location of the signer
   Reason As String          ' Optional reason why the file was signed
   Signer As String          ' Optional, the issuer of the certificate takes precedence
   Encrypt As Boolean        ' If true, the file will be encrypted
   ' These members will be ignored if Encrypt is set to false
   OpenPwd As String         ' Open password
   OwnerPwd As String        ' Owner password to change the security settings
   KeyLen As TKeyLen         ' Key length to be used to encrypt the file
   Restrict As TRestrictions ' What should be restricted?
End Type

Public Type TPDFStack
   ctm As TCTM                 ' Pre-multiplied global transformation matrix
   tm As TCTM                  ' Pre-multiplied text transformation matrix
   x As Double                 ' unused -> always 0.0
   y As Double                 ' unused -> always 0.0
   FontSize As Double          ' Font size
   CharSP As Double            ' Character spacing
   WordSP As Double            ' Word spacing
   HScale As Double            ' Horizontal text scaling
   TextRise As Double          ' Text rise -> always 0.0 because it is already included in the text transformation matrix
   Leading As Double           ' Leading
   LineWidth As Double         ' Line width
   DrawMode As TDrawMode       ' Text draw mode
   FillCS As TPDFColorSpace    ' Fill color space
   StrokeCS As TPDFColorSpace  ' Stroke color space
   FillColor As Long           ' Fill color
   StrokeColor As Long         ' Stroke color
   CIDFont As Boolean          ' If true, ReplacePageText() can only be used to delete a string.
   Text As String              ' Raw text without kerning space
   RawKern() As TTextRecordA   ' Raw kerning array
   Kerning() As TTextRecordW   ' Already translated Unicode kerning array
   TextWidth As Single         ' Measured in text space
   spaceWidth As Single        ' Measured in text space
   IFont As Long               ' Font object used to print the string -> fntGetFont() can be used to return the font properties.
   Embedded As Boolean         ' If true the font is embedded
   ConvColors As Boolean       ' If set to true (default), all colors are converted to the specified destination color space
   DestSpace As TPDFColorSpace ' Destination color space -> default == csDeviceRGB
   ' This member can be used in combination with ReplacePageText() to preserve a number
   ' of kerning records from deletion. All records above this value will be deleted.
   ' Take a look into the VB .Net demo examples/util/pdf_edit_text.vb to determine how this member
   ' can be used.
   DeleteKerningAt As Long
   FontFlags As Long           ' PDF font flags
End Type

Public Type TRectL
   rLeft As Long
   rTop As Long
   rRight As Long
   rBottom As Long
End Type

' The following callback functions are used to raise events. Set a break point into the callback to see how the strings
' must be copied into VB strings.

' FamilyName is a Unicode string, the PostScriptName is an ANSI string!
Public Function pdf_EnumFontsProc(ByVal Data As Long, ByVal FamilyName As Long, ByVal PostScriptName As Long, ByVal Style As Long) As Long
   On Error Resume Next
   Dim clsPDF As IPDFCallBack
   Dim p As Object
   Dim DoBreak As Boolean
   ' Get the instance pointer back
   pdfCopyPtr Data, p
   ' Copy the pointer
   Set clsPDF = p
   ' Set the pointer back to nothing
   pdfCopyPtr 0&, p

   DoBreak = False
   If clsPDF Is Nothing Then
      pdf_EnumFontsProc = 1
   Else
      clsPDF.EnumFonts FamilyName, PostScriptName, Style, DoBreak
      pdf_EnumFontsProc = (DoBreak = True)
   End If
End Function

Public Function pdf_EnumFontsProc2(ByVal Data As Long, ByVal PDFFont As Long, ByVal FontType As TFontType, ByVal BaseFont As Long, ByVal FontName As Long, ByVal Embedded As Long, ByVal IsFormFont As Long, ByVal Flags As Long) As Long
   On Error Resume Next
   Dim clsPDF As IPDFCallBack
   Dim p As Object
   Dim DoBreak As Boolean
   ' Get the instance pointer back
   pdfCopyPtr Data, p
   ' Copy the pointer
   Set clsPDF = p
   ' Set the pointer back to nothing
   pdfCopyPtr 0&, p

   DoBreak = False
   If clsPDF Is Nothing Then
      pdf_EnumFontsProc2 = 1
   Else
      clsPDF.EnumDocFonts PDFFont, FontType, BaseFont, FontName, Embedded, IsFormFont, Flags, DoBreak
      pdf_EnumFontsProc2 = (DoBreak = True)
   End If
End Function

Public Function pdf_EnumFontsProcEx(ByVal Data As Long, ByVal FamilyName As Long, ByVal PostScriptName As Long, ByVal Style As Long, ByVal BaseType As Long, ByVal Embeddable As Long, ByVal FilePath As Long) As Long
   On Error Resume Next
   Dim clsPDF As IPDFCallBack
   Dim p As Object
   Dim DoBreak As Boolean
   ' Get the instance pointer back
   pdfCopyPtr Data, p
   ' Copy the pointer
   Set clsPDF = p
   ' Set the pointer back to nothing
   pdfCopyPtr 0&, p

   DoBreak = False
   If clsPDF Is Nothing Then
      pdf_EnumFontsProcEx = 1
   Else
      clsPDF.EnumFontsEx FamilyName, PostScriptName, Style, BaseType, Embeddable <> 0, FilePath, DoBreak
      pdf_EnumFontsProcEx = (DoBreak = True)
   End If
End Function

' Notice: Data is the user defined pointer that was passed to pdfSetOnErrorProc(). You can use pdfCopyMem() to copy the
' data behind the pointer to any user defined data structure.
Public Function pdf_ErrorProc(ByVal Data As Long, ByVal ErrCode As Long, ByVal ErrMessage As Long, ByVal ErrType As Long) As Long
   On Error Resume Next
   Dim clsPDF As IPDFCallBack
   Dim p As Object
   Dim DoBreak As Boolean
   ' Get the instance pointer back
   pdfCopyPtr Data, p
   ' Copy the pointer
   Set clsPDF = p
   ' Set the pointer back to nothing
   pdfCopyPtr 0&, p
   
   DoBreak = False
   If clsPDF Is Nothing Then
      'Default to ABORT Processing
      pdf_ErrorProc = 1
   Else
      clsPDF.ErrorProc ErrCode, ErrMessage, ErrType, DoBreak
      pdf_ErrorProc = DoBreak
   End If
End Function

' ------------------------------------- Callback function for WriteFText ----------------------------------------

' The TOnPageBreakProc (see below) function is called by WriteFText when the rectangle was filled with text,
' or when a manual page break was inserted to the text and more text is available for output.
' You must set the text rectangle with SetTextRect to continue the text on a new position or simply call
' EndPage() and then Append() or EditPage() to continue the text on an arbitrary new page. You can make some
' changes inside the callback function such as setting another font, font color or other thinks.

' When the callback function returns a value greater or equal zero, processing continues on the new
' position or page that was set inside the callback function.

' The text alignment can also be changed with the following return values:

'Const NEW_ALIGN_LEFT = 1
'Const NEW_ALIGN_RIGHT = 2
'Const NEW_ALIGN_CENTER = 3
'Const NEW_ALIGN_JUSTIFY = 4

' When the return value of the callback function is one of the values listed above, then the alignment will
' be changed before the remaining text will be processed. A return value of 0 determines that processing should
' be continued without changing the alignment.

' A negative return value breaks the processing and WriteFText returns emmidiatly with a warning. Note that
' processing stops entirely when warnings should be handled as fatal errors (see SetErrorMode in the help
' file for further information).

' NOTE: The coordinate system is always set to bottom up when the callback function is called, independent
' of the coordinate system that was set before WriteFText was called. WriteFText restores the coordinate system
' and all other temporarily changed variables when the function is leaved. The last text position is
' also returned in bottom up coordinates. LastPosY is the position of the last baseline.

' You can change the coordinate system inside the callback function when needed, DynaPDF checks the state
' before processing continues.

Public Function pdf_OnPageBreakProc(ByVal Data As Long, ByVal LastPosX As Double, ByVal LastPosY As Double, ByVal PageBreak As Long) As Long
   Dim clsPDF As IPDFCallBack
   Dim p As Object
   Dim DoBreak As Boolean
   Dim NewAlign As TNewAlign
   ' Get the instance pointer back
   pdfCopyPtr Data, p
   ' Copy the pointer
   Set clsPDF = p
   ' Set the pointer back to nothing
   pdfCopyPtr 0&, p

   DoBreak = False
   NewAlign = naUnchanged
   If clsPDF Is Nothing Then
      pdf_OnPageBreakProc = -1
   Else
      clsPDF.OnPageBreakProc LastPosX, LastPosY, PageBreak, NewAlign, DoBreak
      If Not DoBreak Then
         pdf_OnPageBreakProc = NewAlign
      Else
         pdf_OnPageBreakProc = -1
      End If
   End If
End Function

' -------------------------- Example callback functions to control a progress bar --------------------------------

' pdf_InitProgress is called before pdf_Progress is called the first time
Public Sub pdf_InitProgress(ByVal Data As Long, ByVal ProgType As Long, ByVal MaxCount As Long)
    Dim clsPDF As IPDFCallBack
    Dim p As Object

    ' Get the instance pointer back
    pdfCopyPtr Data, p
    ' Copy the pointer
    Set clsPDF = p
    ' Set the pointer back to nothing
    pdfCopyPtr 0&, p

    If Not clsPDF Is Nothing Then
        clsPDF.InitProgress ProgType, MaxCount
    End If
End Sub

Public Function pdf_Progress(ByVal Data As Long, ByVal ActivePage As Long) As Long
   Dim clsPDF As IPDFCallBack
   Dim p As Object
   Dim DoBreak As Boolean
   ' Get the instance pointer back
   pdfCopyPtr Data, p
   ' Copy the pointer
   Set clsPDF = p
   ' Set the pointer back to nothing
   pdfCopyPtr 0&, p

   DoBreak = False
   If clsPDF Is Nothing Then
      pdf_Progress = 1
   Else
      clsPDF.Progress ActivePage, DoBreak
      pdf_Progress = (DoBreak = True)
   End If
End Function
