unit dynapdf;
{$LONGSTRINGS ON}
// Treat enums as 32 bit integer!!!
{$Z4}
interface

uses Windows, Classes;

{
                   DELPHI INTERFACE FILE FOR DYNAPDF VERSION 2.5.4.
   Copyright (c) Jens Boschulte, DynaForms GmbH, www.dynaforms.com.
   Please report errors and other potential problems to support@dynaforms.com.

   --------------------------------- Thank you! -------------------------------------

   Creation date: February 9, 2011
}

{ -------------------------------------- Error types, specific error values ------------------------------------- }

{$ifdef VER120} // Delphi 4
   type PCardinal = ^Cardinal;
   type PPchar    = ^PAnsiChar;
{$else}
   {$ifdef VER130} // Delphi 5
      type PCardinal = ^Cardinal;
      type PPchar    = ^PAnsiChar;
   {$endif}
{$endif}

type TErrMode = Integer;
const emIgnoreAll    = $00000000;
const emSyntaxError  = $00000001;
const emValueError   = $00000002;
const emWarning      = $00000004;
const emFileError    = $00000008;
const emFontError    = $00000010;
const emAllErrors    = $0000FFFF;
const emNoFuncNames  = $10000000; // Do not print function names in error messages

const E_WARNING      = $02000000;
const E_SYNTAX_ERROR = $04000000;
const E_VALUE_ERROR  = $08000000;
const E_FONT_ERROR   = $10000000;
const E_FATAL_ERROR  = $20000000;
const E_FILE_ERROR   = $40000000;

const ENEED_PWD        = -($000000B2 or E_FILE_ERROR);
const EWRONG_OPEN_PWD  = -($000000B3 or E_FILE_ERROR);
const EWRONG_OWNER_PWD = -($000000B4 or E_FILE_ERROR);
const EWRONG_PWD       = -($000000B5 or E_FILE_ERROR);

const PDF_MAX_INT      = $7FFFFFFF;

{ ------------------------------------------------- Default types ----------------------------------------------- }

const NO_COLOR: Cardinal = $FFFFFFF1; // Transparent color used by annotations and form fields

type T3DActivationType =
(
   at3D_AppDefault,
   at3D_PageOpen,    // The annotaiton should be activated when the page is opened.
   at3D_PageVisible, // The annotaiton should be activated when the page becomes visible.
   at3D_Explicit     // The annotation should remain inactive until explicitely activated by a script or action (default).
);

type T3DDeActivateType =
(
   dt3D_AppDefault,
   dt3D_PageClosed,    // The annotaiton should be deactivated as soon as the page is closed.
   dt3D_PageInvisible, // The annotaiton should be deactivated as soon as the page becomes invisible (default).
   dt3D_Explicit       // The annotation should remain active until explicitely deactivated by a script or action.
);

type T3DDeActInstance =
(
   di3D_AppDefault,
   di3D_UnInstantiated, // The annotation will be uninstantiated (default)
   di3D_Instantiated,   // The annotation is left instantiated
   di3D_Live            // If the 3D artwork contains an animation then it will stay live
);

type T3DInstanceType =
(
   it3D_AppDefault,
   it3D_Instantiated, // The annotation will be instantiated but animations are disabled.
   it3D_Live          // The annotation will be instantiated and animations are enabled (default).
);

type T3DLightingSheme =
(
   lsArtwork,
   lsBlue,
   lsCAD,
   lsCube,
   lsDay,
   lsHard,
   lsHeadlamp,
   lsNight,
   lsNoLights,
   lsPrimary,
   lsRed,
   lsWhite,
   lsNotSet
);

type T3DNamedAction =
(
   naDefault,
   naFirst,
   naLast,
   naNext,
   naPrevious
);

type T3DProjType =
(
   pt3DOrthographic,
   pt3DPerspective
);

type T3DRenderingMode =
(
   rmBoundingBox,
   rmHiddenWireframe,
   rmIllustration,
   rmShadedIllustration,
   rmShadedVertices,
   rmShadedWireframe,
   rmSolid,
   rmSolidOutline,
   rmSolidWireframe,
   rmTransparent,
   rmTranspBBox,
   rmTranspBBoxOutline,
   rmTranspWireframe,
   rmVertices,
   rmWireframe,
   rmNotSet
);

type T3DScaleType =
(
   st3DValue,
   st3DWidth,
   st3DHeight,
   st3DMin,
   st3DMax
);

type TAnnotFlags = Cardinal;
const afNone       = $00000000;
const afInvisible  = $00000001;
const afHidden     = $00000002;
const afPrint      = $00000004;
const afNoZoom     = $00000008;
const afNoRotate   = $00000010;
const afNoView     = $00000020;
const afReadOnly   = $00000040;

{
   By default all annotations which have an appearance stream and which have the print flag set are flattened.
   All annotations are deleted when the function returns with the exception of file attachment annotations.
   If you want to flatten the view state then set the flag affUseViewState.
}
type TAnnotFlattenFlags = Cardinal;
const affNone         = $00000000; // Printable annotations independent of the type
const affUseViewState = $00000001; // If set, annotations which are visible in a viewer become flattened.
const affMarkupAnnots = $00000002; // If set, markup annotations are flattened only. Link, Sound, or FileAttach
                                   // annotations are no markup annotations. These types will be left intact.

type TAnnotIcon =
(
  aiComment,
  aiHelp,
  aiInsert,
  aiKey,
  aiNewParagraph,
  aiNote,
  aiParagraph
);

type TActionType =
(
   atGoTo,
   atGoToR,
   atHide,
   atImportData,
   atJavaScript,
   atLaunch,
   atMovie,
   atNamed,
   atRendition,
   atReset,
   atSetOCGState,
   atSound,
   atSubmit,
   atThread,
   atTransition,
   atURI,
   atGoTo3DView
);

type TAnnotString =
(
   asAuthor,
   asContent,
   asName,
   asSubject
);

type TAnnotType =
(
   atCaret,
   atCircle,
   atFileLink,
   atFreeText,
   atHighlight,
   atInk,
   atLine,
   atPageLink,
   atPolygon,
   atPolyLine,
   atPopUp,
   atSquare,
   atSquiggly,
   atStamp,
   atStrikeOut,
   atText,
   atUnderline,
   atWebLink,
   atWidget,
   at3D,
   atSoundAnnot,
   atFileAttach,
   atRedact,
   atWatermark,
   atUnknown    // Unknown annotation type
);

type TBlendMode =
(
   bmNotSet,
   bmNormal,
   bmColor,
   bmColorBurn,
   bmColorDodge,
   bmDarken,
   bmDifference,
   bmExclusion,
   bmHardLight,
   bmHue,
   bmLighten,
   bmLuminosity,
   bmMultiply,
   bmOverlay,
   bmSaturation,
   bmScreen,
   bmSoftLight
);

type TBorderStyle =
(
   bsSolid,
   bsBevelled,
   bsInset,
   bsUnderline,
   bsDashed
);

type TButtonState =
(
   bsUp,
   bsDown,
   bsRollOver
);

type TCheckBoxChar =
(
   ccCheck,
   ccCircle,
   ccCross1,
   ccCross2,
   ccCross3,
   ccCross4,
   ccDiamond,
   ccSquare,
   ccStar
);

type TCheckOptions = Cardinal;
const coDefault                  = $00000FFF;
const coEmbedSubsets             = $00000001;
const coDeleteTransferFuncs      = $00000002;
const coDeleteMultiMediaContents = $00000004;
const coDeleteActionsAndScripts  = $00000008;
const coDeleteInvRenderingIntent = $00000010;
const coFlattenFormFields        = $00000020;
const coReplaceV4ICCProfiles     = $00000040;
const coDeleteEmbeddedFiles      = $00000080;
const coDeleteOPIComments        = $00000100;
const coDeleteSignatures         = $00000200;
const coReplCCITTFaxWithFlate    = $00010000;
const coNoFontEmbedding          = $10000000;


type TCheckBoxState =
(
   cbUnknown,
   cbChecked,
   cbUnChecked
);

type TClippingMode =
(
  cmEvenOdd,
  cmWinding,
  cmAllEvenOdd,
  cmAllWinding
);

type TCodepage =
(
   cp1250,
   cp1251,
   cp1252,
   cp1253,
   cp1254,
   cp1255,
   cp1256,
   cp1257,
   cp1258,
   cp8859_2,
   cp8859_3,
   cp8859_4,
   cp8859_5,
   cp8859_6,
   cp8859_7,
   cp8859_8,
   cp8859_9,
   cp8859_10,
   cp8859_13,
   cp8859_14,
   cp8859_15,
   cp8859_16,
   cpSymbol,
   cp437,
   cp737,
   cp775,
   cp850,
   cp852,
   cp855,
   cp857,
   cp860,
   cp861,
   cp862,
   cp863,
   cp864,
   cp865,
   cp866,
   cp869,
   cp874,
   cpUnicode,
   cpCJK_Big5_Uni,    // Big5 plus HKSCS extension.
   cpCJK_EUC_JP_Uni,  // EUC-JP
   cpCJK_EUC_KR_Uni,  // EUC-KR
   cpCJK_EUC_TW_Uni,  // CNS-11643-1992 (Planes 1-15).
   cpCJK_GBK_Uni,     // GBK is the Microsoft code page 936 (GB2312, EUC-CN plus GBK extension).
   cpCJK_GB12345_Uni, // GB-12345-1990 (Traditional Chinese form of GB-2312).
   cpCJK_HZ_Uni,      // Mixed ASCII / GB-2312 encoding
   cpCJK_2022_CN_Uni, // ISO-2022-CN-EXT (GB-2312 plus ISO-11643 Planes 1-7).
   cpCJK_2022_JP_Uni, // ISO-2022-JP
   cpCJK_2022_KR_Uni, // ISO-2022-KR
   cpCJK_646_CN_Uni,  // ISO-646-CN (GB-1988-80)
   cpCJK_646_JP_Uni,  // ISO-646-JP (JIS_C6220-1969-RO).
   cpCJK_IR_165_Uni,  // ISO-IR-165 (extended version of GB-2312).
   cpCJK_932_Uni,     // Microsoft extended version of SHIFT_JIS.
   cpCJK_949_Uni,     // EUC-KR extended with UHC (Unified Hangul Codes).
   cpCJK_950_Uni,     // Microsoft extended version of Big5.
   cpCJK_JOHAB_Uni,   // JOHAB
   cpShiftJIS,        // ShiftJIS charset plus code page 932 ectension.
   cpBig5,            // Big5 plus HKSCS extension.
   cpGB2312,          // GB2312 charset plus GBK and cp936 extension.
   cpWansung,         // Wansung
   cpJohab,           // JOHAB
   cpMacRoman,        // Mac Roman
   cpAdobeStd,        // This is an encoding for Type1 fonts. It should normally not be used.
   cpInternal,        // Internal -> not usable
   cpGlyphIndexes,    // Can be used with TrueType and OpenType fonts only. DynaPDF creates a reverse mapping so that copy & paste will work.
   cpPDFDocEnc,       // Internal -> not usable. Used for form fonts. This is a superset of the code page 1252 and MacRoman.
   cpExtCMap,         // Internal -> not usable. This code page is set when a font was loaded with an external cmap.
   cpDingbats         // Internal -> Special encoding for ZapfDingbats
);

type TColorConvFlags = Cardinal;
const ccfBW_To_Gray   = $00000000; // Default, RGB Black and White set with rg or RG inline operators are converted to gray
const ccfRGB_To_Gray  = $00000001; // If set, inline color operators rg and RG are converted to gray

type TCompBBoxFlags = Cardinal;
const cbfNone              = $00000000;
const cbfIgnoreWhiteAreas  = $00000001; // Ignore white vector graphics or text.
{
   Please note that images must be decompressed if one of the following flags are set. Parsing gray or color images
   is in most cases not useful and you should not parse such images if it is not really required.
}
const cbfParse1BitImages   = $00000002; // Find the visible area in 1 bit images. This is the most important case
                                        // since scanned faxes are usually 1 bit images.
const cbfParseGrayImages   = $00000004; // Find the visible area in gray images.
const cbfParseColorImages  = $00000008; // Find the visible area in color images. This is usually not required
                                        // and slow downs processing a lot.
const cbfParseAllImages    = $0000000E; // Find the visible area in all images.

// The data for user defined columns is stored in collection items.
type TColColumnType =
(
   cisCreationDate, // Data comes from the embedded file
   cisDescription,  // Data comes from the embedded file
   cisFileName,     // Data comes from the embedded file
   cisModDate,      // Data comes from the embedded file
   cisSize,         // Data comes from the embedded file
   cisCustomDate,   // User defined date.
   cisCustomNumber, // User defined nummber.
   cisCustomString  // User defined string.
);

type TColorMode =
(
   cmFill,
   cmStroke,
   cmFillStroke
);

type TColView =
(
   civNotSet,
   civDetails,
   civTile,
   civHidden
);

type TCompressionLevel =
(
   clNone,
   clDefault,
   clFastest,
   clMax
);

type TCompressionFilter =
(
   cfFlate,    // PDF or TIFF output
   cfJPEG,     // PDF, JPEG, or TIFF output
   cfCCITT3,   // PDF or TIFF output -> B&W CCITT Fax G3 compression -> fast but less compression ratio
   cfCCITT4,   // PDF or TIFF output -> B&W CCITT Fax G4 compression -> slower but higher compression ratio
   cfLZW,      // TIFF or GIF output -> Very fast but less compression ratios than flate
   cfReserved, // Reserved for future extensions.
   cfFlateBW,  // TIFF, PNG, or BMP output -> Dithered black & white output. The resulting image will be
               // compressed with Flate or left uncompressed if the output image format is a bitmap. If
               // you want to use CCITT Fax 4 compression (TIFF only) set the flag icUseCCITT4 in the
               // AddImage() function call. Note that this filter is not supported for PDF creation!
   cfJP2K      // PDF or JPEG2000 output
);

type TConformanceType =
(
   ctPDFA_1b_2005,
   ctNormalize
);

type TCTM = record
   a: Double;
   b: Double;
   c: Double;
   d: Double;
   x: Double;
   y: Double;
end;

type PCTM = ^TCTM;

type TDestType =
(
   dtXY_Zoom,       // three parameters (a, b, c) -> (X, Y, Zoom)
   dtFit,           // no parameters
   dtFitH_Top,      // one parameter    (a)
   dtFitV_Left,     // one parameter    (a)
   dtFit_Rect,      // four parameters  (a, b, c, d) -> (left, bottom, right, top)
   dtFitB,          // no parameter
   dtFitBH_Top,     // one parameter    (a)
   dtFitBV_Left     // one parameter    (a)
);

type TDuplexMode =
(
   dpmNone,          // Default
   dpmSimplex,
   dpmFlipShortEdge,
   dpmFlipLongEdge
);

type TPDFRect = record
   Left,
   Bottom,
   Right,
   Top: Double;
end;

{
   The font search run works as follows:

      - DynaPDF tries always to find the exact weight, if it cannot be found then a font with
        the next smaller weight is selected (if available).
      - Italic styles can always be emulated but it is not possible to emulate thinner weights or
        regular styles with an italic font.
      - If the specified weight is larger as the font weight the remaining weight will be emulated
        if the difference to the requested weight is larger than 200.
        With SetFontWeight() it is possible to control whether a missing weight should be emulated.
        If FontWeight is smaller or equal to the requested font weight then emulation will be disabled.

   TFStyle is a bitmask that is defined as follows:

      - Bits 0..7   // Style bits fsItalic, fsUnderlined, fsStriked
      - Bits 8..19  // Width class -> Defined for future use.
      - Bits 20..31 // Font Weight

   - A width class can be converted to a style constant by multiplying it with 256 (width shl 8).
   - A font weight can be converted to a style constant by multiplying it with 1048576 (weight shl 20).
   - Additional attributes can be added with a binary or operator (e.g. style or fsItalic).
   - Only one width class and one font weight can be set at time.

   - WidthFromStyle() extracts the width class.
   - WeightFromStyle() extracts the font weight.

   The following functions extract the width class or font weight from a style variable:

      widthClass  = WidthFromStyle(style);
      weightClass = WeightFromStyle(style);
}

type TFStyle = Integer;
const
   fsNone           = $00000000; // Regular weight (400)
   fsItalic         = $00000001;
   fsUnderlined     = $00000004;
   fsStriked        = $00000008;
   fsVerticalMode   = $00000010; // Not considered at this time
   // Width class
   fsUltraCondensed = $00000100; // 1
   fsExtraCondensed = $00000200; // 2
   fsCondensed      = $00000300; // 3
   fsSemiCondensed  = $00000400; // 4
   fsNormal         = $00000500; // 5
   fsSemiExpanded   = $00000600; // 6
   fsExpanded       = $00000700; // 7
   fsExtraExpanded  = $00000800; // 8
   fsUltraExpanded  = $00000900; // 9
   // Weight class
   fsThin           = $06400000; // 100
   fsExtraLight     = $0c800000; // 200
   fsLight          = $12C00000; // 300
   fsRegular        = $19000000; // 400 -> Same as fsNone
   fsMedium         = $1F400000; // 500
   fsDemiBold       = $25800000; // 600
   fsBold           = $2BC00000; // 700 -> The old constant 2 is still supported to preserve backward compatibility
   fsExtraBold      = $32000000; // 800
   fsBlack          = $38400000; // 900
   fsUltraBlack     = $3E800000; // 1000

type TBookmark = record
   Color: Cardinal;
   DestPage: Integer;
   DestPos: TPDFRect;
   DestType: TDestType;
   Open: LongBool;
   Parent: Integer;
   Style: TFStyle;
   Title: Pointer;
   TitleLen: Cardinal;
   Unicode: LongBool;
end;

type TDecodeFilter =
(
   dfNone,
   dfASCII85Decode,  // No parameters
   dfASCIIHexDecode, // No parameters
   dfCCITTFaxDecode, // Optional Parameters
   dfDCTDecode,      // Optional Parameters
   dfFlateDecode,    // Optional Parameters
   dfJBIG2Decode,    // Optional Parameters
   dfJPXDecode,      // No parameters
   dfLZWDecode,      // Optional Parameters
   dfRunLengthDecode // No parameters
);

type TDecSeparator =
(  // per thousand separator, decimal separator
   dsCommaDot,
   dsNoneDot,
   dsDotComma,
   dsNoneComma
);

type TDocumentInfo =
(
   diAuthor,
   diCreator,
   diKeywords,
   diProducer,
   diSubject,
   diTitle,
   diCompany,
   diPDFX_Ver, // GetInDocInfo() only -> The PDF/X version is set by SetPDFVersion()!
   diCustom    // User defined key
);

type TDrawDirection =
(
   ddCounterClockwise,
   ddClockwise
);

type TDrawMode =
(
   dmNormal,
   dmStroke,
   dmFillStroke,
   dmInvisible,
   dmFillClip,
   dmStrokeClip,
   dmFillStrokeClip,
   dmClipping
);

type TExtColorSpace =
(
   esDeviceRGB,   // Device color space
   esDeviceCMYK,  // Device color space
   esDeviceGray,  // Device color space
   esCalGray,     // CIE-based color space
   esCalRGB,      // CIE-based color space
   esLab,         // CIE-based color space
   esICCBased,    // CIE-based color space -> contains an ICC profile
   esPattern,     // Special color space
   esIndexed,     // Special color space
   esSeparation,  // Special color space
   esDeviceN,     // Special color space
   esNChannel     // Special color space
);

type TFieldColor =
(
   fcBackColor,
   fcBorderColor,
   fcTextColor
);

type TAnnotColor = TFieldColor;

type TFieldFlags = Cardinal;
const ffReadOnly         = $00000001;
const ffRequired         = $00000002;
const ffNoExport         = $00000004;

const ffInvisible        = $00000008;
const ffHidden           = $00000010;
const ffPrint            = $00000020;
const ffNoZoom           = $00000040;
const ffNoRotate         = $00000080;
const ffNoView           = $00000100;

const ffMultiline        = $00001000;  // Text fields only
const ffPassword         = $00002000;  // Text fields only
const ffNoToggleToOff    = $00004000;  // Radio buttons
const ffRadioIsUnion     = $02000000;  // PDF-1.5 radio buttons only
const ffCommitOnSelCh    = $04000000;  // PDF-1.5 combo boxes, list boxes

const ffEdit             = $00040000;  // Combo boxes only
const ffSorted           = $00080000;  // Do not use this flag; CreateComboBox or CreateListBox set this flag automatically
const ffFileSelect       = $00100000;  // PDF 1.4 Text fields only
const ffMultiSelect      = $00200000;  // PDF 1.4 List boxes only
const ffDoNotSpellCheck  = $00400000;  // PDF 1.4 Text fields, combo boxes. If the field is a combo box, this flag is meaningful only if ffEdit is also set.
const ffDoNotScroll      = $00800000;  // PDF 1.4 Text fields only
const ffComb             = $01000000;  // PDF 1.5 Text fields only

type TFieldType =
(
   ftButton,
   ftCheckBox,
   ftRadioBtn,
   ftComboBox,
   ftListBox,
   ftText,
   ftSignature,
   ftGroup     // this is not a real field type, it is just an array of fields
);

type TFileAttachIcon =
(
   faiGraph,
   faiPaperClip,
   faiPushPin,
   faiTag
);

type TFileOP =
(
   foOpen,
   foPrint
);

type TFontBaseType =
(
   fbtTrueType, // TrueType, TrueType Collections, or OpenType fonts with TrueType outlines
   fbtType1,    // Type1 font
   fbtOpenType, // OpenType font with Postscript outlines
   fbtStdFont,  // PDF Standard font
   fbtDisabled  // This value can be used to disable a specific font format. See SetFontSearchOrder() for further information.
);
type PFontBaseType = ^TFontBaseType;

type TFontFileSubtype =
(
	ffsType1C,        // CFF based Type1 font
	ffsCIDFontType0C, // CFF based Type1 CID font
	ffsOpenType,      // TrueType based OpenType font
   ffsOpenTypeC,     // CFF based OpenType font
   ffsCIDFontType2,  // TrueType based CID Font
   ffsReserved1,
   ffsReserved2,
   ffsReserved3,
   ffsReserved4,
	ffsNoSubtype
);

type TFontSelMode =
(
   smFamilyName,
   smPostScriptName
);

type TFontType =
(
   ftMMType1,
   ftTrueType,
   ftType0,    // Check the font file type to determine the font sub type
   ftType1,
   ftType3
);

type TGStateFlags = Cardinal;
const gfCompatible         = $00000000; // Compatible graphics state to earlier DynaPDF versions -> default
const gfRestorePageCoords  = $00000001; // Restore the coordinate system with the graphics state (the value of PageCoords, see SetPageCoords())
const gfRealTopDownCoords  = $00000002; // If set, the page coordinate system is not reset to bottom-up when transforming
                                        // the coordinate system. However, real top-down coordinates require a large internal
                                        // overhead and where never fully implemented. The usage of this flag should be avoided
                                        // if possible.
const gfNativeBlackWhite   = $00000004; // Do not convert RGB black or white to DeviceGray
const gfUseImageColorSpace = $00000008; // If set, the active color space is ignored when inserting an image. The color space is taken
                                        // from the image file instead.
const gfIgnoreICCProfiles  = $00000010; // Meaningful only if the flag gfUseImageColorSpace is set. If set, an embedded profile is not used to
                                        // create an ICCBased color space for the image. The image is inserted in the corresponding device
                                        // color space instead.

type THashType =
(
   htDetached, // CloseAndSignFileExt() returns the byte ranges of the finish PDF buffer to create a detached signature
   htSHA1      // CloseAndSignFileExt() returns the SHA1 hash of the PDF file so that it can be signed
);
                                        
type THighlightMode =
(
  hmNone,
  hmInvert,
  hmOutline,
  hmPush,
  hmPushUpd  // Update appereance stream on changes
);

type TICCProfileType =
(
   ictGray,
   ictRGB,
   ictCMYK,
   ictLab
);

type TImageConversionFlags = Cardinal;
const icNone      = 0; // Default
const icUseCCITT4 = 1; // Use CCITT Fax 4 compression instead of Flate for dithered images.

{
   TIFF is the only format that supports different compression filters. The Filter parameter of the function
   AddImage() is ignored if the image format supports only one specific compression filter.
   Note that images are automatically converted to the nearest supported color space if the image format does
   not support the color space of the image.
}
type TImageFormat =
(
   ifmTIFF,     // DeviceRGB, DeviceCMYK, DeviceGray, Black & White -> CCITT Fax Group 3/4, JPEG, Flate, LZW.
   ifmJPEG,     // DeviceRGB, DeviceCMYK, DeviceGray    -> JPEG compression.
   ifmPNG,      // DeviceGray, DeviceRGB, Black & White -> Flate compression.
   ifmReserved, // Reserved for future extensions.
   ifmBMP,      // DeviceGray, DeviceRGB, Black & White -> Uncompressed.
   ifmJPC       // DeviceRGB, DeviceCMYK, DeviceGray    -> JPEG2000 compression.
);

type TImportFlags = Cardinal;
const ifImportAll        = $0FFFFFFE; // default
const ifContentOnly      = $00000000;
// If this flag is set, only interactive objects are imported if any, Otherwise only empty pages are imported.
// This flag can be used to copy an interactive form to another PDF file.
const ifNoContent        = $00000001;
// The imported page is not converted to a template if ifImportAsPage is set.
// Note that this flag can cause resource conflicts. Use this flag carefully!
const ifImportAsPage     = $80000000;
// base objects
const ifCatalogAction    = $00000002;
const ifPageActions      = $00000004;
const ifBookmarks        = $00000008;
const ifArticles         = $00000010;
const ifPageLabels       = $00000020;
const ifThumbs           = $00000040;
const ifTranspGroups     = $00000080; // This flag is not longer considered.
const ifSeparationInfo   = $00000100;
const ifBoxColorInfo     = $00000200;
const ifStructureTree    = $00000400;
const ifTransition       = $00000800;
const ifSearchIndex      = $00001000;
const ifJavaScript       = $00002000;
const ifJSActions        = $00004000;
const ifDocInfo          = $00008000;
const ifEmbeddedFiles    = $00200000; // File attachments
const ifFileCollections  = $00400000; // File collections (PDF 1.7)
// Annotations -> Only the most important annotation types can be selected directly.
// Note that all annotation types can be deleted with DeleteAnnotation.
const ifAllAnnots        = $009F0000;
   const ifFreeText      = $00010000;
   const ifTextAnnot     = $00020000;
   const ifLink          = $00040000;
   const ifStamp         = $00080000;
   const if3DAnnot       = $00100000;
   const ifOtherAnnots   = $00800000;

// Interactive Form fields are also annotations but we handle this type separately!
const ifFormFields       = $01000000;
{ -------------------- Special flags -------------------- }
const ifPrepareForPDFA   = $10000000; // Replace LZW compression with Flate, set the Interpolate key of images to false, do not import embedded files.
const ifEnumFonts        = $20000000; // Import fonts for EnumDocFonts(). The document must be deleted when this flag is set!!!
const ifAllPageObjects   = $40000000; // Import links when using ImportPageEx() within an open page. The entire document should be imported in this case.

type TImportFlags2 = Cardinal;
const if2MergeLayers     = $00000001; // If set, layers with identical name are merged. If this flag is absent DynaPDF
                                      // imports such layers separately so that each layer refers still to the pages
                                      // where it was orignally used.
const if2Normalize       = $00000002; // Replace LZW compression with Flate, apply limit checks, repair errors if possible.

type TKeyLen =
(
   kl40bit,    // RC4 Encryption -> Acrobat 3 or higher
   kl128bit,   // RC4 Encryption -> Acrobat 5 or higher
   kl128bitEx, // RC4 Encryption -> Acrobat 6 or higher
   klAES128,   // AES Encryption -> Acrobat 7 or higher
   klAES256    // AES Encryption -> Acrobat 9 or higher
);

type TLineCapStyle =
(
  csButtCap,
  csRoundCap,
  csSquareCap
);

type TLineJoinStyle =
(
  jsMiterJoin,
  jsRoundJoin,
  jsBevelJoin
);

type TMetaFlags = Cardinal;
const mfDefault           = $00000000; // Default conversion
const mfDebug             = $00000001; // Write debug information into the content stream
const mfShowBounds        = $00000002; // Show the bounding boxes of text strings
const mfNoTextScaling     = $00000004; // Do not scale text
const mfClipView          = $00000008; // Draw the file into a clipping rectangle
const mfUseRclBounds      = $00000010; // Use rclBounds instead of rclFrame
const mfNoClippingRgn     = $00000040; // Disables SelectClippingRegion and IntersectClipRect
const mfNoFontEmbedding   = $00000080; // Do not embed fonts -> Fonts should be embedded!!!
const mfNoImages          = $00000100; // Ignore image records
const mfNoStdPatterns     = $00000200; // Ignore standard patterns
const mfNoBmpPatterns     = $00000400; // Ignore bitmap patterns
const mfNoText            = $00000800; // Ignore text records
const mfUseUnicode        = $00001000; // Ignore ANSI_CHARSET
const mfUseTextScaling    = $00004000; // Scale text instead of using the intercharacter spacing array
const mfNoUnicode         = $00008000; // Avoid usage of Unicode fonts -> recommended to enable PDF 1.2 compability
const mfFullScale         = $00010000; // Scale coordinates to the window size. Recommended if 32 bit coordinates are used.
const mfUseRclFrame       = $00020000; // This flag should be set if the rclFrame rectangle is properly set
const mfDefBkModeTransp   = $00040000; // Initialize the background mode to transparent (SetBkMode() overrides this state).
const mfApplyBidiAlgo     = $00080000; // Apply the bidirectional algorithm on Unicode strings
const mfGDIFontSelection  = $00100000; // Use the GDI to select fonts
const mfRclFrameEx        = $00200000; // If set, and if the rclBounds rectangle is larger than rclFrame, the function
                                       // extends the output rectangle according to rclBounds and uses the resulting
                                       // bounding box to calculate the image size (rclBounds represents the unscaled
                                       // image size). This is probably the correct way to calculate the image size.
                                       // However, to preserve backward compatibility the default calculation cannot
                                       // be changed.
const mfNoTextClipping    = $00400000; // If set, the ETO_CLIPPED flag in text records is ignored.
const mfSrcCopy_Only      = $00800000; // If set, images which use a ROP code other than SRCCOPY are ignored. This is useful when processing Excel 2007 spool files.
const mfClipRclBounds     = $01000000; // If set, the graphic is drawn into a clipping path with the size of rclBounds.
                                       // This flag is useful if the graphic contains content outside of its bounding box.
const mfDisableRasterEMF  = $02000000; // If set, EMF files which use unsupported ROP codes are not rastered.

// Obsolete flags -> these flags are ignored, do no longer use them!
const mfUseSpacingArray   = $00000020; // enabled by default -> can be disabled with mfUseTextScaling
const mfIntersectClipRect = $00002000; // enabled by default -> can be disabled with mfNoClippingRgn

type TNamedAction =
(
   naFirstPage,
   naLastPage,
   naNexPage,
   naPrevPage,
   naGoBack,
   naOpenDlg,
   naPrintDlg,
   naGeneralInfo,
   naFontsInfo,
   naSaveAs,
   naSecurityInfo,
   naFitPage,
   naFullScreen,
   naDeletePages,
   naQuit
);

type TNegativeStyle =
(
   nsMinusBlack,
   nsRed,
   nsParensBlack,
   nsParensRed
);

type TObjEvent =
( // All actions which should be applied to an event (except On Mouse Up) must be a JavaScript action!
   oeNoEvent,           // Internal use only -> DO NOT USE THIS VALUE!!!
   oeOnOpen,            // Catalog, Pages
   oeOnClose,           // Pages only
   oeOnMouseUp,         // All fields, page link annotations, bookmarks
   oeOnMouseEnter,      // Form fields only
   oeOnMouseExit,       // Form fields only
   oeOnMouseDown,       // Form fields only
   oeOnFocus,           // Form fields only
   oeOnBlur,            // Form fields only
   oeOnKeyStroke,       // Text fields only
   oeOnFormat,          // Text fields only
   oeOnCalc,            // Text fields, combo boxes, list boxes
   oeOnValidate,        // All form fields, except buttons
   oeOnPageVisible,     // PDF 1.5 -> Form fields only
   oeOnPageInVisible,   // PDF 1.5 -> Form fields only
   oeOnPageOpen,        // PDF 1.5 -> Form fields only
   oeOnPageClose,       // PDF 1.5 -> Form fields only
   oeOnBeforeClosing,   // PDF 1.4 -> Catalog only
   oeOnBeforeSaving,    // PDF 1.4 -> Catalog only
   oeOnAfterSaving,     // PDF 1.4 -> Catalog only
   oeOnBeforePrinting,  // PDF 1.4 -> Catalog only
   oeOnAfterPrinting    // PDF 1.4 -> Catalog only
);

type TObjType =
(
   otAction,
   otAnnotation,
   otBookmark,
   otCatalog,           // PDF 1.4
   otField,
   otPage,
   otPageLink
);

type TOCObject =
(
   ooAnnotation,
   ooField,
   ooImage,
   ooTemplate
);

{$ifndef CompilerVersion or CompilerVersion < 5.0}
   type TOCGIntent = Cardinal;
   const oiDesign = 2;
   const oiView   = 4;  // Default
   const oiAll    = 8;
   const oiEmpty  = 16; // Internal
{$else}
   type TOCGIntent =
   (
      oiDesign = 2,
      oiView   = 4, // Default
      oiAll    = 8,
      oiEmpty  = 16 // Internal
   );
{$endif}

type TOCVisibility =
(
   ovAllOff,
   ovAllOn,
   ovAnyOff,
   ovAnyOn,
   ovNotSet  // Internal
);

type TOrigin =
(
  orDownLeft,
  orTopLeft
);

type TPageCoord =
(
  pcBottomUp,
  pcTopDown
);

type TPageFormat =
(
  pfDIN_A3,
  pfDIN_A4,
  pfDIN_A5,
  pfDIN_B4,
  pfDIN_B5,
  pfDIN_B6,
  pfDIN_C3,
  pfDIN_C4,
  pfDIN_C5,
  pfDIN_C6,
  pfDIN_C65,
  pfDIN_DL,
  pfDIN_E4,
  pfDIN_E5,
  pfDIN_E6,
  pfDIN_E65,
  pfDIN_M5,
  pfDIN_M65,
  pfUS_Legal,
  pfUS_Letter
);

type TPageBoundary =
(
   pbArtBox,
   pbBleedBox,
   pbCropBox,
   pbTrimBox,
   pbMediaBox
);

type TPageMode =
(
   pmUseNone,
   pmUseOutLines,
   pmUseThumbs,
   pmFullScreen,
   pmUseOC,
   pmUseAttachments
);

type TPageLabelFormat =
(
   plfDecimalArabic,    // 1,2,3,4...
   plfUppercaseRoman,   // I,II,III,IV...
   plfLowercaseRoman,   // i,ii,iii,iv...
   plfUppercaseLetters, // A,B,C,D...
   plfLowercaseLetters, // a,b,c,d...
   plfNone
);

type TPageLayout =
(
   plSinglePage,
   plOneColumn,
   plTwoColumnLeft,
   plTwoColumnRight,
   plTwoPageLeft,
   plTwoPageRight,
   plDefault
);

type TParseFlags = Cardinal;
const pfNone             = $00000000;
const pfDecomprAllImages = $00000002; // This flag causes that all image formats will be decompressed
                                      // with the exception of JBIG2 compressed images. If this flag is
                                      // not set, images which are already stored in a valid image file
                                      // format are returned as is. This is the case for Gray and RGB JPEG
                                      // images and for JPEG2000 images. If you want to extract the images
                                      // of a PDF file this flag should NOT be set!

const pfNoJPXDecode      = $00000004; // Meaningful only if the flag pfDecomprAllImages is set. If set,
                                      // JPEG2000 images are returned as is so that you can use your own
                                      // library to decompress such images since the the entire JPEG2000
                                      // codec is still marked as experimental. If we find an alternative
                                      // to the currently used Jasper library then we will replace it
                                      // immediatly with another one...

// The following flags are ignored if an image is not decompressed. Note that only one flag must be set
// at time. If no color space conversion flag is set images are returned in their native or alternate
// device color space. Note that these flags do not convert colors of vector graphics and so on.
// Use the function ConvColor() to convert colors of the graphics state into a device space.
const pfDitherImagesToBW = $00000008; // Floyd-Steinberg dithering.
const pfConvImagesToGray = $00000010;
const pfConvImagesToRGB  = $00000020;
const pfConvImagesToCMYK = $00000040;
const pfImageInfoOnly    = $00000080; // If set, images are not decompressed. This flag is useful if you want
                                      // to enumerate the images of a PDF file or if you want to determine how
                                      // many images are stored in it.
                                      // Note that images can be compressed with multiple filters. The member
                                      // Filter of the structure TPDFImage contains only the last filter with
                                      // which the image was compressed. There is no indication whether multiple
                                      // decode filters are required to decompress the image buffer. So, it
                                      // makes no sense to set this flag if you want to try to decompress the
                                      // image buffer manually with your own decode filters.

type TPathFillMode =
(
   fmFillNoClose,
   fmStrokeNoClose,
   fmFillStrokeNoClose,
   fmFill,
   fmStroke,
   fmFillStroke,
   fmFillEvOdd,
   fmFillStrokeEvOdd,
   fmFillEvOddNoClose,
   fmFillStrokeEvOddNoClose,
   fmNoFill,
   fmClose
);

type TPatternType =
(
   ptColored,
   ptUnColored,
   ptShadingPattern // Cannot be created with DynaPDF
);

{
   Notice:
   When using a bidirectional 8 bit code page the bidi algorithm is applied by default in Left to Right mode
   also if the bidi mode is set to bmNone (default). This mode produces identical results in comparison to
   applications like Edit or WordPad.

   The Right to Left mode is available in applications which use Microsoft's Uniscribe, e.g. BabelPad or MS Word.
   This mode works very well with the Reference Bidi Algorithm which is used by DynaPDF but Uniscribe's Left to Right
   mode produces different results in comparison to the Reference Bidi Algorithm. Because the bidi algorithm that is
   used in Uniscribe is not published it is practically impossible to get the same result in Left to Right mode
   without using this library.
}
type TPDFBidiMode =
(
   bmLeftToRight, // Apply the bidi algorithm on Unicode strings in Left to Right layout.
   bmRightToLeft, // Apply the bidi algorithm on Unicode strings in Right to Left layout.
   bmNone         // Default -> do not apply the bidi algorithm
);

type TPrintScaling =
(
   psAppDefault, // Default
   psNone
);

type TPDFPrintSettings = record
   DuplexMode:        TDuplexMode;
   NumCopies:         Integer;      // -1 means not set. Values larger than 5 are ignored in viewer applications.
   PickTrayByPDFSize: Integer;      // -1 means not set. 0 == false, 1 == true.
   PrintRanges:       PCardinal;    // If set, the array contains PrintRangesCount * 2 values. Each pair consists
                                    // of the first and last pages in the sub-range. The first page in the PDF file
                                    // is denoted by 0.
   PrintRangesCount: Cardinal;      // Number of ranges available in PrintRanges.
   PrintScaling:     TPrintScaling; // dpmNone means not set.
   { Reserved fields for future extensions }
   Reserved0: Cardinal;
   Reserved1: Cardinal;
   Reserved2: Cardinal;
   Reserved3: Cardinal;
   Reserved4: Cardinal;
   Reserved5: Cardinal;
   Reserved6: Cardinal;
   Reserved7: Cardinal;
   Reserved8: Cardinal;
   Reserved9: Cardinal;
end;

type TPwdType =
(
   ptOpen,
   ptOwner
);

type TRawImageFlags = Cardinal;
const
   rifByteAligned  = $1000;
   rifRGBData      = $2000;
   rifCMYKData     = $4000;

type TRenderingIntent =
(
   riAbsoluteColorimetric,
   riPerceptual,
   riRelativeColorimetric,
   riSaturation,
   riNone
);

type TRestrictions = Integer;
const
   rsDenyNothing      = $00000000;
   rsDenyAll          = $00000F3C;
   rsPrint            = $00000004;
   rsModify           = $00000008;
   rsCopyObj          = $00000010;
   rsAddObj           = $00000020;
   // 128/256 bit encryption only -> these flags are ignored if 40 bit encryption is used
   rsFillInFormFields = $00000100;
   rsExtractObj       = $00000200;
   rsAssemble         = $00000400;
   rsPrintHighRes     = $00000800;
   rsExlMetadata      = $00001000; // PDF 1.5 Exclude metadata streams -> 128/256 bit encryption bit only.
   rsEmbFilesOnly     = $00002000; // PDF 1.6 Encrypt embedded files only -> Requires AES encryption.

type TRubberStamp =
(
  rsApproved,
  rsAsIs,
  rsConfidential,
  rsDepartmental,
  rsDraft,
  rsExperimental,
  rsExpired,
  rsFinal,
  rsForComment,
  rsForPublicRelease,
  rsNotApproved,
  rsNotForPublicRelease,
  rsSold,
  rsTopSecred,
  rsUserDefined
);

type TTilingType =
(
   ttConstSpacing,
   ttNoDistortion,
   ttFastConstSpacing
);

type TTextAlign = Cardinal;
const
   taLeft      = 0;
   taCenter    = 1;
   taRight     = 2;
   taJustify   = 3;
   taPlainText = $10000000; // If this flag is set alignment and command tags are interpreted as plain text.
                            // See WriteFText() in the help file for further information.

type TTextRecordA = record
   Advance: Single;    // Negative values move the cursor to right, positive to left. The value is measured in text space!
   Text:    PAnsiChar; // Raw text (not null-terminated)
   Length:  Integer;   // Raw text length in bytes
end;

type TTextRecordAPtr = ^TTextRecordA;

type TTextRecordW = record
   Advance: Single;    // Negative values move the cursor to right, positive to left. The value is measured in text space!
   Text:    PWideChar; // Already translated Unicode string (not null-terminated)
   Length:  Integer;   // Length in characters
   Width:   Single;    // AnsiString width measured in text space
end;

type TTextRecordWPtr = ^TTextRecordW;

type TPDFAnnotation = record
   AnnotType:     TAnnotType;
   Deleted:       LongBool;
   BBox:          TPDFRect;
   BorderWidth:   Double;
   BorderColor:   Cardinal;
   BorderStyle:   TBorderStyle;
   BackColor:     Cardinal;
   Handle:        Cardinal;
   AuthorA:       PAnsiChar;
   AuthorW:       PWideChar;
   ContentA:      PAnsiChar;
   ContentW:      PWideChar;
   NameA:         PAnsiChar;
   NameW:         PWideChar;
   SubjectA:      PAnsiChar;
   SubjectW:      PWideChar;
   PageNum:       Cardinal;
   HighlightMode: THighlightMode;
end;

type TPDFAnnotationEx = record
   AnnotType:     TAnnotType;
   Deleted:       LongBool;
   BBox:          TPDFRect;
   BorderWidth:   Single;
   BorderColor:   Cardinal;
   BorderStyle:   TBorderStyle;
   BackColor:     Cardinal;
   Handle:        Cardinal;
   AuthorA:       PAnsiChar;
   AuthorW:       PWideChar;
   ContentA:      PAnsiChar;
   ContentW:      PWideChar;
   NameA:         PAnsiChar;
   NameW:         PWideChar;
   SubjectA:      PAnsiChar;
   SubjectW:      PWideChar;
   PageNum:       Cardinal;
   HighlightMode: THighlightMode;
   // Page link annotations only
   DestPage:      Integer;
   DestPos:       TPDFRect;
   DestType:      TDestType;
   // File link or web link annotations
   DestFile:      PAnsiChar;
   // The Icon type depends on the annotation type. If the annotation type is atText then the Icon
   // is of type TAnnotIcon. If the annotation type is atFileAttach then it is of type
   // TFileAttachIcon. If the annotation type is atStamp then the Icon is the stamp type (TRubberStamp).
   // For any other annotation type this value is not set (-1).
   Icon:          Integer;
   StampName:     PAnsiChar; // Set only, if Icon == rsUserDefined
   AnnotFlags:    Cardinal;  // See TAnnotFlags for available flags
   CreateDate:    PAnsiChar; // Creation Date -> Optional
   ModDate:       PAnsiChar; // Modification Date -> Optional
   Grouped:       LongBool;  // (Reply type) Meaningful only if Parent != -1 and Type != atPopUp. If true,
                             // the annotation is part of an annotation group. Properties like Content, CreateDate,
                             // ModDate, BackColor, Subject, and Open must be taken from the parent annotation.
   Open:          LongBool;  // Meaningful only for annotations which have a corresponding PopUp annotation.
   Parent:        Integer;   // Parent annotation handle of a PopUp Annotation or the parent annotation if
                             // this annotation represents a state of a base annotation. In this case,
                             // the annotation type is always atText and only the following members should
                             // be considered:
                             //    State      // The current state
                             //    StateModel // Marked, Review, and so on
                             //    CreateDate // Creation Date
                             //    ModDate    // Modification Date
                             //    Author     // The user who has set the state
                             //    Content    // Not displayed in Adobe's Acrobat...
                             //    Subject    // Not displayed in Adobe's Acrobat...
                             // The PopUp annotation of a text annotation which represent an Annotation State
                             // must be ignored.
   PopUp:         Integer;   // Handle of the corresponding PopUp annotation if any.
   State:         PAnsiChar; // The state of the annotation.
   StateModel:    PAnsiChar; // The state model (Marked, Review, and so on).
   EmbeddedFile:  Integer;   // FileAttach annotations only. A handle of an embedded file -> GetEmbeddedFile().
   Subtype:       PAnsiChar; // Set only, if Type == atUnknownAnnot
   { Reserved fields for future extensions }
   Reserved01:    Pointer;
   Reserved02:    Pointer;
   Reserved03:    Pointer;
   Reserved04:    Pointer;
   Reserved05:    Pointer;
   Reserved06:    Pointer;
   Reserved07:    Pointer;
   Reserved08:    Pointer;
   Reserved09:    Pointer;
   Reserved10:    Pointer;
   Reserved11:    Pointer;
   Reserved12:    Pointer;
   Reserved13:    Pointer;
   Reserved14:    Pointer;
   Reserved15:    Pointer;
   Reserved16:    Pointer;
   Reserved17:    Pointer;
   Reserved18:    Pointer;
   Reserved19:    Pointer;
   Reserved20:    Pointer;
   Reserved21:    Pointer;
   Reserved22:    Pointer;
   Reserved23:    Pointer;
   Reserved24:    Pointer;
   Reserved25:    Pointer;
   Reserved26:    Pointer;
   Reserved27:    Pointer;
end;

// The tags have the same meaning as the corresponding HTML tags.
// See PDF Reference for further information.
type TPDFBaseTag =
(
   btArt,
   btArtifact,
   btAnnot,
   btBibEntry,      // BibEntry -> Bibliography entry
   btBlockQuote,
   btCaption,
   btCode,
   btDiv,
   btDocument,
   btFigure,
   btForm,
   btFormula,
   btH,
   btH1,
   btH2,
   btH3,
   btH4,
   btH5,
   btH6,
   btIndex,
   btLink,
   btList,          // L
   btListElem,      // LI
   btListText,      // LBody
   btNote,
   btP,
   btPart,
   btQuote,
   btReference,
   btSection,       // Sect
   btSpan,
   btTable,
   btTableDataCell, // TD
   btTableHeader,   // TH
   btTableRow,      // TR
   btTOC,
   btTOCEntry       // TOCI
);

type TPDFChoiceValue = record
   StructSize:  Cardinal; // Must be set with sizeof(TPDFChoiceValue)
   ExpValueA:   PAnsiChar;
   ExpValueW:   PWideChar;
   ExpValueLen: Cardinal;
   ValueA:      PAnsiChar;
   ValueW:      PWideChar;
   ValueLen:    Cardinal;
   Selected:    LongBool;
end;

{
   The structure contains several duplicate fields because CMap files contain usually a DSC comment
   section which provides Postscript specific initialization code. With exception of DSCResName the
   strings in the DSC section should not differ from their CMap counterparts. The Identity mapping
   of a character collection should contain the DSC comment "%%BeginResource: CMap (Identity)".
   Otherwise the string should be set to the CMap name.
}
type TPDFCMap = record
   StructSize:     Cardinal;  // Must be set to sizeof(TPDFCMap) before calling GetCMap()!
   BaseCMap:       PAnsiChar; // If set, this base cmap is required when loading the cmap.
   CIDCount:       Cardinal;  // 0 if not set.
   CMapName:       PAnsiChar; // The CMap name.
   CMapType:       Cardinal;  // Should be 1!
   CMapVersion:    Single;    // The CMap version.
   DSCBaseCMap:    PAnsiChar; // DSC comment.
   DSCCMapVersion: Single;    // DSC comment.
   DSCResName:     PAnsiChar; // DSC comment. If the CMap uses an Identity mapping this string should be set to Identity.
   DSCTitle:       PAnsiChar; // DSC comment -> DSC CMap name + Registry + Ordering + Supplement, e.g. "GB-EUC-H Adobe GB1 0"
   FileNameA:      PAnsiChar; // The file name and CMap name should be identical!
   FileNameW:      PWideChar; // The file name and CMap name should be identical!
   FilePathA:      PAnsiChar; // The Ansi string is set if the Ansi version of SetCMapDir() was used.
   FilePathW:      PWideChar; // The Unicode string is set if the Unicode version of SetCMapDir() was used.
   Ordering:       PAnsiChar; // CIDSystemInfo -> The Character Collection, e.g. Japan1.
   Registry:       PAnsiChar; // CIDSystemInfo -> The registrant of the Character Collection is usually Adobe.
   Supplement:     Cardinal;  // CIDSystemInfo -> The Supplement number should be supported in the used PDF Version.
   WritingMode:    Cardinal;  // 0 == Horizontal, 1 == Vertical
end;

type TPDFColorSpace =
(
   csDeviceRGB,
   csDeviceCMYK,
   csDeviceGray
);

type TPDFColorSpaceObj = record
   ColorSpaceType:   TExtColorSpace;
   Alternate:        TExtColorSpace; // Alternate color space or base space of an Indexed or Pattern color space.
   IAlternate:       Pointer;        // Only set if the color space contains an alternate or base color space -> GetColorSpaceObjEx().
   Buffer:           PByte;          // Contains either an ICC profile or the color table of an Indexed color space.
   BufSize:          Cardinal;       // Buffer length in bytes.
   BlackPoint:       PSingle;        // CIE blackpoint. If set, the array contains exactly 3 values.
   WhitePoint:       PSingle;        // CIE whitepoint. If set, the array contains exactly 3 values.
   Gamma:            PSingle;        // If set, one value per component.
   Range:            PSingle;        // min/max for each component or for the .a and .b components of a Lab color space.
   Matrix:           PSingle;        // XYZ matrix. If set, the array contains exactly 9 values.
   NumInComponents:  Cardinal;       // Number of input components.
   NumOutComponents: Cardinal;       // Number of output components.
   NumColors:        Cardinal;       // HiVal + 1 as specified in the color space. Indexed color space only.
   Colorants:        Array[0..31] of PAnsiChar; // UTF-8 Colorant names (Separation, DeviceN, and NChannel only).
   ColorantsCount:   Cardinal;       // The number of colorants in the array.
   Metadata:         PByte;          // Optional XMP metadata stream -> ICCBased only.
   MetadataSize:     Cardinal;       // Metadata length in bytes.
   IFunction:        Pointer;        // Pointer to function object -> Separation, DeviceN, and NChannel only.
   IAttributes:      Pointer;        // Optional attributes of DeviceN or NChannel color spaces -> GetNChannelOptions().
   IColorSpaceObj:   Pointer;        // Pointer of the corresponding color space object
   Reserved01:       Pointer;
   Reserved02:       Pointer;
   Reserved03:       Pointer;
   Reserved04:       Pointer;
   Reserved05:       Pointer;
   Reserved06:       Pointer;
   Reserved07:       Pointer;
   Reserved08:       Pointer;
   Reserved09:       Pointer;
end;

type TDeviceNAttributes = record
   IProcessColorSpace:    Pointer;    // Pointer to process color space or NULL -> GetColorSpaceEx().
   ProcessColorants:      Array[0..7] of PAnsiChar; // UTF-8 Process colorant names
   ProcessColorantsCount: Cardinal;   // Number of process colorants in the array or zero if not set.
   Separations:           Array[0..31] of Pointer; // Optional pointers to Separation color spaces -> GetColorSpaceEx().
   SeparationsCount:      Cardinal;   // Number of color spaces in the array.
   IMixingHints:          Pointer;    // Optional mixing hints. There is no API function at this time to access mixing hints.
   Reserved01:            Pointer;
   Reserved02:            Pointer;
   Reserved03:            Pointer;
   Reserved04:            Pointer;
end;

type TPDFDateTime =
(
   dfMM_D,
   dfM_D_YY,
   dfMM_DD_YY,
   dfMM_YY,
   dfD_MMM,
   dfD_MMM_YY,
   dfDD_MMM_YY,
   dfYY_MM_DD,
   dfMMM_YY,
   dfMMMM_YY,
   dfMMM_D_YYYY,
   dfMMMM_D_YYYY,
   dfM_D_YY_H_MM_TT,
   dfM_D_YY_HH_MM,
   { time formats }
   df24HR_MM,
   df12HR_MM,
   df24HR_MM_SS,
   df12HR_MM_SS
);

// It is not possible to set all available graphic state parameters with DynaPDF, such as black generation functions,
// halftones and so on. The structure will be extended when further objects are supported.
type TPDFExtGState = record
   AutoStrokeAdjust: Cardinal;          // PDF_MAX_INT if not set
   BlendMode:        TBlendMode;        // Default bmNotSet
   FlatnessTol:      Single;            // -1.0 if not set
   OverPrintFill:    Cardinal;          // PDF_MAX_INT if not set
   OverPrintStroke:  Cardinal;          // PDF_MAX_INT if not set
   OverPrintMode:    Cardinal;          // PDF_MAX_INT if not set
   RenderingIntent:  TRenderingIntent;  // riNone if not set
   SmoothnessTol:    Single;            // -1.0 if not set
   FillAlpha:        Single;            // -1.0 if not set
   StrokeAlpha:      Single;            // -1.0 if not set
   AlphaIsShape:     Cardinal;          // PDF_MAX_INT if not set
   TextKnockout:     Cardinal;          // PDF_MAX_INT if not set
   Reserved1:        Pointer;
   Reserved2:        Pointer;
   Reserved3:        Pointer;
   Reserved4:        Pointer;
   Reserved5:        Pointer;
   Reserved6:        Pointer;
   Reserved7:        Pointer;
   Reserved8:        Pointer;
   Reserved9:        Pointer;
end;

// Extended graphics state dictionary
type TPDFExtGState2 = record
   AlphaIsShape:       Integer;          // PDF_MAX_INT if not set
   AutoStrokeAdjust:   Integer;          // PDF_MAX_INT if not set
   BlackGen:           Pointer;          // Function handle or NULL -> GetFunction()
   BlackGen2:          Pointer;          // Function handle or NULL -> GetFunction()
   BlendMode:         ^TBlendMode;       // Array of blend modes or nil
   BlendModeCount:     Cardinal;         // Number of blend modes
   FillAlpha:          Single;           // -1.0 if not set
   FlatnessTol:        Single;           // -1.0 if not set
   Halftone:           Pointer;          // Halftone handle or NULL -> GetHalftoneDict()
   OverPrintFill:      Integer;          // PDF_MAX_INT if not set
   OverPrintStroke:    Integer;          // PDF_MAX_INT if not set
   OverPrintMode:      Integer;          // PDF_MAX_INT if not set
   RenderingIntent:    TRenderingIntent; // riNone if not set
   SmoothnessTol:      Single;           // -1.0 if not set
   SoftMask:           Pointer;          // Soft mask handle or NULL
   StrokeAlpha:        Single;           // -1.0 if not set
   TextKnockout:       Integer;          // PDF_MAX_INT if not set
   TransferFunc:       Pointer;          // Array of functions -> GetFunction()
   TransferFuncCount:  Cardinal;         // Number of transfer functions
   TransferFunc2:      Pointer;          // Array of functions -> GetFunction()
   TransferFunc2Count: Cardinal;         // Number of transfer functions
   UnderColorRem:      Pointer;          // Function handle or NULL -> GetFunction()
   UnderColorRem2:     Pointer;          // Function handle or NULL -> GetFunction()
   Reserved1:          Pointer;
   Reserved2:          Pointer;
   Reserved3:          Pointer;
   Reserved4:          Pointer;
end;

type TPDFField = record
   FieldType:    Integer;
   Deleted:      LongBool;
   BBox:         TPDFRect;
   Handle:       Integer;
   FieldName:    PAnsiChar;
   FieldNameLen: Cardinal;
   BackCS:       Cardinal;
   TextCS:       Cardinal;
   BackColor:    Cardinal;
   BorderColor:  Cardinal;
   TextColor:    Cardinal;
   Checked:      LongBool;
   Parent:       Integer;
   KidCount:     Cardinal;
   Font:         PAnsiChar;
   FontSize:     Double;
   Value:        Pointer;
   UniVal:       LongBool;
   ValLen:       Cardinal;
   ToolTip:      Pointer;
   UniToolTip:   LongBool;
   ToolTipLen:   Cardinal;
end;

type TPDFFieldEx = record
   StructSize:    Cardinal;       // Must be set to sizeof(TPDFFieldEx)
   Deleted:       LongBool;       // If true, the field was marked as deleted by DeleteField()
   BBox:          TPDFRect;       // Bounding box of the field in bottom-up coordinates
   FieldType:     TFieldType;     // Field type
   GroupType:     TFieldType;     // If GroupType != FieldType the field is a terminal field of a field group
   Handle:        Cardinal;       // Field handle
   BackColor:     Cardinal;       // Background color
   BackColorSP:   TExtColorSpace; // Color space of the background color
   BorderColor:   Cardinal;       // Border color
   BorderColorSP: TExtColorSpace; // Color space of the border color
   BorderStyle:   TBorderStyle;   // Border style
   BorderWidth:   Single;         // Border width
   CharSpacing:   Single;         // Text fields only
   Checked:       LongBool;       // Check boxes only
   CheckBoxChar:  Cardinal;       // ZapfDingbats character that is used to display the on state
   DefState:      TCheckBoxState; // Check boxes only
   DefValueA:     PAnsiChar;      // Optional default value
   DefValueW:     PWideChar;      // Optional default value
   IEditFont:     Pointer;        // Pointer to default editing font
   EditFont:      PAnsiChar;      // Postscript name of the editing font
   ExpValCount:   Cardinal;       // Combo and list boxes only. The values can be accessed with GetFieldExpValueEx()
   ExpValueA:     PAnsiChar;      // Check boxes only
   ExpValueW:     PWideChar;      // Check boxes only
   FieldFlags:    TFieldFlags;    // Field flags
   IFieldFont:    Pointer;        // Pointer to the font that is used by the field
   FieldFont:     PAnsiChar;      // Postscript name of the font
   Reserved:      Integer;        // Reserved field to avoid alignment errors
   FontSize:      Double;         // Font size. 0.0 means auto font size
   FieldNameA:    PAnsiChar;      // Note that children of a field group or radio button have no name
   FieldNameW:    PWideChar;      // Note that children of a field group or radio button have no name
   HighlightMode: THighlightMode; // Highlight mode
   IsCalcField:   LongBool;       // If true, the OnCalc event of the field is connected with a JavaScript action
   MapNameA:      PAnsiChar;      // Optional unique mapping name of the field
   MapNameW:      PWideChar;      // If true, MapName is an Unicode string
   MaxLen:        Cardinal;       // Text fields only -> zero means not restricted
   Kids:          Array of Pointer; // Array of child fields -> GetFieldEx2()
   KidCount:      Cardinal;       // Number of fields in the array
   Parent:        Pointer;        // Pointer to parent field or NULL
   PageNum:       Integer;        // Page on which the field is used or -1
   Rotate:        Integer;        // Rotation angle in degrees
   TextAlign:     TTextAlign;     // Text fields only
   TextColor:     Cardinal;       // Text color
   TextColorSP:   TExtColorSpace; // Color space of the field's text
   TextScaling:   Single;         // Text fields only
   ToolTipA:      PAnsiChar;      // Optional tool tip
   ToolTipW:      PWideChar;      // Optional tool tip
   UniqueNameA:   PAnsiChar;      // Optional unique name (NM key)
   UniqueNameW:   PWideChar;      // Optional unique name (NM key)
   ValueA:        PAnsiChar;      // Field value
   ValueW:        PWideChar;      // Field value
   WordSpacing:   Single;         // Text fields only
end;


type TPDFFileSpec = record
   Buffer:      PAnsiChar; // Buffer of an embedded file
   BufSize:     Cardinal;  // Buffer size in bytes.
   Compressed:  LongBool;  // Should be false if Decompress was true in the GetEmbeddedFile() call, otherwise usually true.
                           // DynaPDF decompresses Flate encoded streams only. Other filters can occur but this is very unusual.
   ColItem:     Pointer;   // If != NULL the embedded file contains a collection item with user defined data. This entry
                           // can occur in PDF Collections only (PDF 1.7). See "PDF Collections" in the help file for further
                           // information.
   Name:        PAnsiChar; // Name of the file specification in the name tree. This value is always present.
   NameUnicode: LongBool;  // Is Name in Unicode format?
   FileName:    PAnsiChar; // File name as 7 bit ASCII string.
   IsURL:       LongBool;  // If true, FileName contains a URL.
   UF:          PAnsiChar; // PDF 1.7. Same as FileName but Unicode is allowed.
   UFUnicode:   LongBool;  // Is UF in Unicode format?
   Desc:        PAnsiChar; // Description
   DescUnicode: LongBool;  // Is Desc in Unicode format?
   FileSize:    Cardinal;  // Size of the decompressed stream or zero if not known. Note: this is either the Size key of
                           // the Params dictionary if present or the DL key in the file stream. Whether this value is
                           // correct depends on the file creator! The parameter is definitely correct if the file was
                           // decompressed.
   MIMEType:    PAnsiChar; // MIME media type name as defined in Internet RFC 2046.
   CreateDate:  PAnsiChar; // Creation date as string. See help file "The standard date format".
   ModDate:     PAnsiChar; // Modification date as string. See help file "The standard date format".
   CheckSum:    PAnsiChar; // 16 byte MD5 digest. Note that this is a binary string. It is exactly 16 bytes long if set!
end;

type TPDFFontObj = record
   Ascent:        Single;    // Ascent
   BaseFont:      PAnsiChar; // PostScript Name or Family Name
   CapHeight:     Single;    // Cap height
   Descent:       Single;    // Descent
   Encoding:      PWideChar; // Unicode mapping 0..255 -> not set if a CID font is selected
   FirstChar:     Cardinal;  // First char
   Flags:         Cardinal;  // Font flags -> font descriptor
   FontFamily:    PAnsiChar; // Optional Font Family (Family Name)
   FontFamilyUni: LongBool;  // If true, FontFamily is in Unicode format -> make a type cast to PWideChar in this case
   FontName:      PAnsiChar; // Font name -> font descriptor
   FontType:      TFontType; // If ftType0 the font is a CID font. The Encoding is not set in this case.
   ItalicAngle:   Single;    // Italic angle
   LastChar:      Cardinal;  // Last char
   SpaceWidth:    Single;    // Space width in font units. A default value is set if the font contains no space character.
   Widths:        PSingle;   // Glyph widths -> 0..WidthsCount -1
   WidthsCount:   Cardinal;  // Number of widths in the array
   XHeight:       Single;    // x-height
   DefWidth:      Single;    // Default character widths -> CID fonts only
   FontFile:      PAnsiChar; // Font file buffer -> only imported fonts are returned.
   Length1:       Cardinal;  // Length of the clear text portion of the Type1 font, or the length of the entire font program if FontType != ffType1.
   Length2:       Cardinal;  // Length of the encrypted portion of the Type1 font program (Type1 fonts only).
   Length3:       Cardinal;  // Length of the fixed-content portion of the Type1 font program or zero if not present.
   FontFileType:  TFontFileSubtype; // See description in the help file for further information.
end;

type TPDFImage = record
   Buffer:           PAnsiChar;         // Each scanline is aligned to a full byte.
   BufSize:          Cardinal;          // The size of the image buffer in bytes.
   Filter:           TDecodeFilter;     // Required decode filter if the image is compressed.
                                        // Possible values are dfDCTDecode (JPEG), dfJPXDecode (JPEG2000),
                                        // and dfJBIG2Decode. Other filters are always removed by DynaPDF since
                                        // a conversion to a native file format is then always required.
   OrgFilter:        TDecodeFilter;     // The image was compressed with this filter in the PDF file. This info is
                                        // useful to determine which compression filter should be used when creating
                                        // a new image file from the image buffer.
   JBIG2Globals:     PAnsiChar;         // Optional global page 0 segment (dfJBIG2Decode filter only).
   JBIG2GlobalsSize: Cardinal;          // The size of the bit stream in bytes.
   BitsPerPixel:     Cardinal;          // Bit depth of the image buffer. Possible values are 1, 2, 4, 8, 24, 32, and 64.
   ColorSpace:       TExtColorSpace;    // The color space refers either to the image buffer or to the color table if set.
                                        // Note that 1 bit images can occur with and without a color table.
   NumComponents:    Cardinal;          // The number of components stored in the image buffer.
   MinIsWhite:       LongBool;          // If true, the colors of 1 bit images are reversed.
   IColorSpaceObj:   Pointer;           // Pointer to the original color space.
   ColorTable:       PAnsiChar;         // The color table or NULL.
   ColorCount:       Cardinal;          // The number of colors in the color table.
   Width:            Cardinal;          // Image width in pixel.
   Height:           Cardinal;          // Image height in pixel.
   ScanLineLength:   Cardinal;          // The length of a scanline in bytes.
   InlineImage:      LongBool;          // If true, the image is an inline image.
   Interpolate:      LongBool;          // If true, image interpolation should be performed.
   Transparent:      LongBool;          // The meaning is different depending on the bit depth and whether a color
                                        // table is available. If the image is a 1 bit image and if no color table is available,
                                        // black pixels must be drawn with the current fill color.
                                        // If the image contains a color table ColorMask contains the range of indexes
                                        // in the form min/max index which appears transparent. If no color table is
                                        // present ColorMask contains the transparent ranges in the form min/max for
                                        // each component.
   ColorMask:        PAnsiChar;         // The array contains ranges in the form min/max (2 values per component) for each
                                        // component before decoding.
   IMaskImage:       Pointer;           // If set, a 1 bit image is used as a transparency mask. Call GetImage() to decode the image.
   ISoftMask:        Pointer;           // If set, a grayscale image is used as alpha channel. Call GetImage() to decode the image.
   Decode:           PSingle;           // If set, samples must be decoded. The array contains 2 * NumComponents values.
                                        // The decode array is never set if the image is returned decompressed since
                                        // it is already applied during decompression.
   Intent:           TRenderingIntent;  // Default riNone.
   SMaskInData:      Cardinal;          // JPXDecode only, PDF_MAX_INT if not set. See PDF Reference for further information.
   OCG:              Pointer;           // Pointer to Optional Content Group if any.
   Metadata:         PAnsiChar;         // Optional XML Metadata stream.
   MetadataSize:     Cardinal;          // Length of Metadata in bytes.
   ObjectPtr:        Pointer;           // Internal pointer to the image class.
   ResolutionX:      Single;            // Image resolution on the x-axis.
   ResolutionY:      Single;            // Image resolution on the y-axis.
   // Reserved fields for future extensions
   Reserved1:        Cardinal;
   Reserved2:        Cardinal;
   Reserved3:        Cardinal;
   Reserved4:        Cardinal;
   Reserved5:        Cardinal;
   Reserved6:        Cardinal;
   Reserved7:        Cardinal;
   Reserved8:        Cardinal;
   Reserved9:        Cardinal;
   Reserved10:       Cardinal;
   Reserved11:       Cardinal;
   Reserved12:       Cardinal;
   Reserved13:       Cardinal;
   Reserved14:       Cardinal;
   Reserved15:       Cardinal;
   Reserved16:       Cardinal;
   Reserved17:       Cardinal;
   Reserved18:       Cardinal;
end;

// Named Destination
type TPDFNamedDest = record
   NameA:    PAnsiChar;
   NameW:    PWideChar;
   NameLen:  Cardinal;
   DestPage: Integer;
   DestPos:  TPDFRect;
   DestType: TDestType;
end;

type TPDFPageLabel = record
   StartRange: Cardinal;           // Number of the first page in the range. If no further label follows, the last
                                   // page in the range is pdfGetPageCount(). The first page is denoted by 1.
   Format:       TPageLabelFormat; // Number format to be used.
   FirstPageNum: Integer;          // First page number to be displayed in the page label. Subsequent pages are
                                   // numbered sequentially from this value.
   Prefix:       PAnsiChar;        // Optional prefix
   PrefixLen:    Cardinal;         // Length of the prefix in characters
   PrefixUni:    LongBool;         // If true, the prefix is a Unicode string
end;

type TPDFSigParms = record
   StructSize:   Cardinal;  // Must be set to sizeof(TSignParms)
   PKCS7ObjLen:  Cardinal;  // The maximum length of the signed PKCS#7 object
   HashType:     THashType; // If set to htDetached, the bytes ranges of the PDF file will be returned.
   Range1:       PByte;     // Out -> Contains either the hash or the first byte range to create a detached signature
   Range1Len:    Cardinal;  // Out -> Length of the buffer
   Range2:       PByte;     // Out -> Set only if HashType == htDetached
   Range2Len:    Cardinal;  // Out -> Length of the buffer
   ContactInfoA: PAnsiChar; // Optional, e.g. an email address
   ContactInfoW: PWideChar; // Optional, e.g. an email address
   LocationA:    PAnsiChar; // Optional location of the signer
   LocationW:    PWideChar; // Optional location of the signer
   ReasonA:      PAnsiChar; // Optional reason why the file was signed
   ReasonW:      PWideChar; // Optional reason why the file was signed
   SignerA:      PAnsiChar; // Optional, the issuer of the certificate takes precedence
   SignerW:      PWideChar; // Optional, the issuer of the certificate takes precedence
   Encrypt:      LongBool;  // If true, the file will be encrypted
   // These members will be ignored if Encrypt is set to false
   OpenPwd:      PAnsiChar; // Open password
   OwnerPwd:     PAnsiChar; // Owner password to change the security settings
   KeyLen:       TKeyLen;   // Key length to be used to encrypt the file
   Restrict:     TRestrictions; // What should be restricted?
end;

type TPDFStack = record
   ctm:          TCTM;               // Pre-multiplied global transformation matrix
   tm:           TCTM;               // Pre-multiplied text transformation matrix
   x:            Double;             // Unused -> always 0.0
   y:            Double;             // Unused -> always 0.0
   FontSize:     Double;             // Font size
   CharSP:       Double;             // Character spacing
   WordSP:       Double;             // Word spacing
   HScale:       Double;             // Horizontal text scaling
   TextRise:     Double;             // Text rise -> always 0.0 because it is already included in the text transformation matrix
   Leading:      Double;             // Leading
   LineWidth:    Double;             // Line width
   DrawMode:     TDrawMode;          // Text draw mode
   FillCS:       TPDFColorSpace;     // Fill color space
   StrokeCS:     TPDFColorSpace;     // Stroke color space
   FillColor:    Cardinal;           // Fill color
   StrokeColor:  Cardinal;           // Stroke color
   BaseObject:   Pointer;            // Internal
   CIDFont:      LongBool;           // If true, ReplacePageText() can only be used to delete a string
   Text:         PAnsiChar;          // Raw text without kerning space
   TextLen:      Cardinal;           // Raw text length
   RawKern:      TTextRecordAPtr;    // Raw kerning array
   Kerning:      TTextRecordWPtr;    // Already translated Unicode kerning array
   KerningCount: Cardinal;           // Number of kerning records
   TextWidth:    Single;             // The width of the entire text record measured in text space
   IFont:        Pointer;            // Font object used to print the string -> fntGetFont() can be used to return the font properties
   Embedded:     LongBool;           // If true, the font is embedded
   SpaceWidth:   Single;             // Measured in text space
   {
      These members can be modified after the structure has been initialized with InitStack().
      If the destination color space should be DeviceCMYK initialize FillColor and StrokeColor
      with PDF_CMYK(0,0,0,255); which represents black.
   }
   ConvColors:   LongBool;           // If set to true (default), all colors are converted to the specified destination color space
   DestSpace:    TPDFColorSpace;     // Destination color space -> default == csDeviceRGB

   // This member can be used in combination with ReplacePageText() to preserve a number
   // of kerning records from deletion. All records above this value will be deleted.
   // Take a look into the file examples/util/pdf_edit_text.cpp to determine how this member
   // can be used.
   DeleteKerningAt: Cardinal;
   FontFlags:       Cardinal;    // PDF font flags
   // ------------------------------- Reserved fields -------------------------------
   Reserved1:  Integer;
   Reserved2:  Integer;
   Reserved3:  Integer;
   Reserved4:  Integer;
   Reserved5:  Integer;
   Reserved6:  Integer;
   Reserved7:  Integer;
   Reserved8:  Integer;
   Reserved9:  Integer;
   Reserved10: Integer;
   Reserved11: Integer;
   ContentPtr: Pointer;
end;

type TPDFVersion =
(
  pvPDF_1_0,
  pvPDF_1_1,
  pvPDF_1_2,
  pvPDF_1_3,
  pvPDF_1_4,
  pvPDF_1_5,
  pvPDF_1_6,
  pvPDF_1_7,
  pvPDF_1_8,
  pvPDF_1_9,     // reserved for future use
  pvPDFX1a_2001, // PDF/X-1a:2001
  pvPDFX1a_2003, // PDF/X-1a:2002
  pvPDFX3_2002,  // PDF/X-3:2002
  pvPDFX3_2003,  // PDF/X-3:2003
  pvPDFA_2005    // PDF/A-1b 2005
);

type TRectL = record
   Left: Integer;
   Top: Integer;
   Right: Integer;
   Bottom: Integer;
end;

type TShadingType =
(
   stUnknown, // cannot occur -> internal use
   stFunctionBased,
   stAxial,
   stRadial,
   stFreeFormGouraud,
   stLatticeFormGouraud,
   stCoonsPatch,
   stTensorProduct
);

type TStdPattern =
(
   spHorizontal, { ----- }
   spVertical,   { ||||| }
   spRDiagonal,  { \\\\\ }
   spLDiagonal,  { ///// }
   spCross,      { +++++ }
   spDiaCross    { xxxxx }
);

type TSubmitFlags = Integer;
const sfNone             = $00000000;
const sfExlude           = $00000001; // if set, the fields in a sumbmit or reset form action are excluded
const sfInclNoValFields  = $00000002;
const sfHTML             = $00000004;
const sfGetMethod        = $00000008;
const sfSubmCoords       = $00000010;
const sfXML              = $00000024;
const sfInclAppSaves     = $00000040;
const sfInclAnnots       = $00000080;
const sfPDF              = $00000100;
const sfCanonicalFormat  = $00000200;
const sfExlNonUserAnnots = $00000400;
const sfExlFKey          = $00000800;
const sfEmbedForm        = $00002000; // PDF 1.5 embed the entire form into a file stream inside the FDF file -> requires the full version of Adobe's Acrobat

type TViewerPreference = Integer;
const
  vpUseNone             = $00000000;
  vpHideToolBar         = $00000001;
  vpHideMenuBar         = $00000002;
  vpHideWindowUI        = $00000004;
  vpFitWindow           = $00000008;
  vpCenterWindow        = $00000010;
  vpDisplayDocTitle     = $00000020;
  vpNonFullScrPageMode  = $00000040;
  vpDirection           = $00000080;
  vpViewArea            = $00000100;
  vpViewClip            = $00000200;
  vpPrintArea           = $00000400;
  vpPrintClip           = $00000800;

type TViewPrefAddVal = Integer;
const
  avNone                   = $00000000;
  avNonFullScrUseNone      = $00000001;
  avNonFullScrUseOutlines  = $00000002;
  avNonFullScrUseThumbs    = $00000004;
  avNonFullScrUseOC        = $00000400; // PDF 1.6
  avDirectionL2R           = $00000008;
  avDirectionR2L           = $00000010;
  avViewPrintArtBox        = $00000020;
  avViewPrintBleedBox      = $00000040;
  avViewPrintCropBox       = $00000080;
  avViewPrintMediaBox      = $00000100;
  avViewPrintTrimBox       = $00000200;

  // Use these masks to determine which value is defined.
  AV_NON_FULL_SRC_MASK     = $00000005;
  AV_DIRECTION_MASK        = $00000018;
  AV_VIEW_PRINT_MASK       = $000003E0;

{ --------------------------------------------- Callback functions ---------------------------------------------- }

// Data is always a user defined pointer which is passed unchanged to the callback funtion
type TErrorProc      = function(const Data: Pointer; ErrCode: Integer; const ErrMessage: PAnsiChar; ErrType: Integer): Integer; stdcall;
type TEnumFontProc   = function(const Data: Pointer; const FamilyName: PWideChar; const PostScriptName: PAnsiChar; Style: TFStyle): Integer; stdcall;
type TEnumFontProc2  = function(const Data, PDFFont: Pointer; FontType: TFontType; const BaseFont, FontName: PAnsiChar; Embedded, IsFormFont: LongBool; Flags: Cardinal): Integer; stdcall;
type TEnumFontProcEx = function(const Data: Pointer; const FamilyName: PWideChar; const PostScriptName: PAnsiChar; Style: TFStyle; BaseType: TFontBaseType; Embeddable: LongBool; const FilePath: PAnsiChar): Integer; stdcall;

{ ---------------------------------- Callback functions for CheckConformance() ---------------------------------- }

type TOnFontNotFoundProc  = function(const Data, PDFFont: Pointer; const FontName: PAnsiChar; Style: TFStyle; StdFontIndex: Integer; IsSymbolFont: LongBool): Integer; stdcall;
type TOnReplaceICCProfile = function(const Data: Pointer; ProfileType: TICCProfileType; ColorSpace: Integer): Integer; stdcall;

{ ------------------------------------- Callback function for WriteFText ---------------------------------------- }

// The TOnPageBreakProc function is called by WriteFText when the rectangle was filled with text, or when a
// manual page break was inserted to the text and more text is available for output.
// You must set the text rectangle with SetTextRect to continue the text on a new position or simply call
// EndPage() and then Append() or EditPage() to continue the text on an arbitrary new page. You can make some
// changes inside the callback function such as setting another font, font color or other thinks.

// When the callback function returns a value greater or equal zero, processing continues on the new
// position or page that was set inside the callback function.

// The text alignment can also be changed with the following return values:

const NEW_ALIGN_LEFT    = 1;
const NEW_ALIGN_RIGHT   = 2;
const NEW_ALIGN_CENTER  = 3;
const NEW_ALIGN_JUSTIFY = 4;

type TOnPageBreakProc = function(const Data: Pointer; LastPosX, LastPosY: Double; PageBreak: LongBool): Integer; cdecl;

// When the return value of the callback function is one of the values listed above, then the alignment will
// be changed before the remaining text is processed. A return value of 0 determines that processing should
// be continued without changing the alignment.

// A negative return value breaks processing and WriteFText returns emmidiatly with a warning. Note that
// processing stops entirely when warnings should be handled as fatal errors (see SetErrorMode() in the help
// file for further information).

// NOTE: The coordinate system is always set to bottom up when the callback function is called, independent
// of the coordinate system that was set before WriteFText was called. WriteFText restores the coordinate system
// and all other temporarily changed variables when the function is leaved. The last text text position is
// also returned in bottom up coordinates. LastPosY is the position of the last baseline.

// You can change the coordinate system inside the callback function when needed, DynaPDF checks the state
// before processing continues.

{ ------------------------------- Callback functions to control a progress bar ---------------------------------- }

type TProgType = Integer;
const ptImportPage = 0;
const ptWritePage  = 1;

// TInitProgress is called before TProgress is called the first time
type TInitProgress = function(const Data: Pointer; ProgType: TProgType; MaxCount: Integer): Integer; stdcall;
type TProgress     = function(const Data: Pointer; ActivePage: Integer): Integer; stdcall;

{------------------------------------------- Parser Interface ---------------------------------------------------}

type TApplyPattern        = function(const Data, PDFObject: Pointer; PatternType: TPatternType; const Pattern: Pointer): Integer; stdcall;
type TBeginPattern        = function(const Data: Pointer; Fill: LongBool; Handle: Integer; PatternType: TPatternType; var BBox: TPDFRect; Matrix: PCTM; XStep, YStep: PDouble): Integer; stdcall;
type TBeginTemplate       = function(const Data, PDFObject: Pointer; Handle: Integer; var BBox: TPDFRect; Matrix: PCTM): Integer; stdcall;
type TBezierTo1           = function(const Data, PDFObject: Pointer; x1, y1, x3, y3: Double): Integer; stdcall;
type TBezierTo2           = function(const Data, PDFObject: Pointer; x2, y2, x3, y3: Double): Integer; stdcall;
type TBezierTo3           = function(const Data, PDFObject: Pointer; x1, y1, x2, y2, x3, y3: Double): Integer; stdcall;
type TClipPath            = function(const Data, PDFObject: Pointer; EvenOdd: LongBool; Mode: TPathFillMode): Integer; stdcall;
type TClosePath           = function(const Data, PDFObject: Pointer; Mode: TPathFillMode): Integer; stdcall;
type TDrawShading         = function(const Data, PDFObject: Pointer; ShadType: TShadingType; const Shading: Pointer): Integer; stdcall;
type TEndPattern          = procedure(const Data: Pointer); stdcall;
type TEndTemplate         = procedure(const Data: Pointer); stdcall;
type TInsertImage         = function(const Data: Pointer; var Image: TPDFImage): Integer; stdcall;
type TLineTo              = function(const Data, PDFObject: Pointer; x, y: Double): Integer; stdcall;
type TMoveTo              = function(const Data, PDFObject: Pointer; x, y: Double): Integer; stdcall;
type TMulMatrix           = procedure(const Data, PDFObject: Pointer; var Matrix: TCTM); stdcall;
type TRectangle           = function(const Data, PDFObject: Pointer; x, y, w, h: Double): Integer; stdcall;
type TRestoreGraphicState = function(const Data: Pointer): Integer; stdcall;
type TSaveGraphicState    = function(const Data: Pointer): Integer; stdcall;
type TSetCharSpacing      = procedure(const Data, PDFObject: Pointer; Value: Double); stdcall;
type TSetExtGState        = procedure(const Data, PDFObject: Pointer; var GS: TPDFExtGState2); stdcall;
type TSetFillColor        = procedure(const Data, PDFObject: Pointer; Color: PDouble; NumComps: Cardinal; CS: TExtColorSpace; const IColorSpace: Pointer); stdcall;
type TSetFont             = procedure(const Data, PDFObject: Pointer; FontType: TFontType; Embedded: LongBool; const FontName: PAnsiChar; Style: TFStyle; FontSize: Double; const Font: Pointer); stdcall;
type TSetLeading          = procedure(const Data, PDFObject: Pointer; Value: Double); stdcall;
type TSetLineCapStyle     = procedure(const Data, PDFObject: Pointer; Style: TLineCapStyle); stdcall;
type TSetLineDashPattern  = procedure(const Data, PDFObject: Pointer; Dash: PDouble; NumValues, Phase: Cardinal); stdcall;
type TSetLineJoinStyle    = procedure(const Data, PDFObject: Pointer; Style: TLineJoinStyle); stdcall;
type TSetLineWidth        = procedure(const Data, PDFObject: Pointer; Value: Double); stdcall;
type TSetMiterLimit       = procedure(const Data, PDFObject: Pointer; Value: Double); stdcall;
type TSetStrokeColor      = procedure(const Data, PDFObject: Pointer; Color: PDouble; NumComps: Cardinal; CS: TExtColorSpace; const IColorSpace: Pointer); stdcall;
type TSetTextDrawMode     = procedure(const Data, PDFObject: Pointer; Mode: TDrawMode); stdcall;
type TSetTextScale        = procedure(const Data, PDFObject: Pointer; Value: Double); stdcall;
type TSetWordSpacing      = procedure(const Data, PDFObject: Pointer; Value: Double); stdcall;
type TShowTextArrayA      = function(const Data: Pointer; Obj: PAnsiChar; var Matrix: TCTM; const Source: TTextRecordAPtr; Count: Cardinal; Width: Double): Integer; stdcall;
type TShowTextArrayW      = function(const Data: Pointer; const Source: TTextRecordAPtr; var Matrix: TCTM; const Kerning: TTextRecordWPtr; Count: Cardinal; Width: Double; Decoded: LongBool): Integer; stdcall;

// All callback functions are optional. Set unnecessary callback functions to nil.
type TPDFParseInterface = record
   ApplyPattern:        TApplyPattern;
   BeginPattern:        TBeginPattern;
   BeginTemplate:       TBeginTemplate;
   BezierTo1:           TBezierTo1;
   BezierTo2:           TBezierTo2;
   BezierTo3:           TBezierTo3;
   ClipPath:            TClipPath;
   ClosePath:           TClosePath;
   DrawShading:         TDrawShading;
   EndPattern:          TEndPattern;
   EndTemplate:         TEndTemplate;
   LineTo:              TLineTo;
   MoveTo:              TMoveTo;
   MulMatrix:           TMulMatrix;
   Rectangle:           TRectangle;
   RestoreGraphicState: TRestoreGraphicState;
   SaveGraphicState:    TSaveGraphicState;
   SetCharSpacing:      TSetCharSpacing;
   SetExtGState:        TSetExtGState;
   SetFillColor:        TSetFillColor;
   SetFont:             TSetFont;
   SetLeading:          TSetLeading;
   SetLineCapStyle:     TSetLineCapStyle;
   SetLineDashPattern:  TSetLineDashPattern;
   SetLineJoinStyle:    TSetLineJoinStyle;
   SetLineWidth:        TSetLineWidth;
   SetMiterLimit:       TSetMiterLimit;
   SetStrokeColor:      TSetStrokeColor;
   SetTextDrawMode:     TSetTextDrawMode;
   SetTextScale:        TSetTextScale;
   SetWordSpacing:      TSetWordSpacing;
   Reserved001:         Pointer; // The old callabck functions TShowTextA and TShowTextW are no longer defined
   Reserved002:         Pointer; // The old callabck functions TShowTextA and TShowTextW are no longer defined
   ShowTextArrayW:      TShowTextArrayW;
   InsertImage:         TInsertImage;
   ShowTextArrayA:      TShowTextArrayA;
   { These variables must be initialized with nil! }
   Reserved01: Pointer;
   Reserved02: Pointer;
   Reserved03: Pointer;
   Reserved04: Pointer;
   Reserved05: Pointer;
   Reserved06: Pointer;
   Reserved07: Pointer;
   Reserved08: Pointer;
   Reserved09: Pointer;
   Reserved10: Pointer;
   Reserved11: Pointer;
   Reserved12: Pointer;
   Reserved13: Pointer;
   Reserved14: Pointer;
   Reserved15: Pointer;
   Reserved16: Pointer;
   Reserved17: Pointer;
   Reserved18: Pointer;
   Reserved19: Pointer;
   Reserved20: Pointer;
   Reserved21: Pointer;
   Reserved22: Pointer;
   Reserved23: Pointer;
   Reserved24: Pointer;
   Reserved25: Pointer;
   Reserved26: Pointer;
   Reserved27: Pointer;
end;

{------------------------------------------ Function Prototypes --------------------------------------------------}

type
   { --------------------------------------------- Font API ----------------------------------------------------- }
   TfntBuildFamilyNameAndStyle = function(const IFont: Pointer; Name: PAnsiChar {Array[0..127] of AnsiChar}; var Style: Integer): LongBool; stdcall;
   TfntGetFont                 = function(const IFont: Pointer; var F: TPDFFontObj): LongBool; stdcall;
   TfntGetSpaceWidth           = function(const IFont: Pointer; FontSize: Double): Double; stdcall;
   TfntGetTextWidth            = function(const IFont: Pointer; const Text: PAnsiChar; Len: Cardinal; CharSpacing, WordSpacing, TextScale: Single): Double; stdcall;
   TfntTranslateRawCode        = function(const IFont: Pointer; const Text: PAnsiChar; Len: Cardinal; var Width: Double; OutText: PWideChar {Must be 32 characters long!}; var OutLen: Integer; var Decoded: LongBool; CharSpacing, WordSpacing, TextScale: Single): Cardinal; stdcall;
   TfntTranslateString         = function(var Stack: TPDFStack; Buffer: PWideChar; Size, Flags: Cardinal): Integer; stdcall;
   TfntTranslateString2        = function(const IFont: Pointer; const Text: PAnsiChar; Len: Cardinal; OutBuf: PWideChar; Size, Flags: Cardinal): Integer; stdcall;

   { ------------------------------------------- DynaPDF API ---------------------------------------------------- }
   TpdfAddActionToObj = function(const IPDF: Pointer; ObjType, Event: Integer; ActHandle, ObjHandle: Cardinal): Integer; stdcall;
   TpdfAddArticle = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double): Integer; stdcall;
   TpdfAddBookmarkA = function(const IPDF: Pointer; const Title: PAnsiChar; Parent: Integer; DestPage: Cardinal; Open: Integer): Integer; stdcall;
   TpdfAddBookmarkW = function(const IPDF: Pointer; const Title: PWideChar; Parent: Integer; DestPage: Cardinal; Open: Integer): Integer; stdcall;
   TpdfAddBookmarkExA = function(const IPDF: Pointer; const Title: PAnsiChar; Parent: Integer; NamedDest: Cardinal; Open: LongBool): Integer; stdcall;
   TpdfAddBookmarkExW = function(const IPDF: Pointer; const Title: PWideChar; Parent: Integer; NamedDest: Cardinal; Open: LongBool): Integer; stdcall;
   TpdfAddButtonImage = function(const IPDF: Pointer; BtnHandle: Cardinal; State: Integer; const Caption, ImgFile: PAnsiChar): LongBool; stdcall;
   TpdfAddButtonImageEx = function(const IPDF: Pointer; BtnHandle: Cardinal; State: Integer; const Caption: PAnsiChar; Handle: Pointer): LongBool; stdcall;
   TpdfAddContinueTextA = function(const IPDF: Pointer; const AText: PAnsiChar): Integer; stdcall;
   TpdfAddContinueTextW = function(const IPDF: Pointer; const AText: PWideChar): Integer; stdcall;
   TpdfAddDeviceNProcessColorants = function(const IPDF: Pointer; DeviceNCS: Cardinal; const Colorants: PPchar; NumColorants: Cardinal; ProcessCS: TExtColorSpace; Handle: Integer): LongBool; stdcall;
   TpdfAddDeviceNSeparations = function(const IPDF: Pointer; DeviceNCS: Cardinal; const Colorants: PPChar; SeparationCS: PInteger; NumColorants: Cardinal): LongBool; stdcall;
   TpdfAddFieldToFormAction = function(const IPDF: Pointer; Action, AField: Cardinal; Include: LongBool): Integer; stdcall;
   TpdfAddFieldToHideAction = function(const IPDF: Pointer; HideAct, AField: Cardinal): Integer; stdcall;
   TpdfAddFileCommentA = function(const IPDF: Pointer; const AText: PAnsiChar): LongBool; stdcall;
   TpdfAddFileCommentW = function(const IPDF: Pointer; const AText: PWideChar): LongBool; stdcall;
   TpdfAddFontSearchPath = function(const IPDF: Pointer; const APath: PAnsiChar; Recursive: Integer): Integer; stdcall;
   TpdfAddImage = function(const IPDF: Pointer; Filter: TCompressionFilter; Flags: Cardinal; var Image: TPDFImage): LongBool; stdcall;
   TpdfAddJavaScriptA = function(const IPDF: Pointer; const Name, Script: PAnsiChar): Integer; stdcall;
   TpdfAddJavaScriptW = function(const IPDF: Pointer; const Name: PAnsiChar; const Script: PWideChar): Integer; stdcall;
   TpdfAddObjectToLayer = function(const IPDF: Pointer; OCG: Cardinal; ObjType: TOCObject; Handle: Cardinal): LongBool; stdcall;
   TpdfAddPageLabelA = function(const IPDF: Pointer; StartRange: Cardinal; Format: TPageLabelFormat; const Prefix: PAnsiChar; AddNum: Integer): Integer; stdcall;
   TpdfAddPageLabelW = function(const IPDF: Pointer; StartRange: Cardinal; Format: TPageLabelFormat; const Prefix: PWideChar; AddNum: Integer): Integer; stdcall;
   TpdfAddRenderingIntentA = function(const IPDF: Pointer; const ICCFile: PAnsiChar): Integer; stdcall;
   TpdfAddRenderingIntentW = function(const IPDF: Pointer; const ICCFile: PWideChar): Integer; stdcall;
   TpdfAddRenderingIntentEx = function(const IPDF, Buffer: Pointer; BufSize: Cardinal): Integer; stdcall;
   TpdfAddValToChoiceFieldA = function(const IPDF: Pointer; Field: Cardinal; const ExpValue, Value: PAnsiChar; Selected: Integer): LongBool; stdcall;
   TpdfAddValToChoiceFieldW = function(const IPDF: Pointer; Field: Cardinal; const ExpValue: PAnsiChar; const Value: PWideChar; Selected: Integer): LongBool; stdcall;
   TpdfAppend = function(IPDF: Pointer): LongBool; stdcall;
   TpdfApplyPattern = function(const IPDF: Pointer; PattHandle: Integer; ColorMode: TColorMode; Color: Cardinal): LongBool; stdcall;
   TpdfApplyShading = function(const IPDF: Pointer; ShadHandle: Integer): LongBool; stdcall;
   TpdfAttachFileA = function(const IPDF: Pointer; const FilePath, Description: PAnsiChar; Compress: LongBool): Integer; stdcall;
   TpdfAttachFileW = function(const IPDF: Pointer; const FilePath, Description: PWideChar; Compress: LongBool): Integer; stdcall;
   TpdfAttachFileExA = function(const IPDF, Buffer: Pointer; BufSize: Cardinal; const FileName, Description: PAnsiChar; Compress: LongBool): Integer; stdcall;
   TpdfAttachFileExW = function(const IPDF, Buffer: Pointer; BufSize: Cardinal; const FileName, Description: PWideChar; Compress: LongBool): Integer; stdcall;
   TpdfAutoTemplate = function(const IPDF: Pointer; Templ: Cardinal; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfBeginClipPath = function(IPDF: Pointer): LongBool; stdcall;
   TpdfBeginContinueText = function(const IPDF: Pointer; PosX, PosY: Double): Integer; stdcall;
   TpdfBeginLayer = function(const IPDF: Pointer; OCG: Cardinal): LongBool; stdcall;
   TpdfBeginPattern = function(const IPDF: Pointer; PatternType: TPatternType; TilingType: TTilingType; Width, Height: Double): Integer; stdcall;
   TpdfBeginTemplate = function(const IPDF: Pointer; Width, Height: Double): Integer; stdcall;
   TpdfBeginTemplateEx = function(const IPDF: Pointer; var BBox: TPDFRect; var Matrix: TCTM): Integer; stdcall;
   TpdfBezier_1_2_3 = function(const IPDF: Pointer; x1, y1, x2, y2, x3, y3: Double): LongBool; stdcall;
   TpdfBezier_1_3 = function(const IPDF: Pointer; x1, y1, x3, y3: Double): LongBool; stdcall;
   TpdfBezier_2_3 = function(const IPDF: Pointer; x2, y2, x3, y3: Double): LongBool; stdcall;
   TpdfCalcWidthHeight = function(const IPDF: Pointer; OrgWidth, OrgHeight, ScaledWidth, ScaledHeight: Double): Double; stdcall;
   TpdfChangeAnnotNameA = function(const IPDF: Pointer; Handle: Cardinal; const Name: PAnsiChar): LongBool; stdcall;
   TpdfChangeAnnotNameW = function(const IPDF: Pointer; Handle: Cardinal; const Name: PWideChar): LongBool; stdcall;
   TpdfChangeAnnotPos = function(const IPDF: Pointer; Handle: Cardinal; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfChangeBookmarkA = function(const IPDF: Pointer; ABmk: Integer; const Title: PAnsiChar; Parent: Integer; DestPage: Cardinal; Open: Integer): LongBool; stdcall;
   TpdfChangeBookmarkW = function(const IPDF: Pointer; ABmk: Integer; const Title: PWideChar; Parent: Integer; DestPage: Cardinal; Open: Integer): LongBool; stdcall;
   TpdfChangeFont = function(const IPDF: Pointer; Handle: Integer): LongBool; stdcall;
   TpdfChangeFontSize = function(const IPDF: Pointer; Size: Double): LongBool; stdcall;
   TpdfChangeFontStyle = function(const IPDF: Pointer; Style: Cardinal): LongBool; stdcall;
   TpdfChangeFontStyleEx = function(const IPDF: Pointer; Style: Cardinal): LongBool; stdcall;
   TpdfChangeJavaScriptA = function(const IPDF: Pointer; Handle: Cardinal; const NewScript: PAnsiChar): LongBool; stdcall;
   TpdfChangeJavaScriptW = function(const IPDF: Pointer; Handle: Cardinal; const NewScript: PWideChar): LongBool; stdcall;
   TpdfChangeJavaScriptActionA = function(const IPDF: Pointer; Handle: Cardinal; const NewScript: PAnsiChar): LongBool; stdcall;
   TpdfChangeJavaScriptActionW = function(const IPDF: Pointer; Handle: Cardinal; const NewScript: PWideChar): LongBool; stdcall;
   TpdfChangeJavaScriptNameA = function(const IPDF: Pointer; Handle: Cardinal; const Name: PAnsiChar): LongBool; stdcall;
   TpdfChangeJavaScriptNameW = function(const IPDF: Pointer; Handle: Cardinal; const Name: PWideChar): LongBool; stdcall;
   TpdfChangeLinkAnnot = function(const IPDF: Pointer; Handle: Cardinal; const URL: PAnsiChar): LongBool; stdcall;
   TpdfCheckCollection = function(const IPDF: Pointer): LongBool; stdcall;
   TpdfCheckConformance = function(const IPDF: Pointer; ConfType: Integer; Options: Cardinal; const UserData: Pointer; OnFontNotFound: TOnFontNotFoundProc; OnReplaceICCProfile: TOnReplaceICCProfile): Integer; stdcall;
   TpdfCheckFieldNames = function(IPDF: Pointer): Integer; stdcall;
   TpdfClearAutoTemplates = function(IPDF: Pointer): LongBool; stdcall;
   TpdfClearHostFonts = function(IPDF: Pointer): LongBool; stdcall;
   TpdfClipPath = function(const IPDF: Pointer; ClipMode: TClippingMode; FillMode: TPathFillMode): LongBool; stdcall;
   TpdfCloseAndSignFile = function(const IPDF: Pointer; const CertFile, Password, Reason, Location: PAnsiChar): LongBool; stdcall;
   TpdfCloseAndSignFileEx = function(const IPDF: Pointer; const OpenPwd, OwnerPwd: PAnsiChar; KeyLen, Restrict: Integer; const CertFile, Password, Reason, Location: PAnsiChar): LongBool; stdcall;
   TpdfCloseAndSignFileExt = function(const IPDF: Pointer; var SigParms: TPDFSigParms): LongBool; stdcall;
   TpdfCloseFile = function(IPDF: Pointer): LongBool; stdcall;
   TpdfCloseFileEx = function(const IPDF: Pointer; const OpenPwd, OwnerPwd: PAnsiChar; KeyLen, Restrict: Integer): LongBool; stdcall;
   TpdfCloseImage = function(const IPDF: Pointer): LongBool; stdcall;
   TpdfCloseImportFile = function(IPDF: Pointer): LongBool; stdcall;
   TpdfClosePath = function(const IPDF: Pointer; FillMode: TPathFillMode): LongBool; stdcall;
   TpdfCloseTag = function(IPDF: Pointer): LongBool; stdcall;
   TpdfComputeBBox = function(const IPDF: Pointer; var BBox: TPDFRect; Flags: TCompBBoxFlags): Integer; stdcall;
   TpdfConvColor = function(const Color: PDouble; NumComps: Cardinal; SourceCS: Integer; const IColorSpace: Pointer; DestCS: Integer): Cardinal; stdcall;
   TpdfConvertColors = function(const IPDF: Pointer; Flags: TColorConvFlags; Reserved: Pointer): Integer; stdcall;
   TpdfConvToUnicode = function(const IPDF: Pointer; const AString: PAnsiChar; CP: Integer): PWideChar; stdcall;
   TpdfCreate3DAnnot = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; const Author, AnnotName, U3DFile, ImageFile: PAnsiChar): Integer; stdcall;
   TpdfCreate3DBackground = function(const IPDF, IView: Pointer; BackColor: Cardinal): LongBool; stdcall;
   TpdfCreate3DGotoViewAction = function(const IPDF: Pointer; Base3DAnnot: Cardinal; const IView: Pointer; Named: Integer): Integer; stdcall;
   TpdfCreate3DProjection = function(const IPDF, IView: Pointer; ProjType, ScaleType: Integer; Diameter, FOV: Double): LongBool; stdcall;
   TpdfCreate3DView = function(const IPDF: Pointer; Base3DAnnot: Cardinal; const Name: PAnsiChar; SetAsDefault: LongBool; const Matrix: PDouble; CamDistance: Double; RM, LS: Integer): Pointer; stdcall;
   TpdfCreateAnnotAP = function(const IPDF: Pointer; Annot: Cardinal): Integer; stdcall;
   TpdfCreateArticleThreadA = function(const IPDF: Pointer; const ThreadName: PAnsiChar): Integer; stdcall;
   TpdfCreateArticleThreadW = function(const IPDF: Pointer; const ThreadName: PWideChar): Integer; stdcall;
   TpdfCreateAxialShading = function(const IPDF: Pointer; sX, sY, eX, eY, SCenter: Double; SColor, EColor: Cardinal; Extend1, Extend2: Integer): Integer; stdcall;
   TpdfCreateButtonA = function(const IPDF: Pointer; const Name, Caption: PAnsiChar; Parent: Integer; PosX, PosY, Width, Height: Double): Integer; stdcall;
   TpdfCreateButtonW = function(const IPDF: Pointer; const Name: PAnsiChar; const Caption: PWideChar; Parent: Integer; PosX, PosY, Width, Height: Double): Integer; stdcall;
   TpdfCreateCheckBox = function(const IPDF: Pointer; const Name, ExpValue: PAnsiChar; Checked, Parent: Integer; PosX, PosY, Width, Height: Double): Integer; stdcall;
   TpdfCreateCIEColorSpace = function(const IPDF: Pointer; Base: TExtColorSpace; WhitePoint, BlackPoint, Gamma, Matrix: PSingle): Integer; stdcall;
   TpdfCreateCollection = function(const IPDF: Pointer; View: Integer): LongBool; stdcall;
   TpdfCreateCollectionFieldA = function(const IPDF: Pointer; ColType, Column: Integer; const Name, Key: PAnsiChar; Visible, Editable: LongBool): Integer; stdcall;
   TpdfCreateCollectionFieldW = function(const IPDF: Pointer; ColType, Column: Integer; const Name: PWideChar; const Key: PAnsiChar; Visible, Editable: LongBool): Integer; stdcall;
   TpdfCreateColItemDate = function(const IPDF: Pointer; EmbFile: Cardinal; const Key: PAnsiChar; Date: LongInt; const Prefix: PAnsiChar): LongBool; stdcall;
   TpdfCreateColItemNumber = function(const IPDF: Pointer; EmbFile: Cardinal; const Key: PAnsiChar; Value: Double; const Prefix: PAnsiChar): LongBool; stdcall;
   TpdfCreateColItemStringA = function(const IPDF: Pointer; EmbFile: Cardinal; const Key, Value, Prefix: PAnsiChar): LongBool; stdcall;
   TpdfCreateColItemStringW = function(const IPDF: Pointer; EmbFile: Cardinal; const Key: PAnsiChar; const Value, Prefix: PWideChar): LongBool; stdcall;
   TpdfCreateComboBox = function(const IPDF: Pointer; const Name: PAnsiChar; Sort, Parent: Integer; PosX, PosY, Width, Height: Double): Integer; stdcall;
   TpdfCreateDeviceNColorSpace = function(const IPDF: Pointer; const Colorants: PPChar; NumColorants: Cardinal; const PostScriptFunc: PAnsiChar; Alternate: TExtColorSpace; Handle: Integer): Integer; stdcall;
   TpdfCreateExtGState = function(const IPDF: Pointer; var GS: TPDFExtGState): Integer; stdcall;
   TpdfCreateGoToAction = function(const IPDF: Pointer; DestType: TDestType; PageNum: Cardinal; a, b, c, d: Double): Integer; stdcall;
   TpdfCreateGoToActionEx = function(const IPDF: Pointer; NamedDest: Cardinal): Integer; stdcall;
   TpdfCreateGoToRAction = function(const IPDF: Pointer; const FileName: PAnsiChar; PageNum: Cardinal): Integer; stdcall;
   TpdfCreateGoToRActionExA = function(const IPDF: Pointer; const FileName: PAnsiChar; DestName: PAnsiChar; NewWindow: LongBool): Integer; stdcall;
   TpdfCreateGoToRActionExW = function(const IPDF: Pointer; const FileName: PAnsiChar; DestName: PWideChar; NewWindow: LongBool): Integer; stdcall;
   TpdfCreateGroupField = function(const IPDF: Pointer; const Name: PAnsiChar; Parent: Integer): Integer; stdcall;
   TpdfCreateHideAction = function(const IPDF: Pointer; AField: Cardinal; Hide: LongBool): Integer; stdcall;
   TpdfCreateICCBasedColorSpaceA = function(const IPDF: Pointer; const ICCProfile: PAnsiChar): Integer; stdcall;
   TpdfCreateICCBasedColorSpaceW = function(const IPDF: Pointer; const ICCProfile: PWideChar): Integer; stdcall;
   TpdfCreateImageA = function(const IPDF: Pointer; const FileName: PAnsiChar; Format: TImageFormat): LongBool; stdcall;
   TpdfCreateImageW = function(const IPDF: Pointer; const FileName: PWideChar; Format: TImageFormat): LongBool; stdcall;
   TpdfCreateImpDataActionA = function(const IPDF: Pointer; const DataFile: PAnsiChar): Integer; stdcall;
   TpdfCreateImpDataActionW = function(const IPDF: Pointer; const DataFile: PWideChar): Integer; stdcall;
   TpdfCreateIndexedColorSpace = function(const IPDF: Pointer; Base, Handle: Integer; const ColorTable: Pointer; NumColors: Cardinal): Integer; stdcall;
   TpdfCreateJSActionA = function(const IPDF: Pointer; const Script: PAnsiChar): Integer; stdcall;
   TpdfCreateJSActionW = function(const IPDF: Pointer; const Script: PWideChar): Integer; stdcall;
   TpdfCreateLaunchAction = function(const IPDF: Pointer; OP: Integer; const FileName, DefDir, Param: PAnsiChar; NewWindow: LongBool): Integer; stdcall;
   TpdfCreateListBox = function(const IPDF: Pointer; const Name: PAnsiChar; Sort: LongBool; Parent: Integer; PosX, PosY, Width, Height: Double): Integer; stdcall;
   TpdfCreateNamedAction = function(const IPDF: Pointer; Action: Integer): Integer; stdcall;
   TpdfCreateNamedDestA = function(const IPDF: Pointer; const Name: PAnsiChar; DestPage: Cardinal; DestType: TDestType; a, b, c, d: Double): Integer; stdcall;
   TpdfCreateNamedDestW = function(const IPDF: Pointer; const Name: PWideChar; DestPage: Cardinal; DestType: TDestType; a, b, c, d: Double): Integer; stdcall;
   TpdfCreateNewPDFA = function(const IPDF: Pointer; const OutPDF: PAnsiChar): LongBool; stdcall;
   TpdfCreateNewPDFW = function(const IPDF: Pointer; const OutPDF: PWideChar): LongBool; stdcall;
   TpdfCreateOCGA = function(const IPDF: Pointer; const Name: PAnsiChar; DisplayInUI, Visible: LongBool; Intent: TOCGIntent): Integer; stdcall;
   TpdfCreateOCGW = function(const IPDF: Pointer; const Name: PWideChar; DisplayInUI, Visible: LongBool; Intent: TOCGIntent): Integer; stdcall;
   TpdfCreateOCMD = function(const IPDF: Pointer; Visibility: TOCVisibility; const OCGs: PCardinal; Count: Cardinal): Integer; stdcall;
   TpdfCreateRadialShading = function(const IPDF: Pointer; sX, sY, R1, eX, eY, R2, SCenter: Double; SColor, EColor: Cardinal; Extend1, Extend2: Integer): Integer; stdcall;
   TpdfCreateRadioButton = function(const IPDF: Pointer; const Name, ExpValue: PAnsiChar; Checked, Parent: Integer; PosX, PosY, Width, Height: Double): Integer; stdcall;
   TpdfCreateResetAction = function(IPDF: Pointer): Integer; stdcall;
   TpdfCreateSeparationCS = function(const IPDF: Pointer; const Colorant: PAnsiChar; Alternate, Handle: Integer; Color: Cardinal): Integer; stdcall;
   TpdfCreateSetOCGStateAction = function(const IPDF: Pointer; const On_: PCardinal; OnCount: Cardinal; const Off: PCardinal; OffCount: Cardinal; const Toggle: PCardinal; ToggleCount: Cardinal; PreserveRB: LongBool): Integer; stdcall;
   TpdfCreateSigField = function(const IPDF: Pointer; const Name: PAnsiChar; Parent: Integer; PosX, PosY, Width, Height: Double): Integer; stdcall;
   TpdfCreateSigFieldAP = function(const IPDF: Pointer; SigField: Cardinal): Integer; stdcall;
   TpdfCreateStdPattern = function(const IPDF: Pointer; Pattern: Integer; LineWidth, Distance: Double; LineColor, BackColor: Cardinal): Integer; stdcall;
   TpdfCreateStructureTree = function(IPDF: Pointer): LongBool; stdcall;
   TpdfCreateSubmitAction = function(const IPDF: Pointer; Flags: Integer; const URL: PAnsiChar): Integer; stdcall;
   TpdfCreateTextField = function(const IPDF: Pointer; const Name: PAnsiChar; Parent, Multiline, MaxLen: Integer; PosX, PosY, Width, Height: Double): Integer; stdcall;
   TpdfCreateURIAction = function(const IPDF: Pointer; const URL: PAnsiChar): Integer; stdcall;
   TpdfDecryptPDFA = function(const IPDF: Pointer; const FileName: PAnsiChar; PwdType: Integer; const Password: PAnsiChar): Integer; stdcall;
   TpdfDecryptPDFW = function(const IPDF: Pointer; const FileName: PWideChar; PwdType: Integer; const Password: PAnsiChar): Integer; stdcall;
   TpdfDeleteAcroForm = procedure(const IPDF: Pointer); stdcall;
   TpdfDeleteActionFromObj = function(const IPDF: Pointer; ObjType: Integer; ActHandle, ObjHandle: Cardinal): Integer; stdcall;
   TpdfDeleteActionFromObjEx = function(const IPDF: Pointer; ObjType: Integer; ObjHandle, ActIndex: Cardinal): Integer; stdcall;
   TpdfDeleteAnnotation = function(const IPDF: Pointer; Handle: Cardinal): LongBool; stdcall;
   TpdfDeleteBookmark = function(const IPDF: Pointer; ABmk: Cardinal): Integer; stdcall;
   TpdfDeleteEmbeddedFile = function(const IPDF: Pointer; Handle: Cardinal): LongBool; stdcall;
   TpdfDeleteField = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfDeleteFieldEx = function(const IPDF: Pointer; const Name: PAnsiChar): Integer; stdcall;
   TpdfDeleteJavaScripts = procedure(const IPDF: Pointer; DelJavaScriptActions: LongBool); stdcall;
   TpdfDeletePageLabels = procedure(const IPDF: Pointer); stdcall;
   TpdfDeletePage = function(const IPDF: Pointer; PageNum: Cardinal): Integer; stdcall;
   TpdfDeletePDF = function(IPDF: Pointer): LongBool; stdcall;
   TpdfDeleteSeparationInfo = function(const IPDF: Pointer; AllPages: LongBool): LongBool; stdcall;
   TpdfDeleteTemplate = function(const IPDF: Pointer; Handle: Cardinal): LongBool; stdcall;
   TpdfDeleteTemplateEx = function(const IPDF: Pointer; Index: Cardinal): Integer; stdcall;
   TpdfDeleteXFAForm = procedure(const IPDF: Pointer); stdcall;
   TpdfDrawArc = function(const IPDF: Pointer; PosX, PosY, Radius, StartAngle, EndAngle: Double): LongBool; stdcall;
   TpdfDrawArcEx = function(const IPDF: Pointer; PosX, PosY, Width, Height, StartAngle, EndAngle: Double): LongBool; stdcall;
   TpdfDrawChord = function(const IPDF: Pointer; PosX, PosY, Width, Height, StartAngle, EndAngle: Double; FillMode: Integer): LongBool; stdcall;
   TpdfDrawCircle = function(const IPDF: Pointer; PosX, PosY, Radius: Double; FillMode: Integer): LongBool; stdcall;
   TpdfDrawPie = function(const IPDF: Pointer; PosX, PosY, Width, Height, StartAngle, EndAngle: Double; FillMode: Integer): LongBool; stdcall;
   TpdfEditPage = function(const IPDF: Pointer; PageNum: Integer): LongBool; stdcall;
   TpdfEditTemplate = function(const IPDF: Pointer; Index: Cardinal): LongBool; stdcall;
   TpdfEditTemplate2 = function(const IPDF: Pointer; Handle: Cardinal): LongBool; stdcall;
   TpdfEllipse = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; FillMode: Integer): LongBool; stdcall;
   TpdfEncryptPDFA = function(const IPDF: Pointer; const FileName, OpenPwd, OwnerPwd: PAnsiChar; KeyLen, Restrict: Integer): Integer; stdcall;
   TpdfEncryptPDFW = function(const IPDF: Pointer; const FileName: PWideChar; const OpenPwd, OwnerPwd: PAnsiChar; KeyLen, Restrict: Integer): Integer; stdcall;
   TpdfEndContinueText = function(IPDF: Pointer): Integer; stdcall;
   TpdfEndLayer = function(const IPDF: Pointer): LongBool; stdcall;
   TpdfEndPage = function(IPDF: Pointer): LongBool; stdcall;
   TpdfEndPattern = function(IPDF: Pointer): LongBool; stdcall;
   TpdfEndTemplate = function(IPDF: Pointer): LongBool; stdcall;
   TpdfEnumDocFonts = function(const IPDF, Data: Pointer; EnumProc: TEnumFontProc2): Integer; stdcall;
   TpdfEnumHostFonts = function(const IPDF: Pointer; const Data: Pointer; EnumProc: TEnumFontProc): Integer; stdcall;
   TpdfEnumHostFontsEx = function(const IPDF, Data: Pointer; EnumProc: TEnumFontProcEx): Integer; stdcall;
   TpdfExchangeBookmarks = function(const IPDF: Pointer; Bmk1, Bmk2: Integer): LongBool; stdcall;
   TpdfExchangePages = function(const IPDF: Pointer; First, Second: Cardinal): LongBool; stdcall;
   TpdfFileAttachAnnotA = function(const IPDF: Pointer; PosX, PosY: Double; Icon: Integer; const Author, Desc, AFile: PAnsiChar; Compress: LongBool): Integer; stdcall;
   TpdfFileAttachAnnotW = function(const IPDF: Pointer; PosX, PosY: Double; Icon: Integer; const Author, Desc, AFile: PWideChar; Compress: LongBool): Integer; stdcall;
   TpdfFileLinkA = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; const AFilePath: PAnsiChar): Integer; stdcall;
   TpdfFileLinkW = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; const AFilePath: PWideChar): Integer; stdcall;
   TpdfFindBookmarkA = function(const IPDF: Pointer; DestPage: Integer; const Title: PAnsiChar): Integer; stdcall;
   TpdfFindBookmarkW = function(const IPDF: Pointer; DestPage: Integer; const Title: PWideChar): Integer; stdcall;
   TpdfFindFieldA = function(const IPDF: Pointer; const Name: PAnsiChar): Integer; stdcall;
   TpdfFindFieldW = function(const IPDF: Pointer; const Name: PWideChar): Integer; stdcall;
   TpdfFindLinkAnnot = function(const IPDF: Pointer; const URL: PAnsiChar): Integer; stdcall;
   TpdfFindNextBookmark = function(IPDF: Pointer): Integer; stdcall;
   TpdfFinishSignature = function(const IPDF, PKCS7Obj: Pointer; Length: Cardinal): LongBool; stdcall;
   TpdfFlattenAnnots = function(IPDF: Pointer; Flags: TAnnotFlattenFlags): Integer; stdcall;
   TpdfFlattenForm = function(IPDF: Pointer): LongBool; stdcall;
   TpdfFlushPageContent = function(const IPDF: Pointer; var Stack: TPDFStack): LongBool; stdcall;
   TpdfFreeImageBuffer = procedure(const IPDF: Pointer); stdcall;
   TpdfFreePDF = function(IPDF: Pointer): LongBool; stdcall;
   TpdfFreeTextAnnotA = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; const Author, AText: PAnsiChar; Align: Integer): Integer; stdcall;
   TpdfFreeTextAnnotW = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; const Author, AText: PWideChar; Align: Integer): Integer; stdcall;
   TpdfFreeUniBuf = function(IPDF: Pointer): LongBool; stdcall;
   TpdfGetActionCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetActionType = function(const IPDF: Pointer; ActHandle: Cardinal): Integer; stdcall;
   TpdfGetActionTypeEx = function(const IPDF: Pointer; ObjType: TObjType; ObjHandle, ActIndex: Cardinal): Integer; stdcall;
   TpdfGetActiveFont = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetAllocBy = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetAnnot = function(const IPDF: Pointer; Handle: Cardinal; var Annot: TPDFAnnotation): LongBool; stdcall;
   TpdfGetAnnotBBox = function(const IPDF: Pointer; Handle: Cardinal; var BBox: TPDFRect): LongBool; stdcall;
   TpdfGetAnnotCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetAnnotEx = function(const IPDF: Pointer; Handle: Cardinal; var Annot: TPDFAnnotationEx): LongBool; stdcall;
   TpdfGetAnnotFlags = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetAnnotLink = function(const IPDF: Pointer; Handle: Cardinal): PAnsiChar; stdcall;
   TpdfGetAnnotType = function(const IPDF: Pointer; Handle: Cardinal): Integer; stdcall;
   TpdfGetAscent = function(IPDF: Pointer): Double; stdcall;
   TpdfGetBBox = function(const IPDF: Pointer; Boundary: TPageBoundary; var BBox: TPDFRect): LongBool; stdcall;
   TpdfGetBidiMode = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetBookmark = function(const IPDF: Pointer; Handle: Integer; var Bmk: TBookmark): LongBool; stdcall;
   TpdfGetBookmarkCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetBorderStyle = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetBuffer = function(const IPDF: Pointer; var BufSize: Cardinal): PAnsiChar; stdcall;
   TpdfGetCapHeight = function(IPDF: Pointer): Double; stdcall;
   TpdfGetCharacterSpacing = function(IPDF: Pointer): Double; stdcall;
   TpdfGetCheckBoxChar = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetCheckBoxCharEx = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetCheckBoxDefState = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetCMap = function(const IPDF: Pointer; Index: Cardinal; var CMap: TPDFCMap): LongBool; stdcall;
   TpdfGetCMapCount = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetColorSpace = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetColorSpaceCount = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetColorSpaceObj = function(const IPDF: Pointer; Handle: Cardinal; var CS: TPDFColorSpaceObj): LongBool; stdcall;
   TpdfGetColorSpaceObjEx = function(const IColorSpace: Pointer; var CS: TPDFColorSpaceObj): LongBool; stdcall;
   TpdfGetCompressionFilter = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetCompressionLevel = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetContent = function(const IPDF: Pointer; var Buffer: PAnsiChar): Integer; stdcall;
   TpdfGetDefBitsPerPixel = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetDescent = function(IPDF: Pointer): Double; stdcall;
   TpdfGetDeviceNAttributes = function(const IAttributes: Pointer; var Attributes: TDeviceNAttributes): LongBool; stdcall;
   TpdfGetDocInfo = function(const IPDF: Pointer; DInfo: Integer; var Value: PWideChar): Integer; stdcall;
   TpdfGetDocInfoCount = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetDocInfoEx = function(const IPDF: Pointer; Index: Cardinal; var DInfo: TDocumentInfo; var Key, Value: PAnsiChar; var Unicode: LongBool): Integer; stdcall;
   TpdfGetDrawDirection = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetDynaPDFVersion = function(): PAnsiChar; stdcall;
   TpdfGetEmbeddedFile = function(const IPDF: Pointer; Handle: Cardinal; var FileSpec: TPDFFileSpec; Decompress: LongBool): LongBool; stdcall;
   TpdfGetEmbeddedFileCount = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetEMFPatternDistance = function(IPDF: Pointer): Double; stdcall;
   TpdfGetErrorMessage = function(IPDF: Pointer): PAnsiChar; stdcall;
   TpdfGetErrorMode = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetField = function(const IPDF: Pointer; Handle: Cardinal; var Field: TPDFField): LongBool; stdcall;
   TpdfGetFieldBackColor = function(IPDF: Pointer): Cardinal; stdcall;
   TpdfGetFieldBorderColor = function(IPDF: Pointer): Cardinal; stdcall;
   TpdfGetFieldBorderStyle = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetFieldBorderWidth = function(const IPDF: Pointer; AField: Cardinal): Double; stdcall;
   TpdfGetFieldChoiceValue = function(const IPDF: Pointer; AField, ValIndex: Cardinal; var Value: TPDFChoiceValue): LongBool; stdcall;
   TpdfGetFieldColor = function(const IPDF: Pointer; AField: Cardinal; ColorType: Integer; var ColorSpace: Integer; var Color: Cardinal): LongBool; stdcall;
   TpdfGetFieldCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetFieldEx = function(const IPDF: Pointer; Handle: Cardinal; var Field: TPDFFieldEx): LongBool; stdcall;
   TpdfGetFieldEx2 = function(const IField: Pointer; var Field: TPDFFieldEx): LongBool; stdcall;
   TpdfGetFieldExpValCount = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetFieldExpValue = function(const IPDF: Pointer; AField: Cardinal; var Value: PAnsiChar): Integer; stdcall;
   TpdfGetFieldExpValueEx = function(const IPDF: Pointer; AField, ValIndex: Cardinal; var Value, ExpValue: PAnsiChar; var Selected: LongBool): LongBool; stdcall;
   TpdfGetFieldFlags = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetFieldGroupType = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetFieldHighlightMode = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetFieldIndex = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetFieldMapName = function(const IPDF: Pointer; AField: Cardinal; var Value: Pointer; var Unicode: LongBool): Integer; stdcall;
   TpdfGetFieldName = function(const IPDF: Pointer; AField: Cardinal; var Name: PAnsiChar): Integer; stdcall;
   TpdfGetFieldOrientation = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetFieldTextAlign = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetFieldTextColor = function(IPDF: Pointer): Cardinal; stdcall;
   TpdfGetFieldToolTip = function(const IPDF: Pointer; AField: Cardinal; var Value: Pointer; var Unicode: LongBool): Integer; stdcall;
   TpdfGetFieldType = function(const IPDF: Pointer; AField: Cardinal): Integer; stdcall;
   TpdfGetFillColor = function(IPDF: Pointer): Cardinal; stdcall;
   TpdfGetFontCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetFontEx = function(const IPDF: Pointer; Handle: Cardinal; var F: TPDFFontObj): LongBool; stdcall;
   TpdfGetFontOrigin = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetFontSearchOrder = procedure(const IPDF: Pointer; Order: PFontBaseType{Array[0..3] of TFontBaseType}); stdcall;
   TpdfGetFontSelMode = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetFontWeight = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetFTextHeightA = function(const IPDF: Pointer; Align: Integer; const AText: PAnsiChar): Double; stdcall;
   TpdfGetFTextHeightExA = function(const IPDF: Pointer; Width: Double; Align: Integer; const AText: PAnsiChar): Double; stdcall;
   TpdfGetFTextHeightExW = function(const IPDF: Pointer; Width: Double; Align: Integer; const AText: PWideChar): Double; stdcall;
   TpdfGetFTextHeightW = function(const IPDF: Pointer; Align: Integer; const AText: PWideChar): Double; stdcall;
   TpdfGetGStateFlags = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetIconColor = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetImageBuffer = function(const IPDF: Pointer; var BufSize: Cardinal): PAnsiChar; stdcall;
   TpdfGetImageCount = function(const IPDF: Pointer; const FileName: PAnsiChar): Integer; stdcall;
   TpdfGetImageCountEx = function(const IPDF: Pointer; const Buffer: Pointer; BufSize: Cardinal): Integer; stdcall;
   TpdfGetImageHeight = function(const IPDF: Pointer; Handle: Cardinal): Integer; stdcall;
   TpdfGetImageWidth = function(const IPDF: Pointer; Handle: Cardinal): Integer; stdcall;
   TpdfGetImportFlags = function(IPDF: Pointer): Cardinal; stdcall;
   TpdfGetInBBox = function(const IPDF: Pointer; PageNum: Cardinal; Boundary: Integer; var BBox: TPDFRect): LongBool; stdcall;
   TpdfGetInDocInfo = function(const IPDF: Pointer; DInfo: Integer; var Value: PWideChar): Integer; stdcall;
   TpdfGetInDocInfoCount = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetInDocInfoEx = function(const IPDF: Pointer; Index: Cardinal; var DInfo: Integer; var Key, Value: PAnsiChar; var Unicode: LongBool): LongBool; stdcall;
   TpdfGetInEncryptionFlags = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetInIsCollection = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetInIsEncrypted = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetInIsSigned = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetInIsTrapped = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetInIsXFAForm = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetInNamedDest = function(const IPDF: Pointer; Index: Cardinal; var Dest: TPDFNamedDest): LongBool; stdcall;
   TpdfGetInNamedDestCount = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetInOrientation = function(const IPDF: Pointer; PageNum: Integer): Integer; stdcall;
   TpdfGetInPageCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetInPDFVersion = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetInPrintSettings = function(const IPDF: Pointer; var Settings: TPDFPrintSettings): LongBool; stdcall;
   TpdfGetIsFixedPitch = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetItalicAngle = function(IPDF: Pointer): Double; stdcall;
   TpdfGetJavaScript = function(const IPDF: Pointer; Handle: Cardinal; var Len: Cardinal; var Unicode: LongBool): PAnsiChar; stdcall;
   TpdfGetJavaScriptAction = function(const IPDF: Pointer; Handle: Cardinal; var Len: Cardinal; var Unicode: LongBool): PAnsiChar; stdcall;
   TpdfGetJavaScriptAction2 = function(const IPDF: Pointer; ObjType: TObjType; ObjHandle, ActIndex: Cardinal; var Len: Cardinal; var Unicode: LongBool; var Event: TObjEvent): PAnsiChar; stdcall;
   TpdfGetJavaScriptCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetJavaScriptEx = function(const IPDF: Pointer; const Name: PAnsiChar; var Len: Cardinal; var Unicode: LongBool): PAnsiChar; stdcall;
   TpdfGetJavaScriptName = function(const IPDF: Pointer; Handle: Cardinal; var Len: Cardinal; var Unicode: LongBool): PAnsiChar; stdcall;
   TpdfGetJPEGQuality = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetLanguage = function(IPDF: Pointer): PAnsiChar; stdcall;
   TpdfGetLastTextPosX = function(IPDF: Pointer): Double; stdcall;
   TpdfGetLastTextPosY = function(IPDF: Pointer): Double; stdcall;
   TpdfGetLeading = function(IPDF: Pointer): Double; stdcall;
   TpdfGetLineCapStyle = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetLineJoinStyle = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetLineWidth = function(IPDF: Pointer): Double; stdcall;
   TpdfGetLinkHighlightMode = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetLogMetafileSize = function(const IPDF: Pointer; const FileName: PAnsiChar; var R: TRectL): LongBool; stdcall;
   TpdfGetLogMetafileSizeEx = function(const IPDF: Pointer; const Buffer: Pointer; BufSize: Cardinal; var R: TRectL): LongBool; stdcall;
   TpdfGetMatrix = function(const IPDF: Pointer; var M: TCTM): LongBool; stdcall;
   TpdfGetMaxFieldLen = function(const IPDF: Pointer; TxtField: Cardinal): Integer; stdcall;
   TpdfGetMetaConvFlags = function(IPDF: Pointer): Cardinal; stdcall;
   TpdfGetMissingGlyphs = function(const IPDF: Pointer; var Count: Cardinal): PCardinal; stdcall;
   TpdfGetMiterLimit = function(IPDF: Pointer): Double; stdcall;
   TpdfGetNeedAppearance = function(const IPDF: Pointer): LongBool; stdcall;
   TpdfGetObjActionCount = function(const IPDF: Pointer; ObjType: Integer; ObjHandle: Cardinal): Integer; stdcall;
   TpdfGetOpacity = function(IPDF: Pointer): Double; stdcall;
   TpdfGetOrientation = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetPageAnnot = function(const IPDF: Pointer; Index: Cardinal; var Annot: TPDFAnnotation): LongBool; stdcall;
   TpdfGetPageAnnotCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetPageAnnotEx = function(const IPDF: Pointer; Index: Cardinal; var Annot: TPDFAnnotationEx): LongBool; stdcall;
   TpdfGetPageCoords = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetPageCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetPageField = function(const IPDF: Pointer; Index: Cardinal; var Field: TPDFField): LongBool; stdcall;
   TpdfGetPageFieldCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetPageFieldEx = function(const IPDF: Pointer; Index: Cardinal; var Field: TPDFFieldEx): LongBool; stdcall;
   TpdfGetPageHeight = function(IPDF: Pointer): Double; stdcall;
   TpdfGetPageLabel = function(const IPDF: Pointer; Index: Cardinal; var Lbl: TPDFPageLabel): LongBool; stdcall;
   TpdfGetPageLabelCount = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetPageLayout = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetPageMode = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetPageNum = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetPageText = function(const IPDF: Pointer; var Stack: TPDFStack): LongBool; stdcall;
   TpdfGetPageWidth = function(IPDF: Pointer): Double; stdcall;
   TpdfGetPDFVersion = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetPrintSettings = function(const IPDF: Pointer; var Settings: TPDFPrintSettings): LongBool; stdcall;
   TpdfGetResolution = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetSaveNewImageFormat = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetSeparationInfo = function(const IPDF: Pointer; var Colorant: PAnsiChar; var CS: TExtColorSpace): LongBool; stdcall;
   TpdfGetStrokeColor = function(IPDF: Pointer): Cardinal; stdcall;
   TpdfGetTabLen = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetTemplCount = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetTemplHandle = function(const IPDF: Pointer): Integer; stdcall;
   TpdfGetTemplHeight = function(const IPDF: Pointer; TmplHandle: Integer): Double; stdcall;
   TpdfGetTemplWidth = function(const IPDF: Pointer; TmplHandle: Integer): Double; stdcall;
   TpdfGetTextDrawMode = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetTextFieldValue = function(const IPDF: Pointer; AField: Cardinal; var Value: PAnsiChar; var ValIsUnicode: LongBool; var DefValue: PAnsiChar; var DefValIsUnicode: LongBool): LongBool; stdcall;
   TpdfGetTextRect = function(const IPDF: Pointer; var PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfGetTextRise = function(IPDF: Pointer): Double; stdcall;
   TpdfGetTextScaling = function(IPDF: Pointer): Double; stdcall;
   TpdfGetTextWidthA = function(const IPDF: Pointer; const AText: PAnsiChar): Double; stdcall;
   TpdfGetTextWidthW = function(const IPDF: Pointer; const AText: PWideChar): Double; stdcall;
   TpdfGetTextWidthExA = function(const IPDF: Pointer; const AText: PAnsiChar; Len: Cardinal): Double; stdcall;
   TpdfGetTextWidthExW = function(const IPDF: Pointer; const AText: PWideChar; Len: Cardinal): Double; stdcall;
   TpdfGetTrapped = function(const IPDF: Pointer): LongBool; stdcall;
   TpdfGetTransparentColor = function(IPDF: Pointer): Cardinal; stdcall;
   TpdfGetUseExactPwd = function(IPDF: Pointer): LongBool; stdcall;
   TpdfGetUseGlobalImpFiles = function(const IPDF: Pointer): LongBool; stdcall;
   TpdfGetUserRights = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetUseStdFonts = function(IPDF: Pointer): LongBool; stdcall;
   TpdfGetUseSystemFonts = function(IPDF: Pointer): LongBool; stdcall;
   TpdfGetUseTransparency = function(IPDF: Pointer): LongBool; stdcall;
   TpdfGetUseVisibleCoords = function(IPDF: Pointer): LongBool; stdcall;
   TpdfGetViewerPrefrences = function(const IPDF: Pointer; var Preference, AddVal: Integer): LongBool; stdcall;
   TpdfGetWMFDefExtent = function(const IPDF: Pointer; var Width, Height: Cardinal): LongBool; stdcall;
   TpdfGetWMFPixelPerInch = function(IPDF: Pointer): Integer; stdcall;
   TpdfGetWordSpacing = function(IPDF: Pointer): Double; stdcall;
   TpdfHaveOpenDoc = function(const IPDF: Pointer): LongBool; stdcall;
   TpdfHighlightAnnotA = function(const IPDF: Pointer; SubType: TAnnotType; PosX, PosY, Width, Height: Double; Color: Cardinal; const Author, Subject, Comment: PAnsiChar): Integer; stdcall;
   TpdfHighlightAnnotW = function(const IPDF: Pointer; SubType: TAnnotType; PosX, PosY, Width, Height: Double; Color: Cardinal; const Author, Subject, Comment: PWideChar): Integer; stdcall;
   TpdfImportBookmarks = function(const IPDF: Pointer): Integer; stdcall;
   TpdfImportCatalogObjects = function(const IPDF: Pointer): LongBool; stdcall;
   TpdfImportDocInfo = function(const IPDF: Pointer): LongBool; stdcall;
   TpdfImportPage = function(const IPDF: Pointer; PageNum: Cardinal): Integer; stdcall;
   TpdfImportPageEx = function(const IPDF: Pointer; PageNum: Cardinal; ScaleFactX, ScaleFactY: Double): Integer; stdcall;
   TpdfImportPDFFile = function(const IPDF: Pointer; DestPage: Cardinal; ScaleFactX, ScaleFactY: Double): Integer; stdcall;
   TpdfInitExtGState = function(var GS: TPDFExtGState): LongBool; stdcall;
   TpdfInitStack = function(const IPDF: Pointer; var Stack: TPDFStack): LongBool; stdcall;
   TpdfInsertBMPFromBuffer = function(const IPDF: Pointer; PosX, PosY, ScaleWidth, ScaleHeight: Double; const Buffer: Pointer): Integer; stdcall;
   TpdfInsertBMPFromHandle = function(const IPDF: Pointer; PosX, PosY, ScaleWidth, ScaleHeight: Double; Handle: Pointer): Integer; stdcall;
   TpdfInsertImage = function(const IPDF: Pointer; PosX, PosY, ScaleWidth, ScaleHeight: Double; const AFile: PAnsiChar): Integer; stdcall;
   TpdfInsertImageEx = function(const IPDF: Pointer; PosX, PosY, ScaleWidth, ScaleHeight: Double; const AFile: PAnsiChar; Index: Cardinal): Integer; stdcall;
   TpdfInsertImageFromBuffer = function(const IPDF: Pointer; PosX, PosY, ScaleWidth, ScaleHeight: Double; const Buffer: Pointer; BufSize, Index: Cardinal): Integer; stdcall;
   TpdfInsertMetafile = function(const IPDF: Pointer; const FileName: PAnsiChar; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfInsertMetafileEx = function(const IPDF: Pointer; const Buffer: Pointer; BufSize: Cardinal; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfInsertMetafileExt = function(const IPDF: Pointer; const FileName: PAnsiChar; var View: TRectL; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfInsertMetafileExtEx = function(const IPDF: Pointer; const Buffer: Pointer; BufSize: Cardinal; var View: TRectL; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfInsertMetafileFromHandle = function(const IPDF, hEnhMetafile: Pointer; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfInsertMetafileFromHandleEx = function(const IPDF, hEnhMetafile: Pointer; var View: TRectL; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfInsertRawImage = function(const IPDF: Pointer; const Buffer: Pointer; BitsPerPixel, ColorCount, ImgWidth, ImgHeight: Cardinal; PosX, PosY, ScaleWidth, ScaleHeight: Double): Integer; stdcall;
   TpdfIsBidiText = function(const IPDF: Pointer; const AText: PWideChar): Integer; stdcall;
   TpdfIsColorPage = function(const IPDF: Pointer; GrayIsColor: LongBool): Integer; stdcall;
   TpdfIsEmptyPage = function(IPDF: Pointer): Integer; stdcall;
   TpdfLineTo = function(const IPDF: Pointer; PosX, PosY: Double): LongBool; stdcall;
   TpdfLoadCMap = function(const IPDF: Pointer; const CMapName: PAnsiChar; Embed: LongBool): Integer; stdcall;
   TpdfLoadFDFDataA = function(const IPDF: Pointer; const FileName, Password: PAnsiChar; Flags: Cardinal): LongBool; stdcall;
   TpdfLoadFDFDataW = function(const IPDF: Pointer; const FileName: PWideChar; const Password: PAnsiChar; Flags: Cardinal): LongBool; stdcall;
   TpdfLoadFDFDataEx = function(const IPDF, Buffer: Pointer; BufSize: Cardinal; const Password: PAnsiChar; Flags: Cardinal): LongBool; stdcall;
   TpdfLoadFont = function(const IPDF, Buffer: Pointer; BufSize: Cardinal; Style: Integer; Size: Double; Embed: LongBool; CP: Integer): Integer; stdcall;
   TpdfMovePage = function(const IPDF: Pointer; Source, Dest: Cardinal): LongBool; stdcall;
   TpdfMoveTo = function(const IPDF: Pointer; PosX, PosY: Double): LongBool; stdcall;
   TpdfNewPDF = function(): Pointer; stdcall;
   TpdfOpenImportBuffer = function(const IPDF: Pointer; const Buffer: Pointer; BufSize: Cardinal; PwdType: Integer; const Password: PAnsiChar): Integer; stdcall;
   TpdfOpenImportFileA = function(const IPDF: Pointer; const FileName: PAnsiChar; PwdType: Integer; const Password: PAnsiChar): Integer; stdcall;
   TpdfOpenImportFileW = function(const IPDF: Pointer; const FileName: PWideChar; PwdType: Integer; const Password: PAnsiChar): Integer; stdcall;
   TpdfOpenOutputFileA = function(const IPDF: Pointer; const OutPDF: PAnsiChar): LongBool; stdcall;
   TpdfOpenOutputFileW = function(const IPDF: Pointer; const OutPDF: PWideChar): LongBool; stdcall;
   TpdfOpenTagA = function(const IPDF: Pointer; Tag: Integer; const Lang, AltText, Expansion: PAnsiChar): LongBool; stdcall;
   TpdfOpenTagW = function(const IPDF: Pointer; Tag: Integer; const Lang: PAnsiChar; const AltText, Expansion: PWideChar): LongBool; stdcall;
   TpdfPageLink = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; DestPage: Cardinal): Integer; stdcall;
   TpdfPageLink2 = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; NamedDest: Cardinal): Integer; stdcall;
   TpdfPageLinkEx = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; DestType: Integer; DestPage: Cardinal; a, b, c, d: Double): Integer; stdcall;
   TpdfParseContent = function(const IPDF, Data: Pointer; var Stack: TPDFParseInterface; Flags: Integer): LongBool; stdcall;
   TpdfPlaceImage = function(const IPDF: Pointer; ImgHandle: Integer; PosX, PosY, ScaleWidth, ScaleHeight: Double): LongBool; stdcall;
   TpdfPlaceSigFieldValidateIcon = function(const IPDF: Pointer; SigField: Cardinal; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfPlaceTemplate = function(const IPDF: Pointer; TmplHandle: Integer; PosX, PosY, ScaleWidth, ScaleHeight: Double): LongBool; stdcall;
   TpdfPlaceTemplateEx = function(const IPDF: Pointer; TmplHandle: Integer; PosX, PosY, ScaleWidth, ScaleHeight: Double): LongBool; stdcall;
   TpdfPlaceTemplByMatrix = function(const IPDF: Pointer; TmplHandle: Integer): LongBool; stdcall;
   TpdfReadImageFormat = function(const IPDF: Pointer; const FileName: PAnsiChar; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): LongBool; stdcall;
   TpdfReadImageFormat2 = function(const IPDF: Pointer; const FileName: PAnsiChar; Index: Cardinal; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): LongBool; stdcall;
   TpdfReadImageFormatEx = function(const IPDF: Pointer; hBitmap: Pointer; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): LongBool; stdcall;
   TpdfReadImageFormatFromBuffer = function(IPDF, Buffer: Pointer; BufSize, Index: Cardinal; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): LongBool; stdcall;
   TpdfReadImageResolution = function(const IPDF: Pointer; const FileName: PAnsiChar; Index: Cardinal; var ResX, ResY: Cardinal): LongBool; stdcall;
   TpdfReadImageResolutionEx = function(const IPDF, Buffer: Pointer; BufSize, Index: Cardinal; var ResX, ResY: Cardinal): LongBool; stdcall;
   TpdfRectangle = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; FillMode: Integer): LongBool; stdcall;
   TpdfReEncryptPDFA = function(const IPDF: Pointer; const FileName: PAnsiChar; PwdType: Integer; const InPwd, NewOpenPwd, NewOwnerPwd: PAnsiChar; NewKeyLen, Restrict: Integer): Integer; stdcall;
   TpdfReEncryptPDFW = function(const IPDF: Pointer; const FileName: PWideChar; PwdType: Integer; const InPwd, NewOpenPwd, NewOwnerPwd: PAnsiChar; NewKeyLen, Restrict: Integer): Integer; stdcall;
   TpdfRenameSpotColor = function(const IPDF: Pointer; const Colorant, NewName: PAnsiChar): Integer; stdcall;
   TpdfReplacePageTextA = function(const IPDF: Pointer; const NewText: PAnsiChar; var Stack: TPDFStack): LongBool; stdcall;
   TpdfReplacePageTextExA = function(const IPDF: Pointer; const NewText: PAnsiChar; var Stack: TPDFStack): LongBool; stdcall;
   TpdfReplacePageTextExW = function(const IPDF: Pointer; const NewText: PWideChar; var Stack: TPDFStack): LongBool; stdcall;
   TpdfResetLineDashPattern = function(IPDF: Pointer): LongBool; stdcall;
   TpdfRestoreGraphicState = function(IPDF: Pointer): LongBool; stdcall;
   TpdfRotateCoords = function(const IPDF: Pointer; alpha, OriginX, OriginY: Double): LongBool; stdcall;
   TpdfRoundRect = function(const IPDF: Pointer; PosX, PosY, Width, Height, Radius: Double; FillMode: Integer): LongBool; stdcall;
   TpdfRoundRectEx = function(const IPDF: Pointer; PosX, PosY, Width, Height, rWidth, rHeight: Double; FillMode: Integer): LongBool; stdcall;
   TpdfSaveGraphicState = function(IPDF: Pointer): LongBool; stdcall;
   TpdfScaleCoords = function(const IPDF: Pointer; sx, sy: Double): LongBool; stdcall;
   TpdfSelfTest = function(IPDF: Pointer): LongBool; stdcall;
   TpdfSet3DAnnotProps = function(const IPDF: Pointer; Annot: Cardinal; ActType, DeActType, InstType, DeInstType: Integer; DisplToolbar, DisplModelTree: LongBool): LongBool; stdcall;
   TpdfSet3DAnnotScriptA = function(const IPDF: Pointer; Annot: Cardinal; const Value: PAnsiChar; Len: Cardinal): LongBool; stdcall;
   TpdfSetAllocBy = function(const IPDF: Pointer; Value: Integer): LongBool; stdcall;
   TpdfSetAnnotBorderStyle = function(const IPDF: Pointer; Handle: Cardinal; Style: TBorderStyle): LongBool; stdcall;
   TpdfSetAnnotBorderWidth = function(const IPDF: Pointer; Handle: Cardinal; LineWidth: Double): LongBool; stdcall;
   TpdfSetAnnotColor = function(const IPDF: Pointer; Handle: Cardinal; ColorType: TAnnotColor; CS: TPDFColorSpace; Color: Cardinal): LongBool; stdcall;
   TpdfSetAnnotFlags = function(const IPDF: Pointer; Flags: Cardinal): LongBool; stdcall;
   TpdfSetAnnotFlagsEx = function(const IPDF: Pointer; Handle: Cardinal; Flags: Integer): LongBool; stdcall;
   TpdfSetAnnotHighlightMode = function(const IPDF: Pointer; Handle: Cardinal; Mode: THighlightMode): LongBool; stdcall;
   TpdfSetAnnotIcon = function(const IPDF: Pointer; Handle: Cardinal; Icon: TAnnotIcon): LongBool; stdcall;
   TpdfSetAnnotOpenState = function(const IPDF: Pointer; Handle: Cardinal; Open: LongBool): LongBool; stdcall;
   TpdfSetAnnotStringA = function(const IPDF: Pointer; Handle: Cardinal; StringType: Integer; const Value: PAnsiChar): LongBool; stdcall;
   TpdfSetAnnotStringW = function(const IPDF: Pointer; Handle: Cardinal; StringType: Integer; const Value: PWideChar): LongBool; stdcall;
   TpdfSetAnnotSubjectA = function(const IPDF: Pointer; Handle: Cardinal; const Value: PAnsiChar): LongBool; stdcall;
   TpdfSetAnnotSubjectW = function(const IPDF: Pointer; Handle: Cardinal; const Value: PWideChar): LongBool; stdcall;
   TpdfSetBBox = function(const IPDF: Pointer; Boundary: Integer; X0, Y0, Width, Height: Double): LongBool; stdcall;
   TpdfSetBidiMode = function(const IPDF: Pointer; Mode: Integer): LongBool; stdcall;
   TpdfSetBookmarkDest = function(const IPDF: Pointer; ABmk, DestType: Integer; a, b, c, d: Double): LongBool; stdcall;
   TpdfSetBookmarkStyle = function(const IPDF: Pointer; ABmk: Integer; Style: TFStyle; RGBColor: Cardinal): LongBool; stdcall;
   TpdfSetBorderStyle = function(const IPDF: Pointer; Style: Integer): LongBool; stdcall;
   TpdfSetCharacterSpacing = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetCheckBoxChar = function(const IPDF: Pointer; CheckBoxChar: Integer): LongBool; stdcall;
   TpdfSetCheckBoxDefState = function(const IPDF: Pointer; AField: Cardinal; Checked: LongBool): LongBool; stdcall;
   TpdfSetCheckBoxState = function(const IPDF: Pointer; AField: Cardinal; Checked: LongBool): LongBool; stdcall;
   TpdfSetCIDFontA = function(const IPDF: Pointer; CMapHandle: Cardinal; const Name: PAnsiChar; Style: Integer; Size: Double; Embed: LongBool): Integer; stdcall;
   TpdfSetCIDFontW = function(const IPDF: Pointer; CMapHandle: Cardinal; const Name: PWideChar; Style: Integer; Size: Double; Embed: LongBool): Integer; stdcall;
   TpdfSetCMapDirA = function(const IPDF: Pointer; const Path: PAnsiChar; Recursive: LongBool): Integer; stdcall;
   TpdfSetCMapDirW = function(const IPDF: Pointer; const Path: PWideChar; Recursive: LongBool): Integer; stdcall;
   TpdfSetColDefFile = function(const IPDF: Pointer; EmbFile: Cardinal): LongBool; stdcall;
   TpdfSetColors = function(const IPDF: Pointer; Color: Cardinal): LongBool; stdcall;
   TpdfSetColorSpace = function(const IPDF: Pointer; ColorSpace: Integer): LongBool; stdcall;
   TpdfSetColSortField = function(const IPDF: Pointer; ColField: Cardinal; AscendingOrder: LongBool): LongBool; stdcall;
   TpdfSetCompressionFilter = function(const IPDF: Pointer; Filter: Integer): LongBool; stdcall;
   TpdfSetCompressionLevel = function(const IPDF: Pointer; CompressLevel: Integer): LongBool; stdcall;
   TpdfSetContent = function(const IPDF: Pointer; const Buffer: PAnsiChar; BufSize: Cardinal): LongBool; stdcall;
   TpdfSetDateTimeFormat = function(const IPDF: Pointer; TxtField: Cardinal; Fmt: Integer): LongBool; stdcall;
   TpdfSetDefBitsPerPixel = function(const IPDF: Pointer; Value: Integer): LongBool; stdcall;
   TpdfSetDocInfoA = function(const IPDF: Pointer; DInfo: Integer; const Value: PAnsiChar): LongBool; stdcall;
   TpdfSetDocInfoW = function(const IPDF: Pointer; DInfo: Integer; const Value: PWideChar): LongBool; stdcall;
   TpdfSetDocInfoExA = function(const IPDF: Pointer; DInfo: Integer; const Key, Value: PAnsiChar): LongBool; stdcall;
   TpdfSetDocInfoExW = function(const IPDF: Pointer; DInfo: Integer; const Key: PAnsiChar; const Value: PWideChar): LongBool; stdcall;
   TpdfSetDrawDirection = function(const IPDF: Pointer; Direction: Integer): LongBool; stdcall;
   TpdfSetEMFFrameDPI = function(const IPDF: Pointer; DPIX, DPIY: Cardinal): LongBool; stdcall;
   TpdfSetEMFPatternDistance = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetErrorMode = function(const IPDF: Pointer; ErrMode: Integer): LongBool; stdcall;
   TpdfSetExtColorSpace = function(const IPDF: Pointer; Handle: Cardinal): LongBool; stdcall;
   TpdfSetExtGState = function(const IPDF: Pointer; Handle: Cardinal): LongBool; stdcall;
   TpdfSetFieldBackColor = function(const IPDF: Pointer; AColor: Cardinal): LongBool; stdcall;
   TpdfSetFieldBBox = function(const IPDF: Pointer; AField: Cardinal; var BBox: TPDFRect): LongBool; stdcall;
   TpdfSetFieldBorderColor = function(const IPDF: Pointer; AColor: Cardinal): LongBool; stdcall;
   TpdfSetFieldBorderStyle = function(const IPDF: Pointer; AField: Cardinal; Style: Integer): LongBool; stdcall;
   TpdfSetFieldBorderWidth = function(const IPDF: Pointer; AField: Cardinal; LineWidth: Double): LongBool; stdcall;
   TpdfSetFieldColor = function(const IPDF: Pointer; AField: Cardinal; ColorType, CS: Integer; Color: Cardinal): LongBool; stdcall;
   TpdfSetFieldExpValue = function(const IPDF: Pointer; AField, ValIndex: Cardinal; const Value, ExpValue: PAnsiChar; Selected: LongBool): LongBool; stdcall;
   TpdfSetFieldFlags = function(const IPDF: Pointer; AField: Cardinal; Flags: Integer; Reset: LongBool): LongBool; stdcall;
   TpdfSetFieldFontSize = function(const IPDF: Pointer; AField: Cardinal; FontSize: double): LongBool; stdcall;
   TpdfSetFieldHighlightMode = function(const IPDF: Pointer; AField: Cardinal; Mode: Integer): LongBool; stdcall;
   TpdfSetFieldIndex = function(const IPDF: Pointer; AField, Index: Cardinal): LongBool; stdcall;
   TpdfSetFieldMapNameA = function(const IPDF: Pointer; AField: Cardinal; const Name: PAnsiChar): LongBool; stdcall;
   TpdfSetFieldMapNameW = function(const IPDF: Pointer; AField: Cardinal; const Name: PWideChar): LongBool; stdcall;
   TpdfSetFieldName = function(const IPDF: Pointer; AField: Cardinal; const NewName: PAnsiChar): LongBool; stdcall;
   TpdfSetFieldOrientation = function(const IPDF: Pointer; AField: Cardinal; Value: Integer): LongBool; stdcall;
   TpdfSetFieldTextAlign = function(const IPDF: Pointer; AField: Cardinal; Align: Integer): LongBool; stdcall;
   TpdfSetFieldTextColor = function(const IPDF: Pointer; Color: Cardinal): LongBool; stdcall;
   TpdfSetFieldToolTipA = function(const IPDF: Pointer; AField: Cardinal; const Value: PAnsiChar): LongBool; stdcall;
   TpdfSetFieldToolTipW = function(const IPDF: Pointer; AField: Cardinal; const Value: PWideChar): LongBool; stdcall;
   TpdfSetFillColor = function(const IPDF: Pointer; Color: Cardinal): LongBool; stdcall;
   TpdfSetFillColorEx = function(const IPDF: Pointer; const Color: PByte; NumComponents: Cardinal): LongBool; stdcall;
   TpdfSetFillColorSpace = procedure(const IPDF: Pointer; CS: Integer); stdcall;
   TpdfSetFontA = function(const IPDF: Pointer; const Name: PAnsiChar; Style: TFStyle; Size: Double; Embed, CP: Integer): Integer stdcall;
   TpdfSetFontW = function(const IPDF: Pointer; const Name: PWideChar; Style: TFStyle; Size: Double; Embed, CP: Integer): Integer; stdcall;
   TpdfSetFontExA = function(const IPDF: Pointer; const Name: PAnsiChar; Style: Integer; Size: Double; Embed: LongBool; CP: Integer): Integer; stdcall;
   TpdfSetFontExW = function(const IPDF: Pointer; const Name: PWideChar; Style: Integer; Size: Double; Embed: LongBool; CP: Integer): Integer; stdcall;
   TpdfSetFontOrigin = function(const IPDF: Pointer; Origin: Integer): LongBool; stdcall;
   TpdfSetFontSearchOrder = procedure(const IPDF: Pointer; Order: PFontBaseType{Array[0..3] of TFontBaseType}); stdcall;
   TpdfSetFontSelMode = function(const IPDF: Pointer; Mode: Integer): LongBool; stdcall;
   TpdfSetFontWeight = function(const IPDF: Pointer; Weight: Integer): LongBool; stdcall;
   TpdfSetGStateFlags = function(const IPDF: Pointer; Flags: TGStateFlags; Reset: LongBool): LongBool; stdcall;
   TpdfSetIconColor = function(const IPDF: Pointer; Color: Cardinal): LongBool; stdcall;
   TpdfSetImportFlags = function(const IPDF: Pointer; Flags: Cardinal): LongBool; stdcall;
   TpdfSetImportFlags2 = function(const IPDF: Pointer; Flags: Cardinal): LongBool; stdcall;
   TpdfSetItalicAngle = function(const IPDF: Pointer; Angle: Double): LongBool; stdcall;
   TpdfSetJPEGQuality = function(const IPDF: Pointer; Value: Integer): LongBool; stdcall;
   TpdfSetLanguage = function(const IPDF: Pointer; const ISOTag: PAnsiChar): LongBool; stdcall;
   TpdfSetLeading = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetLicenseKey = function(const IPDF: Pointer; const Key: PAnsiChar): LongBool; stdcall;
   TpdfSetLineCapStyle = function(const IPDF: Pointer; Style: Integer): LongBool; stdcall;
   TpdfSetLineDashPattern = function(const IPDF: Pointer; const Dash: PAnsiChar; Phase: Integer): LongBool; stdcall;
   TpdfSetLineDashPatternEx = function(const IPDF: Pointer; const Dash: Pointer; NumValues: Cardinal; Phase: Integer): LongBool; stdcall;
   TpdfSetLineJoinStyle = function(const IPDF: Pointer; Style: Integer): LongBool; stdcall;
   TpdfSetLineWidth = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetLinkHighlightMode = function(const IPDF: Pointer; Mode: Integer): LongBool; stdcall;
   TpdfSetListFont = function(const IPDF: Pointer; Handle: Cardinal): LongBool; stdcall;
   TpdfSetMatrix = function(const IPDF: Pointer; var Matrix: TCTM): LongBool; stdcall;
   TpdfSetMaxFieldLen = function(const IPDF: Pointer; TxtField: Cardinal; MaxLen: Integer): LongBool; stdcall;
   TpdfSetMetaConvFlags = function(const IPDF: Pointer; Flags: Integer): LongBool; stdcall;
   TpdfSetMiterLimit = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetNeedAppearance = function(const IPDF: Pointer; Value: LongBool): LongBool; stdcall;
   TpdfSetNumberFormat = function(const IPDF: Pointer; TxtField: Cardinal; Sep: Integer; DecPlaces: Cardinal; NegStyle: Integer; const CurrStr: PAnsiChar; Prepend: LongBool): LongBool; stdcall;
   TpdfSetOnErrorProc = function(const IPDF: Pointer; const Data, ErrProc: Pointer): LongBool; stdcall;
   TpdfSetOnPageBreakProc = function(const IPDF: Pointer; const Data, OnBreakProc: Pointer): LongBool; stdcall;
   TpdfSetOpacity = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetOrientation = function(const IPDF: Pointer; Value: Integer): LongBool; stdcall;
   TpdfSetOrientationEx = function(const IPDF: Pointer; Value: Integer): LongBool; stdcall;
   TpdfSetPageCoords = function(const IPDF: Pointer; PageCoords: Integer): LongBool; stdcall;
   TpdfSetPageFormat = function(const IPDF: Pointer; Value: Integer): LongBool; stdcall;
   TpdfSetPageHeight = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetPageLayout = function(const IPDF: Pointer; Layout: Integer): LongBool; stdcall;
   TpdfSetPageMode = function(const IPDF: Pointer; Mode: Integer): LongBool; stdcall;
   TpdfSetPageWidth = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetPDFVersion = function(const IPDF: Pointer; Version: Integer): LongBool; stdcall;
   TpdfSetPrintSettings = function(const IPDF: Pointer; Mode: TDuplexMode; PickTrayByPDFSize: Integer; NumCopies: Cardinal; PrintScaling: TPrintScaling; const PrintRanges: PCardinal; NumRanges: Cardinal): LongBool; stdcall;
   TpdfSetProgressProc = function(const IPDF: Pointer; const Data, InitProgress, Progress: Pointer): LongBool; stdcall;
   TpdfSetResolution = function(const IPDF: Pointer; Value: Cardinal): LongBool; stdcall;
   TpdfReplaceFontA = function(const IPDF, PDFFont: Pointer; const Name: PAnsiChar; Style: Integer; NameIsFamilyName: LongBool): Integer; stdcall;
   TpdfReplaceFontW = function(const IPDF, PDFFont: Pointer; const Name: PWideChar; Style: Integer; NameIsFamilyName: LongBool): Integer; stdcall;
   TpdfReplaceFontExA = function(const IPDF, PDFFont: Pointer; const FontFile: PAnsiChar; Embed: LongBool): Integer; stdcall;
   TpdfReplaceFontExW = function(const IPDF, PDFFont: Pointer; const FontFile: PWideChar; Embed: LongBool): Integer; stdcall;
   TpdfReplaceICCProfileA = function(const IPDF: Pointer; ColorSpace: Cardinal; const ICCFile: PAnsiChar): Integer; stdcall;
   TpdfReplaceICCProfileW = function(const IPDF: Pointer; ColorSpace: Cardinal; const ICCFile: PWideChar): Integer; stdcall;
   TpdfSetSaveNewImageFormat = function(const IPDF: Pointer; Value: LongBool): LongBool; stdcall;
   TpdfSetSeparationInfo = function(const IPDF: Pointer; Handle: Cardinal): LongBool; stdcall;
   TpdfSetStrokeColor = function(const IPDF: Pointer; Color: Cardinal): LongBool; stdcall;
   TpdfSetStrokeColorEx = function(const IPDF: Pointer; const Color: PByte; NumComponents: Cardinal): LongBool; stdcall;
   TpdfSetStrokeColorSpace = procedure(const IPDF: Pointer; CS: Integer); stdcall;
   TpdfSetTabLen = function(const IPDF: Pointer; TabLen: Integer): LongBool; stdcall;
   TpdfSetTextDrawMode = function(const IPDF: Pointer; Mode: Integer): LongBool; stdcall;
   TpdfSetTextFieldValueA = function(const IPDF: Pointer; Field: Cardinal; const Value, DefValue: PAnsiChar; Align: Integer): LongBool; stdcall;
   TpdfSetTextFieldValueW = function(const IPDF: Pointer; Field: Cardinal; const Value, DefValue: PWideChar; Align: Integer): LongBool; stdcall;
   TpdfSetTextRect = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double): LongBool; stdcall;
   TpdfSetTextRise = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetTextScaling = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSetTrapped = procedure(const IPDF: Pointer; Value: LongBool); stdcall;
   TpdfSetTransparentColor = function(const IPDF: Pointer; AColor: Cardinal): LongBool; stdcall;
   TpdfSetUseExactPwd = function(const IPDF: Pointer; Value: Integer): LongBool; stdcall;
   TpdfSetUseGlobalImpFiles = function(const IPDF: Pointer; Value: LongBool): LongBool; stdcall;
   TpdfSetUseStdFonts = function(const IPDF: Pointer; Value: LongBool): LongBool; stdcall;
   TpdfSetUseSwapFile = function(const IPDF: Pointer; SwapContents: LongBool; SwapLimit: Cardinal): LongBool; stdcall;
   TpdfSetUseSwapFileEx = function(const IPDF: Pointer; SwapContents: LongBool; SwapLimit: Cardinal; const SwapDir: PAnsiChar): LongBool; stdcall;
   TpdfSetUseSystemFonts = function(const IPDF: Pointer; Value: LongBool): LongBool; stdcall;
   TpdfSetUseTransparency = function(const IPDF: Pointer; Value: LongBool): LongBool; stdcall;
   TpdfSetUseVisibleCoords = function(const IPDF: Pointer; Value: LongBool): LongBool; stdcall;
   TpdfSetViewerPreferences = function(const IPDF: Pointer; Value, AddVal: Integer): LongBool; stdcall;
   TpdfSetWMFDefExtent = function(const IPDF: Pointer; Width, Height: Cardinal): LongBool; stdcall;
   TpdfSetWMFPixelPerInch = function(const IPDF: Pointer; Value: Cardinal): LongBool; stdcall;
   TpdfSetWordSpacing = function(const IPDF: Pointer; Value: Double): LongBool; stdcall;
   TpdfSkewCoords = function(const IPDF: Pointer; alpha, beta, OriginX, OriginY: Double): LongBool; stdcall;
   TpdfSortFieldsByIndex = function(IPDF: Pointer): LongBool; stdcall;
   TpdfSortFieldsByName = function(IPDF: Pointer): LongBool; stdcall;
   TpdfStampAnnotA = function(const IPDF: Pointer; SubType: TRubberStamp; PosX, PosY, Width, Height: Double; const Author, Subject, Comment: PAnsiChar): Integer; stdcall;
   TpdfStampAnnotW = function(const IPDF: Pointer; SubType: TRubberStamp; PosX, PosY, Width, Height: Double; const Author, Subject, Comment: PWideChar): Integer; stdcall;
   TpdfStrokePath = function(IPDF: Pointer): LongBool; stdcall;
   TpdfTextAnnotA = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; const Author, Text: PAnsiChar; Icon, Open: Integer): Integer; stdcall;
   TpdfTextAnnotW = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; const Author, Text: PWideChar; Icon, Open: Integer): Integer; stdcall;
   TpdfSetTextFieldValueExA = function(const IPDF: Pointer; Field: Cardinal; const Value: PAnsiChar): LongBool; stdcall;
   TpdfSetTextFieldValueExW = function(const IPDF: Pointer; Field: Cardinal; const Value: PWideChar): LongBool; stdcall;
   TpdfTranslateCoords = function(const IPDF: Pointer; NewOriginX, NewOriginY: Double): LongBool; stdcall;
   TpdfTriangle = function(const IPDF: Pointer; x1, y1, x2, y2, x3, y3: Double; FillMode: Integer): LongBool; stdcall;
   TpdfUTF16ToUTF32 = function(const IPDF: Pointer; const Source: PWideChar): Pointer; stdcall;
   TpdfUTF16ToUTF32Ex = function(const IPDF: Pointer; const Source: PWideChar; var Len: Cardinal): Pointer; stdcall;
   TpdfUTF32ToUTF16 = function(const IPDF: Pointer; const Source: Pointer): PWideChar; stdcall;
   TpdfUTF32ToUTF16Ex = function(const IPDF: Pointer; const Source: Pointer; var Len: Cardinal): PWideChar; stdcall;
   TpdfWebLinkA = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; const URL: PAnsiChar): Integer; stdcall;
   TpdfWebLinkW = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; const URL: PWideChar): Integer; stdcall;
   TpdfWriteAngleTextA = function(const IPDF: Pointer; const AText: PAnsiChar; Angle, PosX, PosY, Radius, YOrigin: Double): LongBool; stdcall;
   TpdfWriteAngleTextW = function(const IPDF: Pointer; const AText: PWideChar; Angle, PosX, PosY, Radius, YOrigin: Double): LongBool; stdcall;
   TpdfWriteFTextA = function(const IPDF: Pointer; Align: Integer; const AText: PAnsiChar): LongBool; stdcall;
   TpdfWriteFTextW = function(const IPDF: Pointer; Align: Integer; const AText: PWideChar): LongBool; stdcall;
   TpdfWriteFTextExA = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; Align: Integer; const AText: PAnsiChar): LongBool; stdcall;
   TpdfWriteFTextExW = function(const IPDF: Pointer; PosX, PosY, Width, Height: Double; Align: Integer; const AText: PWideChar): LongBool; stdcall;
   TpdfWriteTextMatrixA = function(const IPDF: Pointer; var Matrix: TCTM; const AText: PAnsiChar): LongBool; stdcall;
   TpdfWriteTextMatrixW = function(const IPDF: Pointer; var Matrix: TCTM; const AText: PWideChar): LongBool; stdcall;
   TpdfWriteTextMatrixExA = function(const IPDF: Pointer; var M: TCTM; const AText: PAnsiChar; Len: Cardinal): LongBool; stdcall;
   TpdfWriteTextMatrixExW = function(const IPDF: Pointer; var M: TCTM; const AText: PWideChar; Len: Cardinal): LongBool; stdcall;
   TpdfWriteTextA = function(const IPDF: Pointer; PosX, PosY: Double; const AText: PAnsiChar): LongBool; stdcall;
   TpdfWriteTextW = function(const IPDF: Pointer; PosX, PosY: Double; const AText: PWideChar): LongBool; stdcall;
   TpdfWriteTextExA = function(const IPDF: Pointer; PosX, PosY: Double; const AText: PAnsiChar; Len: Cardinal): LongBool; stdcall;
   TpdfWriteTextExW = function(const IPDF: Pointer; PosX, PosY: Double; const AText: PWideChar; Len: Cardinal): LongBool; stdcall;

type TPDF = class(TObject)
 public
   constructor Create(const LibName: WideString = 'dynapdf.dll'); overload;
   destructor Destroy(); override;
   function AddActionToObj(ObjType: TObjType; Event: TObjEvent; ActHandle, ObjHandle: Cardinal): Integer;
   function AddArticle(PosX, PosY, Width, Height: Double): Integer;
   function AddBookmarkA(const Title: AnsiString; Parent: Integer; DestPage: Cardinal; Open: Boolean): Integer;
   function AddBookmarkW(const Title: WideString; Parent: Integer; DestPage: Cardinal; Open: Boolean): Integer;
   function AddBookmarkExA(const Title: AnsiString; Parent: Integer; NamedDest: Cardinal; Open: Boolean): Integer;
   function AddBookmarkExW(const Title: WideString; Parent: Integer; NamedDest: Cardinal; Open: Boolean): Integer;
   function AddButtonImage(BtnHandle: Cardinal; State: TButtonState; const Caption, ImgFile: AnsiString): Boolean;
   function AddButtonImageEx(BtnHandle: Cardinal; State: TButtonState; const Caption: AnsiString; Handle: HBITMAP): Boolean;
   function AddContinueTextA(const AText: AnsiString): Integer;
   function AddContinueTextW(const AText: WideString): Integer;
   function AddDeviceNProcessColorants(DeviceNCS: Cardinal; const Colorants: Array of AnsiString; NumColorants: Cardinal; ProcessCS: TExtColorSpace; Handle: Integer): Boolean;
   function AddDeviceNSeparations(DeviceNCS: Cardinal; const Colorants: Array of AnsiString; SeparationCS: Array of Integer; NumColorants: Cardinal): Boolean;
   function AddFieldToFormAction(Action, AField: Cardinal; Include: Boolean): Integer;
   function AddFieldToHideAction(HideAct, AField: Cardinal): Integer;
   function AddFileCommentA(const AText: AnsiString): Boolean;
   function AddFileCommentW(const AText: WideString): Boolean;
   function AddFontSearchPath(const APath: AnsiString; Recursive: Boolean): Integer;
   function AddImage(Filter: TCompressionFilter; Flags: TImageConversionFlags; var Image: TPDFImage): Boolean;
   function AddJavaScriptA(const Name, Script: AnsiString): Integer;
   function AddJavaScriptW(const Name: AnsiString; const Script: WideString): Integer;
   function AddObjectToLayer(OCG: Cardinal; ObjType: TOCObject; Handle: Cardinal): Boolean;
   function AddPageLabelA(StartRange: Cardinal; Format: TPageLabelFormat; const Prefix: AnsiString; AddNum: Integer): Integer;
   function AddPageLabelW(StartRange: Cardinal; Format: TPageLabelFormat; const Prefix: WideString; AddNum: Integer): Integer;
   function AddRenderingIntentA(const ICCFile: AnsiString): Integer;
   function AddRenderingIntentW(const ICCFile: WideString): Integer;
   function AddRenderingIntentEx(const Buffer: Pointer; BufSize: Cardinal): Integer;
   function AddValToChoiceFieldA(Field: Cardinal; const ExpValue, Value: AnsiString; Selected: Boolean): Boolean; 
   function AddValToChoiceFieldW(Field: Cardinal; const ExpValue: AnsiString; const Value: WideString; Selected: Boolean): Boolean; 
   function Append(): Boolean;
   function ApplyPattern(PattHandle: Integer; ColorMode: TColorMode; Color: Cardinal): Boolean;
   function ApplyShading(ShadHandle: Integer): Boolean;
   function AttachFileA(const FilePath, Description: AnsiString; Compress: Boolean): Integer;
   function AttachFileW(const FilePath, Description: WideString; Compress: Boolean): Integer;
   function AttachFileExA(const Buffer: Pointer; BufSize: Cardinal; const FileName, Description: AnsiString; Compress: Boolean): Integer;
   function AttachFileExW(const Buffer: Pointer; BufSize: Cardinal; const FileName, Description: WideString; Compress: Boolean): Integer;
   function AutoTemplate(Templ: Cardinal; PosX, PosY, Width, Height: Double): Boolean;
   function BeginClipPath(): Boolean;
   function BeginContinueText(PosX, PosY: Double): Integer;
   function BeginLayer(OCG: Cardinal): Boolean;
   function BeginPattern(PatternType: TPatternType; TilingType: TTilingType; Width, Height: Double): Integer;
   function BeginTemplate(Width, Height: Double): Integer;
   function BeginTemplateEx(var BBox: TPDFRect; var Matrix: TCTM): Integer;
   function Bezier_1_2_3(x1, y1, x2, y2, x3, y3: Double): Boolean;
   function Bezier_1_3(x1, y1, x3, y3: Double): Boolean;
   function Bezier_2_3(x2, y2, x3, y3: Double): Boolean;
   function BuildFamilyNameAndStyle(const IFont: Pointer; Name: PAnsiChar{Array[0..127] of AnsiChar}; var Style: TFStyle): Boolean;
   function CalcWidthHeight(OrgWidth, OrgHeight, ScaledWidth, ScaledHeight: Double): Double;
   function ChangeAnnotNameA(Handle: Cardinal; const Name: AnsiString): Boolean;
   function ChangeAnnotNameW(Handle: Cardinal; const Name: WideString): Boolean;
   function ChangeAnnotPos(Handle: Cardinal; PosX, PosY, Width, Height: Double): Boolean;
   function ChangeBookmarkA(ABmk: Integer; const Title: AnsiString; Parent: Integer; DestPage: Cardinal; Open: Boolean): Boolean; 
   function ChangeBookmarkW(ABmk: Integer; const Title: WideString; Parent: Integer; DestPage: Cardinal; Open: Boolean): Boolean; 
   function ChangeFont(Handle: Integer): Boolean;
   function ChangeFontSize(Size: Double): Boolean;
   function ChangeFontStyle(Style: TFStyle): Boolean;
   function ChangeFontStyleEx(Style: TFStyle): Boolean;
   function ChangeJavaScriptA(Handle: Cardinal; const NewScript: AnsiString): Boolean;
   function ChangeJavaScriptW(Handle: Cardinal; const NewScript: WideString): Boolean;
   function ChangeJavaScriptActionA(Handle: Cardinal; const NewScript: AnsiString): Boolean;
   function ChangeJavaScriptActionW(Handle: Cardinal; const NewScript: WideString): Boolean;
   function ChangeJavaScriptNameA(Handle: Cardinal; const Name: AnsiString): Boolean;
   function ChangeJavaScriptNameW(Handle: Cardinal; const Name: WideString): Boolean;
   function ChangeLinkAnnot(Handle: Cardinal; const URL: AnsiString): Boolean;
   function CheckCollection(): Boolean;
   function CheckConformance(ConfType: TConformanceType; Options: TCheckOptions; UserData: Pointer; OnFontNotFound: TOnFontNotFoundProc; OnReplaceICCProfile: TOnReplaceICCProfile): Integer;
   function CheckFieldNames(): Integer;
   function ClearAutoTemplates(): Boolean;
   function ClearHostFonts(): Boolean;
   function ClipPath(ClipMode: TClippingMode; FillMode: TPathFillMode): Boolean;
   function CloseAndSignFile(const CertFile, Password, Reason, Location: AnsiString): Boolean;
   function CloseAndSignFileEx(const OpenPwd, OwnerPwd: AnsiString; KeyLen: TKeyLen; Restrict: TRestrictions; const CertFile, Password, Reason, Location: AnsiString): Boolean;
   function CloseAndSignFileExt(var SigParms: TPDFSigParms): Boolean;
   function CloseFile(): Boolean;
   function CloseFileEx(const OpenPwd, OwnerPwd: AnsiString; KeyLen: TKeyLen; Restrict: TRestrictions): Boolean;
   function CloseImage(): Boolean;
   function CloseImportFile(): Boolean;
   function ClosePath(FillMode: TPathFillMode): Boolean;
   function CloseTag(): Boolean;
   function ConvColor(const Color: PDouble; NumComps: Cardinal; SourceCS: TExtColorSpace; const IColorSpace: Pointer; DestCS: TExtColorSpace): Cardinal;
   function ConvertColors(Flags: TColorConvFlags): Integer;
   function ComputeBBox(var BBox: TPDFRect; Flags: TCompBBoxFlags): Integer;
   function ConvToUnicode(const AString: AnsiString; CP: TCodepage): WideString;
   function Create3DAnnot(PosX, PosY, Width, Height: Double; const Author, AnnotName, U3DFile, ImageFile: AnsiString): Integer;
   function Create3DBackground(const IView: Pointer; BackColor: Cardinal): Boolean;
   function Create3DGotoViewAction(Base3DAnnot: Cardinal; const IView: Pointer; Named: T3DNamedAction): Integer;
   function Create3DProjection(const IView: Pointer; ProjType: T3DProjType; ScaleType: T3DScaleType; Diameter, FOV: Double): Boolean;
   function Create3DView(Base3DAnnot: Cardinal; const Name: AnsiString; SetAsDefault: Boolean; Matrix: PDouble; CamDistance: Double; RM: T3DRenderingMode; LS: T3DLightingSheme): Pointer;
   function CreateAnnotAP(Annot: Cardinal): Integer;
   function CreateArticleThreadA(const ThreadName: AnsiString): Integer;
   function CreateArticleThreadW(const ThreadName: WideString): Integer; 
   function CreateAxialShading(sX, sY, eX, eY, SCenter: Double; SColor, EColor: Cardinal; Extend1, Extend2: Boolean): Integer;
   function CreateButtonA(const Name, Caption: AnsiString; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
   function CreateButtonW(const Name: AnsiString; const Caption: WideString; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
   function CreateCheckBox(const Name, ExpValue: AnsiString; Checked: Boolean; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
   function CreateCIEColorSpace(Base: TExtColorSpace; WhitePoint, BlackPoint, Gamma, Matrix: PSingle): Integer;
   function CreateCollection(View: TColView): Boolean;
   function CreateCollectionFieldA(ColType: TColColumnType; Column: Integer; const Name, Key: AnsiString; Visible, Editable: Boolean): Integer;
   function CreateCollectionFieldW(ColType: TColColumnType; Column: Integer; const Name: WideString; const Key: AnsiString; Visible, Editable: Boolean): Integer;
   function CreateColItemDate(EmbFile: Cardinal; const Key: AnsiString; Date: LongInt; const Prefix: AnsiString): Boolean;
   function CreateColItemNumber(EmbFile: Cardinal; const Key: AnsiString; Value: Double; const Prefix: AnsiString): Boolean;
   function CreateColItemStringA(EmbFile: Cardinal; const Key, Value, Prefix: AnsiString): Boolean;
   function CreateColItemStringW(EmbFile: Cardinal; const Key: AnsiString; const Value, Prefix: WideString): Boolean;
   function CreateComboBox(const Name: AnsiString; Sort: Boolean; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
   function CreateDeviceNColorSpace(const Colorants: Array of PAnsiChar; NumColorants: Cardinal; const PostScriptFunc: AnsiString; Alternate: TExtColorSpace; Handle: Integer): Integer;
   function CreateExtGState(var GS: TPDFExtGState): Integer;
   function CreateGoToAction(DestType: TDestType; PageNum: Cardinal; a, b, c, d: Double): Integer;
   function CreateGoToActionEx(NamedDest: Cardinal): Integer;
   function CreateGoToRAction(const FileName: AnsiString; PageNum: Cardinal): Integer;
   function CreateGoToRActionExA(const FileName: AnsiString; DestName: AnsiString; NewWindow: Boolean): Integer;
   function CreateGoToRActionExW(const FileName: AnsiString; DestName: WideString; NewWindow: Boolean): Integer;
   function CreateGroupField(const Name: AnsiString; Parent: Integer): Integer;
   function CreateHideAction(AField: Cardinal; Hide: Boolean): Integer;
   function CreateICCBasedColorSpaceA(const ICCProfile: AnsiString): Integer;
   function CreateICCBasedColorSpaceW(const ICCProfile: WideString): Integer;
   function CreateImageA(const FileName: AnsiString; Format: TImageFormat): Boolean;
   function CreateImageW(const FileName: WideString; Format: TImageFormat): Boolean;
   function CreateImpDataActionA(const DataFile: AnsiString): Integer;
   function CreateImpDataActionW(const DataFile: WideString): Integer;
   function CreateIndexedColorSpace(Base: TExtColorSpace; Handle: Integer; const ColorTable: Pointer; NumColors: Cardinal): Integer;
   function CreateJSActionA(const Script: AnsiString): Integer;
   function CreateJSActionW(const Script: WideString): Integer;
   function CreateLaunchAction(OP: TFileOP; const FileName, DefDir, Param: AnsiString; NewWindow: Boolean): Integer;
   function CreateListBox(const Name: PAnsiChar; Sort: Boolean; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
   function CreateNamedAction(Action: TNamedAction): Integer;
   function CreateNamedDestA(const Name: AnsiString; DestPage: Cardinal; DestType: TDestType; a, b, c, d: Double): Integer;
   function CreateNamedDestW(const Name: WideString; DestPage: Cardinal; DestType: TDestType; a, b, c, d: Double): Integer;
   function CreateNewPDFA(const OutPDF: AnsiString): Boolean; 
   function CreateNewPDFW(const OutPDF: WideString): Boolean;
   function CreateOCGA(const Name: AnsiString; DisplayInUI, Visible: Boolean; Intent: TOCGIntent): Integer;
   function CreateOCGW(const Name: WideString; DisplayInUI, Visible: Boolean; Intent: TOCGIntent): Integer;
   function CreateOCMD(Visibility: TOCVisibility; const OCGs: PCardinal; Count: Cardinal): Integer;
   function CreateRadialShading(sX, sY, R1, eX, eY, R2, SCenter: Double; SColor, EColor: Cardinal; Extend1, Extend2: Boolean): Integer;
   function CreateRadioButton(const Name, ExpValue: AnsiString; Checked: Boolean; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
   function CreateResetAction(): Integer;
   function CreateSeparationCS(const Colorant: AnsiString; Alternate: TExtColorSpace; Handle: Integer; Color: Cardinal): Integer;
   function CreateSetOCGStateAction(const On_: PCardinal; OnCount: Cardinal; const Off: PCardinal; OffCount: Cardinal; const Toggle: PCardinal; ToggleCount: Cardinal; PreserveRB: Boolean): Integer;
   function CreateSigField(const Name: AnsiString; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
   function CreateSigFieldAP(SigField: Cardinal): Integer;
   function CreateStdPattern(Pattern: TStdPattern; LineWidth, Distance: Double; LineColor, BackColor: Cardinal): Integer;
   function CreateStructureTree(): Boolean;
   function CreateSubmitAction(Flags: TSubmitFlags; const URL: AnsiString): Integer;
   function CreateTextField(const Name: AnsiString; Parent: Integer; Multiline: Boolean; MaxLen: Integer; PosX, PosY, Width, Height: Double): Integer;
   function CreateURIAction(const URL: AnsiString): Integer;
   function DecryptPDFA(const FileName: AnsiString; PwdType: TPwdType; const Password: AnsiString): Integer; 
   function DecryptPDFW(const FileName: WideString; PwdType: TPwdType; const Password: AnsiString): Integer; 
   procedure DeleteAcroForm();
   function DeleteActionFromObj(ObjType: TObjType; ActHandle, ObjHandle: Cardinal): Integer;
   function DeleteActionFromObjEx(ObjType: TObjType; ObjHandle, ActIndex: Cardinal): Integer;
   function DeleteAnnotation(Handle: Cardinal): Boolean;
   function DeleteBookmark(ABmk: Cardinal): Integer;
   function DeleteEmbeddedFile(Handle: Cardinal): Boolean;
   function DeleteField(AField: Cardinal): Integer;
   function DeleteFieldEx(const Name: AnsiString): Integer;
   procedure DeleteJavaScripts(DelJavaScriptActions: Boolean);
   function DeletePage(PageNum: Cardinal): Integer;
   procedure DeletePageLabels();
   function DeleteSeparationInfo(AllPages: Boolean): Boolean;
   function DeleteTemplate(Handle: Cardinal): Boolean;
   function DeleteTemplateEx(Index: Cardinal): Integer;
   procedure DeleteXFAForm();
   function DrawArc(PosX, PosY, Radius, StartAngle, EndAngle: Double): Boolean;
   function DrawArcEx(PosX, PosY, Width, Height, StartAngle, EndAngle: Double): Boolean;
   function DrawChord(PosX, PosY, Width, Height, StartAngle, EndAngle: Double; FillMode: TPathFillMode): Boolean;
   function DrawCircle(PosX, PosY, Radius: Double; FillMode: TPathFillMode): Boolean;
   function DrawPie(PosX, PosY, Width, Height, StartAngle, EndAngle: Double; FillMode: TPathFillMode): Boolean;
   function EditPage(PageNum: Integer): Boolean;
   function EditTemplate(Index: Cardinal): Boolean;
   function EditTemplate2(Handle: Cardinal): Boolean;
   function Ellipse(PosX, PosY, Width, Height: Double; FillMode: TPathFillMode): Boolean;
   function EncryptPDFA(const FileName, OpenPwd, OwnerPwd: AnsiString; KeyLen: TKeyLen; Restrict: TRestrictions): Integer; 
   function EncryptPDFW(const FileName: WideString; const OpenPwd, OwnerPwd: AnsiString; KeyLen: TKeyLen; Restrict: TRestrictions): Integer; 
   function EndContinueText(): Integer;
   function EndLayer(): Boolean;
   function EndPage(): Boolean;
   function EndPattern(): Boolean;
   function EndTemplate(): Boolean;
   function EnumDocFonts(const Data: Pointer; EnumProc: TEnumFontProc2): Integer;
   function EnumHostFonts(const Data: Pointer; EnumProc: TEnumFontProc): Integer;
   function EnumHostFontsEx(const Data: Pointer; EnumProc: TEnumFontProcEx): Integer;
   function ExchangeBookmarks(Bmk1, Bmk2: Integer): Boolean;
   function ExchangePages(First, Second: Cardinal): Boolean;
   function FileAttachAnnotA(PosX, PosY: Double; Icon: TFileAttachIcon; const Author, Desc, AFile: AnsiString; Compress: Boolean): Integer;
   function FileAttachAnnotW(PosX, PosY: Double; Icon: TFileAttachIcon; const Author, Desc, AFile: WideString; Compress: Boolean): Integer;
   function FileLinkA(PosX, PosY, Width, Height: Double; const AFilePath: AnsiString): Integer; 
   function FileLinkW(PosX, PosY, Width, Height: Double; const AFilePath: WideString): Integer; 
   function FindBookmarkA(DestPage: Integer; const Title: AnsiString): Integer; 
   function FindBookmarkW(DestPage: Integer; const Title: WideString): Integer;
   function FindFieldA(const Name: AnsiString): Integer;
   function FindFieldW(const Name: WideString): Integer;
   function FindLinkAnnot(const URL: AnsiString): Integer;
   function FindNextBookmark(): Integer;
   function FinishSignature(const PKCS7Obj: Pointer; Length: Cardinal): Boolean;
   function FlattenAnnots(Flags: TAnnotFlattenFlags): Integer;
   function FlattenForm(): Boolean;
   function FlushPageContent(var Stack: TPDFStack): Boolean;
   procedure FreeImageBuffer();
   function FreePDF(): Boolean;
   function FreeTextAnnotA(PosX, PosY, Width, Height: Double; const Author, AText: AnsiString; Align: TTextAlign): Integer; 
   function FreeTextAnnotW(PosX, PosY, Width, Height: Double; const Author, AText: WideString; Align: TTextAlign): Integer;
   function FreeUniBuf(): Boolean;
   function GetActionCount(): Integer;
   function GetActionType(ActHandle: Cardinal): Integer;
   function GetActionTypeEx(ObjType: TObjType; ObjHandle, ActIndex: Cardinal): Integer;
   function GetActiveFont(): Integer;
   function GetAllocBy(): Integer;
   function GetAnnot(Handle: Cardinal; var Annot: TPDFAnnotation): Boolean;
   function GetAnnotBBox(Handle: Cardinal; var BBox: TPDFRect): Boolean;
   function GetAnnotCount(): Integer;
   function GetAnnotEx(Handle: Cardinal; var Annot: TPDFAnnotationEx): Boolean;
   function GetAnnotFlags(): Integer;
   function GetAnnotLink(Handle: Cardinal): AnsiString;
   function GetAnnotType(Handle: Cardinal): Integer;
   function GetAscent(): Double;
   function GetBBox(Boundary: TPageBoundary; var BBox: TPDFRect): Boolean;
   function GetBidiMode(): TPDFBidiMode;
   function GetBookmark(Handle: Integer; var Bmk: TBookmark): Boolean;
   function GetBookmarkCount(): Integer;
   function GetBorderStyle(): TBorderStyle;
   function GetBuffer(var BufSize: Cardinal): PAnsiChar;
   function GetCapHeight(): Double;
   function GetCharacterSpacing(): Double;
   function GetCheckBoxChar(): Integer;
   function GetCheckBoxCharEx(AField: Cardinal): Integer;
   function GetCheckBoxDefState(AField: Cardinal): Integer;
   function GetCMap(Index: Cardinal; var CMap: TPDFCMap): Boolean;
   function GetCMapCount(): Integer;
   function GetColorSpace(): Integer;
   function GetColorSpaceCount(): Integer;
   function GetColorSpaceObj(Handle: Cardinal; var CS: TPDFColorSpaceObj): Boolean;
   function GetColorSpaceObjEx(const IColorSpace: Pointer; var CS: TPDFColorSpaceObj): Boolean;
   function GetCompressionFilter(): Integer;
   function GetCompressionLevel(): Integer;
   function GetContent(var Buffer: PAnsiChar): Integer;
   function GetDefBitsPerPixel(): Integer;
   function GetDescent(): Double;
   function GetDeviceNAttributes(IAttributes: Pointer; var Attributes: TDeviceNAttributes): Boolean;
   function GetDocInfo(DInfo: TDocumentInfo; var Value: PWideChar): Integer;
   function GetDocInfoCount(): Integer;
   function GetDocInfoEx(Index: Cardinal; var DInfo: TDocumentInfo; var Key, Value: PAnsiChar; var Unicode: LongBool): Integer;
   function GetDrawDirection(): Integer;
   function GetDynaPDFVersion(): AnsiString;
   function GetEmbeddedFile(Handle: Cardinal; var FileSpec: TPDFFileSpec; Decompress: Boolean): Boolean;
   function GetEmbeddedFileCount(): Integer;
   function GetEMFPatternDistance(): Double;
   function GetErrorMessage(): AnsiString;
   function GetErrorMode(): Integer;
   function GetField(Handle: Cardinal; var Field: TPDFField): Boolean;
   function GetFieldBackColor(): Cardinal;
   function GetFieldBorderColor(): Cardinal;
   function GetFieldBorderStyle(AField: Cardinal): Integer;
   function GetFieldBorderWidth(AField: Cardinal): Double;
   function GetFieldChoiceValue(AField, ValIndex: Cardinal; var Value: TPDFChoiceValue): Boolean;
   function GetFieldColor(AField: Cardinal; ColorType: TFieldColor; var ColorSpace: Integer; var Color: Cardinal): Boolean;
   function GetFieldCount(): Integer;
   function GetFieldEx(Handle: Cardinal; var Field: TPDFFieldEx): Boolean;
   function GetFieldEx2(const IField: Pointer; var Field: TPDFFieldEx): Boolean;
   function GetFieldExpValCount(AField: Cardinal): Integer;
   function GetFieldExpValue(AField: Cardinal; var Value: AnsiString): Integer;
   function GetFieldExpValueEx(AField, ValIndex: Cardinal; var Value, ExpValue: AnsiString; var Selected: Boolean): Boolean;
   function GetFieldFlags(AField: Cardinal): Integer;
   function GetFieldGroupType(AField: Cardinal): Integer;
   function GetFieldHighlightMode(AField: Cardinal): Integer;
   function GetFieldIndex(AField: Cardinal): Integer;
   function GetFieldMapName(AField: Cardinal; var Value: Pointer; var Unicode: LongBool): Integer;
   function GetFieldName(AField: Cardinal; var Name: AnsiString): Integer;
   function GetFieldOrientation(AField: Cardinal): Integer;
   function GetFieldTextAlign(AField: Cardinal): Integer;
   function GetFieldTextColor(): Cardinal;
   function GetFieldToolTip(AField: Cardinal; var Value: Pointer; var Unicode: LongBool): Integer;
   function GetFieldType(AField: Cardinal): Integer;
   function GetFillColor(): Cardinal;
   function GetFont(const IFont: Pointer; var F: TPDFFontObj): Boolean;
   function GetFontCount(): Integer;
   function GetFontEx(Handle: Cardinal; var F: TPDFFontObj): Boolean;
   function GetFontOrigin(): Integer;
   procedure GetFontSearchOrder(Order: PFontBaseType {Array[0..3] of TFontBaseType});
   function GetFontSelMode(): Integer;
   function GetFontWeight(): Integer;
   function GetFTextHeightA(Align: TTextAlign; const AText: AnsiString): Double;
   function GetFTextHeightExA(Width: Double; Align: TTextAlign; const AText: AnsiString): Double;
   function GetFTextHeightExW(Width: Double; Align: TTextAlign; const AText: WideString): Double;
   function GetFTextHeightW(Align: TTextAlign; const AText: WideString): Double;
   function GetGStateFlags(): Integer;
   function GetIconColor(): Integer;
   function GetImageBuffer(var BufSize: Cardinal): PAnsiChar;
   function GetImageCount(const FileName: AnsiString): Integer;
   function GetImageCountEx(const Buffer: Pointer; BufSize: Cardinal): Integer;
   function GetImageHeight(Handle: Cardinal): Integer;
   function GetImageWidth(Handle: Cardinal): Integer;
   function GetImportFlags(): Cardinal;
   function GetInBBox(PageNum: Cardinal; Boundary: TPageBoundary; var BBox: TPDFRect): Boolean;
   function GetInDocInfo(DInfo: TDocumentInfo; var Value: PWideChar): Integer;
   function GetInDocInfoCount(): Integer;
   function GetInDocInfoEx(Index: Cardinal; var DInfo: TDocumentInfo; var Key, Value: PAnsiChar; var Unicode: LongBool): Boolean;
   function GetInEncryptionFlags(): Integer;
   function GetInIsCollection(): Boolean;
   function GetInIsEncrypted(): Boolean;
   function GetInIsSigned(): Boolean;
   function GetInIsTrapped(): Boolean;
   function GetInIsXFAForm(): Boolean;
   function GetInNamedDest(Index: Cardinal; var Dest: TPDFNamedDest): Boolean;
   function GetInNamedDestCount(): Integer;
   function GetInOrientation(PageNum: Integer): Integer;
   function GetInPageCount(): Integer;
   function GetInPDFVersion(): Integer;
   function GetInPrintSettings(var Settings: TPDFPrintSettings): Boolean;
   function GetIsFixedPitch(): Integer;
   function GetItalicAngle(): Double;
   function GetJavaScript(Handle: Cardinal; var Len: Cardinal; var Unicode: LongBool): PAnsiChar;
   function GetJavaScriptAction(Handle: Cardinal; var Len: Cardinal; var Unicode: LongBool): PAnsiChar;
   function GetJavaScriptAction2(ObjType: TObjType; ObjHandle, ActIndex: Cardinal; var Len: Cardinal; var Unicode: LongBool; var Event: TObjEvent): PAnsiChar;
   function GetJavaScriptCount(): Integer;
   function GetJavaScriptEx(const Name: AnsiString; var Len: Cardinal; var Unicode: LongBool): PAnsiChar;
   function GetJavaScriptName(Handle: Cardinal; var Len: Cardinal; var Unicode: LongBool): PAnsiChar;
   function GetJPEGQuality(): Integer;
   function GetLanguage(): AnsiString;
   function GetLastTextPosX(): Double;
   function GetLastTextPosY(): Double;
   function GetLeading(): Double;
   function GetLineCapStyle(): Integer;
   function GetLineJoinStyle(): Integer;
   function GetLineWidth(): Double;
   function GetLinkHighlightMode(): Integer;
   function GetLogMetafileSize(const FileName: AnsiString; var R: TRectL): Boolean;
   function GetLogMetafileSizeEx(const Buffer: Pointer; BufSize: Cardinal; var R: TRectL): Boolean;
   function GetMatrix(var M: TCTM): Boolean;
   function GetMaxFieldLen(TxtField: Cardinal): Integer;
   function GetMetaConvFlags(): Cardinal;
   function GetMissingGlyphs(var Count: Cardinal): PCardinal;
   function GetMiterLimit(): Double;
   function GetNeedAppearance(): Boolean;
   function GetObjActionCount(ObjType: TObjType; ObjHandle: Cardinal): Integer;
   function GetOpacity(): Double;
   function GetOrientation(): Integer;
   function GetPageAnnot(Index: Cardinal; var Annot: TPDFAnnotation): Boolean;
   function GetPageAnnotCount(): Integer;
   function GetPageAnnotEx(Index: Cardinal; var Annot: TPDFAnnotationEx): Boolean;
   function GetPageCoords(): TPageCoord;
   function GetPageCount(): Integer;
   function GetPageField(Index: Cardinal; var Field: TPDFField): Boolean;
   function GetPageFieldCount(): Integer;
   function GetPageFieldEx(Index: Cardinal; var Field: TPDFFieldEx): Boolean;
   function GetPageHeight(): Double;
   function GetPageLabel(Index: Cardinal; var Lbl: TPDFPageLabel): Boolean;
   function GetPageLabelCount(): Integer;
   function GetPageLayout(): Integer;
   function GetPageMode(): Integer;
   function GetPageNum(): Integer;
   function GetPageText(var Stack: TPDFStack): Boolean;
   function GetPageWidth(): Double;
   function GetPDFVersion(): Integer;
   function GetPrintSettings(var Settings: TPDFPrintSettings): Boolean;
   function GetResolution(): Integer;
   function GetSaveNewImageFormat(): Integer;
   function GetSeparationInfo(var Colorant: AnsiString; var CS: TExtColorSpace): Boolean;
   function GetSpaceWidth(const IFont: Pointer; FontSize: Double): Double;
   function GetStrokeColor(): Cardinal;
   function GetTabLen(): Integer;
   function GetTemplCount(): Integer;
   function GetTemplHandle(): Integer;
   function GetTemplHeight(TmplHandle: Integer): Double;
   function GetTemplWidth(TmplHandle: Integer): Double;
   function GetTextDrawMode(): Integer;
   function GetTextFieldValue(AField: Cardinal; var Value: PAnsiChar; var ValIsUnicode: LongBool; var DefValue: PAnsiChar; var DefValIsUnicode: LongBool): Boolean;
   function GetTextRect(var PosX, PosY, Width, Height: Double): Boolean;
   function GetTextRise(): Double;
   function GetTextScaling(): Double;
   function GetTextWidth(const IFont: Pointer; const Text: PAnsiChar; Len: Cardinal; CharSpacing, WordSpacing, TextScale: Single): Double;
   function GetTextWidthA(const AText: AnsiString): Double;
   function GetTextWidthW(const AText: WideString): Double;
   function GetTextWidthExA(const AText: AnsiString; Len: Cardinal): Double;
   function GetTextWidthExW(const AText: WideString; Len: Cardinal): Double;
   function GetTrapped(): Boolean;
   function GetTransparentColor(): Cardinal;
   function GetUseExactPwd(): Boolean;
   function GetUseGlobalImpFiles(): Boolean;
   function GetUserRights(): Integer;
   function GetUseStdFonts(): Boolean;
   function GetUseSystemFonts(): Boolean;
   function GetUseTransparency(): Boolean;
   function GetUseVisibleCoords(): Boolean;
   function GetViewerPrefrences(var Preference, AddVal: Integer): Boolean;
   function GetWMFDefExtent(var Width, Height: Cardinal): Boolean;
   function GetWMFPixelPerInch(): Integer;
   function GetWordSpacing(): Double;
   function HaveOpenDoc(): Boolean;
   function HighlightAnnotA(SubType: TAnnotType; PosX, PosY, Width, Height: Double; Color: Cardinal; const Author, Subject, Comment: AnsiString): Integer;
   function HighlightAnnotW(SubType: TAnnotType; PosX, PosY, Width, Height: Double; Color: Cardinal; const Author, Subject, Comment: WideString): Integer;
   function ImportBookmarks(): Integer;
   function ImportCatalogObjects(): Boolean;
   function ImportDocInfo(): Boolean;
   function ImportPage(PageNum: Cardinal): Integer;
   function ImportPageEx(PageNum: Cardinal; ScaleFactX, ScaleFactY: Double): Integer;
   function ImportPDFFile(DestPage: Cardinal; ScaleFactX, ScaleFactY: Double): Integer;
   function InitExtGState(var GS: TPDFExtGState): Boolean;
   function InitStack(var Stack: TPDFStack): Boolean;
   function InsertBMPFromBuffer(PosX, PosY, ScaleWidth, ScaleHeight: Double; const Buffer: Pointer): Integer;
   function InsertBMPFromHandle(PosX, PosY, ScaleWidth, ScaleHeight: Double; Handle: HBITMAP): Integer;
   function InsertImage(PosX, PosY, ScaleWidth, ScaleHeight: Double; const AFile: AnsiString): Integer;
   function InsertImageEx(PosX, PosY, ScaleWidth, ScaleHeight: Double; const AFile: AnsiString; Index: Cardinal): Integer;
   function InsertImageFromBuffer(PosX, PosY, ScaleWidth, ScaleHeight: Double; const Buffer: Pointer; BufSize, Index: Cardinal): Integer;
   function InsertMetafile(const FileName: AnsiString; PosX, PosY, Width, Height: Double): Boolean;
   function InsertMetafileEx(const Buffer: Pointer; BufSize: Cardinal; PosX, PosY, Width, Height: Double): Boolean;
   function InsertMetafileExt(const FileName: AnsiString; var View: TRectL; PosX, PosY, Width, Height: Double): Boolean;
   function InsertMetafileExtEx(const Buffer: Pointer; BufSize: Cardinal; var View: TRectL; PosX, PosY, Width, Height: Double): Boolean;
   function InsertMetafileFromHandle(Handle: HENHMETAFILE; PosX, PosY, Width, Height: Double): Boolean;
   function InsertMetafileFromHandleEx(Handle: HENHMETAFILE; var View: TRectL; PosX, PosY, Width, Height: Double): Boolean;
   function InsertRawImage(const Buffer: Pointer; BitsPerPixel, ColorCount, ImgWidth, ImgHeight: Cardinal; PosX, PosY, ScaleWidth, ScaleHeight: Double): Integer;
   function IsBidiText(const AText: WideString): Integer;
   function IsColorPage(GrayIsColor: Boolean): Integer;
   function IsEmptyPage(): Integer;
   function IsWrongPwd(ErrCode: Integer): Boolean;
   function LineTo(PosX, PosY: Double): Boolean;
   function LoadCMap(const CMapName: AnsiString; Embed: Boolean): Integer;
   function LoadFDFDataA(const FileName, Password: AnsiString; Flags: Cardinal): Boolean;
   function LoadFDFDataW(const FileName: WideString; const Password: AnsiString; Flags: Cardinal): Boolean;
   function LoadFDFDataEx(const Buffer: Pointer; BufSize: Cardinal; const Password: AnsiString; Flags: Cardinal): Boolean;
   function LoadFont(const Buffer: Pointer; BufSize: Cardinal; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage): Integer;
   function MovePage(Source, Dest: Cardinal): Boolean;
   function MoveTo(PosX, PosY: Double): Boolean;
   procedure MultiplyMatrix(var M1, M2, NewMatrix: TCTM);
   function OpenImportBuffer(const Buffer: Pointer; BufSize: Cardinal; PwdType: TPwdType; const Password: AnsiString): Integer;
   function OpenImportFileA(const FileName: AnsiString; PwdType: TPwdType; const Password: AnsiString): Integer;
   function OpenImportFileW(const FileName: WideString; PwdType: TPwdType; const Password: AnsiString): Integer; 
   function OpenOutputFileA(const OutPDF: AnsiString): Boolean;
   function OpenOutputFileW(const OutPDF: WideString): Boolean;
   function OpenTagA(Tag: TPDFBaseTag; const Lang, AltText, Expansion: AnsiString): Boolean;
   function OpenTagW(Tag: TPDFBaseTag; const Lang: AnsiString; const AltText, Expansion: WideString): Boolean;
   function PageLink(PosX, PosY, Width, Height: Double; DestPage: Cardinal): Integer;
   function PageLink2(PosX, PosY, Width, Height: Double; NamedDest: Cardinal): Integer;
   function PageLinkEx(PosX, PosY, Width, Height: Double; DestType: TDestType; DestPage: Cardinal; a, b, c, d: Double): Integer;
   function ParseContent(const UserData: Pointer; var Stack: TPDFParseInterface; Flags: TParseFlags): Boolean;
   function PlaceImage(ImgHandle: Integer; PosX, PosY, ScaleWidth, ScaleHeight: Double): Boolean;
   function PlaceSigFieldValidateIcon(SigField: Cardinal; PosX, PosY, Width, Height: Double): Boolean;
   function PlaceTemplate(TmplHandle: Integer; PosX, PosY, ScaleWidth, ScaleHeight: Double): Boolean;
   function PlaceTemplateEx(TmplHandle: Integer; PosX, PosY, ScaleWidth, ScaleHeight: Double): Boolean;
   function PlaceTemplByMatrix(TmplHandle: Integer): Boolean;
   function ReadImageFormat(const FileName: AnsiString; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): Boolean;
   function ReadImageFormat2(const FileName: AnsiString; Index: Cardinal; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): Boolean;
   function ReadImageFormatEx(hBitmap: HBITMAP; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): Boolean;
   function ReadImageFormatFromBuffer(const Buffer: Pointer; BufSize, Index: Cardinal; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): Boolean;
   function ReadImageResolution(const FileName: AnsiString; Index: Cardinal; var ResX, ResY: Cardinal): Boolean;
   function ReadImageResolutionEx(const Buffer: Pointer; BufSize, Index: Cardinal; var ResX, ResY: Cardinal): Boolean;
   function Rectangle(PosX, PosY, Width, Height: Double; FillMode: TPathFillMode): Boolean;
   function ReEncryptPDFA(const FileName: AnsiString; PwdType: TPwdType; const InPwd, NewOpenPwd, NewOwnerPwd: AnsiString; NewKeyLen: TKeyLen; Restrict: TRestrictions): Integer; 
   function ReEncryptPDFW(const FileName: WideString; PwdType: TPwdType; const InPwd, NewOpenPwd, NewOwnerPwd: AnsiString; NewKeyLen: TKeyLen; Restrict: TRestrictions): Integer; 
   function RenameSpotColor(const Colorant, NewName: AnsiString): Integer;
   function ReplaceFontA(const PDFFont: Pointer; const Name: AnsiString; Style: TFStyle; NameIsFamilyName: Boolean): Integer;
   function ReplaceFontW(const PDFFont: Pointer; const Name: WideString; Style: TFStyle; NameIsFamilyName: Boolean): Integer;
   function ReplaceFontExA(const PDFFont: Pointer; const FontFile: AnsiString; Embed: Boolean): Integer;
   function ReplaceFontExW(const PDFFont: Pointer; const FontFile: WideString; Embed: Boolean): Integer;
   function ReplaceICCProfileA(ColorSpace: Cardinal; const ICCFile: AnsiString): Integer;
   function ReplaceICCProfileW(ColorSpace: Cardinal; const ICCFile: WideString): Integer;
   function ReplacePageText(const NewText: AnsiString; var Stack: TPDFStack): Boolean;
   function ReplacePageTextEx(const NewText: AnsiString; var Stack: TPDFStack): Boolean; overload;
   function ReplacePageTextEx(const NewText: WideString; var Stack: TPDFStack): Boolean; overload;
   function ResetLineDashPattern(): Boolean;
   function RestoreGraphicState(): Boolean;
   function RotateCoords(alpha, OriginX, OriginY: Double): Boolean;
   function RoundRect(PosX, PosY, Width, Height, Radius: Double; FillMode: TPathFillMode): Boolean;
   function RoundRectEx(PosX, PosY, Width, Height, rWidth, rHeight: Double; FillMode: TPathFillMode): Boolean;
   function SaveGraphicState(): Boolean;
   function ScaleCoords(sx, sy: Double): Boolean;
   function SelfTest(): Boolean;
   function Set3DAnnotProps(Annot: Cardinal; ActType: T3DActivationType; DeActType: T3DDeActivateType; InstType: T3DInstanceType; DeInstType: T3DDeActInstance; DisplToolbar, DisplModelTree: Boolean): Boolean;
   function Set3DAnnotScriptA(Annot: Cardinal; const Value: AnsiString): Boolean;
   function SetAllocBy(Value: Integer): Boolean;
   function SetAnnotBorderStyle(Handle: Cardinal; Style: TBorderStyle): Boolean;
   function SetAnnotBorderWidth(Handle: Cardinal; LineWidth: Double): Boolean;
   function SetAnnotColor(Handle: Cardinal; ColorType: TAnnotColor; CS: TPDFColorSpace; Color: Cardinal): Boolean;
   function SetAnnotFlags(Flags: TAnnotFlags): Boolean;
   function SetAnnotFlagsEx(Handle: Cardinal; Flags: Integer): Boolean;
   function SetAnnotHighlightMode(Handle: Cardinal; Mode: THighlightMode): Boolean;
   function SetAnnotIcon(Handle: Cardinal; Icon: TAnnotIcon): Boolean;
   function SetAnnotOpenState(Handle: Cardinal; Open: Boolean): Boolean;
   function SetAnnotStringA(Handle: Cardinal; StringType: TAnnotString; const Value: AnsiString): Boolean;
   function SetAnnotStringW(Handle: Cardinal; StringType: TAnnotString; const Value: WideString): Boolean;
   function SetAnnotSubjectA(Handle: Cardinal; const Value: AnsiString): Boolean;
   function SetAnnotSubjectW(Handle: Cardinal; const Value: WideString): Boolean;
   function SetBBox(Boundary: TPageBoundary; X0, Y0, Width, Height: Double): Boolean;
   function SetBidiMode(Mode: TPDFBidiMode): Boolean;
   function SetBookmarkDest(ABmk: Integer; DestType: TDestType; a, b, c, d: Double): Boolean;
   function SetBookmarkStyle(ABmk: Integer; Style: TFStyle; RGBColor: Cardinal): Boolean;
   function SetBorderStyle(Style: TBorderStyle): Boolean;
   function SetCharacterSpacing(Value: Double): Boolean;
   function SetCheckBoxChar(CheckBoxChar: TCheckBoxChar): Boolean;
   function SetCheckBoxDefState(AField: Cardinal; Checked: Boolean): Boolean;
   function SetCheckBoxState(AField: Cardinal; Checked: Boolean): Boolean;
   function SetCIDFontA(CMapHandle: Cardinal; const Name: AnsiString; Style: TFStyle; Size: Double; Embed: Boolean): Integer;
   function SetCIDFontW(CMapHandle: Cardinal; const Name: WideString; Style: TFStyle; Size: Double; Embed: Boolean): Integer;
   function SetCMapDirA(const Path: AnsiString; Recursive: Boolean): Integer;
   function SetCMapDirW(const Path: WideString; Recursive: Boolean): Integer;
   function SetColDefFile(EmbFile: Cardinal): Boolean;
   function SetColors(Color: Cardinal): Boolean;
   function SetColorSpace(ColorSpace: TPDFColorSpace): Boolean;
   function SetColSortField(ColField: Cardinal; AscendingOrder: Boolean): Boolean;
   function SetCompressionFilter(Filter: TCompressionFilter): Boolean;
   function SetCompressionLevel(CompressLevel: TCompressionLevel): Boolean;
   function SetContent(const Buffer: PAnsiChar; BufSize: Cardinal): Boolean;
   function SetDateTimeFormat(TxtField: Cardinal; Fmt: TPDFDateTime): Boolean;
   function SetDefBitsPerPixel(Value: Integer): Boolean;
   function SetDocInfoA(DInfo: TDocumentInfo; const Value: AnsiString): Boolean; 
   function SetDocInfoW(DInfo: TDocumentInfo; const Value: WideString): Boolean;
   function SetDocInfoExA(DInfo: TDocumentInfo; const Key, Value: AnsiString): Boolean;
   function SetDocInfoExW(DInfo: TDocumentInfo; const Key: AnsiString; const Value: WideString): Boolean;
   function SetDrawDirection(Direction: TDrawDirection): Boolean;
   function SetEMFFrameDPI(DPIX, DPIY: Cardinal): Boolean;
   function SetEMFPatternDistance(Value: Double): Boolean;
   function SetErrorMode(ErrMode: TErrMode): Boolean;
   function SetExtColorSpace(Handle: Cardinal): Boolean;
   function SetExtGState(Handle: Cardinal): Boolean;
   function SetFieldBackColor(AColor: Cardinal): Boolean;
   function SetFieldBBox(AField: Cardinal; var BBox: TPDFRect): Boolean;
   function SetFieldBorderColor(AColor: Cardinal): Boolean;
   function SetFieldBorderStyle(AField: Cardinal; Style: TBorderStyle): Boolean;
   function SetFieldBorderWidth(AField: Cardinal; LineWidth: Double): Boolean;
   function SetFieldColor(AField: Cardinal; ColorType: TFieldColor; CS: TPDFColorSpace; Color: Cardinal): Boolean;
   function SetFieldExpValue(AField, ValIndex: Cardinal; const Value, ExpValue: AnsiString; Selected: Boolean): Boolean;
   function SetFieldFlags(AField: Cardinal; Flags: TFieldFlags; Reset: Boolean): Boolean;
   function SetFieldFontSize(AField: Cardinal; FontSize: double): Boolean;
   function SetFieldHighlightMode(AField: Cardinal; Mode: THighlightMode): Boolean;
   function SetFieldIndex(AField, Index: Cardinal): Boolean;
   function SetFieldMapNameA(AField: Cardinal; const Name: AnsiString): Boolean;
   function SetFieldMapNameW(AField: Cardinal; const Name: WideString): Boolean;
   function SetFieldName(AField: Cardinal; const NewName: AnsiString): Boolean;
   function SetFieldOrientation(AField: Cardinal; Value: Integer): Boolean;
   function SetFieldTextAlign(AField: Cardinal; Align: TTextAlign): Boolean;
   function SetFieldTextColor(Color: Cardinal): Boolean;
   function SetFieldToolTipA(AField: Cardinal; const Value: AnsiString): Boolean;
   function SetFieldToolTipW(AField: Cardinal; const Value: WideString): Boolean;
   function SetFillColor(Color: Cardinal): Boolean;
   function SetFillColorEx(const Color: Array of Byte; NumComponents: Cardinal): Boolean;
   procedure SetFillColorSpace(CS: TPDFColorSpace);
   function SetFontA(const Name: AnsiString; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage = cp1252): Integer; 
   function SetFontW(const Name: WideString; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage = cp1252): Integer; 
   function SetFontExA(const Name: AnsiString; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage): Integer;
   function SetFontExW(const Name: WideString; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage): Integer;
   function SetFontOrigin(Origin: TOrigin): Boolean;
   procedure SetFontSearchOrder(Order: PFontBaseType{Array[0..3] of TFontBaseType});
   function SetFontSelMode(Mode: TFontSelMode): Boolean;
   function SetFontWeight(Weight: Integer): Boolean;
   function SetGStateFlags(Flags: TGStateFlags; Reset: Boolean): Boolean;
   function SetIconColor(Color: Cardinal): Boolean;
   function SetImportFlags(Flags: TImportFlags): Boolean;
   function SetImportFlags2(Flags: TImportFlags2): Boolean;
   function SetItalicAngle(Angle: Double): Boolean;
   function SetJPEGQuality(Value: Integer): Boolean;
   function SetLanguage(const ISOTag: AnsiString): Boolean;
   function SetLeading(Value: Double): Boolean;
   function SetLicenseKey(const Key: AnsiString): Boolean;
   function SetLineCapStyle(Style: TLineCapStyle): Boolean;
   function SetLineDashPattern(const Dash: AnsiString; Phase: Integer): Boolean;
   function SetLineDashPatternEx(const Dash: Array of Double; NumValues: Cardinal; Phase: Integer): Boolean;
   function SetLineJoinStyle(Style: TLineJoinStyle): Boolean;
   function SetLineWidth(Value: Double): Boolean;
   function SetLinkHighlightMode(Mode: THighlightMode): Boolean;
   function SetListFont(Handle: Cardinal): Boolean;
   function SetMatrix(var Matrix: TCTM): Boolean;
   function SetMaxFieldLen(TxtField: Cardinal; MaxLen: Integer): Boolean;
   function SetMetaConvFlags(Flags: TMetaFlags): Boolean;
   function SetMiterLimit(Value: Double): Boolean;
   function SetNeedAppearance(Value: Boolean): Boolean;
   function SetNumberFormat(TxtField: Cardinal; Sep: TDecSeparator; DecPlaces: Cardinal; NegStyle: TNegativeStyle; const CurrStr: AnsiString; Prepend: Boolean): Boolean;
   function SetOnErrorProc(const Data, ErrProc: Pointer): Boolean;
   function SetOnPageBreakProc(const Data, OnBreakProc: Pointer): Boolean;
   function SetOpacity(Value: Double): Boolean;
   function SetOrientation(Value: Integer): Boolean;
   function SetOrientationEx(Value: Integer): Boolean;
   function SetPageCoords(PageCoords: TPageCoord): Boolean;
   function SetPageFormat(Value: TPageFormat): Boolean;
   function SetPageHeight(Value: Double): Boolean;
   function SetPageLayout(Layout: TPageLayout): Boolean;
   function SetPageMode(Mode: TPageMode): Boolean;
   function SetPageWidth(Value: Double): Boolean;
   function SetPDFVersion(Version: TPDFVersion): Boolean;
   function SetPrintSettings(Mode: TDuplexMode; PickTrayByPDFSize: Integer; NumCopies: Cardinal; PrintScaling: TPrintScaling; const PrintRanges: Array of Cardinal; NumRanges: Cardinal): Boolean;
   function SetProgressProc(const Data, InitProgressProc, ProgressProc: Pointer): Boolean;
   function SetResolution(Value: Cardinal): Boolean;
   function SetSaveNewImageFormat(Value: Boolean): Boolean;
   function SetSeparationInfo(Handle: Cardinal): Boolean;
   function SetStrokeColor(Color: Cardinal): Boolean;
   function SetStrokeColorEx(const Color: Array of Byte; NumComponents: Cardinal): Boolean;
   procedure SetStrokeColorSpace(CS: TPDFColorSpace);
   function SetTabLen(TabLen: Integer): Boolean;
   function SetTextDrawMode(Mode: TDrawMode): Boolean;
   function SetTextFieldValueA(Field: Cardinal; const Value, DefValue: AnsiString; Align: TTextAlign): Boolean; 
   function SetTextFieldValueW(Field: Cardinal; const Value, DefValue: WideString; Align: TTextAlign): Boolean;
   function SetTextFieldValueExA(Field: Cardinal; const Value: AnsiString): Boolean;
   function SetTextFieldValueExW(Field: Cardinal; const Value: WideString): Boolean;
   function SetTextRect(PosX, PosY, Width, Height: Double): Boolean;
   function SetTextRise(Value: Double): Boolean;
   function SetTextScaling(Value: Double): Boolean;
   procedure SetTrapped(Value: Boolean);
   function SetTransparentColor(AColor: Cardinal): Boolean;
   function SetUseExactPwd(Value: Boolean): Boolean;
   function SetUseGlobalImpFiles(Value: Boolean): Boolean;
   function SetUseStdFonts(Value: Boolean): Boolean;
   function SetUseSwapFile(SwapContents: Boolean; SwapLimit: Cardinal): Boolean;
   function SetUseSwapFileEx(SwapContents: Boolean; SwapLimit: Cardinal; const SwapDir: AnsiString): Boolean;
   function SetUseSystemFonts(Value: Boolean): Boolean;
   function SetUseTransparency(Value: Boolean): Boolean;
   function SetUseVisibleCoords(Value: Boolean): Boolean;
   function SetViewerPreferences(Value: TViewerPreference; AddVal: TViewPrefAddVal): Boolean;
   function SetWMFDefExtent(Width, Height: Cardinal): Boolean;
   function SetWMFPixelPerInch(Value: Cardinal): Boolean;
   function SetWordSpacing(Value: Double): Boolean;
   function SkewCoords(alpha, beta, OriginX, OriginY: Double): Boolean;
   function SortFieldsByIndex(): Boolean;
   function SortFieldsByName(): Boolean;
   function StampAnnotA(SubType: TRubberStamp; PosX, PosY, Width, Height: Double; const Author, Subject, Comment: AnsiString): Integer;
   function StampAnnotW(SubType: TRubberStamp; PosX, PosY, Width, Height: Double; const Author, Subject, Comment: WideString): Integer;
   function StrokePath(): Boolean;
{$ifdef VER120}
   function ToUnicodeA(const S: AnsiString): WideString;
   function ToUnicodeW(const S: PAnsiChar; Len: Integer): WideString;
{$else}
   function ToUnicode(const S: AnsiString): WideString; overload;
   function ToUnicode(const S: PAnsiChar; Len: Integer): WideString; overload;
{$endif}
   function TranslateRawCode(const IFont: Pointer; const Text: PAnsiChar; Len: Cardinal; var Width: Double; OutText: PWideChar{Array[0..31] of WideChar}; var OutLen: Integer; var Decoded: LongBool; CharSpacing, WordSpacing, TextScale: Single): Cardinal;
   function TranslateString(var Stack: TPDFStack; var Buffer: PWideChar; var Size: Cardinal; Flags: Cardinal): Integer;
   function TranslateString2(const IFont: Pointer; const Text: PAnsiChar; Len: Cardinal; OutBuf: WideString; Size, Flags: Cardinal): Integer;
   function TextAnnotA(PosX, PosY, Width, Height: Double; const Author, Text: AnsiString; Icon: TAnnotIcon; Open: Boolean): Integer;
   function TextAnnotW(PosX, PosY, Width, Height: Double; const Author, Text: WideString; Icon: TAnnotIcon; Open: Boolean): Integer;
   function TranslateCoords(NewOriginX, NewOriginY: Double): Boolean;
   function Triangle(x1, y1, x2, y2, x3, y3: Double; FillMode: TPathFillMode): Boolean;
   function UTF16ToUTF32(const Source: WideString): Pointer;
   function UTF16ToUTF32Ex(const Source: WideString; var Len: Cardinal): Pointer;
   function UTF32ToUTF16(const Source: Pointer): PWideChar;
   function UTF32ToUTF16Ex(const Source: Pointer; var Len: Cardinal): PWideChar;
   function WebLinkA(PosX, PosY, Width, Height: Double; const URL: AnsiString): Integer; 
   function WebLinkW(PosX, PosY, Width, Height: Double; const URL: WideString): Integer; 
   function WeigthFromStyle(Style: TFStyle): Cardinal;
   function WeightToStyle(Weight: Cardinal): TFStyle;
   function WidthFromStyle(Style: TFStyle): Cardinal;
   function WidthToStyle(Width: Cardinal): TFStyle;
   function WriteAngleTextA(const AText: AnsiString; Angle, PosX, PosY, Radius, YOrigin: Double): Boolean; 
   function WriteAngleTextW(const AText: WideString; Angle, PosX, PosY, Radius, YOrigin: Double): Boolean; 
   function WriteFTextA(Align: TTextAlign; const AText: AnsiString): Boolean; 
   function WriteFTextW(Align: TTextAlign; const AText: WideString): Boolean; 
   function WriteFTextExA(PosX, PosY, Width, Height: Double; Align: TTextAlign; const AText: AnsiString): Boolean;
   function WriteFTextExW(PosX, PosY, Width, Height: Double; Align: TTextAlign; const AText: WideString): Boolean; 
   function WriteTextA(PosX, PosY: Double; const AText: AnsiString): Boolean;
   function WriteTextW(PosX, PosY: Double; const AText: WideString): Boolean;
   function WriteTextExA(PosX, PosY: Double; const AText: AnsiString; Len: Cardinal): Boolean;
   function WriteTextExW(PosX, PosY: Double; const AText: WideString; Len: Cardinal): Boolean;
   function WriteTextMatrixA(var Matrix: TCTM; const AText: AnsiString): Boolean;
   function WriteTextMatrixW(var Matrix: TCTM; const AText: WideString): Boolean;
   function WriteTextMatrixExA(var M: TCTM; const AText: AnsiString; Len: Cardinal): Boolean;
   function WriteTextMatrixExW(var M: TCTM; const AText: WideString; Len: Cardinal): Boolean;
 protected
   fntBuildFamilyNameAndStyle: TfntBuildFamilyNameAndStyle;
   fntGetFont: TfntGetFont;
   fntGetTextWidth: TfntGetTextWidth;
   fntGetSpaceWidth: TfntGetSpaceWidth;
   fntTranslateRawCode: TfntTranslateRawCode;
   fntTranslateString: TfntTranslateString;
   fntTranslateString2: TfntTranslateString2;

   pdfAddActionToObj: TpdfAddActionToObj;
   pdfAddArticle: TpdfAddArticle;
   pdfAddBookmarkA: TpdfAddBookmarkA;
   pdfAddBookmarkW: TpdfAddBookmarkW;
   pdfAddBookmarkExA: TpdfAddBookmarkExA;
   pdfAddBookmarkExW: TpdfAddBookmarkExW;
   pdfAddButtonImage: TpdfAddButtonImage;
   pdfAddButtonImageEx: TpdfAddButtonImageEx;
   pdfAddContinueTextA: TpdfAddContinueTextA;
   pdfAddContinueTextW: TpdfAddContinueTextW;
   pdfAddDeviceNProcessColorants: TpdfAddDeviceNProcessColorants;
   pdfAddDeviceNSeparations: TpdfAddDeviceNSeparations;
   pdfAddFieldToFormAction: TpdfAddFieldToFormAction;
   pdfAddFieldToHideAction: TpdfAddFieldToHideAction;
   pdfAddFileCommentA: TpdfAddFileCommentA;
   pdfAddFileCommentW: TpdfAddFileCommentW;
   pdfAddFontSearchPath: TpdfAddFontSearchPath;
   pdfAddImage: TpdfAddImage;
   pdfAddJavaScriptA: TpdfAddJavaScriptA;
   pdfAddJavaScriptW: TpdfAddJavaScriptW;
   pdfAddObjectToLayer: TpdfAddObjectToLayer;
   pdfAddPageLabelA: TpdfAddPageLabelA;
   pdfAddPageLabelW: TpdfAddPageLabelW;
   pdfAddRenderingIntentA: TpdfAddRenderingIntentA;
   pdfAddRenderingIntentW: TpdfAddRenderingIntentW;
   pdfAddRenderingIntentEx: TpdfAddRenderingIntentEx;
   pdfAddValToChoiceFieldA: TpdfAddValToChoiceFieldA;
   pdfAddValToChoiceFieldW: TpdfAddValToChoiceFieldW;
   pdfAppend: TpdfAppend;
   pdfApplyPattern: TpdfApplyPattern;
   pdfApplyShading: TpdfApplyShading;
   pdfAttachFileA: TpdfAttachFileA;
   pdfAttachFileW: TpdfAttachFileW;
   pdfAttachFileExA: TpdfAttachFileExA;
   pdfAttachFileExW: TpdfAttachFileExW;
   pdfAutoTemplate: TpdfAutoTemplate;
   pdfBeginClipPath: TpdfBeginClipPath;
   pdfBeginContinueText: TpdfBeginContinueText;
   pdfBeginLayer: TpdfBeginLayer;
   pdfBeginPattern: TpdfBeginPattern;
   pdfBeginTemplate: TpdfBeginTemplate;
   pdfBeginTemplateEx: TpdfBeginTemplateEx;
   pdfBezier_1_2_3: TpdfBezier_1_2_3;
   pdfBezier_1_3: TpdfBezier_1_3;
   pdfBezier_2_3: TpdfBezier_2_3;
   pdfCalcWidthHeight: TpdfCalcWidthHeight;
   pdfChangeAnnotNameA: TpdfChangeAnnotNameA;
   pdfChangeAnnotNameW: TpdfChangeAnnotNameW;
   pdfChangeAnnotPos: TpdfChangeAnnotPos;
   pdfChangeBookmarkA: TpdfChangeBookmarkA;
   pdfChangeBookmarkW: TpdfChangeBookmarkW;
   pdfChangeFont: TpdfChangeFont;
   pdfChangeFontSize: TpdfChangeFontSize;
   pdfChangeFontStyle: TpdfChangeFontStyle;
   pdfChangeFontStyleEx: TpdfChangeFontStyleEx;
   pdfChangeJavaScriptA: TpdfChangeJavaScriptA;
   pdfChangeJavaScriptW: TpdfChangeJavaScriptW;
   pdfChangeJavaScriptActionA: TpdfChangeJavaScriptActionA;
   pdfChangeJavaScriptActionW: TpdfChangeJavaScriptActionW;
   pdfChangeJavaScriptNameA: TpdfChangeJavaScriptNameA;
   pdfChangeJavaScriptNameW: TpdfChangeJavaScriptNameW;
   pdfChangeLinkAnnot: TpdfChangeLinkAnnot;
   pdfCheckCollection: TpdfCheckCollection;
   pdfCheckConformance: TpdfCheckConformance;
   pdfCheckFieldNames: TpdfCheckFieldNames;
   pdfClearAutoTemplates: TpdfClearAutoTemplates;
   pdfClearHostFonts: TpdfClearHostFonts;
   pdfClipPath: TpdfClipPath;
   pdfCloseAndSignFile: TpdfCloseAndSignFile;
   pdfCloseAndSignFileEx: TpdfCloseAndSignFileEx;
   pdfCloseAndSignFileExt: TpdfCloseAndSignFileExt;
   pdfCloseFile: TpdfCloseFile;
   pdfCloseFileEx: TpdfCloseFileEx;
   pdfCloseImage: TpdfCloseImage;
   pdfCloseImportFile: TpdfCloseImportFile;
   pdfClosePath: TpdfClosePath;
   pdfCloseTag: TpdfCloseTag;
   pdfComputeBBox: TpdfComputeBBox;
   pdfConvColor: TpdfConvColor;
   pdfConvertColors: TpdfConvertColors;
   pdfConvToUnicode: TpdfConvToUnicode;
   pdfCreate3DAnnot: TpdfCreate3DAnnot;
   pdfCreate3DBackground: TpdfCreate3DBackground;
   pdfCreate3DGotoViewAction: TpdfCreate3DGotoViewAction;
   pdfCreate3DProjection: TpdfCreate3DProjection;
   pdfCreate3DView: TpdfCreate3DView;
   pdfCreateAnnotAP: TpdfCreateAnnotAP;
   pdfCreateArticleThreadA: TpdfCreateArticleThreadA;
   pdfCreateArticleThreadW: TpdfCreateArticleThreadW;
   pdfCreateAxialShading: TpdfCreateAxialShading;
   pdfCreateButtonA: TpdfCreateButtonA;
   pdfCreateButtonW: TpdfCreateButtonW;
   pdfCreateCheckBox: TpdfCreateCheckBox;
   pdfCreateCIEColorSpace: TpdfCreateCIEColorSpace;
   pdfCreateCollection: TpdfCreateCollection;
   pdfCreateCollectionFieldA: TpdfCreateCollectionFieldA;
   pdfCreateCollectionFieldW: TpdfCreateCollectionFieldW;
   pdfCreateColItemDate: TpdfCreateColItemDate;
   pdfCreateColItemNumber: TpdfCreateColItemNumber;
   pdfCreateColItemStringA: TpdfCreateColItemStringA;
   pdfCreateColItemStringW: TpdfCreateColItemStringW;
   pdfCreateComboBox: TpdfCreateComboBox;
   pdfCreateDeviceNColorSpace: TpdfCreateDeviceNColorSpace;
   pdfCreateExtGState: TpdfCreateExtGState;
   pdfCreateGoToAction: TpdfCreateGoToAction;
   pdfCreateGoToActionEx: TpdfCreateGoToActionEx;
   pdfCreateGoToRAction: TpdfCreateGoToRAction;
   pdfCreateGoToRActionExA: TpdfCreateGoToRActionExA;
   pdfCreateGoToRActionExW: TpdfCreateGoToRActionExW;
   pdfCreateGroupField: TpdfCreateGroupField;
   pdfCreateHideAction: TpdfCreateHideAction;
   pdfCreateICCBasedColorSpaceA: TpdfCreateICCBasedColorSpaceA;
   pdfCreateICCBasedColorSpaceW: TpdfCreateICCBasedColorSpaceW;
   pdfCreateImageA: TpdfCreateImageA;
   pdfCreateImageW: TpdfCreateImageW;
   pdfCreateImpDataActionA: TpdfCreateImpDataActionA;
   pdfCreateImpDataActionW: TpdfCreateImpDataActionW;
   pdfCreateIndexedColorSpace: TpdfCreateIndexedColorSpace;
   pdfCreateJSActionA: TpdfCreateJSActionA;
   pdfCreateJSActionW: TpdfCreateJSActionW;
   pdfCreateLaunchAction: TpdfCreateLaunchAction;
   pdfCreateListBox: TpdfCreateListBox;
   pdfCreateNamedAction: TpdfCreateNamedAction;
   pdfCreateNamedDestA: TpdfCreateNamedDestA;
   pdfCreateNamedDestW: TpdfCreateNamedDestW;
   pdfCreateNewPDFA: TpdfCreateNewPDFA;
   pdfCreateNewPDFW: TpdfCreateNewPDFW;
   pdfCreateOCGA: TpdfCreateOCGA;
   pdfCreateOCGW: TpdfCreateOCGW;
   pdfCreateOCMD: TpdfCreateOCMD;
   pdfCreateRadialShading: TpdfCreateRadialShading;
   pdfCreateRadioButton: TpdfCreateRadioButton;
   pdfCreateResetAction: TpdfCreateResetAction;
   pdfCreateSeparationCS: TpdfCreateSeparationCS;
   pdfCreateSetOCGStateAction: TpdfCreateSetOCGStateAction;
   pdfCreateSigField: TpdfCreateSigField;
   pdfCreateSigFieldAP: TpdfCreateSigFieldAP;
   pdfCreateStdPattern: TpdfCreateStdPattern;
   pdfCreateStructureTree: TpdfCreateStructureTree;
   pdfCreateSubmitAction: TpdfCreateSubmitAction;
   pdfCreateTextField: TpdfCreateTextField;
   pdfCreateURIAction: TpdfCreateURIAction;
   pdfDecryptPDFA: TpdfDecryptPDFA;
   pdfDecryptPDFW: TpdfDecryptPDFW;
   pdfDeleteAcroForm: TpdfDeleteAcroForm;
   pdfDeleteActionFromObj: TpdfDeleteActionFromObj;
   pdfDeleteActionFromObjEx: TpdfDeleteActionFromObjEx;
   pdfDeleteAnnotation: TpdfDeleteAnnotation;
   pdfDeleteBookmark: TpdfDeleteBookmark;
   pdfDeleteEmbeddedFile: TpdfDeleteEmbeddedFile;
   pdfDeleteField: TpdfDeleteField;
   pdfDeleteFieldEx: TpdfDeleteFieldEx;
   pdfDeleteJavaScripts: TpdfDeleteJavaScripts;
   pdfDeletePage: TpdfDeletePage;
   pdfDeletePageLabels: TpdfDeletePageLabels;
   pdfDeleteSeparationInfo: TpdfDeleteSeparationInfo;
   pdfDeleteTemplate: TpdfDeleteTemplate;
   pdfDeleteTemplateEx: TpdfDeleteTemplateEx;
   pdfDeleteXFAForm: TpdfDeleteXFAForm;
   pdfDrawArc: TpdfDrawArc;
   pdfDrawArcEx: TpdfDrawArcEx;
   pdfDrawChord: TpdfDrawChord;
   pdfDrawCircle: TpdfDrawCircle;
   pdfDrawPie: TpdfDrawPie;
   pdfEditPage: TpdfEditPage;
   pdfEditTemplate: TpdfEditTemplate;
   pdfEditTemplate2: TpdfEditTemplate2;
   pdfEllipse: TpdfEllipse;
   pdfEncryptPDFA: TpdfEncryptPDFA;
   pdfEncryptPDFW: TpdfEncryptPDFW;
   pdfEndContinueText: TpdfEndContinueText;
   pdfEndLayer: TpdfEndLayer;
   pdfEndPage: TpdfEndPage;
   pdfEndPattern: TpdfEndPattern;
   pdfEndTemplate: TpdfEndTemplate;
   pdfEnumDocFonts: TpdfEnumDocFonts;
   pdfEnumHostFonts: TpdfEnumHostFonts;
   pdfEnumHostFontsEx: TpdfEnumHostFontsEx;
   pdfExchangeBookmarks: TpdfExchangeBookmarks;
   pdfExchangePages: TpdfExchangePages;
   pdfFileAttachAnnotA: TpdfFileAttachAnnotA;
   pdfFileAttachAnnotW: TpdfFileAttachAnnotW;
   pdfFileLinkA: TpdfFileLinkA;
   pdfFileLinkW: TpdfFileLinkW;
   pdfFindBookmarkA: TpdfFindBookmarkA;
   pdfFindBookmarkW: TpdfFindBookmarkW;
   pdfFindFieldA: TpdfFindFieldA;
   pdfFindFieldW: TpdfFindFieldW;
   pdfFindLinkAnnot: TpdfFindLinkAnnot;
   pdfFindNextBookmark: TpdfFindNextBookmark;  
   pdfFinishSignature: TpdfFinishSignature;
   pdfFlattenAnnots: TpdfFlattenAnnots;
   pdfFlattenForm: TpdfFlattenForm;
   pdfFlushPageContent: TpdfFlushPageContent;
   pdfFreeImageBuffer: TpdfFreeImageBuffer;
   pdfFreePDF: TpdfFreePDF;
   pdfFreeTextAnnotA: TpdfFreeTextAnnotA;
   pdfFreeTextAnnotW: TpdfFreeTextAnnotW;
   pdfFreeUniBuf: TpdfFreeUniBuf;
   pdfGetActionCount: TpdfGetActionCount;
   pdfGetActionType: TpdfGetActionType;
   pdfGetActionTypeEx: TpdfGetActionTypeEx;
   pdfGetActiveFont: TpdfGetActiveFont;
   pdfGetAllocBy: TpdfGetAllocBy;
   pdfGetAnnot: TpdfGetAnnot;
   pdfGetAnnotBBox: TpdfGetAnnotBBox;
   pdfGetAnnotCount: TpdfGetAnnotCount;
   pdfGetAnnotEx: TpdfGetAnnotEx;
   pdfGetAnnotFlags: TpdfGetAnnotFlags;
   pdfGetAnnotLink: TpdfGetAnnotLink;
   pdfGetAnnotType: TpdfGetAnnotType;
   pdfGetAscent: TpdfGetAscent;
   pdfGetBBox: TpdfGetBBox;
   pdfGetBidiMode: TpdfGetBidiMode;
   pdfGetBookmark: TpdfGetBookmark;
   pdfGetBookmarkCount: TpdfGetBookmarkCount;
   pdfGetBorderStyle: TpdfGetBorderStyle;
   pdfGetBuffer: TpdfGetBuffer;
   pdfGetCapHeight: TpdfGetCapHeight;
   pdfGetCharacterSpacing: TpdfGetCharacterSpacing;
   pdfGetCheckBoxChar: TpdfGetCheckBoxChar;
   pdfGetCheckBoxCharEx: TpdfGetCheckBoxCharEx;
   pdfGetCheckBoxDefState: TpdfGetCheckBoxDefState;
   pdfGetCMap: TpdfGetCMap;
   pdfGetCMapCount: TpdfGetCMapCount;
   pdfGetColorSpace: TpdfGetColorSpace;
   pdfGetColorSpaceCount: TpdfGetColorSpaceCount;
   pdfGetColorSpaceObj: TpdfGetColorSpaceObj;
   pdfGetColorSpaceObjEx: TpdfGetColorSpaceObjEx;
   pdfGetCompressionFilter: TpdfGetCompressionFilter;
   pdfGetCompressionLevel: TpdfGetCompressionLevel;
   pdfGetContent: TpdfGetContent;
   pdfGetDefBitsPerPixel: TpdfGetDefBitsPerPixel;
   pdfGetDescent: TpdfGetDescent;
   pdfGetDeviceNAttributes: TpdfGetDeviceNAttributes;
   pdfGetDocInfo: TpdfGetDocInfo;
   pdfGetDocInfoCount: TpdfGetDocInfoCount;
   pdfGetDocInfoEx: TpdfGetDocInfoEx;
   pdfGetDrawDirection: TpdfGetDrawDirection;
   pdfGetDynaPDFVersion: TpdfGetDynaPDFVersion;
   pdfGetEmbeddedFile: TpdfGetEmbeddedFile;
   pdfGetEmbeddedFileCount: TpdfGetEmbeddedFileCount;
   pdfGetEMFPatternDistance: TpdfGetEMFPatternDistance;
   pdfGetErrorMessage: TpdfGetErrorMessage;
   pdfGetErrorMode: TpdfGetErrorMode;
   pdfGetField: TpdfGetField;
   pdfGetFieldBackColor: TpdfGetFieldBackColor;
   pdfGetFieldBorderColor: TpdfGetFieldBorderColor;
   pdfGetFieldBorderStyle: TpdfGetFieldBorderStyle;
   pdfGetFieldBorderWidth: TpdfGetFieldBorderWidth;
   pdfGetFieldChoiceValue: TpdfGetFieldChoiceValue;
   pdfGetFieldColor: TpdfGetFieldColor;
   pdfGetFieldCount: TpdfGetFieldCount;
   pdfGetFieldEx: TpdfGetFieldEx;
   pdfGetFieldEx2: TpdfGetFieldEx2;
   pdfGetFieldExpValCount: TpdfGetFieldExpValCount;
   pdfGetFieldExpValue: TpdfGetFieldExpValue;
   pdfGetFieldExpValueEx: TpdfGetFieldExpValueEx;
   pdfGetFieldFlags: TpdfGetFieldFlags;
   pdfGetFieldGroupType: TpdfGetFieldGroupType;
   pdfGetFieldHighlightMode: TpdfGetFieldHighlightMode;
   pdfGetFieldIndex: TpdfGetFieldIndex;
   pdfGetFieldMapName: TpdfGetFieldMapName;
   pdfGetFieldName: TpdfGetFieldName;
   pdfGetFieldOrientation: TpdfGetFieldOrientation;
   pdfGetFieldTextAlign: TpdfGetFieldTextAlign;
   pdfGetFieldTextColor: TpdfGetFieldTextColor;
   pdfGetFieldToolTip: TpdfGetFieldToolTip;
   pdfGetFieldType: TpdfGetFieldType;
   pdfGetFillColor: TpdfGetFillColor;
   pdfGetFontCount: TpdfGetFontCount;
   pdfGetFontEx: TpdfGetFontEx;
   pdfGetFontOrigin: TpdfGetFontOrigin;
   pdfGetFontSearchOrder: TpdfGetFontSearchOrder;
   pdfGetFontSelMode: TpdfGetFontSelMode;
   pdfGetFontWeight: TpdfGetFontWeight;
   pdfGetFTextHeightA: TpdfGetFTextHeightA;
   pdfGetFTextHeightExA: TpdfGetFTextHeightExA;
   pdfGetFTextHeightExW: TpdfGetFTextHeightExW;
   pdfGetFTextHeightW: TpdfGetFTextHeightW;
   pdfGetGStateFlags: TpdfGetGStateFlags;
   pdfGetIconColor: TpdfGetIconColor;
   pdfGetImageBuffer: TpdfGetImageBuffer;
   pdfGetImageCount: TpdfGetImageCount;
   pdfGetImageCountEx: TpdfGetImageCountEx;
   pdfGetImageHeight: TpdfGetImageHeight;
   pdfGetImageWidth: TpdfGetImageWidth;
   pdfGetImportFlags: TpdfGetImportFlags;
   pdfGetInBBox: TpdfGetInBBox;
   pdfGetInDocInfo: TpdfGetInDocInfo;
   pdfGetInDocInfoCount: TpdfGetInDocInfoCount;
   pdfGetInDocInfoEx: TpdfGetInDocInfoEx;
   pdfGetInEncryptionFlags: TpdfGetInEncryptionFlags;
   pdfGetInIsCollection: TpdfGetInIsCollection;
   pdfGetInIsEncrypted: TpdfGetInIsEncrypted;
   pdfGetInIsSigned: TpdfGetInIsSigned;
   pdfGetInIsTrapped: TpdfGetInIsTrapped;
   pdfGetInIsXFAForm: TpdfGetInIsXFAForm;
   pdfGetInNamedDest: TpdfGetInNamedDest;
   pdfGetInNamedDestCount: TpdfGetInNamedDestCount;
   pdfGetInOrientation: TpdfGetInOrientation;
   pdfGetInPageCount: TpdfGetInPageCount;
   pdfGetInPDFVersion: TpdfGetInPDFVersion;
   pdfGetInPrintSettings: TpdfGetInPrintSettings;
   pdfGetIsFixedPitch: TpdfGetIsFixedPitch;
   pdfGetItalicAngle: TpdfGetItalicAngle;
   pdfGetJavaScript: TpdfGetJavaScript;
   pdfGetJavaScriptAction: TpdfGetJavaScriptAction;
   pdfGetJavaScriptAction2: TpdfGetJavaScriptAction2;
   pdfGetJavaScriptCount: TpdfGetJavaScriptCount;
   pdfGetJavaScriptEx: TpdfGetJavaScriptEx;
   pdfGetJavaScriptName: TpdfGetJavaScriptName;
   pdfGetJPEGQuality: TpdfGetJPEGQuality;
   pdfGetLanguage: TpdfGetLanguage;
   pdfGetLastTextPosX: TpdfGetLastTextPosX;
   pdfGetLastTextPosY: TpdfGetLastTextPosY;
   pdfGetLeading: TpdfGetLeading;
   pdfGetLineCapStyle: TpdfGetLineCapStyle;
   pdfGetLineJoinStyle: TpdfGetLineJoinStyle;
   pdfGetLineWidth: TpdfGetLineWidth;
   pdfGetLinkHighlightMode: TpdfGetLinkHighlightMode;
   pdfGetLogMetafileSize: TpdfGetLogMetafileSize;
   pdfGetLogMetafileSizeEx: TpdfGetLogMetafileSizeEx;
   pdfGetMatrix: TpdfGetMatrix;
   pdfGetMaxFieldLen: TpdfGetMaxFieldLen;
   pdfGetMetaConvFlags: TpdfGetMetaConvFlags;
   pdfGetMissingGlyphs: TpdfGetMissingGlyphs;
   pdfGetMiterLimit: TpdfGetMiterLimit;
   pdfGetNeedAppearance: TpdfGetNeedAppearance;
   pdfGetObjActionCount: TpdfGetObjActionCount;
   pdfGetOpacity: TpdfGetOpacity;
   pdfGetOrientation: TpdfGetOrientation;
   pdfGetPageAnnot: TpdfGetPageAnnot;
   pdfGetPageAnnotCount: TpdfGetPageAnnotCount;
   pdfGetPageAnnotEx: TpdfGetPageAnnotEx;
   pdfGetPageCoords: TpdfGetPageCoords;
   pdfGetPageCount: TpdfGetPageCount;
   pdfGetPageField: TpdfGetPageField;
   pdfGetPageFieldCount: TpdfGetPageFieldCount;
   pdfGetPageFieldEx: TpdfGetPageFieldEx;
   pdfGetPageHeight: TpdfGetPageHeight;
   pdfGetPageLabel: TpdfGetPageLabel;
   pdfGetPageLabelCount: TpdfGetPageLabelCount;
   pdfGetPageLayout: TpdfGetPageLayout;
   pdfGetPageMode: TpdfGetPageMode;
   pdfGetPageNum: TpdfGetPageNum;
   pdfGetPageText: TpdfGetPageText;
   pdfGetPageWidth: TpdfGetPageWidth;
   pdfGetPDFVersion: TpdfGetPDFVersion;
   pdfGetPrintSettings: TpdfGetPrintSettings;
   pdfGetResolution: TpdfGetResolution;
   pdfGetSaveNewImageFormat: TpdfGetSaveNewImageFormat;
   pdfGetSeparationInfo: TpdfGetSeparationInfo;
   pdfGetStrokeColor: TpdfGetStrokeColor;
   pdfGetTabLen: TpdfGetTabLen;
   pdfGetTemplCount: TpdfGetTemplCount;
   pdfGetTemplHandle: TpdfGetTemplHandle;
   pdfGetTemplHeight: TpdfGetTemplHeight;
   pdfGetTemplWidth: TpdfGetTemplWidth;
   pdfGetTextDrawMode: TpdfGetTextDrawMode;
   pdfGetTextFieldValue: TpdfGetTextFieldValue;
   pdfGetTextRect: TpdfGetTextRect;
   pdfGetTextRise: TpdfGetTextRise;
   pdfGetTextScaling: TpdfGetTextScaling;
   pdfGetTextWidthA: TpdfGetTextWidthA;
   pdfGetTextWidthW: TpdfGetTextWidthW;
   pdfGetTextWidthExA: TpdfGetTextWidthExA;
   pdfGetTextWidthExW: TpdfGetTextWidthExW;
   pdfGetTrapped: TpdfGetTrapped;
   pdfGetTransparentColor: TpdfGetTransparentColor;
   pdfGetUseExactPwd: TpdfGetUseExactPwd;
   pdfGetUseGlobalImpFiles: TpdfGetUseGlobalImpFiles;
   pdfGetUserRights: TpdfGetUserRights;
   pdfGetUseStdFonts: TpdfGetUseStdFonts;
   pdfGetUseSystemFonts: TpdfGetUseSystemFonts;
   pdfGetUseTransparency: TpdfGetUseTransparency;
   pdfGetUseVisibleCoords: TpdfGetUseVisibleCoords;
   pdfGetViewerPrefrences: TpdfGetViewerPrefrences;
   pdfGetWMFDefExtent: TpdfGetWMFDefExtent;
   pdfGetWMFPixelPerInch: TpdfGetWMFPixelPerInch;
   pdfGetWordSpacing: TpdfGetWordSpacing;
   pdfHaveOpenDoc: TpdfHaveOpenDoc;
   pdfHighlightAnnotA: TpdfHighlightAnnotA;
   pdfHighlightAnnotW: TpdfHighlightAnnotW;
   pdfImportBookmarks: TpdfImportBookmarks;
   pdfImportCatalogObjects: TpdfImportCatalogObjects;
   pdfImportDocInfo: TpdfImportDocInfo;
   pdfImportPage: TpdfImportPage;
   pdfImportPageEx: TpdfImportPageEx;
   pdfImportPDFFile: TpdfImportPDFFile;
   pdfInitExtGState: TpdfInitExtGState;
   pdfInitStack: TpdfInitStack;
   pdfInsertBMPFromBuffer: TpdfInsertBMPFromBuffer;
   pdfInsertBMPFromHandle: TpdfInsertBMPFromHandle;
   pdfInsertImage: TpdfInsertImage;
   pdfInsertImageEx: TpdfInsertImageEx;
   pdfInsertImageFromBuffer: TpdfInsertImageFromBuffer;
   pdfInsertMetafile: TpdfInsertMetafile;
   pdfInsertMetafileEx: TpdfInsertMetafileEx;
   pdfInsertMetafileExt: TpdfInsertMetafileExt;
   pdfInsertMetafileExtEx: TpdfInsertMetafileExtEx;
   pdfInsertMetafileFromHandle: TpdfInsertMetafileFromHandle;
   pdfInsertMetafileFromHandleEx: TpdfInsertMetafileFromHandleEx;
   pdfInsertRawImage: TpdfInsertRawImage;
   pdfIsBidiText: TpdfIsBidiText;
   pdfIsColorPage: TpdfIsColorPage;
   pdfIsEmptyPage: TpdfIsEmptyPage;
   pdfLineTo: TpdfLineTo;
   pdfLoadCMap: TpdfLoadCMap;
   pdfLoadFDFDataA: TpdfLoadFDFDataA;
   pdfLoadFDFDataW: TpdfLoadFDFDataW;
   pdfLoadFDFDataEx: TpdfLoadFDFDataEx;
   pdfLoadFont: TpdfLoadFont;
   pdfMovePage: TpdfMovePage;
   pdfMoveTo: TpdfMoveTo;
   pdfOpenImportBuffer: TpdfOpenImportBuffer;
   pdfOpenImportFileA: TpdfOpenImportFileA;
   pdfOpenImportFileW: TpdfOpenImportFileW;
   pdfOpenOutputFileA: TpdfOpenOutputFileA;
   pdfOpenOutputFileW: TpdfOpenOutputFileW;
   pdfOpenTagA: TpdfOpenTagA;
   pdfOpenTagW: TpdfOpenTagW;
   pdfPageLink: TpdfPageLink;
   pdfPageLink2: TpdfPageLink2;
   pdfPageLinkEx: TpdfPageLinkEx;
   pdfParseContent: TpdfParseContent;
   pdfPlaceImage: TpdfPlaceImage;
   pdfPlaceSigFieldValidateIcon: TpdfPlaceSigFieldValidateIcon;
   pdfPlaceTemplate: TpdfPlaceTemplate;
   pdfPlaceTemplateEx: TpdfPlaceTemplateEx;
   pdfPlaceTemplByMatrix: TpdfPlaceTemplByMatrix;
   pdfReadImageFormat: TpdfReadImageFormat;
   pdfReadImageFormat2: TpdfReadImageFormat2;
   pdfReadImageFormatEx: TpdfReadImageFormatEx;
   pdfReadImageFormatFromBuffer: TpdfReadImageFormatFromBuffer;
   pdfReadImageResolution: TpdfReadImageResolution;
   pdfReadImageResolutionEx: TpdfReadImageResolutionEx;
   pdfRectangle: TpdfRectangle;
   pdfReEncryptPDFA: TpdfReEncryptPDFA;
   pdfReEncryptPDFW: TpdfReEncryptPDFW;
   pdfRenameSpotColor: TpdfRenameSpotColor;
   pdfReplaceFontA: TpdfReplaceFontA;
   pdfReplaceFontW: TpdfReplaceFontW;
   pdfReplaceFontExA: TpdfReplaceFontExA;
   pdfReplaceFontExW: TpdfReplaceFontExW;
   pdfReplaceICCProfileA: TpdfReplaceICCProfileA;
   pdfReplaceICCProfileW: TpdfReplaceICCProfileW;
   pdfReplacePageTextA: TpdfReplacePageTextA;
   pdfReplacePageTextExA: TpdfReplacePageTextExA;
   pdfReplacePageTextExW: TpdfReplacePageTextExW;
   pdfResetLineDashPattern: TpdfResetLineDashPattern;
   pdfRestoreGraphicState: TpdfRestoreGraphicState;
   pdfRotateCoords: TpdfRotateCoords;
   pdfRoundRect: TpdfRoundRect;
   pdfRoundRectEx: TpdfRoundRectEx;
   pdfSaveGraphicState: TpdfSaveGraphicState;
   pdfScaleCoords: TpdfScaleCoords;
   pdfSelfTest: TpdfSelfTest;
   pdfSet3DAnnotProps: TpdfSet3DAnnotProps;
   pdfSet3DAnnotScriptA: TpdfSet3DAnnotScriptA;
   pdfSetAllocBy: TpdfSetAllocBy;
   pdfSetAnnotBorderStyle: TpdfSetAnnotBorderStyle;
   pdfSetAnnotBorderWidth: TpdfSetAnnotBorderWidth;
   pdfSetAnnotColor: TpdfSetAnnotColor;
   pdfSetAnnotFlags: TpdfSetAnnotFlags;
   pdfSetAnnotFlagsEx: TpdfSetAnnotFlagsEx;
   pdfSetAnnotHighlightMode: TpdfSetAnnotHighlightMode;
   pdfSetAnnotIcon: TpdfSetAnnotIcon;
   pdfSetAnnotOpenState: TpdfSetAnnotOpenState;
   pdfSetAnnotStringA: TpdfSetAnnotStringA;
   pdfSetAnnotStringW: TpdfSetAnnotStringW;
   pdfSetAnnotSubjectA: TpdfSetAnnotSubjectA;
   pdfSetAnnotSubjectW: TpdfSetAnnotSubjectW;
   pdfSetBBox: TpdfSetBBox;
   pdfSetBidiMode: TpdfSetBidiMode;
   pdfSetBookmarkDest: TpdfSetBookmarkDest;
   pdfSetBookmarkStyle: TpdfSetBookmarkStyle;
   pdfSetBorderStyle: TpdfSetBorderStyle;
   pdfSetCharacterSpacing: TpdfSetCharacterSpacing;
   pdfSetCheckBoxChar: TpdfSetCheckBoxChar;
   pdfSetCheckBoxDefState: TpdfSetCheckBoxDefState;
   pdfSetCheckBoxState: TpdfSetCheckBoxState;
   pdfSetCIDFontA: TpdfSetCIDFontA;
   pdfSetCIDFontW: TpdfSetCIDFontW;
   pdfSetCMapDirA: TpdfSetCMapDirA;
   pdfSetCMapDirW: TpdfSetCMapDirW;
   pdfSetColDefFile: TpdfSetColDefFile;
   pdfSetColors: TpdfSetColors;
   pdfSetColorSpace: TpdfSetColorSpace;
   pdfSetColSortField: TpdfSetColSortField;
   pdfSetCompressionFilter: TpdfSetCompressionFilter;
   pdfSetCompressionLevel: TpdfSetCompressionLevel;
   pdfSetContent: TpdfSetContent;
   pdfSetDateTimeFormat: TpdfSetDateTimeFormat;
   pdfSetDefBitsPerPixel: TpdfSetDefBitsPerPixel;
   pdfSetDocInfoA: TpdfSetDocInfoA;
   pdfSetDocInfoW: TpdfSetDocInfoW;
   pdfSetDocInfoExA: TpdfSetDocInfoExA;
   pdfSetDocInfoExW: TpdfSetDocInfoExW;
   pdfSetDrawDirection: TpdfSetDrawDirection;
   pdfSetEMFFrameDPI: TpdfSetEMFFrameDPI;
   pdfSetEMFPatternDistance: TpdfSetEMFPatternDistance;
   pdfSetErrorMode: TpdfSetErrorMode;
   pdfSetExtColorSpace: TpdfSetExtColorSpace;
   pdfSetExtGState: TpdfSetExtGState;
   pdfSetFieldBackColor: TpdfSetFieldBackColor;
   pdfSetFieldBBox: TpdfSetFieldBBox;
   pdfSetFieldBorderColor: TpdfSetFieldBorderColor;
   pdfSetFieldBorderStyle: TpdfSetFieldBorderStyle;
   pdfSetFieldBorderWidth: TpdfSetFieldBorderWidth;
   pdfSetFieldColor: TpdfSetFieldColor;
   pdfSetFieldExpValue: TpdfSetFieldExpValue;
   pdfSetFieldFlags: TpdfSetFieldFlags;
   pdfSetFieldFontSize: TpdfSetFieldFontSize;
   pdfSetFieldHighlightMode: TpdfSetFieldHighlightMode;
   pdfSetFieldIndex: TpdfSetFieldIndex;
   pdfSetFieldMapNameA: TpdfSetFieldMapNameA;
   pdfSetFieldMapNameW: TpdfSetFieldMapNameW;
   pdfSetFieldName: TpdfSetFieldName;
   pdfSetFieldOrientation: TpdfSetFieldOrientation;
   pdfSetFieldTextAlign: TpdfSetFieldTextAlign;
   pdfSetFieldTextColor: TpdfSetFieldTextColor;
   pdfSetFieldToolTipA: TpdfSetFieldToolTipA;
   pdfSetFieldToolTipW: TpdfSetFieldToolTipW;
   pdfSetFillColor: TpdfSetFillColor;
   pdfSetFillColorEx: TpdfSetFillColorEx;
   pdfSetFillColorSpace: TpdfSetFillColorSpace;
   pdfSetFontA: TpdfSetFontA;
   pdfSetFontW: TpdfSetFontW;
   pdfSetFontExA: TpdfSetFontExA;
   pdfSetFontExW: TpdfSetFontExW;
   pdfSetFontOrigin: TpdfSetFontOrigin;
   pdfSetFontSearchOrder: TpdfSetFontSearchOrder;
   pdfSetFontSelMode: TpdfSetFontSelMode;
   pdfSetFontWeight: TpdfSetFontWeight;
   pdfSetGStateFlags: TpdfSetGStateFlags;
   pdfSetIconColor: TpdfSetIconColor;
   pdfSetImportFlags: TpdfSetImportFlags;
   pdfSetImportFlags2: TpdfSetImportFlags2;
   pdfSetItalicAngle: TpdfSetItalicAngle;
   pdfSetJPEGQuality: TpdfSetJPEGQuality;
   pdfSetLanguage: TpdfSetLanguage;
   pdfSetLeading: TpdfSetLeading;
   pdfSetLicenseKey: TpdfSetLicenseKey;
   pdfSetLineCapStyle: TpdfSetLineCapStyle;
   pdfSetLineDashPattern: TpdfSetLineDashPattern;
   pdfSetLineDashPatternEx: TpdfSetLineDashPatternEx;
   pdfSetLineJoinStyle: TpdfSetLineJoinStyle;
   pdfSetLineWidth: TpdfSetLineWidth;
   pdfSetLinkHighlightMode: TpdfSetLinkHighlightMode;
   pdfSetListFont: TpdfSetListFont;
   pdfSetMatrix: TpdfSetMatrix;
   pdfSetMaxFieldLen: TpdfSetMaxFieldLen;
   pdfSetMetaConvFlags: TpdfSetMetaConvFlags;
   pdfSetMiterLimit: TpdfSetMiterLimit;
   pdfSetNeedAppearance: TpdfSetNeedAppearance;
   pdfSetNumberFormat: TpdfSetNumberFormat;
   pdfSetOnErrorProc: TpdfSetOnErrorProc;
   pdfSetOnPageBreakProc: TpdfSetOnPageBreakProc;
   pdfSetOpacity: TpdfSetOpacity;
   pdfSetOrientation: TpdfSetOrientation;
   pdfSetOrientationEx: TpdfSetOrientationEx;
   pdfSetPageCoords: TpdfSetPageCoords;
   pdfSetPageFormat: TpdfSetPageFormat;
   pdfSetPageHeight: TpdfSetPageHeight;
   pdfSetPageLayout: TpdfSetPageLayout;
   pdfSetPageMode: TpdfSetPageMode;
   pdfSetPageWidth: TpdfSetPageWidth;
   pdfSetPDFVersion: TpdfSetPDFVersion;
   pdfSetPrintSettings: TpdfSetPrintSettings;
   pdfSetProgressProc: TpdfSetProgressProc;
   pdfSetResolution: TpdfSetResolution;
   pdfSetSaveNewImageFormat: TpdfSetSaveNewImageFormat;
   pdfSetSeparationInfo: TpdfSetSeparationInfo;
   pdfSetStrokeColor: TpdfSetStrokeColor;
   pdfSetStrokeColorEx: TpdfSetStrokeColorEx;
   pdfSetStrokeColorSpace: TpdfSetStrokeColorSpace;
   pdfSetTabLen: TpdfSetTabLen;
   pdfSetTextDrawMode: TpdfSetTextDrawMode;
   pdfSetTextFieldValueA: TpdfSetTextFieldValueA;
   pdfSetTextFieldValueW: TpdfSetTextFieldValueW;
   pdfSetTextFieldValueExA: TpdfSetTextFieldValueExA;
   pdfSetTextFieldValueExW: TpdfSetTextFieldValueExW;
   pdfSetTextRect: TpdfSetTextRect;
   pdfSetTextRise: TpdfSetTextRise;
   pdfSetTextScaling: TpdfSetTextScaling;
   pdfSetTrapped: TpdfSetTrapped;
   pdfSetTransparentColor: TpdfSetTransparentColor;
   pdfSetUseExactPwd: TpdfSetUseExactPwd;
   pdfSetUseGlobalImpFiles: TpdfSetUseGlobalImpFiles;
   pdfSetUseStdFonts: TpdfSetUseStdFonts;
   pdfSetUseSwapFile: TpdfSetUseSwapFile;
   pdfSetUseSwapFileEx: TpdfSetUseSwapFileEx;
   pdfSetUseSystemFonts: TpdfSetUseSystemFonts;
   pdfSetUseTransparency: TpdfSetUseTransparency;
   pdfSetUseVisibleCoords: TpdfSetUseVisibleCoords;
   pdfSetViewerPreferences: TpdfSetViewerPreferences;
   pdfSetWMFDefExtent: TpdfSetWMFDefExtent;
   pdfSetWMFPixelPerInch: TpdfSetWMFPixelPerInch;
   pdfSetWordSpacing: TpdfSetWordSpacing;
   pdfSkewCoords: TpdfSkewCoords;
   pdfSortFieldsByIndex: TpdfSortFieldsByIndex;
   pdfSortFieldsByName: TpdfSortFieldsByName;
   pdfStampAnnotA: TpdfStampAnnotA;
   pdfStampAnnotW: TpdfStampAnnotW;
   pdfStrokePath: TpdfStrokePath;
   pdfTextAnnotA: TpdfTextAnnotA;
   pdfTextAnnotW: TpdfTextAnnotW;
   pdfTranslateCoords: TpdfTranslateCoords;
   pdfTriangle: TpdfTriangle;
   pdfUTF16ToUTF32: TpdfUTF16ToUTF32;
   pdfUTF16ToUTF32Ex: TpdfUTF16ToUTF32Ex;
   pdfUTF32ToUTF16: TpdfUTF32ToUTF16;
   pdfUTF32ToUTF16Ex: TpdfUTF32ToUTF16Ex;
   pdfWebLinkA: TpdfWebLinkA;
   pdfWebLinkW: TpdfWebLinkW;
   pdfWriteAngleTextA: TpdfWriteAngleTextA;
   pdfWriteAngleTextW: TpdfWriteAngleTextW;
   pdfWriteFTextA: TpdfWriteFTextA;
   pdfWriteFTextW: TpdfWriteFTextW;
   pdfWriteFTextExA: TpdfWriteFTextExA;
   pdfWriteFTextExW: TpdfWriteFTextExW;
   pdfWriteTextA: TpdfWriteTextA;
   pdfWriteTextW: TpdfWriteTextW;
   pdfWriteTextExA: TpdfWriteTextExA;
   pdfWriteTextExW: TpdfWriteTextExW;
   pdfWriteTextMatrixA: TpdfWriteTextMatrixA;
   pdfWriteTextMatrixW: TpdfWriteTextMatrixW;
   pdfWriteTextMatrixExA: TpdfWriteTextMatrixExA;
   pdfWriteTextMatrixExW: TpdfWriteTextMatrixExW;
   FDLL_Handle: Cardinal;
   FInstance: Pointer;
   procedure NewPDF();
   procedure LoadFunction(var Func: Pointer; const FuncName: AnsiString);
end;

implementation

uses SysUtils;

// The constructor was changed in Version 2.0.49.485 to archive compability with Delphi 2009 or higher.
constructor TPDF.Create(const LibName: WideString = 'dynapdf.dll');
begin
   FDLL_Handle := LoadLibraryW(PWideChar(LibName));
   if FDLL_Handle = 0 then raise Exception.Create('Error: Cannot find dynapdf.dll');
   NewPDF;
end;

destructor TPDF.Destroy;
var pdfDeletePDF: TpdfDeletePDF;
begin
   if FDLL_Handle = 0 then Exit;
   // To avoid memory leaks it is highly recommended to delete the used instance before unloading the library.
   pdfDeletePDF := GetProcAddress(FDLL_Handle, 'pdfDeletePDF');
   if @pdfDeletePDF = nil then begin
      // Maybe a wrong library loaded?
      // If this function cannot be found then most other functions should not be found too.
      // We assume that this error was already handled...
      FreeLibrary(FDLL_Handle);
      Exit;
   end;
   pdfDeletePDF(FInstance);
   FreeLibrary(FDLL_Handle);
   inherited;
end;

procedure TPDF.LoadFunction(var Func: Pointer; const FuncName: AnsiString);
begin
   if Func = nil then begin
      Func := GetProcAddress(FDLL_Handle, PAnsiChar(FuncName));
      if Func = nil then raise Exception.Create(Format('Error loading function: "%s"', [FuncName]));
   end;
end;

function TPDF.AddActionToObj(ObjType: TObjType; Event: TObjEvent; ActHandle, ObjHandle: Cardinal): Integer;
begin
   LoadFunction(@pdfAddActionToObj, 'pdfAddActionToObj');
   Result := pdfAddActionToObj(FInstance, Integer(ObjType), Integer(Event), ActHandle, ObjHandle);
end;

function TPDF.AddArticle(PosX, PosY, Width, Height: Double): Integer;
begin
   LoadFunction(@pdfAddArticle, 'pdfAddArticle');
   Result := pdfAddArticle(FInstance, PosX, PosY, Width, Height);
end;

function TPDF.AddBookmarkA(const Title: AnsiString; Parent: Integer; DestPage: Cardinal; Open: Boolean): Integer;
begin
   LoadFunction(@pdfAddBookmarkA, 'pdfAddBookmarkA');
   Result := pdfAddBookmarkA(FInstance, PAnsiChar(Title), Parent, DestPage, Integer(Open));
end;

function TPDF.AddBookmarkW(const Title: WideString; Parent: Integer; DestPage: Cardinal; Open: Boolean): Integer;
begin
   LoadFunction(@pdfAddBookmarkW, 'pdfAddBookmarkW');
   Result := pdfAddBookmarkW(FInstance, PWideChar(Title), Parent, DestPage, Integer(Open));
end;

function TPDF.AddBookmarkExA(const Title: AnsiString; Parent: Integer; NamedDest: Cardinal; Open: Boolean): Integer;
begin
   LoadFunction(@pdfAddBookmarkExA, 'pdfAddBookmarkExA');
   Result := pdfAddBookmarkExA(FInstance, PAnsiChar(Title), Parent, NamedDest, Open);
end;

function TPDF.AddBookmarkExW(const Title: WideString; Parent: Integer; NamedDest: Cardinal; Open: Boolean): Integer;
begin
   LoadFunction(@pdfAddBookmarkExW, 'pdfAddBookmarkExW');
   Result := pdfAddBookmarkExW(FInstance, PWideChar(Title), Parent, NamedDest, Open);
end;

function TPDF.AddButtonImage(BtnHandle: Cardinal; State: TButtonState; const Caption, ImgFile: AnsiString): Boolean;
begin
   LoadFunction(@pdfAddButtonImage, 'pdfAddButtonImage');
   Result := pdfAddButtonImage(FInstance, BtnHandle, Integer(State), PAnsiChar(Caption), PAnsiChar(ImgFile));
end;

function TPDF.AddButtonImageEx(BtnHandle: Cardinal; State: TButtonState; const Caption: AnsiString; Handle: HBITMAP): Boolean;
begin
   LoadFunction(@pdfAddButtonImageEx, 'pdfAddButtonImageEx');
   Result := pdfAddButtonImageEx(FInstance, BtnHandle, Integer(State), PAnsiChar(Caption), Pointer(Handle));
end;

function TPDF.AddContinueTextA(const AText: AnsiString): Integer;
begin
   LoadFunction(@pdfAddContinueTextA, 'pdfAddContinueTextA');
   Result := pdfAddContinueTextA(FInstance, PAnsiChar(AText));
end;

function TPDF.AddContinueTextW(const AText: WideString): Integer;
begin
   LoadFunction(@pdfAddContinueTextW, 'pdfAddContinueTextW');
   Result := pdfAddContinueTextW(FInstance, PWideChar(AText));
end;

function TPDF.AddDeviceNProcessColorants(DeviceNCS: Cardinal; const Colorants: Array of AnsiString; NumColorants: Cardinal; ProcessCS: TExtColorSpace; Handle: Integer): Boolean;
begin
   LoadFunction(@pdfAddDeviceNProcessColorants, 'pdfAddDeviceNProcessColorants');
   Result := pdfAddDeviceNProcessColorants(FInstance, DeviceNCS, @Colorants, NumColorants, ProcessCS, Handle);
end;

function TPDF.AddDeviceNSeparations(DeviceNCS: Cardinal; const Colorants: Array of AnsiString; SeparationCS: Array of Integer; NumColorants: Cardinal): Boolean;
begin
   LoadFunction(@pdfAddDeviceNSeparations, 'pdfAddDeviceNSeparations');
   Result := pdfAddDeviceNSeparations(FInstance, DeviceNCS, @Colorants, @SeparationCS, NumColorants);
end;

function TPDF.AddFieldToFormAction(Action, AField: Cardinal; Include: Boolean): Integer;
begin
   LoadFunction(@pdfAddFieldToFormAction, 'pdfAddFieldToFormAction');
   Result := pdfAddFieldToFormAction(FInstance, Action, AField, Include);
end;

function TPDF.AddFieldToHideAction(HideAct, AField: Cardinal): Integer;
begin
   LoadFunction(@pdfAddFieldToHideAction, 'pdfAddFieldToHideAction');
   Result := pdfAddFieldToHideAction(FInstance, HideAct, AField);
end;

function TPDF.AddFileCommentA(const AText: AnsiString): Boolean;
begin
   LoadFunction(@pdfAddFileCommentA, 'pdfAddFileCommentA');
   Result := pdfAddFileCommentA(FInstance, PAnsiChar(AText));
end;

function TPDF.AddFileCommentW(const AText: WideString): Boolean;
begin
   LoadFunction(@pdfAddFileCommentW, 'pdfAddFileCommentW');
   Result := pdfAddFileCommentW(FInstance, PWideChar(AText));
end;

function TPDF.AddFontSearchPath(const APath: AnsiString; Recursive: Boolean): Integer;
begin
   LoadFunction(@pdfAddFontSearchPath, 'pdfAddFontSearchPathA');
   Result := pdfAddFontSearchPath(FInstance, PAnsiChar(APath), Integer(Recursive));
end;

function TPDF.AddImage(Filter: TCompressionFilter; Flags: TImageConversionFlags; var Image: TPDFImage): Boolean;
begin
   LoadFunction(@pdfAddImage, 'pdfAddImage');
   Result := pdfAddImage(FInstance, Filter, Flags, Image);
end;

function TPDF.AddJavaScriptA(const Name, Script: AnsiString): Integer;
begin
   LoadFunction(@pdfAddJavaScriptA, 'pdfAddJavaScriptA');
   Result := pdfAddJavaScriptA(FInstance, PAnsiChar(Name), PAnsiChar(Script));
end;

function TPDF.AddJavaScriptW(const Name: AnsiString; const Script: WideString): Integer;
begin
   LoadFunction(@pdfAddJavaScriptW, 'pdfAddJavaScriptW');
   Result := pdfAddJavaScriptW(FInstance, PAnsiChar(Name), PWideChar(Script));
end;

function TPDF.AddObjectToLayer(OCG: Cardinal; ObjType: TOCObject; Handle: Cardinal): Boolean;
begin
   LoadFunction(@pdfAddObjectToLayer, 'pdfAddObjectToLayer');
   Result := pdfAddObjectToLayer(FInstance, OCG, ObjType, Handle);
end;

function TPDF.AddPageLabelA(StartRange: Cardinal; Format: TPageLabelFormat; const Prefix: AnsiString; AddNum: Integer): Integer;
begin
   LoadFunction(@pdfAddPageLabelA, 'pdfAddPageLabelA');
   Result := pdfAddPageLabelA(FInstance, StartRange, Format, PAnsiChar(Prefix), AddNum);
end;

function TPDF.AddPageLabelW(StartRange: Cardinal; Format: TPageLabelFormat; const Prefix: WideString; AddNum: Integer): Integer;
begin
   LoadFunction(@pdfAddPageLabelW, 'pdfAddPageLabelW');
   Result := pdfAddPageLabelW(FInstance, StartRange, Format, PWideChar(Prefix), AddNum);
end;

function TPDF.AddRenderingIntentA(const ICCFile: AnsiString): Integer;
begin
   LoadFunction(@pdfAddRenderingIntentA, 'pdfAddRenderingIntentA');
   Result := pdfAddRenderingIntentA(FInstance, PAnsiChar(ICCFile));
end;

function TPDF.AddRenderingIntentW(const ICCFile: WideString): Integer;
begin
   LoadFunction(@pdfAddRenderingIntentW, 'pdfAddRenderingIntentW');
   Result := pdfAddRenderingIntentW(FInstance, PWideChar(ICCFile));
end;

function TPDF.AddRenderingIntentEx(const Buffer: Pointer; BufSize: Cardinal): Integer;
begin
   LoadFunction(@pdfAddRenderingIntentEx, 'pdfAddRenderingIntentEx');
   Result := pdfAddRenderingIntentEx(FInstance, Buffer, BufSize);
end;

function TPDF.AddValToChoiceFieldA(Field: Cardinal; const ExpValue, Value: AnsiString; Selected: Boolean): Boolean;
begin
   LoadFunction(@pdfAddValToChoiceFieldA, 'pdfAddValToChoiceFieldA');
   Result := pdfAddValToChoiceFieldA(FInstance, Field, PAnsiChar(ExpValue), PAnsiChar(Value), Integer(Selected));
end;

function TPDF.AddValToChoiceFieldW(Field: Cardinal; const ExpValue: AnsiString; const Value: WideString; Selected: Boolean): Boolean;
begin
   LoadFunction(@pdfAddValToChoiceFieldW, 'pdfAddValToChoiceFieldW');
   Result := pdfAddValToChoiceFieldW(FInstance, Field, PAnsiChar(ExpValue), PWideChar(Value), Integer(Selected));
end;

function TPDF.Append(): Boolean;
begin
   LoadFunction(@pdfAppend, 'pdfAppend');
   Result := pdfAppend(FInstance);
end;

function TPDF.ApplyPattern(PattHandle: Integer; ColorMode: TColorMode; Color: Cardinal): Boolean;
begin
   LoadFunction(@pdfApplyPattern, 'pdfApplyPattern');
   Result := pdfApplyPattern(FInstance, PattHandle, ColorMode, Color);
end;

function TPDF.ApplyShading(ShadHandle: Integer): Boolean;
begin
   LoadFunction(@pdfApplyShading, 'pdfApplyShading');
   Result := pdfApplyShading(FInstance, ShadHandle);
end;

function TPDF.AttachFileA(const FilePath, Description: AnsiString; Compress: Boolean): Integer;
begin
   LoadFunction(@pdfAttachFileA, 'pdfAttachFileA');
   Result := pdfAttachFileA(FInstance, PAnsiChar(FilePath), PAnsiChar(Description), Compress);
end;

function TPDF.AttachFileW(const FilePath, Description: WideString; Compress: Boolean): Integer;
begin
   LoadFunction(@pdfAttachFileW, 'pdfAttachFileW');
   Result := pdfAttachFileW(FInstance, PWideChar(FilePath), PWideChar(Description), Compress);
end;

function TPDF.AttachFileExA(const Buffer: Pointer; BufSize: Cardinal; const FileName, Description: AnsiString; Compress: Boolean): Integer;
begin
   LoadFunction(@pdfAttachFileExA, 'pdfAttachFileExA');
   Result := pdfAttachFileExA(FInstance, Pointer(Buffer), BufSize, PAnsiChar(FileName), PAnsiChar(Description), Compress);
end;

function TPDF.AttachFileExW(const Buffer: Pointer; BufSize: Cardinal; const FileName, Description: WideString; Compress: Boolean): Integer;
begin
   LoadFunction(@pdfAttachFileExW, 'pdfAttachFileExW');
   Result := pdfAttachFileExW(FInstance, Pointer(Buffer), BufSize, PWideChar(FileName), PWideChar(Description), Compress);
end;

function TPDF.AutoTemplate(Templ: Cardinal; PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfAutoTemplate, 'pdfAutoTemplate');
   Result := pdfAutoTemplate(FInstance, Templ, PosX, PosY, Width, Height);
end;

function TPDF.BeginClipPath(): Boolean;
begin
   LoadFunction(@pdfBeginClipPath, 'pdfBeginClipPath');
   Result := pdfBeginClipPath(FInstance);
end;

function TPDF.BeginContinueText(PosX, PosY: Double): Integer;
begin
   LoadFunction(@pdfBeginContinueText, 'pdfBeginContinueText');
   Result := pdfBeginContinueText(FInstance, PosX, PosY);
end;

function TPDF.BeginLayer(OCG: Cardinal): Boolean;
begin
   LoadFunction(@pdfBeginLayer, 'pdfBeginLayer');
   Result := pdfBeginLayer(FInstance, OCG);
end;

function TPDF.BeginPattern(PatternType: TPatternType; TilingType: TTilingType; Width, Height: Double): Integer;
begin
   LoadFunction(@pdfBeginPattern, 'pdfBeginPattern');
   Result := pdfBeginPattern(FInstance, PatternType, TilingType, Width, Height);
end;

function TPDF.BeginTemplate(Width, Height: Double): Integer;
begin
   LoadFunction(@pdfBeginTemplate, 'pdfBeginTemplate');
   Result := pdfBeginTemplate(FInstance, Width, Height);
end;

function TPDF.BeginTemplateEx(var BBox: TPDFRect; var Matrix: TCTM): Integer;
begin
   LoadFunction(@pdfBeginTemplateEx, 'pdfBeginTemplateEx');
   Result := pdfBeginTemplateEx(FInstance, BBox, Matrix);
end;

function TPDF.Bezier_1_2_3(x1, y1, x2, y2, x3, y3: Double): Boolean;
begin
   LoadFunction(@pdfBezier_1_2_3, 'pdfBezier_1_2_3');
   Result := pdfBezier_1_2_3(FInstance, x1, y1, x2, y2, x3, y3);
end;

function TPDF.Bezier_1_3(x1, y1, x3, y3: Double): Boolean;
begin
   LoadFunction(@pdfBezier_1_3, 'pdfBezier_1_3');
   Result := pdfBezier_1_3(FInstance, x1, y1, x3, y3);
end;

function TPDF.Bezier_2_3(x2, y2, x3, y3: Double): Boolean;
begin
   LoadFunction(@pdfBezier_2_3, 'pdfBezier_2_3');
   Result := pdfBezier_2_3(FInstance, x2, y2, x3, y3);
end;

function TPDF.BuildFamilyNameAndStyle(const IFont: Pointer; Name: PAnsiChar; var Style: TFStyle): Boolean;
begin
   LoadFunction(@fntBuildFamilyNameAndStyle, 'fntBuildFamilyNameAndStyle');
   Result := fntBuildFamilyNameAndStyle(IFont, Name, Style);
end;

function TPDF.CalcWidthHeight(OrgWidth, OrgHeight, ScaledWidth, ScaledHeight: Double): Double;
begin
   LoadFunction(@pdfCalcWidthHeight, 'pdfCalcWidthHeight');
   Result := pdfCalcWidthHeight(FInstance, OrgWidth, OrgHeight, ScaledWidth, ScaledHeight);
end;

function TPDF.ChangeAnnotNameA(Handle: Cardinal; const Name: AnsiString): Boolean;
begin
   LoadFunction(@pdfChangeAnnotNameA, 'pdfChangeAnnotNameA');
   Result := pdfChangeAnnotNameA(FInstance, Handle, PAnsiChar(Name));
end;

function TPDF.ChangeAnnotNameW(Handle: Cardinal; const Name: WideString): Boolean;
begin
   LoadFunction(@pdfChangeAnnotNameW, 'pdfChangeAnnotNameW');
   Result := pdfChangeAnnotNameW(FInstance, Handle, PWideChar(Name));
end;

function TPDF.ChangeAnnotPos(Handle: Cardinal; PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfChangeAnnotPos, 'pdfChangeAnnotPos');
   Result := pdfChangeAnnotPos(FInstance, Handle, PosX, PosY, Width, Height);
end;

function TPDF.ChangeBookmarkA(ABmk: Integer; const Title: AnsiString; Parent: Integer; DestPage: Cardinal; Open: Boolean): Boolean;
begin
   LoadFunction(@pdfChangeBookmarkA, 'pdfChangeBookmarkA');
   Result := pdfChangeBookmarkA(FInstance, ABmk, PAnsiChar(Title), Parent, DestPage, Integer(Open));
end;

function TPDF.ChangeBookmarkW(ABmk: Integer; const Title: WideString; Parent: Integer; DestPage: Cardinal; Open: Boolean): Boolean;
begin
   LoadFunction(@pdfChangeBookmarkW, 'pdfChangeBookmarkW');
   Result := pdfChangeBookmarkW(FInstance, ABmk, PWideChar(Title), Parent, DestPage, Integer(Open));
end;

function TPDF.ChangeFont(Handle: Integer): Boolean;
begin
   LoadFunction(@pdfChangeFont, 'pdfChangeFont');
   Result := pdfChangeFont(FInstance, Handle);
end;

function TPDF.ChangeFontSize(Size: Double): Boolean;
begin
   LoadFunction(@pdfChangeFontSize, 'pdfChangeFontSize');
   Result := pdfChangeFontSize(FInstance, Size);
end;

function TPDF.ChangeFontStyle(Style: TFStyle): Boolean;
begin
   LoadFunction(@pdfChangeFontStyle, 'pdfChangeFontStyle');
   Result := pdfChangeFontStyle(FInstance, Style);
end;

function TPDF.ChangeFontStyleEx(Style: TFStyle): Boolean;
begin
   LoadFunction(@pdfChangeFontStyleEx, 'pdfChangeFontStyleEx');
   Result := pdfChangeFontStyleEx(FInstance, Style);
end;

function TPDF.ChangeJavaScriptA(Handle: Cardinal; const NewScript: AnsiString): Boolean;
begin
   LoadFunction(@pdfChangeJavaScriptA, 'pdfChangeJavaScriptA');
   Result := pdfChangeJavaScriptA(FInstance, Handle, PAnsiChar(NewScript));
end;

function TPDF.ChangeJavaScriptW(Handle: Cardinal; const NewScript: WideString): Boolean;
begin
   LoadFunction(@pdfChangeJavaScriptW, 'pdfChangeJavaScriptW');
   Result := pdfChangeJavaScriptW(FInstance, Handle, PWideChar(NewScript));
end;

function TPDF.ChangeJavaScriptActionA(Handle: Cardinal; const NewScript: AnsiString): Boolean;
begin
   LoadFunction(@pdfChangeJavaScriptActionA, 'pdfChangeJavaScriptActionA');
   Result := pdfChangeJavaScriptActionA(FInstance, Handle, PAnsiChar(NewScript));
end;

function TPDF.ChangeJavaScriptActionW(Handle: Cardinal; const NewScript: WideString): Boolean;
begin
   LoadFunction(@pdfChangeJavaScriptActionW, 'pdfChangeJavaScriptActionW');
   Result := pdfChangeJavaScriptActionW(FInstance, Handle, PWideChar(NewScript));
end;

function TPDF.ChangeJavaScriptNameA(Handle: Cardinal; const Name: AnsiString): Boolean;
begin
   LoadFunction(@pdfChangeJavaScriptNameA, 'pdfChangeJavaScriptNameA');
   Result := pdfChangeJavaScriptNameA(FInstance, Handle, PAnsiChar(Name));
end;

function TPDF.ChangeJavaScriptNameW(Handle: Cardinal; const Name: WideString): Boolean;
begin
   LoadFunction(@pdfChangeJavaScriptNameW, 'pdfChangeJavaScriptNameW');
   Result := pdfChangeJavaScriptNameW(FInstance, Handle, PWideChar(Name));
end;

function TPDF.ChangeLinkAnnot(Handle: Cardinal; const URL: AnsiString): Boolean;
begin
   LoadFunction(@pdfChangeLinkAnnot, 'pdfChangeLinkAnnot');
   Result := pdfChangeLinkAnnot(FInstance, Handle, PAnsiChar(URL));
end;

function TPDF.CheckCollection: Boolean;
begin
   LoadFunction(@pdfCheckCollection, 'pdfCheckCollection');
   Result := pdfCheckCollection(FInstance);
end;

function TPDF.CheckConformance(ConfType: TConformanceType; Options: TCheckOptions; UserData: Pointer; OnFontNotFound: TOnFontNotFoundProc; OnReplaceICCProfile: TOnReplaceICCProfile): Integer;
begin
   LoadFunction(@pdfCheckConformance, 'pdfCheckConformance');
   Result := pdfCheckConformance(FInstance, Integer(ConfType), Options, UserData, OnFontNotFound, OnReplaceICCProfile);
end;

function TPDF.CheckFieldNames(): Integer;
begin
   LoadFunction(@pdfCheckFieldNames, 'pdfCheckFieldNames');
   Result := pdfCheckFieldNames(FInstance);
end;

function TPDF.ClearAutoTemplates: Boolean;
begin
   LoadFunction(@pdfClearAutoTemplates, 'pdfClearAutoTemplates');
   Result := pdfClearAutoTemplates(FInstance);
end;

function TPDF.ClearHostFonts(): Boolean;
begin
   LoadFunction(@pdfClearHostFonts, 'pdfClearHostFonts');
   Result := pdfClearHostFonts(FInstance);
end;

function TPDF.ClipPath(ClipMode: TClippingMode; FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfClipPath, 'pdfClipPath');
   Result := pdfClipPath(FInstance, ClipMode, FillMode);
end;

function TPDF.CloseAndSignFile(const CertFile, Password, Reason, Location: AnsiString): Boolean;
begin
   LoadFunction(@pdfCloseAndSignFile, 'pdfCloseAndSignFile');
   Result := pdfCloseAndSignFile(FInstance, PAnsiChar(CertFile), PAnsiChar(Password), PAnsiChar(Reason), PAnsiChar(Location));
end;

function TPDF.CloseAndSignFileEx(const OpenPwd, OwnerPwd: AnsiString; KeyLen: TKeyLen; Restrict: TRestrictions; const CertFile, Password, Reason, Location: AnsiString): Boolean;
begin
   LoadFunction(@pdfCloseAndSignFileEx, 'pdfCloseAndSignFileEx');
   Result := pdfCloseAndSignFileEx(FInstance, PAnsiChar(OpenPwd), PAnsiChar(OwnerPwd), Ord(KeyLen), Ord(Restrict), PAnsiChar(CertFile), PAnsiChar(Password), PAnsiChar(Reason), PAnsiChar(Location));
end;

function TPDF.CloseAndSignFileExt(var SigParms: TPDFSigParms): Boolean;
begin
   LoadFunction(@pdfCloseAndSignFileExt, 'pdfCloseAndSignFileExt');
   Result := pdfCloseAndSignFileExt(FInstance, SigParms);
end;

function TPDF.CloseFile(): Boolean;
begin
   LoadFunction(@pdfCloseFile, 'pdfCloseFile');
   Result := pdfCloseFile(FInstance);
end;

function TPDF.CloseFileEx(const OpenPwd, OwnerPwd: AnsiString; KeyLen: TKeyLen; Restrict: TRestrictions): Boolean;
begin
   LoadFunction(@pdfCloseFileEx, 'pdfCloseFileEx');
   Result := pdfCloseFileEx(FInstance, PAnsiChar(OpenPwd), PAnsiChar(OwnerPwd), Ord(KeyLen), Ord(Restrict));
end;

function TPDF.CloseImage: Boolean;
begin
   LoadFunction(@pdfCloseImage, 'pdfCloseImage');
   Result := pdfCloseImage(FInstance);
end;

function TPDF.CloseImportFile(): Boolean;
begin
   LoadFunction(@pdfCloseImportFile, 'pdfCloseImportFile');
   Result := pdfCloseImportFile(FInstance);
end;

function TPDF.ClosePath(FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfClosePath, 'pdfClosePath');
   Result := pdfClosePath(FInstance, FillMode);
end;

function TPDF.CloseTag: Boolean;
begin
   LoadFunction(@pdfCloseTag, 'pdfCloseTag');
   Result := pdfCloseTag(FInstance);
end;

function TPDF.ComputeBBox(var BBox: TPDFRect; Flags: TCompBBoxFlags): Integer;
begin
   LoadFunction(@pdfComputeBBox, 'pdfComputeBBox');
   Result := pdfComputeBBox(FInstance, BBox, Flags);
end;

function TPDF.ConvColor(const Color: PDouble; NumComps: Cardinal; SourceCS: TExtColorSpace; const IColorSpace: Pointer; DestCS: TExtColorSpace): Cardinal;
begin
   LoadFunction(@pdfConvColor, 'pdfConvColor');
   Result := pdfConvColor(Color, NumComps, Integer(SourceCS), IColorSpace, Integer(DestCS));
end;

function TPDF.ConvertColors(Flags: TColorConvFlags): Integer;
begin
   LoadFunction(@pdfConvertColors, 'pdfConvertColors');
   Result := pdfConvertColors(FInstance, Flags, nil);
end;

function TPDF.ConvToUnicode(const AString: AnsiString; CP: TCodepage): WideString;
begin
   LoadFunction(@pdfConvToUnicode, 'pdfConvToUnicode');
   Result := pdfConvToUnicode(FInstance, PAnsiChar(AString), Integer(CP));
end;

function TPDF.Create3DAnnot(PosX, PosY, Width, Height: Double; const Author, AnnotName, U3DFile, ImageFile: AnsiString): Integer;
begin
   LoadFunction(@pdfCreate3DAnnot, 'pdfCreate3DAnnot');
   Result := pdfCreate3DAnnot(FInstance, PosX, PosY, Width, Height, PAnsiChar(Author), PAnsiChar(AnnotName), PAnsiChar(U3DFile), PAnsiChar(ImageFile));
end;

function TPDF.Create3DBackground(const IView: Pointer; BackColor: Cardinal): Boolean;
begin
   LoadFunction(@pdfCreate3DBackground, 'pdfCreate3DBackground');
   Result := pdfCreate3DBackground(FInstance, IView, BackColor);
end;

function TPDF.Create3DGotoViewAction(Base3DAnnot: Cardinal; const IView: Pointer; Named: T3DNamedAction): Integer;
begin
   LoadFunction(@pdfCreate3DGotoViewAction, 'pdfCreate3DGotoViewAction');
   Result := pdfCreate3DGotoViewAction(FInstance, Base3DAnnot, IView, Integer(Named));
end;

function TPDF.Create3DProjection(const IView: Pointer; ProjType: T3DProjType; ScaleType: T3DScaleType; Diameter, FOV: Double): Boolean;
begin
   LoadFunction(@pdfCreate3DProjection, 'pdfCreate3DProjection');
   Result := pdfCreate3DProjection(FInstance, IView, Integer(ProjType), Integer(ScaleType), Diameter, FOV);
end;

function TPDF.Create3DView(Base3DAnnot: Cardinal; const Name: AnsiString; SetAsDefault: Boolean; Matrix: PDouble; CamDistance: Double; RM: T3DRenderingMode; LS: T3DLightingSheme): Pointer;
begin
   LoadFunction(@pdfCreate3DView, 'pdfCreate3DView');
   Result := pdfCreate3DView(FInstance, Base3DAnnot, PAnsiChar(Name), SetAsDefault, Matrix, CamDistance, Integer(RM), Integer(LS));
end;

function TPDF.CreateAnnotAP(Annot: Cardinal): Integer;
begin
   LoadFunction(@pdfCreateAnnotAP, 'pdfCreateAnnotAP');
   Result := pdfCreateAnnotAP(FInstance, Annot);
end;

function TPDF.CreateArticleThreadA(const ThreadName: AnsiString): Integer;
begin
   LoadFunction(@pdfCreateArticleThreadA, 'pdfCreateArticleThreadA');
   Result := pdfCreateArticleThreadA(FInstance, PAnsiChar(ThreadName));
end;

function TPDF.CreateArticleThreadW(const ThreadName: WideString): Integer;
begin
   LoadFunction(@pdfCreateArticleThreadW, 'pdfCreateArticleThreadW');
   Result := pdfCreateArticleThreadW(FInstance, PWideChar(ThreadName));
end;

function TPDF.CreateAxialShading(sX, sY, eX, eY, SCenter: Double; SColor, EColor: Cardinal; Extend1, Extend2: Boolean): Integer;
begin
   LoadFunction(@pdfCreateAxialShading, 'pdfCreateAxialShading');
   Result := pdfCreateAxialShading(FInstance, sX, sY, eX, eY, SCenter, SColor, EColor, Integer(Extend1), Integer(Extend2));
end;

function TPDF.CreateButtonA(const Name, Caption: AnsiString; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
begin
   LoadFunction(@pdfCreateButtonA, 'pdfCreateButtonA');
   Result := pdfCreateButtonA(FInstance, PAnsiChar(Name), PAnsiChar(Caption), Parent, PosX, PosY, Width, Height);
end;

function TPDF.CreateButtonW(const Name: AnsiString; const Caption: WideString; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
begin
   LoadFunction(@pdfCreateButtonW, 'pdfCreateButtonW');
   Result := pdfCreateButtonW(FInstance, PAnsiChar(Name), PWideChar(Caption), Parent, PosX, PosY, Width, Height);
end;

function TPDF.CreateCheckBox(const Name, ExpValue: AnsiString; Checked: Boolean; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
begin
   LoadFunction(@pdfCreateCheckBox, 'pdfCreateCheckBox');
   Result := pdfCreateCheckBox(FInstance, PAnsiChar(Name), PAnsiChar(ExpValue), Integer(Checked), Parent, PosX, PosY, Width, Height);
end;

function TPDF.CreateCIEColorSpace(Base: TExtColorSpace; WhitePoint, BlackPoint, Gamma, Matrix: PSingle): Integer;
begin
   LoadFunction(@pdfCreateCIEColorSpace, 'pdfCreateCIEColorSpace');
   Result := pdfCreateCIEColorSpace(FInstance, Base, WhitePoint, BlackPoint, Gamma, Matrix);
end;

function TPDF.CreateCollection(View: TColView): Boolean;
begin
   LoadFunction(@pdfCreateCollection, 'pdfCreateCollection');
   Result := pdfCreateCollection(FInstance, Integer(View));
end;

function TPDF.CreateCollectionFieldA(ColType: TColColumnType; Column: Integer; const Name, Key: AnsiString; Visible, Editable: Boolean): Integer;
begin
   LoadFunction(@pdfCreateCollectionFieldA, 'pdfCreateCollectionFieldA');
   Result := pdfCreateCollectionFieldA(FInstance, Integer(ColType), Column, PAnsiChar(Name), PAnsiChar(Key), Visible, Editable);
end;

function TPDF.CreateCollectionFieldW(ColType: TColColumnType; Column: Integer; const Name: WideString; const Key: AnsiString; Visible, Editable: Boolean): Integer;
begin
   LoadFunction(@pdfCreateCollectionFieldW, 'pdfCreateCollectionFieldW');
   Result := pdfCreateCollectionFieldW(FInstance, Integer(ColType), Column, PWideChar(Name), PAnsiChar(Key), Visible, Editable);
end;

function TPDF.CreateColItemDate(EmbFile: Cardinal; const Key: AnsiString; Date: Integer; const Prefix: AnsiString): Boolean;
begin
   LoadFunction(@pdfCreateColItemDate, 'pdfCreateColItemDate');
   Result := pdfCreateColItemDate(FInstance, EmbFile, PAnsiChar(Key), Date, PAnsiChar(Prefix));
end;

function TPDF.CreateColItemNumber(EmbFile: Cardinal; const Key: AnsiString; Value: Double; const Prefix: AnsiString): Boolean;
begin
   LoadFunction(@pdfCreateColItemNumber, 'pdfCreateColItemNumber');
   Result := pdfCreateColItemNumber(FInstance, EmbFile, PAnsiChar(Key), Value, PAnsiChar(Prefix));
end;

function TPDF.CreateColItemStringA(EmbFile: Cardinal; const Key, Value, Prefix: AnsiString): Boolean;
begin
   LoadFunction(@pdfCreateColItemStringA, 'pdfCreateColItemStringA');
   Result := pdfCreateColItemStringA(FInstance, EmbFile, PAnsiChar(Key), PAnsiChar(Value), PAnsiChar(Prefix));
end;

function TPDF.CreateColItemStringW(EmbFile: Cardinal; const Key: AnsiString; const Value, Prefix: WideString): Boolean;
begin
   LoadFunction(@pdfCreateColItemStringW, 'pdfCreateColItemStringW');
   Result := pdfCreateColItemStringW(FInstance, EmbFile, PAnsiChar(Key), PWideChar(Value), PWideChar(Prefix));
end;

function TPDF.CreateComboBox(const Name: AnsiString; Sort: Boolean; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
begin
   LoadFunction(@pdfCreateComboBox, 'pdfCreateComboBox');
   Result := pdfCreateComboBox(FInstance, PAnsiChar(Name), Integer(Sort), Parent, PosX, PosY, Width, Height);
end;

function TPDF.CreateDeviceNColorSpace(const Colorants: Array of PAnsiChar; NumColorants: Cardinal; const PostScriptFunc: AnsiString; Alternate: TExtColorSpace; Handle: Integer): Integer;
begin
   LoadFunction(@pdfCreateDeviceNColorSpace, 'pdfCreateDeviceNColorSpace');
   Result := pdfCreateDeviceNColorSpace(FInstance, @Colorants, NumColorants, PAnsiChar(PostScriptFunc), Alternate, Handle);
end;

function TPDF.CreateExtGState(var GS: TPDFExtGState): Integer;
begin
   LoadFunction(@pdfCreateExtGState, 'pdfCreateExtGState');
   Result := pdfCreateExtGState(FInstance, GS);
end;

function TPDF.CreateGoToAction(DestType: TDestType; PageNum: Cardinal; a, b, c, d: Double): Integer;
begin
   LoadFunction(@pdfCreateGoToAction, 'pdfCreateGoToAction');
   Result := pdfCreateGoToAction(FInstance, DestType, PageNum, a, b, c, d);
end;

function TPDF.CreateGoToActionEx(NamedDest: Cardinal): Integer;
begin
   LoadFunction(@pdfCreateGoToActionEx, 'pdfCreateGoToActionEx');
   Result := pdfCreateGoToActionEx(FInstance, NamedDest);
end;

function TPDF.CreateGoToRAction(const FileName: AnsiString; PageNum: Cardinal): Integer;
begin
   LoadFunction(@pdfCreateGoToRAction, 'pdfCreateGoToRAction');
   Result := pdfCreateGoToRAction(FInstance, PAnsiChar(FileName), PageNum);
end;

function TPDF.CreateGoToRActionExA(const FileName: AnsiString; DestName: AnsiString; NewWindow: Boolean): Integer;
begin
   LoadFunction(@pdfCreateGoToRActionExA, 'pdfCreateGoToRActionExA');
   Result := pdfCreateGoToRActionExA(FInstance, PAnsiChar(FileName), PAnsiChar(DestName), NewWindow);
end;

function TPDF.CreateGoToRActionExW(const FileName: AnsiString; DestName: WideString; NewWindow: Boolean): Integer;
begin
   LoadFunction(@pdfCreateGoToRActionExW, 'pdfCreateGoToRActionExW');
   Result := pdfCreateGoToRActionExW(FInstance, PAnsiChar(FileName), PWideChar(DestName), NewWindow);
end;

function TPDF.CreateGroupField(const Name: AnsiString; Parent: Integer): Integer;
begin
   LoadFunction(@pdfCreateGroupField, 'pdfCreateGroupField');
   Result := pdfCreateGroupField(FInstance, PAnsiChar(Name), Parent);
end;

function TPDF.CreateHideAction(AField: Cardinal; Hide: Boolean): Integer;
begin
   LoadFunction(@pdfCreateHideAction, 'pdfCreateHideAction');
   Result := pdfCreateHideAction(FInstance, AField, Hide);
end;

function TPDF.CreateICCBasedColorSpaceA(const ICCProfile: AnsiString): Integer;
begin
   LoadFunction(@pdfCreateICCBasedColorSpaceA, 'pdfCreateICCBasedColorSpaceA');
   Result := pdfCreateICCBasedColorSpaceA(FInstance, PAnsiChar(ICCProfile));
end;

function TPDF.CreateICCBasedColorSpaceW(const ICCProfile: WideString): Integer;
begin
   LoadFunction(@pdfCreateICCBasedColorSpaceW, 'pdfCreateICCBasedColorSpaceW');
   Result := pdfCreateICCBasedColorSpaceW(FInstance, PWideChar(ICCProfile));
end;

function TPDF.CreateImageA(const FileName: AnsiString; Format: TImageFormat): Boolean;
begin
   LoadFunction(@pdfCreateImageA, 'pdfCreateImageA');
   Result := pdfCreateImageA(FInstance, PAnsiChar(FileName), Format);
end;

function TPDF.CreateImageW(const FileName: WideString; Format: TImageFormat): Boolean;
begin
   LoadFunction(@pdfCreateImageW, 'pdfCreateImageW');
   Result := pdfCreateImageW(FInstance, PWideChar(FileName), Format);
end;

function TPDF.CreateImpDataActionA(const DataFile: AnsiString): Integer;
begin
   LoadFunction(@pdfCreateImpDataActionA, 'pdfCreateImpDataActionA');
   Result := pdfCreateImpDataActionA(FInstance, PAnsiChar(DataFile));
end;

function TPDF.CreateImpDataActionW(const DataFile: WideString): Integer;
begin
   LoadFunction(@pdfCreateImpDataActionW, 'pdfCreateImpDataActionW');
   Result := pdfCreateImpDataActionW(FInstance, PWideChar(DataFile));
end;

function TPDF.CreateIndexedColorSpace(Base: TExtColorSpace; Handle: Integer; const ColorTable: Pointer; NumColors: Cardinal): Integer;
begin
   LoadFunction(@pdfCreateIndexedColorSpace, 'pdfCreateIndexedColorSpace');
   Result := pdfCreateIndexedColorSpace(FInstance, Integer(Base), Handle, ColorTable, NumColors);
end;

function TPDF.CreateJSActionA(const Script: AnsiString): Integer;
begin
   LoadFunction(@pdfCreateJSActionA, 'pdfCreateJSActionA');
   Result := pdfCreateJSActionA(FInstance, PAnsiChar(Script));
end;

function TPDF.CreateJSActionW(const Script: WideString): Integer;
begin
   LoadFunction(@pdfCreateJSActionW, 'pdfCreateJSActionW');
   Result := pdfCreateJSActionW(FInstance, PWideChar(Script));
end;

function TPDF.CreateLaunchAction(OP: TFileOP; const FileName, DefDir, Param: AnsiString; NewWindow: Boolean): Integer;
begin
   LoadFunction(@pdfCreateLaunchAction, 'pdfCreateLaunchAction');
   Result := pdfCreateLaunchAction(FInstance, Ord(OP), PAnsiChar(FileName), PAnsiChar(DefDir), PAnsiChar(Param), NewWindow);
end;

function TPDF.CreateListBox(const Name: PAnsiChar; Sort: Boolean; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
begin
   LoadFunction(@pdfCreateListBox, 'pdfCreateListBox');
   Result := pdfCreateListBox(FInstance, PAnsiChar(Name), Sort, Parent, PosX, PosY, Width, Height);
end;

function TPDF.CreateNamedAction(Action: TNamedAction): Integer;
begin
   LoadFunction(@pdfCreateNamedAction, 'pdfCreateNamedAction');
   Result := pdfCreateNamedAction(FInstance, Ord(Action));
end;

function TPDF.CreateNamedDestA(const Name: AnsiString; DestPage: Cardinal; DestType: TDestType; a, b, c, d: Double): Integer;
begin
   LoadFunction(@pdfCreateNamedDestA, 'pdfCreateNamedDestA');
   Result := pdfCreateNamedDestA(FInstance, PAnsiChar(Name), DestPage, DestType, a, b, c, d);
end;

function TPDF.CreateNamedDestW(const Name: WideString; DestPage: Cardinal; DestType: TDestType; a, b, c, d: Double): Integer;
begin
   LoadFunction(@pdfCreateNamedDestW, 'pdfCreateNamedDestW');
   Result := pdfCreateNamedDestW(FInstance, PWideChar(Name), DestPage, DestType, a, b, c, d);
end;

function TPDF.CreateNewPDFA(const OutPDF: AnsiString): Boolean;
begin
   LoadFunction(@pdfCreateNewPDFA, 'pdfCreateNewPDFA');
   Result := pdfCreateNewPDFA(FInstance, PAnsiChar(OutPDF));
end;

function TPDF.CreateNewPDFW(const OutPDF: WideString): Boolean;
begin
   LoadFunction(@pdfCreateNewPDFW, 'pdfCreateNewPDFW');
   Result := pdfCreateNewPDFW(FInstance, PWideChar(OutPDF));
end;

function TPDF.CreateOCGA(const Name: AnsiString; DisplayInUI, Visible: Boolean; Intent: TOCGIntent): Integer;
begin
   LoadFunction(@pdfCreateOCGA, 'pdfCreateOCGA');
   Result := pdfCreateOCGA(FInstance, PAnsiChar(Name), DisplayInUI, Visible, Intent);
end;

function TPDF.CreateOCGW(const Name: WideString; DisplayInUI, Visible: Boolean; Intent: TOCGIntent): Integer;
begin
   LoadFunction(@pdfCreateOCGW, 'pdfCreateOCGW');
   Result := pdfCreateOCGW(FInstance, PWideChar(Name), DisplayInUI, Visible, Intent);
end;

function TPDF.CreateOCMD(Visibility: TOCVisibility; const OCGs: PCardinal; Count: Cardinal): Integer;
begin
   LoadFunction(@pdfCreateOCMD, 'pdfCreateOCMD');
   Result := pdfCreateOCMD(FInstance, Visibility, OCGs, Count);
end;

function TPDF.CreateRadialShading(sX, sY, R1, eX, eY, R2, SCenter: Double; SColor, EColor: Cardinal; Extend1, Extend2: Boolean): Integer;
begin
   LoadFunction(@pdfCreateRadialShading, 'pdfCreateRadialShading');
   Result := pdfCreateRadialShading(FInstance, sX, sY, R1, eX, eY, R2, SCenter, SColor, EColor, Integer(Extend1), Integer(Extend2));
end;

function TPDF.CreateRadioButton(const Name, ExpValue: AnsiString; Checked: Boolean; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
begin
   LoadFunction(@pdfCreateRadioButton, 'pdfCreateRadioButton');
   Result := pdfCreateRadioButton(FInstance, PAnsiChar(Name), PAnsiChar(ExpValue), Integer(Checked), Parent, PosX, PosY, Width, Height);
end;

function TPDF.CreateResetAction(): Integer;
begin
   LoadFunction(@pdfCreateResetAction, 'pdfCreateResetAction');
   Result := pdfCreateResetAction(FInstance);
end;

function TPDF.CreateSeparationCS(const Colorant: AnsiString; Alternate: TExtColorSpace; Handle: Integer; Color: Cardinal): Integer;
begin
   LoadFunction(@pdfCreateSeparationCS, 'pdfCreateSeparationCS');
   Result := pdfCreateSeparationCS(FInstance, PAnsiChar(Colorant), Integer(Alternate), Handle, Color);
end;

function TPDF.CreateSetOCGStateAction(const On_: PCardinal; OnCount: Cardinal; const Off: PCardinal; OffCount: Cardinal; const Toggle: PCardinal; ToggleCount: Cardinal; PreserveRB: Boolean): Integer;
begin
   LoadFunction(@pdfCreateSetOCGStateAction, 'pdfCreateSetOCGStateAction');
   Result := pdfCreateSetOCGStateAction(FInstance, On_, OnCount, Off, OffCount, Toggle, ToggleCount, PreserveRB);
end;

function TPDF.CreateSigField(const Name: AnsiString; Parent: Integer; PosX, PosY, Width, Height: Double): Integer;
begin
   LoadFunction(@pdfCreateSigField, 'pdfCreateSigField');
   Result := pdfCreateSigField(FInstance, PAnsiChar(Name), Parent, PosX, PosY, Width, Height);
end;

function TPDF.CreateSigFieldAP(SigField: Cardinal): Integer;
begin
   LoadFunction(@pdfCreateSigFieldAP, 'pdfCreateSigFieldAP');
   Result := pdfCreateSigFieldAP(FInstance, SigField);
end;

function TPDF.CreateStdPattern(Pattern: TStdPattern; LineWidth, Distance: Double; LineColor, BackColor: Cardinal): Integer;
begin
   LoadFunction(@pdfCreateStdPattern, 'pdfCreateStdPattern');
   Result := pdfCreateStdPattern(FInstance, Ord(Pattern), LineWidth, Distance, LineColor, BackColor);
end;

function TPDF.CreateStructureTree: Boolean;
begin
   LoadFunction(@pdfCreateStructureTree, 'pdfCreateStructureTree');
   Result := pdfCreateStructureTree(FInstance);
end;

function TPDF.CreateSubmitAction(Flags: TSubmitFlags; const URL: AnsiString): Integer;
begin
   LoadFunction(@pdfCreateSubmitAction, 'pdfCreateSubmitAction');
   Result := pdfCreateSubmitAction(FInstance, Flags, PAnsiChar(URL));
end;

function TPDF.CreateTextField(const Name: AnsiString; Parent: Integer; Multiline: Boolean; MaxLen: Integer; PosX, PosY, Width, Height: Double): Integer;
begin
   LoadFunction(@pdfCreateTextField, 'pdfCreateTextField');
   Result := pdfCreateTextField(FInstance, PAnsiChar(Name), Parent, Integer(Multiline), MaxLen, PosX, PosY, Width, Height);
end;

function TPDF.CreateURIAction(const URL: AnsiString): Integer;
begin
   LoadFunction(@pdfCreateURIAction, 'pdfCreateURIAction');
   Result := pdfCreateURIAction(FInstance, PAnsiChar(URL));
end;

function TPDF.DecryptPDFA(const FileName: AnsiString; PwdType: TPwdType; const Password: AnsiString): Integer;
begin
   LoadFunction(@pdfDecryptPDFA, 'pdfDecryptPDFA');
   Result := pdfDecryptPDFA(FInstance, PAnsiChar(FileName), Ord(PwdType), PAnsiChar(Password));
end;

function TPDF.DecryptPDFW(const FileName: WideString; PwdType: TPwdType; const Password: AnsiString): Integer;
begin
   LoadFunction(@pdfDecryptPDFW, 'pdfDecryptPDFW');
   Result := pdfDecryptPDFW(FInstance, PWideChar(FileName), Ord(PwdType), PAnsiChar(Password));
end;

procedure TPDF.DeleteAcroForm;
begin
   LoadFunction(@pdfDeleteAcroForm, 'pdfDeleteAcroForm');
   pdfDeleteAcroForm(FInstance);
end;

function TPDF.DeleteActionFromObj(ObjType: TObjType; ActHandle, ObjHandle: Cardinal): Integer;
begin
   LoadFunction(@pdfDeleteActionFromObj, 'pdfDeleteActionFromObj');
   Result := pdfDeleteActionFromObj(FInstance, Ord(ObjType), ActHandle, ObjHandle);
end;

function TPDF.DeleteActionFromObjEx(ObjType: TObjType; ObjHandle, ActIndex: Cardinal): Integer;
begin
   LoadFunction(@pdfDeleteActionFromObjEx, 'pdfDeleteActionFromObjEx');
   Result := pdfDeleteActionFromObjEx(FInstance, Ord(ObjType), ObjHandle, ActIndex);
end;

function TPDF.DeleteAnnotation(Handle: Cardinal): Boolean;
begin
   LoadFunction(@pdfDeleteAnnotation, 'pdfDeleteAnnotation');
   Result := pdfDeleteAnnotation(FInstance, Handle);
end;

function TPDF.DeleteBookmark(ABmk: Cardinal): Integer;
begin
   LoadFunction(@pdfDeleteBookmark, 'pdfDeleteBookmark');
   Result := pdfDeleteBookmark(FInstance, ABmk);
end;

function TPDF.DeleteEmbeddedFile(Handle: Cardinal): Boolean;
begin
   LoadFunction(@pdfDeleteEmbeddedFile, 'pdfDeleteEmbeddedFile');
   Result := pdfDeleteEmbeddedFile(FInstance, Handle);
end;

function TPDF.DeleteField(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfDeleteField, 'pdfDeleteField');
   Result := pdfDeleteField(FInstance, AField);
end;

function TPDF.DeleteFieldEx(const Name: AnsiString): Integer;
begin
   LoadFunction(@pdfDeleteFieldEx, 'pdfDeleteFieldEx');
   Result := pdfDeleteFieldEx(FInstance, PAnsiChar(Name));
end;

procedure TPDF.DeleteJavaScripts(DelJavaScriptActions: Boolean);
begin
   LoadFunction(@pdfDeleteJavaScripts, 'pdfDeleteJavaScripts');
   pdfDeleteJavaScripts(FInstance, DelJavaScriptActions);
end;

function TPDF.DeletePage(PageNum: Cardinal): Integer;
begin
   LoadFunction(@pdfDeletePage, 'pdfDeletePage');
   Result := pdfDeletePage(FInstance, PageNum);
end;

procedure TPDF.DeletePageLabels;
begin
   LoadFunction(@pdfDeletePageLabels, 'pdfDeletePageLabels');
   pdfDeletePageLabels(FInstance);
end;

function TPDF.DeleteSeparationInfo(AllPages: Boolean): Boolean;
begin
   LoadFunction(@pdfDeleteSeparationInfo, 'pdfDeleteSeparationInfo');
   Result := pdfDeleteSeparationInfo(FInstance, AllPages);
end;

function TPDF.DeleteTemplate(Handle: Cardinal): Boolean;
begin
   LoadFunction(@pdfDeleteTemplate, 'pdfDeleteTemplate');
   Result := pdfDeleteTemplate(FInstance, Handle);
end;

function TPDF.DeleteTemplateEx(Index: Cardinal): Integer;
begin
   LoadFunction(@pdfDeleteTemplateEx, 'pdfDeleteTemplateEx');
   Result := pdfDeleteTemplateEx(FInstance, Index);
end;

procedure TPDF.DeleteXFAForm;
begin
   LoadFunction(@pdfDeleteXFAForm, 'pdfDeleteXFAForm');
   pdfDeleteXFAForm(FInstance);
end;

function TPDF.DrawArc(PosX, PosY, Radius, StartAngle, EndAngle: Double): Boolean;
begin
   LoadFunction(@pdfDrawArc, 'pdfDrawArc');
   Result := pdfDrawArc(FInstance, PosX, PosY, Radius, StartAngle, EndAngle);
end;

function TPDF.DrawArcEx(PosX, PosY, Width, Height, StartAngle, EndAngle: Double): Boolean;
begin
   LoadFunction(@pdfDrawArcEx, 'pdfDrawArcEx');
   Result := pdfDrawArcEx(FInstance, PosX, PosY, Width, Height, StartAngle, EndAngle);
end;

function TPDF.DrawChord(PosX, PosY, Width, Height, StartAngle, EndAngle: Double; FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfDrawChord, 'pdfDrawChord');
   Result := pdfDrawChord(FInstance, PosX, PosY, Width, Height, StartAngle, EndAngle, Ord(FillMode));
end;

function TPDF.DrawCircle(PosX, PosY, Radius: Double; FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfDrawCircle, 'pdfDrawCircle');
   Result := pdfDrawCircle(FInstance, PosX, PosY, Radius, Ord(FillMode));
end;

function TPDF.DrawPie(PosX, PosY, Width, Height, StartAngle, EndAngle: Double; FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfDrawPie, 'pdfDrawPie');
   Result := pdfDrawPie(FInstance, PosX, PosY, Width, Height, StartAngle, EndAngle, Ord(FillMode));
end;

function TPDF.EditPage(PageNum: Integer): Boolean;
begin
   LoadFunction(@pdfEditPage, 'pdfEditPage');
   Result := pdfEditPage(FInstance, PageNum);
end;

function TPDF.EditTemplate(Index: Cardinal): Boolean;
begin
   LoadFunction(@pdfEditTemplate, 'pdfEditTemplate');
   Result := pdfEditTemplate(FInstance, Index);
end;

function TPDF.EditTemplate2(Handle: Cardinal): Boolean;
begin
   LoadFunction(@pdfEditTemplate2, 'pdfEditTemplate2');
   Result := pdfEditTemplate2(FInstance, Handle);
end;

function TPDF.Ellipse(PosX, PosY, Width, Height: Double; FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfEllipse, 'pdfEllipse');
   Result := pdfEllipse(FInstance, PosX, PosY, Width, Height, Ord(FillMode));
end;

function TPDF.EncryptPDFA(const FileName, OpenPwd, OwnerPwd: AnsiString; KeyLen: TKeyLen; Restrict: TRestrictions): Integer;
begin
   LoadFunction(@pdfEncryptPDFA, 'pdfEncryptPDFA');
   Result := pdfEncryptPDFA(FInstance, PAnsiChar(FileName), PAnsiChar(OpenPwd), PAnsiChar(OwnerPwd), Ord(KeyLen), Ord(Restrict));
end;

function TPDF.EncryptPDFW(const FileName: WideString; const OpenPwd, OwnerPwd: AnsiString; KeyLen: TKeyLen; Restrict: TRestrictions): Integer;
begin
   LoadFunction(@pdfEncryptPDFW, 'pdfEncryptPDFW');
   Result := pdfEncryptPDFW(FInstance, PWideChar(FileName), PAnsiChar(OpenPwd), PAnsiChar(OwnerPwd), Ord(KeyLen), Ord(Restrict));
end;

function TPDF.EndContinueText(): Integer;
begin
   LoadFunction(@pdfEndContinueText, 'pdfEndContinueText');
   Result := pdfEndContinueText(FInstance);
end;

function TPDF.EndLayer: Boolean;
begin
   LoadFunction(@pdfEndLayer, 'pdfEndLayer');
   Result := pdfEndLayer(FInstance);
end;

function TPDF.EndPage(): Boolean;
begin
   LoadFunction(@pdfEndPage, 'pdfEndPage');
   Result := pdfEndPage(FInstance);
end;

function TPDF.EndPattern(): Boolean;
begin
   LoadFunction(@pdfEndPattern, 'pdfEndPattern');
   Result := pdfEndPattern(FInstance);
end;

function TPDF.EndTemplate(): Boolean;
begin
   LoadFunction(@pdfEndTemplate, 'pdfEndTemplate');
   Result := pdfEndTemplate(FInstance);
end;

function TPDF.EnumDocFonts(const Data: Pointer; EnumProc: TEnumFontProc2): Integer;
begin
   LoadFunction(@pdfEnumDocFonts, 'pdfEnumDocFonts');
   Result := pdfEnumDocFonts(FInstance, Data, EnumProc);
end;

function TPDF.EnumHostFonts(const Data: Pointer; EnumProc: TEnumFontProc): Integer;
begin
   LoadFunction(@pdfEnumHostFonts, 'pdfEnumHostFonts');
   Result := pdfEnumHostFonts(FInstance, Data, EnumProc);
end;

function TPDF.EnumHostFontsEx(const Data: Pointer; EnumProc: TEnumFontProcEx): Integer;
begin
   LoadFunction(@pdfEnumHostFontsEx, 'pdfEnumHostFontsEx');
   Result := pdfEnumHostFontsEx(FInstance, Data, EnumProc);
end;

function TPDF.ExchangeBookmarks(Bmk1, Bmk2: Integer): Boolean;
begin
   LoadFunction(@pdfExchangeBookmarks, 'pdfExchangeBookmarks');
   Result := pdfExchangeBookmarks(FInstance, Bmk1, Bmk2);
end;

function TPDF.ExchangePages(First, Second: Cardinal): Boolean;
begin
   LoadFunction(@pdfExchangePages, 'pdfExchangePages');
   Result := pdfExchangePages(FInstance, First, Second);
end;

function TPDF.FileAttachAnnotA(PosX, PosY: Double; Icon: TFileAttachIcon; const Author, Desc, AFile: AnsiString; Compress: Boolean): Integer;
begin
   LoadFunction(@pdfFileAttachAnnotA, 'pdfFileAttachAnnotA');
   Result := pdfFileAttachAnnotA(FInstance, PosX, PosY, Integer(Icon), PAnsiChar(Author), PAnsiChar(Desc), PAnsiChar(AFile), Compress);
end;

function TPDF.FileAttachAnnotW(PosX, PosY: Double; Icon: TFileAttachIcon; const Author, Desc, AFile: WideString; Compress: Boolean): Integer;
begin
   LoadFunction(@pdfFileAttachAnnotW, 'pdfFileAttachAnnotW');
   Result := pdfFileAttachAnnotW(FInstance, PosX, PosY, Integer(Icon), PWideChar(Author), PWideChar(Desc), PWideChar(AFile), Compress);
end;

function TPDF.FileLinkA(PosX, PosY, Width, Height: Double; const AFilePath: AnsiString): Integer;
begin
   LoadFunction(@pdfFileLinkA, 'pdfFileLinkA');
   Result := pdfFileLinkA(FInstance, PosX, PosY, Width, Height, PAnsiChar(AFilePath));
end;

function TPDF.FileLinkW(PosX, PosY, Width, Height: Double; const AFilePath: WideString): Integer;
begin
   LoadFunction(@pdfFileLinkW, 'pdfFileLinkW');
   Result := pdfFileLinkW(FInstance, PosX, PosY, Width, Height, PWideChar(AFilePath));
end;

function TPDF.FindBookmarkA(DestPage: Integer; const Title: AnsiString): Integer;
begin
   LoadFunction(@pdfFindBookmarkA, 'pdfFindBookmarkA');
   Result := pdfFindBookmarkA(FInstance, DestPage, PAnsiChar(Title));
end;

function TPDF.FindBookmarkW(DestPage: Integer; const Title: WideString): Integer;
begin
   LoadFunction(@pdfFindBookmarkW, 'pdfFindBookmarkW');
   Result := pdfFindBookmarkW(FInstance, DestPage, PWideChar(Title));
end;

function TPDF.FindFieldA(const Name: AnsiString): Integer;
begin
   LoadFunction(@pdfFindFieldA, 'pdfFindFieldA');
   Result := pdfFindFieldA(FInstance, PAnsiChar(Name));
end;

function TPDF.FindFieldW(const Name: WideString): Integer;
begin
   LoadFunction(@pdfFindFieldW, 'pdfFindFieldW');
   Result := pdfFindFieldW(FInstance, PWideChar(Name));
end;

function TPDF.FindLinkAnnot(const URL: AnsiString): Integer;
begin
   LoadFunction(@pdfFindLinkAnnot, 'pdfFindLinkAnnot');
   Result := pdfFindLinkAnnot(FInstance, PAnsiChar(URL));
end;

function TPDF.FindNextBookmark(): Integer;
begin
   LoadFunction(@pdfFindNextBookmark, 'pdfFindNextBookmark');
   Result := pdfFindNextBookmark(FInstance);
end;

function TPDF.FinishSignature(const PKCS7Obj: Pointer; Length: Cardinal): Boolean;
begin
   LoadFunction(@pdfFinishSignature, 'pdfFinishSignature');
   Result := pdfFinishSignature(FInstance, PKCS7Obj, Length);
end;

function TPDF.FlattenAnnots(Flags: TAnnotFlattenFlags): Integer;
begin
   LoadFunction(@pdfFlattenAnnots, 'pdfFlattenAnnots');
   Result := pdfFlattenAnnots(FInstance, Flags);
end;

function TPDF.FlattenForm(): Boolean;
begin
   LoadFunction(@pdfFlattenForm, 'pdfFlattenForm');
   Result := pdfFlattenForm(FInstance);
end;

function TPDF.FlushPageContent(var Stack: TPDFStack): Boolean;
begin
   LoadFunction(@pdfFlushPageContent, 'pdfFlushPageContent');
   Result := pdfFlushPageContent(FInstance, Stack);
end;

procedure TPDF.FreeImageBuffer;
begin
   LoadFunction(@pdfFreeImageBuffer, 'pdfFreeImageBuffer');
   pdfFreeImageBuffer(FInstance);
end;

function TPDF.FreePDF(): Boolean;
begin
   LoadFunction(@pdfFreePDF, 'pdfFreePDF');
   Result := pdfFreePDF(FInstance);
end;

function TPDF.FreeTextAnnotA(PosX, PosY, Width, Height: Double; const Author, AText: AnsiString; Align: TTextAlign): Integer;
begin
   LoadFunction(@pdfFreeTextAnnotA, 'pdfFreeTextAnnotA');
   Result := pdfFreeTextAnnotA(FInstance, PosX, PosY, Width, Height, PAnsiChar(Author), PAnsiChar(AText), Ord(Align));
end;

function TPDF.FreeTextAnnotW(PosX, PosY, Width, Height: Double; const Author, AText: WideString; Align: TTextAlign): Integer;
begin
   LoadFunction(@pdfFreeTextAnnotW, 'pdfFreeTextAnnotW');
   Result := pdfFreeTextAnnotW(FInstance, PosX, PosY, Width, Height, PWideChar(Author), PWideChar(AText), Ord(Align));
end;

function TPDF.FreeUniBuf(): Boolean;
begin
   LoadFunction(@pdfFreeUniBuf, 'pdfFreeUniBuf');
   Result := pdfFreeUniBuf(FInstance);
end;

function TPDF.GetActionCount(): Integer;
begin
   LoadFunction(@pdfGetActionCount, 'pdfGetActionCount');
   Result := pdfGetActionCount(FInstance);
end;

function TPDF.GetActionType(ActHandle: Cardinal): Integer;
begin
   LoadFunction(@pdfGetActionType, 'pdfGetActionType');
   Result := pdfGetActionType(FInstance, ActHandle);
end;

function TPDF.GetActionTypeEx(ObjType: TObjType; ObjHandle, ActIndex: Cardinal): Integer;
begin
   LoadFunction(@pdfGetActionTypeEx, 'pdfGetActionTypeEx');
   Result := pdfGetActionTypeEx(FInstance, ObjType, ObjHandle, ActIndex);
end;

function TPDF.GetActiveFont: Integer;
begin
   LoadFunction(@pdfGetActiveFont, 'pdfGetActiveFont');
   Result := pdfGetActiveFont(FInstance);
end;

function TPDF.GetAllocBy(): Integer;
begin
   LoadFunction(@pdfGetAllocBy, 'pdfGetAllocBy');
   Result := pdfGetAllocBy(FInstance);
end;

function TPDF.GetAnnot(Handle: Cardinal; var Annot: TPDFAnnotation): Boolean;
begin
   LoadFunction(@pdfGetAnnot, 'pdfGetAnnot');
   Result := pdfGetAnnot(FInstance, Handle, Annot);
end;

function TPDF.GetAnnotBBox(Handle: Cardinal; var BBox: TPDFRect): Boolean;
begin
   LoadFunction(@pdfGetAnnotBBox, 'pdfGetAnnotBBox');
   Result := pdfGetAnnotBBox(FInstance, Handle, BBox);
end;

function TPDF.GetAnnotCount(): Integer;
begin
   LoadFunction(@pdfGetAnnotCount, 'pdfGetAnnotCount');
   Result := pdfGetAnnotCount(FInstance);
end;

function TPDF.GetAnnotEx(Handle: Cardinal; var Annot: TPDFAnnotationEx): Boolean;
begin
   LoadFunction(@pdfGetAnnotEx, 'pdfGetAnnotEx');
   Result := pdfGetAnnotEx(FInstance, Handle, Annot);
end;

function TPDF.GetAnnotFlags(): Integer;
begin
   LoadFunction(@pdfGetAnnotFlags, 'pdfGetAnnotFlags');
   Result := pdfGetAnnotFlags(FInstance);
end;

function TPDF.GetAnnotLink(Handle: Cardinal): AnsiString;
begin
   LoadFunction(@pdfGetAnnotLink, 'pdfGetAnnotLink');
   Result := pdfGetAnnotLink(FInstance, Handle);
end;

function TPDF.GetAnnotType(Handle: Cardinal): Integer;
begin
   LoadFunction(@pdfGetAnnotType, 'pdfGetAnnotType');
   Result := pdfGetAnnotType(FInstance, Handle);
end;

function TPDF.GetAscent(): Double;
begin
   LoadFunction(@pdfGetAscent, 'pdfGetAscent');
   Result := pdfGetAscent(FInstance);
end;

function TPDF.GetBBox(Boundary: TPageBoundary; var BBox: TPDFRect): Boolean;
begin
   LoadFunction(@pdfGetBBox, 'pdfGetBBox');
   Result := pdfGetBBox(FInstance, Boundary, BBox);
end;

function TPDF.GetBidiMode: TPDFBidiMode;
begin
   LoadFunction(@pdfGetBidiMode, 'pdfGetBidiMode');
   Result := TPDFBidiMode(pdfGetBidiMode(FInstance));
end;

function TPDF.GetBookmark(Handle: Integer; var Bmk: TBookmark): Boolean;
begin
   LoadFunction(@pdfGetBookmark, 'pdfGetBookmark');
   Result := pdfGetBookmark(FInstance, Handle, Bmk);
end;

function TPDF.GetBookmarkCount(): Integer;
begin
   LoadFunction(@pdfGetBookmarkCount, 'pdfGetBookmarkCount');
   Result := pdfGetBookmarkCount(FInstance);
end;

function TPDF.GetBorderStyle(): TBorderStyle;
begin
   LoadFunction(@pdfGetBorderStyle, 'pdfGetBorderStyle');
   Result := TBorderStyle(pdfGetBorderStyle(FInstance));
end;

function TPDF.GetBuffer(var BufSize: Cardinal): PAnsiChar;
begin
   LoadFunction(@pdfGetBuffer, 'pdfGetBuffer');
   Result := pdfGetBuffer(FInstance, BufSize);
end;

function TPDF.GetCapHeight(): Double;
begin
   LoadFunction(@pdfGetCapHeight, 'pdfGetCapHeight');
   Result := pdfGetCapHeight(FInstance);
end;

function TPDF.GetCharacterSpacing(): Double;
begin
   LoadFunction(@pdfGetCharacterSpacing, 'pdfGetCharacterSpacing');
   Result := pdfGetCharacterSpacing(FInstance);
end;

function TPDF.GetCheckBoxChar: Integer;
begin
   LoadFunction(@pdfGetCheckBoxChar, 'pdfGetCheckBoxChar');
   Result := pdfGetCheckBoxChar(FInstance);
end;

function TPDF.GetCheckBoxCharEx(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetCheckBoxCharEx, 'pdfGetCheckBoxCharEx');
   Result := pdfGetCheckBoxCharEx(FInstance, AField);
end;

function TPDF.GetCheckBoxDefState(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetCheckBoxDefState, 'pdfGetCheckBoxDefState');
   Result := pdfGetCheckBoxDefState(FInstance, AField);
end;

function TPDF.GetCMap(Index: Cardinal; var CMap: TPDFCMap): Boolean;
begin
   LoadFunction(@pdfGetCMap, 'pdfGetCMap');
   CMap.StructSize := sizeof(CMap);
   Result := pdfGetCMap(FInstance, Index, CMap);
end;

function TPDF.GetCMapCount: Integer;
begin
   LoadFunction(@pdfGetCMapCount, 'pdfGetCMapCount');
   Result := pdfGetCMapCount(FInstance);
end;

function TPDF.GetColorSpace(): Integer;
begin
   LoadFunction(@pdfGetColorSpace, 'pdfGetColorSpace');
   Result := pdfGetColorSpace(FInstance);
end;

function TPDF.GetColorSpaceCount: Integer;
begin
   LoadFunction(@pdfGetColorSpaceCount, 'pdfGetColorSpaceCount');
   Result := pdfGetColorSpaceCount(FInstance);
end;

function TPDF.GetColorSpaceObj(Handle: Cardinal; var CS: TPDFColorSpaceObj): Boolean;
begin
   LoadFunction(@pdfGetColorSpaceObj, 'pdfGetColorSpaceObj');
   Result := pdfGetColorSpaceObj(FInstance, Handle, CS);
end;

function TPDF.GetColorSpaceObjEx(const IColorSpace: Pointer; var CS: TPDFColorSpaceObj): Boolean;
begin
   LoadFunction(@pdfGetColorSpaceObjEx, 'pdfGetColorSpaceObjEx');
   Result := pdfGetColorSpaceObjEx(IColorSpace, CS);
end;

function TPDF.GetCompressionFilter: Integer;
begin
   LoadFunction(@pdfGetCompressionFilter, 'pdfGetCompressionFilter');
   Result := pdfGetCompressionFilter(FInstance);
end;

function TPDF.GetCompressionLevel(): Integer;
begin
   LoadFunction(@pdfGetCompressionLevel, 'pdfGetCompressionLevel');
   Result := pdfGetCompressionLevel(FInstance);
end;

function TPDF.GetContent(var Buffer: PAnsiChar): Integer;
begin
   LoadFunction(@pdfGetContent, 'pdfGetContent');
   Result := pdfGetContent(FInstance, Buffer);
end;

function TPDF.GetDefBitsPerPixel(): Integer;
begin
   LoadFunction(@pdfGetDefBitsPerPixel, 'pdfGetDefBitsPerPixel');
   Result := pdfGetDefBitsPerPixel(FInstance);
end;

function TPDF.GetDescent(): Double;
begin
   LoadFunction(@pdfGetDescent, 'pdfGetDescent');
   Result := pdfGetDescent(FInstance);
end;

function TPDF.GetDeviceNAttributes(IAttributes: Pointer; var Attributes: TDeviceNAttributes): Boolean;
begin
   LoadFunction(@pdfGetDeviceNAttributes, 'pdfGetDeviceNAttributes');
   Result := pdfGetDeviceNAttributes(IAttributes, Attributes);
end;

function TPDF.GetDocInfo(DInfo: TDocumentInfo; var Value: PWideChar): Integer;
begin
   LoadFunction(@pdfGetDocInfo, 'pdfGetDocInfo');
   Result := pdfGetDocInfo(FInstance, Ord(DInfo), Value);
end;

function TPDF.GetDocInfoCount: Integer;
begin
   LoadFunction(@pdfGetDocInfoCount, 'pdfGetDocInfoCount');
   Result := pdfGetDocInfoCount(FInstance);
end;

function TPDF.GetDocInfoEx(Index: Cardinal; var DInfo: TDocumentInfo; var Key, Value: PAnsiChar; var Unicode: LongBool): Integer;
begin
   LoadFunction(@pdfGetDocInfoEx, 'pdfGetDocInfoEx');
   Result := pdfGetDocInfoEx(FInstance, Index, DInfo, Key, Value, Unicode);
end;

function TPDF.GetDrawDirection(): Integer;
begin
   LoadFunction(@pdfGetDrawDirection, 'pdfGetDrawDirection');
   Result := pdfGetDrawDirection(FInstance);
end;

function TPDF.GetDynaPDFVersion: AnsiString;
begin
   LoadFunction(@pdfGetDynaPDFVersion, 'pdfGetDynaPDFVersion');
   Result := pdfGetDynaPDFVersion();
end;

function TPDF.GetEmbeddedFile(Handle: Cardinal; var FileSpec: TPDFFileSpec; Decompress: Boolean): Boolean;
begin
   LoadFunction(@pdfGetEmbeddedFile, 'pdfGetEmbeddedFile');
   Result := pdfGetEmbeddedFile(FInstance, Handle, FileSpec, Decompress);
end;

function TPDF.GetEmbeddedFileCount: Integer;
begin
   LoadFunction(@pdfGetEmbeddedFileCount, 'pdfGetEmbeddedFileCount');
   Result := pdfGetEmbeddedFileCount(FInstance);
end;

function TPDF.GetEMFPatternDistance(): Double;
begin
   LoadFunction(@pdfGetEMFPatternDistance, 'pdfGetEMFPatternDistance');
   Result := pdfGetEMFPatternDistance(FInstance);
end;

function TPDF.GetErrorMessage(): AnsiString;
begin
   LoadFunction(@pdfGetErrorMessage, 'pdfGetErrorMessage');
   Result := pdfGetErrorMessage(FInstance);
end;

function TPDF.GetErrorMode(): Integer;
begin
   LoadFunction(@pdfGetErrorMode, 'pdfGetErrorMode');
   Result := pdfGetErrorMode(FInstance);
end;

function TPDF.GetField(Handle: Cardinal; var Field: TPDFField): Boolean;
begin
   LoadFunction(@pdfGetField, 'pdfGetField');
   {
      Note: FieldName can be an Unicode string!
      To determine whether FieldName contains an ANSI or Unicode string
      proceed as follows:
      if f.FielNameLen > 0 and StrLen(f.FieldName) <> f.FieldNameLen then begin
         // f.FieldName contains a null-terminated Unicode string
         // f.FieldNameLen is the length in Unicode characters, but we have an ANSI string.
         // So, we must multiply the length with 2 to convert the entire string!
         nameW := pdf.ToUnicode(f.FieldName, f.FieldNameLen * 2);         
      end;
   }
   Result := pdfGetField(FInstance, Handle, Field);
end;

function TPDF.GetFieldBackColor(): Cardinal;
begin
   LoadFunction(@pdfGetFieldBackColor, 'pdfGetFieldBackColor');
   Result := pdfGetFieldBackColor(FInstance);
end;

function TPDF.GetFieldBorderColor(): Cardinal;
begin
   LoadFunction(@pdfGetFieldBorderColor, 'pdfGetFieldBorderColor');
   Result := pdfGetFieldBorderColor(FInstance);
end;

function TPDF.GetFieldBorderStyle(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetFieldBorderStyle, 'pdfGetFieldBorderStyle');
   Result := pdfGetFieldBorderStyle(FInstance, AField);
end;

function TPDF.GetFieldBorderWidth(AField: Cardinal): Double;
begin
   LoadFunction(@pdfGetFieldBorderWidth, 'pdfGetFieldBorderWidth');
   Result := pdfGetFieldBorderWidth(FInstance, AField);
end;

function TPDF.GetFieldChoiceValue(AField, ValIndex: Cardinal; var Value: TPDFChoiceValue): Boolean;
begin
   LoadFunction(@pdfGetFieldChoiceValue, 'pdfGetFieldChoiceValue');
   Result := pdfGetFieldChoiceValue(FInstance, AField, ValIndex, Value);
end;

function TPDF.GetFieldColor(AField: Cardinal; ColorType: TFieldColor; var ColorSpace: Integer; var Color: Cardinal): Boolean;
begin
   LoadFunction(@pdfGetFieldColor, 'pdfGetFieldColor');
   Result := pdfGetFieldColor(FInstance, AField, Integer(ColorType), ColorSpace, Color);
end;

function TPDF.GetFieldCount(): Integer;
begin
   LoadFunction(@pdfGetFieldCount, 'pdfGetFieldCount');
   Result := pdfGetFieldCount(FInstance);
end;

function TPDF.GetFieldEx(Handle: Cardinal; var Field: TPDFFieldEx): Boolean;
begin
   LoadFunction(@pdfGetFieldEx, 'pdfGetFieldEx');
   Field.StructSize := sizeof(Field);
   Result := pdfGetFieldEx(FInstance, Handle, Field);
end;

function TPDF.GetFieldEx2(const IField: Pointer; var Field: TPDFFieldEx): Boolean;
begin
   LoadFunction(@pdfGetFieldEx2, 'pdfGetFieldEx2');
   Field.StructSize := sizeof(Field);
   Result := pdfGetFieldEx2(IField, Field);
end;

function TPDF.GetFieldExpValCount(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetFieldExpValCount, 'pdfGetFieldExpValCount');
   Result := pdfGetFieldExpValCount(FInstance, AField);
end;

function TPDF.GetFieldExpValue(AField: Cardinal; var Value: AnsiString): Integer;
var retval: PAnsiChar;
begin
   retval := nil;
   LoadFunction(@pdfGetFieldExpValue, 'pdfGetFieldExpValue');
   Result := pdfGetFieldExpValue(FInstance, AField, retval);
   Value := retval;
end;

function TPDF.GetFieldExpValueEx(AField, ValIndex: Cardinal; var Value, ExpValue: AnsiString; var Selected: Boolean): Boolean;
var val, expVal: PAnsiChar; sel: LongBool;
begin
   LoadFunction(@pdfGetFieldExpValueEx, 'pdfGetFieldExpValueEx');
   val      := nil;
   expVal   := nil;
   sel      := false;
   Result   := pdfGetFieldExpValueEx(FInstance, AField, ValIndex, val, expVal, sel);
   Value    := val;
   ExpValue := expVal;
   Selected := sel;
end;

function TPDF.GetFieldFlags(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetFieldFlags, 'pdfGetFieldFlags');
   Result := pdfGetFieldFlags(FInstance, AField);
end;

function TPDF.GetFieldGroupType(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetFieldGroupType, 'pdfGetFieldGroupType');
   Result := pdfGetFieldGroupType(FInstance, AField);
end;

function TPDF.GetFieldHighlightMode(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetFieldHighlightMode, 'pdfGetFieldHighlightMode');
   Result := pdfGetFieldHighlightMode(FInstance, AField);
end;

function TPDF.GetFieldIndex(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetFieldIndex, 'pdfGetFieldIndex');
   Result := pdfGetFieldIndex(FInstance, AField);
end;

function TPDF.GetFieldMapName(AField: Cardinal; var Value: Pointer; var Unicode: LongBool): Integer;
begin
   LoadFunction(@pdfGetFieldMapName, 'pdfGetFieldMapName');
   Result := pdfGetFieldMapName(FInstance, AField, Value, Unicode);
end;

function TPDF.GetFieldName(AField: Cardinal; var Name: AnsiString): Integer;
var retval: PAnsiChar; i, p, len: Integer;
begin
   retval := nil;
   LoadFunction(@pdfGetFieldName, 'pdfGetFieldName');
   Result := pdfGetFieldName(FInstance, AField, retval);
   if Result > 0 then begin
      len := StrLen(retval);
      if len <> Result then
         len := Result * 2
      else
         len := Result;
      SetLength(Name, len);
      {
         A field name can be an Unicode string. To determine whether we have an ANSI or an Unicode
         string proceed as follows:
         nameLen := pdf.GetFieldName(myField, name);
         if nameLen > 0 then begin
            if Length(name) <> nameLen then begin
               nameW := pdf.ToUnicode(name);
               ...
            end else begin
               ...
            end;            
         end;
      }
      p := 0;
      for i := 1 to len do begin
         Name[i] := retval[p];
         Inc(p);
      end;
   end else
      Name := '';
end;

function TPDF.GetFieldOrientation(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetFieldOrientation, 'pdfGetFieldOrientation');
   Result := pdfGetFieldOrientation(FInstance, AField);
end;

function TPDF.GetFieldTextAlign(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetFieldTextAlign, 'pdfGetFieldTextAlign');
   Result := pdfGetFieldTextAlign(FInstance, AField);
end;

function TPDF.GetFieldTextColor(): Cardinal;
begin
   LoadFunction(@pdfGetFieldTextColor, 'pdfGetFieldTextColor');
   Result := pdfGetFieldTextColor(FInstance);
end;

function TPDF.GetFieldToolTip(AField: Cardinal; var Value: Pointer; var Unicode: LongBool): Integer;
begin
   LoadFunction(@pdfGetFieldToolTip, 'pdfGetFieldToolTip');
   Result := pdfGetFieldToolTip(FInstance, AField, Value, Unicode);
end;

function TPDF.GetFieldType(AField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetFieldType, 'pdfGetFieldType');
   Result := pdfGetFieldType(FInstance, AField);
end;

function TPDF.GetFillColor(): Cardinal;
begin
   LoadFunction(@pdfGetFillColor, 'pdfGetFillColor');
   Result := pdfGetFillColor(FInstance);
end;

function TPDF.GetFont(const IFont: Pointer; var F: TPDFFontObj): Boolean;
begin
   // if F.Length1 = 0 and if F.FontFile <> nil then the variable contains
   // the file path to the used font file. This string is always an Ansi string at this time.
   LoadFunction(@fntGetFont, 'fntGetFont');
   Result := fntGetFont(IFont, F);
end;

function TPDF.GetFontCount: Integer;
begin
   LoadFunction(@pdfGetFontCount, 'pdfGetFontCount');
   Result := pdfGetFontCount(FInstance);
end;

function TPDF.GetFontEx(Handle: Cardinal; var F: TPDFFontObj): Boolean;
begin
   // if F.Length1 = 0 and if F.FontFile <> nil then the variable contains
   // the file path to the used font file. This string is always an Ansi string at this time.
   LoadFunction(@pdfGetFontEx, 'pdfGetFontEx');
   Result := pdfGetFontEx(FInstance, Handle, F);
end;

function TPDF.GetFontOrigin(): Integer;
begin
   LoadFunction(@pdfGetFontOrigin, 'pdfGetFontOrigin');
   Result := pdfGetFontOrigin(FInstance);
end;

procedure TPDF.GetFontSearchOrder(Order: PFontBaseType);
begin
   LoadFunction(@pdfGetFontSearchOrder, 'pdfGetFontSearchOrder');
   pdfGetFontSearchOrder(FInstance, Order);
end;

function TPDF.GetFontSelMode(): Integer;
begin
   LoadFunction(@pdfGetFontSelMode, 'pdfGetFontSelMode');
   Result := pdfGetFontSelMode(FInstance);
end;

function TPDF.GetFontWeight(): Integer;
begin
   LoadFunction(@pdfGetFontWeight, 'pdfGetFontWeight');
   Result := pdfGetFontWeight(FInstance);
end;

function TPDF.GetFTextHeightA(Align: TTextAlign; const AText: AnsiString): Double;
begin
   LoadFunction(@pdfGetFTextHeightA, 'pdfGetFTextHeightA');
   Result := pdfGetFTextHeightA(FInstance, Ord(Align), PAnsiChar(AText));
end;

function TPDF.GetFTextHeightW(Align: TTextAlign; const AText: WideString): Double;
begin
   LoadFunction(@pdfGetFTextHeightW, 'pdfGetFTextHeightW');
   Result := pdfGetFTextHeightW(FInstance, Ord(Align), PWideChar(AText));
end;

function TPDF.GetFTextHeightExA(Width: Double; Align: TTextAlign; const AText: AnsiString): Double;
begin
   LoadFunction(@pdfGetFTextHeightExA, 'pdfGetFTextHeightExA');
   Result := pdfGetFTextHeightExA(FInstance, Width, Ord(Align), PAnsiChar(AText));
end;

function TPDF.GetFTextHeightExW(Width: Double; Align: TTextAlign; const AText: WideString): Double;
begin
   LoadFunction(@pdfGetFTextHeightExW, 'pdfGetFTextHeightExW');
   Result := pdfGetFTextHeightExW(FInstance, Width, Ord(Align), PWideChar(AText));
end;

function TPDF.GetGStateFlags: Integer;
begin
   LoadFunction(@pdfGetGStateFlags, 'pdfGetGStateFlags');
   Result := pdfGetGStateFlags(FInstance);
end;

function TPDF.GetIconColor(): Integer;
begin
   LoadFunction(@pdfGetIconColor, 'pdfGetIconColor');
   Result := pdfGetIconColor(FInstance);
end;

function TPDF.GetImageBuffer(var BufSize: Cardinal): PAnsiChar;
begin
   LoadFunction(@pdfGetImageBuffer, 'pdfGetImageBuffer');
   Result := pdfGetImageBuffer(FInstance, BufSize);
end;

function TPDF.GetImageCount(const FileName: AnsiString): Integer;
begin
   LoadFunction(@pdfGetImageCount, 'pdfGetImageCount');
   Result := pdfGetImageCount(FInstance, PAnsiChar(FileName));
end;

function TPDF.GetImageCountEx(const Buffer: Pointer; BufSize: Cardinal): Integer;
begin
   LoadFunction(@pdfGetImageCountEx, 'pdfGetImageCountEx');
   Result := pdfGetImageCountEx(FInstance, Buffer, BufSize);
end;

function TPDF.GetImageHeight(Handle: Cardinal): Integer;
begin
   LoadFunction(@pdfGetImageHeight, 'pdfGetImageHeight');
   Result := pdfGetImageHeight(FInstance, Handle);
end;

function TPDF.GetImageWidth(Handle: Cardinal): Integer;
begin
   LoadFunction(@pdfGetImageWidth, 'pdfGetImageWidth');
   Result := pdfGetImageWidth(FInstance, Handle);
end;

function TPDF.GetImportFlags(): Cardinal;
begin
   LoadFunction(@pdfGetImportFlags, 'pdfGetImportFlags');
   Result := pdfGetImportFlags(FInstance);
end;

function TPDF.GetInBBox(PageNum: Cardinal; Boundary: TPageBoundary; var BBox: TPDFRect): Boolean;
begin
   LoadFunction(@pdfGetInBBox, 'pdfGetInBBox');
   Result := pdfGetInBBox(FInstance, PageNum, Ord(Boundary), BBox);
end;

function TPDF.GetInDocInfo(DInfo: TDocumentInfo; var Value: PWideChar): Integer;
begin
   LoadFunction(@pdfGetInDocInfo, 'pdfGetInDocInfo');
   Result := pdfGetInDocInfo(FInstance, Ord(DInfo), Value);
end;

function TPDF.GetInDocInfoCount: Integer;
begin
   LoadFunction(@pdfGetInDocInfoCount, 'pdfGetInDocInfoCount');
   Result := pdfGetInDocInfoCount(FInstance);
end;

function TPDF.GetInDocInfoEx(Index: Cardinal; var DInfo: TDocumentInfo; var Key, Value: PAnsiChar; var Unicode: LongBool): Boolean;
begin
   LoadFunction(@pdfGetInDocInfoEx, 'pdfGetInDocInfoEx');
   Result := pdfGetInDocInfoEx(FInstance, Index, Integer(DInfo), Key, Value, Unicode);
end;

function TPDF.GetInEncryptionFlags: Integer;
begin
   LoadFunction(@pdfGetInEncryptionFlags, 'pdfGetInEncryptionFlags');
   Result := pdfGetInEncryptionFlags(FInstance);
end;

function TPDF.GetInIsCollection: Boolean;
var retval: Integer;
begin
   LoadFunction(@pdfGetInIsCollection, 'pdfGetInIsCollection');
   retval := pdfGetInIsCollection(FInstance);
   if retval < 0 then
      Result := false
   else
      Result := retval <> 0;
end;

function TPDF.GetInIsEncrypted: Boolean;
var retval: Integer;
begin
   LoadFunction(@pdfGetInIsEncrypted, 'pdfGetInIsEncrypted');
   retval := pdfGetInIsEncrypted(FInstance);
   if retval < 0 then
      Result := false
   else
      Result := retval <> 0;
end;

function TPDF.GetInIsSigned: Boolean;
var retval: Integer;
begin
   LoadFunction(@pdfGetInIsSigned, 'pdfGetInIsSigned');
   retval := pdfGetInIsSigned(FInstance);
   if retval < 0 then
      Result := false
   else
      Result := retval <> 0;
end;

function TPDF.GetInIsTrapped: Boolean;
var retval: Integer;
begin
   LoadFunction(@pdfGetInIsTrapped, 'pdfGetInIsTrapped');
   retval := pdfGetInIsTrapped(FInstance);
   if retval < 0 then
      Result := false
   else
      Result := retval <> 0;
end;

function TPDF.GetInIsXFAForm: Boolean;
var retval: Integer;
begin
   LoadFunction(@pdfGetInIsXFAForm, 'pdfGetInIsXFAForm');
   retval := pdfGetInIsXFAForm(FInstance);
   if retval < 0 then
      Result := false
   else
      Result := retval <> 0;
end;

function TPDF.GetInNamedDest(Index: Cardinal; var Dest: TPDFNamedDest): Boolean;
begin
   LoadFunction(@pdfGetInNamedDest, 'pdfGetInNamedDest');
   Result := pdfGetInNamedDest(FInstance, Index, Dest);
end;

function TPDF.GetInNamedDestCount: Integer;
begin
   LoadFunction(@pdfGetInNamedDestCount, 'pdfGetInNamedDestCount');
   Result := pdfGetInNamedDestCount(FInstance);
end;

function TPDF.GetInOrientation(PageNum: Integer): Integer;
begin
   LoadFunction(@pdfGetInOrientation, 'pdfGetInOrientation');
   Result := pdfGetInOrientation(FInstance, PageNum);
end;

function TPDF.GetInPageCount(): Integer;
begin
   LoadFunction(@pdfGetInPageCount, 'pdfGetInPageCount');
   Result := pdfGetInPageCount(FInstance);
end;

function TPDF.GetInPDFVersion(): Integer;
begin
   LoadFunction(@pdfGetInPDFVersion, 'pdfGetInPDFVersion');
   Result := pdfGetInPDFVersion(FInstance);
end;

function TPDF.GetInPrintSettings(var Settings: TPDFPrintSettings): Boolean;
begin
   LoadFunction(@pdfGetInPrintSettings, 'pdfGetInPrintSettings');
   Result := pdfGetInPrintSettings(FInstance, Settings);
end;

function TPDF.GetIsFixedPitch(): Integer;
begin
   LoadFunction(@pdfGetIsFixedPitch, 'pdfGetIsFixedPitch');
   Result := pdfGetIsFixedPitch(FInstance);
end;

function TPDF.GetItalicAngle(): Double;
begin
   LoadFunction(@pdfGetItalicAngle, 'pdfGetItalicAngle');
   Result := pdfGetItalicAngle(FInstance);
end; 

function TPDF.GetJavaScript(Handle: Cardinal; var Len: Cardinal; var Unicode: LongBool): PAnsiChar;
begin
   LoadFunction(@pdfGetJavaScript, 'pdfGetJavaScript');
   Result := pdfGetJavaScript(FInstance, Handle, Len, Unicode);
end;

function TPDF.GetJavaScriptAction(Handle: Cardinal; var Len: Cardinal; var Unicode: LongBool): PAnsiChar;
begin
   LoadFunction(@pdfGetJavaScriptAction, 'pdfGetJavaScriptAction');
   Result := pdfGetJavaScriptAction(FInstance, Handle, Len, Unicode);
end;

function TPDF.GetJavaScriptAction2(ObjType: TObjType; ObjHandle, ActIndex: Cardinal; var Len: Cardinal; var Unicode: LongBool; var Event: TObjEvent): PAnsiChar;
begin
   LoadFunction(@pdfGetJavaScriptAction2, 'pdfGetJavaScriptAction2');
   Result := pdfGetJavaScriptAction2(FInstance, ObjType, ObjHandle, ActIndex, Len, Unicode, Event);
end;

function TPDF.GetJavaScriptCount(): Integer;
begin
   LoadFunction(@pdfGetJavaScriptCount, 'pdfGetJavaScriptCount');
   Result := pdfGetJavaScriptCount(FInstance);
end;

function TPDF.GetJavaScriptEx(const Name: AnsiString; var Len: Cardinal; var Unicode: LongBool): PAnsiChar;
begin
   LoadFunction(@pdfGetJavaScriptEx, 'pdfGetJavaScriptEx');
   Result := pdfGetJavaScriptEx(FInstance, PAnsiChar(Name), Len, Unicode);
end;

function TPDF.GetJavaScriptName(Handle: Cardinal; var Len: Cardinal; var Unicode: LongBool): PAnsiChar;
begin
   LoadFunction(@pdfGetJavaScriptName, 'pdfGetJavaScriptName');
   Result := pdfGetJavaScriptName(FInstance, Handle, Len, Unicode);
end;

function TPDF.GetJPEGQuality: Integer;
begin
   LoadFunction(@pdfGetJPEGQuality, 'pdfGetJPEGQuality');
   Result := pdfGetJPEGQuality(FInstance);
end;

function TPDF.GetLanguage(): AnsiString;
begin
   LoadFunction(@pdfGetLanguage, 'pdfGetLanguage');
   Result := pdfGetLanguage(FInstance);
end;

function TPDF.GetLastTextPosX(): Double;
begin
   LoadFunction(@pdfGetLastTextPosX, 'pdfGetLastTextPosX');
   Result := pdfGetLastTextPosX(FInstance);
end;

function TPDF.GetLastTextPosY(): Double;
begin
   LoadFunction(@pdfGetLastTextPosY, 'pdfGetLastTextPosY');
   Result := pdfGetLastTextPosY(FInstance);
end;

function TPDF.GetLeading(): Double;
begin
   LoadFunction(@pdfGetLeading, 'pdfGetLeading');
   Result := pdfGetLeading(FInstance);
end;

function TPDF.GetLineCapStyle(): Integer;
begin
   LoadFunction(@pdfGetLineCapStyle, 'pdfGetLineCapStyle');
   Result := pdfGetLineCapStyle(FInstance);
end;

function TPDF.GetLineJoinStyle(): Integer;
begin
   LoadFunction(@pdfGetLineJoinStyle, 'pdfGetLineJoinStyle');
   Result := pdfGetLineJoinStyle(FInstance);
end;

function TPDF.GetLineWidth(): Double;
begin
   LoadFunction(@pdfGetLineWidth, 'pdfGetLineWidth');
   Result := pdfGetLineWidth(FInstance);
end;

function TPDF.GetLinkHighlightMode(): Integer;
begin
   LoadFunction(@pdfGetLinkHighlightMode, 'pdfGetLinkHighlightMode');
   Result := pdfGetLinkHighlightMode(FInstance);
end;

function TPDF.GetLogMetafileSize(const FileName: AnsiString; var R: TRectL): Boolean;
begin
   LoadFunction(@pdfGetLogMetafileSize, 'pdfGetLogMetafileSize');
   Result := pdfGetLogMetafileSize(FInstance, PAnsiChar(FileName), R);
end;

function TPDF.GetLogMetafileSizeEx(const Buffer: Pointer; BufSize: Cardinal; var R: TRectL): Boolean;
begin
   LoadFunction(@pdfGetLogMetafileSizeEx, 'pdfGetLogMetafileSizeEx');
   Result := pdfGetLogMetafileSizeEx(FInstance, Buffer, BufSize, R);
end;

function TPDF.GetMatrix(var M: TCTM): Boolean;
begin
   LoadFunction(@pdfGetMatrix, 'pdfGetMatrix');
   Result := pdfGetMatrix(FInstance, M);
end;

function TPDF.GetMaxFieldLen(TxtField: Cardinal): Integer;
begin
   LoadFunction(@pdfGetMaxFieldLen, 'pdfGetMaxFieldLen');
   Result := pdfGetMaxFieldLen(FInstance, TxtField);
end;

function TPDF.GetMetaConvFlags(): Cardinal;
begin
   LoadFunction(@pdfGetMetaConvFlags, 'pdfGetMetaConvFlags');
   Result := pdfGetMetaConvFlags(FInstance);
end;

function TPDF.GetMissingGlyphs(var Count: Cardinal): PCardinal;
begin
   LoadFunction(@pdfGetMissingGlyphs, 'pdfGetMissingGlyphs');
   Result := pdfGetMissingGlyphs(FInstance, Count);
end;

function TPDF.GetMiterLimit(): Double;
begin
   LoadFunction(@pdfGetMiterLimit, 'pdfGetMiterLimit');
   Result := pdfGetMiterLimit(FInstance);
end;

function TPDF.GetNeedAppearance: Boolean;
begin
   LoadFunction(@pdfGetNeedAppearance, 'pdfGetNeedAppearance');
   Result := pdfGetNeedAppearance(FInstance);
end;

function TPDF.GetObjActionCount(ObjType: TObjType; ObjHandle: Cardinal): Integer;
begin
   LoadFunction(@pdfGetObjActionCount, 'pdfGetObjActionCount');
   Result := pdfGetObjActionCount(FInstance, Ord(ObjType), ObjHandle);
end;

function TPDF.GetOpacity(): Double;
begin
   LoadFunction(@pdfGetOpacity, 'pdfGetOpacity');
   Result := pdfGetOpacity(FInstance);
end;

function TPDF.GetOrientation(): Integer;
begin
   LoadFunction(@pdfGetOrientation, 'pdfGetOrientation');
   Result := pdfGetOrientation(FInstance);
end;

function TPDF.GetPageAnnot(Index: Cardinal; var Annot: TPDFAnnotation): Boolean;
begin
   LoadFunction(@pdfGetPageAnnot, 'pdfGetPageAnnot');
   Result := pdfGetPageAnnot(FInstance, Index, Annot);
end;

function TPDF.GetPageAnnotCount: Integer;
begin
   LoadFunction(@pdfGetPageAnnotCount, 'pdfGetPageAnnotCount');
   Result := pdfGetPageAnnotCount(FInstance);
end;

function TPDF.GetPageAnnotEx(Index: Cardinal; var Annot: TPDFAnnotationEx): Boolean;
begin
   LoadFunction(@pdfGetPageAnnotEx, 'pdfGetPageAnnotEx');
   Result := pdfGetPageAnnotEx(FInstance, Index, Annot);
end;

function TPDF.GetPageCoords(): TPageCoord;
begin
   LoadFunction(@pdfGetPageCoords, 'pdfGetPageCoords');
   Result := TPageCoord(pdfGetPageCoords(FInstance));
end;

function TPDF.GetPageCount(): Integer;
begin
   LoadFunction(@pdfGetPageCount, 'pdfGetPageCount');
   Result := pdfGetPageCount(FInstance);
end;

function TPDF.GetPageField(Index: Cardinal; var Field: TPDFField): Boolean;
begin
   {
      Note: FieldName can be an Unicode string!
      To determine whether FieldName contains an ANSI or Unicode string
      proceed as follows:
      if f.FielNameLen > 0 and StrLen(f.FieldName) <> f.FieldNameLen then begin
         // f.FieldName contains a null-terminated Unicode string
         // f.FieldNameLen is the length in Unicode characters, but we have an ANSI string!
         // So, we must multiply the length with 2 to convert the entire string!
         nameW := pdf.ToUnicode(f.FieldName, f.FieldNameLen * 2);         
      end;
   }
   LoadFunction(@pdfGetPageField, 'pdfGetPageField');
   Result := pdfGetPageField(FInstance, Index, Field);
end;

function TPDF.GetPageFieldCount(): Integer;
begin
   LoadFunction(@pdfGetPageFieldCount, 'pdfGetPageFieldCount');
   Result := pdfGetPageFieldCount(FInstance);
end;

function TPDF.GetPageFieldEx(Index: Cardinal; var Field: TPDFFieldEx): Boolean;
begin
   LoadFunction(@pdfGetPageFieldEx, 'pdfGetPageFieldEx');
   Field.StructSize := sizeof(Field);
   Result := pdfGetPageFieldEx(FInstance, Index, Field);
end;

function TPDF.GetPageHeight(): Double;
begin
   LoadFunction(@pdfGetPageHeight, 'pdfGetPageHeight');
   Result := pdfGetPageHeight(FInstance);
end;

function TPDF.GetPageLabel(Index: Cardinal; var Lbl: TPDFPageLabel): Boolean;
begin
   LoadFunction(@pdfGetPageLabel, 'pdfGetPageLabel');
   Result := pdfGetPageLabel(FInstance, Index, Lbl);
end;

function TPDF.GetPageLabelCount: Integer;
begin
   LoadFunction(@pdfGetPageLabelCount, 'pdfGetPageLabelCount');
   Result := pdfGetPageLabelCount(FInstance);
end;

function TPDF.GetPageLayout(): Integer;
begin
   LoadFunction(@pdfGetPageLayout, 'pdfGetPageLayout');
   Result := pdfGetPageLayout(FInstance);
end;

function TPDF.GetPageMode(): Integer;
begin
   LoadFunction(@pdfGetPageMode, 'pdfGetPageMode');
   Result := pdfGetPageMode(FInstance);
end;

function TPDF.GetPageNum: Integer;
begin
   LoadFunction(@pdfGetPageNum, 'pdfGetPageNum');
   Result := pdfGetPageNum(FInstance);
end;

function TPDF.GetPageText(var Stack: TPDFStack): Boolean;
begin
   LoadFunction(@pdfGetPageText, 'pdfGetPageText');
   Result := pdfGetPageText(FInstance, Stack);
end;

function TPDF.GetPageWidth(): Double;
begin
   LoadFunction(@pdfGetPageWidth, 'pdfGetPageWidth');
   Result := pdfGetPageWidth(FInstance);
end;

function TPDF.GetPDFVersion(): Integer;
begin
   LoadFunction(@pdfGetPDFVersion, 'pdfGetPDFVersion');
   Result := pdfGetPDFVersion(FInstance);
end;

function TPDF.GetPrintSettings(var Settings: TPDFPrintSettings): Boolean;
begin
   LoadFunction(@pdfGetPrintSettings, 'pdfGetPrintSettings');
   Result := pdfGetPrintSettings(FInstance, Settings);
end;

function TPDF.GetResolution(): Integer;
begin
   LoadFunction(@pdfGetResolution, 'pdfGetResolution');
   Result := pdfGetResolution(FInstance);
end;

function TPDF.GetSaveNewImageFormat(): Integer;
begin
   LoadFunction(@pdfGetSaveNewImageFormat, 'pdfGetSaveNewImageFormat');
   Result := pdfGetSaveNewImageFormat(FInstance);
end;

function TPDF.GetSeparationInfo(var Colorant: AnsiString; var CS: TExtColorSpace): Boolean;
var clt: PAnsiChar;
begin
   LoadFunction(@pdfGetSeparationInfo, 'pdfGetSeparationInfo');
   clt := nil;
   Result := pdfGetSeparationInfo(FInstance, clt, CS);
   Colorant := clt;
end;

function TPDF.GetSpaceWidth(const IFont: Pointer; FontSize: Double): Double;
begin
   LoadFunction(@fntGetSpaceWidth, 'fntGetSpaceWidth');
   Result := fntGetSpaceWidth(IFont, FontSize);
end;

function TPDF.GetStrokeColor(): Cardinal;
begin
   LoadFunction(@pdfGetStrokeColor, 'pdfGetStrokeColor');
   Result := pdfGetStrokeColor(FInstance);
end;

function TPDF.GetTabLen: Integer;
begin
   LoadFunction(@pdfGetTabLen, 'pdfGetTabLen');
   Result := pdfGetTabLen(FInstance);
end;

function TPDF.GetTemplCount: Integer;
begin
   LoadFunction(@pdfGetTemplCount, 'pdfGetTemplCount');
   Result := pdfGetTemplCount(FInstance);
end;

function TPDF.GetTemplHandle: Integer;
begin
   LoadFunction(@pdfGetTemplHandle, 'pdfGetTemplHandle');
   Result := pdfGetTemplHandle(FInstance);
end;

function TPDF.GetTemplHeight(TmplHandle: Integer): Double;
begin
   LoadFunction(@pdfGetTemplHeight, 'pdfGetTemplHeight');
   Result := pdfGetTemplHeight(FInstance, TmplHandle);
end;

function TPDF.GetTemplWidth(TmplHandle: Integer): Double;
begin
   LoadFunction(@pdfGetTemplWidth, 'pdfGetTemplWidth');
   Result := pdfGetTemplWidth(FInstance, TmplHandle);
end;

function TPDF.GetTextDrawMode(): Integer;
begin
   LoadFunction(@pdfGetTextDrawMode, 'pdfGetTextDrawMode');
   Result := pdfGetTextDrawMode(FInstance);
end;

function TPDF.GetTextFieldValue(AField: Cardinal; var Value: PAnsiChar; var ValIsUnicode: LongBool; var DefValue: PAnsiChar; var DefValIsUnicode: LongBool): Boolean;
begin
   LoadFunction(@pdfGetTextFieldValue, 'pdfGetTextFieldValue');
   Result := pdfGetTextFieldValue(FInstance, AField, Value, ValIsUnicode, DefValue, DefValIsUnicode);
end;

function TPDF.GetTextRect(var PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfGetTextRect, 'pdfGetTextRect');
   Result := pdfGetTextRect(FInstance, PosX, PosY, Width, Height);
end;

function TPDF.GetTextRise(): Double;
begin
   LoadFunction(@pdfGetTextRise, 'pdfGetTextRise');
   Result := pdfGetTextRise(FInstance);
end;

function TPDF.GetTextScaling(): Double;
begin
   LoadFunction(@pdfGetTextScaling, 'pdfGetTextScaling');
   Result := pdfGetTextScaling(FInstance);
end;

function TPDF.GetTextWidth(const IFont: Pointer; const Text: PAnsiChar; Len: Cardinal; CharSpacing, WordSpacing, TextScale: Single): Double;
begin
   LoadFunction(@fntGetTextWidth, 'fntGetTextWidth');
   Result := fntGetTextWidth(IFont, Text, Len, CharSpacing, WordSpacing, TextScale);
end;

function TPDF.GetTextWidthA(const AText: AnsiString): Double;
begin
   LoadFunction(@pdfGetTextWidthA, 'pdfGetTextWidthA');
   Result := pdfGetTextWidthA(FInstance, PAnsiChar(AText));
end;

function TPDF.GetTextWidthW(const AText: WideString): Double;
begin
   LoadFunction(@pdfGetTextWidthW, 'pdfGetTextWidthW');
   Result := pdfGetTextWidthW(FInstance, PWideChar(AText));
end;

function TPDF.GetTextWidthExA(const AText: AnsiString; Len: Cardinal): Double;
begin
   LoadFunction(@pdfGetTextWidthExA, 'pdfGetTextWidthExA');
   Result := pdfGetTextWidthExA(FInstance, PAnsiChar(AText), Len);
end;

function TPDF.GetTextWidthExW(const AText: WideString; Len: Cardinal): Double;
begin
   LoadFunction(@pdfGetTextWidthExW, 'pdfGetTextWidthExW');
   Result := pdfGetTextWidthExW(FInstance, PWideChar(AText), Len);
end;

function TPDF.GetTransparentColor(): Cardinal;
begin
   LoadFunction(@pdfGetTransparentColor, 'pdfGetTransparentColor');
   Result := pdfGetTransparentColor(FInstance);
end;

function TPDF.GetTrapped: Boolean;
begin
   LoadFunction(@pdfGetTrapped, 'pdfGetTrapped');
   Result := pdfGetTrapped(FInstance);
end;

function TPDF.GetUseTransparency(): Boolean;
begin
   LoadFunction(@pdfGetUseTransparency, 'pdfGetUseTransparency');
   Result := pdfGetUseTransparency(FInstance);
end;

function TPDF.GetUseVisibleCoords(): Boolean;
begin
   LoadFunction(@pdfGetUseVisibleCoords, 'pdfGetUseVisibleCoords');
   Result := pdfGetUseVisibleCoords(FInstance);
end;

function TPDF.GetUseExactPwd(): Boolean;
begin
   LoadFunction(@pdfGetUseExactPwd, 'pdfGetUseExactPwd');
   Result := pdfGetUseExactPwd(FInstance);
end;

function TPDF.GetUseGlobalImpFiles(): Boolean;
begin
   LoadFunction(@pdfGetUseGlobalImpFiles, 'pdfGetUseGlobalImpFiles');
   Result := pdfGetUseGlobalImpFiles(FInstance);
end;

function TPDF.GetUserRights(): Integer;
begin
   LoadFunction(@pdfGetUserRights, 'pdfGetUserRights');
   Result := pdfGetUserRights(FInstance);
end;

function TPDF.GetUseStdFonts(): Boolean;
begin
   LoadFunction(@pdfGetUseStdFonts, 'pdfGetUseStdFonts');
   Result := pdfGetUseStdFonts(FInstance);
end;

function TPDF.GetUseSystemFonts(): Boolean;
begin
   LoadFunction(@pdfGetUseSystemFonts, 'pdfGetUseSystemFonts');
   Result := pdfGetUseSystemFonts(FInstance);
end;

function TPDF.GetViewerPrefrences(var Preference, AddVal: Integer): Boolean;
begin
   LoadFunction(@pdfGetViewerPrefrences, 'pdfGetViewerPrefrences');
   Result := pdfGetViewerPrefrences(FInstance, Preference, AddVal);
end;

function TPDF.GetWMFDefExtent(var Width, Height: Cardinal): Boolean;
begin
   LoadFunction(@pdfGetWMFDefExtent, 'pdfGetWMFDefExtent');
   Result := pdfGetWMFDefExtent(FInstance, Width, Height);
end;

function TPDF.GetWMFPixelPerInch(): Integer;
begin
   LoadFunction(@pdfGetWMFPixelPerInch, 'pdfGetWMFPixelPerInch');
   Result := pdfGetWMFPixelPerInch(FInstance);
end;

function TPDF.GetWordSpacing(): Double;
begin
   LoadFunction(@pdfGetWordSpacing, 'pdfGetWordSpacing');
   Result := pdfGetWordSpacing(FInstance);
end;

function TPDF.HaveOpenDoc: Boolean;
begin
   LoadFunction(@pdfHaveOpenDoc, 'pdfHaveOpenDoc');
   Result := pdfHaveOpenDoc(FInstance);
end;

function TPDF.HighlightAnnotA(SubType: TAnnotType; PosX, PosY, Width, Height: Double; Color: Cardinal; const Author, Subject, Comment: AnsiString): Integer;
begin
   LoadFunction(@pdfHighlightAnnotA, 'pdfHighlightAnnotA');
   Result := pdfHighlightAnnotA(FInstance, SubType, PosX, PosY, Width, Height, Color, PAnsiChar(Author), PAnsiChar(Subject), PAnsiChar(Comment));
end;

function TPDF.HighlightAnnotW(SubType: TAnnotType; PosX, PosY, Width, Height: Double; Color: Cardinal; const Author, Subject, Comment: WideString): Integer;
begin
   LoadFunction(@pdfHighlightAnnotW, 'pdfHighlightAnnotW');
   Result := pdfHighlightAnnotW(FInstance, SubType, PosX, PosY, Width, Height, Color, PWideChar(Author), PWideChar(Subject), PWideChar(Comment));
end;

function TPDF.ImportBookmarks: Integer;
begin
   LoadFunction(@pdfImportBookmarks, 'pdfImportBookmarks');
   Result := pdfImportBookmarks(FInstance);
end;

function TPDF.ImportCatalogObjects: Boolean;
begin
   LoadFunction(@pdfImportCatalogObjects, 'pdfImportCatalogObjects');
   Result := pdfImportCatalogObjects(FInstance);
end;

function TPDF.ImportDocInfo: Boolean;
begin
   LoadFunction(@pdfImportDocInfo, 'pdfImportDocInfo');
   Result := pdfImportDocInfo(FInstance);
end;

function TPDF.ImportPage(PageNum: Cardinal): Integer;
begin
   LoadFunction(@pdfImportPage, 'pdfImportPage');
   Result := pdfImportPage(FInstance, PageNum);
end;

function TPDF.ImportPageEx(PageNum: Cardinal; ScaleFactX, ScaleFactY: Double): Integer;
begin
   LoadFunction(@pdfImportPageEx, 'pdfImportPageEx');
   Result := pdfImportPageEx(FInstance, PageNum, ScaleFactX, ScaleFactY);
end;

function TPDF.ImportPDFFile(DestPage: Cardinal; ScaleFactX, ScaleFactY: Double): Integer;
begin
   LoadFunction(@pdfImportPDFFile, 'pdfImportPDFFile');
   Result := pdfImportPDFFile(FInstance, DestPage, ScaleFactX, ScaleFactY);
end;

function TPDF.InitExtGState(var GS: TPDFExtGState): Boolean;
begin
   LoadFunction(@pdfInitExtGState, 'pdfInitExtGState');
   Result := pdfInitExtGState(GS);
end;

function TPDF.InitStack(var Stack: TPDFStack): Boolean;
begin
   LoadFunction(@pdfInitStack, 'pdfInitStack');
   Result := pdfInitStack(FInstance, Stack);
end;

function TPDF.InsertBMPFromBuffer(PosX, PosY, ScaleWidth, ScaleHeight: Double; const Buffer: Pointer): Integer;
begin
   LoadFunction(@pdfInsertBMPFromBuffer, 'pdfInsertBMPFromBuffer');
   Result := pdfInsertBMPFromBuffer(FInstance, PosX, PosY, ScaleWidth, ScaleHeight, Buffer);
end;

function TPDF.InsertBMPFromHandle(PosX, PosY, ScaleWidth, ScaleHeight: Double; Handle: HBITMAP): Integer;
begin
   LoadFunction(@pdfInsertBMPFromHandle, 'pdfInsertBMPFromHandle');
   Result := pdfInsertBMPFromHandle(FInstance, PosX, PosY, ScaleWidth, ScaleHeight, Pointer(Handle));
end;

function TPDF.InsertImage(PosX, PosY, ScaleWidth, ScaleHeight: Double; const AFile: AnsiString): Integer;
begin
   LoadFunction(@pdfInsertImage, 'pdfInsertImage');
   Result := pdfInsertImage(FInstance, PosX, PosY, ScaleWidth, ScaleHeight, PAnsiChar(AFile));
end;

function TPDF.InsertImageEx(PosX, PosY, ScaleWidth, ScaleHeight: Double; const AFile: AnsiString; Index: Cardinal): Integer;
begin
   LoadFunction(@pdfInsertImageEx, 'pdfInsertImageEx');
   Result := pdfInsertImageEx(FInstance, PosX, PosY, ScaleWidth, ScaleHeight, PAnsiChar(AFile), Index);
end;

function TPDF.InsertImageFromBuffer(PosX, PosY, ScaleWidth, ScaleHeight: Double; const Buffer: Pointer; BufSize, Index: Cardinal): Integer;
begin
   LoadFunction(@pdfInsertImageFromBuffer, 'pdfInsertImageFromBuffer');
   Result := pdfInsertImageFromBuffer(FInstance, PosX, PosY, ScaleWidth, ScaleHeight, Buffer, BufSize, Index);
end;

function TPDF.InsertRawImage(const Buffer: Pointer; BitsPerPixel, ColorCount, ImgWidth, ImgHeight: Cardinal; PosX, PosY, ScaleWidth, ScaleHeight: Double): Integer;
begin
   LoadFunction(@pdfInsertRawImage, 'pdfInsertRawImage');
   Result := pdfInsertRawImage(FInstance, Buffer, BitsPerPixel, ColorCount, ImgWidth, ImgHeight, PosX, PosY, ScaleWidth, ScaleHeight);
end;

function TPDF.InsertMetafile(const FileName: AnsiString; PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfInsertMetafile, 'pdfInsertMetafile');
   Result := pdfInsertMetafile(FInstance, PAnsiChar(FileName), PosX, PosY, Width, Height);
end;

function TPDF.InsertMetafileEx(const Buffer: Pointer; BufSize: Cardinal; PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfInsertMetafileEx, 'pdfInsertMetafileEx');
   Result := pdfInsertMetafileEx(FInstance, Buffer, BufSize, PosX, PosY, Width, Height);
end;

function TPDF.InsertMetafileExt(const FileName: AnsiString; var View: TRectL; PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfInsertMetafileExt, 'pdfInsertMetafileExt');
   Result := pdfInsertMetafileExt(FInstance, PAnsiChar(FileName), View, PosX, PosY, Width, Height);
end;

function TPDF.InsertMetafileExtEx(const Buffer: Pointer; BufSize: Cardinal; var View: TRectL; PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfInsertMetafileExtEx, 'pdfInsertMetafileExtEx');
   Result := pdfInsertMetafileExtEx(FInstance, Buffer, BufSize, View, PosX, PosY, Width, Height);
end;

function TPDF.InsertMetafileFromHandle(Handle: HENHMETAFILE; PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfInsertMetafileFromHandle, 'pdfInsertMetafileFromHandle');
   Result := pdfInsertMetafileFromHandle(FInstance, Pointer(Handle), PosX, PosY, Width, Height);
end;

function TPDF.InsertMetafileFromHandleEx(Handle: HENHMETAFILE; var View: TRectL; PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfInsertMetafileFromHandleEx, 'pdfInsertMetafileFromHandleEx');
   Result := pdfInsertMetafileFromHandleEx(FInstance, Pointer(Handle), View, PosX, PosY, Width, Height);
end;

function TPDF.IsBidiText(const AText: WideString): Integer;
begin
   LoadFunction(@pdfIsBidiText, 'pdfIsBidiText');
   Result := pdfIsBidiText(FInstance, PWideChar(AText));
end;

{
   Return values:
   0: // Page is black & white
   1: // Page use colors
   Negative values indicate an error. The page must be opened beforehand with EditPage().
}

function TPDF.IsColorPage(GrayIsColor: Boolean): Integer;
begin
   LoadFunction(@pdfIsColorPage, 'pdfIsColorPage');
   Result := pdfIsColorPage(FInstance, GrayIsColor);
end;

{
   Return values:
   0: // Page is not empty
   1: // Page is empty
   Negative values indicate an error. The page must be opened beforehand with EditPage().
}

function TPDF.IsEmptyPage(): Integer;
begin
   LoadFunction(@pdfIsEmptyPage, 'pdfIsEmptyPage');
   Result := pdfIsEmptyPage(FInstance);
end;

function TPDF.IsWrongPwd(ErrCode: Integer): Boolean;
begin
   Result := (ErrCode = ENEED_PWD) or (ErrCode = EWRONG_OPEN_PWD) or (ErrCode = EWRONG_OWNER_PWD) or (ErrCode = EWRONG_PWD);
end;

function TPDF.LineTo(PosX, PosY: Double): Boolean;
begin
   LoadFunction(@pdfLineTo, 'pdfLineTo');
   Result := pdfLineTo(FInstance, PosX, PosY);
end;

function TPDF.LoadCMap(const CMapName: AnsiString; Embed: Boolean): Integer;
begin
   LoadFunction(@pdfLoadCMap, 'pdfLoadCMap');
   Result := pdfLoadCMap(FInstance, PAnsiChar(CMapName), Embed);
end;

function TPDF.LoadFont(const Buffer: Pointer; BufSize: Cardinal; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage): Integer;
begin
   LoadFunction(@pdfLoadFont, 'pdfLoadFont');
   Result := pdfLoadFont(FInstance, Buffer, BufSize, Style, Size, Embed, Integer(CP));
end;

function TPDF.LoadFDFDataA(const FileName, Password: AnsiString; Flags: Cardinal): Boolean;
begin
   LoadFunction(@pdfLoadFDFDataA, 'pdfLoadFDFDataA');
   Result := pdfLoadFDFDataA(FInstance, PAnsiChar(FileName), PAnsiChar(Password), Flags);
end;

function TPDF.LoadFDFDataW(const FileName: WideString; const Password: AnsiString; Flags: Cardinal): Boolean;
begin
   LoadFunction(@pdfLoadFDFDataW, 'pdfLoadFDFDataW');
   Result := pdfLoadFDFDataW(FInstance, PWideChar(FileName), PAnsiChar(Password), Flags);
end;

function TPDF.LoadFDFDataEx(const Buffer: Pointer; BufSize: Cardinal; const Password: AnsiString; Flags: Cardinal): Boolean;
begin
   LoadFunction(@pdfLoadFDFDataEx, 'pdfLoadFDFDataEx');
   Result := pdfLoadFDFDataEx(FInstance, Buffer, BufSize, PAnsiChar(Password), Flags);
end;

function TPDF.MovePage(Source, Dest: Cardinal): Boolean;
begin
   LoadFunction(@pdfMovePage, 'pdfMovePage');
   Result := pdfMovePage(FInstance, Source, Dest);
end;

function TPDF.MoveTo(PosX, PosY: Double): Boolean;
begin
   LoadFunction(@pdfMoveTo, 'pdfMoveTo');
   Result := pdfMoveTo(FInstance, PosX, PosY);
end;

procedure TPDF.MultiplyMatrix(var M1, M2, NewMatrix: TCTM);
begin
   NewMatrix.a := M2.a * M1.a + M2.b * M1.c;
   NewMatrix.b := M2.a * M1.b + M2.b * M1.d;
   NewMatrix.c := M2.c * M1.a + M2.d * M1.c;
   NewMatrix.d := M2.c * M1.b + M2.d * M1.d;
   NewMatrix.x := M2.x * M1.a + M2.y * M1.c + M1.x;
   NewMatrix.y := M2.x * M1.b + M2.y * M1.d + M1.y;
end;

procedure TPDF.NewPDF();
var pdfNewPDF: TpdfNewPDF;
begin
   pdfNewPDF := GetProcAddress(FDLL_Handle, PAnsiChar('pdfNewPDF'));
   if @pdfNewPDF = nil then raise Exception.Create('Error loading function: pdfNewPDF');
   FInstance := pdfNewPDF;
   if (FInstance = nil) then Exception.Create('Out of memory!');
end;

function TPDF.OpenImportBuffer(const Buffer: Pointer; BufSize: Cardinal; PwdType: TPwdType; const Password: AnsiString): Integer;
begin
   LoadFunction(@pdfOpenImportBuffer, 'pdfOpenImportBuffer');
   Result := pdfOpenImportBuffer(FInstance, Buffer, BufSize, Ord(PwdType), PAnsiChar(Password));
end;

function TPDF.OpenImportFileA(const FileName: AnsiString; PwdType: TPwdType; const Password: AnsiString): Integer;
begin
   LoadFunction(@pdfOpenImportFileA, 'pdfOpenImportFileA');
   Result := pdfOpenImportFileA(FInstance, PAnsiChar(FileName), Ord(PwdType), PAnsiChar(Password));
end;

function TPDF.OpenImportFileW(const FileName: WideString; PwdType: TPwdType; const Password: AnsiString): Integer;
begin
   LoadFunction(@pdfOpenImportFileW, 'pdfOpenImportFileW');
   Result := pdfOpenImportFileW(FInstance, PWideChar(FileName), Ord(PwdType), PAnsiChar(Password));
end;

function TPDF.OpenOutputFileA(const OutPDF: AnsiString): Boolean;
begin
   LoadFunction(@pdfOpenOutputFileA, 'pdfOpenOutputFileA');
   Result := pdfOpenOutputFileA(FInstance, PAnsiChar(OutPDF));
end;

function TPDF.OpenOutputFileW(const OutPDF: WideString): Boolean;
begin
   LoadFunction(@pdfOpenOutputFileW, 'pdfOpenOutputFileW');
   Result := pdfOpenOutputFileW(FInstance, PWideChar(OutPDF));
end;

function TPDF.OpenTagA(Tag: TPDFBaseTag; const Lang, AltText, Expansion: AnsiString): Boolean;
begin
   LoadFunction(@pdfOpenTagA, 'pdfOpenTagA');
   Result := pdfOpenTagA(FInstance, Integer(Tag), PAnsiChar(Lang), PAnsiChar(AltText), PAnsiChar(Expansion));
end;

function TPDF.OpenTagW(Tag: TPDFBaseTag; const Lang: AnsiString; const AltText, Expansion: WideString): Boolean;
begin
   LoadFunction(@pdfOpenTagW, 'pdfOpenTagW');
   Result := pdfOpenTagW(FInstance, Integer(Tag), PAnsiChar(Lang), PWideChar(AltText), PWideChar(Expansion));
end;

function TPDF.PageLink(PosX, PosY, Width, Height: Double; DestPage: Cardinal): Integer;
begin
   LoadFunction(@pdfPageLink, 'pdfPageLink');
   Result := pdfPageLink(FInstance, PosX, PosY, Width, Height, DestPage);
end;

function TPDF.PageLink2(PosX, PosY, Width, Height: Double; NamedDest: Cardinal): Integer;
begin
   LoadFunction(@pdfPageLink2, 'pdfPageLink2');
   Result := pdfPageLink2(FInstance, PosX, PosY, Width, Height, NamedDest);
end;

function TPDF.PageLinkEx(PosX, PosY, Width, Height: Double; DestType: TDestType; DestPage: Cardinal; a, b, c, d: Double): Integer;
begin
   LoadFunction(@pdfPageLinkEx, 'pdfPageLinkEx');
   Result := pdfPageLinkEx(FInstance, PosX, PosY, Width, Height, Integer(DestType), DestPage, a, b, c, d);
end;

function TPDF.ParseContent(const UserData: Pointer; var Stack: TPDFParseInterface; Flags: TParseFlags): Boolean;
begin
   LoadFunction(@pdfParseContent, 'pdfParseContent');
   Result := pdfParseContent(FInstance, UserData, Stack, Flags);
end;

function TPDF.PlaceImage(ImgHandle: Integer; PosX, PosY, ScaleWidth, ScaleHeight: Double): Boolean;
begin
   LoadFunction(@pdfPlaceImage, 'pdfPlaceImage');
   Result := pdfPlaceImage(FInstance, ImgHandle, PosX, PosY, ScaleWidth, ScaleHeight);
end;

function TPDF.PlaceSigFieldValidateIcon(SigField: Cardinal; PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfPlaceSigFieldValidateIcon, 'pdfPlaceSigFieldValidateIcon');
   Result := pdfPlaceSigFieldValidateIcon(FInstance, SigField, PosX, PosY, Width, Height);
end;

function TPDF.PlaceTemplate(TmplHandle: Integer; PosX, PosY, ScaleWidth, ScaleHeight: Double): Boolean;
begin
   LoadFunction(@pdfPlaceTemplate, 'pdfPlaceTemplate');
   Result := pdfPlaceTemplate(FInstance, TmplHandle, PosX, PosY, ScaleWidth, ScaleHeight);
end;

function TPDF.PlaceTemplateEx(TmplHandle: Integer; PosX, PosY, ScaleWidth, ScaleHeight: Double): Boolean;
begin
   LoadFunction(@pdfPlaceTemplateEx, 'pdfPlaceTemplateEx');
   Result := pdfPlaceTemplateEx(FInstance, TmplHandle, PosX, PosY, ScaleWidth, ScaleHeight);
end;

function TPDF.PlaceTemplByMatrix(TmplHandle: Integer): Boolean;
begin
   LoadFunction(@pdfPlaceTemplByMatrix, 'pdfPlaceTemplByMatrix');
   Result := pdfPlaceTemplByMatrix(FInstance, TmplHandle);
end;

function TPDF.ReadImageFormat(const FileName: AnsiString; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): Boolean;
begin
   LoadFunction(@pdfReadImageFormat, 'pdfReadImageFormat');
   Result := pdfReadImageFormat(FInstance, PAnsiChar(FileName), Width, Height, BitsPerPixel, UseZip);
end;

function TPDF.ReadImageFormat2(const FileName: AnsiString; Index: Cardinal; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): Boolean;
begin
   LoadFunction(@pdfReadImageFormat2, 'pdfReadImageFormat2');
   Result := pdfReadImageFormat2(FInstance, PAnsiChar(FileName), Index, Width, Height, BitsPerPixel, UseZip);
end;

function TPDF.ReadImageFormatEx(hBitmap: HBITMAP; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): Boolean;
begin
   LoadFunction(@pdfReadImageFormatEx, 'pdfReadImageFormatEx');
   Result := pdfReadImageFormatEx(FInstance, Pointer(hBitmap), Width, Height, BitsPerPixel, UseZip);
end;

function TPDF.ReadImageFormatFromBuffer(const Buffer: Pointer; BufSize, Index: Cardinal; var Width, Height: Cardinal; var BitsPerPixel, UseZip: Integer): Boolean;
begin
   LoadFunction(@pdfReadImageFormatFromBuffer, 'pdfReadImageFormatFromBuffer');
   Result := pdfReadImageFormatFromBuffer(FInstance, Buffer, BufSize, Index, Width, Height, BitsPerPixel, UseZip);
end;

function TPDF.ReadImageResolution(const FileName: AnsiString; Index: Cardinal; var ResX, ResY: Cardinal): Boolean;
begin
   LoadFunction(@pdfReadImageResolution, 'pdfReadImageResolution');
   Result := pdfReadImageResolution(FInstance, PAnsiChar(FileName), Index, ResX, ResY);
end;

function TPDF.ReadImageResolutionEx(const Buffer: Pointer; BufSize, Index: Cardinal; var ResX, ResY: Cardinal): Boolean;
begin
   LoadFunction(@pdfReadImageResolutionEx, 'pdfReadImageResolutionEx');
   Result := pdfReadImageResolutionEx(FInstance, Buffer, BufSize, Index, ResX, ResY);
end;

function TPDF.Rectangle(PosX, PosY, Width, Height: Double; FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfRectangle, 'pdfRectangle');
   Result := pdfRectangle(FInstance, PosX, PosY, Width, Height, Ord(FillMode));
end;

function TPDF.ReEncryptPDFA(const FileName: AnsiString; PwdType: TPwdType; const InPwd, NewOpenPwd, NewOwnerPwd: AnsiString; NewKeyLen: TKeyLen; Restrict: TRestrictions): Integer;
begin
   LoadFunction(@pdfReEncryptPDFA, 'pdfReEncryptPDFA');
   Result := pdfReEncryptPDFA(FInstance, PAnsiChar(FileName), Ord(PwdType), PAnsiChar(InPwd), PAnsiChar(NewOpenPwd), PAnsiChar(NewOwnerPwd), Ord(NewKeyLen), Ord(Restrict))
end;

function TPDF.ReEncryptPDFW(const FileName: WideString; PwdType: TPwdType; const InPwd, NewOpenPwd, NewOwnerPwd: AnsiString; NewKeyLen: TKeyLen; Restrict: TRestrictions): Integer;
begin
   LoadFunction(@pdfReEncryptPDFW, 'pdfReEncryptPDFW');
   Result := pdfReEncryptPDFW(FInstance, PWideChar(FileName), Ord(PwdType), PAnsiChar(InPwd), PAnsiChar(NewOpenPwd), PAnsiChar(NewOwnerPwd), Ord(NewKeyLen), Ord(Restrict));
end;

function TPDF.RenameSpotColor(const Colorant, NewName: AnsiString): Integer;
begin
   LoadFunction(@pdfRenameSpotColor, 'pdfRenameSpotColor');
   Result := pdfRenameSpotColor(FInstance, PAnsiChar(Colorant), PAnsiChar(NewName));
end;

function TPDF.ReplaceFontA(const PDFFont: Pointer; const Name: AnsiString; Style: TFStyle; NameIsFamilyName: Boolean): Integer;
begin
   LoadFunction(@pdfReplaceFontA, 'pdfReplaceFontA');
   Result := pdfReplaceFontA(FInstance, PDFFont, PAnsiChar(Name), Integer(Style), NameIsFamilyName);
end;

function TPDF.ReplaceFontW(const PDFFont: Pointer; const Name: WideString; Style: TFStyle; NameIsFamilyName: Boolean): Integer;
begin
   LoadFunction(@pdfReplaceFontW, 'pdfReplaceFontW');
   Result := pdfReplaceFontW(FInstance, PDFFont, PWideChar(Name), Integer(Style), NameIsFamilyName);
end;

function TPDF.ReplaceFontExA(const PDFFont: Pointer; const FontFile: AnsiString; Embed: Boolean): Integer;
begin
   LoadFunction(@pdfReplaceFontExA, 'pdfReplaceFontExA');
   Result := pdfReplaceFontExA(FInstance, PDFFont, PAnsiChar(FontFile), Embed);
end;

function TPDF.ReplaceFontExW(const PDFFont: Pointer; const FontFile: WideString; Embed: Boolean): Integer;
begin
   LoadFunction(@pdfReplaceFontExW, 'pdfReplaceFontExW');
   Result := pdfReplaceFontExW(FInstance, PDFFont, PWideChar(FontFile), Embed);
end;

function TPDF.ReplaceICCProfileA(ColorSpace: Cardinal; const ICCFile: AnsiString): Integer;
begin
   LoadFunction(@pdfReplaceICCProfileA, 'pdfReplaceICCProfileA');
   Result := pdfReplaceICCProfileA(FInstance, ColorSpace, PAnsiChar(ICCFile));
end;

function TPDF.ReplaceICCProfileW(ColorSpace: Cardinal; const ICCFile: WideString): Integer;
begin
   LoadFunction(@pdfReplaceICCProfileW, 'pdfReplaceICCProfileW');
   Result := pdfReplaceICCProfileW(FInstance, ColorSpace, PWideChar(ICCFile));
end;

function TPDF.ReplacePageText(const NewText: AnsiString; var Stack: TPDFStack): Boolean;
begin
   LoadFunction(@pdfReplacePageTextA, 'pdfReplacePageTextA');
   Result := pdfReplacePageTextA(FInstance, PAnsiChar(NewText), Stack);
end;

function TPDF.ReplacePageTextEx(const NewText: AnsiString; var Stack: TPDFStack): Boolean;
begin
   LoadFunction(@pdfReplacePageTextExA, 'pdfReplacePageTextExA');
   Result := pdfReplacePageTextExA(FInstance, PAnsiChar(NewText), Stack);
end;

function TPDF.ReplacePageTextEx(const NewText: WideString; var Stack: TPDFStack): Boolean;
begin
   LoadFunction(@pdfReplacePageTextExW, 'pdfReplacePageTextExW');
   Result := pdfReplacePageTextExW(FInstance, PWideChar(NewText), Stack);
end;

function TPDF.ResetLineDashPattern(): Boolean;
begin
   LoadFunction(@pdfResetLineDashPattern, 'pdfResetLineDashPattern');
   Result := pdfResetLineDashPattern(FInstance);
end;

function TPDF.RestoreGraphicState(): Boolean;
begin
   LoadFunction(@pdfRestoreGraphicState, 'pdfRestoreGraphicState');
   Result := pdfRestoreGraphicState(FInstance);
end;

function TPDF.RotateCoords(alpha, OriginX, OriginY: Double): Boolean;
begin
   LoadFunction(@pdfRotateCoords, 'pdfRotateCoords');
   Result := pdfRotateCoords(FInstance, alpha, OriginX, OriginY);
end;

function TPDF.RoundRect(PosX, PosY, Width, Height, Radius: Double; FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfRoundRect, 'pdfRoundRect');
   Result := pdfRoundRect(FInstance, PosX, PosY, Width, Height, Radius, Ord(FillMode));
end;

function TPDF.RoundRectEx(PosX, PosY, Width, Height, rWidth, rHeight: Double; FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfRoundRectEx, 'pdfRoundRectEx');
   Result := pdfRoundRectEx(FInstance, PosX, PosY, Width, Height, rWidth, rHeight, Ord(FillMode));
end;

function TPDF.SaveGraphicState(): Boolean;
begin
   LoadFunction(@pdfSaveGraphicState, 'pdfSaveGraphicState');
   Result := pdfSaveGraphicState(FInstance);
end;

function TPDF.ScaleCoords(sx, sy: Double): Boolean;
begin
   LoadFunction(@pdfScaleCoords, 'pdfScaleCoords');
   Result := pdfScaleCoords(FInstance, sx, sy);
end;

function TPDF.SelfTest(): Boolean;
begin
   LoadFunction(@pdfSelfTest, 'pdfSelfTest');
   Result := pdfSelfTest(FInstance);
end;

function TPDF.Set3DAnnotProps(Annot: Cardinal; ActType: T3DActivationType; DeActType: T3DDeActivateType; InstType: T3DInstanceType; DeInstType: T3DDeActInstance; DisplToolbar, DisplModelTree: Boolean): Boolean;
begin
   LoadFunction(@pdfSet3DAnnotProps, 'pdfSet3DAnnotProps');
   Result := pdfSet3DAnnotProps(FInstance, Annot, Ord(ActType), Ord(DeActType), Ord(InstType), Ord(DeInstType), DisplToolbar, DisplModelTree);
end;

function TPDF.Set3DAnnotScriptA(Annot: Cardinal; const Value: AnsiString): Boolean;
begin
   LoadFunction(@pdfSet3DAnnotScriptA, 'pdfSet3DAnnotScriptA');
   Result := pdfSet3DAnnotScriptA(FInstance, Annot, PAnsiChar(Value), Length(Value));
end;

function TPDF.SetAllocBy(Value: Integer): Boolean;
begin
   LoadFunction(@pdfSetAllocBy, 'pdfSetAllocBy');
   Result := pdfSetAllocBy(FInstance, Value);
end;

function TPDF.SetAnnotBorderStyle(Handle: Cardinal; Style: TBorderStyle): Boolean;
begin
   LoadFunction(@pdfSetAnnotBorderStyle, 'pdfSetAnnotBorderStyle');
   Result := pdfSetAnnotBorderStyle(FInstance, Handle, Style);
end;

function TPDF.SetAnnotBorderWidth(Handle: Cardinal; LineWidth: Double): Boolean;
begin
   LoadFunction(@pdfSetAnnotBorderWidth, 'pdfSetAnnotBorderWidth');
   Result := pdfSetAnnotBorderWidth(FInstance, Handle, LineWidth);
end;

function TPDF.SetAnnotColor(Handle: Cardinal; ColorType: TAnnotColor; CS: TPDFColorSpace; Color: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetAnnotColor, 'pdfSetAnnotColor');
   Result := pdfSetAnnotColor(FInstance, Handle, ColorType, CS, Color);
end;

function TPDF.SetAnnotFlags(Flags: TAnnotFlags): Boolean;
begin
   LoadFunction(@pdfSetAnnotFlags, 'pdfSetAnnotFlags');
   Result := pdfSetAnnotFlags(FInstance, Flags);
end;

function TPDF.SetAnnotFlagsEx(Handle: Cardinal; Flags: Integer): Boolean;
begin
   LoadFunction(@pdfSetAnnotFlagsEx, 'pdfSetAnnotFlagsEx');
   Result := pdfSetAnnotFlagsEx(FInstance, Handle, Flags);
end;

function TPDF.SetAnnotHighlightMode(Handle: Cardinal; Mode: THighlightMode): Boolean;
begin
   LoadFunction(@pdfSetAnnotHighlightMode, 'pdfSetAnnotHighlightMode');
   Result := pdfSetAnnotHighlightMode(FInstance, Handle, Mode);
end;

function TPDF.SetAnnotIcon(Handle: Cardinal; Icon: TAnnotIcon): Boolean;
begin
   LoadFunction(@pdfSetAnnotIcon, 'pdfSetAnnotIcon');
   Result := pdfSetAnnotIcon(FInstance, Handle, Icon);
end;

function TPDF.SetAnnotOpenState(Handle: Cardinal; Open: Boolean): Boolean;
begin
   LoadFunction(@pdfSetAnnotOpenState, 'pdfSetAnnotOpenState');
   Result := pdfSetAnnotOpenState(FInstance, Handle, Open);
end;

function TPDF.SetAnnotStringA(Handle: Cardinal; StringType: TAnnotString; const Value: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetAnnotStringA, 'pdfSetAnnotStringA');
   Result := pdfSetAnnotStringA(FInstance, Handle, Integer(StringType), PAnsiChar(Value));
end;

function TPDF.SetAnnotStringW(Handle: Cardinal; StringType: TAnnotString; const Value: WideString): Boolean;
begin
   LoadFunction(@pdfSetAnnotStringW, 'pdfSetAnnotStringW');
   Result := pdfSetAnnotStringW(FInstance, Handle, Integer(StringType), PWideChar(Value));
end;

function TPDF.SetAnnotSubjectA(Handle: Cardinal; const Value: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetAnnotSubjectA, 'pdfSetAnnotSubjectA');
   Result := pdfSetAnnotSubjectA(FInstance, Handle, PAnsiChar(Value));
end;

function TPDF.SetAnnotSubjectW(Handle: Cardinal; const Value: WideString): Boolean;
begin
   LoadFunction(@pdfSetAnnotSubjectW, 'pdfSetAnnotSubjectW');
   Result := pdfSetAnnotSubjectW(FInstance, Handle, PWideChar(Value));
end;

function TPDF.SetBBox(Boundary: TPageBoundary; X0, Y0, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfSetBBox, 'pdfSetBBox');
   Result := pdfSetBBox(FInstance, Ord(Boundary), X0, Y0, Width, Height);
end;

function TPDF.SetBidiMode(Mode: TPDFBidiMode): Boolean;
begin
   LoadFunction(@pdfSetBidiMode, 'pdfSetBidiMode');
   Result := pdfSetBidiMode(FInstance, Integer(Mode));
end;

function TPDF.SetBookmarkDest(ABmk: Integer; DestType: TDestType; a, b, c, d: Double): Boolean;
begin
   LoadFunction(@pdfSetBookmarkDest, 'pdfSetBookmarkDest');
   Result := pdfSetBookmarkDest(FInstance, ABmk, Ord(DestType), a, b, c, d);
end;

function TPDF.SetBookmarkStyle(ABmk: Integer; Style: TFStyle; RGBColor: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetBookmarkStyle, 'pdfSetBookmarkStyle');
   Result := pdfSetBookmarkStyle(FInstance, ABmk, Style, RGBColor);
end;

function TPDF.SetBorderStyle(Style: TBorderStyle): Boolean;
begin
   LoadFunction(@pdfSetBorderStyle, 'pdfSetBorderStyle');
   Result := pdfSetBorderStyle(FInstance, Integer(Style));
end;

function TPDF.SetCharacterSpacing(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetCharacterSpacing, 'pdfSetCharacterSpacing');
   Result := pdfSetCharacterSpacing(FInstance, Value);
end;

function TPDF.SetCheckBoxChar(CheckBoxChar: TCheckBoxChar): Boolean;
begin
   LoadFunction(@pdfSetCheckBoxChar, 'pdfSetCheckBoxChar');
   Result := pdfSetCheckBoxChar(FInstance, Ord(CheckBoxChar));
end;

function TPDF.SetCheckBoxDefState(AField: Cardinal; Checked: Boolean): Boolean;
begin
   LoadFunction(@pdfSetCheckBoxDefState, 'pdfSetCheckBoxDefState');
   Result := pdfSetCheckBoxDefState(FInstance, AField, Checked);
end;

function TPDF.SetCheckBoxState(AField: Cardinal; Checked: Boolean): Boolean;
begin
   LoadFunction(@pdfSetCheckBoxState, 'pdfSetCheckBoxState');
   Result := pdfSetCheckBoxState(FInstance, AField, Checked);
end;

function TPDF.SetCIDFontA(CMapHandle: Cardinal; const Name: AnsiString; Style: TFStyle; Size: Double; Embed: Boolean): Integer;
begin
   LoadFunction(@pdfSetCIDFontA, 'pdfSetCIDFontA');
   Result := pdfSetCIDFontA(FInstance, CMapHandle, PAnsiChar(Name), Style, Size, Embed);
end;

function TPDF.SetCIDFontW(CMapHandle: Cardinal; const Name: WideString; Style: TFStyle; Size: Double; Embed: Boolean): Integer;
begin
   LoadFunction(@pdfSetCIDFontW, 'pdfSetCIDFontW');
   Result := pdfSetCIDFontW(FInstance, CMapHandle, PWideChar(Name), Style, Size, Embed);
end;

function TPDF.SetCMapDirA(const Path: AnsiString; Recursive: Boolean): Integer;
begin
   LoadFunction(@pdfSetCMapDirA, 'pdfSetCMapDirA');
   Result := pdfSetCMapDirA(FInstance, PAnsiChar(Path), Recursive);
end;

function TPDF.SetCMapDirW(const Path: WideString; Recursive: Boolean): Integer;
begin
   LoadFunction(@pdfSetCMapDirW, 'pdfSetCMapDirW');
   Result := pdfSetCMapDirW(FInstance, PWideChar(Path), Recursive);
end;

function TPDF.SetColDefFile(EmbFile: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetColDefFile, 'pdfSetColDefFile');
   Result := pdfSetColDefFile(FInstance, EmbFile);
end;

function TPDF.SetColors(Color: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetColors, 'pdfSetColors');
   Result := pdfSetColors(FInstance, Color);
end;

function TPDF.SetColorSpace(ColorSpace: TPDFColorSpace): Boolean;
begin
   LoadFunction(@pdfSetColorSpace, 'pdfSetColorSpace');
   Result := pdfSetColorSpace(FInstance, Ord(ColorSpace));
end;

function TPDF.SetColSortField(ColField: Cardinal; AscendingOrder: Boolean): Boolean;
begin
   LoadFunction(@pdfSetColSortField, 'pdfSetColSortField');
   Result := pdfSetColSortField(FInstance, ColField, AscendingOrder);
end;

function TPDF.SetCompressionFilter(Filter: TCompressionFilter): Boolean;
begin
   LoadFunction(@pdfSetCompressionFilter, 'pdfSetCompressionFilter');
   Result := pdfSetCompressionFilter(FInstance, Ord(Filter));
end;

function TPDF.SetCompressionLevel(CompressLevel: TCompressionLevel): Boolean;
begin
   LoadFunction(@pdfSetCompressionLevel, 'pdfSetCompressionLevel');
   Result := pdfSetCompressionLevel(FInstance, Ord(CompressLevel));
end;

function TPDF.SetContent(const Buffer: PAnsiChar; BufSize: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetContent, 'pdfSetContent');
   Result := pdfSetContent(FInstance, Buffer, BufSize);
end;

function TPDF.SetDateTimeFormat(TxtField: Cardinal; Fmt: TPDFDateTime): Boolean;
begin
   LoadFunction(@pdfSetDateTimeFormat, 'pdfSetDateTimeFormat');
   Result := pdfSetDateTimeFormat(FInstance, TxtField, Integer(Fmt));
end;

function TPDF.SetDefBitsPerPixel(Value: Integer): Boolean;
begin
   LoadFunction(@pdfSetDefBitsPerPixel, 'pdfSetDefBitsPerPixel');
   Result := pdfSetDefBitsPerPixel(FInstance, Value);
end;

function TPDF.SetDocInfoA(DInfo: TDocumentInfo; const Value: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetDocInfoA, 'pdfSetDocInfoA');
   Result := pdfSetDocInfoA(FInstance, Ord(DInfo), PAnsiChar(Value));
end;

function TPDF.SetDocInfoW(DInfo: TDocumentInfo; const Value: WideString): Boolean;
begin
   LoadFunction(@pdfSetDocInfoW, 'pdfSetDocInfoW');
   Result := pdfSetDocInfoW(FInstance, Ord(DInfo), PWideChar(Value));
end;

function TPDF.SetDocInfoExA(DInfo: TDocumentInfo; const Key, Value: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetDocInfoExA, 'pdfSetDocInfoExA');
   Result := pdfSetDocInfoExA(FInstance, Integer(DInfo), PAnsiChar(Key), PAnsiChar(Value));
end;

function TPDF.SetDocInfoExW(DInfo: TDocumentInfo; const Key: AnsiString; const Value: WideString): Boolean;
begin
   LoadFunction(@pdfSetDocInfoExW, 'pdfSetDocInfoExW');
   Result := pdfSetDocInfoExW(FInstance, Integer(DInfo), PAnsiChar(Key), PWideChar(Value));
end;

function TPDF.SetDrawDirection(Direction: TDrawDirection): Boolean;
begin
   LoadFunction(@pdfSetDrawDirection, 'pdfSetDrawDirection');
   Result := pdfSetDrawDirection(FInstance, Ord(Direction));
end;

function TPDF.SetEMFFrameDPI(DPIX, DPIY: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetEMFFrameDPI, 'pdfSetEMFFrameDPI');
   Result := pdfSetEMFFrameDPI(FInstance, DPIX, DPIY);
end;

function TPDF.SetEMFPatternDistance(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetEMFPatternDistance, 'pdfSetEMFPatternDistance');
   Result := pdfSetEMFPatternDistance(FInstance, Value);
end;

function TPDF.SetErrorMode(ErrMode: TErrMode): Boolean;
begin
   LoadFunction(@pdfSetErrorMode, 'pdfSetErrorMode');
   Result := pdfSetErrorMode(FInstance, ErrMode);
end;

function TPDF.SetExtColorSpace(Handle: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetExtColorSpace, 'pdfSetExtColorSpace');
   Result := pdfSetExtColorSpace(FInstance, Handle);
end;

function TPDF.SetExtGState(Handle: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetExtGState, 'pdfSetExtGState');
   Result := pdfSetExtGState(FInstance, Handle);
end;

function TPDF.SetFieldBackColor(AColor: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetFieldBackColor, 'pdfSetFieldBackColor');
   Result := pdfSetFieldBackColor(FInstance, AColor);
end;

function TPDF.SetFieldBBox(AField: Cardinal; var BBox: TPDFRect): Boolean;
begin
   LoadFunction(@pdfSetFieldBBox, 'pdfSetFieldBBox');
   Result := pdfSetFieldBBox(FInstance, AField, BBox);
end;

function TPDF.SetFieldBorderColor(AColor: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetFieldBorderColor, 'pdfSetFieldBorderColor');
   Result := pdfSetFieldBorderColor(FInstance, AColor);
end;

function TPDF.SetFieldBorderStyle(AField: Cardinal; Style: TBorderStyle): Boolean;
begin
   LoadFunction(@pdfSetFieldBorderStyle, 'pdfSetFieldBorderStyle');
   Result := pdfSetFieldBorderStyle(FInstance, AField, Integer(Style));
end;

function TPDF.SetFieldBorderWidth(AField: Cardinal; LineWidth: Double): Boolean;
begin
   LoadFunction(@pdfSetFieldBorderWidth, 'pdfSetFieldBorderWidth');
   Result := pdfSetFieldBorderWidth(FInstance, AField, LineWidth);
end;

function TPDF.SetFieldColor(AField: Cardinal; ColorType: TFieldColor; CS: TPDFColorSpace; Color: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetFieldColor, 'pdfSetFieldColor');
   Result := pdfSetFieldColor(FInstance, AField, Integer(ColorType), Integer(CS), Color);
end;

function TPDF.SetFieldExpValue(AField, ValIndex: Cardinal; const Value, ExpValue: AnsiString; Selected: Boolean): Boolean;
begin
   LoadFunction(@pdfSetFieldExpValue, 'pdfSetFieldExpValue');
   Result := pdfSetFieldExpValue(FInstance, AField, ValIndex, PAnsiChar(Value), PAnsiChar(ExpValue), Selected);
end;

function TPDF.SetFieldFlags(AField: Cardinal; Flags: TFieldFlags; Reset: Boolean): Boolean;
begin
   LoadFunction(@pdfSetFieldFlags, 'pdfSetFieldFlags');
   Result := pdfSetFieldFlags(FInstance, AField, Integer(Flags), Reset);
end;

function TPDF.SetFieldFontSize(AField: Cardinal; FontSize: double): Boolean;
begin
   LoadFunction(@pdfSetFieldFontSize, 'pdfSetFieldFontSize');
   Result := pdfSetFieldFontSize(FInstance, AField, FontSize);
end;

function TPDF.SetFieldHighlightMode(AField: Cardinal; Mode: THighlightMode): Boolean;
begin
   LoadFunction(@pdfSetFieldHighlightMode, 'pdfSetFieldHighlightMode');
   Result := pdfSetFieldHighlightMode(FInstance, AField, Integer(Mode));
end;

function TPDF.SetFieldIndex(AField, Index: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetFieldIndex, 'pdfSetFieldIndex');
   Result := pdfSetFieldIndex(FInstance, AField, Index);
end;

function TPDF.SetFieldMapNameA(AField: Cardinal; const Name: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetFieldMapNameA, 'pdfSetFieldMapNameA');
   Result := pdfSetFieldMapNameA(FInstance, AField, PAnsiChar(Name));
end;

function TPDF.SetFieldMapNameW(AField: Cardinal; const Name: WideString): Boolean;
begin
   LoadFunction(@pdfSetFieldMapNameW, 'pdfSetFieldMapNameW');
   Result := pdfSetFieldMapNameW(FInstance, AField, PWideChar(Name));
end;

function TPDF.SetFieldName(AField: Cardinal; const NewName: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetFieldName, 'pdfSetFieldName');
   Result := pdfSetFieldName(FInstance, AField, PAnsiChar(NewName));
end;

function TPDF.SetFieldOrientation(AField: Cardinal; Value: Integer): Boolean;
begin
   LoadFunction(@pdfSetFieldOrientation, 'pdfSetFieldOrientation');
   Result := pdfSetFieldOrientation(FInstance, AField, Value);
end;

function TPDF.SetFieldTextAlign(AField: Cardinal; Align: TTextAlign): Boolean;
begin
   LoadFunction(@pdfSetFieldTextAlign, 'pdfSetFieldTextAlign');
   Result := pdfSetFieldTextAlign(FInstance, AField, Integer(Align));
end;

function TPDF.SetFieldTextColor(Color: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetFieldTextColor, 'pdfSetFieldTextColor');
   Result := pdfSetFieldTextColor(FInstance, Color);
end;

function TPDF.SetFieldToolTipA(AField: Cardinal; const Value: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetFieldToolTipA, 'pdfSetFieldToolTipA');
   Result := pdfSetFieldToolTipA(FInstance, AField, PAnsiChar(Value));
end;

function TPDF.SetFieldToolTipW(AField: Cardinal; const Value: WideString): Boolean;
begin
   LoadFunction(@pdfSetFieldToolTipW, 'pdfSetFieldToolTipW');
   Result := pdfSetFieldToolTipW(FInstance, AField, PWideChar(Value));
end;

function TPDF.SetFillColor(Color: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetFillColor, 'pdfSetFillColor');
   Result := pdfSetFillColor(FInstance, Color);
end;

function TPDF.SetFillColorEx(const Color: Array of Byte; NumComponents: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetFillColorEx, 'pdfSetFillColorEx');
   Result := pdfSetFillColorEx(FInstance, @Color[0], NumComponents);
end;

procedure TPDF.SetFillColorSpace(CS: TPDFColorSpace);
begin
   LoadFunction(@pdfSetFillColorSpace, 'pdfSetFillColorSpace');
   pdfSetFillColorSpace(FInstance, Integer(CS));
end;

function TPDF.SetFontA(const Name: AnsiString; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage = cp1252): Integer;
begin
   LoadFunction(@pdfSetFontA, 'pdfSetFontA');
   Result := pdfSetFontA(FInstance, PAnsiChar(Name), Style, Size, Integer(Embed), Ord(CP));
end;

function TPDF.SetFontW(const Name: WideString; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage = cp1252): Integer;
begin
   LoadFunction(@pdfSetFontW, 'pdfSetFontW');
   Result := pdfSetFontW(FInstance, PWideChar(Name), Style, Size, Integer(Embed), Ord(CP));
end;

function TPDF.SetFontExA(const Name: AnsiString; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage): Integer;
begin
   LoadFunction(@pdfSetFontExA, 'pdfSetFontExA');
   Result := pdfSetFontExA(FInstance, PAnsiChar(Name), Style, Size, Embed, Integer(CP));
end;

function TPDF.SetFontExW(const Name: WideString; Style: TFStyle; Size: Double; Embed: Boolean; CP: TCodepage): Integer;
begin
   LoadFunction(@pdfSetFontExW, 'pdfSetFontExW');
   Result := pdfSetFontExW(FInstance, PWideChar(Name), Style, Size, Embed, Integer(CP));
end;

function TPDF.SetFontOrigin(Origin: TOrigin): Boolean;
begin
   LoadFunction(@pdfSetFontOrigin, 'pdfSetFontOrigin');
   Result := pdfSetFontOrigin(FInstance, Integer(Origin));
end;

procedure TPDF.SetFontSearchOrder(Order: PFontBaseType);
begin
   LoadFunction(@pdfSetFontSearchOrder, 'pdfSetFontSearchOrder');
   pdfSetFontSearchOrder(FInstance, Order);
end;

function TPDF.SetFontSelMode(Mode: TFontSelMode): Boolean;
begin
   LoadFunction(@pdfSetFontSelMode, 'pdfSetFontSelMode');
   Result := pdfSetFontSelMode(FInstance, Ord(Mode));
end;

function TPDF.SetFontWeight(Weight: Integer): Boolean;
begin
   LoadFunction(@pdfSetFontWeight, 'pdfSetFontWeight');
   Result := pdfSetFontWeight(FInstance, Weight);
end;

function TPDF.SetGStateFlags(Flags: TGStateFlags; Reset: Boolean): Boolean;
begin
   LoadFunction(@pdfSetGStateFlags, 'pdfSetGStateFlags');
   Result := pdfSetGStateFlags(FInstance, Flags, Reset);
end;

function TPDF.SetIconColor(Color: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetIconColor, 'pdfSetIconColor');
   Result := pdfSetIconColor(FInstance, Color);
end;

function TPDF.SetImportFlags(Flags: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetImportFlags, 'pdfSetImportFlags');
   Result := pdfSetImportFlags(FInstance, Flags);
end;

function TPDF.SetImportFlags2(Flags: TImportFlags2): Boolean;
begin
   LoadFunction(@pdfSetImportFlags2, 'pdfSetImportFlags2');
   Result := pdfSetImportFlags2(FInstance, Flags);
end;

function TPDF.SetItalicAngle(Angle: Double): Boolean;
begin
   LoadFunction(@pdfSetItalicAngle, 'pdfSetItalicAngle');
   Result := pdfSetItalicAngle(FInstance, Angle);
end;

function TPDF.SetJPEGQuality(Value: Integer): Boolean;
begin
   LoadFunction(@pdfSetJPEGQuality, 'pdfSetJPEGQuality');
   Result := pdfSetJPEGQuality(FInstance, Value);
end;

function TPDF.SetLanguage(const ISOTag: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetLanguage, 'pdfSetLanguage');
   Result := pdfSetLanguage(FInstance, PAnsiChar(ISOTag));
end;

function TPDF.SetLeading(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetLeading, 'pdfSetLeading');
   Result := pdfSetLeading(FInstance, Value);
end;

function TPDF.SetLicenseKey(const Key: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetLicenseKey, 'pdfSetLicenseKey');
   Result := pdfSetLicenseKey(FInstance, PAnsiChar(Key));
end;

function TPDF.SetLineCapStyle(Style: TLineCapStyle): Boolean;
begin
   LoadFunction(@pdfSetLineCapStyle, 'pdfSetLineCapStyle');
   Result := pdfSetLineCapStyle(FInstance, Integer(Style));
end;

function TPDF.SetLineDashPattern(const Dash: AnsiString; Phase: Integer): Boolean;
begin
   LoadFunction(@pdfSetLineDashPattern, 'pdfSetLineDashPattern');
   Result := pdfSetLineDashPattern(FInstance, PAnsiChar(Dash), Phase);
end;

function TPDF.SetLineDashPatternEx(const Dash: Array of Double; NumValues: Cardinal; Phase: Integer): Boolean;
begin
   LoadFunction(@pdfSetLineDashPatternEx, 'pdfSetLineDashPatternEx');
   Result := pdfSetLineDashPatternEx(FInstance, Pointer(@Dash), NumValues, Phase);
end;

function TPDF.SetLineJoinStyle(Style: TLineJoinStyle): Boolean;
begin
   LoadFunction(@pdfSetLineJoinStyle, 'pdfSetLineJoinStyle');
   Result := pdfSetLineJoinStyle(FInstance, Integer(Style));
end;

function TPDF.SetLineWidth(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetLineWidth, 'pdfSetLineWidth');
   Result := pdfSetLineWidth(FInstance, Value);
end;

function TPDF.SetLinkHighlightMode(Mode: THighlightMode): Boolean;
begin
   LoadFunction(@pdfSetLinkHighlightMode, 'pdfSetLinkHighlightMode');
   Result := pdfSetLinkHighlightMode(FInstance, Ord(Mode));
end;

function TPDF.SetListFont(Handle: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetListFont, 'pdfSetListFont');
   Result := pdfSetListFont(FInstance, Handle);
end;

function TPDF.SetMatrix(var Matrix: TCTM): Boolean;
begin
   LoadFunction(@pdfSetMatrix, 'pdfSetMatrix');
   Result := pdfSetMatrix(FInstance, Matrix);
end;

function TPDF.SetMaxFieldLen(TxtField: Cardinal; MaxLen: Integer): Boolean;
begin
   LoadFunction(@pdfSetMaxFieldLen, 'pdfSetMaxFieldLen');
   Result := pdfSetMaxFieldLen(FInstance, TxtField, MaxLen);
end;

function TPDF.SetMetaConvFlags(Flags: TMetaFlags): Boolean;
begin
   LoadFunction(@pdfSetMetaConvFlags, 'pdfSetMetaConvFlags');
   Result := pdfSetMetaConvFlags(FInstance, Flags);
end;

function TPDF.SetMiterLimit(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetMiterLimit, 'pdfSetMiterLimit');
   Result := pdfSetMiterLimit(FInstance, Value);
end;

function TPDF.SetNeedAppearance(Value: Boolean): Boolean;
begin
   LoadFunction(@pdfSetNeedAppearance, 'pdfSetNeedAppearance');
   Result := pdfSetNeedAppearance(FInstance, Value);
end;

function TPDF.SetNumberFormat(TxtField: Cardinal; Sep: TDecSeparator; DecPlaces: Cardinal; NegStyle: TNegativeStyle; const CurrStr: AnsiString; Prepend: Boolean): Boolean;
begin
   LoadFunction(@pdfSetNumberFormat, 'pdfSetNumberFormat');
   Result := pdfSetNumberFormat(FInstance, TxtField, Integer(Sep), DecPlaces, Integer(NegStyle), PAnsiChar(CurrStr), Prepend);
end;

function TPDF.SetOnErrorProc(const Data, ErrProc: Pointer): Boolean;
begin
   LoadFunction(@pdfSetOnErrorProc, 'pdfSetOnErrorProc');
   Result := pdfSetOnErrorProc(FInstance, Data, ErrProc);
end;

function TPDF.SetOnPageBreakProc(const Data, OnBreakProc: Pointer): Boolean;
begin
   LoadFunction(@pdfSetOnPageBreakProc, 'pdfSetOnPageBreakProc');
   Result := pdfSetOnPageBreakProc(FInstance, Data, OnBreakProc);
end;

function TPDF.SetOpacity(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetOpacity, 'pdfSetOpacity');
   Result := pdfSetOpacity(FInstance, Value);
end;

function TPDF.SetOrientation(Value: Integer): Boolean;
begin
   LoadFunction(@pdfSetOrientation, 'pdfSetOrientation');
   Result := pdfSetOrientation(FInstance, Value);
end;

function TPDF.SetOrientationEx(Value: Integer): Boolean;
begin
   LoadFunction(@pdfSetOrientationEx, 'pdfSetOrientationEx');
   Result := pdfSetOrientationEx(FInstance, Value);
end;

function TPDF.SetPageCoords(PageCoords: TPageCoord): Boolean;
begin
   LoadFunction(@pdfSetPageCoords, 'pdfSetPageCoords');
   Result := pdfSetPageCoords(FInstance, Ord(PageCoords));
end;

function TPDF.SetPageFormat(Value: TPageFormat): Boolean;
begin
   LoadFunction(@pdfSetPageFormat, 'pdfSetPageFormat');
   Result := pdfSetPageFormat(FInstance, Integer(Value));
end;

function TPDF.SetPageHeight(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetPageHeight, 'pdfSetPageHeight');
   Result := pdfSetPageHeight(FInstance, Value);
end;

function TPDF.SetPageLayout(Layout: TPageLayout): Boolean;
begin
   LoadFunction(@pdfSetPageLayout, 'pdfSetPageLayout');
   if Layout = plDefault then
      Result := pdfSetPageLayout(FInstance, -1)
   else
      Result := pdfSetPageLayout(FInstance, Ord(Layout));
end;

function TPDF.SetPageMode(Mode: TPageMode): Boolean;
begin
   LoadFunction(@pdfSetPageMode, 'pdfSetPageMode');
   Result := pdfSetPageMode(FInstance, Ord(Mode));
end;

function TPDF.SetPageWidth(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetPageWidth, 'pdfSetPageWidth');
   Result := pdfSetPageWidth(FInstance, Value);
end;

function TPDF.SetPDFVersion(Version: TPDFVersion): Boolean;
begin
   LoadFunction(@pdfSetPDFVersion, 'pdfSetPDFVersion');
   Result := pdfSetPDFVersion(FInstance, Ord(Version));
end;

function TPDF.SetPrintSettings(Mode: TDuplexMode; PickTrayByPDFSize: Integer; NumCopies: Cardinal; PrintScaling: TPrintScaling; const PrintRanges: Array of Cardinal; NumRanges: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetPrintSettings, 'pdfSetPrintSettings');
   Result := pdfSetPrintSettings(FInstance, Mode, PickTrayByPDFSize, NumCopies, PrintScaling, @PrintRanges, NumRanges);
end;

function TPDF.SetProgressProc(const Data, InitProgressProc, ProgressProc: Pointer): Boolean;
begin
   LoadFunction(@pdfSetProgressProc, 'pdfSetProgressProc');
   Result := pdfSetProgressProc(FInstance, Data, InitProgressProc, ProgressProc);
end;

function TPDF.SetResolution(Value: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetResolution, 'pdfSetResolution');
   Result := pdfSetResolution(FInstance, Value);
end;

function TPDF.SetSaveNewImageFormat(Value: Boolean): Boolean;
begin
   LoadFunction(@pdfSetSaveNewImageFormat, 'pdfSetSaveNewImageFormat');
   Result := pdfSetSaveNewImageFormat(FInstance, Value);
end;

function TPDF.SetSeparationInfo(Handle: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetSeparationInfo, 'pdfSetSeparationInfo');
   Result := pdfSetSeparationInfo(FInstance, Handle);
end;

function TPDF.SetStrokeColor(Color: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetStrokeColor, 'pdfSetStrokeColor');
   Result := pdfSetStrokeColor(FInstance, Color);
end;

function TPDF.SetStrokeColorEx(const Color: Array of Byte; NumComponents: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetStrokeColorEx, 'pdfSetStrokeColorEx');
   Result := pdfSetStrokeColorEx(FInstance, @Color[0], NumComponents);
end;

procedure TPDF.SetStrokeColorSpace(CS: TPDFColorSpace);
begin
   LoadFunction(@pdfSetStrokeColorSpace, 'pdfSetStrokeColorSpace');
   pdfSetStrokeColorSpace(FInstance, Integer(CS));
end;

function TPDF.SetTabLen(TabLen: Integer): Boolean;
begin
   LoadFunction(@pdfSetTabLen, 'pdfSetTabLen');
   Result := pdfSetTabLen(FInstance, TabLen);
end;

function TPDF.SetTextDrawMode(Mode: TDrawMode): Boolean;
begin
   LoadFunction(@pdfSetTextDrawMode, 'pdfSetTextDrawMode');
   Result := pdfSetTextDrawMode(FInstance, Ord(Mode));
end;

function TPDF.SetTextFieldValueA(Field: Cardinal; const Value, DefValue: AnsiString; Align: TTextAlign): Boolean;
begin
   LoadFunction(@pdfSetTextFieldValueA, 'pdfSetTextFieldValueA');
   Result := pdfSetTextFieldValueA(FInstance, Field, PAnsiChar(Value), PAnsiChar(DefValue), Ord(Align));
end;

function TPDF.SetTextFieldValueW(Field: Cardinal; const Value, DefValue: WideString; Align: TTextAlign): Boolean;
begin
   LoadFunction(@pdfSetTextFieldValueW, 'pdfSetTextFieldValueW');
   Result := pdfSetTextFieldValueW(FInstance, Field, PWideChar(Value), PWideChar(DefValue), Ord(Align));
end;

function TPDF.SetTextFieldValueExA(Field: Cardinal; const Value: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetTextFieldValueExA, 'pdfSetTextFieldValueExA');
   Result := pdfSetTextFieldValueExA(FInstance, Field, PAnsiChar(Value));
end;

function TPDF.SetTextFieldValueExW(Field: Cardinal; const Value: WideString): Boolean;
begin
   LoadFunction(@pdfSetTextFieldValueExW, 'pdfSetTextFieldValueExW');
   Result := pdfSetTextFieldValueExW(FInstance, Field, PWideChar(Value));
end;

function TPDF.SetTextRect(PosX, PosY, Width, Height: Double): Boolean;
begin
   LoadFunction(@pdfSetTextRect, 'pdfSetTextRect');
   Result := pdfSetTextRect(FInstance, PosX, PosY, Width, Height);
end;

function TPDF.SetTextRise(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetTextRise, 'pdfSetTextRise');
   Result := pdfSetTextRise(FInstance, Value);
end;

function TPDF.SetTextScaling(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetTextScaling, 'pdfSetTextScaling');
   Result := pdfSetTextScaling(FInstance, Value);
end;

function TPDF.SetTransparentColor(AColor: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetTransparentColor, 'pdfSetTransparentColor');
   Result := pdfSetTransparentColor(FInstance, AColor);
end;

procedure TPDF.SetTrapped(Value: Boolean);
begin
   LoadFunction(@pdfSetTrapped, 'pdfSetTrapped');
   pdfSetTrapped(FInstance, Value);
end;

function TPDF.SetUseExactPwd(Value: Boolean): Boolean;
begin
   LoadFunction(@pdfSetUseExactPwd, 'pdfSetUseExactPwd');
   Result := pdfSetUseExactPwd(FInstance, Ord(Value));
end;

function TPDF.SetUseGlobalImpFiles(Value: Boolean): Boolean;
begin
   LoadFunction(@pdfSetUseGlobalImpFiles, 'pdfSetUseGlobalImpFiles');
   Result := pdfSetUseGlobalImpFiles(FInstance, Value);
end;

function TPDF.SetUseStdFonts(Value: Boolean): Boolean;
begin
   LoadFunction(@pdfSetUseStdFonts, 'pdfSetUseStdFonts');
   Result := pdfSetUseStdFonts(FInstance, Value);
end;

function TPDF.SetUseSwapFile(SwapContents: Boolean; SwapLimit: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetUseSwapFile, 'pdfSetUseSwapFile');
   Result := pdfSetUseSwapFile(FInstance, SwapContents, SwapLimit);
end;

function TPDF.SetUseSwapFileEx(SwapContents: Boolean; SwapLimit: Cardinal; const SwapDir: AnsiString): Boolean;
begin
   LoadFunction(@pdfSetUseSwapFileEx, 'pdfSetUseSwapFileEx');
   Result := pdfSetUseSwapFileEx(FInstance, SwapContents, SwapLimit, PAnsiChar(SwapDir));
end;

function TPDF.SetUseSystemFonts(Value: Boolean): Boolean;
begin
   LoadFunction(@pdfSetUseSystemFonts, 'pdfSetUseSystemFonts');
   Result := pdfSetUseSystemFonts(FInstance, Value);
end;

function TPDF.SetUseTransparency(Value: Boolean): Boolean;
begin
   LoadFunction(@pdfSetUseTransparency, 'pdfSetUseTransparency');
   Result := pdfSetUseTransparency(FInstance, Value);
end;

function TPDF.SetUseVisibleCoords(Value: Boolean): Boolean;
begin
   LoadFunction(@pdfSetUseVisibleCoords, 'pdfSetUseVisibleCoords');
   Result := pdfSetUseVisibleCoords(FInstance, Value);
end;

function TPDF.SetViewerPreferences(Value: TViewerPreference; AddVal: TViewPrefAddVal): Boolean;
begin
   LoadFunction(@pdfSetViewerPreferences, 'pdfSetViewerPreferences');
   Result := pdfSetViewerPreferences(FInstance, Value, AddVal);
end;

function TPDF.SetWMFDefExtent(Width, Height: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetWMFDefExtent, 'pdfSetWMFDefExtent');
   Result := pdfSetWMFDefExtent(FInstance, Width, Height);
end;

function TPDF.SetWMFPixelPerInch(Value: Cardinal): Boolean;
begin
   LoadFunction(@pdfSetWMFPixelPerInch, 'pdfSetWMFPixelPerInch');
   Result := pdfSetWMFPixelPerInch(FInstance, Value);
end;

function TPDF.SetWordSpacing(Value: Double): Boolean;
begin
   LoadFunction(@pdfSetWordSpacing, 'pdfSetWordSpacing');
   Result := pdfSetWordSpacing(FInstance, Value);
end;

function TPDF.SkewCoords(alpha, beta, OriginX, OriginY: Double): Boolean;
begin
   LoadFunction(@pdfSkewCoords, 'pdfSkewCoords');
   Result := pdfSkewCoords(FInstance, alpha, beta, OriginX, OriginY);
end;

function TPDF.SortFieldsByIndex(): Boolean;
begin
   LoadFunction(@pdfSortFieldsByIndex, 'pdfSortFieldsByIndex');
   Result := pdfSortFieldsByIndex(FInstance);
end;

function TPDF.SortFieldsByName(): Boolean;
begin
   LoadFunction(@pdfSortFieldsByName, 'pdfSortFieldsByName');
   Result := pdfSortFieldsByName(FInstance);
end;

function TPDF.StampAnnotA(SubType: TRubberStamp; PosX, PosY, Width, Height: Double; const Author, Subject, Comment: AnsiString): Integer;
begin
   LoadFunction(@pdfStampAnnotA, 'pdfStampAnnotA');
   Result := pdfStampAnnotA(FInstance, SubType, PosX, PosY, Width, Height, PAnsiChar(Author), PAnsiChar(Subject), PAnsiChar(Comment));
end;

function TPDF.StampAnnotW(SubType: TRubberStamp; PosX, PosY, Width, Height: Double; const Author, Subject, Comment: WideString): Integer;
begin
   LoadFunction(@pdfStampAnnotW, 'pdfStampAnnotW');
   Result := pdfStampAnnotW(FInstance, SubType, PosX, PosY, Width, Height, PWideChar(Author), PWideChar(Subject), PWideChar(Comment));
end;

function TPDF.StrokePath(): Boolean;
begin
   LoadFunction(@pdfStrokePath, 'pdfStrokePath');
   Result := pdfStrokePath(FInstance);
end;

function TPDF.TextAnnotA(PosX, PosY, Width, Height: Double; const Author, Text: AnsiString; Icon: TAnnotIcon; Open: Boolean): Integer;
begin
   LoadFunction(@pdfTextAnnotA, 'pdfTextAnnotA');
   Result := pdfTextAnnotA(FInstance, PosX, PosY, Width, Height, PAnsiChar(Author), PAnsiChar(Text), Ord(Icon), Integer(Open));
end;

function TPDF.TextAnnotW(PosX, PosY, Width, Height: Double; const Author, Text: WideString; Icon: TAnnotIcon; Open: Boolean): Integer;
begin
   LoadFunction(@pdfTextAnnotW, 'pdfTextAnnotW');
   Result := pdfTextAnnotW(FInstance, PosX, PosY, Width, Height, PWideChar(Author), PWideChar(Text), Ord(Icon), Integer(Open));
end;
{$ifdef VER120} // Delphi 4
function TPDF.ToUnicodeA(const S: AnsiString): WideString;
var i, p, len: Integer;
begin
   len := Length(S) div 2;
   if len = 0 then Exit;
   SetLength(Result, len);
   p := 1;
   for i := 1 to len do begin
      Result[i] := WideChar(MakeWord(Byte(S[p]), Byte(S[p+1])));
      Inc(p, 2);
   end;
end;

function TPDF.ToUnicodeW(const S: PAnsiChar; Len: Integer): WideString;
var i, p: Integer;
begin
   Len := Len div 2;
   if Len = 0 then Exit;
   SetLength(Result, Len);
   p := 0;
   for i := 1 to Len do begin
      Result[i] := WideChar(MakeWord(Byte(S[p]), Byte(S[p+1])));
      Inc(p, 2);
   end;
end;
{$else}
function TPDF.ToUnicode(const S: AnsiString): WideString;
var i, p, len: Integer;
begin
   len := Length(S) div 2;
   if len = 0 then Exit;
   SetLength(Result, len);
   p := 1;
   for i := 1 to len do begin
      Result[i] := WideChar(MakeWord(Byte(S[p]), Byte(S[p+1])));
      Inc(p, 2);
   end;
end;

function TPDF.ToUnicode(const S: PAnsiChar; Len: Integer): WideString;
var i, p: Integer;
begin
   Len := Len div 2;
   if Len = 0 then Exit;
   SetLength(Result, Len);
   p := 0;
   for i := 1 to Len do begin
      Result[i] := WideChar(MakeWord(Byte(S[p]), Byte(S[p+1])));
      Inc(p, 2);
   end;
end;
{$endif}
function TPDF.TranslateCoords(NewOriginX, NewOriginY: Double): Boolean;
begin
   LoadFunction(@pdfTranslateCoords, 'pdfTranslateCoords');
   Result := pdfTranslateCoords(FInstance, NewOriginX, NewOriginY);
end;

function TPDF.TranslateRawCode(const IFont: Pointer; const Text: PAnsiChar; Len: Cardinal; var Width: Double; OutText: PWideChar; var OutLen: Integer; var Decoded: LongBool; CharSpacing, WordSpacing, TextScale: Single): Cardinal;
begin
   LoadFunction(@fntTranslateRawCode, 'fntTranslateRawCode');
   Result := fntTranslateRawCode(IFont, Text, Len, Width, OutText, OutLen, Decoded, CharSpacing, WordSpacing, TextScale);
end;

function TPDF.TranslateString(var Stack: TPDFStack; var Buffer: PWideChar; var Size: Cardinal; Flags: Cardinal): Integer;
begin
   // The buffer should be initially allocated with enough room for 128 characters
   if Size < Stack.TextLen then begin
      Inc(Size, Stack.TextLen);
      ReAllocMem(Buffer, Size * sizeof(WideChar));
   end;
   LoadFunction(@fntTranslateString, 'fntTranslateString');
   Result := fntTranslateString(Stack, Buffer, Size, Flags);
end;

function TPDF.TranslateString2(const IFont: Pointer; const Text: PAnsiChar; Len: Cardinal; OutBuf: WideString; Size, Flags: Cardinal): Integer;
begin
   LoadFunction(@fntTranslateString2, 'fntTranslateString2');
   Result := fntTranslateString2(IFont, Text, Len, PWideChar(OutBuf), Size, Flags);
end;

function TPDF.Triangle(x1, y1, x2, y2, x3, y3: Double; FillMode: TPathFillMode): Boolean;
begin
   LoadFunction(@pdfTriangle, 'pdfTriangle');
   Result := pdfTriangle(FInstance, x1, y1, x2, y2, x3, y3, Ord(FillMode));
end;

function TPDF.UTF16ToUTF32(const Source: WideString): Pointer;
begin
   LoadFunction(@pdfUTF16ToUTF32, 'pdfUTF16ToUTF32');
   Result := pdfUTF16ToUTF32(FInstance, PWideChar(Source));
end;

function TPDF.UTF16ToUTF32Ex(const Source: WideString; var Len: Cardinal): Pointer;
begin
   LoadFunction(@pdfUTF16ToUTF32Ex, 'pdfUTF16ToUTF32Ex');
   Result := pdfUTF16ToUTF32Ex(FInstance, PWideChar(Source), Len);
end;

function TPDF.UTF32ToUTF16(const Source: Pointer): PWideChar;
begin
   LoadFunction(@pdfUTF32ToUTF16, 'pdfUTF32ToUTF16');
   Result := pdfUTF32ToUTF16(FInstance, Source);
end;

function TPDF.UTF32ToUTF16Ex(const Source: Pointer; var Len: Cardinal): PWideChar;
begin
   LoadFunction(@pdfUTF32ToUTF16Ex, 'pdfUTF32ToUTF16Ex');
   Result := pdfUTF32ToUTF16Ex(FInstance, Source, Len);
end;

function TPDF.WebLinkA(PosX, PosY, Width, Height: Double; const URL: AnsiString): Integer;
begin
   LoadFunction(@pdfWebLinkA, 'pdfWebLinkA');
   Result := pdfWebLinkA(FInstance, PosX, PosY, Width, Height, PAnsiChar(URL));
end;

function TPDF.WebLinkW(PosX, PosY, Width, Height: Double; const URL: WideString): Integer;
begin
   LoadFunction(@pdfWebLinkW, 'pdfWebLinkW');
   Result := pdfWebLinkW(FInstance, PosX, PosY, Width, Height, PWideChar(URL));
end;

function TPDF.WeigthFromStyle(Style: TFStyle): Cardinal;
begin
   Result := (Style and not $C00FFFFF) shr 20;
end;

function TPDF.WeightToStyle(Weight: Cardinal): TFStyle;
begin
   Result := Weight shl 20;
end;

function TPDF.WidthFromStyle(Style: TFStyle): Cardinal;
begin
   Result := (Style and not $FFF2D00F) shr 8;
end;

function TPDF.WidthToStyle(Width: Cardinal): TFStyle;
begin
   Result := Width shl 8;
end;

function TPDF.WriteAngleTextA(const AText: AnsiString; Angle, PosX, PosY, Radius, YOrigin: Double): Boolean;
begin
   LoadFunction(@pdfWriteAngleTextA, 'pdfWriteAngleTextA');
   Result := pdfWriteAngleTextA(FInstance, PAnsiChar(AText), Angle, PosX, PosY, Radius, YOrigin);
end;

function TPDF.WriteAngleTextW(const AText: WideString; Angle, PosX, PosY, Radius, YOrigin: Double): Boolean;
begin
   LoadFunction(@pdfWriteAngleTextW, 'pdfWriteAngleTextW');
   Result := pdfWriteAngleTextW(FInstance, PWideChar(AText), Angle, PosX, PosY, Radius, YOrigin);
end;

function TPDF.WriteFTextA(Align: TTextAlign; const AText: AnsiString): Boolean;
begin
   LoadFunction(@pdfWriteFTextA, 'pdfWriteFTextA');
   Result := pdfWriteFTextA(FInstance, Ord(Align), PAnsiChar(AText));
end;

function TPDF.WriteFTextW(Align: TTextAlign; const AText: WideString): Boolean;
begin
   LoadFunction(@pdfWriteFTextW, 'pdfWriteFTextW');
   Result := pdfWriteFTextW(FInstance, Ord(Align), PWideChar(AText));
end;

function TPDF.WriteFTextExA(PosX, PosY, Width, Height: Double; Align: TTextAlign; const AText: AnsiString): Boolean;
begin
   LoadFunction(@pdfWriteFTextExA, 'pdfWriteFTextExA');
   Result := pdfWriteFTextExA(FInstance, PosX, PosY, Width, Height, Ord(Align), PAnsiChar(AText));
end;

function TPDF.WriteFTextExW(PosX, PosY, Width, Height: Double; Align: TTextAlign; const AText: WideString): Boolean;
begin
   LoadFunction(@pdfWriteFTextExW, 'pdfWriteFTextExW');
   Result := pdfWriteFTextExW(FInstance, PosX, PosY, Width, Height, Ord(Align), PWideChar(AText));
end;

function TPDF.WriteTextA(PosX, PosY: Double; const AText: AnsiString): Boolean;
begin
   LoadFunction(@pdfWriteTextA, 'pdfWriteTextA');
   Result := pdfWriteTextA(FInstance, PosX, PosY, PAnsiChar(AText));
end;

function TPDF.WriteTextW(PosX, PosY: Double; const AText: WideString): Boolean;
begin
   LoadFunction(@pdfWriteTextW, 'pdfWriteTextW');
   Result := pdfWriteTextW(FInstance, PosX, PosY, PWideChar(AText));
end;

function TPDF.WriteTextExA(PosX, PosY: Double; const AText: AnsiString; Len: Cardinal): Boolean;
begin
   LoadFunction(@pdfWriteTextExA, 'pdfWriteTextExA');
   Result := pdfWriteTextExA(FInstance, PosX, PosY, PAnsiChar(AText), Len);
end;

function TPDF.WriteTextExW(PosX, PosY: Double; const AText: WideString; Len: Cardinal): Boolean;
begin
   LoadFunction(@pdfWriteTextExW, 'pdfWriteTextExW');
   Result := pdfWriteTextExW(FInstance, PosX, PosY, PWideChar(AText), Len);
end;

function TPDF.WriteTextMatrixA(var Matrix: TCTM; const AText: AnsiString): Boolean;
begin
   LoadFunction(@pdfWriteTextMatrixA, 'pdfWriteTextMatrixA');
   Result := pdfWriteTextMatrixA(FInstance, Matrix, PAnsiChar(AText));
end;

function TPDF.WriteTextMatrixW(var Matrix: TCTM; const AText: WideString): Boolean;
begin
   LoadFunction(@pdfWriteTextMatrixW, 'pdfWriteTextMatrixW');
   Result := pdfWriteTextMatrixW(FInstance, Matrix, PWideChar(AText));
end;

function TPDF.WriteTextMatrixExA(var M: TCTM; const AText: AnsiString; Len: Cardinal): Boolean;
begin
   LoadFunction(@pdfWriteTextMatrixExA, 'pdfWriteTextMatrixExA');
   Result := pdfWriteTextMatrixExA(FInstance, M, PAnsiChar(AText), Len);
end;

function TPDF.WriteTextMatrixExW(var M: TCTM; const AText: WideString; Len: Cardinal): Boolean;
begin
   LoadFunction(@pdfWriteTextMatrixExW, 'pdfWriteTextMatrixExW');
   Result := pdfWriteTextMatrixExW(FInstance, M, PWideChar(AText), Len);
end;

end.
