Option Strict On
Option Explicit On
Imports System.Runtime.InteropServices

Friend Class CPDF

   '----------------------------------------------------------+
   ' DynaPDF 2.5                                              |
   '----------------------------------------------------------+
   ' Copyright (C) 2003-2011                                  |
   ' Jens Boschulte, DynaForms GmbH.                          |
   ' All rights reserved.                                     |
   '----------------------------------------------------------+
   ' Please report errors or other potential problems to      |
   ' support@dynaforms.com.                                   |
   ' The actual version is available at www.dynaforms.com.    |
   '----------------------------------------------------------+
   '             Creation date: February 8, 2011              |
   '----------------------------------------------------------+

   ' Event procedures
   Public Event PDFEnumDocFont(ByVal PDFFont As IntPtr, ByVal FontType As TFontType, ByVal BaseFont As String, ByVal FontName As String, ByVal Embedded As Boolean, ByVal IsFormFont As Boolean, ByVal Flags As Integer, ByRef DoBreak As Boolean)
   Public Event PDFEnumFont(ByVal FamilyName As String, ByRef PostScriptName As String, ByVal Style As Integer, ByRef DoBreak As Boolean)
   Public Event PDFEnumFontEx(ByVal FamilyName As String, ByVal PostScriptName As String, ByVal Style As Integer, ByVal BaseType As TFontBaseType, ByVal Embeddable As Boolean, ByVal FilePath As String, ByRef DoBreak As Boolean)
   Public Event PDFError(ByVal Description As String, ByVal ErrType As Integer, ByRef DoBreak As Boolean)
   Public Event PDFInitProgress(ByVal ProgType As Integer, ByVal MaxCount As Integer)
   Public Event PDFPageBreak(ByVal LastPosX As Double, ByVal LastPosY As Double, ByVal PageBreak As Integer, ByRef NewAlign As TNewAlign, ByRef DoBreak As Boolean)
   Public Event PDFProgress(ByVal ActivePage As Integer, ByRef DoBreak As Boolean)

   ' ------------------------------------------------- Default types -----------------------------------------------

   ' Error types
   Public Const E_WARNING As Integer = &H2000000
   Public Const E_SYNTAX_ERROR As Integer = &H4000000
   Public Const E_VALUE_ERROR As Integer = &H8000000
   Public Const E_FONT_ERROR As Integer = &H10000000
   Public Const E_FATAL_ERROR As Integer = &H20000000
   Public Const E_FILE_ERROR As Integer = &H40000000

   ' Specific error codes to determine whether the supplied to decrypt the input file password was wrong
   Public Const ENEED_PWD As Integer = -CLng(&HB2S Or E_FILE_ERROR)
   Public Const EWRONG_OPEN_PWD As Integer = -CLng(&HB3S Or E_FILE_ERROR)
   Public Const EWRONG_OWNER_PWD As Integer = -CLng(&HB4S Or E_FILE_ERROR)
   Public Const EWRONG_PWD As Integer = -CLng(&HB5S Or E_FILE_ERROR)

   Public Const PDF_MAX_INT As Integer = &H7FFFFFFF
   ' Basic RGB colors
   Public Const PDF_AQUA As Integer = &HFFFF00
   Public Const PDF_BLACK As Integer = &H0
   Public Const PDF_BLUE As Integer = &HFF0000
   Public Const PDF_CREAM As Integer = &HF0FBFF
   Public Const PDF_DKGRAY As Integer = &H808080
   Public Const PDF_FUCHSIA As Integer = &HFF00FF
   Public Const PDF_GRAY As Integer = &H808080
   Public Const PDF_GREEN As Integer = &H8000
   Public Const PDF_LIME As Integer = &HFF00
   Public Const PDF_LTGRAY As Integer = &HC0C0C0
   Public Const PDF_MAROON As Integer = &H80
   Public Const PDF_MEDGRAY As Integer = &HA4A0A0
   Public Const PDF_MOGREEN As Integer = &HC0DCC0
   Public Const PDF_NAVY As Integer = &H800000
   Public Const PDF_OLIVE As Integer = &H8080
   Public Const PDF_PURPLE As Integer = &H800080
   Public Const PDF_RED As Integer = &HFF
   Public Const PDF_SILVER As Integer = &HC0C0C0
   Public Const PDF_SKYBLUE As Integer = &HF0CAA6
   Public Const PDF_WHITE As Integer = &HFFFFFF
   Public Const PDF_TEAL As Integer = &H808000
   Public Const PDF_YELLOW As Integer = &HFFFF
   Public Const NO_COLOR As Integer = &HFFFFFFF1 ' Transparent color used by annotaions and form fields

   ' Specific return values of the OnPageBreak callback function
   Public Const NEW_ALIGN_LEFT As Integer = 1
   Public Const NEW_ALIGN_RIGHT As Integer = 2
   Public Const NEW_ALIGN_CENTER As Integer = 3
   Public Const NEW_ALIGN_JUSTIFY As Integer = 4

   ' Use these masks to determine which viewer preference values are defined.
   Public Const AV_NON_FULL_SRC_MASK As Integer = &H5S
   Public Const AV_DIRECTION_MASK As Integer = &H18S
   Public Const AV_VIEW_PRINT_MASK As Integer = &H3E0S

   Enum TErrMode
      emIgnoreAll = 0
      emSyntaxError = 1
      emValueError = 2
      emWarning = 4
      emFileError = 8
      emFontError = 16
      emAllErrors = 65535
      emNoFuncNames = &H10000000 ' Do not print function names in error messages
   End Enum

   Enum T3DActivationType
      at3D_AppDefault
      at3D_PageOpen    ' The annotaiton should be activated when the page is opened.
      at3D_PageVisible ' The annotaiton should be activated when the page becomes visible.
      at3D_Explicit    ' The annotation should remain inactive until explicitely activated by a script or action (default).
   End Enum

   Enum T3DDeActivateType
      dt3D_AppDefault
      dt3D_PageClosed    ' The annotaiton should be deactivated as soon as the page is closed.
      dt3D_PageInvisible ' The annotaiton should be deactivated as soon as the page becomes invisible (default).
      dt3D_Explicit      ' The annotation should remain active until explicitely deactivated by a script or action.
   End Enum

   Enum T3DDeActInstance
      di3D_AppDefault
      di3D_UnInstantiated ' The annotation will be uninstantiated (default)
      di3D_Instantiated   ' The annotation is left instantiated
      di3D_Live           ' If the 3D artwork contains an animation then it will stay live
   End Enum

   Enum T3DInstanceType
      it3D_AppDefault
      it3D_Instantiated ' The annotation will be instantiated but animations are disabled.
      it3D_Live         ' The annotation will be instantiated and animations are enabled (default).
   End Enum

   Enum T3DLightingSheme
      lsArtwork
      lsBlue
      lsCAD
      lsCube
      lsDay
      lsHard
      lsHeadlamp
      lsNight
      lsNoLights
      lsPrimary
      lsRed
      lsWhite
      lsNotSet
   End Enum

   Enum T3DNamedAction
      naDefault
      naFirst
      naLast
      naNext
      naPrevious
   End Enum

   Enum T3DProjType
      pt3DOrthographic
      pt3DPerspective
   End Enum

   Enum T3DRenderingMode
      rmBoundingBox
      rmHiddenWireframe
      rmIllustration
      rmShadedIllustration
      rmShadedVertices
      rmShadedWireframe
      rmSolid
      rmSolidOutline
      rmSolidWireframe
      rmTransparent
      rmTranspBBox
      rmTranspBBoxOutline
      rmTranspWireframe
      rmVertices
      rmWireframe
      rmNotSet
   End Enum

   Enum T3DScaleType
      st3DValue
      st3DWidth
      st3DHeight
      st3DMin
      st3DMax
   End Enum

   Enum TActionType
      atGoTo
      atGoToR
      atHide
      atImportData
      atJavaScript
      atLaunch
      atMovie
      atNamed
      atRendition
      atReset
      atSetOCGState
      atSound
      atSubmit
      atThread
      atTransition
      atURI
      atGoTo3DView
   End Enum

   Enum TAnnotColor
      acBackColor
      acBorderColor
      acTextColor
   End Enum

   Enum TAnnotFlags
      afNone = &H0
      afInvisible = &H1
      afHidden = &H2
      afPrint = &H4
      afNoZoom = &H8
      afNoRotate = &H10
      afNoView = &H20
      afReadOnly = &H40
   End Enum

   'By default all annotations which have an appearance stream and which have the print flag set are flattened.
   'All annotations are deleted when the function returns with the exception of file attachment annotations.
   'If you want to flatten the view state then set the flag affUseViewState.
   Enum TAnnotFlattenFlags
      affNone = &H0         ' Printable annotations independent of the type
      affUseViewState = &H1 ' If set, annotations which are visible in a viewer become flattened.
      affMarkupAnnots = &H2 ' If set, markup annotations are flattened only. Link, Sound, or FileAttach
                            ' annotations are no markup annotations. These types will be left intact.
   End Enum

   Enum TAnnotIcon
      aiComment
      aiHelp
      aiInsert
      aiKey
      aiNewParagraph
      aiNote
      aiParagraph
   End Enum

   Enum TAnnotString
      asAuthor
      asContent
      asName
      asSubject
   End Enum

   Enum TAnnotType
      atCaret
      atCircle
      atFileLink
      atFreeText
      atHighlight
      atInk
      atLine
      atPageLink
      atPolygon
      atPolyLine
      atPopUp
      atSquare
      atSquiggly
      atStamp
      atStrikeOut
      atText
      atUnderline
      atWebLink
      atWidget
      at3D
      atSoundAnnot
      atFileAttach
      atRedact
      atWatermark
      atUnknown    ' Unknown annotation type
   End Enum

   Enum TBlendMode
      bmNotSet
      bmNormal
      bmColor
      bmColorBurn
      bmColorDodge
      bmDarken
      bmDifference
      bmExclusion
      bmHardLight
      bmHue
      bmLighten
      bmLuminosity
      bmMultiply
      bmOverlay
      bmSaturation
      bmScreen
      bmSoftLight
   End Enum

   Enum TBorderStyle
      bsSolid
      bsBevelled
      bsInset
      bsUnderline
      bsDashed
   End Enum

   Enum TButtonState
      bsUp
      bsDown
      bsRollOver
   End Enum

   Enum TCheckBoxChar
      ccCheck
      ccCircle
      ccCross1
      ccCross2
      ccCross3
      ccCross4
      ccDiamond
      ccSquare
      ccStar
   End Enum

   Enum TCheckBoxState
      cbUnknown
      cbChecked
      cbUnChecked
   End Enum

   Enum TCheckOptions
      coDefault = &HFFF
      coEmbedSubsets = &H1
      coDeleteTransferFuncs = &H2
      coDeleteMultiMediaContents = &H4
      coDeleteActionsAndScripts = &H8
      coDeleteInvRenderingIntent = &H10
      coFlattenFormFields = &H20
      coReplaceV4ICCProfiles = &H40
      coDeleteEmbeddedFiles = &H80
      coDeleteOPIComments = &H100
      coDeleteSignatures = &H200
      coReplCCITTFaxWithFlate = &H10000
      coNoFontEmbedding = &H10000000
   End Enum

   Enum TClippingMode
      cmEvenOdd
      cmWinding
   End Enum

   Enum TCodepage
      cp1250
      cp1251
      cp1252
      cp1253
      cp1254
      cp1255
      cp1256
      cp1257
      cp1258
      cp8859_2
      cp8859_3
      cp8859_4
      cp8859_5
      cp8859_6
      cp8859_7
      cp8859_8
      cp8859_9
      cp8859_10
      cp8859_13
      cp8859_14
      cp8859_15
      cp8859_16
      cpSymbol
      cp437
      cp737
      cp775
      cp850
      cp852
      cp855
      cp857
      cp860
      cp861
      cp862
      cp863
      cp864
      cp865
      cp866
      cp869
      cp874
      cpUnicode
      cpCJK_Big5_Uni    ' Big5 plus HKSCS extension.
      cpCJK_EUC_JP_Uni  ' EUC-JP
      cpCJK_EUC_KR_Uni  ' EUC-KR
      cpCJK_EUC_TW_Uni  ' CNS-11643-1992 (Planes 1-15).
      cpCJK_GBK_Uni     ' GBK is the Microsoft code page 936 (GB2312 EUC-CN plus GBK extension).
      cpCJK_GB12345_Uni ' GB-12345-1990 (Traditional Chinese form of GB-2312).
      cpCJK_HZ_Uni      ' Mixed ASCII / GB-2312 encoding
      cpCJK_2022_CN_Uni ' ISO-2022-CN-EXT (GB-2312 plus ISO-11643 Planes 1-7).
      cpCJK_2022_JP_Uni ' ISO-2022-JP
      cpCJK_2022_KR_Uni ' ISO-2022-KR
      cpCJK_646_CN_Uni  ' ISO-646-CN (GB-1988-80)
      cpCJK_646_JP_Uni  ' ISO-646-JP (JIS_C6220-1969-RO).
      cpCJK_IR_165_Uni  ' ISO-IR-165 (extended version of GB-2312).
      cpCJK_932_Uni     ' Microsoft extended version of SHIFT_JIS.
      cpCJK_949_Uni     ' EUC-KR extended with UHC (Unified Hangul Codes).
      cpCJK_950_Uni     ' Microsoft extended version of Big5.
      cpCJK_JOHAB_Uni   ' JOHAB
      cpShiftJIS        ' ShiftJIS charset plus code page 932 ectension.
      cpBig5            ' Big5 plus HKSCS extension.
      cpGB2312          ' GB2312 charset plus GBK and cp936 extension.
      cpWansung         ' Wansung
      cpJohab           ' JOHAB
      cpMacRoman        ' Mac Roman
      cpAdobeStd        ' This is an encoding for Type1 fonts. It should normally not be used.
      cpInternal        ' Internal -> not usable
      cpGlyphIndexes    ' Can be used with TrueType and OpenType fonts only. DynaPDF creates a reverse mapping so that copy & paste will work.
      cpPDFDocEnc       ' Internal -> not usable. Used for form fonts. This is a superset of the code page 1252 and MacRoman.
      cpExtCMap         ' Internal -> not usable. This code page is set when a font was loaded with an external cmap.
      cpDingbats        ' Internal -> Special encoding for ZapfDingbats
   End Enum

   ' The data for user defined columns is stored in collection items.
   Enum TColColumnType
      cisCreationDate ' Data comes from the embedded file
      cisDescription  ' Data comes from the embedded file
      cisFileName     ' Data comes from the embedded file
      cisModDate      ' Data comes from the embedded file
      cisSize         ' Data comes from the embedded file
      cisCustomDate   ' User defined date.
      cisCustomNumber ' User defined nummber.
      cisCustomString ' User defined string.
   End Enum

   Enum TColorConvFlags
   ccfBW_To_Gray = 0   ' Default, RGB Black and White set with rg or RG inline operators are converted to gray
   ccfRGB_To_Gray = 1  ' If set, inline color operators rg and RG are converted to gray
   End Enum

   Enum TColorMode
      cmFill
      cmStroke
      cmFillStroke
   End Enum

   Enum TColView
      civNotSet
      civDetails
      civTile
      civHidden
   End Enum

   Enum TCompBBoxFlags
      cbfNone = 0
      cbfIgnoreWhiteAreas = 1 ' Ignore white vector graphics or text.
      ' Please note that images must be decompressed if one of the following flags are set. Parsing gray or color images
      ' is in most cases not useful and you should not parse such images if it is not really required.
      cbfParse1BitImages = 2  ' Find the visible area in 1 bit images. This is the most important case
                              ' since scanned faxes are usually 1 bit images.
      cbfParseGrayImages = 4  ' Find the visible area in gray images.
      cbfParseColorImages = 8 ' Find the visible area in color images. This is usually not required
                              ' and slow downs processing a lot.
      cbfParseAllImages = 14  ' Find the visible area in all images.
   End Enum

   Enum TCompressionLevel
      clNone
      clDefault
      clFastest
      clMax
   End Enum

   Enum TCompressionFilter
      cfFlate = 0    ' Flate or Zip compression
      cfJPEG = 1     ' JPEG compression
      cfCCITT3 = 2   ' CCITT Fax G3 compression
      cfCCITT4 = 3   ' CCITT Fax G4 compression
      cfLZW = 4      ' TIFF or GIF output -> Very fast but less compression ratios than flate
      cfReserved = 5 ' Reserved for future extensions.
      cfFlateBW = 6  ' TIFF, PNG, or BMP output -> Dithered black & white output. The resulting image will be
                     ' compressed with Flate or left uncompressed if the output image format is a bitmap. If
                     ' you want to use CCITT Fax 4 compression (TIFF only) set the flag icUseCCITT4 in the
                     ' AddImage() function call. Note that this filter is not supported for PDF creation!
      cfJP2K = 7     ' PDF or JPEG2000 output
   End Enum

   Enum TConformanceType
      ctPDFA_1b_2005
      ctNormalize
   End Enum

   Enum TDecodeFilter
      dfNone
      dfASCII85Decode   ' No parameters
      dfASCIIHexDecode  ' No parameters
      dfCCITTFaxDecode  ' Optional Parameters
      dfDCTDecode       ' Optional Parameters
      dfFlateDecode     ' Optional Parameters
      dfJBIG2Decode     ' Optional Parameters
      dfJPXDecode       ' No parameters
      dfLZWDecode       ' Optional Parameters
      dfRunLengthDecode ' No parameters
   End Enum

   Enum TDecSeparator
      ' per thousand separator, decimal separator
      dsCommaDot
      dsNoneDot
      dsDotComma
      dsNoneComma
   End Enum

   Enum TDestType
      dtXY_Zoom ' three parameters (a, b, c) -> (X, Y, Zoom)
      dtFit ' no parameters
      dtFitH_Top ' one parameter    (a)
      dtFitV_Left ' one parameter    (a)
      dtFit_Rect ' four parameters  (a, b, c, d) -> (left, bottom, right, top)
      dtFitB ' no parameter
      dtFitBH_Top ' one parameter    (a)
      dtFitBV_Left ' one parameter    (a)
   End Enum

   Enum TDocumentInfo
      diAuthor
      diCreator
      diKeywords
      diProducer
      diSubject
      diTitle
      diCompany
      diPDFX_Ver ' GetInDocInfo() only -> The PDF/X version is set by SetPDFVersion()!
      diCustom   ' User defined key
   End Enum

   Enum TDrawDirection
      ddCounterClockwise
      ddClockwise
   End Enum

   Enum TDrawMode
      dmNormal
      dmStroke
      dmFillStroke
      dmInvisible
      dmFillClip
      dmStrokeClip
      dmFillStrokeClip
      dmClipping
   End Enum

   Enum TDuplexMode
      dpmNone          ' Default
      dpmSimplex
      dpmFlipShortEdge
      dpmFlipIntegerEdge
   End Enum

   Enum TExtColorSpace
      esDeviceRGB   ' Device color space
      esDeviceCMYK  ' Device color space
      esDeviceGray  ' Device color space
      esCalGray     ' CIE-based color space
      esCalRGB      ' CIE-based color space
      esLab         ' CIE-based color space
      esICCBased    ' CIE-based color space -> contains an ICC profile
      esPattern     ' Special color space
      esIndexed     ' Special color space
      esSeparation  ' Special color space
      esDeviceN     ' Special color space
      esNChannel    ' Special color space
   End Enum

   Enum TFieldColor
      fcBackColor
      fcBorderColor
      fcTextColor
   End Enum

   Enum TFieldFlags
      ffReadOnly = &H1
      ffRequired = &H2
      ffNoExport = &H4

      ffInvisible = &H8
      ffHidden = &H10
      ffPrint = &H20
      ffNoZoom = &H40
      ffNoRotate = &H80
      ffNoView = &H100

      ffMultiline = &H1000         ' Text fields only
      ffPassword = &H2000          ' Text fields only
      ffNoToggleToOff = &H4000     ' Radio buttons
      ffRadioIsUnion = &H2000000   ' PDF-1.5 radio buttons only
      ffCommitOnSelCh = &H4000000  ' PDF-1.5 combo boxes, list boxes

      ffEdit = &H40000             ' Combo boxes only
      ffSorted = &H80000           ' Do not use this flag CreateComboBox or CreateListBox set this flag automatically
      ffFileSelect = &H100000      ' PDF 1.4 Text fields only
      ffMultiSelect = &H200000     ' PDF 1.4 List boxes only
      ffDoNotSpellCheck = &H400000 ' PDF 1.4 Text fields, combo boxes. If the field is a combo box, this flag is meaningful only if ffEdit is also set.
      ffDoNotScroll = &H800000     ' PDF 1.4 Text fields only
      ffComb = &H1000000
   End Enum

   Enum TFieldType
      ftButton
      ftCheckBox
      ftRadioBtn
      ftComboBox
      ftListBox
      ftText
      ftSignature
      ftGroup ' this is not a real field type, it is just an array of fields
   End Enum

   Enum TFileAttachIcon
      faiGraph
      faiPaperClip
      faiPushPin
      faiTag
   End Enum

   Enum TFileOP
      foOpen
      foPrint
   End Enum

   Enum TFontBaseType
      fbtTrueType ' TrueType, TrueType Collections, or OpenType fonts with TrueType outlines
      fbtType1    ' Type1 font
      fbtOpenType ' OpenType font with Postscript outlines
      fbtStdFont  ' PDF Standard font
      fbtDisabled ' This value can be used to disable a specific font format. See SetFontSearchOrder() for further information.
   End Enum

   Enum TFontFileSubtype
      ffsType1C        ' CFF based Type1 font
      ffsCIDFontType0C ' CFF based Type1 CID font
      ffsOpenType      ' TrueType based OpenType font
      ffsOpenTypeC     ' CFF based OpenType font
      ffsCIDFontType2  ' TrueType based CID Font
      ffsReserved1
      ffsReserved2
      ffsReserved3
      ffsReserved4
   End Enum

   Enum TFontSelMode
      smFamilyName
      smPostScriptName
   End Enum

   Enum TFontType
      ftMMType1
      ftTrueType
      ftType0    ' Check the font file type to determine the font sub type
      ftType1
      ftType3
   End Enum

   'The font search run works as follows:

   '   - DynaPDF tries always to find the exact weight, if it cannot be found then a font with
   '     the next smaller weight is selected (if available).
   '   - Italic styles can always be emulated but it is not possible to emulate thinner weights or
   '     regular styles with an italic font.
   '   - If the specified weight is larger as the font weight the remaining weight will be emulated
   '     if the difference to the requested weight is larger than 200.
   '     With SetFontWeight() it is possible to control whether a missing weight should be emulated.
   '     If FontWeight is smaller or equal to the requested font weight then emulation will be disabled.

   'TFStyle is a bitmask that is defined as follows:

   '   - Bits 0..7   // Style bits fsItalic, fsUnderlined, fsStriked
   '   - Bits 8..19  // Width class -> Defined for future use.
   '   - Bits 20..31 // Font Weight

   '- A width class can be converted to a style constant by multiplying it with 256 (width shl 8).
   '- A font weight can be converted to a style constant by multiplying it with 1048576 (weight shl 20).
   '- Additional attributes can be added with a binary or operator (e.g. style or fsItalic).
   '- Only one width class and one font weight can be set at time.

   '- WidthFromStyle() extracts the width class.
   '- WeightFromStyle() extracts the font weight.

   'The following functions extract the width class or font weight from a style variable:

   '   widthClass  = WidthFromStyle(style);
   '   weightClass = WeightFromStyle(style);

   Enum TFStyle
      fsNone = &H0                  ' Regular weight (400)
      fsItalic = &H1
      fsUnderlined = &H4
      fsStriked = &H8
      fsVerticalMode = &H10         ' Not considered at this time
      ' Width class
      fsUltraCondensed = &H100      ' 1
      fsExtraCondensed = &H200      ' 2
      fsCondensed = &H300           ' 3
      fsSemiCondensed = &H400       ' 4
      fsNormal = &H500              ' 5
      fsSemiExpanded = &H600        ' 6
      fsExpanded = &H700            ' 7
      fsExtraExpanded = &H800       ' 8
      fsUltraExpanded = &H900       ' 9
      ' Weight class
      fsThin = &H6400000            ' 100
      fsExtraLight = &HC800000      ' 200
      fsLight = &H12C00000          ' 300
      fsRegular = &H19000000        ' 400 -> Same as fsNone
      fsMedium = &H1F400000         ' 500
      fsDemiBold = &H25800000       ' 600
      fsBold = &H2BC00000           ' 700 -> The old constant 2 is still supported to preserve backward compatibility
      fsExtraBold = &H32000000      ' 800
      fsBlack = &H38400000          ' 900
      fsUltraBlack = &H3E800000     ' 1000
   End Enum

   Enum TGStateFlags
      gfCompatible = 0         ' Compatible graphics state to earlier DynaPDF versions -> default
      gfRestorePageCoords = 1  ' Restore the coordinate system with the graphics state (the value of PageCoords, see SetPageCoords())
      gfRealTopDownCoords = 2  ' If set, the page coordinate system is not reset to bottom-up when transforming
                               ' the coordinate system. However, real top-down coordinates require a large internal
                               ' overhead and where never fully implemented. The usage of this flag should be avoided
                               ' if possible.
      gfUseImageColorSpace = 8 ' If set, the active color space is ignored when inserting an image. The color space is taken
                               ' from the image file instead.
      gfIgnoreICCProfiles = 16 ' Meaningful only if the flag gfUseImageColorSpace is set. If set, an embedded profile is not used to
                               ' create an ICCBased color space for the image. The image is inserted in the corresponding device
                               ' color space instead.
   End Enum

   Enum THashType
      htDetached ' CloseAndSignFileExt() returns the byte ranges of the finish PDF buffer to create a detached signature
      htSHA1     ' CloseAndSignFileExt() returns the SHA1 hash of the PDF file so that it can be signed
   End Enum

   Enum THighlightMode
      hmNone
      hmInvert
      hmOutline
      hmPush
      hmPushUpd ' Update appereance stream on changes
   End Enum

   Enum TICCProfileType
      ictGray
      ictRGB
      ictCMYK
      ictLab
   End Enum

   'TIFF is the only format that supports different compression filters. The Filter parameter of the function
   'AddImage() is ignored if the image format supports only one specific compression filter.
   'Note that images are automatically converted to the nearest supported color space if the image format does
   'not support the color space of the image.
   Enum TImageFormat
      ifmTIFF     ' DeviceRGB, DeviceCMYK, DeviceGray, Black & White -> CCITT Fax Group 3/4, JPEG, Flate, LZW.
      ifmJPEG     ' DeviceRGB, DeviceCMYK, DeviceGray    -> JPEG compression.
      ifmPNG      ' DeviceGray, DeviceRGB, Black & White -> Flate compression.
      ifmReserved ' Reserved for future extensions.
      ifmBMP      ' DeviceGray, DeviceRGB, Black & White -> Uncompressed.
      ifmJPC      ' DeviceRGB, DeviceCMYK, DeviceGray    -> JPEG2000 compression.
   End Enum

   Enum TImageConversionFlags
      icNone      ' Default
      icUseCCITT4 ' Use CCITT Fax 4 compression instead of Flate for dithered images.
   End Enum

   Enum TImportFlags
      ifImportAll = &HFFFFFFE ' default
      ifContentOnly = &H0
      ' If this flag is set, only interactive objects are imported if any, Otherwise only empty pages are imported.
      ' This flag can be used to copy an interactive form to another PDF file.
      ifNoContent = &H1
      ' The imported page is not converted to a template if ifImportAsPage is set.
      ' Note that this flag can cause resource conflicts. Use this flag carefully!
      ifImportAsPage = &H80000000
      ' base objects
      ifCatalogAction = &H2
      ifPageActions = &H4
      ifBookmarks = &H8
      ifArticles = &H10
      ifPageLabels = &H20
      ifThumbs = &H40
      ifTranspGroups = &H80        ' This flag is not Integerer considered.
      ifSeparationInfo = &H100
      ifBoxColorInfo = &H200
      ifStructureTree = &H400
      ifTransition = &H800
      ifSearchIndex = &H1000
      ifJavaScript = &H2000
      ifJSActions = &H4000
      ifDocInfo = &H8000           ' Document info entries
      ifEmbeddedFiles = &H200000   ' File attachments
      ifFileCollections = &H400000 ' File collections (PDF 1.7)
      ' Annotations -> Only the most important annotation types can be selected directly.
      ' Note that all annotation types can be deleted with DeleteAnnotation.
      ifAllAnnots = &H9F0000
         ifFreeText = &H10000
         ifTextAnnot = &H20000
         ifLink = &H40000
         ifStamp = &H80000
         if3DAnnot = &H100000
         ifOtherAnnots = &H800000
      ' Interactive Form fields are also annotations but we handle this type separately!
      ifFormFields = &H1000000
   ' -------------------- Special flags --------------------
      ifPrepareForPDFA = &H10000000 ' Replace LZW compression with Flate, set the Interpolate key of images to false, do not import embedded files.
      ifEnumFonts = &H20000000      ' Import fonts for EnumDocFonts(). The document must be deleted when this flag is set!!!
      ifAllPageObjects = &H40000000 ' Import links when using ImportPageEx() within an open page. The entire document should be imported in this case.
   End Enum

   Enum TImportFlags2
      if2MergeLayers = &H1          ' If set, layers with identical name are merged. If this flag is absent DynaPDF
                                    ' imports such layers separately so that each layer refers still to the pages
                                    ' where it was orignally used.
      if2Normalize = &H2            ' Replace LZW compression with Flate, apply limit checks, repair errors if possible.
   End Enum

   Enum TKeyLen
      kl40bit    ' RC4 Encryption -> Acrobat 3 or higher
      kl128bit   ' RC4 Encryption -> Acrobat 5 or higher
      kl128bitEx ' RC4 Encryption -> Acrobat 6 or higher
      klAES128   ' AES Encryption -> Acrobat 7 or higher
      klAES256   ' AES Encryption -> Acrobat 9 or higher
   End Enum

   Enum TLineCapStyle
      csButtCap
      csRoundCap
      csSquareCap
   End Enum

   Enum TLineJoinStyle
      jsMiterJoin
      jsRoundJoin
      jsBevelJoin
   End Enum

   Enum TMetaFlags
      mfDefault = 0                   ' Default conversion
      mfDebug = 1                     ' Write debug information into the content stream
      mfShowBounds = 2                ' Show the bounding boxes of text strings
      mfNoTextScaling = 4             ' Do not scale text
      mfClipView = 8                  ' Draw the file into a clipping rectangle
      mfUseRclBounds = 16             ' Use rclBounds instead of rclFrame
      mfNoClippingRgn = 64            ' Disables SelectClippingRegion and IntersectClipRect
      mfNoFontEmbedding = 128         ' Do not embed fonts -> Fonts should be embedded!!!
      mfNoImages = 256                ' Ignore image records
      mfNoStdPatterns = 512           ' Ignore standard patterns
      mfNoBmpPatterns = 1024          ' Ignore bitmap patterns
      mfNoText = 2048                 ' Ignore text records
      mfUseUnicode = 4096             ' Ignore ANSI_CHARSET
      mfUseTextScaling = 16384        ' Scale text instead of using the intercharacter spacing array
      mfNoUnicode = 32768             ' Avoid usage of Unicode fonts -> recommended to enable PDF 1.2 compability
      mfFullScale = 65536             ' Scale coordinates to the window size. Recommended if 32 bit coordinates are used.
      mfUseRclFrame = 131072          ' This flag should be set if the rclFrame rectangle is properly set
      mfDefBkModeTransp = 262144      ' Initialize the background mode to transparent (SetBkMode() overrides this state).
      mfApplyBidiAlgo = &H80000       ' Apply the bidirectional algorithm on Unicode strings
      mfGDIFontSelection = &H100000   ' Use the GDI to select fonts
      mfRclFrameEx = &H200000         ' If set, and if the rclBounds rectangle is larger than rclFrame, the function
                                      ' extends the output rectangle according to rclBounds and uses the resulting
                                      ' bounding box to calculate the image size (rclBounds represents the unscaled
                                      ' image size). This is probably the correct way to calculate the image size.
                                      ' However, to preserve backward compatibility the default calculation cannot
                                      ' be changed.
      mfNoTextClipping = &H400000     ' If set, the ETO_CLIPPED flag in text records is ignored.
      mfSrcCopy_Only = &H800000       ' If set, images which use a ROP code other than SRCCOPY are ignored. This is useful when processing Excel 2007 spool files.
      mfClipRclBounds = &H1000000     ' If set, the graphic is drawn into a clipping path with the size of rclBounds.
                                      ' This flag is useful if the graphic contains content outside of its bounding box.
      mfDisableRasterEMF = &H2000000  ' If set, EMF files which use unsupported ROP codes are not rastered.
      ' Obsolete flags -> these flags are ignored, do no Integerer use them!
      mfUseSpacingArray = 32          ' enabled by default -> can be disabled with mfUseTextScaling
      mfIntersectClipRect = 8192      ' enabled by default -> can be disabled with mfNoClippingRgn
   End Enum

   Enum TNamedAction
      naFirstPage
      naLastPage
      naNexPage
      naPrevPage
      naGoBack
      naOpenDlg
      naPrintDlg
      naGeneralInfo
      naFontsInfo
      naSaveAs
      naSecurityInfo
      naFitPage
      naFullScreen
      naDeletePages
      naQuit
   End Enum

   Enum TNegativeStyle
      nsMinusBlack
      nsRed
      nsParensBlack
      nsParensRed
   End Enum

   Enum TNewAlign
      naUnchanged = 0
      naLeft = 1
      naCenter = 2
      naRight = 3
      naJustify = 4
   End Enum

   ' All actions which should be applied to an event except On Mouse Upend Enum must be a JavaScript action!
   Enum TObjEvent
      oeNoEvent          ' Internal use only -> DO NOT USE THIS VALUE!!!
      oeOnOpen           ' Catalog Pages
      oeOnClose          ' Pages only
      oeOnMouseUp        ' All fields page link annotations bookmarks
      oeOnMouseEnter     ' Form fields only
      oeOnMouseExit      ' Form fields only
      oeOnMouseDown      ' Form fields only
      oeOnFocus          ' Form fields only
      oeOnBlur           ' Form fields only
      oeOnKeyStroke      ' Text fields only
      oeOnFormat         ' Text fields only
      oeOnCalc           ' Text fields combo boxes list boxes
      oeOnValidate       ' All form fields except buttons
      oeOnPageVisible    ' PDF 1.5 -> Form fields only
      oeOnPageInVisible  ' PDF 1.5 -> Form fields only
      oeOnPageOpen       ' PDF 1.5 -> Form fields only
      oeOnPageClose      ' PDF 1.5 -> Form fields only
      oeOnBeforeClosing  ' PDF 1.4 -> Catalog only
      oeOnBeforeSaving   ' PDF 1.4 -> Catalog only
      oeOnAfterSaving    ' PDF 1.4 -> Catalog only
      oeOnBeforePrinting ' PDF 1.4 -> Catalog only
      oeOnAfterPrinting  ' PDF 1.4 -> Catalog only
   End Enum

   Enum TObjType
      otAction
      otAnnotation
      otBookmark
      otCatalog ' PDF 1.4
      otField
      otPage
      otPageLink
   End Enum

   Enum TOCObject
      ooAnnotation
      ooField
      ooImage
      ooTemplate
   End Enum

   Enum TOCGIntent
      oiDesign = 2
      oiView = 4   ' Default
      oiAll = 8
      oiEmpty = 16 ' Internal
   End Enum

   Enum TOCVisibility
      ovAllOff
      ovAllOn
      ovAnyOff
      ovAnyOn
      ovNotSet ' Internal
   End Enum

   Enum TOrigin
      orDownLeft
      orTopLeft
   End Enum

   Enum TPageCoord
      pcBottomUp
      pcTopDown
   End Enum

   Enum TPageFormat
      pfDIN_A3
      pfDIN_A4
      pfDIN_A5
      pfDIN_B4
      pfDIN_B5
      pfDIN_B6
      pfDIN_C3
      pfDIN_C4
      pfDIN_C5
      pfDIN_C6
      pfDIN_C65
      pfDIN_DL
      pfDIN_E4
      pfDIN_E5
      pfDIN_E6
      pfDIN_E65
      pfDIN_M5
      pfDIN_M65
      pfUS_Legal
      pfUS_Letter
   End Enum

   Enum TPageBoundary
      pbArtBox
      pbBleedBox
      pbCropBox
      pbTrimBox
      pbMediaBox
   End Enum

   Enum TPageLabelFormat
      plfDecimalArabic    ' 1,2,3,4...
      plfUppercaseRoman   ' I,II,III,IV...
      plfLowercaseRoman   ' i,ii,iii,iv...
      plfUppercaseLetters ' A,B,C,D...
      plfLowercaseLetters ' a,b,c,d...
      plfNone
   End Enum

   Enum TPageLayout
      plSinglePage
      plOneColumn
      plTwoColumnLeft
      plTwoColumnRight
      plTwoPageLeft
      plTwoPageRight
      plDefault        ' Use viewer's default settings
   End Enum

   Enum TPageMode
      pmUseNone
      pmUseOutLines
      pmUseThumbs
      pmFullScreen
      pmUseOC           ' Optional Content (Layers)
      pmUseAttachments
   End Enum

   Enum TParseFlags
      pfNone = &H0
      pfDecomprAllImages = &H2 ' This flag causes that all image formats will be decompressed
                               ' with the exception of JBIG2 compressed images. If this flag is
                               ' not set, images which are already stored in a valid image file
                               ' format are returned as is. This is the case for Gray and RGB JPEG
                               ' images and for JPEG2000 images. If you want to extract the images
                               ' of a PDF file this flag should NOT be set!

      pfNoJPXDecode = &H4      ' Meaningful only if the flag pfDecomprAllImages is set. If set,
                               ' JPEG2000 images are returned as is so that you can use your own
                               ' library to decompress such images since the the entire JPEG2000
                               ' codec is still marked as experimental. If we find an alternative
                               ' to the currently used Jasper library then we will replace it
                               ' immediatly with another one...

      ' The following flags are ignored if an image is not decompressed. Note that only one flag must be set
      ' at time. If no color space conversion flag is set images are returned in their native or alternate
      ' device color space. Note that these flags do not convert colors of vector graphics and so on.
      ' Use the function ConvColor() to convert colors of the graphics state into a device space.
      pfDitherImagesToBW = &H8 ' Floyd-Steinberg dithering.
      pfConvImagesToGray = &H10
      pfConvImagesToRGB = &H20
      pfConvImagesToCMYK = &H40
      pfImageInfoOnly = &H80   ' If set, images are not decompressed. This flag is useful if you want
                               ' to enumerate the images of a PDF file or if you want to determine how
                               ' many images are stored in it.
                               ' Note that images can be compressed with multiple filters. The member
                               ' Filter of the structure TPDFImage contains only the last filter with
                               ' which the image was compressed. There is no indication whether multiple
                               ' decode filters are required to decompress the image buffer. So, it
                               ' makes no sense to set this flag if you want to try to decompress the
                               ' image buffer manually with your own decode filters.
   End Enum

   Enum TPathFillMode
      fmFillNoClose
      fmStrokeNoClose
      fmFillStrokeNoClose
      fmFill
      fmStroke
      fmFillStroke
      fmFillEvOdd
      fmFillStrokeEvOdd
      fmFillEvOddNoClose
      fmFillStrokeEvOddNoClose
      fmNoFill
      fmClose
   End Enum

   Enum TPatternType
      ptColored
      ptUnColored
      ptShadingPattern ' Cannot be created with DynaPDF
   End Enum

   ' The tags have the same meaning as the corresponding HTML tags.
   ' See PDF Reference for further information.
   Enum TPDFBaseTag
      btArt
      btArtifact
      btAnnot
      btBibEntry      ' BibEntry -> Bibliography entry
      btBlockQuote
      btCaption
      btCode
      btDiv
      btDocument
      btFigure
      btForm
      btFormula
      btH
      btH1
      btH2
      btH3
      btH4
      btH5
      btH6
      btIndex
      btLink
      btList          ' L
      btListElem      ' LI
      btListText      ' LBody
      btNote
      btP
      btPart
      btQuote
      btReference
      btSection       ' Sect
      btSpan
      btTable
      btTableDataCell ' TD
      btTableHeader   ' TH
      btTableRow      ' TR
      btTOC
      btTOCEntry      ' TOCI
   End Enum

   ' Notice:
   '   When using a bidirectional 8 bit code page the bidi algorithm is applied by default in Left to Right mode
   '   also if the bidi mode is set to bmNone (default). This mode produces identical results in comparison to
   '   applications like Edit or WordPad.
   '
   '   The Right to Left mode is available in applications which use Microsoft's Uniscribe, e.g. BabelPad. This
   '   mode works very well with the Reference Bidi Algorithm which is used by DynaPDF.
   '
   '   However , Uniscribe 's Left to Right mode produces different results in comparison to the Reference Bidi
   '   Algorithm. Because the bidi algorithm that is used in Uniscribe is not published it is practically
   '   impossible to get the same result in Left to Right mode without using this library.

   Enum TPDFBidiMode
      bmLeftToRight ' Apply the bidi algorithm on Unicode strings in Left to Right layout.
      bmRightToLeft ' Apply the bidi algorithm on Unicode strings in Right to Left layout.
      bmNone        ' Default -> not apply the bidi algorithm
   End Enum

   Enum TPDFColorSpace
      csDeviceRGB
      csDeviceCMYK
      csDeviceGray
   End Enum

   Enum TPDFDateTime
      dfMM_D
      dfM_D_YY
      dfMM_DD_YY
      dfMM_YY
      dfD_MMM
      dfD_MMM_YY
      dfDD_MMM_YY
      dfYY_MM_DD
      dfMMM_YY
      dfMMMM_YY
      dfMMM_D_YYYY
      dfMMMM_D_YYYY
      dfM_D_YY_H_MM_TT
      dfM_D_YY_HH_MM
      ' time formats
      df24HR_MM
      df12HR_MM
      df24HR_MM_SS
      df12HR_MM_SS
   End Enum

   Enum TPDFVersion
      pvPDF_1_0 = 0
      pvPDF_1_1 = 1
      pvPDF_1_2 = 2
      pvPDF_1_3 = 3
      pvPDF_1_4 = 4
      pvPDF_1_5 = 5
      pvPDF_1_6 = 6
      pvPDF_1_7 = 7
      pvPDF_1_8 = 8
      pvPDF_1_9 = 9
      pvPDFX1a_2001 = 10 ' PDF/X-1a:2001
      pvPDFX1a_2003 = 11 ' PDF/X-1a:2002
      pvPDFX3_2002 = 12  ' PDF/X-3:2002
      pvPDFX3_2003 = 13  ' PDF/X-3:2003
      pvPDFA_2005 = 14   ' PDF/A-1b 2005
   End Enum

   Public Enum TPrintScaling
      psAppDefault ' Default
      psNone
   End Enum

   Private Enum TProgType
      ptImportPage = 0
      ptWritePage = 1
   End Enum

   Enum TPwdType
      ptOpen
      ptOwner
   End Enum

   Enum TRawImageFlags
      rifByteAligned = &H1000
      rifRGBData = &H2000
      rifCMYKData = &H4000
   End Enum

   Enum TRenderingIntent
      riAbsoluteColorimetric
      riPerceptual
      riRelativeColorimetric
      riSaturation
      riNone
   End Enum

   Enum TRestrictions
      rsDenyNothing = 0
      rsDenyAll = 3900
      rsPrint = 4
      rsModify = 8
      rsCopyObj = 16
      rsAddObj = 32
      ' 128/256 bit encryption only -> these flags are ignored if 40 bit encryption is used
      rsFillInFormFields = 256
      rsExtractObj = 512
      rsAssemble = 1024
      rsPrintHighRes = 2048
      rsExlMetadata = 4096    ' PDF 1.5 Exclude metadata streams -> 128/256 bit encryption bit only.
      rsEmbFilesOnly = &H2000 ' PDF 1.6 Encrypt embedded files only -> Requires AES encryption.
   End Enum

   Enum TRubberStamp
      rsApproved
      rsAsIs
      rsConfidential
      rsDepartmental
      rsDraft
      rsExperimental
      rsExpired
      rsFinal
      rsForComment
      rsForPublicRelease
      rsNotApproved
      rsNotForPublicRelease
      rsSold
      rsTopSecred
   End Enum

   Enum TShadingType
      stUnknown       ' cannot occur -> internal use
      stFunctionBased
      stAxial
      stRadial
      stFreeFormGouraud
      stLatticeFormGouraud
      stCoonsPatch
      stTensorProduct
   End Enum

   Enum TTilingType
      ttConstSpacing
      ttNoDistortion
      ttFastConstSpacing
   End Enum

   Enum TTextAlign
      taLeft = 0
      taCenter = 1
      taRight = 2
      taJustify = 3
      taPlainText = &H10000000 ' If this flag is set alignment and command tags are interpreted as plain text.
                               ' See WriteFText() in the help file for further information.
   End Enum

   Enum TStdPattern
      spHorizontal ' -----
      spVertical ' |||||
      spRDiagonal ' \\\\\
      spLDiagonal ' /////
      spCross ' +++++
      spDiaCross ' xxxxx
   End Enum

   Enum TSubmitFlags
      sfNone = &H0
      sfExlude = &H1 ' If set, fields in a submit or reset form action are excluded
      sfInclNoValFields = &H2
      sfHTML = &H4
      sfGetMethod = &H8
      sfSubmCoords = &H10
      sfXML = &H24
      sfInclAppSaves = &H40
      sfInclAnnots = &H80
      sfPDF = &H100
      sfCanonicalFormat = &H200
      sfExlNonUserAnnots = &H400
      sfExlFKey = &H800
      sfEmbedForm = &H2000 ' PDF 1.5 embed the entire form into a file stream inside the FDF file -> requires
                           ' the full version of Adobe's Acrobat
   End Enum

   Enum TViewerPreference
      vpUseNone = &H0
      vpHideToolBar = &H1
      vpHideMenuBar = &H2
      vpHideWindowUI = &H4
      vpFitWindow = &H8
      vpCenterWindow = &H10
      vpDisplayDocTitle = &H20
      vpNonFullScrPageMode = &H40
      vpDirection = &H80
      vpViewArea = &H100
      vpViewClip = &H200
      vpPrintArea = &H400
      vpPrintClip = &H800
   End Enum

   Enum TViewPrefAddVal
      avNone = &H0
      avNonFullScrUseNone = &H1
      avNonFullScrUseOutlines = &H2
      avNonFullScrUseThumbs = &H4
      avNonFullScrUseOC = &H400 ' PDF 1.6
      avDirection2R = &H8
      avDirectionR2 = &H10
      avViewPrintArtBox = &H20
      avViewPrintBleedBox = &H40
      avViewPrintCropBox = &H80
      avViewPrintMediaBox = &H100
      avViewPrintTrimBox = &H200
   End Enum

   Public Structure TBookmark
      Dim Color As Integer
      Dim DestPage As Integer
      Dim DestPos As TPDFRect
      Dim DestType As TDestType
      Dim DoOpen As Boolean
      Dim Parent As Integer
      Dim Style As Integer
      Dim Title As String
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TCTM
      Dim a As Double
      Dim b As Double
      Dim c As Double
      Dim d As Double
      Dim x As Double
      Dim y As Double
   End Structure

   Public Structure TDeviceNAttributes
      Dim IProcessColorSpace As IntPtr  ' Pointer to process color space or NULL -> GetColorSpaceEx().
      Dim ProcessColorants() As String  ' Process colorant names
      Dim Separations() As IntPtr       ' Optional pointers to Separation color spaces -> GetColorSpaceEx().
      Dim IMixingHints As IntPtr        ' Optional pointer to mixing hints. There is no API function at this time to access mixing hints.
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TRectL
      Dim rLeft As Integer
      Dim rTop As Integer
      Dim rRight As Integer
      Dim rBottom As Integer
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TTextRecordA
      Dim Advance As Single  ' Negative values move the cursor to right, positive to left. The value is measured in text space!
      Dim Text As IntPtr     ' Source string (not null-terminated)
      Dim Length As Integer  ' Length in characters
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TTextRecordW
      Dim Advance As Single  ' Negative values move the cursor to right, positive to left. The value is measured in text space!
      Dim Text As IntPtr     ' Already translated Unicode string (not null-terminated)
      Dim Length As Integer  ' Length in characters
      Dim Width As Single    ' String width measured in text space
   End Structure

   Public Structure TPDFAnnotation
      Dim AnnotType As CPDF.TAnnotType
      Dim Deleted As Boolean
      Dim BBox As TPDFRect
      Dim BorderWidth As Double
      Dim BorderColor As Integer
      Dim BorderStyle As CPDF.TBorderStyle
      Dim BackColor As Integer
      Dim Handle As Integer
      Dim Author As String
      Dim Content As String
      Dim Name As String
      Dim Subject As String
      Dim PageNum As Integer
      Dim HighlightMode As CPDF.THighlightMode
   End Structure

   Public Structure TPDFAnnotationEx
      Dim AnnotType As CPDF.TAnnotType
      Dim Deleted As Boolean
      Dim BBox As TPDFRect
      Dim BorderWidth As Single
      Dim BorderColor As Integer
      Dim BorderStyle As CPDF.TBorderStyle
      Dim BackColor As Integer
      Dim Handle As Integer
      Dim Author As String
      Dim Content As String
      Dim Name As String
      Dim Subject As String
      Dim PageNum As Integer
      Dim HighlightMode As CPDF.THighlightMode
      ' Page link annotations only
      Dim DestPage As Integer
      Dim DestPos As TPDFRect
      Dim DestType As TDestType
      Dim DestFile As String         ' File link or web link annotations
      Dim Icon As Integer            ' The Icon type depends on the annotation type. If the annotation type is atText then the Icon
      ' is of type TAnnotIcon. If the annotation type is atFileAttach then it is of type
      ' TFileAttachIcon. If the annotation type is atStamp then the Icon is the stamp type (TRubberStamp).
      ' For any other annotation type this value is not set (-1).
      Dim StampName As String        ' Set only, if Icon == rsUserDefined
      Dim AnnotFlags As Integer      ' See TAnnotFlags for available flags
      Dim CreateDate As String       ' Creation Date -> Optional
      Dim ModDate As String          ' Modification Date -> Optional
      Dim Grouped As Boolean         ' (Reply type) Meaningful only if Parent != -1 and Type != atPopUp. If true,
      ' the annotation is part of an annotation group. Properties like Content, CreateDate,
      ' ModDate, BackColor, Subject, and Open must be taken from the parent annotation.
      Dim Open As Boolean            ' Meaningful only for annotations which have a corresponding PopUp annotation.
      Dim Parent As Integer          ' Parent annotation handle of a PopUp Annotation or the parent annotation if
      ' this annotation represents a state of a base annotation. In this case,
      ' the annotation type is always atText and only the following members should
      ' be considered:
      '    State      // The current state
      '    StateModel // Marked, Review, and so on
      '    CreateDate // Creation Date
      '    ModDate    // Modification Date
      '    Author     // The user who has set the state
      '    Content    // Not displayed in Adobe's Acrobat...
      '    Subject    // Not displayed in Adobe's Acrobat...
      ' The PopUp annotation of a text annotation which represent an Annotation State
      ' must be ignored.
      Dim PopUp As Integer           ' Handle of the corresponding PopUp annotation if any.
      Dim State As String            ' The state of the annotation.
      Dim StateModel As String       ' The state model (Marked, Review, and so on).
      Dim EmbeddedFile As Integer    ' FileAttach annotations only. A handle of an embedded file -> GetEmbeddedFile().
      Dim Subtype As String          ' Set only, if Type = atUnknownAnnot
   End Structure

   Public Structure TPDFChoiceValue
      Dim ExpValue As String
      Dim Value As String
      Dim Selected As Boolean
   End Structure

   ' The structure contains several duplicate fields because CMap files contain usually a DSC comment
   ' section which provides Postscript specific initialization code. With exception of DSCResName the
   ' strings in the DSC section should not differ from their CMap counterparts. The Identity mapping
   ' of a character collection should contain the DSC comment "%%BeginResource: CMap (Identity)".
   ' Otherwise the string should be set to the CMap name.

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TPDFCMap
      Dim StructSize As Integer     ' Must be set to sizeof(TPDFCMap) before calling GetCMap()!
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim BaseCMap As String        ' If set, this base cmap is required when loading the cmap.
      Dim CIDCount As Integer       ' 0 if not set.
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim CMapName As String        ' The CMap name.
      Dim CMapType As Integer       ' Should be 1!
      Dim CMapVersion As Single     ' The CMap version.
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim DSCBaseCMap As String     ' DSC comment.
      Dim DSCCMapVersion As Single  ' DSC comment.
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim DSCResName As String      ' DSC comment. If the CMap uses an Identity mapping this string should be set to Identity.
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim DSCTitle As String        ' DSC comment -> DSC CMap name + Registry + Ordering + Supplement, e.g. "GB-EUC-H Adobe GB1 0"
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim FileNameA As String       ' The file name and CMap name should be identical!
      <MarshalAs(UnmanagedType.LPWStr)> _
      Dim FileNameW As String       ' The file name and CMap name should be identical!
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim FilePathA As String       ' The Ansi string is set if the Ansi version of SetCMapDir() was used.
      <MarshalAs(UnmanagedType.LPWStr)> _
      Dim FilePathW As String       ' The Unicode string is set if the Unicode version of SetCMapDir() was used.
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim Ordering As String        ' CIDSystemInfo -> The Character Collection, e.g. Japan1.
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim Registry As String        ' CIDSystemInfo -> The registrant of the Character Collection is usually Adobe.
      Dim Supplement As Integer     ' CIDSystemInfo -> The Supplement number should be supported in the used PDF Version.
      Dim WritingMode As Integer    ' 0 == Horizontal, 1 == Vertical
   End Structure

   Public Structure TPDFColorSpaceObj
      Dim ColorSpaceType As CPDF.TExtColorSpace
      Dim Alternate As CPDF.TExtColorSpace ' Alternate or base color space of an Indexed or Pattern color space.
      Dim IAlternate As IntPtr             ' Only set if the color space contains an alternate or base color space -> GetColorSpaceObjEx().
      Dim Buffer As IntPtr                 ' Contains either an ICC profile or the color table of an Indexed color space.
      Dim BufSize As Integer               ' Buffer length in bytes.
      Dim BlackPoint() As Single           ' CIE blackpoint. If set, the array contains exactly 3 values.
      Dim WhitePoint() As Single           ' CIE whitepoint. If set, the array contains exactly 3 values.
      Dim Gamma() As Single                ' If set, one value per component.
      Dim Range() As Single                ' The allowed range of input values (min/max for each component).
      Dim Matrix() As Single               ' XYZ matrix. If set, the array contains exactly 9 values.
      Dim NumInComponents As Integer       ' Number of input components.
      Dim NumOutComponents As Integer      ' Number of output components.
      Dim NumColors As Integer             ' HiVal + 1 as specified in the color space. Indexed color space only.
      Dim Colorants() As String            ' Colorant names (Separation, DeviceN, and NChannel only).
      Dim Metadata As IntPtr               ' Optional XMP metadata stream -> ICCBased only.
      Dim MetadataSize As Integer          ' Metadata length in bytes.
      Dim IFunction As IntPtr              ' Pointer to function object -> Separation, DeviceN, and NChannel only.
      Dim IAttributes As IntPtr            ' Optional attributes of DeviceN or NChannel color spaces -> GetNChannelAttributes().
      Dim IColorSpaceObj As IntPtr         ' Pointer of the corresponding color space object
   End Structure

   ' It is not possible to set all available graphic state parameters with DynaPDF, such as black generation functions
   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TPDFExtGState
      Dim AutoStrokeAdjust As Integer              ' PDF_MAX_INT if not set
      Dim BlendMode As CPDF.TBlendMode             ' Default bmNotSet
      Dim FlatnessTol As Single                    ' -1.0 if not set
      Dim OverPrintFill As Integer                 ' PDF_MAX_INT if not set
      Dim OverPrintStroke As Integer               ' PDF_MAX_INT if not set
      Dim OverPrintMode As Integer                 ' PDF_MAX_INT if not set
      Dim RenderingIntent As CPDF.TRenderingIntent ' riNone if not set
      Dim SmoothnessTol As Single                  ' -1.0 if not set
      Dim FillAlpha As Single                      ' -1.0 if not set
      Dim StrokeAlpha As Single                    ' -1.0 if not set
      Dim AlphaIsShape As Integer                  ' PDF_MAX_INT if not set
      Dim TextKnockout As Integer                  ' PDF_MAX_INT if not set
      Private Reserved1 As IntPtr
      Private Reserved2 As IntPtr
      Private Reserved3 As IntPtr
      Private Reserved4 As IntPtr
      Private Reserved5 As IntPtr
      Private Reserved6 As IntPtr
      Private Reserved7 As IntPtr
      Private Reserved8 As IntPtr
      Private Reserved9 As IntPtr
   End Structure

   ' Extended graphics state dictionary
   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TPDFExtGState2
      Dim AlphaIsShape As Integer       ' PDF_MAX_INT if not set
      Dim AutoStrokeAdjust As Integer   ' PDF_MAX_INT if not set
      Dim BlackGen As IntPtr            ' Function handle or NULL -> GetFunction()
      Dim BlackGen2 As IntPtr           ' Function handle or NULL -> GetFunction()
      Dim BlendMode As IntPtr           ' Array of blend modes
      Dim BlendModeCount As Integer     ' Number of blend modes
      Dim FillAlpha As Single           ' -1.0 if not set
      Dim FlatnessTol As Single         ' -1.0 if not set
      Dim Halftone As IntPtr            ' Halftone handle or NULL -> GetHalftoneDict()
      Dim OverPrintFill As Integer      ' PDF_MAX_INT if not set
      Dim OverPrintStroke As Integer    ' PDF_MAX_INT if not set
      Dim OverPrintMode As Integer      ' PDF_MAX_INT if not set
      Dim RenderingIntent As CPDF.TRenderingIntent ' riNone if not set
      Dim SmoothnessTol As Single       ' -1.0 if not set
      Dim SoftMask As IntPtr            ' Soft mask handle or NULL
      Dim StrokeAlpha As Single         ' -1.0 if not set
      Dim TextKnockout As Integer       ' PDF_MAX_INT if not set
      Dim TransferFunc As IntPtr        ' Array of functions -> GetFunction()
      Dim TransferFuncCount As Integer  ' Number of transfer functions
      Dim TransferFunc2 As IntPtr       ' Array of functions -> GetFunction()
      Dim TransferFunc2Count As Integer ' Number of transfer functions
      Dim UnderColorRem As IntPtr       ' Function handle or NULL -> GetFunction()
      Dim UnderColorRem2 As IntPtr      ' Function handle or NULL -> GetFunction()
      Private Reserved1 As IntPtr
      Private Reserved2 As IntPtr
      Private Reserved3 As IntPtr
      Private Reserved4 As IntPtr
   End Structure

   Public Structure TPDFField
      Dim BBox As TPDFRect
      Dim FieldType As TFieldType
      Dim Deleted As Boolean
      Dim Handle As Integer
      Dim FieldName As String
      Dim BackCS As TPDFColorSpace
      Dim TextCS As TPDFColorSpace
      Dim BackColor As Integer
      Dim BorderColor As Integer
      Dim TextColor As Integer
      Dim Checked As Boolean
      Dim Parent As Integer
      Dim KidCount As Integer
      Dim FontName As String
      Dim FontSize As Double
      Dim Value As String
      Dim ToolTip As String
   End Structure

   Public Structure TPDFFieldEx
      Dim Deleted As Boolean               ' If true, the field was marked as deleted by DeleteField()
      Dim BBox As TPDFRect                 ' Bounding box of the field in bottom-up coordinates
      Dim FieldType As TFieldType          ' Field type
      Dim GroupType As TFieldType          ' If GroupType != FieldType the field is a terminal field of a field group
      Dim Handle As Integer                ' Field handle
      Dim BackColor As Integer             ' Background color
      Dim BackColorSP As TExtColorSpace    ' Color space of the background color
      Dim BorderColor As Integer           ' Border color
      Dim BorderColorSP As TExtColorSpace  ' Color space of the border color
      Dim BorderStyle As TBorderStyle      ' Border style
      Dim BorderWidth As Single            ' Border width
      Dim CharSpacing As Single            ' Text fields only
      Dim Checked As Boolean               ' Check boxes only
      Dim CheckBoxChar As Integer          ' ZapfDingbats character that is used to display the on state
      Dim DefState As TCheckBoxState       ' Check boxes only
      Dim DefValue As String               ' Optional default value
      Dim IEditFont As IntPtr              ' Pointer to default editing font
      Dim EditFont As String               ' Postscript name of the editing font
      Dim ExpValCount As Integer           ' Combo and list boxes only. The values can be accessed with GetFieldExpValueEx()
      Dim ExpValue As String               ' Check boxes only
      Dim FieldFlags As TFieldFlags        ' Field flags
      Dim IFieldFont As IntPtr             ' Pointer to the font that is used by the field
      Dim FieldFont As String              ' Postscript name of the font
      Dim FontSize As Double               ' Font size. 0.0 means auto font size
      Dim FieldName As String              ' Note that the children of a field group or radio button have no name
      Dim HighlightMode As THighlightMode  ' Highlight mode
      Dim IsCalcField As Boolean           ' If true, the OnCalc event of the field is connected with a JavaScript action
      Dim MapName As String                ' Optional unique mapping name of the field
      Dim MaxLen As Integer                ' Text fields only -> zero means not restricted
      Dim Kids() As IntPtr                 ' Array of child fields -> GetFieldEx2()
      Dim Parent As IntPtr                 ' Pointer to parent field or NULL
      Dim PageNum As Integer               ' Page on which the field is used or -1
      Dim Rotate As Integer                ' Rotation angle in degrees
      Dim TextAlign As TTextAlign          ' Text fields only
      Dim TextColor As Integer             ' Text color
      Dim TextColorSP As TExtColorSpace    ' Color space of the field's text
      Dim TextScaling As Single            ' Text fields only
      Dim ToolTip As String                ' Optional tool tip
      Dim UniqueName As String             ' Optional unique name (NM key)
      Dim Value As String                  ' Field value
      Dim WordSpacing As Single            ' Text fields only
   End Structure

   Public Structure TPDFFileSpec
      Dim Buffer() As Byte       ' Buffer of an embedded file
      Dim Compressed As Boolean  ' Should be false if Decompress was true in the GetEmbeddedFile() call, otherwise usually true.
      ' DynaPDF decompresses Flate encoded streams only. Other filters can occur but this is very unusual.
      Dim ColItem As IntPtr      ' If != NULL the embedded file contains a collection item with user defined data. This entry
      ' can occur in PDF Collections only (PDF 1.7). See "PDF Collections" in the help file for further
      ' information.
      Dim Name As String         ' Name of the file specification in the name tree. This value is always present.
      Dim FileName As String     ' File name as 7 bit ASCII string.
      Dim IsURL As Boolean       ' If true, FileName contains a URL.
      Dim UF As String           ' PDF 1.7. Same as FileName but Unicode is allowed.
      Dim Desc As String         ' Description
      Dim FileSize As Integer    ' Size of the decompressed stream or zero if not known. Note: this is either the Size key of
      ' the Params dictionary if present or the DL key in the file stream. Whether this value is
      ' correct depends on the file creator! The parameter is definitely correct if the file was
      ' decompressed.
      Dim MIMEType As String     ' MIME media type name as defined in Internet RFC 2046.
      Dim CreateDate As String   ' Creation date as string. See help file "The standard date format".
      Dim ModDate As String      ' Modification date as string. See help file "The standard date format".
      Dim CheckSum() As Byte     ' 16 byte MD5 digest. Note that this is a binary string. It is exactly 16 bytes long if set!
   End Structure

   Public Structure TPDFFontObj
      Dim Ascent As Single      ' Ascent
      Dim BaseFont As String    ' PostScript Name or Family Name
      Dim CapHeight As Single   ' Cap height
      Dim Descent As Single     ' Descent
      Dim Encoding As Char()    ' Unicode mapping 0..255 -> not set if a CID font is selected
      Dim FirstChar As Integer  ' First char
      Dim Flags As Integer      ' Font flags -> font descriptor
      Dim FontFamily As String  ' Optional Font Family (Family Name)
      Dim FontName As String    ' Font name -> font descriptor
      Dim FontType As TFontType ' If ftType0 the font is a CID font. The Encoding is not set in this case.
      Dim ItalicAngle As Single ' Italic angle
      Dim LastChar As Integer   ' Last char
      Dim SpaceWidth As Single  ' Space width in font units. A default value is set if the font contains no space character.
      Dim Widths As Single()    ' Glyph widths -> 0..WidthsCount -1
      Dim XHeight As Single     ' x-height
      Dim DefWidth As Single    ' Default character widths -> CID fonts only
      Dim FontFile As IntPtr    ' Font file buffer -> only imported fonts are returned.
      Dim Length1 As Integer    ' Length of the clear text portion of the Type1 font, or the length of the entire font program if FontType != ffType1.
      Dim Length2 As Integer    ' Length of the encrypted portion of the Type1 font program (Type1 fonts only).
      Dim Length3 As Integer    ' Length of the fixed-content portion of the Type1 font program or zero if not present.
      Dim FontFileType As CPDF.TFontFileSubtype  ' See description in the help file for further information.
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TPDFImage
      Dim Buffer As IntPtr                  ' Each scanline is aligned to a full byte.
      Dim BufSize As Integer                ' The size of the image buffer in bytes.
      Dim Filter As CPDF.TDecodeFilter      ' Required decode filter if the image is compressed.
      ' Possible values are dfDCTDecode (JPEG), dfJPXDecode (JPEG2000),
      ' and dfJBIG2Decode. Other filters are always removed by DynaPDF since
      ' a conversion to a native file format is then always required.
      Dim OrgFilter As CPDF.TDecodeFilter   ' The image was compressed with this filter in the PDF file. This info is
      ' useful to determine which compression filter should be used when creating
      ' a new image file from the image buffer.
      Dim JBIG2Globals As IntPtr            ' Optional global page 0 segment (dfJBIG2Decode filter only).
      Dim JBIG2GlobalsSize As Integer       ' The size of the bit stream in bytes.
      Dim BitsPerPixel As Integer           ' Bit depth of the image buffer. Possible values are 1, 2, 4, 8, 16, 24, and 32.
      Dim ColorSpace As CPDF.TExtColorSpace ' The color space refers either to the image buffer or to the color table if set.
      ' Note that 1 bit images can occur with and without a color table.
      Dim NumComponents As Integer          ' The number of components stored in the image buffer.
      Dim MinIsWhite As Integer             ' If true, the colors of 1 bit images are reversed.
      Dim IColorSpaceObj As IntPtr          ' Pointer to the original color space.
      Dim ColorTable As IntPtr              ' The color table or NULL.
      Dim ColorCount As Integer             ' The number of colors in the color table.
      Dim Width As Integer                  ' Image width in pixel.
      Dim Height As Integer                 ' Image height in pixel.
      Dim ScanLineLength As Integer         ' The length of a scanline in bytes.
      Dim InlineImage As Integer            ' Boolean -> If true, the image is an inline image.
      Dim Interpolate As Integer            ' Boolean -> If true, image interpolation should be performed.
      Dim Transparent As Integer            ' Boolean -> The meaning is different depending on the bit depth and whether a color
                                            ' table is available. If the image is a 1 bit image and if no color table is available,
                                            ' black pixels must be drawn with the current fill color.
                                            ' If the image contains a color table ColorMask contains the range of indexes
                                            ' in the form min/max index which appears transparent. If no color table is
                                            ' present ColorMask contains the transparent ranges in the form min/max for
                                            ' each component.
      Dim ColorMask As IntPtr               ' The array contains ranges in the form min/max (2 values per component) for each
      ' component before decoding (data type Byte).
      Dim IMaskImage As IntPtr              ' If set, a 1 bit image is used as a transparency mask. Call GetImage() to decode the image.
      Dim ISoftMask As IntPtr               ' If set, a grayscale image is used as alpha channel. Call GetImage() to decode the image.
      Dim Decode As IntPtr                  ' If set, samples must be decoded. The array contains 2 * NumComponents values (data type Single).
      ' The decode array is never set if the image is returned decompressed since
      ' it is already applied during decompression.
      Dim Intent As CPDF.TRenderingIntent   ' Default riNone.
      Dim SMaskInData As Integer            ' JPXDecode only, PDF_MAX_INT if not set. See PDF Reference for further information.
      Dim OCG As IntPtr                     ' Pointer to Optional Content Group if any.
      Dim Metadata As IntPtr                ' Optional XML Metadata stream.
      Dim MetadataSize As Integer           ' Length of Metadata in bytes.
      Dim ObjectPtr As IntPtr               ' Internal pointer to the image class.
      Dim ResolutionX As Single             ' Image resolution on the x-axis.
      Dim ResolutionY As Single             ' Image resolution on the y-axis.
      ' Reserved fields for future extensions
      Private Reserved1 As Integer
      Private Reserved2 As Integer
      Private Reserved3 As Integer
      Private Reserved4 As Integer
      Private Reserved5 As Integer
      Private Reserved6 As Integer
      Private Reserved7 As Integer
      Private Reserved8 As Integer
      Private Reserved9 As Integer
      Private Reserved10 As Integer
      Private Reserved11 As Integer
      Private Reserved12 As Integer
      Private Reserved13 As Integer
      Private Reserved14 As Integer
      Private Reserved15 As Integer
      Private Reserved16 As Integer
      Private Reserved17 As Integer
      Private Reserved18 As Integer
   End Structure

   Public Structure TPDFNamedDest
     Dim Name As String
     Dim DestPage As Integer
     Dim DestPos As TPDFRect
     Dim DestType As TDestType
   End Structure

   Public Structure TPDFPageLabel
      Dim StartRange As Integer      ' Number of the first page in the range. If no further label follows, the last
                                     ' page in the range is pdfGetPageCount(). The first page is denoted by 1.
      Dim Format As TPageLabelFormat ' Number format to be used.
      Dim FirstPageNum As Integer    ' First page number to be displayed in the page label. Subsequent pages are
                                     ' numbered sequentially from this value.
      Dim Prefix As String           ' Optional prefix
   End Structure

   Public Structure TPDFPrintSettings
      Dim DuplexMode As CPDF.TDuplexMode
      Dim NumCopies As Integer               ' -1 means not set. Values larger than 5 are ignored in viewer applications.
      Dim PickTrayByPDFSize As Integer       ' -1 means not set. 0 == false, 1 == true.
      Dim PrintRanges() As Integer           ' If set, the array contains PrintRangesCount * 2 values. Each pair consists
      ' of the first and last pages in the sub-range. The first page in the PDF file
      ' is denoted by 0.
      Dim PrintScaling As CPDF.TPrintScaling ' dpmNone means not set.
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TPDFRect
      Dim rLeft As Double
      Dim rBottom As Double
      Dim rRight As Double
      Dim rTop As Double
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TPDFSigParms
      Dim PKCS7ObjLen As Integer    ' The maximum length of the signed PKCS#7 object
      Dim HashType As THashType     ' If set to htDetached, the bytes ranges of the PDF file will be returned.
      Dim Range1() As Byte          ' Out -> Contains either the hash or the first byte range to create a detached signature
      Dim Range2() As Byte          ' Out -> Set only if HashType == htDetached
      Dim ContactInfo As String     ' Optional, e.g. an email address_
      Dim Location As String        ' Optional location of the signer
      Dim Reason As String          ' Optional reason why the file was signed
      Dim Signer As String          ' Optional, the issuer of the certificate takes precedence
      Dim Encrypt As Boolean        ' If true, the file will be encrypted
      ' These members will be ignored if Encrypt is set to false
      Dim OpenPwd As String         ' Open password
      Dim OwnerPwd As String        ' Owner password to change the security settings
      Dim KeyLen As TKeyLen         ' Key length to be used to encrypt the file
      Dim Restrict As TRestrictions ' What should be restricted?
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Public Structure TPDFStack
      Dim ctm As TCTM                ' Pre-multiplied global transformation matrix
      Dim tm As TCTM                 ' Pre-multiplied text transformation matrix
      Dim x As Double                ' Unused -> always 0.0
      Dim y As Double                ' Unused -> always 0.0
      Dim FontSize As Double         ' Font size
      Dim CharSP As Double           ' Character spacing
      Dim WordSP As Double           ' Word spacing
      Dim HScale As Double           ' Horizontal text scaling
      Dim TextRise As Double         ' Text rise -> always 0.0 because it is already included in the text transformation matrix
      Dim Leading As Double          ' Leading
      Dim LineWidth As Double        ' Line width
      Dim DrawMode As TDrawMode      ' Text draw mode
      Dim FillCS As TPDFColorSpace   ' Fill color space
      Dim StrokeCS As TPDFColorSpace ' Stroke color space
      Dim FillColor As Integer       ' Fill color
      Dim StrokeColor As Integer     ' Stroke color
      Dim BaseObject As IntPtr       ' Internal
      Dim CIDFont As Integer         ' If true, ReplacePageText() can only be used to delete a string
      Dim Text As IntPtr             ' Raw text without kerning space
      Dim TextLen As Integer         ' Raw text length
      Dim RawKern As IntPtr          ' Raw kerning array
      Dim Kerning As IntPtr          ' Already translated Unicode kerning array
      Dim KerningCount As Integer    ' Number of kerning records
      Dim TextWidth As Single        ' The width of the entire text record measured in text space
      Dim IFont As IntPtr            ' Font object used to print the string -> fntGetFont() can be used to return the font properties
      Dim Embedded As Integer        ' If true, the font is embedded
      Dim SpaceWidth As Single       ' Measured in text space
      'These members can be modified after the structure has been initialized with InitStack().
      'If the destination color space should be DeviceCMYK initialize FillColor and StrokeColor
      'with PDF_CMYK(0,0,0,255); which represents black.
      Dim ConvColors As Integer       ' If set to true (default), all colors are converted to the specified destination color space
      Dim DestSpace As TPDFColorSpace ' Destination color space -> default == csDeviceRGB

      ' This member can be used in combination with ReplacePageText() to preserve a number
      ' of kerning records from deletion. All records above this value will be deleted.
      ' Take a look into the file examples/util/pdf_edit_text.vb to determine how this member
      ' can be used.
      Dim DeleteKerningAt As Integer
      Dim FontFlags As Integer        ' PDF font flags

      ' ------------------------------- Reserved fields -------------------------------
      Private Reserved1 As Integer
      Private Reserved2 As Integer
      Private Reserved3 As Integer
      Private Reserved4 As Integer
      Private Reserved5 As Integer
      Private Reserved6 As Integer
      Private Reserved7 As Integer
      Private Reserved8 As Integer
      Private Reserved9 As Integer
      Private Reserved10 As Integer
      Private Reserved11 As Integer
      Private ContentPtr As IntPtr
   End Structure

   ' ---------------------------------------------- Parser Interface ----------------------------------------------

   Public Delegate Function TApplyPattern(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Type As CPDF.TPatternType, ByVal Pattern As IntPtr) As Integer
   Public Delegate Function TBeginPattern(ByVal Data As IntPtr, ByVal Fill As Integer, ByVal Handle As Integer, ByVal Type As CPDF.TPatternType, ByRef BBox As CPDF.TPDFRect, ByVal Matrix As IntPtr, ByVal XStep As IntPtr, ByVal YStep As IntPtr) As Integer
   Public Delegate Function TBeginTemplate(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Handle As Integer, ByRef BBox As CPDF.TPDFRect, ByVal Matrix As IntPtr) As Integer
   Public Delegate Function TBezierTo1(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal x1 As Double, ByVal y1 As Double, ByVal x3 As Double, ByVal y3 As Double) As Integer
   Public Delegate Function TBezierTo2(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal x2 As Double, ByVal y2 As Double, ByVal x3 As Double, ByVal y3 As Double) As Integer
   Public Delegate Function TBezierTo3(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal x1 As Double, ByVal y1 As Double, ByVal x2 As Double, ByVal y2 As Double, ByVal x3 As Double, ByVal y3 As Double) As Integer
   Public Delegate Function TClipPath(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal EvenOdd As Integer, ByVal Mode As CPDF.TPathFillMode) As Integer
   Public Delegate Function TClosePath(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Mode As CPDF.TPathFillMode) As Integer
   Public Delegate Function TDrawShading(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Type As CPDF.TShadingType, ByVal Shading As IntPtr) As Integer
   Public Delegate Sub TEndPattern(ByVal Data As IntPtr)
   Public Delegate Sub TEndTemplate(ByVal Data As IntPtr)
   Public Delegate Function TInsertImage(ByVal Data As IntPtr, ByRef Image As CPDF.TPDFImage) As Integer
   Public Delegate Function TLineTo(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal x As Double, ByVal y As Double) As Integer
   Public Delegate Function TMoveTo(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal x As Double, ByVal y As Double) As Integer
   Public Delegate Sub TMulMatrix(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByRef M As CPDF.TCTM)
   Public Delegate Function TRectangle(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal x As Double, ByVal y As Double, ByVal w As Double, ByVal h As Double) As Integer
   Public Delegate Function TRestoreGraphicState(ByVal Data As IntPtr) As Integer
   Public Delegate Function TSaveGraphicState(ByVal Data As IntPtr) As Integer
   Public Delegate Sub TSetCharSpacing(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Value As Double)
   Public Delegate Sub TSetExtGState(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByRef GS As CPDF.TPDFExtGState2)
   Public Delegate Sub TSetFillColor(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Color As IntPtr, ByVal NumComps As Integer, ByVal CS As CPDF.TExtColorSpace, ByVal IColorSpace As IntPtr)
   Public Delegate Sub TSetFont(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Type As CPDF.TFontType, ByVal Embedded As Integer, ByVal FontName As IntPtr, ByVal Style As CPDF.TFStyle, ByVal FontSize As Double, ByVal IFont As IntPtr)
   Public Delegate Sub TSetLeading(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Value As Double)
   Public Delegate Sub TSetLineCapStyle(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Style As CPDF.TLineCapStyle)
   Public Delegate Sub TSetLineDashPattern(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Dash As IntPtr, ByVal NumValues As Integer, ByVal Phase As Integer)
   Public Delegate Sub TSetLineJoinStyle(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Style As CPDF.TLineJoinStyle)
   Public Delegate Sub TSetLineWidth(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Value As Double)
   Public Delegate Sub TSetMiterLimit(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Value As Double)
   Public Delegate Sub TSetStrokeColor(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Color As IntPtr, ByVal NumComps As Integer, ByVal CS As CPDF.TExtColorSpace, ByVal IColorSpace As IntPtr)
   Public Delegate Sub TSetTextDrawMode(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Mode As CPDF.TDrawMode)
   Public Delegate Sub TSetTextScale(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Value As Double)
   Public Delegate Sub TSetWordSpacing(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByVal Value As Double)
   ' It is not required to declare the marshalling attributes in the real callback function.
   Public Delegate Function TShowTextArrayA(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByRef Matrix As CPDF.TCTM, <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> ByVal Source() As CPDF.TTextRecordA, ByVal Count As Integer, ByVal Width As Double) As Integer
   Public Delegate Function TShowTextArrayW(ByVal Data As IntPtr, <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> ByVal Source() As CPDF.TTextRecordA, ByRef Matrix As CPDF.TCTM, <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> ByVal Kerning() As CPDF.TTextRecordW, ByVal Count As Integer, ByVal Width As Double, ByVal Decoded As Integer) As Integer
   'Public Delegate Function TShowTextArrayA(ByVal Data As IntPtr, ByVal Obj As IntPtr, ByRef Matrix As CPDF.TCTM, ByVal Source As IntPtr, ByVal Count As Integer, ByVal Width As Double) As Integer
   'Public Delegate Function TShowTextArrayW(ByVal Data As IntPtr, ByVal Source As IntPtr, ByRef Matrix As TCTM, ByVal Kerning As IntPtr, ByVal Count As Integer, ByVal Width As Double, ByVal Decoded As Integer) As Integer

   ' Unnecessary callback functions can be set to null -> already default when creating a new instance of the structure.
   Public Structure TPDFParseInterface
      Dim ApplyPattern As TApplyPattern
      Dim BeginPattern As TBeginPattern
      Dim BeginTemplate As TBeginTemplate
      Dim BezierTo1 As TBezierTo1
      Dim BezierTo2 As TBezierTo2
      Dim BezierTo3 As TBezierTo3
      Dim ClipPath As TClipPath
      Dim ClosePath As TClosePath
      Dim DrawShading As TDrawShading
      Dim EndPattern As TEndPattern
      Dim EndTemplate As TEndTemplate
      Dim LineTo As TLineTo
      Dim MoveTo As TMoveTo
      Dim MulMatrix As TMulMatrix
      Dim Rectangle As TRectangle
      Dim RestoreGraphicState As TRestoreGraphicState
      Dim SaveGraphicState As TSaveGraphicState
      Dim SetCharSpacing As TSetCharSpacing
      Dim SetExtGState As TSetExtGState
      Dim SetFillColor As TSetFillColor
      Dim SetFont As TSetFont
      Dim SetLeading As TSetLeading
      Dim SetLineCapStyle As TSetLineCapStyle
      Dim SetLineDashPattern As TSetLineDashPattern
      Dim SetLineJoinStyle As TSetLineJoinStyle
      Dim SetLineWidth As TSetLineWidth
      Dim SetMiterLimit As TSetMiterLimit
      Dim SetStrokeColor As TSetStrokeColor
      Dim SetTextDrawMode As TSetTextDrawMode
      Dim SetTextScale As TSetTextScale
      Dim SetWordSpacing As TSetWordSpacing
      Private Reserved001 As IntPtr
      Private Reserved002 As IntPtr
      Dim ShowTextArrayW As TShowTextArrayW
      Dim InsertImage As TInsertImage
      Dim ShowTextArrayA As TShowTextArrayA
      Private Reserved01 As IntPtr
      Private Reserved02 As IntPtr
      Private Reserved03 As IntPtr
      Private Reserved04 As IntPtr
      Private Reserved05 As IntPtr
      Private Reserved06 As IntPtr
      Private Reserved07 As IntPtr
      Private Reserved08 As IntPtr
      Private Reserved09 As IntPtr
      Private Reserved10 As IntPtr
      Private Reserved11 As IntPtr
      Private Reserved12 As IntPtr
      Private Reserved13 As IntPtr
      Private Reserved14 As IntPtr
      Private Reserved15 As IntPtr
      Private Reserved16 As IntPtr
      Private Reserved17 As IntPtr
      Private Reserved18 As IntPtr
      Private Reserved19 As IntPtr
      Private Reserved20 As IntPtr
      Private Reserved21 As IntPtr
      Private Reserved22 As IntPtr
      Private Reserved23 As IntPtr
      Private Reserved24 As IntPtr
      Private Reserved25 As IntPtr
      Private Reserved26 As IntPtr
      Private Reserved27 As IntPtr
   End Structure

   ' Callback function protoypes. These prototypes are requires to enable event support in VB .Net.
   ' Data contains the current instance pointer of the event interface.
   Delegate Function TEnumFontsProc(ByVal Data As IntPtr, ByVal FamilyName As IntPtr, ByVal PostScriptName As IntPtr, ByVal Style As Integer) As Integer
   Delegate Function TEnumFontsProc2(ByVal Data As IntPtr, ByVal PDFFont As IntPtr, ByVal FontType As Integer, ByVal BaseFont As IntPtr, ByVal FontName As IntPtr, ByVal Embedded As Integer, ByVal IsFormFont As Integer, ByVal Flags As Integer) As Integer
   Delegate Function TEnumFontsProcEx(ByVal Data As IntPtr, ByVal FamilyName As IntPtr, ByVal PostScriptName As IntPtr, ByVal Style As Integer, ByVal BaseType As TFontBaseType, ByVal Embeddable As Integer, ByVal FilePath As IntPtr) As Integer
   Delegate Function TErrorProc(ByVal Data As IntPtr, ByVal ErrCode As Integer, ByVal ErrMessage As IntPtr, ByVal ErrType As Integer) As Integer
   Delegate Function TOnPageBreakProc(ByVal Data As IntPtr, ByVal LastPosX As Double, ByVal LastPosY As Double, ByVal PageBreak As Integer) As Integer
   Delegate Function TOnFontNotFoundProc(ByVal Data As IntPtr, ByVal PDFFont As IntPtr, ByVal FontName As IntPtr, ByVal Style As TFStyle, ByVal StdFontIndex As Integer, ByVal IsSymbolFont As Integer) As Integer
   Delegate Function TOnReplaceICCProfile(ByVal Data As IntPtr, ByVal Type As TICCProfileType, ByVal ColorSpace As Integer) As Integer
   Delegate Sub TInitProgress(ByVal Data As IntPtr, ByVal ProgType As Integer, ByVal MaxCount As Integer)
   Delegate Function TProgress(ByVal Data As IntPtr, ByVal ActivePage As Integer) As Integer

   Private m_Instance As IntPtr
   Private m_AddrEnumFonts As TEnumFontsProc
   Private m_AddrEnumFontsEx As TEnumFontsProcEx
   Private m_AddrEnumDocFonts As TEnumFontsProc2
   Private m_AddrErrorProc As TErrorProc
   Private m_AddrInitProgress As TInitProgress
   Private m_AddrOnPageBreak As TOnPageBreakProc
   Private m_AddrProgress As TProgress

   Public Sub New()
      MyBase.New()
      m_Instance = pdfNewPDF()
      If IntPtr.Zero.Equals(m_Instance) Then
         MsgBox("Out of Memory!", MsgBoxStyle.Exclamation, "Fatal error!")
         Exit Sub
      End If

      m_AddrEnumFonts = AddressOf pdf_EnumFontsProc
      m_AddrEnumFontsEx = AddressOf pdf_EnumFontsProcEx
      m_AddrEnumDocFonts = AddressOf pdf_EnumDocFontsProc
      m_AddrErrorProc = AddressOf pdf_ErrorProc
      m_AddrInitProgress = AddressOf pdf_InitProgress
      m_AddrProgress = AddressOf pdf_Progress
      m_AddrOnPageBreak = AddressOf pdf_OnPageBreakProc
      pdfSetOnErrorProc(m_Instance, Nothing, m_AddrErrorProc)
      pdfSetProgressProc(m_Instance, Nothing, m_AddrInitProgress, m_AddrProgress)
   End Sub

   Protected Overrides Sub Finalize()
      If pdfDeletePDF(m_Instance) = 0 Then
         MsgBox("Cannot delete PDF instance!", MsgBoxStyle.Exclamation, "Fatal error!")
      End If
      MyBase.Finalize()
   End Sub

   ' The following callback functions are used to raise events.
   ' FamilyName is a Unicode string, the PostScriptName is an ANSI string!
   Private Function pdf_EnumFontsProc(ByVal Data As IntPtr, ByVal FamilyName As IntPtr, ByVal PostScriptName As IntPtr, ByVal Style As Integer) As Integer
      On Error Resume Next
      Dim doBreak As Boolean
      Dim famName As String
      Dim postName As String
      doBreak = False

      famName = ToString(FamilyName, True)
      postName = ToString(PostScriptName, False)
      RaiseEvent PDFEnumFont(famName, postName, Style, doBreak)
      Return CInt(doBreak = True)
   End Function

   Private Function pdf_EnumFontsProcEx(ByVal Data As IntPtr, ByVal FamilyName As IntPtr, ByVal PostScriptName As IntPtr, ByVal Style As Integer, ByVal BaseType As TFontBaseType, ByVal Embeddable As Integer, ByVal FilePath As IntPtr) As Integer
      Dim doBreak As Boolean
      Dim famName As String
      Dim postName As String
      Dim fPath As String
      doBreak = False
      famName = ToString(FamilyName, True)
      postName = ToString(PostScriptName, False)
      fPath = ToString(FilePath, False)
      RaiseEvent PDFEnumFontEx(famName, postName, Style, BaseType, Embeddable <> 0, fPath, doBreak)
      Return CInt(doBreak = True)
   End Function

   Private Function pdf_EnumDocFontsProc(ByVal Data As IntPtr, ByVal PDFFont As IntPtr, ByVal FontType As Integer, ByVal BaseFont As IntPtr, ByVal FontName As IntPtr, ByVal Embedded As Integer, ByVal IsFormFont As Integer, ByVal Flags As Integer) As Integer
      Dim doBreak As Boolean
      Dim bFont As String
      Dim fName As String
      bFont = ToString(BaseFont, False)
      fName = ToString(FontName, False)
      RaiseEvent PDFEnumDocFont(PDFFont, CType(FontType, TFontType), bFont, fName, CBool(Embedded), CBool(IsFormFont), Flags, doBreak)
      Return CInt(doBreak = True)
   End Function

   Private Function pdf_ErrorProc(ByVal Data As IntPtr, ByVal ErrCode As Integer, ByVal ErrMessage As IntPtr, ByVal ErrType As Integer) As Integer
      On Error Resume Next
      Dim doBreak As Boolean
      doBreak = False
      RaiseEvent PDFError(ToString(ErrMessage, False), ErrType, doBreak)
      Return CInt(doBreak = True)
   End Function

   Private Function pdf_OnPageBreakProc(ByVal Data As IntPtr, ByVal LastPosX As Double, ByVal LastPosY As Double, ByVal PageBreak As Integer) As Integer
      On Error Resume Next
      Dim doBreak As Boolean
      Dim newAlign As TNewAlign
      doBreak = False
      newAlign = TNewAlign.naUnchanged
      RaiseEvent PDFPageBreak(LastPosX, LastPosY, PageBreak, newAlign, doBreak)
      If doBreak Then
         Return -1
      Else
         Return newAlign
      End If
   End Function

   ' pdf_InitProgress is called before pdf_Progress is called the first time
   Private Sub pdf_InitProgress(ByVal Data As IntPtr, ByVal ProgType As Integer, ByVal MaxCount As Integer)
      On Error Resume Next
      RaiseEvent PDFInitProgress(ProgType, MaxCount)
   End Sub

   Private Function pdf_Progress(ByVal Data As IntPtr, ByVal ActivePage As Integer) As Integer
      On Error Resume Next
      Dim doBreak As Boolean
      doBreak = False
      RaiseEvent PDFProgress(ActivePage, doBreak)
      Return CInt(doBreak = True)
   End Function

   Public Function AddActionToObj(ByVal ObjType As TObjType, ByVal ObjEvent As TObjEvent, ByVal ActHandle As Integer, ByVal ObjHandle As Integer) As Integer
      Return pdfAddActionToObj(m_Instance, ObjType, ObjEvent, ActHandle, ObjHandle)
   End Function

   Public Function AddArticle(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfAddArticle(m_Instance, PosX, PosY, Width, Height))
   End Function

   Public Function AddBookmarkA(ByVal Title As String, ByVal Parent As Integer, ByVal DestPage As Integer, ByVal DoOpen As Boolean) As Integer
      Return pdfAddBookmarkA(m_Instance, Title, Parent, DestPage, CInt(DoOpen))
   End Function

   Public Function AddBookmarkW(ByVal Title As String, ByVal Parent As Integer, ByVal DestPage As Integer, ByVal DoOpen As Boolean) As Integer
      Return pdfAddBookmarkW(m_Instance, Title, Parent, DestPage, CInt(DoOpen))
   End Function

   Public Function AddBookmarkExA(ByVal Title As String, ByVal Parent As Integer, ByVal NamedDest As Integer, ByVal DoOpen As Integer) As Integer
      Return pdfAddBookmarkExA(m_Instance, Title, Parent, NamedDest, CInt(DoOpen))
   End Function

   Public Function AddBookmarkExW(ByVal Title As String, ByVal Parent As Integer, ByVal NamedDest As Integer, ByVal DoOpen As Integer) As Integer
      Return pdfAddBookmarkExW(m_Instance, Title, Parent, NamedDest, CInt(DoOpen))
   End Function

   Public Function AddButtonImage(ByVal BtnHandle As Integer, ByVal State As TButtonState, ByVal Caption As String, ByVal ImgFile As String) As Boolean
      Return CBool(pdfAddButtonImage(m_Instance, BtnHandle, State, Caption, ImgFile))
   End Function

   Public Function AddButtonImageEx(ByVal BtnHandle As Integer, ByVal State As TButtonState, ByVal Caption As String, ByVal hBitmap As IntPtr) As Boolean
      Return CBool(pdfAddButtonImageEx(m_Instance, BtnHandle, State, Caption, hBitmap))
   End Function

   Public Function AddContinueTextA(ByVal AText As String) As Boolean
      Return CBool(pdfAddContinueTextA(m_Instance, AText))
   End Function

   Public Function AddContinueTextW(ByVal AText As String) As Boolean
      AddContinueTextW = CBool(pdfAddContinueTextW(m_Instance, AText))
   End Function

   Public Function AddDeviceNProcessColorants(ByVal DeviceNCS As Integer, ByVal Colorants() As String, ByVal NumColorants As Integer, ByVal ProcessCS As TExtColorSpace, ByVal Handle As Integer) As Boolean
      Return CBool(pdfAddDeviceNProcessColorants(m_Instance, DeviceNCS, Colorants, NumColorants, ProcessCS, Handle))
   End Function

   Public Function AddDeviceNSeparations(ByVal DeviceNCS As Integer, ByVal Colorants() As String, ByVal SeparationCS() As Integer, ByVal NumColorants As Integer) As Boolean
      Return CBool(pdfAddDeviceNSeparations(m_Instance, DeviceNCS, Colorants, SeparationCS, NumColorants))
   End Function

   Public Function AddFieldToFormAction(ByVal Action As Integer, ByVal Field As Integer, ByVal DoInclude As Boolean) As Boolean
      Return CBool(pdfAddFieldToFormAction(m_Instance, Action, Field, CInt(DoInclude)))
   End Function

   Public Function AddFieldToHideAction(ByVal HideAct As Integer, ByVal Field As Integer) As Boolean
      Return CBool(pdfAddFieldToHideAction(m_Instance, HideAct, Field))
   End Function

   Public Function AddFileCommentA(ByVal AText As String) As Boolean
      Return CBool(pdfAddFileCommentA(m_Instance, AText))
   End Function

   Public Function AddFileCommentW(ByVal AText As String) As Boolean
      Return CBool(pdfAddFileCommentW(m_Instance, AText))
   End Function

   Public Function AddFontSearchPathA(ByVal APath As String, ByVal Recursive As Integer) As Integer
      Return pdfAddFontSearchPathA(m_Instance, APath, Recursive)
   End Function

   Public Function AddFontSearchPathW(ByVal APath As String, ByVal Recursive As Integer) As Integer
      Return pdfAddFontSearchPathW(m_Instance, APath, Recursive)
   End Function

   Public Function AddImage(ByVal Filter As TCompressionFilter, ByVal Flags As TImageConversionFlags, ByRef Image As TPDFImage) As Boolean
      Return CBool(pdfAddImage(m_Instance, Filter, Flags, Image))
   End Function

   Public Function AddJavaScriptA(ByVal Name As String, ByVal Script As String) As Integer
      Return pdfAddJavaScriptA(m_Instance, Name, Script)
   End Function

   Public Function AddJavaScriptW(ByVal Name As String, ByVal Script As String) As Integer
      Return pdfAddJavaScriptW(m_Instance, Name, Script)
   End Function

   Public Function AddObjectToLayer(ByVal OCG As Integer, ByVal ObjType As TOCObject, ByVal Handle As Integer) As Boolean
      Return CBool(pdfAddObjectToLayer(m_Instance, OCG, ObjType, Handle))
   End Function

   Public Function AddPageLabelA(ByVal StartRange As Integer, ByVal Format As TPageLabelFormat, ByVal Prefix As String, ByVal AddNum As Integer) As Integer
      Return pdfAddPageLabelA(m_Instance, StartRange, Format, Prefix, AddNum)
   End Function

   Public Function AddPageLabelW(ByVal StartRange As Integer, ByVal Format As TPageLabelFormat, ByVal Prefix As String, ByVal AddNum As Integer) As Integer
      Return pdfAddPageLabelW(m_Instance, StartRange, Format, Prefix, AddNum)
   End Function

   Public Function AddRenderingIntentA(ByVal ICCFile As String) As Integer
      Return pdfAddRenderingIntentA(m_Instance, ICCFile)
   End Function

   Public Function AddRenderingIntentW(ByVal ICCFile As String) As Integer
      Return pdfAddRenderingIntentW(m_Instance, ICCFile)
   End Function

   Public Function AddRenderingIntentEx(ByRef Buffer() As Byte) As Integer
      Return pdfAddRenderingIntentEx(m_Instance, Buffer, Buffer.Length)
   End Function

   Public Function AddValToChoiceFieldA(ByVal Field As Integer, ByVal ExpValue As String, ByVal Value As String, ByVal Selected As Boolean) As Boolean
      Return CBool(pdfAddValToChoiceFieldA(m_Instance, Field, ExpValue, Value, CInt(Selected)))
   End Function

   Public Function AddValToChoiceFieldW(ByVal Field As Integer, ByVal ExpValue As String, ByVal Value As String, ByVal Selected As Boolean) As Boolean
      Return CBool(pdfAddValToChoiceFieldW(m_Instance, Field, ExpValue, Value, CInt(Selected)))
   End Function

   Public Function Append() As Boolean
      Return CBool(pdfAppend(m_Instance))
   End Function

   Public Function ApplyPattern(ByVal PattHandle As Integer, ByVal ColorMode As TColorMode, ByVal Color As Integer) As Boolean
      Return CBool(pdfApplyPattern(m_Instance, PattHandle, ColorMode, Color))
   End Function

   Public Function ApplyShading(ByVal ShadHandle As Integer) As Boolean
      Return CBool(pdfApplyShading(m_Instance, ShadHandle))
   End Function

   Public Function AttachFileA(ByVal FilePath As String, ByVal Description As String, ByVal Compress As Boolean) As Integer
      Return pdfAttachFileA(m_Instance, FilePath, Description, CInt(Compress))
   End Function

   Public Function AttachFileW(ByVal FilePath As String, ByVal Description As String, ByVal Compress As Boolean) As Integer
      Return pdfAttachFileW(m_Instance, FilePath, Description, CInt(Compress))
   End Function

   Public Function AttachFileExA(ByRef Buffer() As Byte, ByVal FileName As String, ByVal Description As String, ByVal Compress As Boolean) As Integer
      Return pdfAttachFileExA(m_Instance, Buffer, Buffer.Length(), FileName, Description, CInt(Compress))
   End Function

   Public Function AttachFileExW(ByRef Buffer() As Byte, ByVal FileName As String, ByVal Description As String, ByVal Compress As Boolean) As Integer
      Return pdfAttachFileExW(m_Instance, Buffer, Buffer.Length(), FileName, Description, CInt(Compress))
   End Function

   Public Function AutoTemplate(ByVal Templ As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfAutoTemplate(m_Instance, Templ, PosX, PosY, Width, Height))
   End Function

   Public Function BeginClipPath() As Boolean
      Return True ' Obsolete function! Nothing to do.
   End Function

   Public Function BeginContinueText(ByVal PosX As Double, ByVal PosY As Double) As Boolean
      Return CBool(pdfBeginContinueText(m_Instance, PosX, PosY))
   End Function

   Public Function BeginLayer(ByVal OCG As Integer) As Integer
      Return pdfBeginLayer(m_Instance, OCG)
   End Function

   Public Function BeginPattern(ByVal PatternType As TPatternType, ByVal TilingType As TTilingType, ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfBeginPattern(m_Instance, PatternType, TilingType, Width, Height)
   End Function

   Public Function BeginTemplate(ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfBeginTemplate(m_Instance, Width, Height)
   End Function

   Public Function Bezier_1_2_3(ByVal x1 As Double, ByVal y1 As Double, ByVal x2 As Double, ByVal y2 As Double, ByVal x3 As Double, ByVal y3 As Double) As Boolean
      Return CBool(pdfBezier_1_2_3(m_Instance, x1, y1, x2, y2, x3, y3))
   End Function

   Public Function Bezier_1_3(ByVal x1 As Double, ByVal y1 As Double, ByVal x3 As Double, ByVal y3 As Double) As Boolean
      Return CBool(pdfBezier_1_3(m_Instance, x1, y1, x3, y3))
   End Function

   Public Function BuildFamilyNameAndStyle(ByVal IFont As IntPtr, ByVal Name As System.Text.StringBuilder, ByVal Style As TFStyle) As Boolean
      If Name.Capacity < 128 Then Name.Capacity = 128
      Name.Length = 0
      Return CBool(fntBuildFamilyNameAndStyle(IFont, Name, Style))
   End Function

   Public Function Bezier_2_3(ByVal x2 As Double, ByVal y2 As Double, ByVal x3 As Double, ByVal y3 As Double) As Boolean
      Return CBool(pdfBezier_2_3(m_Instance, x2, y2, x3, y3))
   End Function

   Public Function CalcWidthHeight(ByVal OrgWidth As Double, ByVal OrgHeight As Double, ByVal ScaledWidth As Double, ByVal ScaledHeight As Double) As Double
      Return pdfCalcWidthHeight(m_Instance, OrgWidth, OrgHeight, ScaledWidth, ScaledHeight)
   End Function

   Public Function ChangeAnnotNameA(ByVal Handle As Integer, ByVal Name As String) As Boolean
      Return CBool(pdfChangeAnnotNameA(m_Instance, Handle, Name))
   End Function

   Public Function ChangeAnnotNameW(ByVal Handle As Integer, ByVal Name As String) As Boolean
      Return CBool(pdfChangeAnnotNameW(m_Instance, Handle, Name))
   End Function

   Public Function ChangeAnnotPos(ByVal Handle As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfChangeAnnotPos(m_Instance, Handle, PosX, PosY, Width, Height))
   End Function

   Public Function ChangeBookmarkA(ByVal ABmk As Integer, ByVal Title As String, ByVal Parent As Integer, ByVal DestPage As Integer, ByVal DoOpen As Boolean) As Boolean
      Return CBool(pdfChangeBookmarkA(m_Instance, ABmk, Title, Parent, DestPage, CInt(DoOpen)))
   End Function

   Public Function ChangeBookmarkW(ByVal ABmk As Integer, ByVal Title As String, ByVal Parent As Integer, ByVal DestPage As Integer, ByVal DoOpen As Boolean) As Boolean
      Return CBool(pdfChangeBookmarkW(m_Instance, ABmk, Title, Parent, DestPage, CInt(DoOpen)))
   End Function

   Public Function ChangeFont(ByVal AHandle As Integer) As Boolean
      Return CBool(pdfChangeFont(m_Instance, AHandle))
   End Function

   Public Function ChangeFontSize(ByVal Size As Double) As Boolean
      Return CBool(pdfChangeFontSize(m_Instance, Size))
   End Function

   Public Function ChangeFontStyle(ByVal Style As TFStyle) As Boolean
      Return CBool(pdfChangeFontStyle(m_Instance, Style))
   End Function

   Public Function ChangeFontStyleEx(ByVal Style As TFStyle) As Boolean
      Return CBool(pdfChangeFontStyleEx(m_Instance, Style))
   End Function

   Public Function ChangeJavaScriptA(ByVal AHandle As Integer, ByVal NewScript As String) As Boolean
      Return CBool(pdfChangeJavaScriptA(m_Instance, AHandle, NewScript))
   End Function

   Public Function ChangeJavaScriptW(ByVal AHandle As Integer, ByVal NewScript As String) As Boolean
      Return CBool(pdfChangeJavaScriptW(m_Instance, AHandle, NewScript))
   End Function

   Public Function ChangeJavaScriptActionA(ByVal AHandle As Integer, ByVal NewScript As String) As Boolean
      Return CBool(pdfChangeJavaScriptActionA(m_Instance, AHandle, NewScript))
   End Function

   Public Function ChangeJavaScriptActionW(ByVal AHandle As Integer, ByVal NewScript As String) As Boolean
      Return CBool(pdfChangeJavaScriptActionW(m_Instance, AHandle, NewScript))
   End Function

   Public Function ChangeJavaScriptNameA(ByVal AHandle As Integer, ByVal Name As String) As Boolean
      Return CBool(pdfChangeJavaScriptNameA(m_Instance, AHandle, Name))
   End Function

   Public Function ChangeJavaScriptNameW(ByVal AHandle As Integer, ByVal Name As String) As Boolean
      Return CBool(pdfChangeJavaScriptNameW(m_Instance, AHandle, Name))
   End Function

   Public Function ChangeLinkAnnot(ByVal Handle As Integer, ByVal URL As String) As Boolean
      Return CBool(pdfChangeLinkAnnot(m_Instance, Handle, URL))
   End Function

   Public Function CheckCollection() As Boolean
      Return CBool(pdfCheckCollection(m_Instance))
   End Function

   Public Function CheckFieldNames() As Integer
      Return pdfCheckFieldNames(m_Instance)
   End Function

   Public Function CheckConformance(ByVal ConfType As TConformanceType, ByVal Options As TCheckOptions, ByVal UserData As IntPtr, ByVal OnFontNotFound As TOnFontNotFoundProc, ByVal OnReplaceICCProfile As TOnReplaceICCProfile) As Integer
      Return pdfCheckConformance(m_Instance, ConfType, Options, UserData, OnFontNotFound, OnReplaceICCProfile)
   End Function

   Public Function ClearAutoTemplates() As Boolean
      Return CBool(pdfClearAutoTemplates(m_Instance))
   End Function

   Public Function ClearHostFonts() As Boolean
      Return CBool(pdfClearHostFonts(m_Instance))
   End Function

   Public Function ClipPath(ByVal ClipMode As TClippingMode, ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfClipPath(m_Instance, ClipMode, FillMode))
   End Function

   Public Function CloseAndSignFile(ByVal CertFile As String, ByVal Password As String, ByVal Reason As String, ByVal Location As String) As Boolean
      Return CBool(pdfCloseAndSignFile(m_Instance, CertFile, Password, Reason, Location))
   End Function

   Public Function CloseAndSignFileEx(ByVal OpenPwd As String, ByVal OwnerPwd As String, ByVal KeyLen As TKeyLen, ByVal Restrict As TRestrictions, ByVal CertFile As String, ByVal Password As String, ByVal Reason As String, ByVal Location As String) As Boolean
      Return CBool(pdfCloseAndSignFileEx(m_Instance, OpenPwd, OwnerPwd, KeyLen, Restrict, CertFile, Password, Reason, Location))
   End Function

   Public Function CloseAndSignFileExt(ByRef SigParms As TPDFSigParms) As Boolean
      Dim p As TPDFSigParms_I = New TPDFSigParms_I()
      p.StructSize = Marshal.SizeOf(p)
      p.ContactInfoW = SigParms.ContactInfo
      p.Encrypt = Convert.ToInt32(SigParms.Encrypt)
      p.HashType = SigParms.HashType
      p.KeyLen = SigParms.KeyLen
      p.LocationW = SigParms.Location
      p.OpenPwd = SigParms.OpenPwd
      p.OwnerPwd = SigParms.OwnerPwd
      p.PKCS7ObjLen = SigParms.PKCS7ObjLen
      p.ReasonW = SigParms.Reason
      p.SignerW = SigParms.Signer
      p.Restrict = SigParms.Restrict
      If (pdfCloseAndSignFileExt(m_Instance, p) = 0) Then Return False
      ReDim SigParms.Range1(p.Range1Len - 1)
      Marshal.Copy(p.Range1, SigParms.Range1, 0, p.Range1Len)
      If (p.Range2Len > 0) Then
         ReDim SigParms.Range2(p.Range2Len - 1)
         Marshal.Copy(p.Range2, SigParms.Range2, 0, p.Range2Len)
      Else
         SigParms.Range2 = Nothing
      End If
      Return True
   End Function

   Public Function CloseFile() As Boolean
      Return CBool(pdfCloseFile(m_Instance))
   End Function

   Public Function CloseFileEx(ByVal OpenPwd As String, ByVal OwnerPwd As String, ByVal KeyLen As TKeyLen, ByVal Restrict As TRestrictions) As Boolean
      Return CBool(pdfCloseFileEx(m_Instance, OpenPwd, OwnerPwd, KeyLen, Restrict))
   End Function

   Public Function CloseImage() As Boolean
      Return CBool(pdfCloseImage(m_Instance))
   End Function

   Public Function CloseImportFile() As Boolean
      Return CBool(pdfCloseImportFile(m_Instance))
   End Function

   Public Function ClosePath(ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfClosePath(m_Instance, FillMode))
   End Function

   Public Function CloseTag() As Boolean
      Return CBool(pdfCloseTag(m_Instance))
   End Function

   Public Function CMYK(ByVal C As Byte, ByVal M As Byte, ByVal Y As Byte, ByVal K As Byte) As Integer
      If C >= &H80S Then
         Return (K Or (Y * &H100) Or (M * &H10000) Or (CShort(C And &H7FS) * &H1000000) Or &H80000000)
      Else
         Return (K Or (Y * &H100) Or (M * &H10000) Or (C * &H1000000))
      End If
   End Function

   Public Function ComputeBBox(ByRef BBox As TPDFRect, ByVal Flags As TCompBBoxFlags) As Boolean
      Return CBool(pdfComputeBBox(m_Instance, BBox, Flags))
   End Function

   Public Sub ConnectPageBreakEvent(ByVal Connect As Boolean)
      If Connect Then
         pdfSetOnPageBreakProc(m_Instance, Nothing, m_AddrOnPageBreak)
      Else
         pdfSetOnPageBreakProc(m_Instance, Nothing, Nothing)
      End If
   End Sub

   Public Function ConvColor(ByVal Color As IntPtr, ByVal NumComps As Integer, ByVal SourceCS As TExtColorSpace, ByVal IColorSpace As IntPtr, ByVal DestCS As TExtColorSpace) As Integer
      Return pdfConvColor(Color, NumComps, SourceCS, IColorSpace, DestCS)
   End Function

   Public Function ConvertColors(ByVal Flags As TColorConvFlags) As Boolean
      Return CBool(pdfConvertColors(m_Instance, Flags, IntPtr.Zero))
   End Function

   Public Function ConvToUnicode(ByVal AString As String, ByVal CP As TCodepage) As String
      Dim retval As String
      Dim pBuffer As IntPtr

      pBuffer = pdfConvToUnicode(m_Instance, AString, CP)
      retval = Marshal.PtrToStringUni(pBuffer)
      Return retval
   End Function

   Public Function CopyIntPtrArray(ByVal Source As IntPtr, ByVal Count As Integer) As IntPtr()
      Dim retval(Count - 1) As IntPtr
      pdfCopyMemIntPtr(Source, retval, Count * IntPtr.Size)
      Return retval
   End Function

   Public Function Create3DAnnot(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Name As String, ByVal U3DFile As String, ByVal ImageFile As String) As Integer
      Return pdfCreate3DAnnot(m_Instance, PosX, PosY, Width, Height, Author, Name, U3DFile, ImageFile)
   End Function

   Public Function Create3DBackground(ByVal IView As IntPtr, ByVal BackColor As Integer) As Boolean
      Return CBool(pdfCreate3DBackground(m_Instance, IView, BackColor))
   End Function

   Public Function Create3DGotoViewAction(ByVal Base3DAnnot As Integer, ByVal IView As IntPtr, ByVal Named As T3DNamedAction) As Integer
      Return pdfCreate3DGotoViewAction(m_Instance, Base3DAnnot, IView, Named)
   End Function

   Public Function Create3DProjection(ByVal IView As IntPtr, ByVal ProjType As T3DProjType, ByVal ScaleType As T3DScaleType, ByVal Diameter As Double, ByVal FOV As Double) As Boolean
      Return CBool(pdfCreate3DProjection(m_Instance, IView, ProjType, ScaleType, Diameter, FOV))
   End Function

   Public Function Create3DView(ByVal Base3DAnnot As Integer, ByVal Name As String, ByVal SetAsDefault As Boolean, ByRef Matrix() As Double, ByVal CamDistance As Double, ByVal RM As T3DRenderingMode, ByVal LS As T3DLightingSheme) As IntPtr
      Try
         ' A 3D Matrix contains exactly 12 elements
         If UBound(Matrix) < 11 Then
            Return pdfCreate3DView(m_Instance, Base3DAnnot, Name, CInt(SetAsDefault), Nothing, CamDistance, RM, LS)
         Else
            Return pdfCreate3DView(m_Instance, Base3DAnnot, Name, CInt(SetAsDefault), Matrix, CamDistance, RM, LS)
         End If
      Catch
         Return pdfCreate3DView(m_Instance, Base3DAnnot, Name, CInt(SetAsDefault), Nothing, CamDistance, RM, LS)
      End Try
   End Function

   Public Function CreateAnnotAP(ByVal Annot As Integer) As Integer
      Return pdfCreateAnnotAP(m_Instance, Annot)
   End Function

   Public Function CreateArticleThreadA(ByVal ThreadName As String) As Integer
      Return pdfCreateArticleThreadA(m_Instance, ThreadName)
   End Function

   Public Function CreateArticleThreadW(ByVal ThreadName As String) As Integer
      Return pdfCreateArticleThreadW(m_Instance, ThreadName)
   End Function

   Public Function CreateAxialShading(ByVal sx As Double, ByVal sy As Double, ByVal eX As Double, ByVal eY As Double, ByVal SCenter As Double, ByVal SColor As Integer, ByVal EColor As Integer, ByVal Extend1 As Integer, ByVal Extend2 As Integer) As Integer
      Return pdfCreateAxialShading(m_Instance, sx, sy, eX, eY, SCenter, SColor, EColor, Extend1, Extend2)
   End Function

   Public Function CreateButtonA(ByVal Name As String, ByVal Caption As String, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfCreateButtonA(m_Instance, Name, Caption, Parent, PosX, PosY, Width, Height)
   End Function

   Public Function CreateButtonW(ByVal Name As String, ByVal Caption As String, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfCreateButtonW(m_Instance, Name, Caption, Parent, PosX, PosY, Width, Height)
   End Function

   Public Function CreateCheckBox(ByVal Name As String, ByVal ExpValue As String, ByVal Checked As Boolean, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfCreateCheckBox(m_Instance, Name, ExpValue, CInt(Checked), Parent, PosX, PosY, Width, Height)
   End Function

   Public Function CreateCIEColorSpace(ByVal Base As TExtColorSpace, ByRef WhitePoint() As Single, ByRef BlackPoint() As Single, ByRef Gamma() As Single, ByRef Matrix() As Single) As Integer
      Return pdfCreateCIEColorSpace(m_Instance, Base, WhitePoint, BlackPoint, Gamma, Matrix)
   End Function

   Public Function CreateCollection(ByVal View As TColView) As Boolean
      Return CBool(pdfCreateCollection(m_Instance, View))
   End Function

   Public Function CreateCollectionFieldA(ByVal ColType As TColColumnType, ByVal Column As Integer, ByVal Name As String, ByVal Key As String, ByVal Visible As Boolean, ByVal Editable As Boolean) As Integer
      Return pdfCreateCollectionFieldA(m_Instance, ColType, Column, Name, Key, CInt(Visible), CInt(Editable))
   End Function

   Public Function CreateCollectionFieldW(ByVal ColType As TColColumnType, ByVal Column As Integer, ByVal Name As String, ByVal Key As String, ByVal Visible As Boolean, ByVal Editable As Boolean) As Integer
      Return pdfCreateCollectionFieldW(m_Instance, ColType, Column, Name, Key, CInt(Visible), CInt(Editable))
   End Function
   ' 32 Bit Implementation due to the stupid data type handling of .Net!
   Public Function CreateColItemDate(ByVal EmbFile As Integer, ByVal Key As String, ByVal DateVal As Integer, ByVal Prefix As String) As Boolean
      Return CBool(pdfCreateColItemDate(m_Instance, EmbFile, Key, DateVal, Prefix))
   End Function

   Public Function CreateColItemNumber(ByVal EmbFile As Integer, ByVal Key As String, ByVal Value As Double, ByVal Prefix As String) As Boolean
      Return CBool(pdfCreateColItemNumber(m_Instance, EmbFile, Key, Value, Prefix))
   End Function

   Public Function CreateColItemStringA(ByVal EmbFile As Integer, ByVal Key As String, ByVal Value As String, ByVal Prefix As String) As Boolean
      Return CBool(pdfCreateColItemStringA(m_Instance, EmbFile, Key, Value, Prefix))
   End Function

   Public Function CreateColItemStringW(ByVal EmbFile As Integer, ByVal Key As String, ByVal Value As String, ByVal Prefix As String) As Boolean
      Return CBool(pdfCreateColItemStringW(m_Instance, EmbFile, Key, Value, Prefix))
   End Function

   Public Function CreateComboBox(ByVal Name As String, ByVal Sort As Integer, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfCreateComboBox(m_Instance, Name, Sort, Parent, PosX, PosY, Width, Height)
   End Function

   Public Function CreateDeviceNColorSpace(ByRef Colorants() As String, ByVal PostScriptFunc As String, ByVal Alternate As TExtColorSpace, ByVal Handle As Integer) As Integer
      Return pdfCreateDeviceNColorSpace(m_Instance, Colorants, Colorants.Length, PostScriptFunc, Alternate, Handle)
   End Function

   Public Function CreateExtGState(ByRef GS As TPDFExtGState) As Integer
      Return pdfCreateExtGState(m_Instance, GS)
   End Function

   Public Function CreateGoToAction(ByVal DestType As TDestType, ByVal PageNum As Integer, ByVal a As Double, ByVal b As Double, ByVal C As Double, ByVal d As Double) As Integer
      Return pdfCreateGoToAction(m_Instance, DestType, PageNum, a, b, C, d)
   End Function

   Public Function CreateGoToActionEx(ByVal NamedDest As Integer) As Integer
      Return pdfCreateGoToActionEx(m_Instance, NamedDest)
   End Function

   Public Function CreateGoToRAction(ByVal FileName As String, ByVal PageNum As Integer) As Integer
      Return pdfCreateGoToRAction(m_Instance, FileName, PageNum)
   End Function

   Public Function CreateGoToRActionExA(ByVal FileName As String, ByVal DestName As String, ByVal NewWindow As Boolean) As Integer
      Return pdfCreateGoToRActionExA(m_Instance, FileName, DestName, CInt(NewWindow))
   End Function

   Public Function CreateGoToRActionExW(ByVal FileName As String, ByVal DestName As String, ByVal NewWindow As Boolean) As Integer
      Return pdfCreateGoToRActionExW(m_Instance, FileName, DestName, CInt(NewWindow))
   End Function

   Public Function CreateGroupField(ByVal Name As String, ByVal Parent As Integer) As Integer
      Return pdfCreateGroupField(m_Instance, Name, Parent)
   End Function

   Public Function CreateHideAction(ByVal Field As Integer, ByVal Hide As Integer) As Integer
      Return pdfCreateHideAction(m_Instance, Field, Hide)
   End Function

   Public Function CreateICCBasedColorSpaceA(ByVal ICCProfile As String) As Integer
      Return pdfCreateICCBasedColorSpaceA(m_Instance, ICCProfile)
   End Function

   Public Function CreateICCBasedColorSpaceW(ByVal ICCProfile As String) As Integer
      Return pdfCreateICCBasedColorSpaceW(m_Instance, ICCProfile)
   End Function

   Public Function CreateImageA(ByVal FileName As String, ByVal Format As TImageFormat) As Boolean
      Return CBool(pdfCreateImageA(m_Instance, FileName, Format))
   End Function

   Public Function CreateImageW(ByVal FileName As String, ByVal Format As TImageFormat) As Boolean
      Return CBool(pdfCreateImageW(m_Instance, FileName, Format))
   End Function

   Public Function CreateImpDataActionA(ByVal DataFile As String) As Integer
      Return pdfCreateImpDataActionA(m_Instance, DataFile)
   End Function

   Public Function CreateImpDataActionW(ByVal DataFile As String) As Integer
      Return pdfCreateImpDataActionW(m_Instance, DataFile)
   End Function

   Public Function CreateIndexedColorSpace(ByVal Base As TExtColorSpace, ByVal Handle As Integer, ByRef ColorTable() As Byte, ByVal NumColors As Integer) As Integer
      CreateIndexedColorSpace = pdfCreateIndexedColorSpace(m_Instance, Base, Handle, ColorTable, NumColors)
   End Function

   Public Function CreateJSActionA(ByVal Script As String) As Integer
      Return pdfCreateJSActionA(m_Instance, Script)
   End Function

   Public Function CreateJSActionW(ByVal Script As String) As Integer
      Return pdfCreateJSActionW(m_Instance, Script)
   End Function

   Public Function CreateLaunchAction(ByVal OP As TFileOP, ByVal FileName As String, ByVal DefDir As String, ByVal Param As String, ByVal NewWindow As Integer) As Integer
      Return pdfCreateLaunchAction(m_Instance, OP, FileName, DefDir, Param, NewWindow)
   End Function

   Public Function CreateListBox(ByVal Name As String, ByVal Sort As Integer, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfCreateListBox(m_Instance, Name, Sort, Parent, PosX, PosY, Width, Height)
   End Function

   Public Function CreateNamedAction(ByVal Action As TNamedAction) As Integer
      Return pdfCreateNamedAction(m_Instance, Action)
   End Function

   Public Function CreateNamedDestA(ByVal Name As String, ByVal DestPage As Integer, ByVal DestType As TDestType, ByVal a As Double, ByVal b As Double, ByVal c As Double, ByVal d As Double) As Integer
      Return pdfCreateNamedDestA(m_Instance, Name, DestPage, DestType, a, b, c, d)
   End Function

   Public Function CreateNamedDestW(ByVal Name As String, ByVal DestPage As Integer, ByVal DestType As TDestType, ByVal a As Double, ByVal b As Double, ByVal c As Double, ByVal d As Double) As Integer
      Return pdfCreateNamedDestW(m_Instance, Name, DestPage, DestType, a, b, c, d)
   End Function

   Public Function CreateNewPDFA(ByVal OutPDF As String) As Boolean
      Return CBool(pdfCreateNewPDFA(m_Instance, OutPDF))
   End Function

   Public Function CreateNewPDFW(ByVal OutPDF As String) As Boolean
      Return CBool(pdfCreateNewPDFW(m_Instance, OutPDF))
   End Function

   Public Function CreateOCGA(ByVal Name As String, ByVal DisplayInUI As Boolean, ByVal Visible As Boolean, ByVal Intent As TOCGIntent) As Integer
      Return pdfCreateOCGA(m_Instance, Name, Convert.ToInt32(DisplayInUI), Convert.ToInt32(Visible), Intent)
   End Function

   Public Function CreateOCGW(ByVal Name As String, ByVal DisplayInUI As Boolean, ByVal Visible As Boolean, ByVal Intent As TOCGIntent) As Integer
      Return pdfCreateOCGW(m_Instance, Name, Convert.ToInt32(DisplayInUI), Convert.ToInt32(Visible), Intent)
   End Function

   Public Function CreateOCMD(ByVal Visibility As TOCVisibility, ByVal OCGs() As Integer) As Integer
      Return pdfCreateOCMD(m_Instance, Visibility, OCGs, OCGs.Length)
   End Function

   Public Function CreateRadialShading(ByVal sx As Double, ByVal sy As Double, ByVal R1 As Double, ByVal eX As Double, ByVal eY As Double, ByVal R2 As Double, ByVal SCenter As Double, ByVal SColor As Integer, ByVal EColor As Integer, ByVal Extend1 As Integer, ByVal Extend2 As Integer) As Integer
      Return pdfCreateRadialShading(m_Instance, sx, sy, R1, eX, eY, R2, SCenter, SColor, EColor, Extend1, Extend2)
   End Function

   Public Function CreateRadioButton(ByVal Name As String, ByVal ExpValue As String, ByVal Checked As Integer, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfCreateRadioButton(m_Instance, Name, ExpValue, Checked, Parent, PosX, PosY, Width, Height)
   End Function

   Public Function CreateResetAction() As Integer
      Return pdfCreateResetAction(m_Instance)
   End Function

   Public Function CreateSeparationCS(ByVal Colorant As String, ByVal Alternate As TExtColorSpace, ByVal Handle As Integer, ByVal Color As Integer) As Integer
      Return pdfCreateSeparationCS(m_Instance, Colorant, Alternate, Handle, Color)
   End Function

   Public Function CreateSetOCGStateAction(ByVal _On() As Integer, ByVal Off() As Integer, ByVal Toggle() As Integer, ByVal PreserveRB As Boolean) As Integer
      Dim numOn As Integer
      Dim numOff As Integer
      Dim numToggle As Integer
      If Not IsNothing(_On) Then
         numOn = _On.Length
      End If
      If Not IsNothing(Off) Then
         numOff = Off.Length
      End If
      If Not IsNothing(Toggle) Then
         numToggle = Toggle.Length
      End If
      Return pdfCreateSetOCGStateAction(m_Instance, _On, numOn, Off, numOff, Toggle, numToggle, Convert.ToInt32(PreserveRB))
   End Function

   Public Function CreateSigField(ByVal Name As String, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfCreateSigField(m_Instance, Name, Parent, PosX, PosY, Width, Height)
   End Function

   Public Function CreateSigFieldAP(ByVal SigField As Integer) As Integer
      Return pdfCreateSigFieldAP(m_Instance, SigField)
   End Function

   Public Function CreateStdPattern(ByVal Pattern As TStdPattern, ByVal LineWidth As Double, ByVal Distance As Double, ByVal LineColor As Integer, ByVal BackColor As Integer) As Integer
      Return pdfCreateStdPattern(m_Instance, Pattern, LineWidth, Distance, LineColor, BackColor)
   End Function

   Public Function CreateStructureTree() As Boolean
      Return CBool(pdfCreateStructureTree(m_Instance))
   End Function

   Public Function CreateSubmitAction(ByVal Flags As TSubmitFlags, ByVal URL As String) As Integer
      Return pdfCreateSubmitAction(m_Instance, Flags, URL)
   End Function

   Public Function CreateTextField(ByVal Name As String, ByVal Parent As Integer, ByVal Multiline As Integer, ByVal MaxLen As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
      Return pdfCreateTextField(m_Instance, Name, Parent, Multiline, MaxLen, PosX, PosY, Width, Height)
   End Function

   Public Function CreateURIAction(ByVal URL As String) As Integer
      Return pdfCreateURIAction(m_Instance, URL)
   End Function

   Public Function DecryptPDFA(ByVal FileName As String, ByVal PwdType As TPwdType, ByVal Password As String) As Integer
      Return pdfDecryptPDFA(m_Instance, FileName, PwdType, Password)
   End Function

   Public Function DecryptPDFW(ByVal FileName As String, ByVal PwdType As TPwdType, ByVal Password As String) As Integer
      Return pdfDecryptPDFW(m_Instance, FileName, PwdType, Password)
   End Function

   Public Sub DeleteAcroForm()
      pdfDeleteAcroForm(m_Instance)
   End Sub

   Public Function DeleteActionFromObj(ByVal ObjType As TObjType, ByVal ActHandle As Integer, ByVal ObjHandle As Integer) As Boolean
      Return CBool(pdfDeleteActionFromObj(m_Instance, ObjType, ActHandle, ObjHandle))
   End Function

   Public Function DeleteActionFromObjEx(ByVal ObjType As TObjType, ByVal ObjHandle As Integer, ByVal ActIndex As Integer) As Boolean
      Return CBool(pdfDeleteActionFromObjEx(m_Instance, ObjType, ObjHandle, ActIndex))
   End Function

   Public Function DeleteAnnotation(ByVal Handle As Integer) As Boolean
      Return CBool(pdfDeleteAnnotation(m_Instance, Handle))
   End Function

   Public Function DeleteBookmark(ByVal ABmk As Integer) As Integer
      Return pdfDeleteBookmark(m_Instance, ABmk)
   End Function

   Public Function DeleteEmbeddedFile(ByVal Handle As Integer) As Boolean
      Return CBool(pdfDeleteEmbeddedFile(m_Instance, Handle))
   End Function

   Public Function DeleteField(ByVal Field As Integer) As Boolean
      Return CBool(pdfDeleteField(m_Instance, Field))
   End Function

   Public Function DeleteFieldEx(ByVal Name As String) As Boolean
      Return CBool(pdfDeleteFieldEx(m_Instance, Name))
   End Function

   Public Sub DeleteJavaScripts(ByVal DelJavaScriptActions As Boolean)
      pdfDeleteJavaScripts(m_Instance, CInt(DelJavaScriptActions))
   End Sub

   Public Function DeletePage(ByVal PageNum As Integer) As Integer
      Return pdfDeletePage(m_Instance, PageNum)
   End Function

   Public Sub DeletePageLabels()
      pdfDeletePageLabels(m_Instance)
   End Sub

   Public Function DeleteSeparationInfo(ByVal AllPages As Boolean) As Boolean
      Return CBool(pdfDeleteSeparationInfo(m_Instance, CInt(AllPages)))
   End Function

   Public Function DeleteTemplate(ByVal Handle As Integer) As Boolean
      Return CBool(pdfDeleteTemplate(m_Instance, Handle))
   End Function

   Public Function DeleteTemplateEx(ByVal Index As Integer) As Boolean
      Return CBool(pdfDeleteTemplateEx(m_Instance, Index))
   End Function

   Public Sub DeleteXFAForm()
      pdfDeleteXFAForm(m_Instance)
   End Sub

   Public Function DrawArc(ByVal PosX As Double, ByVal PosY As Double, ByVal Radius As Double, ByVal StartAngle As Double, ByVal EndAngle As Double) As Boolean
      Return CBool(pdfDrawArc(m_Instance, PosX, PosY, Radius, StartAngle, EndAngle))
   End Function

   Public Function DrawArcEx(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal StartAngle As Double, ByVal EndAngle As Double) As Boolean
      Return CBool(pdfDrawArcEx(m_Instance, PosX, PosY, Width, Height, StartAngle, EndAngle))
   End Function

   Public Function DrawChord(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal StartAngle As Double, ByVal EndAngle As Double, ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfDrawChord(m_Instance, PosX, PosY, Width, Height, StartAngle, EndAngle, FillMode))
   End Function

   Public Function DrawCircle(ByVal PosX As Double, ByVal PosY As Double, ByVal Radius As Double, ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfDrawCircle(m_Instance, PosX, PosY, Radius, FillMode))
   End Function

   Public Function DrawPie(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal StartAngle As Double, ByVal EndAngle As Double, ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfDrawPie(m_Instance, PosX, PosY, Width, Height, StartAngle, EndAngle, FillMode))
   End Function

   Public Function EditPage(ByVal PageNum As Integer) As Boolean
      Return CBool(pdfEditPage(m_Instance, PageNum))
   End Function

   Public Function EditTemplate(ByVal Index As Integer) As Boolean
      Return CBool(pdfEditTemplate(m_Instance, Index))
   End Function

   Public Function EditTemplate2(ByVal Handle As Integer) As Boolean
      Return CBool(pdfEditTemplate2(m_Instance, Handle))
   End Function

   Public Function Ellipse(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfEllipse(m_Instance, PosX, PosY, Width, Height, FillMode))
   End Function

   Public Function EncryptPDFA(ByVal FileName As String, ByVal OpenPwd As String, ByVal OwnerPwd As String, ByVal KeyLen As TKeyLen, ByVal Restrict As TRestrictions) As Integer
      Return pdfEncryptPDFA(m_Instance, FileName, OpenPwd, OwnerPwd, KeyLen, Restrict)
   End Function

   Public Function EncryptPDFW(ByVal FileName As String, ByVal OpenPwd As String, ByVal OwnerPwd As String, ByVal KeyLen As TKeyLen, ByVal Restrict As TRestrictions) As Integer
      Return pdfEncryptPDFW(m_Instance, FileName, OpenPwd, OwnerPwd, KeyLen, Restrict)
   End Function

   Public Function EndContinueText() As Boolean
      Return CBool(pdfEndContinueText(m_Instance))
   End Function

   Public Function EndPage() As Boolean
      Return CBool(pdfEndPage(m_Instance))
   End Function

   Public Function EndLayer() As Boolean
      EndLayer = CBool(pdfEndLayer(m_Instance))
   End Function

   Public Function EndPattern() As Boolean
      Return CBool(pdfEndPattern(m_Instance))
   End Function

   Public Function EndTemplate() As Boolean
      Return CBool(pdfEndTemplate(m_Instance))
   End Function

   Public Function EnumDocFonts() As Integer ' With events
      EnumDocFonts = pdfEnumDocFonts(m_Instance, IntPtr.Zero, m_AddrEnumDocFonts)
   End Function

   Public Function EnumDocFonts(ByVal EnumProc As TEnumFontsProc2) As Integer ' With callback function
      m_AddrEnumDocFonts = EnumProc
      EnumDocFonts = pdfEnumDocFonts(m_Instance, IntPtr.Zero, m_AddrEnumDocFonts)
   End Function

   Public Function EnumHostFonts() As Integer ' With events
      Return pdfEnumHostFonts(m_Instance, IntPtr.Zero, m_AddrEnumFonts)
   End Function

   Public Function EnumHostFonts(ByVal EnumProc As TEnumFontsProc) As Integer ' With callback function
      m_AddrEnumFonts = EnumProc
      Return pdfEnumHostFonts(m_Instance, IntPtr.Zero, m_AddrEnumFonts)
   End Function

   Public Function EnumHostFontsEx() As Integer ' With events
      Return pdfEnumHostFontsEx(m_Instance, IntPtr.Zero, m_AddrEnumFontsEx)
   End Function

   Public Function EnumHostFontsEx(ByVal EnumProc As TEnumFontsProcEx) As Integer ' With callback function
      m_AddrEnumFontsEx = EnumProc
      Return pdfEnumHostFontsEx(m_Instance, IntPtr.Zero, m_AddrEnumFontsEx)
   End Function

   Public Function ExchangeBookmarks(ByVal Bmk1 As Integer, ByVal Bmk2 As Integer) As Boolean
      Return CBool(pdfExchangeBookmarks(m_Instance, Bmk1, Bmk2))
   End Function

   Public Function ExchangePages(ByVal First As Integer, ByVal Second As Integer) As Boolean
      Return CBool(pdfExchangePages(m_Instance, First, Second))
   End Function

   Public Function FileAttachAnnotA(ByVal PosX As Double, ByVal PosY As Double, ByVal Icon As TFileAttachIcon, ByVal Author As String, ByVal Desc As String, ByVal AFile As String, ByVal Compress As Boolean) As Integer
      Return pdfFileAttachAnnotA(m_Instance, PosX, PosY, Icon, Author, Desc, AFile, CInt(Compress))
   End Function

   Public Function FileAttachAnnotW(ByVal PosX As Double, ByVal PosY As Double, ByVal Icon As TFileAttachIcon, ByVal Author As String, ByVal Desc As String, ByVal AFile As String, ByVal Compress As Boolean) As Integer
      Return pdfFileAttachAnnotW(m_Instance, PosX, PosY, Icon, Author, Desc, AFile, CInt(Compress))
   End Function

   Public Function FileLinkA(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal AFilePath As String) As Integer
      Return pdfFileLinkA(m_Instance, PosX, PosY, Width, Height, AFilePath)
   End Function

   Public Function FileLinkW(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal AFilePath As String) As Integer
      Return pdfFileLinkW(m_Instance, PosX, PosY, Width, Height, AFilePath)
   End Function

   Public Function FindBookmarkA(ByVal DestPage As Integer, ByVal Title As String) As Integer
      Return pdfFindBookmarkA(m_Instance, DestPage, Title)
   End Function

   Public Function FindBookmarkW(ByVal DestPage As Integer, ByVal Title As String) As Integer
      Return pdfFindBookmarkW(m_Instance, DestPage, Title)
   End Function

   Public Function FindFieldA(ByVal Name As String) As Integer
      FindFieldA = pdfFindFieldA(m_Instance, Name)
   End Function

   Public Function FindFieldW(ByVal Name As String) As Integer
      FindFieldW = pdfFindFieldW(m_Instance, Name)
   End Function

   Public Function FindLinkAnnot(ByVal URL As String) As Integer
      Return pdfFindLinkAnnot(m_Instance, URL)
   End Function

   Public Function FindNextBookmark() As Integer
      Return pdfFindNextBookmark(m_Instance)
   End Function

   Public Function FinishSignature(ByVal PKCS7Obj() As Byte) As Boolean
      Return pdfFinishSignature(m_Instance, PKCS7Obj, PKCS7Obj.Length) <> 0
   End Function

   Public Function FlattenAnnots(ByVal Flags As TAnnotFlattenFlags) As Integer
      Return pdfFlattenAnnots(m_Instance, Flags)
   End Function

   Public Function FlattenForm() As Boolean
      Return CBool(pdfFlattenForm(m_Instance))
   End Function

   Public Function FlushPageContent(ByRef Stack As TPDFStack) As Boolean
      Return CBool(pdfFlushPageContent(m_Instance, Stack))
   End Function

   Public Sub FreeImageBuffer()
      pdfFreeImageBuffer(m_Instance)
   End Sub

   Public Function FreePDF() As Boolean
      Return CBool(pdfFreePDF(m_Instance))
   End Function

   Public Function FreeTextAnnotA(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal AText As String, ByVal Align As TTextAlign) As Integer
      Return pdfFreeTextAnnotA(m_Instance, PosX, PosY, Width, Height, Author, AText, Align)
   End Function

   Public Function FreeTextAnnotW(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal AText As String, ByVal Align As TTextAlign) As Integer
      Return pdfFreeTextAnnotW(m_Instance, PosX, PosY, Width, Height, Author, AText, Align)
   End Function

   Public Function FreeUniBuf() As Boolean
      Return CBool(pdfFreeUniBuf(m_Instance))
   End Function

   Public Function GetActionCount() As Integer
      Return pdfGetActionCount(m_Instance)
   End Function

   Public Function GetActionType(ByVal ActHandle As Integer) As Integer
      Return pdfGetActionType(m_Instance, ActHandle)
   End Function

   Public Function GetActionTypeEx(ByVal ObjType As TObjType, ByVal ObjHandle As Integer, ByVal ActIndex As Integer) As Integer
      Return pdfGetActionTypeEx(m_Instance, ObjType, ObjHandle, ActIndex)
   End Function

   Public Function GetActiveFont() As Integer
      Return pdfGetActiveFont(m_Instance)
   End Function

   Public Function GetAllocBy() As Integer
      Return pdfGetAllocBy(m_Instance)
   End Function

   Public Function GetAnnot(ByVal Handle As Integer, ByRef Annot As TPDFAnnotation) As Boolean
      Dim retval As TPDFAnnotation_I
      If pdfGetAnnot(m_Instance, Handle, retval) = 0 Then
         Annot = Nothing
         Return False
      End If
      GetIntAnnot(retval, Annot)
      Return True
   End Function

   Public Function GetAnnotBBox(ByVal Handle As Integer, ByRef BBox As TPDFRect) As Boolean
      Return CBool(pdfGetAnnotBBox(m_Instance, Handle, BBox))
   End Function

   Public Function GetAnnotCount() As Integer
      Return pdfGetAnnotCount(m_Instance)
   End Function

   Public Function GetAnnotEx(ByVal Handle As Integer, ByRef Annot As TPDFAnnotationEx) As Boolean
      Dim retval As TPDFAnnotationEx_I
      If pdfGetAnnotEx(m_Instance, Handle, retval) = 0 Then
         Annot = Nothing
         Return False
      End If
      GetIntAnnotEx(retval, Annot)
      Return True
   End Function

   Public Function GetAnnotFlags() As Integer
      Return pdfGetAnnotFlags(m_Instance)
   End Function

   Public Function GetAnnotLink(ByVal Handle As Integer) As String
      Dim pBuffer As IntPtr
      pBuffer = pdfGetAnnotLink(m_Instance, Handle)
      Return ToString(pBuffer, False)
   End Function

   Public Function GetAnnotType(ByVal Handle As Integer) As Integer
      Return pdfGetAnnotType(m_Instance, Handle)
   End Function

   Public Function GetAscent() As Double
      Return pdfGetAscent(m_Instance)
   End Function

   Public Function GetBBox(ByVal Boundary As TPageBoundary, ByRef BBox As TPDFRect) As Integer
      Return pdfGetBBox(m_Instance, Boundary, BBox)
   End Function

   Public Function GetBidiMode() As TPDFBidiMode
      Return CType(pdfGetBidiMode(m_Instance), TPDFBidiMode)
   End Function

   Public Function GetBookmark(ByVal AHandle As Integer, ByRef Bmk As TBookmark) As Boolean
      Dim b As TBookmark_I
      If pdfGetBookmark(m_Instance, AHandle, b) = 0 Then
         Bmk = Nothing
         Return False
      End If
      Bmk.Color = b.Color
      Bmk.DestPage = b.DestPage
      Bmk.DestPos = b.DestPos
      Bmk.DestType = b.DestType
      Bmk.DoOpen = b.DoOpen <> 0
      Bmk.Parent = b.Parent
      Bmk.Style = b.Style
      Bmk.Title = ToString(b.Title, b.TitleLen, b.bUnicode <> 0)
      Return True
   End Function

   Public Function GetBookmarkCount() As Integer
      Return pdfGetBookmarkCount(m_Instance)
   End Function

   Public Function GetBorderStyle() As Integer
      Return pdfGetBorderStyle(m_Instance)
   End Function

   Public Function GetBuffer() As Byte()
      Dim pBuffer As IntPtr
      Dim bufSize As Integer
      Dim retval() As Byte

      pBuffer = pdfGetBuffer(m_Instance, bufSize)
      If bufSize = 0 Or IntPtr.Zero.Equals(pBuffer) Then Return Nothing
      ReDim retval(bufSize - 1)
      Marshal.Copy(pBuffer, retval, 0, bufSize)
      pdfFreePDF(m_Instance)
      Return retval
   End Function

   Public Function GetCapHeight() As Double
      Return pdfGetCapHeight(m_Instance)
   End Function

   Public Function GetCharacterSpacing() As Double
      Return pdfGetCharacterSpacing(m_Instance)
   End Function

   Public Function GetCheckBoxChar() As Integer
      Return pdfGetCheckBoxChar(m_Instance)
   End Function

   Public Function GetCheckBoxCharEx(ByVal Field As Integer) As Integer
      Return pdfGetCheckBoxCharEx(m_Instance, Field)
   End Function

   Public Function GetCheckBoxDefState(ByVal Field As Integer) As Integer
      Return pdfGetCheckBoxDefState(m_Instance, Field)
   End Function

   Public Function GetCMap(ByVal Index As Integer, ByRef CMap As TPDFCMap) As Boolean
      CMap.StructSize = Marshal.SizeOf(CMap)
      Return CBool(pdfGetCMap(m_Instance, Index, CMap))
   End Function

   Public Function GetCMapCount() As Integer
      Return pdfGetCMapCount(m_Instance)
   End Function

   Public Function GetColorSpace() As Integer
      Return pdfGetColorSpace(m_Instance)
   End Function

   Public Function GetColorSpaceCount() As Integer
      GetColorSpaceCount = pdfGetColorSpaceCount(m_Instance)
   End Function

   Public Function GetColorSpaceObj(ByVal Handle As Integer, ByRef cs As TPDFColorSpaceObj) As Boolean
      Dim ics As New TPDFColorSpaceObj_I()
      If pdfGetColorSpaceObj(m_Instance, Handle, ics) = 0 Then Return False
      Return GetIntColorSpaceObj(ics, cs)
   End Function

   Public Function GetColorSpaceObjEx(ByVal IColorSpace As IntPtr, ByRef cs As TPDFColorSpaceObj) As Boolean
      Dim ics As New TPDFColorSpaceObj_I()
      If pdfGetColorSpaceObjEx(IColorSpace, ics) = 0 Then Return False
      Return GetIntColorSpaceObj(ics, cs)
   End Function

   Public Function GetCompressionFilter() As Integer
      Return pdfGetCompressionFilter(m_Instance)
   End Function

   Public Function GetCompressionLevel() As Integer
      Return pdfGetCompressionLevel(m_Instance)
   End Function

   Public Function GetContent(ByRef Buffer() As Byte) As Boolean
      Dim bufSize As Integer
      Dim pBuffer As IntPtr

      bufSize = pdfGetContent(m_Instance, pBuffer)
      If bufSize <= 0 Then
         Buffer = Nothing
         Return False
      End If
      ReDim Buffer(bufSize - 1)
      Marshal.Copy(pBuffer, Buffer, 0, bufSize)
      Return True
   End Function

   Public Function GetDefBitsPerPixel() As Integer
      Return pdfGetDefBitsPerPixel(m_Instance)
   End Function

   Public Function GetDescent() As Double
      Return pdfGetDescent(m_Instance)
   End Function

   Public Function GetDeviceNAttributes(ByVal IAttributes As IntPtr, ByRef Attributes As TDeviceNAttributes) As Boolean
      Dim i As Integer
      Dim attr As New TDeviceNAttributes_I()
      If pdfGetDeviceNAttributes(IAttributes, attr) = 0 Then Return False
      Attributes.IProcessColorSpace = attr.IProcessColorSpace
      If attr.ProcessColorantsCount > 0 Then
         ReDim Attributes.ProcessColorants(attr.ProcessColorantsCount - 1)
         For i = 0 To attr.ProcessColorantsCount - 1
            Attributes.ProcessColorants(i) = ToString(attr.ProcessColorants(i), False)
         Next i
      Else
         Attributes.ProcessColorants = Nothing
      End If
      If attr.SeparationsCount > 0 Then
         Dim bytes() As Byte
         ReDim Attributes.Separations(attr.SeparationsCount - 1)
         For i = 0 To attr.SeparationsCount - 1
            bytes = System.Text.Encoding.Unicode.GetBytes(Marshal.PtrToStringAnsi(attr.ProcessColorants(i)))
            Attributes.ProcessColorants(i) = System.Text.Encoding.UTF8.GetString(bytes)
         Next i
      Else
         Attributes.Separations = Nothing
      End If
      Attributes.IMixingHints = attr.IMixingHints
      Return True
   End Function

   Public Function GetDocInfo(ByVal DInfo As TDocumentInfo, ByRef Value As String) As Boolean
      Dim pBuffer As IntPtr
      If pdfGetDocInfo(m_Instance, DInfo, pBuffer) <= 0 Then
         Value = Nothing
         Return False
      End If
      Value = ToString(pBuffer, True)
      Return True
   End Function

   Public Function GetDocInfoCount() As Integer
      Return pdfGetDocInfoCount(m_Instance)
   End Function

   Public Function GetDocInfoEx(ByVal Index As Integer, ByRef DInfo As TDocumentInfo, ByRef Key As String, ByRef Value As String) As Boolean
      Dim valName As IntPtr
      Dim keyName As IntPtr
      Dim valLen As Integer
      Dim uni As Integer
      Dim info As Integer
      valLen = pdfGetDocInfoEx(m_Instance, Index, info, keyName, valName, uni)
      If valLen <= 0 Then
         Key = Nothing
         Value = Nothing
         Return False
      End If
      DInfo = CType(info, TDocumentInfo)
      Key = ToString(keyName, False)
      Value = ToString(valName, valLen, uni <> 0)
      Return True
   End Function

   Public Function GetDrawDirection() As Integer
      Return pdfGetDrawDirection(m_Instance)
   End Function

   Public Function GetDynaPDFVersion() As String
      Return ToString(pdfGetDynaPDFVersion(), False)
   End Function

   Public Function GetEmbeddedFile(ByVal Handle As Integer, ByRef FileSpec As TPDFFileSpec, ByVal Decompress As Boolean) As Boolean
      Dim fSpec As TPDFFileSpec_I
      If pdfGetEmbeddedFile(m_Instance, Handle, fSpec, CInt(Decompress)) = 0 Then
         FileSpec = Nothing
         Return False
      End If
      If Not IntPtr.Zero.Equals(fSpec.Buffer) Then
         ReDim FileSpec.Buffer(fSpec.BufSize - 1)
         Marshal.Copy(fSpec.Buffer, FileSpec.Buffer, 0, fSpec.BufSize)
      Else
         FileSpec.Buffer = Nothing
      End If
      FileSpec.ColItem = fSpec.ColItem
      FileSpec.Compressed = fSpec.Compressed <> 0
      FileSpec.FileSize = fSpec.FileSize
      FileSpec.IsURL = fSpec.IsURL <> 0
      If Not IntPtr.Zero.Equals(fSpec.CheckSum) Then
         ReDim FileSpec.CheckSum(15)
         Marshal.Copy(fSpec.CheckSum, FileSpec.CheckSum, 0, 16)
      End If
      FileSpec.CreateDate = ToString(fSpec.CreateDate, False)
      FileSpec.Desc = ToString(fSpec.Desc, fSpec.DescUnicode <> 0)
      FileSpec.FileName = ToString(fSpec.FileName, False)
      FileSpec.MIMEType = ToString(fSpec.MIMEType, False)
      FileSpec.ModDate = ToString(fSpec.ModDate, False)
      FileSpec.Name = ToString(fSpec.Name, fSpec.NameUnicode <> 0)
      FileSpec.UF = ToString(fSpec.UF, fSpec.UFUnicode <> 0)
      Return True
   End Function

   Public Function GetEmbeddedFileCount() As Integer
      Return pdfGetEmbeddedFileCount(m_Instance)
   End Function

   Public Function GetErrorMessage() As String
      Return ToString(pdfGetErrorMessage(m_Instance), False)
   End Function

   Public Function GetEMFPatternDistance() As Double
      Return pdfGetEMFPatternDistance(m_Instance)
   End Function

   Public Function GetErrorMode() As Integer
      Return pdfGetErrorMode(m_Instance)
   End Function

   Public Function GetField(ByVal AHandle As Integer, ByRef Field As TPDFField) As Boolean
      Dim f As TPDFField_I
      If pdfGetField(m_Instance, AHandle, f) < 0 Then
         Field = Nothing
         Return False
      End If
      GetIntField(f, Field)
      Return True
   End Function

   Public Function GetFieldBackColor() As Integer
      Return pdfGetFieldBackColor(m_Instance)
   End Function

   Public Function GetFieldBorderColor() As Integer
      Return pdfGetFieldBorderColor(m_Instance)
   End Function

   Public Function GetFieldBorderStyle(ByVal Field As Integer) As Integer
      Return pdfGetFieldBorderStyle(m_Instance, Field)
   End Function

   Public Function GetFieldBorderWidth(ByVal Field As Integer) As Double
      Return pdfGetFieldBorderWidth(m_Instance, Field)
   End Function

   Public Function GetFieldChoiceValue(ByVal Field As Integer, ByVal ValIndex As Integer, ByRef Value As TPDFChoiceValue) As Boolean
      Dim v As TPDFChoiceValue_I
      v.StructSize = Marshal.SizeOf(v)
      If pdfGetFieldChoiceValue(m_Instance, Field, ValIndex, v) = 0 Then Return False
      Value.ExpValue = ToString(v.ExpValueA, v.ExpValueW, v.ExpValueLen)
      Value.Value = ToString(v.ValueA, v.ValueW, v.ValueLen)
      Value.Selected = CBool(v.Selected)
   End Function

   Public Function GetFieldColor(ByVal Field As Integer, ByVal ColorType As TFieldColor, ByRef ColorSpace As TPDFColorSpace, ByRef Color As Integer) As Boolean
      Dim cs As Integer
      If pdfGetFieldColor(m_Instance, Field, ColorType, cs, Color) = 0 Then
         Color = 0
         ColorSpace = TPDFColorSpace.csDeviceRGB
         Return False
      End If
      ColorSpace = CType(cs, TPDFColorSpace)
      Return True
   End Function

   Public Function GetFieldCount() As Integer
      Return pdfGetFieldCount(m_Instance)
   End Function

   Public Function GetFieldEx(ByVal Handle As Integer, ByRef Field As TPDFFieldEx) As Boolean
      Dim f As TPDFFieldEx_I
      f.StructSize = Marshal.SizeOf(f)
      If pdfGetFieldEx(m_Instance, Handle, f) = 0 Then Return False
      GetIntFieldEx(f, Field)
      Return True
   End Function

   Public Function GetFieldEx2(ByVal IField As IntPtr, ByRef Field As TPDFFieldEx) As Boolean
      Dim f As TPDFFieldEx_I
      f.StructSize = Marshal.SizeOf(f)
      If pdfGetFieldEx2(IField, f) = 0 Then Return False
      GetIntFieldEx(f, Field)
   End Function

   Public Function GetFieldExpValCount(ByVal Field As Integer) As Integer
      Return pdfGetFieldExpValCount(m_Instance, Field)
   End Function

   Public Function GetFieldExpValue(ByVal Field As Integer, ByRef Value As String) As Boolean
      Dim pBuffer As IntPtr
      Dim Len As Integer
      Len = pdfGetFieldExpValue(m_Instance, Field, pBuffer)
      Value = ToString(pBuffer, False)
      Return (Len > 0)
   End Function

   Public Function GetFieldExpValueEx(ByVal Field As Integer, ByVal ValIndex As Integer, ByRef Value As String, ByRef ExpValue As String, ByRef bSelected As Integer) As Boolean
      Dim valPtr As IntPtr
      Dim expPtr As IntPtr
      If pdfGetFieldExpValueEx(m_Instance, Field, ValIndex, valPtr, expPtr, bSelected) = 0 Then
         Value = Nothing
         ExpValue = Nothing
         Return False
      End If
      Value = ToString(valPtr, False)
      ExpValue = ToString(expPtr, False)
      Return True
   End Function

   Public Function GetFieldFlags(ByVal Field As Integer) As Integer
      Return pdfGetFieldFlags(m_Instance, Field)
   End Function

   Public Function GetFieldGroupType(ByVal Field As Integer) As Integer
      Return pdfGetFieldGroupType(m_Instance, Field)
   End Function

   Public Function GetFieldHighlightMode(ByVal Field As Integer) As Integer
      Return pdfGetFieldHighlightMode(m_Instance, Field)
   End Function

   Public Function GetFieldIndex(ByVal Field As Integer) As Integer
      Return pdfGetFieldIndex(m_Instance, Field)
   End Function

   Public Function GetFieldMapName(ByVal Field As Integer, ByRef Value As String) As Boolean
      Dim uni As Integer
      Dim pBuffer As IntPtr
      If pdfGetFieldMapName(m_Instance, Field, pBuffer, uni) < 0 Then
         Value = Nothing
         Return False
      End If
      Value = ToString(pBuffer, uni <> 0)
      Return True
   End Function

   Public Function GetFieldName(ByVal Field As Integer, ByRef Name As String) As Boolean
      Dim pBuffer As IntPtr
      Dim asLen As Integer
      asLen = pdfGetFieldName(m_Instance, Field, pBuffer)
      If asLen <= 0 Then
         Name = Nothing
         Return False
      End If
      Dim nameLen As Integer
      nameLen = pdfStrLenA(pBuffer)
      Name = ToString(pBuffer, asLen, asLen <> nameLen)
      Return True
   End Function

   Public Function GetFieldOrientation(ByVal Field As Integer) As Integer
      Return pdfGetFieldOrientation(m_Instance, Field)
   End Function

   Public Function GetFieldTextAlign(ByVal Field As Integer) As TTextAlign
      Dim retval As Integer
      retval = pdfGetFieldTextAlign(m_Instance, Field)
      If retval < 0 Then
         Return TTextAlign.taLeft
      Else
         Return CType(retval, TTextAlign)
      End If
   End Function

   Public Function GetFieldTextColor() As Integer
      Return pdfGetFieldTextColor(m_Instance)
   End Function

   Public Function GetFieldToolTip(ByVal Field As Integer, ByRef Value As String) As Boolean
      Dim uni As Integer
      Dim Len As Integer
      Dim pBuffer As IntPtr
      Len = pdfGetFieldToolTip(m_Instance, Field, pBuffer, uni)
      If Len < 0 Then
         Value = Nothing
         Return False
      End If
      Value = ToString(pBuffer, Len, uni <> 0)
      Return True
   End Function

   Public Function GetFieldType(ByVal Field As Integer, ByRef Value As TFieldType) As Boolean
      Dim retval As Integer
      retval = pdfGetFieldType(m_Instance, Field)
      If retval < 0 Then
         Return False
      Else
         Value = CType(retval, TFieldType)
         Return True
      End If
   End Function

   Public Function GetFillColor() As Integer
      Return pdfGetFillColor(m_Instance)
   End Function

   Public Function GetFont(ByVal IFont As IntPtr, ByRef F As TPDFFontObj) As Boolean
      Dim font As TPDFFontObj_I = New TPDFFontObj_I()
      If fntGetFont(IFont, font) = 0 Then
         F = Nothing
         Return False
      End If
      GetIntFont(font, F)
      Return True
   End Function

   Public Function GetFontCount() As Integer
      Return pdfGetFontCount(m_Instance)
   End Function

   Public Function GetFontEx(ByVal Handle As Integer, ByRef F As TPDFFontObj) As Boolean
      Dim font As TPDFFontObj_I = New TPDFFontObj_I()
      If pdfGetFontEx(m_Instance, Handle, font) = 0 Then
         F = Nothing
         Return False
      End If
      GetIntFont(font, F)
      Return True
   End Function

   Public Function GetFontOrigin() As TOrigin
      Return CType(pdfGetFontOrigin(m_Instance), TOrigin)
   End Function

   Public Sub GetFontSearchOrder(ByRef Order() As TFontBaseType)
      pdfGetFontSearchOrder(m_Instance, Order)
   End Sub

   Public Function GetFontSelMode() As TFontSelMode
      Return CType(pdfGetFontSelMode(m_Instance), TFontSelMode)
   End Function

   Public Function GetFontWeight() As Integer
      Return pdfGetFontWeight(m_Instance)
   End Function

   Public Function GetFTextHeightA(ByVal Align As TTextAlign, ByVal AText As String) As Double
      Return pdfGetFTextHeightA(m_Instance, Align, AText)
   End Function

   Public Function GetFTextHeightW(ByVal Align As TTextAlign, ByVal AText As String) As Double
      Return pdfGetFTextHeightW(m_Instance, Align, AText)
   End Function

   Public Function GetFTextHeightExA(ByVal Width As Double, ByVal Align As TTextAlign, ByVal AText As String) As Double
      Return pdfGetFTextHeightExA(m_Instance, Width, Align, AText)
   End Function

   Public Function GetFTextHeightExW(ByVal Width As Double, ByVal Align As TTextAlign, ByVal AText As String) As Double
      Return pdfGetFTextHeightExW(m_Instance, Width, Align, AText)
   End Function

   Public Function GetGStateFlags() As Integer
      Return pdfGetGStateFlags(m_Instance)
   End Function

   Public Function GetIconColor() As Integer
      Return pdfGetIconColor(m_Instance)
   End Function

   Public Function GetImageBuffer(ByRef BufSize As Integer) As IntPtr
      Return pdfGetImageBuffer(m_Instance, BufSize)
   End Function

   Public Function GetImageCount(ByVal FileName As String) As Integer
      Return pdfGetImageCount(m_Instance, FileName)
   End Function

   Public Function GetImageHeight(ByVal AHandle As Integer) As Integer
      Return pdfGetImageHeight(m_Instance, AHandle)
   End Function

   Public Function GetImageWidth(ByVal AHandle As Integer) As Integer
      Return pdfGetImageWidth(m_Instance, AHandle)
   End Function

   Public Function GetImportFlags() As Integer
      Return pdfGetImportFlags(m_Instance)
   End Function

   Public Function GetInBBox(ByVal PageNum As Integer, ByVal Boundary As TPageBoundary, ByRef BBox As TPDFRect) As Boolean
      Return CBool(pdfGetInBBox(m_Instance, PageNum, Boundary, BBox))
   End Function

   Public Function GetInDocInfo(ByVal DInfo As TDocumentInfo, ByRef Value As String) As Boolean
      Dim pBuffer As IntPtr
      Dim Len As Integer
      Len = pdfGetInDocInfo(m_Instance, DInfo, pBuffer)
      Value = ToString(pBuffer, Len, True)
      Return (Len >= 0)
   End Function

   Public Function GetInDocInfoCount() As Integer
      Return pdfGetInDocInfoCount(m_Instance)
   End Function

   Public Function GetInDocInfoEx(ByVal Index As Integer, ByRef DInfo As TDocumentInfo, ByRef Key As String, ByRef Value As String) As Boolean
      Dim info As Integer
      Dim uni As Integer
      Dim keyBuf As IntPtr
      Dim valBuf As IntPtr
      Dim Len As Integer
      Len = pdfGetInDocInfoEx(m_Instance, Index, info, keyBuf, valBuf, uni)
      If Len <= 0 Then
         Key = Nothing
         Value = Nothing
         Return (Len = 0)
      End If
      DInfo = CType(info, TDocumentInfo)
      Key = ToString(keyBuf, False)
      Value = ToString(valBuf, Len, uni <> 0)
      Return True
   End Function

   Public Function GetInEncryptionFlags() As Integer
      Return pdfGetInEncryptionFlags(m_Instance)
   End Function

   Public Function GetInIsCollection() As Boolean
      Dim retval As Integer
      retval = pdfGetInIsCollection(m_Instance)
      If retval < 0 Then Return False
      Return retval <> 0
   End Function

   Public Function GetInIsEncrypted() As Boolean
      Dim retval As Integer
      retval = pdfGetInIsEncrypted(m_Instance)
      If retval < 0 Then Return False
      Return retval <> 0
   End Function

   Public Function GetInIsSigned() As Boolean
      Dim retval As Integer
      retval = pdfGetInIsSigned(m_Instance)
      If retval < 0 Then Return False
      Return retval <> 0
   End Function

   Public Function GetInIsTrapped() As Boolean
      Dim retval As Integer
      retval = pdfGetInIsTrapped(m_Instance)
      If retval < 0 Then Return False
      Return retval <> 0
   End Function

   Public Function GetInIsXFAForm() As Boolean
      Dim retval As Integer
      retval = pdfGetInIsXFAForm(m_Instance)
      If retval < 0 Then Return False
      Return retval <> 0
   End Function

   Public Function GetInNamedDest(ByVal Index As Integer, ByRef Dest As TPDFNamedDest) As Boolean
   Dim d As TPDFNamedDest_I
   If pdfGetInNamedDest(m_Instance, Index, d) = 0 Then Return False
   Dest.Name = ToString(d.NameA, d.NameW, d.NameLen)
   Dest.DestPage = d.DestPage
   Dest.DestPos = d.DestPos
   Dest.DestType = d.DestType
   End Function

   Public Function GetInNamedDestCount() As Integer
      Return pdfGetInNamedDestCount(m_Instance)
   End Function

   Public Function GetInOrientation(ByVal PageNum As Integer) As Integer
      Return pdfGetInOrientation(m_Instance, PageNum)
   End Function

   Public Function GetInPageCount() As Integer
      Return pdfGetInPageCount(m_Instance)
   End Function

   Public Function GetInPDFVersion() As Integer
      Return pdfGetInPDFVersion(m_Instance)
   End Function

   Public Function GetInPrintSettings(ByRef Settings As TPDFPrintSettings) As Boolean
      Dim s As TPDFPrintSettings_I
      If pdfGetInPrintSettings(m_Instance, s) = 0 Then Return False
      Settings.DuplexMode = s.DuplexMode
      Settings.NumCopies = s.NumCopies
      Settings.PickTrayByPDFSize = s.PickTrayByPDFSize
      Settings.PrintScaling = s.PrintScaling
      If s.PrintRangesCount > 0 Then
         ReDim Settings.PrintRanges(s.PrintRangesCount * 2 - 1)
         Marshal.Copy(s.PrintRanges, Settings.PrintRanges, 0, s.PrintRangesCount * 2)
      End If
      Return True
   End Function

   Public Function GetIsFixedPitch() As Boolean
      Return CBool(pdfGetIsFixedPitch(m_Instance))
   End Function

   Public Function GetItalicAngle() As Double
      Return pdfGetItalicAngle(m_Instance)
   End Function

   Public Function GetJavaScript(ByVal AHandle As Integer, ByRef Script As String) As Boolean
      Dim uni As Integer
      Dim pBuffer As IntPtr
      Dim bufSize As Integer

      pBuffer = pdfGetJavaScript(m_Instance, AHandle, bufSize, uni)
      Script = ToString(pBuffer, bufSize, uni <> 0)
      Return True
   End Function

   Public Function GetJavaScriptAction(ByVal AHandle As Integer, ByRef Script As String) As Boolean
      Dim uni As Integer
      Dim pBuffer As IntPtr
      Dim bufSize As Integer

      pBuffer = pdfGetJavaScriptAction(m_Instance, AHandle, bufSize, uni)
      Script = ToString(pBuffer, bufSize, uni <> 0)
      Return True
   End Function

   Public Function GetJavaScriptAction2(ByVal ObjType As TObjType, ByVal ObjHandle As Integer, ByVal ActIndex As Integer, ByRef Script As String, ByRef ObjEvent As TObjEvent) As Boolean
      Dim uni As Integer
      Dim pBuffer As IntPtr
      Dim bufSize As Integer

      pBuffer = pdfGetJavaScriptAction2(m_Instance, ObjType, ObjHandle, ActIndex, bufSize, uni, ObjEvent)
      Script = ToString(pBuffer, bufSize, uni <> 0)
      Return True
   End Function

   Public Function GetJavaScriptCount() As Integer
      Return pdfGetJavaScriptCount(m_Instance)
   End Function

   Public Function GetJavaScriptEx(ByVal Name As String, ByRef Script As String) As Boolean
      Dim uni As Integer
      Dim pBuffer As IntPtr
      Dim bufSize As Integer

      pBuffer = pdfGetJavaScriptEx(m_Instance, Name, bufSize, uni)
      Script = ToString(pBuffer, bufSize, uni <> 0)
      Return True
   End Function

   Public Function GetJavaScriptName(ByVal Handle As Integer) As String
      Dim uni As Integer
      Dim bufSize As Integer
      Dim pBuffer As IntPtr
      pBuffer = pdfGetJavaScriptName(m_Instance, Handle, bufSize, uni)
      Return ToString(pBuffer, bufSize, uni <> 0)
   End Function

   Public Function GetJPEGQuality() As Integer
      GetJPEGQuality = pdfGetJPEGQuality(m_Instance)
   End Function

   Public Function GetLanguage() As String
      Return ToString(pdfGetLanguage(m_Instance), False)
   End Function

   Public Function GetLastTextPosX() As Double
      Return pdfGetLastTextPosX(m_Instance)
   End Function

   Public Function GetLastTextPosY() As Double
      Return pdfGetLastTextPosY(m_Instance)
   End Function

   Public Function GetLeading() As Double
      Return pdfGetLeading(m_Instance)
   End Function

   Public Function GetLineCapStyle() As TLineCapStyle
      Return CType(pdfGetLineCapStyle(m_Instance), TLineCapStyle)
   End Function

   Public Function GetLineJoinStyle() As TLineJoinStyle
      Return CType(pdfGetLineJoinStyle(m_Instance), TLineJoinStyle)
   End Function

   Public Function GetLineWidth() As Double
      Return pdfGetLineWidth(m_Instance)
   End Function

   Public Function GetLinkHighlightMode() As THighlightMode
      Return CType(pdfGetLinkHighlightMode(m_Instance), THighlightMode)
   End Function

   Public Function GetLogMetafileSize(ByVal FileName As String, ByRef ARect As TRectL) As Boolean
      Return CBool(pdfGetLogMetafileSize(m_Instance, FileName, ARect))
   End Function

   Public Function GetLogMetafileSizeEx(ByRef Buffer() As Byte, ByRef ARect As TRectL) As Boolean
      Return CBool(pdfGetLogMetafileSizeEx(m_Instance, Buffer, Buffer.Length, ARect))
   End Function

   Public Function GetMatrix(ByRef Matrix As TCTM) As Boolean
      Return CBool(pdfGetMatrix(m_Instance, Matrix))
   End Function

   Public Function GetMaxFieldLen(ByVal TxtField As Integer) As Integer
      Return pdfGetMaxFieldLen(m_Instance, TxtField)
   End Function

   Public Function GetMetaConvFlags() As Integer
      Return pdfGetMetaConvFlags(m_Instance)
   End Function

   Public Function GetMissingGlyphs() As System.UInt32()
      Dim count As Integer
      Dim ptr As IntPtr = pdfGetMissingGlyphs(m_Instance, count)
      If (IntPtr.Zero.Equals(ptr)) Then
         Return Nothing
      Else
         Dim retval() As System.UInt32
         ReDim retval(count - 1)
         ' Why does the marshaller not provide a copy function for uint[]?
         pdfCopyMemUInt(ptr, retval, count * 4)
         Return retval
      End If
   End Function

   Public Function GetMiterLimit() As Double
      Return pdfGetMiterLimit(m_Instance)
   End Function

   Public Function GetNeedAppearance() As Boolean
      Return CBool(pdfGetNeedAppearance(m_Instance))
   End Function

   Public Function GetObjActionCount(ByVal ObjType As TObjType, ByVal ObjHandle As Integer) As Integer
      Return pdfGetObjActionCount(m_Instance, ObjType, ObjHandle)
   End Function

   Public Function GetOpacity() As Double
      Return pdfGetOpacity(m_Instance)
   End Function

   Public Function GetOrientation() As Integer
      Return pdfGetOrientation(m_Instance)
   End Function

   Public Function GetPageAnnot(ByVal Index As Integer, ByRef Annot As TPDFAnnotation) As Boolean
      Dim retval As TPDFAnnotation_I
      If pdfGetPageAnnot(m_Instance, Index, retval) = 0 Then
         Annot = Nothing
         Return False
      End If
      GetIntAnnot(retval, Annot)
      Return True
   End Function

   Public Function GetPageAnnotCount() As Integer
      Return pdfGetPageAnnotCount(m_Instance)
   End Function

   Public Function GetPageAnnotEx(ByVal Index As Integer, ByRef Annot As TPDFAnnotationEx) As Boolean
      Dim retval As TPDFAnnotationEx_I
      If pdfGetPageAnnotEx(m_Instance, Index, retval) = 0 Then
         Annot = Nothing
         Return False
      End If
      GetIntAnnotEx(retval, Annot)
      Return True
   End Function

   Public Function GetPageCoords() As TPageCoord
      Return CType(pdfGetPageCoords(m_Instance), TPageCoord)
   End Function

   Public Function GetPageCount() As Integer
      Return pdfGetPageCount(m_Instance)
   End Function

   Public Function GetPageField(ByVal Index As Integer, ByRef Field As TPDFField) As Boolean
      Dim f As TPDFField_I
      If pdfGetPageField(m_Instance, Index, f) < 0 Then
         Field = Nothing
         Return False
      End If
      GetIntField(f, Field)
      Return True
   End Function

   Public Function GetPageFieldCount() As Integer
      Return pdfGetPageFieldCount(m_Instance)
   End Function

   Public Function GetPageFieldEx(ByVal Index As Integer, ByRef Field As TPDFFieldEx) As Boolean
      Dim f As TPDFFieldEx_I
      f.StructSize = Marshal.SizeOf(f)
      If pdfGetPageFieldEx(m_Instance, Index, f) = 0 Then Return False
      GetIntFieldEx(f, Field)
      Return True
   End Function

   Public Function GetPageHeight() As Double
      Return pdfGetPageHeight(m_Instance)
   End Function

   Public Function GetPageLabel(ByVal Index As Integer, ByRef Label As TPDFPageLabel) As Boolean
      Dim lbl As TPDFPageLabel_I
      If pdfGetPageLabel(m_Instance, Index, lbl) = 0 Then Return False
      Label.StartRange = lbl.StartRange
      Label.Format = lbl.Format
      Label.FirstPageNum = lbl.FirstPageNum
      Label.Prefix = ToString(lbl.Prefix, lbl.PrefixLen, lbl.PrefixUni <> 0)
      Return True
   End Function

   Public Function GetPageLabelCount() As Integer
      Return pdfGetPageLabelCount(m_Instance)
   End Function

   Public Function GetPageLayout() As TPageLayout
      Return CType(pdfGetPageLayout(m_Instance), TPageLayout)
   End Function

   Public Function GetPageMode() As TPageMode
      Return CType(pdfGetPageMode(m_Instance), TPageMode)
   End Function

   Public Function GetPageNum() As Integer
      Return pdfGetPageNum(m_Instance)
   End Function

   Public Function GetPageText(ByRef Stack As TPDFStack) As Boolean
      If pdfGetPageText(m_Instance, Stack) = 0 Then Return False
      Return True
   End Function

   Public Function GetPageWidth() As Double
      Return pdfGetPageWidth(m_Instance)
   End Function

   Public Function GetPDFVersion() As Integer
      Return pdfGetPDFVersion(m_Instance)
   End Function

   Public Function GetPrintSettings(ByRef Settings As TPDFPrintSettings) As Boolean
      Dim s As TPDFPrintSettings_I
      If pdfGetPrintSettings(m_Instance, s) = 0 Then Return False
      Settings.DuplexMode = s.DuplexMode
      Settings.NumCopies = s.NumCopies
      Settings.PickTrayByPDFSize = s.PickTrayByPDFSize
      Settings.PrintScaling = s.PrintScaling
      If s.PrintRangesCount > 0 Then
         ReDim Settings.PrintRanges(s.PrintRangesCount * 2 - 1)
         Marshal.Copy(s.PrintRanges, Settings.PrintRanges, 0, s.PrintRangesCount * 2)
      End If
      Return True
   End Function

   Public Function GetResolution() As Integer
      Return pdfGetResolution(m_Instance)
   End Function

   Public Function GetSaveNewImageFormat() As Boolean
      Return CBool(pdfGetSaveNewImageFormat(m_Instance))
   End Function

   Public Function GetSeparationInfo(ByRef Colorant As String, ByRef CS As TExtColorSpace) As Boolean
      Dim clt As IntPtr
      If pdfGetSeparationInfo(m_Instance, clt, CS) = 0 Then Return False
      Colorant = ToString(clt, False)
      Return True
   End Function

   Public Function GetSpaceWidth(ByVal IFont As IntPtr, ByVal FontSize As Double) As Double
      Return fntGetSpaceWidth(IFont, FontSize)
   End Function

   Public Function GetStrokeColor() As Integer
      Return pdfGetStrokeColor(m_Instance)
   End Function

   Public Function GetTabLen() As Integer
      Return pdfGetTabLen(m_Instance)
   End Function

   Public Function GetTemplCount() As Integer
      Return pdfGetTemplCount(m_Instance)
   End Function

   Public Function GetTemplHandle() As Integer
      Return pdfGetTemplHandle(m_Instance)
   End Function

   Public Function GetTemplHeight(ByVal Handle As Integer) As Double
      Return pdfGetTemplHeight(m_Instance, Handle)
   End Function

   Public Function GetTemplWidth(ByVal Handle As Integer) As Double
      Return pdfGetTemplWidth(m_Instance, Handle)
   End Function

   Public Function GetTextDrawMode() As TDrawMode
      Return CType(pdfGetTextDrawMode(m_Instance), TDrawMode)
   End Function

   Public Function GetTextFieldValue(ByVal AField As Integer, ByRef Value As String, ByRef DefValue As String) As Boolean
      Dim valUni As Integer
      Dim defValUni As Integer
      Dim val As IntPtr
      Dim defVal As IntPtr
      If pdfGetTextFieldValue(m_Instance, AField, val, valUni, defVal, defValUni) <> 0 Then
         Value = ""
         DefValue = ""
         Return False
      End If
      Value = ToString(val, valUni <> 0)
      DefValue = ToString(defVal, defValUni <> 0)
      Return True
   End Function

   Public Function GetTextRect(ByRef PosX As Double, ByRef PosY As Double, ByRef Width As Double, ByRef Height As Double) As Boolean
      Return CBool(pdfGetTextRect(m_Instance, PosX, PosY, Width, Height))
   End Function

   Public Function GetTextRise() As Double
      Return pdfGetTextRise(m_Instance)
   End Function

   Public Function GetTextScaling() As Double
      Return pdfGetTextScaling(m_Instance)
   End Function

   Public Function GetTextWidth(ByVal IFont As IntPtr, ByVal Text As IntPtr, ByVal Len As Integer, ByVal CharSpacing As Single, ByVal WordSpacing As Single, ByVal TextScale As Single) As Double
      Return fntGetTextWidth(IFont, Text, Len, CharSpacing, WordSpacing, TextScale)
   End Function

   Public Function GetTextWidthA(ByVal AText As String) As Double
      Return pdfGetTextWidthA(m_Instance, AText)
   End Function

   Public Function GetTextWidthW(ByVal AText As String) As Double
      Return pdfGetTextWidthW(m_Instance, AText)
   End Function

   Public Function GetTextWidthExA(ByVal AText As String, ByVal Len As Integer) As Double
      Return pdfGetTextWidthExA(m_Instance, AText, Len)
   End Function

   Public Function GetTextWidthExW(ByVal AText As String, ByVal Len As Integer) As Double
      Return pdfGetTextWidthExW(m_Instance, AText, Len)
   End Function

   Public Function GetTransparentColor() As Integer
      Return pdfGetTransparentColor(m_Instance)
   End Function

   Public Function GetTrapped() As Boolean
      Return CBool(pdfGetTrapped(m_Instance))
   End Function

   Public Function GetUseExactPwd() As Boolean
      Return CBool(pdfGetUseExactPwd(m_Instance))
   End Function

   Public Function GetUseGlobalImpFiles() As Boolean
      Return CBool(pdfGetUseGlobalImpFiles(m_Instance))
   End Function

   Public Function GetUserRights() As Integer
      Return pdfGetUserRights(m_Instance)
   End Function

   Public Function GetUseStdFonts() As Boolean
      Return CBool(pdfGetUseStdFonts(m_Instance))
   End Function

   Public Function GetUseSystemFonts() As Boolean
      Return CBool(pdfGetUseSystemFonts(m_Instance))
   End Function

   Public Function GetUseTransparency() As Boolean
      Return CBool(pdfGetUseTransparency(m_Instance))
   End Function

   Public Function GetUseVisibleCoords() As Boolean
      Return CBool(pdfGetUseVisibleCoords(m_Instance))
   End Function

   Public Function GetViewerPrefrences(ByRef Preference As Integer, ByRef AddVal As Integer) As Boolean
      Return CBool(pdfGetViewerPrefrences(m_Instance, Preference, AddVal))
   End Function

   Public Function GetWMFDefExtent(ByRef Width As Integer, ByRef Height As Integer) As Boolean
      Return CBool(pdfGetWMFDefExtent(m_Instance, Width, Height))
   End Function

   Public Function GetWMFPixelPerInch() As Integer
      Return pdfGetWMFPixelPerInch(m_Instance)
   End Function

   Public Function GetWordSpacing() As Double
      Return pdfGetWordSpacing(m_Instance)
   End Function

   Public Function HaveOpenDoc() As Boolean
      Return CBool(pdfHaveOpenDoc(m_Instance))
   End Function

   Public Function HighlightAnnotA(ByVal SubType As TAnnotType, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Color As Integer, ByVal Author As String, ByVal Subject As String, ByVal Comment As String) As Integer
      Return pdfHighlightAnnotA(m_Instance, SubType, PosX, PosY, Width, Height, Color, Author, Subject, Comment)
   End Function

   Public Function HighlightAnnotW(ByVal SubType As TAnnotType, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Color As Integer, ByVal Author As String, ByVal Subject As String, ByVal Comment As String) As Integer
      Return pdfHighlightAnnotW(m_Instance, SubType, PosX, PosY, Width, Height, Color, Author, Subject, Comment)
   End Function

   Public Function ImportBookmarks() As Integer
      Return pdfImportBookmarks(m_Instance)
   End Function

   Public Function ImportCatalogObjects() As Boolean
      Return CBool(pdfImportCatalogObjects(m_Instance))
   End Function

   Public Function ImportDocInfo() As Boolean
      Return CBool(pdfImportDocInfo(m_Instance))
   End Function

   Public Function ImportPage(ByVal PageNum As Integer) As Integer
      Return pdfImportPage(m_Instance, PageNum)
   End Function

   Public Function ImportPageEx(ByVal PageNum As Integer, ByVal ScaleX As Double, ByVal ScaleY As Double) As Integer
      Return pdfImportPageEx(m_Instance, PageNum, ScaleX, ScaleY)
   End Function

   Public Function ImportPDFFile(ByVal DestPage As Integer, ByVal ScaleX As Double, ByVal ScaleY As Double) As Integer
      Return pdfImportPDFFile(m_Instance, DestPage, ScaleX, ScaleY)
   End Function

   Public Function InitExtGState(ByRef GS As TPDFExtGState) As Boolean
      Return CBool(pdfInitExtGState(GS))
   End Function

   Public Function InitStack(ByRef Stack As TPDFStack) As Boolean
      Return pdfInitStack(m_Instance, Stack) <> 0
   End Function

   Public Function InsertBMPFromBuffer(ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByRef Buffer() As Byte) As Integer
      Return pdfInsertBMPFromBuffer(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, Buffer)
   End Function

   Public Function InsertBMPFromHandle(ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByVal hBitmap As IntPtr) As Integer
      Return pdfInsertBMPFromHandle(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, hBitmap)
   End Function

   Public Function InsertImage(ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByVal AFile As String) As Integer
      Return pdfInsertImage(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, AFile)
   End Function

   Public Function InsertImageEx(ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByVal AFile As String, ByVal Index As Integer) As Integer
      Return pdfInsertImageEx(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, AFile, Index)
   End Function

   Public Function InsertImageFromBuffer(ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByRef Buffer() As Byte, ByVal Index As Integer) As Integer
      Return pdfInsertImageFromBuffer(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, Buffer, Buffer.Length, Index)
   End Function

   Public Function InsertMetafile(ByVal FileName As String, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfInsertMetafile(m_Instance, FileName, PosX, PosY, Width, Height))
   End Function

   Public Function InsertMetafileEx(ByRef Buffer() As Byte, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfInsertMetafileEx(m_Instance, Buffer, Buffer.Length, PosX, PosY, Width, Height))
   End Function

   Public Function InsertMetafileExt(ByVal FileName As String, ByRef View As TRectL, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfInsertMetafileExt(m_Instance, FileName, View, PosX, PosY, Width, Height))
   End Function

   Public Function InsertMetafileExtEx(ByRef Buffer() As Byte, ByRef View As TRectL, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfInsertMetafileExtEx(m_Instance, Buffer, Buffer.Length, View, PosX, PosY, Width, Height))
   End Function

   Public Function InsertMetafileFromHandle(ByVal hEnhMetafile As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfInsertMetafileFromHandle(m_Instance, hEnhMetafile, PosX, PosY, Width, Height))
   End Function

   Public Function InsertMetafileFromHandleEx(ByVal hEnhMetafile As IntPtr, ByRef View As TRectL, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfInsertMetafileFromHandleEx(m_Instance, hEnhMetafile, View, PosX, PosY, Width, Height))
   End Function

   Public Function InsertRawImage(ByRef Buffer() As Byte, ByVal BitsPerPixel As Integer, ByVal ColorCount As Integer, ByVal ImgWidth As Integer, ByVal ImgHeight As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double) As Integer
      Return pdfInsertRawImage(m_Instance, Buffer, BitsPerPixel, ColorCount, ImgWidth, ImgHeight, PosX, PosY, ScaleWidth, ScaleHeight)
   End Function

   Public Function IsBidiText(ByVal AText As String) As Integer
      Return pdfIsBidiText(m_Instance, AText)
   End Function

   Public Function IsColorPage(ByVal GrayIsColor As Boolean) As Integer
      Return pdfIsColorPage(m_Instance, CInt(GrayIsColor))
   End Function

   Public Function IsEmptyPage() As Integer
      Return pdfIsEmptyPage(m_Instance)
   End Function

   Public Function IsWrongPwd(ByVal ErrCode As Integer) As Boolean
      Return (((ErrCode = ENEED_PWD) Or (ErrCode = EWRONG_OPEN_PWD) Or (ErrCode = EWRONG_OWNER_PWD) Or (ErrCode = EWRONG_PWD)))
   End Function

   Public Function LineTo(ByVal PosX As Double, ByVal PosY As Double) As Boolean
      Return CBool(pdfLineTo(m_Instance, PosX, PosY))
   End Function

   Public Function LoadCMap(ByVal CMapName As String, ByVal Embed As Boolean) As Boolean
      Return CBool(pdfLoadCMap(m_Instance, CMapName, CInt(Embed)))
   End Function

   Public Function LoadFDFDataA(ByVal FileName As String, ByVal Password As String, ByVal Flags As Integer) As Boolean
      LoadFDFDataA = CBool(pdfLoadFDFDataA(m_Instance, FileName, Password, Flags))
   End Function

   Public Function LoadFDFDataW(ByVal FileName As String, ByVal Password As String, ByVal Flags As Integer) As Boolean
      LoadFDFDataW = CBool(pdfLoadFDFDataW(m_Instance, FileName, Password, Flags))
   End Function

   Public Function LoadFDFDataEx(ByRef Buffer() As Byte, ByVal Password As String, ByVal Flags As Integer) As Boolean
      LoadFDFDataEx = CBool(pdfLoadFDFDataEx(m_Instance, Buffer, Buffer.Length, Password, Flags))
   End Function

   Public Function LoadFont(ByRef Buffer() As Byte, ByVal Style As TFStyle, ByVal Size As Double, ByVal Embed As Boolean, ByVal CP As TCodepage) As Integer
      Return pdfLoadFont(m_Instance, Buffer, Buffer.Length, Style, Size, CInt(Embed), CP)
   End Function

   Public Function MovePage(ByVal Source As Integer, ByVal Dest As Integer) As Boolean
      Return CBool(pdfMovePage(m_Instance, Source, Dest))
   End Function

   Public Function MoveTo(ByVal PosX As Double, ByVal PosY As Double) As Boolean
      Return CBool(pdfMoveTo(m_Instance, PosX, PosY))
   End Function

   Public Sub MuliplyMatrix(ByRef M1 As TCTM, ByRef M2 As TCTM, ByRef NewMatrix As TCTM)
      NewMatrix.a = M2.a * M1.a + M2.b * M1.c
      NewMatrix.b = M2.a * M1.b + M2.b * M1.d
      NewMatrix.c = M2.c * M1.a + M2.d * M1.c
      NewMatrix.d = M2.c * M1.b + M2.d * M1.d
      NewMatrix.x = M2.x * M1.a + M2.y * M1.c + M1.x
      NewMatrix.y = M2.x * M1.b + M2.y * M1.d + M1.y
   End Sub

   Public Function OpenImportBuffer(ByRef Buffer() As Byte, ByVal PwdType As TPwdType, ByVal Password As String) As Integer
      Return pdfOpenImportBuffer(m_Instance, Buffer, Buffer.Length, PwdType, Password)
   End Function

   Public Function OpenImportFileA(ByVal FileName As String, ByVal PwdType As TPwdType, ByVal Password As String) As Integer
      Return pdfOpenImportFileA(m_Instance, FileName, PwdType, Password)
   End Function

   Public Function OpenImportFileW(ByVal FileName As String, ByVal PwdType As TPwdType, ByVal Password As String) As Integer
      Return pdfOpenImportFileW(m_Instance, FileName, PwdType, Password)
   End Function

   Public Function OpenOutputFileA(ByVal OutPDF As String) As Boolean
      Return CBool(pdfOpenOutputFileA(m_Instance, OutPDF))
   End Function

   Public Function OpenOutputFileW(ByVal OutPDF As String) As Boolean
      Return CBool(pdfOpenOutputFileW(m_Instance, OutPDF))
   End Function

   Public Function OpenTagA(ByVal Tag As TPDFBaseTag, ByVal Lang As String, ByVal AltText As String, ByVal Expansion As String) As Boolean
      Return CBool(pdfOpenTagA(m_Instance, Tag, Lang, AltText, Expansion))
   End Function
   Public Function OpenTagW(ByVal Tag As TPDFBaseTag, ByVal Lang As String, ByVal AltText As String, ByVal Expansion As String) As Boolean
      Return CBool(pdfOpenTagW(m_Instance, Tag, Lang, AltText, Expansion))
   End Function

   Public Function PageLink(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal DestPage As Integer) As Integer
      Return pdfPageLink(m_Instance, PosX, PosY, Width, Height, DestPage)
   End Function

   Public Function PageLink2(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal NamedDest As Integer) As Integer
      Return pdfPageLink2(m_Instance, PosX, PosY, Width, Height, NamedDest)
   End Function

   Public Function PageLinkEx(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal DestType As TDestType, ByVal DestPage As Integer, ByVal a As Double, ByVal b As Double, ByVal C As Double, ByVal d As Double) As Integer
      Return pdfPageLinkEx(m_Instance, PosX, PosY, Width, Height, DestType, DestPage, a, b, C, d)
   End Function

   Public Function ParseContent(ByRef Stack As TPDFParseInterface, ByVal Flags As TParseFlags) As Boolean
      Return CBool(pdfParseContent(m_Instance, IntPtr.Zero, Stack, Flags))
   End Function

   Public Function PlaceImage(ByVal ImgHandle As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double) As Boolean
      Return CBool(pdfPlaceImage(m_Instance, ImgHandle, PosX, PosY, ScaleWidth, ScaleHeight))
   End Function

   Public Function PlaceSigFieldValidateIcon(ByVal SigField As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfPlaceSigFieldValidateIcon(m_Instance, SigField, PosX, PosY, Width, Height))
   End Function

   Public Function PlaceTemplate(ByVal TmplHandle As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double) As Boolean
      Return CBool(pdfPlaceTemplate(m_Instance, TmplHandle, PosX, PosY, ScaleWidth, ScaleHeight))
   End Function

   Public Function PlaceTemplateEx(ByVal TmplHandle As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double) As Boolean
      Return CBool(pdfPlaceTemplateEx(m_Instance, TmplHandle, PosX, PosY, ScaleWidth, ScaleHeight))
   End Function

   Public Function ReadImageFormat(ByVal FileName As String, ByRef Width As Integer, ByRef Height As Integer, ByRef BitsPerPixel As Integer, ByRef UseZip As Integer) As Boolean
      Return CBool(pdfReadImageFormat(m_Instance, FileName, Width, Height, BitsPerPixel, UseZip))
   End Function

   Public Function ReadImageFormat2(ByVal FileName As String, ByVal Index As Integer, ByRef Width As Integer, ByRef Height As Integer, ByRef BitsPerPixel As Integer, ByRef UseZip As Integer) As Boolean
      Return CBool(pdfReadImageFormat2(m_Instance, FileName, Index, Width, Height, BitsPerPixel, UseZip))
   End Function

   Public Function ReadImageFormatEx(ByVal hBitmap As IntPtr, ByVal Width As Integer, ByVal Height As Integer, ByVal BitsPerPixel As Integer, ByVal UseZip As Integer) As Boolean
      Return CBool(pdfReadImageFormatEx(m_Instance, hBitmap, Width, Height, BitsPerPixel, UseZip))
   End Function

   Public Function ReadImageFormatFromBuffer(ByRef Buffer() As Byte, ByVal Index As Integer, ByVal Width As Integer, ByVal Height As Integer, ByVal BitsPerPixel As Integer, ByVal UseZip As Integer) As Boolean
      Return CBool(pdfReadImageFormatFromBuffer(m_Instance, Buffer, Buffer.Length, Index, Width, Height, BitsPerPixel, UseZip))
   End Function

   Public Function ReadImageResolution(ByVal FileName As String, ByVal Index As Integer, ByRef ResX As Integer, ByRef ResY As Integer) As Boolean
      Return CBool(pdfReadImageResolution(m_Instance, FileName, Index, ResX, ResY))
   End Function

   Public Function ReadImageResolutionEx(ByRef Buffer() As Byte, ByVal Index As Integer, ByRef ResX As Integer, ByRef ResY As Integer) As Boolean
      Return CBool(pdfReadImageResolutionEx(m_Instance, Buffer, Buffer.Length, Index, ResX, ResY))
   End Function

   Public Function Rectangle(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfRectangle(m_Instance, PosX, PosY, Width, Height, FillMode))
   End Function

   Public Function ReEncryptPDFA(ByVal FileName As String, ByVal PwdType As TPwdType, ByVal InPwd As String, ByVal NewOpenPwd As String, ByVal NewOwnerPwd As String, ByVal NewKeyLen As TKeyLen, ByVal Restrict As TRestrictions) As Integer
      Return pdfReEncryptPDFA(m_Instance, FileName, PwdType, InPwd, NewOpenPwd, NewOwnerPwd, NewKeyLen, Restrict)
   End Function

   Public Function ReEncryptPDFW(ByVal FileName As String, ByVal PwdType As TPwdType, ByVal InPwd As String, ByVal NewOpenPwd As String, ByVal NewOwnerPwd As String, ByVal NewKeyLen As TKeyLen, ByVal Restrict As TRestrictions) As Integer
      Return pdfReEncryptPDFW(m_Instance, FileName, PwdType, InPwd, NewOpenPwd, NewOwnerPwd, NewKeyLen, Restrict)
   End Function

   Public Function RenameSpotColor(ByVal Colorant As String, ByVal NewName As String) As Integer
      Return pdfRenameSpotColor(m_Instance, Colorant, NewName)
   End Function

   Public Function ReplaceFontA(ByVal PDFFont As IntPtr, ByVal Name As String, ByVal Style As TFStyle, ByVal NameIsFamilyName As Boolean) As Integer
      Return pdfReplaceFontA(m_Instance, PDFFont, Name, Style, CInt(NameIsFamilyName))
   End Function

   Public Function ReplaceFontW(ByVal PDFFont As IntPtr, ByVal Name As String, ByVal Style As TFStyle, ByVal NameIsFamilyName As Boolean) As Integer
      Return pdfReplaceFontW(m_Instance, PDFFont, Name, Style, CInt(NameIsFamilyName))
   End Function

   Public Function ReplaceFontExA(ByVal PDFFont As IntPtr, ByVal FontFile As String, ByVal Embed As Boolean) As Integer
      Return pdfReplaceFontExA(m_Instance, PDFFont, FontFile, CInt(Embed))
   End Function

   Public Function ReplaceFontExW(ByVal PDFFont As IntPtr, ByVal FontFile As String, ByVal Embed As Boolean) As Integer
      Return pdfReplaceFontExW(m_Instance, PDFFont, FontFile, CInt(Embed))
   End Function

   Public Function ReplaceICCProfileA(ByVal ColorSpace As Integer, ByVal ICCFile As String) As Integer
      Return pdfReplaceICCProfileA(m_Instance, ColorSpace, ICCFile)
   End Function

   Public Function ReplaceICCProfileW(ByVal ColorSpace As Integer, ByVal ICCFile As String) As Integer
      Return pdfReplaceICCProfileW(m_Instance, ColorSpace, ICCFile)
   End Function

   Public Function ReplacePageTextA(ByVal NewText As String, ByRef Stack As TPDFStack) As Boolean
      Return CBool(pdfReplacePageTextA(m_Instance, NewText, Stack))
   End Function

   Public Function ReplacePageTextExA(ByVal NewText As String, ByRef Stack As TPDFStack) As Boolean
      Return CBool(pdfReplacePageTextExA(m_Instance, NewText, Stack))
   End Function

   Public Function ReplacePageTextExW(ByVal NewText As String, ByRef Stack As TPDFStack) As Boolean
      Return CBool(pdfReplacePageTextExW(m_Instance, NewText, Stack))
   End Function

   Public Function ResetLineDashPattern() As Boolean
      Return CBool(pdfResetLineDashPattern(m_Instance))
   End Function

   Public Function RestoreGraphicState() As Boolean
      Return CBool(pdfRestoreGraphicState(m_Instance))
   End Function

   Public Function RotateCoords(ByVal alpha As Double, ByVal OriginX As Double, ByVal OriginY As Double) As Integer
      Return pdfRotateCoords(m_Instance, alpha, OriginX, OriginY)
   End Function

   Public Function RoundRect(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Radius As Double, ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfRoundRect(m_Instance, PosX, PosY, Width, Height, Radius, FillMode))
   End Function

   Public Function RoundRectEx(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal rWidth As Double, ByVal rHeight As Double, ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfRoundRectEx(m_Instance, PosX, PosY, Width, Height, rWidth, rHeight, FillMode))
   End Function

   Public Function SaveGraphicState() As Boolean
      Return CBool(pdfSaveGraphicState(m_Instance))
   End Function

   Public Function ScaleCoords(ByVal sx As Double, ByVal sy As Double) As Integer
      Return pdfScaleCoords(m_Instance, sx, sy)
   End Function

   Public Function SelfTest() As Boolean
      Return CBool(pdfSelfTest(m_Instance))
   End Function

   Public Function Set3DAnnotProps(ByVal Annot As Integer, ByVal ActType As T3DActivationType, ByVal DeActType As T3DDeActivateType, ByVal InstType As T3DInstanceType, ByVal DeInstType As T3DDeActInstance, ByVal DisplToolbar As Boolean, ByVal DisplModelTree As Boolean) As Boolean
      Return pdfSet3DAnnotProps(m_Instance, Annot, ActType, DeActType, InstType, DeInstType, CInt(DisplToolbar), CInt(DisplModelTree)) <> 0
   End Function

   Public Function Set3DAnnotScriptA(ByVal Annot As Integer, ByVal Value As String) As Boolean
      Return pdfSet3DAnnotScriptA(m_Instance, Annot, Value, Value.Length) <> 0
   End Function

   Public Function SetAllocBy(ByVal Value As Integer) As Integer
      Return pdfSetAllocBy(m_Instance, Value)
   End Function

   Public Function SetAnnotBorderStyle(ByVal Handle As Integer, ByVal Style As TBorderStyle) As Boolean
      Return CBool(pdfSetAnnotBorderStyle(m_Instance, Handle, Style))
   End Function

   Public Function SetAnnotBorderWidth(ByVal Handle As Integer, ByVal LineWidth As Double) As Boolean
      Return CBool(pdfSetAnnotBorderWidth(m_Instance, Handle, LineWidth))
   End Function

   Public Function SetAnnotColor(ByVal Handle As Integer, ByVal ColorType As TAnnotColor, ByVal CS As TPDFColorSpace, ByVal Color As Integer) As Boolean
      Return CBool(pdfSetAnnotColor(m_Instance, Handle, ColorType, CS, Color))
   End Function

   Public Function SetAnnotFlags(ByVal Flags As TAnnotFlags) As Boolean
      Return CBool(pdfSetAnnotFlags(m_Instance, Flags))
   End Function

   Public Function SetAnnotFlagsEx(ByVal Handle As Integer, ByVal Flags As Integer) As Boolean
      Return CBool(pdfSetAnnotFlagsEx(m_Instance, Handle, Flags))
   End Function

   Public Function SetAnnotHighlightMode(ByVal Handle As Integer, ByVal Mode As THighlightMode) As Boolean
      Return CBool(pdfSetAnnotHighlightMode(m_Instance, Handle, Mode))
   End Function

   Public Function SetAnnotIcon(ByVal Handle As Integer, ByVal Icon As TAnnotIcon) As Boolean
      Return CBool(pdfSetAnnotIcon(m_Instance, Handle, Icon))
   End Function

   Public Function SetAnnotOpenState(ByVal Handle As Integer, ByVal Open As Boolean) As Boolean
      Return CBool(pdfSetAnnotOpenState(m_Instance, Handle, Convert.ToInt32(Open)))
   End Function

   Public Function SetAnnotStringA(ByVal Handle As Integer, ByVal StringType As TAnnotString, ByVal Value As String) As Boolean
      Return CBool(pdfSetAnnotStringA(m_Instance, Handle, StringType, Value))
   End Function

   Public Function SetAnnotStringW(ByVal Handle As Integer, ByVal StringType As TAnnotString, ByVal Value As String) As Boolean
      Return CBool(pdfSetAnnotStringW(m_Instance, Handle, StringType, Value))
   End Function

   Public Function SetAnnotSubjectA(ByVal Handle As Integer, ByVal Value As String) As Boolean
      Return CBool(pdfSetAnnotSubjectA(m_Instance, Handle, Value))
   End Function

   Public Function SetAnnotSubjectW(ByVal Handle As Integer, ByVal Value As String) As Boolean
      Return CBool(pdfSetAnnotSubjectW(m_Instance, Handle, Value))
   End Function

   Public Function SetBBox(ByVal Boundary As TPageBoundary, ByVal LeftX As Double, ByVal LeftY As Double, ByVal RightX As Double, ByVal RightY As Double) As Boolean
      Return CBool(pdfSetBBox(m_Instance, Boundary, LeftX, LeftY, RightX, RightY))
   End Function

   Public Function SetBidiMode(ByVal Mode As TPDFBidiMode) As Boolean
      Return pdfSetBidiMode(m_Instance, Mode) <> 0
   End Function

   Public Function SetBookmarkDest(ByVal ABmk As Integer, ByVal DestType As TDestType, ByVal a As Double, ByVal b As Double, ByVal C As Double, ByVal d As Double) As Boolean
      Return CBool(pdfSetBookmarkDest(m_Instance, ABmk, DestType, a, b, C, d))
   End Function

   Public Function SetBookmarkStyle(ByVal ABmk As Integer, ByVal Style As TFStyle, ByVal RGBColor As Integer) As Boolean
      Return CBool(pdfSetBookmarkStyle(m_Instance, ABmk, Style, RGBColor))
   End Function

   Public Function SetBorderStyle(ByVal Style As TBorderStyle) As Boolean
      Return CBool(pdfSetBorderStyle(m_Instance, Style))
   End Function

   Public Function SetCharacterSpacing(ByVal Value As Double) As Boolean
      Return CBool(pdfSetCharacterSpacing(m_Instance, Value))
   End Function

   Public Function SetCheckBoxChar(ByVal CheckBoxChar As TCheckBoxChar) As Boolean
      Return CBool(pdfSetCheckBoxChar(m_Instance, CheckBoxChar))
   End Function

   Public Function SetCheckBoxDefState(ByVal Field As Integer, ByVal Checked As Boolean) As Boolean
      Return CBool(pdfSetCheckBoxDefState(m_Instance, Field, CInt(Checked)))
   End Function

   Public Function SetCheckBoxState(ByVal Field As Integer, ByVal Checked As Boolean) As Boolean
      Return CBool(pdfSetCheckBoxState(m_Instance, Field, CInt(Checked)))
   End Function

   Public Function SetCIDFontA(ByVal CMapHandle As Integer, ByVal Name As String, ByVal Style As TFStyle, ByVal Size As Double, ByVal Embed As Boolean) As Integer
      Return pdfSetCIDFontA(m_Instance, CMapHandle, Name, Style, Size, CInt(Embed))
   End Function

   Public Function SetCIDFontW(ByVal CMapHandle As Integer, ByVal Name As String, ByVal Style As TFStyle, ByVal Size As Double, ByVal Embed As Boolean) As Integer
      Return pdfSetCIDFontW(m_Instance, CMapHandle, Name, Style, Size, CInt(Embed))
   End Function

   Public Function SetCMapDirA(ByVal Path As String, ByVal Recursive As Boolean) As Integer
      Return pdfSetCMapDirA(m_Instance, Path, CInt(Recursive))
   End Function

   Public Function SetCMapDirW(ByVal Path As String, ByVal Recursive As Boolean) As Integer
      Return pdfSetCMapDirW(m_Instance, Path, CInt(Recursive))
   End Function

   Public Function SetColDefFile(ByVal EmbFile As Integer) As Boolean
      Return CBool(pdfSetColDefFile(m_Instance, EmbFile))
   End Function

   Public Function SetColors(ByVal Color As Integer) As Boolean
      Return CBool(pdfSetColors(m_Instance, Color))
   End Function

   Public Function SetColorSpace(ByVal ColorSpace As TPDFColorSpace) As Boolean
      Return CBool(pdfSetColorSpace(m_Instance, ColorSpace))
   End Function

   Public Function SetColSortField(ByVal ColField As Integer, ByVal AscendingOrder As Boolean) As Boolean
      Return CBool(pdfSetColSortField(m_Instance, ColField, CInt(AscendingOrder)))
   End Function

   Public Function SetCompressionFilter(ByVal ComprFilter As TCompressionFilter) As Boolean
      Return CBool(pdfSetCompressionFilter(m_Instance, ComprFilter))
   End Function

   Public Function SetCompressionLevel(ByVal CompressLevel As TCompressionLevel) As Boolean
      Return CBool(pdfSetCompressionLevel(m_Instance, CompressLevel))
   End Function

   Public Function SetContent(ByRef Buffer() As Byte) As Boolean
      Return CBool(pdfSetContent(m_Instance, Buffer, Buffer.Length))
   End Function

   Public Function SetDateTimeFormat(ByVal TxtField As Integer, ByVal Fmt As TPDFDateTime) As Boolean
      Return CBool(pdfSetDateTimeFormat(m_Instance, TxtField, Fmt))
   End Function

   Public Function SetDefBitsPerPixel(ByVal Value As Integer) As Boolean
      Return CBool(pdfSetDefBitsPerPixel(m_Instance, Value))
   End Function

   Public Function SetDocInfoA(ByVal DInfo As TDocumentInfo, ByVal Value As String) As Boolean
      Return CBool(pdfSetDocInfoA(m_Instance, DInfo, Value))
   End Function

   Public Function SetDocInfoW(ByVal DInfo As TDocumentInfo, ByVal Value As String) As Boolean
      Return CBool(pdfSetDocInfoW(m_Instance, DInfo, Value))
   End Function

   Public Function SetDocInfoExA(ByVal DInfo As TDocumentInfo, ByVal Key As String, ByVal Value As String) As Boolean
      Return CBool(pdfSetDocInfoExA(m_Instance, DInfo, Key, Value))
   End Function

   Public Function SetDocInfoExW(ByVal DInfo As TDocumentInfo, ByVal Key As String, ByVal Value As String) As Boolean
      Return CBool(pdfSetDocInfoExW(m_Instance, DInfo, Key, Value))
   End Function

   Public Function SetDrawDirection(ByVal Direction As TDrawDirection) As Boolean
      Return CBool(pdfSetDrawDirection(m_Instance, Direction))
   End Function

   Public Function SetEMFFrameDPI(ByVal DPIX As Integer, ByVal DPIY As Integer) As Boolean
      Return CBool(pdfSetEMFFrameDPI(m_Instance, DPIX, DPIY))
   End Function

   Public Function SetEMFPatternDistance(ByVal Value As Double) As Boolean
      Return CBool(pdfSetEMFPatternDistance(m_Instance, Value))
   End Function

   Public Function SetErrorMode(ByVal ErrMode As TErrMode) As Boolean
      Return CBool(pdfSetErrorMode(m_Instance, ErrMode))
   End Function

   Public Function SetExtColorSpace(ByVal Handle As Integer) As Boolean
      Return pdfSetExtColorSpace(m_Instance, Handle) <> 0
   End Function

   Public Function SetExtGState(ByVal Handle As Integer) As Boolean
      Return CBool(pdfSetExtGState(m_Instance, Handle))
   End Function

   Public Function SetFieldBackColor(ByVal AColor As Integer) As Boolean
      Return CBool(pdfSetFieldBackColor(m_Instance, AColor))
   End Function

   Public Function SetFieldBBox(ByVal Field As Integer, ByRef BBox As TPDFRect) As Boolean
      Return CBool(pdfSetFieldBBox(m_Instance, Field, BBox))
   End Function

   Public Function SetFieldBorderColor(ByVal AColor As Integer) As Boolean
      Return CBool(pdfSetFieldBorderColor(m_Instance, AColor))
   End Function

   Public Function SetFieldBorderStyle(ByVal Field As Integer, ByVal Style As TBorderStyle) As Boolean
      Return CBool(pdfSetFieldBorderStyle(m_Instance, Field, Style))
   End Function

   Public Function SetFieldBorderWidth(ByVal Field As Integer, ByVal LineWidth As Double) As Boolean
      Return CBool(pdfSetFieldBorderWidth(m_Instance, Field, LineWidth))
   End Function

   Public Function SetFieldColor(ByVal Field As Integer, ByVal ColorType As TFieldColor, ByVal CS As TPDFColorSpace, ByVal Color As Integer) As Boolean
      Return CBool(pdfSetFieldColor(m_Instance, Field, ColorType, CS, Color))
   End Function

   Public Function SetFieldExpValue(ByVal Field As Integer, ByVal ValIndex As Integer, ByVal Value As String, ByVal ExpValue As String, ByVal Selected As Boolean) As Boolean
      Return CBool(pdfSetFieldExpValue(m_Instance, Field, ValIndex, Value, ExpValue, CInt(Selected)))
   End Function

   Public Function SetFieldFlags(ByVal Field As Integer, ByVal Flags As TFieldFlags, ByVal DoReset As Boolean) As Boolean
      Return CBool(pdfSetFieldFlags(m_Instance, Field, Flags, CInt(DoReset)))
   End Function

   Public Function SetFieldFontSize(ByVal Field As Integer, ByVal FontSize As Double) As Boolean
      Return CBool(pdfSetFieldFontSize(m_Instance, Field, FontSize))
   End Function

   Public Function SetFieldHighlightMode(ByVal Field As Integer, ByVal Mode As THighlightMode) As Boolean
      Return CBool(pdfSetFieldHighlightMode(m_Instance, Field, Mode))
   End Function

   Public Function SetFieldIndex(ByVal Field As Integer, ByVal Index As Integer) As Boolean
      Return CBool(pdfSetFieldIndex(m_Instance, Field, Index))
   End Function

   Public Function SetFieldMapNameA(ByVal Field As Integer, ByVal Value As String) As Boolean
      Return CBool(pdfSetFieldMapNameA(m_Instance, Field, Value))
   End Function

   Public Function SetFieldMapNameW(ByVal Field As Integer, ByVal Value As String) As Boolean
      Return CBool(pdfSetFieldMapNameW(m_Instance, Field, Value))
   End Function

   Public Function SetFieldName(ByVal Field As Integer, ByVal NewName As String) As Boolean
      Return CBool(pdfSetFieldName(m_Instance, Field, NewName))
   End Function

   Public Function SetFieldOrientation(ByVal Field As Integer, ByVal Value As Integer) As Boolean
      Return CBool(pdfSetFieldOrientation(m_Instance, Field, Value))
   End Function

   Public Function SetFieldTextAlign(ByVal Field As Integer, ByVal Align As TTextAlign) As Boolean
      Return CBool(pdfSetFieldTextAlign(m_Instance, Field, Align))
   End Function

   Public Function SetFieldTextColor(ByVal Color As Integer) As Boolean
      Return CBool(pdfSetFieldTextColor(m_Instance, Color))
   End Function

   Public Function SetFieldToolTipA(ByVal Field As Integer, ByVal Value As String) As Boolean
      Return CBool(pdfSetFieldToolTipA(m_Instance, Field, Value))
   End Function

   Public Function SetFieldToolTipW(ByVal Field As Integer, ByVal Value As String) As Boolean
      Return CBool(pdfSetFieldToolTipW(m_Instance, Field, Value))
   End Function

   Public Function SetFillColor(ByVal Color As Integer) As Boolean
      Return CBool(pdfSetFillColor(m_Instance, Color))
   End Function

   Public Function SetFillColorEx(ByVal Color() As Byte, ByVal NumComponents As Integer) As Boolean
      Return pdfSetFillColorEx(m_Instance, Color, NumComponents) <> 0
   End Function

   Public Function SetFontA(ByVal Name As String, ByVal Style As TFStyle, ByVal Size As Double, ByVal Embed As Boolean, ByVal CP As TCodepage) As Integer
      Return pdfSetFontA(m_Instance, Name, Style, Size, CInt(Embed), CP)
   End Function

   Public Function SetFontW(ByVal Name As String, ByVal Style As TFStyle, ByVal Size As Double, ByVal Embed As Boolean, ByVal CP As TCodepage) As Integer
      Return pdfSetFontW(m_Instance, Name, Style, Size, CInt(Embed), CP)
   End Function

   Public Function SetFontExA(ByVal Name As String, ByVal Style As TFStyle, ByVal Size As Double, ByVal Embed As Boolean, ByVal CP As TCodepage) As Integer
      Return pdfSetFontExA(m_Instance, Name, Style, Size, CInt(Embed), CP)
   End Function

   Public Function SetFontExW(ByVal Name As String, ByVal Style As TFStyle, ByVal Size As Double, ByVal Embed As Boolean, ByVal CP As TCodepage) As Integer
      Return pdfSetFontExW(m_Instance, Name, Style, Size, CInt(Embed), CP)
   End Function

   Public Function SetFontOrigin(ByVal Origin As TOrigin) As Boolean
      Return CBool(pdfSetFontOrigin(m_Instance, Origin))
   End Function

   Public Sub SetFontSearchOrder(ByVal Order() As TFontBaseType)
      pdfSetFontSearchOrder(m_Instance, Order)
   End Sub

   Public Function SetFontSelMode(ByVal Mode As TFontSelMode) As Boolean
      Return CBool(pdfSetFontSelMode(m_Instance, Mode))
   End Function

   Public Function SetFontWeight(ByVal Weight As Integer) As Boolean
      Return CBool(pdfSetFontWeight(m_Instance, Weight))
   End Function

   Public Sub SetGStateFlags(ByVal Flags As TGStateFlags, ByVal Reset As Boolean)
      pdfSetGStateFlags(m_Instance, Flags, CInt(Reset))
   End Sub

   Public Function SetIconColor(ByVal Color As Integer) As Boolean
      Return CBool(pdfSetIconColor(m_Instance, Color))
   End Function

   Public Function SetImportFlags(ByVal Flags As TImportFlags) As Boolean
      Return CBool(pdfSetImportFlags(m_Instance, Flags))
   End Function

   Public Function SetImportFlags2(ByVal Flags As TImportFlags2) As Boolean
      Return CBool(pdfSetImportFlags2(m_Instance, Flags))
   End Function

   Public Function SetItalicAngle(ByVal Angle As Double) As Boolean
      Return CBool(pdfSetItalicAngle(m_Instance, Angle))
   End Function

   Public Function SetJPEGQuality(ByVal Value As Integer) As Boolean
      Return CBool(pdfSetJPEGQuality(m_Instance, Value))
   End Function

   Public Function SetLanguage(ByVal ISOTag As String) As Boolean
      Return CBool(pdfSetLanguage(m_Instance, ISOTag))
   End Function

   Public Function SetLeading(ByVal Value As Double) As Boolean
      Return CBool(pdfSetLeading(m_Instance, Value))
   End Function

   Public Function SetLicenseKey(ByVal Value As String) As Boolean
      Return CBool(pdfSetLicenseKey(m_Instance, Value))
   End Function

   Public Function SetLineCapStyle(ByVal Style As TLineCapStyle) As Boolean
      Return CBool(pdfSetLineCapStyle(m_Instance, Style))
   End Function

   Public Function SetLineDashPattern(ByVal Dash As String, ByVal Phase As Integer) As Boolean
      Return CBool(pdfSetLineDashPattern(m_Instance, Dash, Phase))
   End Function

   Public Function SetLineDashPatternEx(ByRef Dash() As Double, ByVal NumValues As Integer, ByVal Phase As Integer) As Boolean
      Return CBool(pdfSetLineDashPatternEx(m_Instance, Dash, NumValues, Phase))
   End Function

   Public Function SetLineJoinStyle(ByVal Style As TLineJoinStyle) As Boolean
      Return CBool(pdfSetLineJoinStyle(m_Instance, Style))
   End Function

   Public Function SetLineWidth(ByVal Value As Double) As Boolean
      Return CBool(pdfSetLineWidth(m_Instance, Value))
   End Function

   Public Function SetLinkHighlightMode(ByVal Mode As THighlightMode) As Boolean
      Return CBool(pdfSetLinkHighlightMode(m_Instance, Mode))
   End Function

   Public Function SetListFont(ByVal Handle As Integer) As Boolean
      Return CBool(pdfSetListFont(m_Instance, Handle))
   End Function

   Public Function SetMatrix(ByRef Matrix As TCTM) As Boolean
      Return CBool(pdfSetMatrix(m_Instance, Matrix))
   End Function

   Public Function SetMaxFieldLen(ByVal TxtField As Integer, ByVal MaxLen As Integer) As Boolean
      Return CBool(pdfSetMaxFieldLen(m_Instance, TxtField, MaxLen))
   End Function

   Public Function SetMetaConvFlags(ByVal Flags As TMetaFlags) As Boolean
      Return CBool(pdfSetMetaConvFlags(m_Instance, Flags))
   End Function

   Public Function SetMiterLimit(ByVal Value As Double) As Boolean
      Return CBool(pdfSetMiterLimit(m_Instance, Value))
   End Function

   Public Function SetNeedAppearance(ByVal Value As Boolean) As Boolean
      Return pdfSetNeedAppearance(m_Instance, CInt(Value)) <> 0
   End Function

   Public Function SetNumberFormat(ByVal TxtField As Integer, ByVal Sep As TDecSeparator, ByVal DecPlaces As Integer, ByVal NegStyle As TNegativeStyle, ByVal CurrStr As String, ByVal Prepend As Integer) As Boolean
      Return CBool(pdfSetNumberFormat(m_Instance, TxtField, Sep, DecPlaces, NegStyle, CurrStr, Prepend))
   End Function

   Public Function SetOnErrorProc(ByVal ErrProc As TErrorProc) As Boolean
      m_AddrErrorProc = ErrProc
      Return CBool(pdfSetOnErrorProc(m_Instance, IntPtr.Zero, m_AddrErrorProc))
   End Function

   Public Function SetOnPageBreakProc(ByVal PageBreakProc As TOnPageBreakProc) As Boolean
      m_AddrOnPageBreak = PageBreakProc
      Return CBool(pdfSetOnPageBreakProc(m_Instance, IntPtr.Zero, m_AddrOnPageBreak))
   End Function

   Public Function SetOpacity(ByVal Value As Double) As Boolean
      Return CBool(pdfSetOpacity(m_Instance, Value))
   End Function

   Public Function SetOrientation(ByVal Value As Integer) As Boolean
      Return CBool(pdfSetOrientation(m_Instance, Value))
   End Function

   Public Function SetOrientationEx(ByVal Value As Integer) As Boolean
      Return CBool(pdfSetOrientationEx(m_Instance, Value))
   End Function

   Public Function SetPageCoords(ByVal PageCoords As TPageCoord) As Boolean
      Return CBool(pdfSetPageCoords(m_Instance, PageCoords))
   End Function

   Public Function SetPageFormat(ByVal Value As TPageFormat) As Boolean
      Return CBool(pdfSetPageFormat(m_Instance, Value))
   End Function

   Public Function SetPageHeight(ByVal Value As Double) As Boolean
      Return CBool(pdfSetPageHeight(m_Instance, Value))
   End Function

   Public Function SetPageLayout(ByVal Layout As TPageLayout) As Boolean
      Return CBool(pdfSetPageLayout(m_Instance, Layout))
   End Function

   Public Function SetPageMode(ByVal Mode As TPageMode) As Boolean
      Return CBool(pdfSetPageMode(m_Instance, Mode))
   End Function

   Public Function SetPageWidth(ByVal Value As Double) As Boolean
      Return CBool(pdfSetPageWidth(m_Instance, Value))
   End Function

   Public Function SetPDFVersion(ByVal Version As TPDFVersion) As Boolean
      Return CBool(pdfSetPDFVersion(m_Instance, Version))
   End Function

   Public Function SetPrintSettings(ByVal Mode As TDuplexMode, ByVal PickTrayByPDFSize As Integer, ByVal NumCopies As Integer, ByVal PrintScaling As TPrintScaling, ByRef PrintRanges() As Integer) As Boolean
      Return pdfSetPrintSettings(m_Instance, Mode, PickTrayByPDFSize, NumCopies, PrintScaling, PrintRanges, CInt(PrintRanges.Length / 2)) <> 0
   End Function

   Public Function SetProgressProc(ByVal InitProc As TInitProgress, ByVal ProgressProc As TProgress) As Integer
      m_AddrInitProgress = InitProc
      m_AddrProgress = ProgressProc
      Return pdfSetProgressProc(m_Instance, IntPtr.Zero, m_AddrInitProgress, m_AddrProgress)
   End Function

   Public Function SetResolution(ByVal Value As Integer) As Boolean
      Return CBool(pdfSetResolution(m_Instance, Value))
   End Function

   Public Function SetSaveNewImageFormat(ByVal Value As Boolean) As Boolean
      Return CBool(pdfSetSaveNewImageFormat(m_Instance, CInt(Value)))
   End Function

   Public Function SetSeparationInfo(ByVal Handle As Integer) As Boolean
      Return CBool(pdfSetSeparationInfo(m_Instance, Handle))
   End Function

   Public Function SetStrokeColor(ByVal Color As Integer) As Boolean
      Return CBool(pdfSetStrokeColor(m_Instance, Color))
   End Function

   Public Function SetStrokeColorEx(ByVal Color() As Byte, ByVal NumComponents As Integer) As Boolean
      Return pdfSetStrokeColorEx(m_Instance, Color, NumComponents) <> 0
   End Function

   Public Function SetTabLen(ByVal TabLen As Integer) As Boolean
      Return CBool(pdfSetTabLen(m_Instance, TabLen))
   End Function

   Public Function SetTextDrawMode(ByVal Mode As TDrawMode) As Boolean
      Return CBool(pdfSetTextDrawMode(m_Instance, Mode))
   End Function

   Public Function SetTextFieldValueA(ByVal Field As Integer, ByVal Value As String, ByVal DefValue As String, ByVal Align As TTextAlign) As Boolean
      Return CBool(pdfSetTextFieldValueA(m_Instance, Field, Value, DefValue, Align))
   End Function

   Public Function SetTextFieldValueW(ByVal Field As Integer, ByVal Value As String, ByVal DefValue As String, ByVal Align As TTextAlign) As Boolean
      Return CBool(pdfSetTextFieldValueW(m_Instance, Field, Value, DefValue, Align))
   End Function

   Public Function SetTextFieldValueExA(ByVal Field As Integer, ByVal Value As String) As Boolean
      Return pdfSetTextFieldValueExA(m_Instance, Field, Value) <> 0
   End Function

   Public Function SetTextFieldValueExW(ByVal Field As Integer, ByVal Value As String) As Boolean
      Return pdfSetTextFieldValueExW(m_Instance, Field, Value) <> 0
   End Function

   Public Function SetTextRect(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Boolean
      Return CBool(pdfSetTextRect(m_Instance, PosX, PosY, Width, Height))
   End Function

   Public Function SetTextRise(ByVal Value As Double) As Boolean
      Return CBool(pdfSetTextRise(m_Instance, Value))
   End Function

   Public Function SetTextScaling(ByVal Value As Double) As Boolean
      Return CBool(pdfSetTextScaling(m_Instance, Value))
   End Function

   Public Function SetTransparentColor(ByVal AColor As Integer) As Boolean
      Return CBool(pdfSetTransparentColor(m_Instance, AColor))
   End Function

   Public Sub SetTrapped(ByVal Value As Boolean)
      pdfSetTrapped(m_Instance, CInt(Value))
   End Sub

   Public Function SetUseExactPwd(ByVal Value As Boolean) As Boolean
      Return CBool(pdfSetUseExactPwd(m_Instance, CInt(Value)))
   End Function

   Public Function SetUseGlobalImpFiles(ByVal Value As Boolean) As Boolean
      Return CBool(pdfSetUseGlobalImpFiles(m_Instance, CInt(Value)))
   End Function

   Public Function SetUseStdFonts(ByVal Value As Boolean) As Boolean
      Return CBool(pdfSetUseStdFonts(m_Instance, CInt(Value)))
   End Function

   Public Function SetUseSwapFile(ByVal SwapContents As Boolean, ByVal SwapLimit As Integer) As Boolean
      Return CBool(pdfSetUseSwapFile(m_Instance, CInt(SwapContents), SwapLimit))
   End Function

   Public Function SetUseSwapFileEx(ByVal SwapContents As Boolean, ByVal SwapLimit As Integer, ByVal SwapDir As String) As Boolean
      Return CBool(pdfSetUseSwapFileEx(m_Instance, CInt(SwapContents), SwapLimit, SwapDir))
   End Function

   Public Function SetUseSystemFonts(ByVal Value As Boolean) As Boolean
      Return CBool(pdfSetUseSystemFonts(m_Instance, CInt(Value)))
   End Function

   Public Function SetUseTransparency(ByVal Value As Boolean) As Boolean
      Return CBool(pdfSetUseTransparency(m_Instance, CInt(Value)))
   End Function

   Public Function SetUseVisibleCoords(ByVal Value As Boolean) As Boolean
      Return CBool(pdfSetUseVisibleCoords(m_Instance, CInt(Value)))
   End Function

   Public Function SetViewerPreferences(ByVal Value As TViewerPreference, ByVal AddVal As TViewPrefAddVal) As Boolean
      Return CBool(pdfSetViewerPreferences(m_Instance, Value, AddVal))
   End Function

   Public Function SetWMFDefExtent(ByVal Width As Integer, ByVal Height As Integer) As Boolean
      Return CBool(pdfSetWMFDefExtent(m_Instance, Width, Height))
   End Function

   Public Function SetWMFPixelPerInch(ByVal Value As Integer) As Boolean
      Return CBool(pdfSetWMFPixelPerInch(m_Instance, Value))
   End Function

   Public Function SetWordSpacing(ByVal Value As Double) As Boolean
      Return CBool(pdfSetWordSpacing(m_Instance, Value))
   End Function

   Public Function SkewCoords(ByVal alpha As Double, ByVal beta As Double, ByVal OriginX As Double, ByVal OriginY As Double) As Boolean
      Return CBool(pdfSkewCoords(m_Instance, alpha, beta, OriginX, OriginY))
   End Function

   Public Function SortFieldsByIndex() As Boolean
      Return CBool(pdfSortFieldsByIndex(m_Instance))
   End Function

   Public Function SortFieldsByName() As Boolean
      Return CBool(pdfSortFieldsByName(m_Instance))
   End Function

   Public Function StampAnnotA(ByVal SubType As TRubberStamp, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Subject As String, ByVal Comment As String) As Integer
      Return pdfStampAnnotA(m_Instance, SubType, PosX, PosY, Width, Height, Author, Subject, Comment)
   End Function

   Public Function StampAnnotW(ByVal SubType As TRubberStamp, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Subject As String, ByVal Comment As String) As Integer
      Return pdfStampAnnotW(m_Instance, SubType, PosX, PosY, Width, Height, Author, Subject, Comment)
   End Function

   Public Function StrokePath() As Boolean
      Return CBool(pdfStrokePath(m_Instance))
   End Function

   Public Function TextAnnotA(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Text As String, ByVal Icon As TAnnotIcon, ByVal DoOpen As Boolean) As Integer
      Return pdfTextAnnotA(m_Instance, PosX, PosY, Width, Height, Author, Text, Icon, CInt(DoOpen))
   End Function

   Public Function TextAnnotW(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Text As String, ByVal Icon As TAnnotIcon, ByVal DoOpen As Boolean) As Integer
      Return pdfTextAnnotW(m_Instance, PosX, PosY, Width, Height, Author, Text, Icon, CInt(DoOpen))
   End Function

   Public Function TranslateCoords(ByVal OriginX As Double, ByVal OriginY As Double) As Boolean
      Return CBool(pdfTranslateCoords(m_Instance, OriginX, OriginY))
   End Function

   Public Function TranslateRawCode(ByVal IFont As IntPtr, ByVal Text As IntPtr, ByVal Len As Integer, ByRef Width As Double, ByRef OutText() As Byte, ByRef OutLen As Integer, ByRef Decoded As Integer, ByVal CharSpacing As Single, ByVal WordSpacing As Single, ByVal TextScale As Single) As Integer
      TranslateRawCode = fntTranslateRawCode(IFont, Text, Len, Width, OutText, OutLen, Decoded, CharSpacing, WordSpacing, TextScale)
   End Function

   Public Function TranslateString(ByRef Stack As TPDFStack, ByRef OutText As String, ByVal Flags As Integer) As Integer
      Dim buf(CInt(Stack.TextLen * 16 / 10) * 2 + 64) As Byte
      Dim retval As Integer = fntTranslateString(Stack, buf, CInt(buf.Length / 2), Flags)
      OutText = System.Text.UnicodeEncoding.Unicode.GetString(buf, 0, retval * 2)
      Return retval
   End Function

   Public Function TranslateString2(ByVal IFont As IntPtr, ByVal Text As IntPtr, ByVal Len As Integer, ByRef OutText As String, ByVal Flags As Integer) As Integer
      Dim buf(CInt(Len * 16 / 10) * 2 + 64) As Byte
      Dim retval As Integer = fntTranslateString2(IFont, Text, Len, buf, CInt(buf.Length / 2), Flags)
      OutText = System.Text.UnicodeEncoding.Unicode.GetString(buf, 0, retval * 2)
      Return retval
   End Function

   Public Function Triangle(ByVal x1 As Double, ByVal y1 As Double, ByVal x2 As Double, ByVal y2 As Double, ByVal x3 As Double, ByVal y3 As Double, ByVal FillMode As TPathFillMode) As Boolean
      Return CBool(pdfTriangle(m_Instance, x1, y1, x2, y2, x3, y3, FillMode))
   End Function

   Public Function WebLinkA(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal URL As String) As Integer
      Return pdfWebLinkA(m_Instance, PosX, PosY, Width, Height, URL)
   End Function

   Public Function WebLinkW(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal URL As String) As Integer
      Return pdfWebLinkW(m_Instance, PosX, PosY, Width, Height, URL)
   End Function

   Public Function WeightFromStyle(ByVal Style As TFStyle) As Integer
      Return CInt((Style And Not &HC00FFFFF) / 1048576)
   End Function

   Public Function WeightToStyle(ByVal Weight As Integer) As TFStyle
      Return CType(Weight * 1048576, TFStyle)
   End Function

   Public Function WidthFromStyle(ByVal Style As TFStyle) As Integer
      Return CInt((Style And Not &HFFF2D00F) / 256)
   End Function

   Public Function WidthToStyle(ByVal Width As Integer) As TFStyle
      Return CType(Width * 256, TFStyle)
   End Function

   Public Function WriteAngleTextA(ByVal AText As String, ByVal Angle As Double, ByVal PosX As Double, ByVal PosY As Double, ByVal Radius As Double, ByVal YOrigin As Double) As Boolean
      Return CBool(pdfWriteAngleTextA(m_Instance, AText, Angle, PosX, PosY, Radius, YOrigin))
   End Function

   Public Function WriteAngleTextW(ByVal AText As String, ByVal Angle As Double, ByVal PosX As Double, ByVal PosY As Double, ByVal Radius As Double, ByVal YOrigin As Double) As Boolean
      Return CBool(pdfWriteAngleTextW(m_Instance, AText, Angle, PosX, PosY, Radius, YOrigin))
   End Function

   Public Function WriteFTextA(ByVal Align As TTextAlign, ByVal AText As String) As Boolean
      Return CBool(pdfWriteFTextA(m_Instance, Align, AText))
   End Function

   Public Function WriteFTextW(ByVal Align As TTextAlign, ByVal AText As String) As Boolean
      Return CBool(pdfWriteFTextW(m_Instance, Align, AText))
   End Function

   Public Function WriteFTextExA(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Align As TTextAlign, ByVal AText As String) As Boolean
      Return CBool(pdfWriteFTextExA(m_Instance, PosX, PosY, Width, Height, Align, AText))
   End Function

   Public Function WriteFTextExW(ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Align As TTextAlign, ByVal AText As String) As Boolean
      Return CBool(pdfWriteFTextExW(m_Instance, PosX, PosY, Width, Height, Align, AText))
   End Function

   Public Function WriteTextA(ByVal PosX As Double, ByVal PosY As Double, ByVal AText As String) As Boolean
      Return CBool(pdfWriteTextA(m_Instance, PosX, PosY, AText))
   End Function

   Public Function WriteTextW(ByVal PosX As Double, ByVal PosY As Double, ByVal AText As String) As Boolean
      Return CBool(pdfWriteTextW(m_Instance, PosX, PosY, AText))
   End Function

   Public Function WriteTextExA(ByVal PosX As Double, ByVal PosY As Double, ByVal AText As String, ByVal Len As Integer) As Boolean
      Return CBool(pdfWriteTextExA(m_Instance, PosX, PosY, AText, Len))
   End Function

   Public Function WriteTextExW(ByVal PosX As Double, ByVal PosY As Double, ByVal AText As String, ByVal Len As Integer) As Boolean
      Return CBool(pdfWriteTextExW(m_Instance, PosX, PosY, AText, Len))
   End Function

   Public Function WriteTextMatrixA(ByRef Matrix As TCTM, ByVal AText As String) As Boolean
      Return CBool(pdfWriteTextMatrixA(m_Instance, Matrix, AText))
   End Function

   Public Function WriteTextMatrixW(ByRef Matrix As TCTM, ByVal AText As String) As Boolean
      Return CBool(pdfWriteTextMatrixW(m_Instance, Matrix, AText))
   End Function

   Public Function WriteTextMatrixExA(ByRef Matrix As TCTM, ByVal AText As String, ByVal Len As Integer) As Boolean
      Return CBool(pdfWriteTextMatrixExA(m_Instance, Matrix, AText, Len))
   End Function

   Public Function WriteTextMatrixExW(ByRef Matrix As TCTM, ByVal AText As String, ByVal Len As Integer) As Boolean
      Return CBool(pdfWriteTextMatrixExW(m_Instance, Matrix, AText, Len))
   End Function

   ' --------------------------------------------- Private Functions ---------------------------------------------

   Private Sub GetIntAnnot(ByRef IAnnot As TPDFAnnotation_I, ByRef Annot As TPDFAnnotation)
      Annot.AnnotType = CType(IAnnot.AnnotType, TAnnotType)
      Annot.Deleted = CBool(IAnnot.Deleted)
      Annot.BBox = IAnnot.BBox
      Annot.BorderWidth = IAnnot.BorderWidth
      Annot.BorderColor = IAnnot.BorderColor
      Annot.BorderStyle = CType(IAnnot.BorderStyle, TBorderStyle)
      Annot.BackColor = IAnnot.BackColor
      Annot.Handle = IAnnot.Handle
      Annot.PageNum = IAnnot.PageNum
      Annot.HighlightMode = CType(IAnnot.HighlightMode, THighlightMode)
      Annot.Author = ToString(IAnnot.AuthorA, IAnnot.AuthorW)
      Annot.Content = ToString(IAnnot.ContentA, IAnnot.ContentW)
      Annot.Name = ToString(IAnnot.NameA, IAnnot.NameW)
      Annot.Subject = ToString(IAnnot.SubjectA, IAnnot.SubjectW)
   End Sub

   Private Sub GetIntAnnotEx(ByRef IAnnot As TPDFAnnotationEx_I, ByRef Annot As TPDFAnnotationEx)
      Annot.AnnotType = CType(IAnnot.AnnotType, TAnnotType)
      Annot.Deleted = CBool(IAnnot.Deleted)
      Annot.BBox = IAnnot.BBox
      Annot.BorderWidth = IAnnot.BorderWidth
      Annot.BorderColor = IAnnot.BorderColor
      Annot.BorderStyle = CType(IAnnot.BorderStyle, TBorderStyle)
      Annot.BackColor = IAnnot.BackColor
      Annot.Handle = IAnnot.Handle
      Annot.PageNum = IAnnot.PageNum
      Annot.HighlightMode = CType(IAnnot.HighlightMode, THighlightMode)
      Annot.Author = ToString(IAnnot.AuthorA, IAnnot.AuthorW)
      Annot.Content = ToString(IAnnot.ContentA, IAnnot.ContentW)
      Annot.Name = ToString(IAnnot.NameA, IAnnot.NameW)
      Annot.Subject = ToString(IAnnot.SubjectA, IAnnot.SubjectW)
      Annot.DestPage = IAnnot.DestPage
      Annot.DestPos = IAnnot.DestPos
      Annot.DestType = IAnnot.DestType
      Annot.DestFile = ToString(IAnnot.DestFile, False)
      Annot.Icon = IAnnot.Icon
      Annot.StampName = ToString(IAnnot.StampName, False)
      Annot.AnnotFlags = IAnnot.AnnotFlags
      Annot.CreateDate = ToString(IAnnot.CreateDate, False)
      Annot.ModDate = ToString(IAnnot.ModDate, False)
      Annot.Grouped = IAnnot.Grouped <> 0
      Annot.Open = IAnnot.Open <> 0
      Annot.Parent = IAnnot.Parent
      Annot.PopUp = IAnnot.PopUp
      Annot.State = ToString(IAnnot.State, False)
      Annot.StateModel = ToString(IAnnot.StateModel, False)
      Annot.EmbeddedFile = IAnnot.EmbeddedFile
      Annot.Subtype = ToString(IAnnot.Subtype, False)
   End Sub

   Private Function GetIntColorSpaceObj(ByRef ics As TPDFColorSpaceObj_I, ByRef cs As TPDFColorSpaceObj) As Boolean
      Dim i As Integer
      cs.ColorSpaceType = ics.ColorSpaceType
      cs.Alternate = ics.Alternate
      cs.IAlternate = ics.IAlternate
      cs.Buffer = ics.Buffer
      cs.BufSize = ics.BufSize
      cs.NumInComponents = ics.NumInComponents
      cs.NumOutComponents = ics.NumOutComponents
      cs.NumColors = ics.NumColors
      cs.Metadata = ics.Metadata
      cs.MetadataSize = ics.MetadataSize
      cs.IFunction = ics.IFunction
      cs.IAttributes = ics.IAttributes
      cs.IColorSpaceObj = ics.IColorSpaceObj
      If Not IntPtr.Zero.Equals(ics.BlackPoint) Then
         ReDim cs.BlackPoint(2)
         Marshal.Copy(ics.BlackPoint, cs.BlackPoint, 0, 3)
      Else
         cs.BlackPoint = Nothing
      End If
      If Not IntPtr.Zero.Equals(ics.WhitePoint) Then
         ReDim cs.WhitePoint(2)
         Marshal.Copy(ics.WhitePoint, cs.WhitePoint, 0, 3)
      Else
         cs.WhitePoint = Nothing
      End If
      If Not IntPtr.Zero.Equals(ics.Gamma) Then
         ReDim cs.Gamma(cs.NumInComponents - 1)
         Marshal.Copy(ics.Gamma, cs.Gamma, 0, cs.NumInComponents)
      Else
         cs.Gamma = Nothing
      End If
      If Not IntPtr.Zero.Equals(ics.Range) Then
         If ics.ColorSpaceType = TExtColorSpace.esLab Then
            ReDim cs.Range(3)
            Marshal.Copy(ics.Range, cs.Range, 0, 4)
         Else
            ReDim cs.Range(cs.NumInComponents * 2 - 1)
            Marshal.Copy(ics.Range, cs.Range, 0, cs.NumInComponents * 2)
         End If
      Else
         cs.Range = Nothing
      End If
      If Not IntPtr.Zero.Equals(ics.Matrix) Then
         ReDim cs.Matrix(7)
         Marshal.Copy(ics.Matrix, cs.Matrix, 0, 8)
      Else
         cs.Matrix = Nothing
      End If
      If ics.ColorantsCount = 0 Then
         cs.Colorants = Nothing
      Else
         Dim len As Integer
         Dim bytes() As Byte
         ReDim cs.Colorants(ics.ColorantsCount - 1)
         For i = 0 To ics.ColorantsCount - 1
            len = pdfStrLenA(ics.Colorants(i))
            ReDim bytes(len)
            Marshal.Copy(ics.Colorants(i), bytes, 0, len)
            bytes = System.Text.Encoding.Convert(System.Text.Encoding.UTF8, System.Text.Encoding.Unicode, bytes)
            cs.Colorants(i) = System.Text.Encoding.Unicode.GetString(bytes)
         Next i
      End If
      Return True
   End Function

   Private Sub GetIntField(ByVal IField As TPDFField_I, ByRef Field As TPDFField)
      Dim nameLen As Integer
      Field.BackColor = IField.BackColor
      Field.BBox.rBottom = IField.BBox.rBottom
      Field.BBox.rLeft = IField.BBox.rLeft
      Field.BBox.rRight = IField.BBox.rRight
      Field.BBox.rTop = IField.BBox.rTop
      Field.BorderColor = IField.BorderColor
      Field.Checked = IField.Checked <> 0
      Field.BackCS = CType(IField.BackCS, TPDFColorSpace)
      Field.TextCS = CType(IField.TextCS, TPDFColorSpace)
      Field.Deleted = IField.Deleted <> 0
      Field.Handle = IField.Handle
      nameLen = pdfStrLenA(IField.FieldName)
      Field.FieldName = ToString(IField.FieldName, IField.FieldNameLen, nameLen <> IField.FieldNameLen)
      Field.FieldType = CType(IField.FieldType, TFieldType)
      Field.FontName = ToString(IField.FontName, False)
      Field.FontSize = IField.FontSize
      Field.KidCount = IField.KidCount
      Field.Parent = IField.Parent
      Field.TextColor = IField.TextColor
      Field.ToolTip = ToString(IField.ToolTip, IField.ToolTipLen, IField.UniToolTip <> 0)
      Field.Value = ToString(IField.Value, IField.ValLen, IField.UniVal <> 0)
   End Sub

   Private Sub GetIntFieldEx(ByVal IField As TPDFFieldEx_I, ByRef Field As TPDFFieldEx)
      Field.BackColor = IField.BackColor
      Field.BackColorSP = IField.BackColorSP
      Field.BBox = IField.BBox
      Field.BorderColor = IField.BorderColor
      Field.BorderColorSP = IField.BorderColorSP
      Field.BorderStyle = IField.BorderStyle
      Field.BorderWidth = IField.BorderWidth
      Field.CharSpacing = IField.CharSpacing
      Field.CheckBoxChar = IField.CheckBoxChar
      Field.Checked = CBool(IField.Checked)
      Field.DefState = IField.DefState
      Field.DefValue = ToString(IField.DefValueA, IField.DefValueW)
      Field.Deleted = CBool(IField.Deleted)
      Field.EditFont = ToString(IField.EditFont, False)
      Field.ExpValCount = IField.ExpValCount
      Field.ExpValue = ToString(IField.ExpValueA, IField.ExpValueW)
      Field.FieldFlags = IField.FieldFlags
      Field.FieldFont = ToString(IField.FieldFont, False)
      Field.FieldName = ToString(IField.FieldNameA, IField.FieldNameW)
      Field.FieldType = IField.FieldType
      Field.FontSize = IField.FontSize
      Field.GroupType = IField.GroupType
      Field.Handle = IField.Handle
      Field.HighlightMode = IField.HighlightMode
      Field.IEditFont = IField.IEditFont
      Field.IFieldFont = IField.IFieldFont
      Field.IsCalcField = CBool(IField.IsCalcField)
      Field.Kids = CopyIntPtrArray(IField.Kids, IField.KidCount)
      Field.MapName = ToString(IField.MapNameA, IField.MapNameW)
      Field.MaxLen = IField.MaxLen
      Field.PageNum = IField.PageNum
      Field.Parent = IField.Parent
      Field.Rotate = IField.Rotate
      Field.TextAlign = IField.TextAlign
      Field.TextColor = IField.TextColor
      Field.TextColorSP = IField.TextColorSP
      Field.TextScaling = IField.TextScaling
      Field.ToolTip = ToString(IField.ToolTipA, IField.ToolTipW)
      Field.UniqueName = ToString(IField.UniqueNameA, IField.UniqueNameW)
      Field.Value = ToString(IField.ValueA, IField.ValueW)
      Field.WordSpacing = IField.WordSpacing
   End Sub

   Private Sub GetIntFont(ByRef IFont As TPDFFontObj_I, ByRef F As TPDFFontObj)
      F.Ascent = IFont.Ascent
      F.BaseFont = ToString(IFont.BaseFont, False)
      F.CapHeight = IFont.CapHeight
      F.DefWidth = IFont.DefWidth
      F.Descent = IFont.Descent
      F.FirstChar = IFont.FirstChar
      F.Flags = IFont.Flags
      F.FontFamily = ToString(IFont.FontFamily, IFont.FontFamilyUni <> 0)
      F.FontName = ToString(IFont.FontName, False)
      F.FontType = IFont.FontType
      F.ItalicAngle = IFont.ItalicAngle
      F.LastChar = IFont.LastChar
      F.SpaceWidth = IFont.SpaceWidth
      F.XHeight = IFont.XHeight
      ' We don't copy the buffer here! Copy the buffer manually with pdfCopyMem() if you need it!
      ' However, if IFont.Length1 == 0 and if Not IntPtr.Zero.Equals(IFont.FontFile) then the variable
      ' contains the file path to the used font file. This is always a null-terminated Ansi string at
      ' this time.
      F.FontFile = IFont.FontFile
      F.Length1 = IFont.Length1
      F.Length2 = IFont.Length2
      F.Length3 = IFont.Length3
      F.FontFileType = IFont.FontFileType
      If Not IntPtr.Zero.Equals(IFont.Encoding) Then
         F.Encoding = ToString(IFont.Encoding, 256, True).ToCharArray()
      Else
         F.Encoding = Nothing
      End If
      If IFont.WidthsCount > 0 Then
         ReDim F.Widths(IFont.WidthsCount - 1)
         Marshal.Copy(IFont.Widths, F.Widths, 0, IFont.WidthsCount)
      Else
         F.Widths = Nothing
      End If
   End Sub

   Private Overloads Function ToString(ByVal Ptr As IntPtr, ByVal IsUnicode As Boolean) As String
      If IntPtr.Zero.Equals(Ptr) Then Return Nothing
      If Not IsUnicode Then
         Return Marshal.PtrToStringAnsi(Ptr)
      Else
         Return Marshal.PtrToStringUni(Ptr)
      End If
   End Function

   Private Overloads Function ToString(ByVal Ptr As IntPtr, ByVal Len As Integer, ByVal IsUnicode As Boolean) As String
      If IntPtr.Zero.Equals(Ptr) Or Len <= 0 Then Return Nothing
      If Not IsUnicode Then
         Return Marshal.PtrToStringAnsi(Ptr, Len)
      Else
         Return Marshal.PtrToStringUni(Ptr, Len)
      End If
   End Function

   Private Overloads Function ToString(ByVal PtrA As IntPtr, ByVal PtrW As IntPtr) As String
      If Not IntPtr.Zero.Equals(PtrA) Then
         Return Marshal.PtrToStringAnsi(PtrA)
      ElseIf Not IntPtr.Zero.Equals(PtrW) Then
         Return Marshal.PtrToStringUni(PtrW)
      Else
         Return Nothing
      End If
   End Function

   Private Overloads Function ToString(ByVal PtrA As IntPtr, ByVal PtrW As IntPtr, ByVal Len As Integer) As String
      If Not IntPtr.Zero.Equals(PtrA) Then
         Return Marshal.PtrToStringAnsi(PtrA, Len)
      ElseIf Not IntPtr.Zero.Equals(PtrW) Then
         Return Marshal.PtrToStringUni(PtrW, Len)
      Else
         Return Nothing
      End If
   End Function

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TBookmark_I
      Dim Color As Integer
      Dim DestPage As Integer
      Dim DestPos As TPDFRect
      Dim DestType As TDestType
      Dim DoOpen As Integer
      Dim Parent As Integer
      Dim Style As Integer
      Dim Title As IntPtr
      Dim TitleLen As Integer
      Dim bUnicode As Integer
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFAnnotation_I
      Dim AnnotType As Integer
      Dim Deleted As Integer
      Dim BBox As TPDFRect
      Dim BorderWidth As Double
      Dim BorderColor As Integer
      Dim BorderStyle As Integer
      Dim BackColor As Integer
      Dim Handle As Integer
      Dim AuthorA As IntPtr
      Dim AuthorW As IntPtr
      Dim ContentA As IntPtr
      Dim ContentW As IntPtr
      Dim NameA As IntPtr
      Dim NameW As IntPtr
      Dim SubjectA As IntPtr
      Dim SubjectW As IntPtr
      Dim PageNum As Integer
      Dim HighlightMode As Integer
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFAnnotationEx_I
      Dim AnnotType As Integer
      Dim Deleted As Integer
      Dim BBox As TPDFRect
      Dim BorderWidth As Single
      Dim BorderColor As Integer
      Dim BorderStyle As Integer
      Dim BackColor As Integer
      Dim Handle As Integer
      Dim AuthorA As IntPtr
      Dim AuthorW As IntPtr
      Dim ContentA As IntPtr
      Dim ContentW As IntPtr
      Dim NameA As IntPtr
      Dim NameW As IntPtr
      Dim SubjectA As IntPtr
      Dim SubjectW As IntPtr
      Dim PageNum As Integer
      Dim HighlightMode As Integer
      ' Page link annotations only
      Dim DestPage As Integer
      Dim DestPos As TPDFRect
      Dim DestType As TDestType
      Dim DestFile As IntPtr         ' File link or web link annotations
      Dim Icon As Integer            ' The Icon type depends on the annotation type. If the annotation type is atText then the Icon
      ' is of type TAnnotIcon. If the annotation type is atFileAttach then it is of type
      ' TFileAttachIcon. If the annotation type is atStamp then the Icon is the stamp type (TRubberStamp).
      ' For any other annotion type this value is not set (-1).
      Dim StampName As IntPtr        ' Set only, if Icon == rsUserDefined
      Dim AnnotFlags As Integer      ' See TAnnotFlags for available flags

      Dim CreateDate As IntPtr       ' Creation Date -> Optional
      Dim ModDate As IntPtr          ' Modification Date -> Optional
      Dim Grouped As Integer         ' (Reply type) Meaningful only if Parent != -1 and Type != atPopUp. If true,
      ' the annotation is part of an annotation group. Properties like Content, CreateDate,
      ' ModDate, BackColor, Subject, and Open must be taken from the parent annotation.
      Dim Open As Integer            ' Meaningful only for annotations which have a corresponding PopUp annotation.
      Dim Parent As Integer          ' Parent annotation handle of a PopUp Annotation or the parent annotation if
      ' this annotation represents a state of a base annotation. In this case,
      ' the annotation type is always atText and only the following members should
      ' be considered:
      '    State      // The current state
      '    StateModel // Marked, Review, and so on
      '    CreateDate // Creation Date
      '    ModDate    // Modification Date
      '    Author     // The user who has set the state
      '    Content    // Not displayed in Adobe's Acrobat...
      '    Subject    // Not displayed in Adobe's Acrobat...
      ' The PopUp annotation of a text annotation which represent an Annotation State
      ' must be ignored.
      Dim PopUp As Integer           ' Handle of the corresponding PopUp annotation if any.
      Dim State As IntPtr            ' The state of the annotation.
      Dim StateModel As IntPtr       ' The state model (Marked, Review, and so on).
      Dim EmbeddedFile As Integer    ' FileAttach annotations only. A handle of an embedded file -> GetEmbeddedFile().
      Dim Subtype As IntPtr          ' Set only, if Type = atUnknownAnnot
      ' Reserved fields for future extensions
      Dim Reserved01 As IntPtr
      Dim Reserved02 As IntPtr
      Dim Reserved03 As IntPtr
      Dim Reserved04 As IntPtr
      Dim Reserved05 As IntPtr
      Dim Reserved06 As IntPtr
      Dim Reserved07 As IntPtr
      Dim Reserved08 As IntPtr
      Dim Reserved09 As IntPtr
      Dim Reserved10 As IntPtr
      Dim Reserved11 As IntPtr
      Dim Reserved12 As IntPtr
      Dim Reserved13 As IntPtr
      Dim Reserved14 As IntPtr
      Dim Reserved15 As IntPtr
      Dim Reserved16 As IntPtr
      Dim Reserved17 As IntPtr
      Dim Reserved18 As IntPtr
      Dim Reserved19 As IntPtr
      Dim Reserved20 As IntPtr
      Dim Reserved21 As IntPtr
      Dim Reserved22 As IntPtr
      Dim Reserved23 As IntPtr
      Dim Reserved24 As IntPtr
      Dim Reserved25 As IntPtr
      Dim Reserved26 As IntPtr
      Dim Reserved27 As IntPtr
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFChoiceValue_I
      Dim StructSize As Integer ' Must be initialzed to sizeof(TPDFChoiceValue_I)
      Dim ExpValueA As IntPtr
      Dim ExpValueW As IntPtr
      Dim ExpValueLen As Integer
      Dim ValueA As IntPtr
      Dim ValueW As IntPtr
      Dim ValueLen As Integer
      Dim Selected As Integer
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFColorSpaceObj_I
      Dim ColorSpaceType As CPDF.TExtColorSpace
      Dim Alternate As CPDF.TExtColorSpace ' Alternate color space or base space of an Indexed or Pattern color space.
      Dim IAlternate As IntPtr             ' Only set if the color space contains an alternate or base color space -> GetColorSpaceObjEx().
      Dim Buffer As IntPtr                 ' Contains either an ICC profile or the color table of an Indexed color space.
      Dim BufSize As Integer               ' Buffer length in bytes.
      Dim BlackPoint As IntPtr             ' CIE blackpoint. If set, the array contains exactly 3 values.
      Dim WhitePoint As IntPtr             ' CIE whitepoint. If set, the array contains exactly 3 values.
      Dim Gamma As IntPtr                  ' If set, one value per component.
      Dim Range As IntPtr                  ' min/max for each component or for the .a and .b components of a Lab color space.
      Dim Matrix As IntPtr                 ' XYZ matrix. If set, the array contains exactly 9 values.
      Dim NumInComponents As Integer       ' Number of input components.
      Dim NumOutComponents As Integer      ' Number of output components.
      Dim NumColors As Integer             ' HiVal + 1 as specified in the color space. Indexed color space only.
      <MarshalAs(UnmanagedType.ByValArray, SizeConst:=32)> Dim Colorants() As IntPtr ' Colorant names (Separation, DeviceN, and NChannel only).
      Dim ColorantsCount As Integer        ' The number of colorants in the array.
      Dim Metadata As IntPtr               ' Optional XMP metadata stream -> ICCBased only.
      Dim MetadataSize As Integer          ' Metadata length in bytes.
      Dim IFunction As IntPtr              ' IntPtr to function object -> Separation, DeviceN, and NChannel only.
      Dim IAttributes As IntPtr            ' Optional attributes of DeviceN or NChannel color spaces -> GetNChannelAttributes().
      Dim IColorSpaceObj As IntPtr         ' Pointer of the corresponding color space object
      Dim Reserved01 As IntPtr
      Dim Reserved02 As IntPtr
      Dim Reserved03 As IntPtr
      Dim Reserved04 As IntPtr
      Dim Reserved05 As IntPtr
      Dim Reserved06 As IntPtr
      Dim Reserved07 As IntPtr
      Dim Reserved08 As IntPtr
      Dim Reserved09 As IntPtr
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TDeviceNAttributes_I
      Dim IProcessColorSpace As IntPtr     ' Pointer to process color space or NULL -> GetColorSpaceEx().
      ' Does a process color space with more than 8 components exist? 6 components
      ' is the maximum so far I know. However, 8 components should be large enough for
      ' the next years...
      <MarshalAs(UnmanagedType.ByValArray, SizeConst:=8)> Dim ProcessColorants() As IntPtr
      Dim ProcessColorantsCount As Integer ' Number of process colorants in the array or zero if not set.
      <MarshalAs(UnmanagedType.ByValArray, SizeConst:=32)> Dim Separations() As IntPtr ' Optional pointers to Separation color spaces -> GetColorSpaceEx().
      Dim SeparationsCount As Integer      ' Number of color spaces in the array.
      Dim IMixingHints As IntPtr           ' Optional pointer to mixing hints. There is no API function at this time to access mixing hints.
      Dim Reserved01 As IntPtr
      Dim Reserved02 As IntPtr
      Dim Reserved03 As IntPtr
      Dim Reserved04 As IntPtr
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFField_I
      Dim FieldType As Integer
      Dim Deleted As Integer
      Dim BBox As TPDFRect
      Dim Handle As Integer
      Dim FieldName As IntPtr
      Dim FieldNameLen As Integer
      Dim BackCS As Integer
      Dim TextCS As Integer
      Dim BackColor As Integer
      Dim BorderColor As Integer
      Dim TextColor As Integer
      Dim Checked As Integer
      Dim Parent As Integer
      Dim KidCount As Integer
      Dim FontName As IntPtr
      Dim FontSize As Double
      Dim Value As IntPtr
      Dim UniVal As Integer
      Dim ValLen As Integer
      Dim ToolTip As IntPtr
      Dim UniToolTip As Integer
      Dim ToolTipLen As Integer
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFFieldEx_I
      Dim StructSize As Integer           ' Must be set to sizeof(TPDFFieldEx_I)
      Dim Deleted As Integer              ' If true, the field was marked as deleted by DeleteField()
      Dim BBox As TPDFRect                ' Bounding box of the field in bottom-up coordinates
      Dim FieldType As TFieldType         ' Field type
      Dim GroupType As TFieldType         ' If GroupType != FieldType the field is a terminal field of a field group
      Dim Handle As Integer               ' Field handle
      Dim BackColor As Integer            ' Background color
      Dim BackColorSP As TExtColorSpace   ' Color space of the background color
      Dim BorderColor As Integer          ' Border color
      Dim BorderColorSP As TExtColorSpace ' Color space of the border color
      Dim BorderStyle As TBorderStyle     ' Border style
      Dim BorderWidth As Single           ' Border width
      Dim CharSpacing As Single           ' Text fields only
      Dim Checked As Integer              ' Check boxes only
      Dim CheckBoxChar As Integer         ' ZapfDingbats character that is used to display the on state
      Dim DefState As TCheckBoxState      ' Check boxes only
      Dim DefValueA As IntPtr             ' Optional default value
      Dim DefValueW As IntPtr             ' Optional default value
      Dim IEditFont As IntPtr             ' Pointer to default editing font
      Dim EditFont As IntPtr              ' Postscript name of the editing font
      Dim ExpValCount As Integer          ' Combo and list boxes only. The values can be accessed with GetFieldExpValueEx()
      Dim ExpValueA As IntPtr             ' Check boxes only
      Dim ExpValueW As IntPtr             ' Check boxes only
      Dim FieldFlags As TFieldFlags       ' Field flags
      Dim IFieldFont As IntPtr            ' Pointer to the font that is used by the field
      Dim FieldFont As IntPtr             ' Postscript name of the font
      Private Reserved As Integer         ' Reserved field to avoid alignment errors
      Dim FontSize As Double              ' Font size. 0.0 means auto font size
      Dim FieldNameA As IntPtr            ' Note that children of a field group or radio button have no name
      Dim FieldNameW As IntPtr            ' Field name length in characters
      Dim HighlightMode As THighlightMode ' Highlight mode
      Dim IsCalcField As Integer          ' If true, the OnCalc event of the field is connected with a JavaScript action
      Dim MapNameA As IntPtr              ' Optional unique mapping name of the field
      Dim MapNameW As IntPtr              ' MapName length in characters
      Dim MaxLen As Integer               ' Text fields only -> zero means not restricted
      Dim Kids As IntPtr                  ' Array of child fields -> GetFieldEx2()
      Dim KidCount As Integer             ' Number of fields in the array
      Dim Parent As IntPtr                ' Pointer to parent field or NULL
      Dim PageNum As Integer              ' Page on which the field is used or -1
      Dim Rotate As Integer               ' Rotation angle in degrees
      Dim TextAlign As TTextAlign         ' Text fields only
      Dim TextColor As Integer            ' Text color
      Dim TextColorSP As TExtColorSpace   ' Color space of the field's text
      Dim TextScaling As Single           ' Text fields only
      Dim ToolTipA As IntPtr              ' Optional tool tip
      Dim ToolTipW As IntPtr              ' Optional tool tip
      Dim UniqueNameA As IntPtr           ' Optional unique name (NM key)
      Dim UniqueNameW As IntPtr           ' Optional unique name (NM key)
      Dim ValueA As IntPtr                ' Field value
      Dim ValueW As IntPtr                ' Field value
      Dim WordSpacing As Single           ' Text fields only
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFFileSpec_I
      Dim Buffer As IntPtr
      Dim BufSize As Integer
      Dim Compressed As Integer
      Dim ColItem As IntPtr
      Dim Name As IntPtr
      Dim NameUnicode As Integer
      Dim FileName As IntPtr
      Dim IsURL As Integer
      Dim UF As IntPtr
      Dim UFUnicode As Integer
      Dim Desc As IntPtr
      Dim DescUnicode As Integer
      Dim FileSize As Integer
      Dim MIMEType As IntPtr
      Dim CreateDate As IntPtr
      Dim ModDate As IntPtr
      Dim CheckSum As IntPtr
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFFontObj_I
      Dim Ascent As Single         ' Ascent
      Dim BaseFont As IntPtr       ' PostScript Name or Family Name
      Dim CapHeight As Single      ' Cap height
      Dim Descent As Single        ' Descent
      Dim Encoding As IntPtr       ' Unicode mapping 0..255 -> not set if a CID font is selected
      Dim FirstChar As Integer     ' First char
      Dim Flags As Integer         ' Font flags -> font descriptor
      Dim FontFamily As IntPtr     ' Optional Font Family (Family Name)
      Dim FontFamilyUni As Integer ' Is FontFamily in Unicode format?
      Dim FontName As IntPtr       ' Font name -> font descriptor
      Dim FontType As TFontType    ' If ftType0 the font is a CID font. The Encoding is not set in this case.
      Dim ItalicAngle As Single    ' Italic angle
      Dim LastChar As Integer      ' Last char
      Dim SpaceWidth As Single     ' Space width in font units. A default value is set if the font contains no space character.
      Dim Widths As IntPtr         ' Glyph widths -> 0..WidthsCount -1
      Dim WidthsCount As Integer   ' Number of widths in the array
      Dim XHeight As Single        ' x-height
      Dim DefWidth As Single       ' Default character widths -> CID fonts only
      Dim FontFile As IntPtr       ' Font file buffer -> only imported fonts are returned.
      Dim Length1 As Integer       ' Length of the clear text portion of the Type1 font, or the length of the entire font program if FontType != ffType1.
      Dim Length2 As Integer       ' Length of the encrypted portion of the Type1 font program (Type1 fonts only).
      Dim Length3 As Integer       ' Length of the fixed-content portion of the Type1 font program or zero if not present.
      Dim FontFileType As CPDF.TFontFileSubtype  ' See PDF Reference for further information.
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFNamedDest_I
      Dim NameA As IntPtr
      Dim NameW As IntPtr
      Dim NameLen As Integer
      Dim DestPage As Integer
      Dim DestPos As TPDFRect
      Dim DestType As TDestType
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFPageLabel_I
      Dim StartRange As Integer
      Dim Format As CPDF.TPageLabelFormat
      Dim FirstPageNum As Integer
      Dim Prefix As IntPtr
      Dim PrefixLen As Integer
      Dim PrefixUni As Integer
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFPrintSettings_I
      Dim DuplexMode As CPDF.TDuplexMode
      Dim NumCopies As Integer               ' -1 means not set. Values larger than 5 are ignored in viewer applications.
      Dim PickTrayByPDFSize As Integer       ' -1 means not set. 0 == false, 1 == true.
      Dim PrintRanges As IntPtr              ' If set, the array contains PrintRangesCount * 2 values. Each pair consists
      ' of the first and last pages in the sub-range. The first page in the PDF file
      ' is denoted by 0.
      Dim PrintRangesCount As Integer        ' Number of ranges available in PrintRanges.
      Dim PrintScaling As CPDF.TPrintScaling ' dpmNone means not set.
      ' Reserved fields for future extensions
      Dim Reserved0 As Integer
      Dim Reserved1 As Integer
      Dim Reserved2 As Integer
      Dim Reserved3 As Integer
      Dim Reserved4 As Integer
      Dim Reserved5 As Integer
      Dim Reserved6 As Integer
      Dim Reserved7 As Integer
      Dim Reserved8 As Integer
      Dim Reserved9 As Integer
   End Structure

   <StructLayout(LayoutKind.Sequential, Pack:=0)> _
   Private Structure TPDFSigParms_I
      Dim StructSize As Integer     ' Must be set to sizeof(TSignParms)
      Dim PKCS7ObjLen As Integer    ' The maximum length of the signed PKCS#7 object
      Dim HashType As THashType     ' If set to htDetached, the bytes ranges of the PDF file will be returned.
      Dim Range1 As IntPtr          ' Out -> Contains either the hash or the first byte range to create a detached signature
      Dim Range1Len As Integer      ' Out -> Length of the buffer
      Dim Range2 As IntPtr          ' Out -> Set only if HashType == htDetached
      Dim Range2Len As Integer      ' Out -> Length of the buffer
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim ContactInfoA As String    ' Optional, e.g. an email address
      <MarshalAs(UnmanagedType.LPWStr)> _
      Dim ContactInfoW As String    ' Optional, e.g. an email address
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim LocationA As String       ' Optional location of the signer
      <MarshalAs(UnmanagedType.LPWStr)> _
      Dim LocationW As String       ' Optional location of the signer
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim ReasonA As String         ' Optional reason why the file was signed
      <MarshalAs(UnmanagedType.LPWStr)> _
      Dim ReasonW As String         ' Optional reason why the file was signed
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim SignerA As String         ' Optional, the issuer of the certificate takes precedence
      <MarshalAs(UnmanagedType.LPWStr)> _
      Dim SignerW As String         ' Optional, the issuer of the certificate takes precedence
      Dim Encrypt As Integer        ' If true, the file will be encrypted
      ' These members will be ignored if Encrypt is set to false
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim OpenPwd As String         ' Open password
      <MarshalAs(UnmanagedType.LPStr)> _
      Dim OwnerPwd As String        ' Owner password to change the security settings
      Dim KeyLen As TKeyLen         ' Key length to be used to encrypt the file
      Dim Restrict As TRestrictions ' What should be restricted?
   End Structure

   ' This is a special version of pdfCopyMem to enable copying of the TPDFStack structure.
   Private Declare Function pdfCopyStack Lib "dynapdf.dll" Alias "pdfCopyMem" (ByVal Source As IntPtr, ByRef Dest As TPDFStack, ByVal Len As Integer) As Integer

   Private Declare Ansi Function fntBuildFamilyNameAndStyle Lib "dynapdf.dll" (ByVal IFont As IntPtr, ByVal Name As System.Text.StringBuilder, ByRef Style As TFStyle) As Integer
   Private Declare Function fntGetFont Lib "dynapdf.dll" (ByVal IFont As IntPtr, ByRef Font As TPDFFontObj_I) As Integer
   Private Declare Function fntGetSpaceWidth Lib "dynapdf.dll" (ByVal IFont As IntPtr, ByVal FontSize As Double) As Double
   Private Declare Function fntGetTextWidth Lib "dynapdf.dll" (ByVal IFont As IntPtr, ByVal Text As IntPtr, ByVal Len As Integer, ByVal CharSpacing As Single, ByVal WordSpacing As Single, ByVal TextScale As Single) As Double
   Private Declare Function fntTranslateRawCode Lib "dynapdf.dll" (ByVal IFont As IntPtr, ByVal Text As IntPtr, ByVal Len As Integer, ByRef Width As Double, ByVal OutText() As Byte, ByRef OutLen As Integer, ByRef Decoded As Integer, ByVal CharSpacing As Single, ByVal WordSpacing As Single, ByVal TextScale As Single) As Integer
   Private Declare Function fntTranslateString Lib "dynapdf.dll" (ByRef Stack As TPDFStack, ByVal OutText() As Byte, ByVal Size As Integer, ByVal Flags As Integer) As Integer
   Private Declare Function fntTranslateString2 Lib "dynapdf.dll" (ByVal IFont As IntPtr, ByVal Text As IntPtr, ByVal Len As Integer, ByVal OutText() As Byte, ByVal Size As Integer, ByVal Flags As Integer) As Integer

   Private Declare Function pdfAddActionToObj Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ObjType As Integer, ByVal AEvent As Integer, ByVal ActHandle As Integer, ByVal ObjHandle As Integer) As Integer
   Private Declare Function pdfAddArticle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Ansi Function pdfAddBookmarkA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Title As String, ByVal Parent As Integer, ByVal DestPage As Integer, ByVal DoOpen As Integer) As Integer
   Private Declare Unicode Function pdfAddBookmarkW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Title As String, ByVal Parent As Integer, ByVal DestPage As Integer, ByVal DoOpen As Integer) As Integer
   Private Declare Ansi Function pdfAddBookmarkExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Title As String, ByVal Parent As Integer, ByVal NamedDest As Integer, ByVal DoOpen As Integer) As Integer
   Private Declare Unicode Function pdfAddBookmarkExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Title As String, ByVal Parent As Integer, ByVal NamedDest As Integer, ByVal DoOpen As Integer) As Integer
   Private Declare Ansi Function pdfAddButtonImage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal BtnHandle As Integer, ByVal State As Integer, ByVal Caption As String, ByVal ImgFile As String) As Integer
   Private Declare Function pdfAddButtonImageEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal BtnHandle As Integer, ByVal State As Integer, ByVal Caption As String, ByVal hBitmap As IntPtr) As Integer
   Private Declare Ansi Function pdfAddContinueTextA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String) As Integer
   Private Declare Unicode Function pdfAddContinueTextW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String) As Integer
   Private Declare Ansi Function pdfAddDeviceNProcessColorants Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DeviceNCS As Integer, ByVal Colorants() As String, ByVal NumColorants As Integer, ByVal ProcessCS As Integer, ByVal Handle As Integer) As Integer
   Private Declare Ansi Function pdfAddDeviceNSeparations Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DeviceNCS As Integer, ByVal Colorants() As String, ByVal SeparationCS() As Integer, ByVal NumColorants As Integer) As Integer
   Private Declare Function pdfAddFieldToFormAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Action As Integer, ByVal Field As Integer, ByVal DoInclude As Integer) As Integer
   Private Declare Function pdfAddFieldToHideAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal HideAct As Integer, ByVal Field As Integer) As Integer
   Private Declare Ansi Function pdfAddFileCommentA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String) As Integer
   Private Declare Unicode Function pdfAddFileCommentW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String) As Integer
   Private Declare Ansi Function pdfAddFontSearchPathA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal APath As String, ByVal Recursive As Integer) As Integer
   Private Declare Unicode Function pdfAddFontSearchPathW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal APath As String, ByVal Recursive As Integer) As Integer
   Private Declare Ansi Function pdfAddImage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Filter As Integer, ByVal Flags As Integer, ByRef Image As TPDFImage) As Integer
   Private Declare Ansi Function pdfAddJavaScriptA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Script As String) As Integer
   Private Declare Unicode Function pdfAddJavaScriptW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, <MarshalAs(UnmanagedType.LPStr)> ByVal Name As String, ByVal Script As String) As Integer
   Private Declare Function pdfAddObjectToLayer Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OCG As Integer, ByVal ObjType As TOCObject, ByVal Handle As Integer) As Integer
   Private Declare Ansi Function pdfAddPageLabelA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal StartRange As Integer, ByVal Format As TPageLabelFormat, ByVal Prefix As String, ByVal AddNum As Integer) As Integer
   Private Declare Unicode Function pdfAddPageLabelW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal StartRange As Integer, ByVal Format As TPageLabelFormat, ByVal Prefix As String, ByVal AddNum As Integer) As Integer
   Private Declare Ansi Function pdfAddRenderingIntentA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ICCFile As String) As Integer
   Private Declare Unicode Function pdfAddRenderingIntentW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ICCFile As String) As Integer
   Private Declare Function pdfAddRenderingIntentEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal BufSize As Integer) As Integer
   Private Declare Ansi Function pdfAddValToChoiceFieldA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal ExpValue As String, ByVal Value As String, ByVal Selected As Integer) As Integer
   Private Declare Unicode Function pdfAddValToChoiceFieldW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, <MarshalAs(UnmanagedType.LPStr)> ByVal ExpValue As String, ByVal Value As String, ByVal Selected As Integer) As Integer
   Private Declare Function pdfAppend Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfApplyPattern Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PattHandle As Integer, ByVal ColorMode As Integer, ByVal Color As Integer) As Integer
   Private Declare Function pdfApplyShading Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ShadHandle As Integer) As Integer
   Private Declare Ansi Function pdfAttachFileA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FilePath As String, ByVal Description As String, ByVal Compress As Integer) As Integer
   Private Declare Unicode Function pdfAttachFileW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FilePath As String, ByVal Description As String, ByVal Compress As Integer) As Integer
   Private Declare Ansi Function pdfAttachFileExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal BufSize As Integer, ByVal FileName As String, ByVal Description As String, ByVal Compress As Integer) As Integer
   Private Declare Unicode Function pdfAttachFileExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal BufSize As Integer, ByVal FileName As String, ByVal Description As String, ByVal Compress As Integer) As Integer
   Private Declare Function pdfAutoTemplate Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Templ As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfBeginClipPath Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfBeginContinueText Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double) As Integer
   Private Declare Function pdfBeginLayer Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OCG As Integer) As Integer
   Private Declare Function pdfBeginPattern Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PatternType As Integer, ByVal TilingType As Integer, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfBeginTemplate Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfBezier_1_2_3 Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal x1 As Double, ByVal y1 As Double, ByVal x2 As Double, ByVal y2 As Double, ByVal x3 As Double, ByVal y3 As Double) As Integer
   Private Declare Function pdfBezier_1_3 Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal x1 As Double, ByVal y1 As Double, ByVal x3 As Double, ByVal y3 As Double) As Integer
   Private Declare Function pdfBezier_2_3 Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal x2 As Double, ByVal y2 As Double, ByVal x3 As Double, ByVal y3 As Double) As Integer
   Private Declare Function pdfCalcWidthHeight Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OrgWidth As Double, ByVal OrgHeight As Double, ByVal ScaledWidth As Double, ByVal ScaledHeight As Double) As Double
   Private Declare Ansi Function pdfChangeAnnotNameA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal Name As String) As Integer
   Private Declare Unicode Function pdfChangeAnnotNameW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal Name As String) As Integer
   Private Declare Function pdfChangeAnnotPos Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Ansi Function pdfChangeBookmarkA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ABmk As Integer, ByVal ATitle As String, ByVal Parent As Integer, ByVal DestPage As Integer, ByVal DoOpen As Integer) As Integer
   Private Declare Unicode Function pdfChangeBookmarkW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ABmk As Integer, ByVal ATitle As String, ByVal Parent As Integer, ByVal DestPage As Integer, ByVal DoOpen As Integer) As Integer
   Private Declare Function pdfChangeFont Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer) As Integer
   Private Declare Function pdfChangeFontSize Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Size As Double) As Integer
   Private Declare Function pdfChangeFontStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Style As Integer) As Integer
   Private Declare Function pdfChangeFontStyleEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Style As Integer) As Integer
   Private Declare Ansi Function pdfChangeJavaScriptA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByVal NewScript As String) As Integer
   Private Declare Unicode Function pdfChangeJavaScriptW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByVal NewScript As String) As Integer
   Private Declare Ansi Function pdfChangeJavaScriptActionA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByVal NewScript As String) As Integer
   Private Declare Unicode Function pdfChangeJavaScriptActionW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByVal NewScript As String) As Integer
   Private Declare Ansi Function pdfChangeJavaScriptNameA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByVal Name As String) As Integer
   Private Declare Unicode Function pdfChangeJavaScriptNameW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByVal Name As String) As Integer
   Private Declare Ansi Function pdfChangeLinkAnnot Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal URL As String) As Integer
   Private Declare Function pdfCheckFieldNames Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfCheckCollection Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfCheckConformance Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ConfType As Integer, ByVal Options As Integer, ByVal UserData As IntPtr, ByVal OnFontNotFound As TOnFontNotFoundProc, ByVal OnReplaceICCProfile As TOnReplaceICCProfile) As Integer
   Private Declare Function pdfClearAutoTemplates Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfClearHostFonts Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfClipPath Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ClipMode As Integer, ByVal FillMode As Integer) As Integer
   Private Declare Ansi Function pdfCloseAndSignFile Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal CertFile As String, ByVal Password As String, ByVal Reason As String, ByVal Location As String) As Integer
   Private Declare Ansi Function pdfCloseAndSignFileEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OpenPwd As String, ByVal OwnerPwd As String, ByVal KeyLen As Integer, ByVal Restrict As Integer, ByVal CertFile As String, ByVal Password As String, ByVal Reason As String, ByVal Location As String) As Integer
   Private Declare Ansi Function pdfCloseAndSignFileExt Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef SigParms As TPDFSigParms_I) As Integer
   Private Declare Function pdfCloseFile Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfCloseFileEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OpenPwd As String, ByVal OwnerPwd As String, ByVal KeyLen As Integer, ByVal Restrict As Integer) As Integer
   Private Declare Ansi Function pdfCloseImage Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfCloseImportFile Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfClosePath Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FillMode As Integer) As Integer
   Private Declare Ansi Function pdfCloseTag Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfComputeBBox Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef BBox As TPDFRect, ByVal Flags As TCompBBoxFlags) As Integer
   Private Declare Ansi Function pdfConvColor Lib "dynapdf.dll" (ByVal Color As IntPtr, ByVal NumComps As Integer, ByVal SourceCS As Integer, ByVal IColorSpace As IntPtr, ByVal DestCS As Integer) As Integer
   Private Declare Ansi Function pdfConvertColors Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Flags As Integer, ByVal Reserved As IntPtr) As Integer
   Private Declare Ansi Function pdfConvToUnicode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AString As String, ByVal CP As Integer) As IntPtr
   Private Declare Ansi Function pdfCreate3DAnnot Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Name As String, ByVal U3DFile As String, ByVal Image As String) As Integer
   Private Declare Ansi Function pdfCreate3DBackground Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal IView As IntPtr, ByVal BackColor As Integer) As Integer
   Private Declare Ansi Function pdfCreate3DGotoViewAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Base3DAnnot As Integer, ByVal IView As IntPtr, ByVal Named As Integer) As Integer
   Private Declare Ansi Function pdfCreate3DProjection Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal IView As IntPtr, ByVal ProjType As Integer, ByVal ScaleType As Integer, ByVal Diameter As Double, ByVal FOV As Double) As Integer
   Private Declare Ansi Function pdfCreate3DView Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Base3DAnnot As Integer, ByVal Name As String, ByVal SetAsDefault As Integer, ByVal Matrix() As Double, ByVal CamDistance As Double, ByVal RM As Integer, ByVal LS As Integer) As IntPtr
   Private Declare Ansi Function pdfCreateAnnotAP Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Annot As Integer) As Integer
   Private Declare Ansi Function pdfCreateArticleThreadA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ThreadName As String) As Integer
   Private Declare Unicode Function pdfCreateArticleThreadW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ThreadName As String) As Integer
   Private Declare Function pdfCreateAxialShading Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal sx As Double, ByVal sy As Double, ByVal eX As Double, ByVal eY As Double, ByVal SCenter As Double, ByVal SColor As Integer, ByVal EColor As Integer, ByVal Extend1 As Integer, ByVal Extend2 As Integer) As Integer
   Private Declare Ansi Function pdfCreateButtonA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Caption As String, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Unicode Function pdfCreateButtonW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, <MarshalAs(UnmanagedType.LPStr)> ByVal Name As String, ByVal Caption As String, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Ansi Function pdfCreateCheckBox Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal ExpValue As String, ByVal Checked As Integer, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfCreateCIEColorSpace Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Base As Integer, ByRef WhitePoint() As Single, ByRef BlackPoint() As Single, ByRef Gamma() As Single, ByRef Matrix() As Single) As Integer
   Private Declare Function pdfCreateCollection Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal View As Integer) As Integer
   Private Declare Ansi Function pdfCreateCollectionFieldA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ColType As Integer, ByVal Column As Integer, ByVal Name As String, ByVal Key As String, ByVal Visible As Integer, ByVal Editable As Integer) As Integer
   Private Declare Unicode Function pdfCreateCollectionFieldW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ColType As Integer, ByVal Column As Integer, ByVal Name As String, <MarshalAs(UnmanagedType.LPStr)> ByVal Key As String, ByVal Visible As Integer, ByVal Editable As Integer) As Integer
   Private Declare Function pdfCreateColItemDate Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal EmbFile As Integer, ByVal Key As String, ByVal DateVal As Integer, ByVal Prefix As String) As Integer
   Private Declare Function pdfCreateColItemNumber Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal EmbFile As Integer, ByVal Key As String, ByVal Value As Double, ByVal Prefix As String) As Integer
   Private Declare Ansi Function pdfCreateColItemStringA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal EmbFile As Integer, ByVal Key As String, ByVal Value As String, ByVal Prefix As String) As Integer
   Private Declare Unicode Function pdfCreateColItemStringW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal EmbFile As Integer, <MarshalAs(UnmanagedType.LPStr)> ByVal Key As String, ByVal Value As String, ByVal Prefix As String) As Integer
   Private Declare Ansi Function pdfCreateComboBox Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Sort As Integer, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Ansi Function pdfCreateDeviceNColorSpace Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Colorants() As String, ByVal NumColorants As Integer, ByVal PostScriptFunc As String, ByVal Alternate As Integer, ByVal Handle As Integer) As Integer
   Private Declare Function pdfCreateExtGState Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef GS As TPDFExtGState) As Integer
   Private Declare Function pdfCreateGoToAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DestType As Integer, ByVal PageNum As Integer, ByVal a As Double, ByVal b As Double, ByVal c As Double, ByVal d As Double) As Integer
   Private Declare Function pdfCreateGoToActionEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal NamedDest As Integer) As Integer
   Private Declare Ansi Function pdfCreateGoToRAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal PageNum As Integer) As Integer
   Private Declare Ansi Function pdfCreateGoToRActionExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal DestName As String, ByVal NewWindow As Integer) As Integer
   Private Declare Unicode Function pdfCreateGoToRActionExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, <MarshalAs(UnmanagedType.LPStr)> ByVal FileName As String, ByVal DestName As String, ByVal NewWindow As Integer) As Integer
   Private Declare Ansi Function pdfCreateGroupField Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Parent As Integer) As Integer
   Private Declare Function pdfCreateHideAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Hide As Integer) As Integer
   Private Declare Ansi Function pdfCreateICCBasedColorSpaceA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ICCProfile As String) As Integer
   Private Declare Unicode Function pdfCreateICCBasedColorSpaceW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ICCProfile As String) As Integer
   Private Declare Ansi Function pdfCreateImageA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal Format As Integer) As Integer
   Private Declare Unicode Function pdfCreateImageW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal Format As Integer) As Integer
   Private Declare Ansi Function pdfCreateImpDataActionA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DataFile As String) As Integer
   Private Declare Unicode Function pdfCreateImpDataActionW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DataFile As String) As Integer
   Private Declare Function pdfCreateIndexedColorSpace Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Base As Integer, ByVal Handle As Integer, ByRef ColorTable() As Byte, ByVal NumColors As Integer) As Integer
   Private Declare Ansi Function pdfCreateJSActionA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Script As String) As Integer
   Private Declare Unicode Function pdfCreateJSActionW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Script As String) As Integer
   Private Declare Ansi Function pdfCreateLaunchAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OP As Integer, ByVal FileName As String, ByVal DefDir As String, ByVal Param As String, ByVal NewWindow As Integer) As Integer
   Private Declare Ansi Function pdfCreateListBox Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Sort As Integer, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfCreateNamedAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Action As Integer) As Integer
   Private Declare Ansi Function pdfCreateNamedDestA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal DestPage As Integer, ByVal DestType As TDestType, ByVal a As Double, ByVal b As Double, ByVal c As Double, ByVal d As Double) As Integer
   Private Declare Unicode Function pdfCreateNamedDestW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal DestPage As Integer, ByVal DestType As TDestType, ByVal a As Double, ByVal b As Double, ByVal c As Double, ByVal d As Double) As Integer
   Private Declare Ansi Function pdfCreateNewPDFA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OutPDF As String) As Integer
   Private Declare Unicode Function pdfCreateNewPDFW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OutPDF As String) As Integer
   Private Declare Ansi Function pdfCreateOCGA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal DisplayInUI As Integer, ByVal Visible As Integer, ByVal Intent As TOCGIntent) As Integer
   Private Declare Unicode Function pdfCreateOCGW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal DisplayInUI As Integer, ByVal Visible As Integer, ByVal Intent As TOCGIntent) As Integer
   Private Declare Function pdfCreateOCMD Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Visibility As TOCVisibility, ByVal OCGs() As Integer, ByVal Count As Integer) As Integer
   Private Declare Function pdfCreateRadialShading Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal sx As Double, ByVal sy As Double, ByVal R1 As Double, ByVal eX As Double, ByVal eY As Double, ByVal R2 As Double, ByVal SCenter As Double, ByVal SColor As Integer, ByVal EColor As Integer, ByVal Extend1 As Integer, ByVal Extend2 As Integer) As Integer
   Private Declare Ansi Function pdfCreateRadioButton Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal ExpValue As String, ByVal Checked As Integer, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfCreateResetAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfCreateSeparationCS Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Colorant As String, ByVal Alternate As Integer, ByVal Handle As Integer, ByVal Color As Integer) As Integer
   Private Declare Function pdfCreateSetOCGStateAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal _On() As Integer, ByVal OnCount As Integer, ByVal Off() As Integer, ByVal OffCount As Integer, ByVal Toggle() As Integer, ByVal ToggleCount As Integer, ByVal PreserveRB As Integer) As Integer
   Private Declare Ansi Function pdfCreateSigField Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Parent As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfCreateSigFieldAP Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal SigField As Integer) As Integer
   Private Declare Function pdfCreateStdPattern Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Pattern As Integer, ByVal LineWidth As Double, ByVal Distance As Double, ByVal LineColor As Integer, ByVal BackColor As Integer) As Integer
   Private Declare Function pdfCreateStructureTree Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfCreateSubmitAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Flags As Integer, ByVal URL As String) As Integer
   Private Declare Ansi Function pdfCreateTextField Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Parent As Integer, ByVal Multiline As Integer, ByVal MaxLen As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfCreateURIAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal URL As String) As Integer
   Private Declare Ansi Function pdfDecryptPDFA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal PwdType As Integer, ByVal Password As String) As Integer
   Private Declare Unicode Function pdfDecryptPDFW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal PwdType As Integer, <MarshalAs(UnmanagedType.LPStr)> ByVal Password As String) As Integer
   Private Declare Sub pdfDeleteAcroForm Lib "dynapdf.dll" (ByVal IPDF As IntPtr)
   Private Declare Function pdfDeleteActionFromObj Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ObjType As Integer, ByVal ActHandle As Integer, ByVal ObjHandle As Integer) As Integer
   Private Declare Function pdfDeleteActionFromObjEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ObjType As Integer, ByVal ObjHandle As Integer, ByVal ActIndex As Integer) As Integer
   Private Declare Function pdfDeleteAnnotation Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Integer
   Private Declare Function pdfDeleteBookmark Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ABmk As Integer) As Integer
   Private Declare Function pdfDeleteEmbeddedFile Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Integer
   Private Declare Function pdfDeleteField Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Ansi Function pdfDeleteFieldEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String) As Integer
   Private Declare Sub pdfDeleteJavaScripts Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DelJavaScriptActions As Integer)
   Private Declare Function pdfDeletePage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PageNum As Integer) As Integer
   Private Declare Function pdfDeletePDF Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Sub pdfDeletePageLabels Lib "dynapdf.dll" (ByVal IPDF As IntPtr)
   Private Declare Function pdfDeleteSeparationInfo Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AllPages As Integer) As Integer
   Private Declare Function pdfDeleteTemplate Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Integer
   Private Declare Function pdfDeleteTemplateEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer) As Integer
   Private Declare Sub pdfDeleteXFAForm Lib "dynapdf.dll" (ByVal IPDF As IntPtr)
   Private Declare Function pdfDrawArc Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Radius As Double, ByVal StartAngle As Double, ByVal EndAngle As Double) As Integer
   Private Declare Function pdfDrawArcEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal StartAngle As Double, ByVal EndAngle As Double) As Integer
   Private Declare Function pdfDrawChord Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal StartAngle As Double, ByVal EndAngle As Double, ByVal FillMode As Integer) As Integer
   Private Declare Function pdfDrawCircle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Radius As Double, ByVal FillMode As Integer) As Integer
   Private Declare Function pdfDrawPie Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal StartAngle As Double, ByVal EndAngle As Double, ByVal FillMode As Integer) As Integer
   Private Declare Function pdfEditPage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PageNum As Integer) As Integer
   Private Declare Function pdfEditTemplate Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer) As Integer
   Private Declare Function pdfEditTemplate2 Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Integer
   Private Declare Function pdfEllipse Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal FillMode As Integer) As Integer
   Private Declare Ansi Function pdfEncryptPDFA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal OpenPwd As String, ByVal OwnerPwd As String, ByVal KeyLen As Integer, ByVal Restrict As Integer) As Integer
   Private Declare Unicode Function pdfEncryptPDFW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, <MarshalAs(UnmanagedType.LPStr)> ByVal OpenPwd As String, <MarshalAs(UnmanagedType.LPStr)> ByVal OwnerPwd As String, ByVal KeyLen As Integer, ByVal Restrict As Integer) As Integer
   Private Declare Function pdfEndContinueText Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfEndLayer Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfEndPage Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfEndPattern Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfEndTemplate Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfEnumDocFonts Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Data As IntPtr, ByVal EnumProc As TEnumFontsProc2) As Integer
   Private Declare Function pdfEnumHostFonts Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Data As IntPtr, ByVal EnumProc As TEnumFontsProc) As Integer
   Private Declare Function pdfEnumHostFontsEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Data As IntPtr, ByVal EnumProc As TEnumFontsProcEx) As Integer
   Private Declare Function pdfExchangeBookmarks Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Bmk1 As Integer, ByVal Bmk2 As Integer) As Integer
   Private Declare Function pdfExchangePages Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal First As Integer, ByVal Second As Integer) As Integer
   Private Declare Ansi Function pdfFileAttachAnnotA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Icon As Integer, ByVal Author As String, ByVal Desc As String, ByVal AFile As String, ByVal Compress As Integer) As Integer
   Private Declare Unicode Function pdfFileAttachAnnotW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Icon As Integer, ByVal Author As String, ByVal Desc As String, ByVal AFile As String, ByVal Compress As Integer) As Integer
   Private Declare Ansi Function pdfFileLinkA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal AFilePath As String) As Integer
   Private Declare Unicode Function pdfFileLinkW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal AFilePath As String) As Integer
   Private Declare Ansi Function pdfFindBookmarkA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DestPage As Integer, ByVal Title As String) As Integer
   Private Declare Unicode Function pdfFindBookmarkW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DestPage As Integer, ByVal Title As String) As Integer
   Private Declare Ansi Function pdfFindFieldA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String) As Integer
   Private Declare Unicode Function pdfFindFieldW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String) As Integer
   Private Declare Ansi Function pdfFindLinkAnnot Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal URL As String) As Integer
   Private Declare Function pdfFindNextBookmark Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfFinishSignature Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PKCS7Obj() As Byte, ByVal Length As Integer) As Integer
   Private Declare Function pdfFlattenAnnots Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Flags As TAnnotFlattenFlags) As Integer
   Private Declare Function pdfFlattenForm Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfFlushPageContent Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Stack As TPDFStack) As Integer
   Private Declare Sub pdfFreeImageBuffer Lib "dynapdf.dll" (ByVal IPDF As IntPtr)
   Private Declare Function pdfFreePDF Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfFreeTextAnnotA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal AText As String, ByVal Align As Integer) As Integer
   Private Declare Unicode Function pdfFreeTextAnnotW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal AText As String, ByVal Align As Integer) As Integer
   Private Declare Function pdfFreeUniBuf Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetActionCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetActionType Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ActHandle As Integer) As Integer
   Private Declare Function pdfGetActionTypeEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ObjType As Integer, ByVal ObjHandle As Integer, ByVal ActIndex As Integer) As Integer
   Private Declare Function pdfGetActiveFont Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetAllocBy Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetAnnot Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByRef Annot As TPDFAnnotation_I) As Integer
   Private Declare Function pdfGetAnnotBBox Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByRef BBox As TPDFRect) As Integer
   Private Declare Function pdfGetAnnotCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetAnnotEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByRef Annot As TPDFAnnotationEx_I) As Integer
   Private Declare Function pdfGetAnnotFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetAnnotLink Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As IntPtr
   Private Declare Function pdfGetAnnotType Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Integer
   Private Declare Function pdfGetAscent Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetBBox Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Boundary As Integer, ByRef BBox As TPDFRect) As Integer
   Private Declare Function pdfGetBidiMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetBookmark Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByRef Bmk As TBookmark_I) As Integer
   Private Declare Function pdfGetBookmarkCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetBorderStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetBuffer Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef BufSize As Integer) As IntPtr
   Private Declare Function pdfGetCapHeight Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetCharacterSpacing Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetCheckBoxChar Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetCheckBoxCharEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetCheckBoxDefState Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetCMap Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer, ByRef CMap As TPDFCMap) As Integer
   Private Declare Function pdfGetCMapCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetColorSpace Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetColorSpaceCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetColorSpaceObj Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByRef cs As TPDFColorSpaceObj_I) As Integer
   Private Declare Function pdfGetColorSpaceObjEx Lib "dynapdf.dll" (ByVal IColorSpace As IntPtr, ByRef cs As TPDFColorSpaceObj_I) As Integer
   Private Declare Function pdfGetCompressionFilter Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetCompressionLevel Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetContent Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Buffer As IntPtr) As Integer
   Private Declare Function pdfGetDefBitsPerPixel Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetDescent Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetDeviceNAttributes Lib "dynapdf.dll" (ByVal IAttributes As IntPtr, ByRef Attributes As TDeviceNAttributes_I) As Integer
   Private Declare Function pdfGetDocInfo Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DInfo As Integer, ByRef Value As IntPtr) As Integer
   Private Declare Function pdfGetDocInfoCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetDocInfoEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer, ByRef DInfo As Integer, ByRef Key As IntPtr, ByRef Value As IntPtr, ByRef IsUnicode As Integer) As Integer
   Private Declare Function pdfGetDrawDirection Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetDynaPDFVersion Lib "dynapdf.dll" () As IntPtr
   Private Declare Function pdfGetEmbeddedFile Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByRef FileSpec As TPDFFileSpec_I, ByVal Decompress As Integer) As Integer
   Private Declare Function pdfGetEmbeddedFileCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetEMFPatternDistance Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetErrorMessage Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As IntPtr
   Private Declare Function pdfGetErrorMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetField Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByRef Field As TPDFField_I) As Integer
   Private Declare Function pdfGetFieldBackColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetFieldBorderColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetFieldBorderStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetFieldBorderWidth Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Double
   Private Declare Function pdfGetFieldChoiceValue Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal ValIndex As Integer, ByRef Value As TPDFChoiceValue_I) As Integer
   Private Declare Function pdfGetFieldColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal ColorType As Integer, ByRef ColorSpace As Integer, ByRef Color As Integer) As Integer
   Private Declare Function pdfGetFieldCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetFieldEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByRef Field As TPDFFieldEx_I) As Integer
   Private Declare Function pdfGetFieldEx2 Lib "dynapdf.dll" (ByVal IField As IntPtr, ByRef Field As TPDFFieldEx_I) As Integer
   Private Declare Function pdfGetFieldExpValCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetFieldExpValue Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByRef Value As IntPtr) As Integer
   Private Declare Function pdfGetFieldExpValueEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal ValIndex As Integer, ByRef Value As IntPtr, ByRef ExpValue As IntPtr, ByRef Selected As Integer) As Integer
   Private Declare Function pdfGetFieldFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetFieldGroupType Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetFieldHighlightMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetFieldIndex Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetFieldMapName Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByRef Value As IntPtr, ByRef bUnicode As Integer) As Integer
   Private Declare Function pdfGetFieldName Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByRef Name As IntPtr) As Integer
   Private Declare Function pdfGetFieldOrientation Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetFieldTextAlign Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetFieldTextColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetFieldToolTip Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByRef Value As IntPtr, ByRef bUnicode As Integer) As Integer
   Private Declare Function pdfGetFieldType Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer) As Integer
   Private Declare Function pdfGetFillColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetFontCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetFontEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByRef Font As TPDFFontObj_I) As Integer
   Private Declare Function pdfGetFontOrigin Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Sub pdfGetFontSearchOrder Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Order() As TFontBaseType)
   Private Declare Function pdfGetFontSelMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetFontWeight Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfGetFTextHeightA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Align As Integer, ByVal AText As String) As Double
   Private Declare Unicode Function pdfGetFTextHeightW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Align As Integer, ByVal AText As String) As Double
   Private Declare Ansi Function pdfGetFTextHeightExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Width As Double, ByVal Align As Integer, ByVal AText As String) As Double
   Private Declare Unicode Function pdfGetFTextHeightExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Width As Double, ByVal Align As Integer, ByVal AText As String) As Double
   Private Declare Function pdfGetGStateFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetIconColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetImageBuffer Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef BufSize As Integer) As IntPtr
   Private Declare Function pdfGetImageCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String) As Integer
   Private Declare Function pdfGetImageHeight Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer) As Integer
   Private Declare Function pdfGetImageWidth Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer) As Integer
   Private Declare Function pdfGetImportFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInBBox Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PageNum As Integer, ByVal Boundary As Integer, ByRef BBox As TPDFRect) As Integer
   Private Declare Function pdfGetInDocInfo Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DInfo As Integer, ByRef Value As IntPtr) As Integer
   Private Declare Function pdfGetInDocInfoCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInDocInfoEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer, ByRef DInfo As Integer, ByRef Key As IntPtr, ByRef Value As IntPtr, ByRef IsUnicode As Integer) As Integer
   Private Declare Function pdfGetInEncryptionFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInIsCollection Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInIsEncrypted Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInIsSigned Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInIsTrapped Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInIsXFAForm Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInNamedDest Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer, ByRef Dest As TPDFNamedDest_I) As Integer
   Private Declare Function pdfGetInNamedDestCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInOrientation Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PageNum As Integer) As Integer
   Private Declare Function pdfGetInPrintSettings Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Settings As TPDFPrintSettings_I) As Integer
   Private Declare Function pdfGetInPageCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetInPDFVersion Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetIsFixedPitch Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetItalicAngle Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetJavaScript Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByRef Len As Integer, ByRef bUnicode As Integer) As IntPtr
   Private Declare Function pdfGetJavaScriptAction Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AHandle As Integer, ByRef Len As Integer, ByRef bUnicode As Integer) As IntPtr
   Private Declare Function pdfGetJavaScriptAction2 Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ObjType As Integer, ByVal ObjHandle As Integer, ByVal ActIndex As Integer, ByRef Len As Integer, ByRef bUnicode As Integer, ByRef ObjEvent As TObjEvent) As IntPtr
   Private Declare Function pdfGetJavaScriptCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetJavaScriptEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByRef Len As Integer, ByRef bUnicode As Integer) As IntPtr
   Private Declare Function pdfGetJavaScriptName Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByRef NameLen As Integer, ByRef IsUnicode As Integer) As IntPtr
   Private Declare Function pdfGetJPEGQuality Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetLanguage Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As IntPtr
   Private Declare Function pdfGetLastTextPosX Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetLastTextPosY Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetLeading Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetLineCapStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetLineJoinStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetLineWidth Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetLinkHighlightMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetLogMetafileSize Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByRef R As TRectL) As Integer
   Private Declare Function pdfGetLogMetafileSizeEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal bufSize As Integer, ByRef R As TRectL) As Integer
   Private Declare Function pdfGetMatrix Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Matrix As TCTM) As Integer
   Private Declare Function pdfGetMaxFieldLen Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal TxtField As Integer) As Integer
   Private Declare Function pdfGetMetaConvFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetMissingGlyphs Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Count As Integer) As IntPtr
   Private Declare Function pdfGetMiterLimit Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetNeedAppearance Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetObjActionCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ObjType As Integer, ByVal ObjHandle As Integer) As Integer
   Private Declare Function pdfGetOpacity Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetOrientation Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPageAnnot Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer, ByRef Annot As TPDFAnnotation_I) As Integer
   Private Declare Function pdfGetPageAnnotCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPageAnnotEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer, ByRef Annot As TPDFAnnotationEx_I) As Integer
   Private Declare Function pdfGetPageCoords Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPageCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPageField Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer, ByRef Field As TPDFField_I) As Integer
   Private Declare Function pdfGetPageFieldCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPageFieldEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer, ByRef Field As TPDFFieldEx_I) As Integer
   Private Declare Function pdfGetPageHeight Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetPageLayout Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPageMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPageNum Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPageText Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Stack As TPDFStack) As Integer
   Private Declare Function pdfGetPageWidth Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetPDFVersion Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPrintSettings Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Settings As TPDFPrintSettings_I) As Integer
   Private Declare Function pdfGetResolution Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetSaveNewImageFormat Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetPageLabel Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Index As Integer, ByRef Label As TPDFPageLabel_I) As Integer
   Private Declare Function pdfGetPageLabelCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetSeparationInfo Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Colorant As IntPtr, ByRef CS As TExtColorSpace) As Integer
   Private Declare Function pdfGetStrokeColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetTabLen Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetTemplCount Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetTemplHandle Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetTemplHeight Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Double
   Private Declare Function pdfGetTemplWidth Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Double
   Private Declare Function pdfGetTextDrawMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetTextFieldValue Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByRef Value As IntPtr, ByRef ValIsUnicode As Integer, ByRef DefValue As IntPtr, ByRef DefValIsUnicode As Integer) As Integer
   Private Declare Function pdfGetTextRect Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef PosX As Double, ByRef PosY As Double, ByRef Width As Double, ByRef Height As Double) As Integer
   Private Declare Function pdfGetTextRise Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfGetTextScaling Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Ansi Function pdfGetTextWidthA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String) As Double
   Private Declare Unicode Function pdfGetTextWidthW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String) As Double
   Private Declare Ansi Function pdfGetTextWidthExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String, ByVal Len As Integer) As Double
   Private Declare Unicode Function pdfGetTextWidthExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String, ByVal Len As Integer) As Double
   Private Declare Function pdfGetTransparentColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetTrapped Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetUseExactPwd Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetUseGlobalImpFiles Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetUserRights Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetUseStdFonts Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetUseSystemFonts Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetUseTransparency Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetUseVisibleCoords Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetViewerPrefrences Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Preference As Integer, ByRef AddVal As Integer) As Integer
   Private Declare Function pdfGetWMFDefExtent Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Width As Integer, ByRef Height As Integer) As Integer
   Private Declare Function pdfGetWMFPixelPerInch Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfGetWordSpacing Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Double
   Private Declare Function pdfHaveOpenDoc Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfHighlightAnnotA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal SubType As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Color As Integer, ByVal Author As String, ByVal Subject As String, ByVal Comment As String) As Integer
   Private Declare Unicode Function pdfHighlightAnnotW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal SubType As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Color As Integer, ByVal Author As String, ByVal Subject As String, ByVal Comment As String) As Integer
   Private Declare Function pdfImportBookmarks Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfImportCatalogObjects Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfImportDocInfo Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfImportPage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PageNum As Integer) As Integer
   Private Declare Function pdfImportPageEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PageNum As Integer, ByVal ScaleX As Double, ByVal ScaleY As Double) As Integer
   Private Declare Function pdfImportPDFFile Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DestPage As Integer, ByVal ScaleX As Double, ByVal ScaleY As Double) As Integer
   Private Declare Function pdfInitExtGState Lib "dynapdf.dll" (ByRef GS As TPDFExtGState) As Integer
   Private Declare Function pdfInitStack Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Stack As TPDFStack) As Integer
   Private Declare Function pdfInsertBMPFromBuffer Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByVal Buffer() As Byte) As Integer
   Private Declare Function pdfInsertBMPFromHandle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByVal hBitmap As IntPtr) As Integer
   Private Declare Function pdfInsertImage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByVal AFile As String) As Integer
   Private Declare Function pdfInsertImageEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByVal AFile As String, ByVal Index As Integer) As Integer
   Private Declare Function pdfInsertImageFromBuffer Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double, ByVal Buffer() As Byte, ByVal bufSize As Integer, ByVal Index As Integer) As Integer
   Private Declare Function pdfInsertMetafile Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfInsertMetafileEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal bufSize As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfInsertMetafileExt Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByRef View As TRectL, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfInsertMetafileExtEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal bufSize As Integer, ByRef View As TRectL, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfInsertMetafileFromHandle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal hEnhMetafile As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfInsertMetafileFromHandleEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal hEnhMetafile As IntPtr, ByRef View As TRectL, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfInsertRawImage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal BitsPerPixel As Integer, ByVal ColorCount As Integer, ByVal ImgWidth As Integer, ByVal ImgHeight As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double) As Integer
   Private Declare Function pdfIsBidiText Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String) As Integer
   Private Declare Function pdfIsColorPage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal GrayIsColor As Integer) As Integer
   Private Declare Function pdfIsEmptyPage Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfLineTo Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double) As Integer
   Private Declare Ansi Function pdfLoadCMap Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal CMapName As String, ByVal Embed As Integer) As Integer
   Private Declare Ansi Function pdfLoadFDFDataA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal Password As String, ByVal Flags As Integer) As Integer
   Private Declare Unicode Function pdfLoadFDFDataW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, <MarshalAs(UnmanagedType.LPStr)> ByVal Password As String, ByVal Flags As Integer) As Integer
   Private Declare Ansi Function pdfLoadFDFDataEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal BufSize As Integer, ByVal Password As String, ByVal Flags As Integer) As Integer
   Private Declare Function pdfLoadFont Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal BufSize As Integer, ByVal Style As Integer, ByVal Size As Double, ByVal Embed As Integer, ByVal CP As Integer) As Integer
   Private Declare Function pdfMovePage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Source As Integer, ByVal Dest As Integer) As Integer
   Private Declare Function pdfMoveTo Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double) As Integer
   Private Declare Function pdfNewPDF Lib "dynapdf.dll" () As IntPtr
   Private Declare Function pdfOpenImportBuffer Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal bufSize As Integer, ByVal PwdType As Integer, ByVal Password As String) As Integer
   Private Declare Ansi Function pdfOpenImportFileA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal PwdType As Integer, ByVal Password As String) As Integer
   Private Declare Unicode Function pdfOpenImportFileW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal PwdType As Integer, <MarshalAs(UnmanagedType.LPStr)> ByVal Password As String) As Integer
   Private Declare Ansi Function pdfOpenOutputFileA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OutPDF As String) As Integer
   Private Declare Unicode Function pdfOpenOutputFileW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OutPDF As String) As Integer
   Private Declare Ansi Function pdfOpenTagA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Tag As Integer, ByVal Lang As String, ByVal AltText As String, ByVal Expansion As String) As Integer
   Private Declare Unicode Function pdfOpenTagW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Tag As Integer, <MarshalAs(UnmanagedType.LPStr)> ByVal Lang As String, ByVal AltText As String, ByVal Expansion As String) As Integer
   Private Declare Function pdfPageLink Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal DestPage As Integer) As Integer
   Private Declare Function pdfPageLink2 Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal NamedDest As Integer) As Integer
   Private Declare Function pdfPageLinkEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal DestType As Integer, ByVal DestPage As Integer, ByVal a As Double, ByVal b As Double, ByVal C As Double, ByVal d As Double) As Integer
   Private Declare Function pdfParseContent Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Data As IntPtr, ByRef Stack As TPDFParseInterface, ByVal Flags As TParseFlags) As Integer
   Private Declare Function pdfPlaceImage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ImgHandle As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double) As Integer
   Private Declare Function pdfPlaceSigFieldValidateIcon Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal SigField As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfPlaceTemplate Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal TmplHandle As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double) As Integer
   Private Declare Function pdfPlaceTemplateEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal TmplHandle As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal ScaleWidth As Double, ByVal ScaleHeight As Double) As Integer
   Private Declare Function pdfReadImageFormat Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByRef Width As Integer, ByRef Height As Integer, ByRef BitsPerPixel As Integer, ByRef UseZip As Integer) As Integer
   Private Declare Function pdfReadImageFormat2 Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal Index As Integer, ByRef Width As Integer, ByRef Height As Integer, ByRef BitsPerPixel As Integer, ByRef UseZip As Integer) As Integer
   Private Declare Function pdfReadImageFormatEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal hBitmap As IntPtr, ByRef Width As Integer, ByRef Height As Integer, ByRef BitsPerPixel As Integer, ByRef UseZip As Integer) As Integer
   Private Declare Function pdfReadImageFormatFromBuffer Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal BufSize As Integer, ByVal Index As Integer, ByRef Width As Integer, ByRef Height As Integer, ByRef BitsPerPixel As Integer, ByRef UseZip As Integer) As Integer
   Private Declare Ansi Function pdfReadImageResolution Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal Index As Integer, ByRef resX As Integer, ByRef resY As Integer) As Integer
   Private Declare Function pdfReadImageResolutionEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer As Byte(), ByVal BufSize As Integer, ByVal Index As Integer, ByRef resX As Integer, ByRef resY As Integer) As Integer
   Private Declare Function pdfRectangle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal FillMode As Integer) As Integer
   Private Declare Ansi Function pdfReEncryptPDFA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal PwdType As Integer, ByVal InPwd As String, ByVal NewOpenPwd As String, ByVal NewOwnerPwd As String, ByVal NewKeyLen As Integer, ByVal Restrict As Integer) As Integer
   Private Declare Unicode Function pdfReEncryptPDFW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal FileName As String, ByVal PwdType As Integer, <MarshalAs(UnmanagedType.LPStr)> ByVal InPwd As String, <MarshalAs(UnmanagedType.LPStr)> ByVal NewOpenPwd As String, <MarshalAs(UnmanagedType.LPStr)> ByVal NewOwnerPwd As String, ByVal NewKeyLen As Integer, ByVal Restrict As Integer) As Integer
   Private Declare Function pdfRenameSpotColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Colorant As String, ByVal NewName As String) As Integer
   Private Declare Ansi Function pdfReplaceFontA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PDFFont As IntPtr, ByVal Name As String, ByVal Style As TFStyle, ByVal NameIsFamilyName As Integer) As Integer
   Private Declare Unicode Function pdfReplaceFontW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PDFFont As IntPtr, ByVal Name As String, ByVal Style As TFStyle, ByVal NameIsFamilyName As Integer) As Integer
   Private Declare Ansi Function pdfReplaceFontExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PDFFont As IntPtr, ByVal FontFile As String, ByVal Embed As Integer) As Integer
   Private Declare Unicode Function pdfReplaceFontExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PDFFont As IntPtr, ByVal FontFile As String, ByVal Embed As Integer) As Integer
   Private Declare Ansi Function pdfReplaceICCProfileA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ColorSpace As Integer, ByVal ICCFile As String) As Integer
   Private Declare Unicode Function pdfReplaceICCProfileW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ColorSpace As Integer, ByVal ICCFile As String) As Integer
   Private Declare Ansi Function pdfReplacePageTextA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal NewText As String, ByRef Stack As TPDFStack) As Integer
   Private Declare Ansi Function pdfReplacePageTextExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal NewText As String, ByRef Stack As TPDFStack) As Integer
   Private Declare Unicode Function pdfReplacePageTextExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal NewText As String, ByRef Stack As TPDFStack) As Integer
   Private Declare Function pdfResetLineDashPattern Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfRestoreGraphicState Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfRotateCoords Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal alpha As Double, ByVal OriginX As Double, ByVal OriginY As Double) As Integer
   Private Declare Function pdfRoundRect Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Radius As Double, ByVal FillMode As Integer) As Integer
   Private Declare Function pdfRoundRectEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal rWidth As Double, ByVal rHeight As Double, ByVal FillMode As Integer) As Integer
   Private Declare Function pdfSaveGraphicState Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfScaleCoords Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal sx As Double, ByVal sy As Double) As Integer
   Private Declare Function pdfSelfTest Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfSet3DAnnotProps Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Annot As Integer, ByVal ActType As Integer, ByVal DeActType As Integer, ByVal InstType As Integer, ByVal DeInstType As Integer, ByVal DisplToolbar As Integer, ByVal DisplModelTree As Integer) As Integer
   Private Declare Ansi Function pdfSet3DAnnotScriptA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Annot As Integer, ByVal Value As String, ByVal Len As Integer) As Integer
   Private Declare Function pdfSetAllocBy Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetAnnotBorderStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal Style As TBorderStyle) As Integer
   Private Declare Function pdfSetAnnotBorderWidth Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal LineWidth As Double) As Integer
   Private Declare Function pdfSetAnnotColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal ColorType As TAnnotColor, ByVal CS As TPDFColorSpace, ByVal Color As Integer) As Integer
   Private Declare Function pdfSetAnnotFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Flags As Integer) As Integer
   Private Declare Function pdfSetAnnotFlagsEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal Flags As Integer) As Integer
   Private Declare Function pdfSetAnnotHighlightMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal Mode As THighlightMode) As Integer
   Private Declare Function pdfSetAnnotIcon Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal Icon As TAnnotIcon) As Integer
   Private Declare Function pdfSetAnnotOpenState Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal DoOpen As Integer) As Integer
   Private Declare Ansi Function pdfSetAnnotStringA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal StringType As Integer, ByVal Value As String) As Integer
   Private Declare Unicode Function pdfSetAnnotStringW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal StringType As Integer, ByVal Value As String) As Integer
   Private Declare Ansi Function pdfSetAnnotSubjectA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal Value As String) As Integer
   Private Declare Unicode Function pdfSetAnnotSubjectW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer, ByVal Value As String) As Integer
   Private Declare Function pdfSetBBox Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Boundary As Integer, ByVal LeftX As Double, ByVal LeftY As Double, ByVal RightX As Double, ByVal RightY As Double) As Integer
   Private Declare Function pdfSetBidiMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Mode As Integer) As Integer
   Private Declare Function pdfSetBookmarkDest Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ABmk As Integer, ByVal DestType As Integer, ByVal a As Double, ByVal b As Double, ByVal C As Double, ByVal d As Double) As Integer
   Private Declare Function pdfSetBookmarkStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ABmk As Integer, ByVal Style As Integer, ByVal RGBColor As Integer) As Integer
   Private Declare Function pdfSetBorderStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Style As Integer) As Integer
   Private Declare Function pdfSetCharacterSpacing Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSetCheckBoxChar Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal CheckBoxChar As Integer) As Integer
   Private Declare Function pdfSetCheckBoxDefState Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Checked As Integer) As Integer
   Private Declare Function pdfSetCheckBoxState Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Checked As Integer) As Integer
   Private Declare Ansi Function pdfSetCIDFontA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal CMapHandle As Integer, ByVal Name As String, ByVal Style As TFStyle, ByVal Size As Double, ByVal Embed As Integer) As Integer
   Private Declare Unicode Function pdfSetCIDFontW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal CMapHandle As Integer, ByVal Name As String, ByVal Style As TFStyle, ByVal Size As Double, ByVal Embed As Integer) As Integer
   Private Declare Ansi Function pdfSetCMapDirA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Path As String, ByVal Recursive As Integer) As Integer
   Private Declare Unicode Function pdfSetCMapDirW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Path As String, ByVal Recursive As Integer) As Integer
   Private Declare Function pdfSetColDefFile Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal EmbFile As Integer) As Integer
   Private Declare Function pdfSetColors Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Color As Integer) As Integer
   Private Declare Function pdfSetColorSpace Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ColorSpace As Integer) As Integer
   Private Declare Function pdfSetColSortField Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ColField As Integer, ByVal AscendingOrder As Integer) As Integer
   Private Declare Function pdfSetCompressionFilter Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ComprFilter As Integer) As Integer
   Private Declare Function pdfSetCompressionLevel Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal CompressLevel As Integer) As Integer
   Private Declare Function pdfSetContent Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Buffer() As Byte, ByVal BufSize As Integer) As Integer
   Private Declare Function pdfSetDateTimeFormat Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal TxtField As Integer, ByVal Fmt As Integer) As Integer
   Private Declare Function pdfSetDefBitsPerPixel Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Ansi Function pdfSetDocInfoA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DInfo As Integer, ByVal Value As String) As Integer
   Private Declare Unicode Function pdfSetDocInfoW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DInfo As Integer, ByVal Value As String) As Integer
   Private Declare Ansi Function pdfSetDocInfoExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DInfo As Integer, ByVal Key As String, ByVal Value As String) As Integer
   Private Declare Unicode Function pdfSetDocInfoExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DInfo As Integer, <MarshalAs(UnmanagedType.LPStr)> ByVal Key As String, ByVal Value As String) As Integer
   Private Declare Function pdfSetDrawDirection Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Direction As Integer) As Integer
   Private Declare Function pdfSetEMFFrameDPI Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal DPIX As Integer, ByVal DPIY As Integer) As Integer
   Private Declare Function pdfSetEMFPatternDistance Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSetErrorMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ErrMode As Integer) As Integer
   Private Declare Function pdfSetExtColorSpace Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Integer
   Private Declare Function pdfSetExtGState Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Integer
   Private Declare Function pdfSetFieldBackColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AColor As Integer) As Integer
   Private Declare Function pdfSetFieldBBox Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByRef BBox As TPDFRect) As Integer
   Private Declare Function pdfSetFieldBorderColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AColor As Integer) As Integer
   Private Declare Function pdfSetFieldBorderStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Style As Integer) As Integer
   Private Declare Function pdfSetFieldBorderWidth Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal LineWidth As Double) As Integer
   Private Declare Function pdfSetFieldColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal ColorType As Integer, ByVal CS As Integer, ByVal Color As Integer) As Integer
   Private Declare Ansi Function pdfSetFieldExpValue Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal ValIndex As Integer, ByVal Value As String, ByVal ExpValue As String, ByVal Selected As Integer) As Integer
   Private Declare Function pdfSetFieldFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Flags As Integer, ByVal DoReset As Integer) As Integer
   Private Declare Function pdfSetFieldFontSize Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal FontSize As Double) As Integer
   Private Declare Function pdfSetFieldHighlightMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Mode As Integer) As Integer
   Private Declare Function pdfSetFieldIndex Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Index As Integer) As Integer
   Private Declare Ansi Function pdfSetFieldMapNameA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Value As String) As Integer
   Private Declare Unicode Function pdfSetFieldMapNameW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Value As String) As Integer
   Private Declare Ansi Function pdfSetFieldName Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal NewName As String) As Integer
   Private Declare Function pdfSetFieldOrientation Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetFieldTextAlign Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Align As Integer) As Integer
   Private Declare Function pdfSetFieldTextColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Color As Integer) As Integer
   Private Declare Ansi Function pdfSetFieldToolTipA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Value As String) As Integer
   Private Declare Unicode Function pdfSetFieldToolTipW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Value As String) As Integer
   Private Declare Function pdfSetFillColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Color As Integer) As Integer
   Private Declare Function pdfSetFillColorEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Color() As Byte, ByVal NumComponents As Integer) As Integer
   Private Declare Ansi Function pdfSetFontA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Style As Integer, ByVal Size As Double, ByVal Embed As Integer, ByVal CP As Integer) As Integer
   Private Declare Unicode Function pdfSetFontW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Style As Integer, ByVal Size As Double, ByVal Embed As Integer, ByVal CP As Integer) As Integer
   Private Declare Ansi Function pdfSetFontExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Style As Integer, ByVal Size As Double, ByVal Embed As Integer, ByVal CP As Integer) As Integer
   Private Declare Unicode Function pdfSetFontExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Name As String, ByVal Style As Integer, ByVal Size As Double, ByVal Embed As Integer, ByVal CP As Integer) As Integer
   Private Declare Function pdfSetFontOrigin Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Origin As Integer) As Integer
   Private Declare Sub pdfSetFontSearchOrder Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Order() As TFontBaseType)
   Private Declare Function pdfSetFontSelMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Mode As Integer) As Integer
   Private Declare Function pdfSetFontWeight Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Weight As Integer) As Integer
   Private Declare Sub pdfSetGStateFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Flags As Integer, ByVal Reset As Integer)
   Private Declare Function pdfSetIconColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Color As Integer) As Integer
   Private Declare Function pdfSetImportFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Flags As Integer) As Integer
   Private Declare Function pdfSetImportFlags2 Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Flags As Integer) As Integer
   Private Declare Function pdfSetItalicAngle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Angle As Double) As Integer
   Private Declare Function pdfSetJPEGQuality Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Ansi Function pdfSetLanguage Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal ISOTag As String) As Integer
   Private Declare Function pdfSetLeading Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Ansi Function pdfSetLicenseKey Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As String) As Integer
   Private Declare Function pdfSetLineCapStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Style As Integer) As Integer
   Private Declare Ansi Function pdfSetLineDashPattern Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Dash As String, ByVal Phase As Integer) As Integer
   Private Declare Function pdfSetLineDashPatternEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Dash() As Double, ByVal NumValues As Integer, ByVal Phase As Integer) As Integer
   Private Declare Function pdfSetLineJoinStyle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Style As Integer) As Integer
   Private Declare Function pdfSetLineWidth Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSetLinkHighlightMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Mode As Integer) As Integer
   Private Declare Function pdfSetListFont Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Integer
   Private Declare Function pdfSetMatrix Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Matrix As TCTM) As Integer
   Private Declare Function pdfSetMaxFieldLen Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal TxtField As Integer, ByVal MaxLen As Integer) As Integer
   Private Declare Function pdfSetMetaConvFlags Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Flags As Integer) As Integer
   Private Declare Function pdfSetMiterLimit Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSetNeedAppearance Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Ansi Function pdfSetNumberFormat Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal TxtField As Integer, ByVal Sep As Integer, ByVal DecPlaces As Integer, ByVal NegStyle As Integer, ByVal CurrStr As String, ByVal Prepend As Integer) As Integer
   Private Declare Function pdfSetOnErrorProc Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Data As IntPtr, ByVal ErrProc As TErrorProc) As Integer
   Private Declare Function pdfSetOnPageBreakProc Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Data As IntPtr, ByVal OnBreakProc As TOnPageBreakProc) As Integer
   Private Declare Function pdfSetOpacity Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSetOrientation Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetOrientationEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetPageCoords Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PageCoords As Integer) As Integer
   Private Declare Function pdfSetPageFormat Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetPageHeight Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSetPageLayout Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Layout As Integer) As Integer
   Private Declare Function pdfSetPageMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Mode As Integer) As Integer
   Private Declare Function pdfSetPageWidth Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSetPDFVersion Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Version As Integer) As Integer
   Private Declare Function pdfSetPrintSettings Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Mode As Integer, ByVal PickTrayByPDFSize As Integer, ByVal NumCopies As Integer, ByVal PrintScaling As Integer, ByVal PrintRanges() As Integer, ByVal NumRanges As Integer) As Integer
   Private Declare Function pdfSetProgressProc Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Data As IntPtr, ByVal InitProgress As TInitProgress, ByVal Progress As TProgress) As Integer
   Private Declare Function pdfSetResolution Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetSaveNewImageFormat Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetSeparationInfo Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Handle As Integer) As Integer
   Private Declare Function pdfSetStrokeColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Color As Integer) As Integer
   Private Declare Function pdfSetStrokeColorEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Color() As Byte, ByVal NumComponents As Integer) As Integer
   Private Declare Function pdfSetTabLen Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal TabLen As Integer) As Integer
   Private Declare Function pdfSetTextDrawMode Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Mode As Integer) As Integer
   Private Declare Ansi Function pdfSetTextFieldValueA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Value As String, ByVal DefValue As String, ByVal Align As Integer) As Integer
   Private Declare Unicode Function pdfSetTextFieldValueW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Value As String, ByVal DefValue As String, ByVal Align As Integer) As Integer
   Private Declare Ansi Function pdfSetTextFieldValueExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Value As String) As Integer
   Private Declare Unicode Function pdfSetTextFieldValueExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Field As Integer, ByVal Value As String) As Integer
   Private Declare Function pdfSetTextRect Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double) As Integer
   Private Declare Function pdfSetTextRise Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSetTextScaling Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSetTransparentColor Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AColor As Integer) As Integer
   Private Declare Sub pdfSetTrapped Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer)
   Private Declare Function pdfSetUseExactPwd Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetUseGlobalImpFiles Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetUseStdFonts Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetUseSwapFile Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal SwapContents As Integer, ByVal SwapLimit As Integer) As Integer
   Private Declare Ansi Function pdfSetUseSwapFileEx Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal SwapContents As Integer, ByVal SwapLimit As Integer, ByVal SwapDir As String) As Integer
   Private Declare Function pdfSetUseSystemFonts Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetUseTransparency Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetUseVisibleCoords Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetViewerPreferences Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer, ByVal AddVal As Integer) As Integer
   Private Declare Function pdfSetWMFDefExtent Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Width As Integer, ByVal Height As Integer) As Integer
   Private Declare Function pdfSetWMFPixelPerInch Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Integer) As Integer
   Private Declare Function pdfSetWordSpacing Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Value As Double) As Integer
   Private Declare Function pdfSkewCoords Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal alpha As Double, ByVal beta As Double, ByVal OriginX As Double, ByVal OriginY As Double) As Integer
   Private Declare Function pdfSortFieldsByIndex Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Function pdfSortFieldsByName Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfStampAnnotA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal SubType As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Subject As String, ByVal Comment As String) As Integer
   Private Declare Unicode Function pdfStampAnnotW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal SubType As Integer, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Subject As String, ByVal Comment As String) As Integer
   Private Declare Function pdfStrokePath Lib "dynapdf.dll" (ByVal IPDF As IntPtr) As Integer
   Private Declare Ansi Function pdfTextAnnotA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Text As String, ByVal Icon As Integer, ByVal DoOpen As Integer) As Integer
   Private Declare Unicode Function pdfTextAnnotW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Author As String, ByVal Text As String, ByVal Icon As Integer, ByVal DoOpen As Integer) As Integer
   Private Declare Function pdfTranslateCoords Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal OriginX As Double, ByVal OriginY As Double) As Integer
   Private Declare Function pdfTriangle Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal x1 As Double, ByVal y1 As Double, ByVal x2 As Double, ByVal y2 As Double, ByVal x3 As Double, ByVal y3 As Double, ByVal FillMode As Integer) As Integer
   Private Declare Ansi Function pdfWebLinkA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal URL As String) As Integer
   Private Declare Unicode Function pdfWebLinkW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal URL As String) As Integer
   Private Declare Ansi Function pdfWriteAngleTextA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String, ByVal Angle As Double, ByVal PosX As Double, ByVal PosY As Double, ByVal Radius As Double, ByVal YOrigin As Double) As Integer
   Private Declare Unicode Function pdfWriteAngleTextW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal AText As String, ByVal Angle As Double, ByVal PosX As Double, ByVal PosY As Double, ByVal Radius As Double, ByVal YOrigin As Double) As Integer
   Private Declare Ansi Function pdfWriteFTextA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Align As Integer, ByVal AText As String) As Integer
   Private Declare Unicode Function pdfWriteFTextW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal Align As Integer, ByVal AText As String) As Integer
   Private Declare Ansi Function pdfWriteFTextExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Align As Integer, ByVal AText As String) As Integer
   Private Declare Unicode Function pdfWriteFTextExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal Width As Double, ByVal Height As Double, ByVal Align As Integer, ByVal AText As String) As Integer
   Private Declare Ansi Function pdfWriteTextA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal AText As String) As Integer
   Private Declare Unicode Function pdfWriteTextW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal AText As String) As Integer
   Private Declare Ansi Function pdfWriteTextExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal AText As String, ByVal Len As Integer) As Integer
   Private Declare Unicode Function pdfWriteTextExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByVal PosX As Double, ByVal PosY As Double, ByVal AText As String, ByVal Len As Integer) As Integer
   Private Declare Ansi Function pdfWriteTextMatrixA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Matrix As TCTM, ByVal AText As String) As Integer
   Private Declare Unicode Function pdfWriteTextMatrixW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Matrix As TCTM, ByVal AText As String) As Integer
   Private Declare Ansi Function pdfWriteTextMatrixExA Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Matrix As TCTM, ByVal AText As String, ByVal Len As Integer) As Integer
   Private Declare Unicode Function pdfWriteTextMatrixExW Lib "dynapdf.dll" (ByVal IPDF As IntPtr, ByRef Matrix As TCTM, ByVal AText As String, ByVal Len As Integer) As Integer
   ' Helper function to copy a kerning record into the managed type TTextRecordW.
   Public Declare Function CopyKernRecord Lib "dynapdf.dll" Alias "pdfCopyMem" (ByVal Source As IntPtr, ByRef Record As TTextRecordW, ByVal Len As Integer) As Integer
   Private Declare Function pdfCopyMemUInt Lib "dynapdf.dll" Alias "pdfCopyMem" (ByVal Source As IntPtr, ByVal Dest() As System.UInt32, ByVal Len As Integer) As Integer
   Private Declare Function pdfCopyMemIntPtr Lib "dynapdf.dll" Alias "pdfCopyMem" (ByVal Source As IntPtr, ByVal Dest() As IntPtr, ByVal Len As Integer) As Integer
   Private Declare Function pdfStrLenA Lib "dynapdf.dll" (ByVal AStr As IntPtr) As Integer
End Class
