/*---------------------------------------------------------+
| DynaPDF 2.5                                              |
+----------------------------------------------------------+
| Copyright (C) 2003-2011                                  |
| Jens Boschulte, DynaForms GmbH.                          |
| All rights reserved.                                     |
+----------------------------------------------------------+
| Please report errors or other potential problems to      |
| support@dynaforms.com.                                   |
| The actual version is available at www.dynaforms.com.    |
+----------------------------------------------------------|
|                Creation date: February 9, 2011           |
+----------------------------------------------------------*/

using System;
using System.Runtime.InteropServices;
using lbool = System.Int32;

namespace DynaPDF
{
   public enum T3DActivationType
   {
      at3D_AppDefault,
      at3D_PageOpen,    // The annotaiton should be activated when the page is opened.
      at3D_PageVisible, // The annotaiton should be activated when the page becomes visible.
      at3D_Explicit     // The annotation should remain inactive until explicitely activated by a script or action (default).
   }

   public enum T3DDeActivateType
   {
      dt3D_AppDefault,
      dt3D_PageClosed,    // The annotaiton should be deactivated as soon as the page is closed.
      dt3D_PageInvisible, // The annotaiton should be deactivated as soon as the page becomes invisible (default).
      dt3D_Explicit       // The annotation should remain active until explicitely deactivated by a script or action.
   }

   public enum T3DDeActInstance
   {
      di3D_AppDefault,
      di3D_UnInstantiated, // The annotation will be uninstantiated (default)
      di3D_Instantiated,   // The annotation is left instantiated
      di3D_Live            // If the 3D artwork contains an animation then it will stay live
   }

   public enum T3DInstanceType
   {
      it3D_AppDefault,
      it3D_Instantiated, // The annotation will be instantiated but animations are disabled.
      it3D_Live          // The annotation will be instantiated and animations are enabled (default).
   }

   public enum T3DLightingSheme
   {
      lsArtwork, // Lights from file
      lsBlue,
      lsCAD,
      lsCube,
      lsDay,
      lsHard,
      lsHeadlamp,
      lsNight,
      lsNoLights,
      lsPrimary,
      lsRed,
      lsWhite,
      lsNotSet
   }

   public enum T3DNamedAction
   {
      naDefault,
      naFirst,
      naLast,
      naNext,
      naPrevious
   }

   public enum T3DProjType
   {
      pt3DOrthographic,
      pt3DPerspective
   }

   public enum T3DRenderingMode
   {
      rmBoundingBox,
      rmHiddenWireframe,
      rmIllustration,
      rmShadedIllustration,
      rmShadedVertices,
      rmShadedWireframe,
      rmSolid,
      rmSolidOutline,
      rmSolidWireframe,
      rmTransparent,
      rmTranspBBox,
      rmTranspBBoxOutline,
      rmTranspWireframe,
      rmVertices,
      rmWireframe,
      rmNotSet
   }

   public enum T3DScaleType
   {
      st3DValue,
      st3DWidth,
      st3DHeight,
      st3DMin,
      st3DMax
   }

   public enum TActionType
   {
      atGoTo,
      atGoToR,
      atHide,
      atImportData,
      atJavaScript,
      atLaunch,
      atMovie,
      atNamed,
      atRendition,
      atReset,
      atSetOCGState,
      atSound,
      atSubmit,
      atThread,
      atTransition,
      atURI,
      atGoTo3DView
   }

   public enum TAnnotColor
   {
      acBackColor,
      acBorderColor,
      acTextColor
   }

   public enum TAnnotFlags
   {
      afNone       = 0x00000000,
      afInvisible  = 0x00000001,
      afHidden     = 0x00000002,
      afPrint      = 0x00000004,
      afNoZoom     = 0x00000008,
      afNoRotate   = 0x00000010,
      afNoView     = 0x00000020,
      afReadOnly   = 0x00000040
   }

   /*
      By default all annotations which have an appearance stream and which have the print flag set are flattened.
      All annotations are deleted when the function returns with the exception of file attachment annotations.
      If you want to flatten the view state then set the flag affUseViewState.
   */
   public enum TAnnotFlattenFlags
   {
      affNone         = 0x00000000, // Printable annotations independent of the type
      affUseViewState = 0x00000001, // If set, annotations which are visible in a viewer become flattened.
      affMarkupAnnots = 0x00000002  // If set, markup annotations are flattened only. Link, Sound, or FileAttach
                                    // annotations are no markup annotations. These types will be left intact.
   }

   public enum TAnnotIcon
   {
      aiComment,
      aiHelp,
      aiInsert,
      aiKey,
      aiNewParagraph,
      aiNote,
      aiParagraph
   }

   public enum TAnnotString
   {
      asAuthor,
      asContent,
      asName,
      asSubject
   }

   public enum TAnnotType
   {
      atCaret,
      atCircle,
      atFileLink,
      atFreeText,
      atHighlight,
      atInk,
      atLine,
      atPageLink,
      atPolygon,
      atPolyLine,
      atPopUp,
      atSquare,
      atSquiggly,
      atStamp,
      atStrikeOut,
      atText,
      atUnderline,
      atWebLink,
      atWidget,
      at3D,
      atSoundAnnot,
      atFileAttach,
      atRedact,
      atWatermark,
      atUnknown    // Unknown annotation type
   }

   public enum TBlendMode
   {
      bmNotSet,
      bmNormal,
      bmColor,
      bmColorBurn,
      bmColorDodge,
      bmDarken,
      bmDifference,
      bmExclusion,
      bmHardLight,
      bmHue,
      bmLighten,
      bmLuminosity,
      bmMultiply,
      bmOverlay,
      bmSaturation,
      bmScreen,
      bmSoftLight
   }

   public enum TDestType
   {
      dtXY_Zoom,     /* three parameters (a, b, c) -> (X, Y, Zoom)                    */
      dtFit,         /* no parameters                                                 */
      dtFitH_Top,    /* one parameter    (a)                                          */
      dtFitV_Left,   /* one parameter    (a)                                          */
      dtFit_Rect,    /* four parameters  (a, b, c, d) -> (left, bottom, right, top)   */
      dtFitB,        /* no parameters                                                 */
      dtFitBH_Top,   /* one parameter    (a)                                          */
      dtFitBV_Left   /* one parameter    (a)                                          */
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TBookmark
   {
      public uint      Color;
      public int       DestPage;
      public TPDFRect  DestPos;
      public TDestType DestType;
      public bool      Open;
      public int       Parent;
      public TFStyle   Style;
      public String    Title;
   }

   public enum TBorderStyle
   {
      bsSolid,
      bsBevelled,
      bsInset,
      bsUnderline,
      bsDashed
   }

   public enum TButtonState
   {
      bsUp,
      bsDown,
      bsRollOver
   }

   public enum TCheckBoxChar
   {
      ccCheck,
      ccCircle,
      ccCross1,
      ccCross2,
      ccCross3,
      ccCross4,
      ccDiamond,
      ccSquare,
      ccStar
   }

   public enum TCheckBoxState
   {
      cbUnknown,
      cbChecked,
      cbUnChecked
   }

   public enum TCheckOptions
   {
      coDefault                  = 0x00000FFF,
      coEmbedSubsets             = 0x00000001,
      coDeleteTransferFuncs      = 0x00000002,
      coDeleteMultiMediaContents = 0x00000004,
      coDeleteActionsAndScripts  = 0x00000008,
      coDeleteInvRenderingIntent = 0x00000010,
      coFlattenFormFields        = 0x00000020,
      coReplaceV4ICCProfiles     = 0x00000040,
      coDeleteEmbeddedFiles      = 0x00000080,
      coDeleteOPIComments        = 0x00000100,
      coDeleteSignatures         = 0x00000200,
      coReplCCITTFaxWithFlate    = 0x00010000,
      coNoFontEmbedding          = 0x10000000 // If this flag is set no PDF/A file will be produced!
   }

   public enum TClippingMode
   {
      cmEvenOdd,
      cmWinding
   }

   public enum TCodepage
   {
      /*---------------------------------------- 8 bit code pages --------------------------------------*/
      cp1250,
      cp1251,
      cp1252,
      cp1253,
      cp1254,
      cp1255,
      cp1256,
      cp1257,
      cp1258,
      cp8859_2,
      cp8859_3,
      cp8859_4,
      cp8859_5,
      cp8859_6,
      cp8859_7,
      cp8859_8,
      cp8859_9,
      cp8859_10,
      cp8859_13,
      cp8859_14,
      cp8859_15,
      cp8859_16,
      cpSymbol,
      cp437,
      cp737,
      cp775,
      cp850,
      cp852,
      cp855,
      cp857,
      cp860,
      cp861,
      cp862,
      cp863,
      cp864,
      cp865,
      cp866,
      cp869,
      cp874,

      /* ----------------------------------------- Unicode (UCS-2) -------------------------------------*/

      cpUnicode,

      /*-------------------------------- CJK encodings for use with Unicode fonts ----------------------*/
      // The following endcodings requires CJK or ISO-2022 input character sequences which are converted to Unicode
      // by DynaPDF. It is not possible to use these encodings with CJK fonts. If a CJK font should be used, choose
      // one of the native CJK character sets listed at the end this block. DynaPDF does not support a conversion
      // algorithm to convert ISO-2022 character sequences to CJK code points. If you needed such a mapping, please
      // send a mail to support@dynaforms.com.

      // UCS-2 does not support all available CJK code points. However, surrogates are still accessible if the used
      // TrueType font contains a cmap format 12. Another way is to use a CJK font with a native CJK charset.

      cpCJK_Big5_Uni,    // Big5 plus HKSCS extension.
      cpCJK_EUC_JP_Uni,  // EUC-JP.
      cpCJK_EUC_KR_Uni,  // EUC-KR.
      cpCJK_EUC_TW_Uni,  // CNS-11643-1992 (Planes 1-15).
      cpCJK_GBK_Uni,     // GBK is the Microsoft code page 936 (GB2312, EUC-CN plus GBK extension).
      cpCJK_GB12345_Uni, // GB-12345-1990 (Traditional Chinese form of GB-2312).
      cpCJK_HZ_Uni,      // Mixed ASCII / GB-2312 encoding
      cpCJK_2022_CN_Uni, // ISO-2022-CN-EXT (GB-2312 plus ISO-11643 Planes 1-7).
      cpCJK_2022_JP_Uni, // ISO-2022-JP.
      cpCJK_2022_KR_Uni, // ISO-2022-KR.
      cpCJK_646_CN_Uni,  // ISO-646-CN (GB-1988-80).
      cpCJK_646_JP_Uni,  // ISO-646-JP (JIS_C6220-1969-RO).
      cpCJK_IR_165_Uni,  // ISO-IR-165 (extended version of GB-2312).
      cpCJK_932_Uni,     // Microsoft extended version of SHIFT_JIS.
      cpCJK_949_Uni,     // EUC-KR extended with UHC (Unified Hangul Codes).
      cpCJK_950_Uni,     // Microsoft extended version of Big5.
      cpCJK_JOHAB_Uni,   // JOHAB.

      /*---------------------------------- Native CJK charsets (CJK fonts only) ------------------------*/

      // When using a CJK font, DynaPDF creates dynamically a "To Unicode CMAP" to enable cut & paste
      // operations in Adobe's Acrobat. If a CJK sequence has no Unicode code point, the CJK code is used instead.
      // In this case, cut & paste works for all other characters, but the unmapped character appears as arbitrary
      // wrong character when the string is paste to another document. A wrongly mapped character in the
      // "To Unicode CMAP" has no effect in viewing or printing the document.

      // The comments describe which mapping tables are available for a character set. Note these are character sets
      // not code pages.

      cpShiftJIS,     // ShiftJIS charset plus code page 932 extension.
      cpBig5,         // Big5 plus HKSCS extension.
      cpGB2312,       // GB2312 charset plus GBK and cp936 extension.
      cpWansung,      // Wansung
      cpJohab,        // Johab
      cpMacRoman,     // Mac Roman
      cpAdobeStd,     // This is an encoding for Type1 fonts. It should normally not be used.
      cpInternal,     // Internal -> not usable
      cpGlyphIndexes, // Can be used with TrueType and OpenType fonts only. DynaPDF creates a reverse mapping so that copy & paste will work.
      cpPDFDocEnc,    // Internal -> not usable. Used for form fonts. This is a superset of the code page 1252 and MacRoman.
      cpExtCMap,      // Internal -> not usable. This code page is set when a font was loaded with an external cmap.
      cpDingbats      // Internal -> Special encoding for ZapfDingbats
   }

   // The data for user defined columns is stored in collection items.
   public enum TColColumnType
   {
      cisCreationDate, // Data comes from the embedded file
      cisDescription,  // Data comes from the embedded file
      cisFileName,     // Data comes from the embedded file
      cisModDate,      // Data comes from the embedded file
      cisSize,         // Data comes from the embedded file
      cisCustomDate,   // User defined date.
      cisCustomNumber, // User defined nummber.
      cisCustomString  // User defined string.
   }

   public enum TColorConvFlags : uint
   {
      ccfBW_To_Gray   = 0, // Default, RGB Black and White set with rg or RG inline operators are converted to gray
      ccfRGB_To_Gray  = 1  // If set, inline color operators rg and RG are converted to gray
   }

   public enum TColorMode
   {
      cmFill,
      cmStroke,
      cmFillStroke
   }

   public enum TColView
   {
      civNotSet,
      civDetails,
      civTile,
      civHidden
   }

   public enum TCompBBoxFlags : uint
   {
      cbfNone =             0x00000000,
      cbfIgnoreWhiteAreas = 0x00000001, // Ignore white vector graphics or text.
   /*
      Please note that images must be decompressed if one of the following flags are set. Parsing gray or color images
      is in most cases not useful and you should not parse such images if it is not really required.
   */
      cbfParse1BitImages  = 0x00000002, // Find the visible area in 1 bit images. This is the most important case
                                        // since scanned faxes are usually 1 bit images.
      cbfParseGrayImages  = 0x00000004, // Find the visible area in gray images.
      cbfParseColorImages = 0x00000008, // Find the visible area in color images. This is usually not required
                                        // and slow downs processing a lot.
      cbfParseAllImages   = 0x0000000E  // Find the visible area in all images.
   }

   public enum TConformanceType
   {
      ctPDFA_1b_2005, // Convert the file to PDF/A if possible
      ctNormalize     // Check the file for errors, rebuild all embedded fonts plus options
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TCTM
   {
      public double a;
      public double b;
      public double c;
      public double d;
      public double x;
      public double y;
   }

   public enum TCompressionFilter
   {
      cfFlate    = 0, // PDF or TIFF output
      cfJPEG     = 1, // PDF, JPEG, or TIFF output
      cfCCITT3   = 2, // PDF or TIFF output -> B&W CCITT Fax G3 compression -> fast but less compression ratio
      cfCCITT4   = 3, // PDF or TIFF output -> B&W CCITT Fax G4 compression -> slower but higher compression ratio
      cfLZW      = 4, // TIFF or GIF output -> Very fast but less compression ratios than flate
      cfReserved = 5, // Reserved for future extensions.
      cfFlateBW  = 6, // TIFF, PNG, or BMP output -> Dithered black & white output. The resulting image will be
                      // compressed with Flate or left uncompressed if the output image format is a bitmap. If
                      // you want to use CCITT Fax 4 compression (TIFF only) set the flag icUseCCITT4 in the
                      // AddImage() function call. Note that this filter is not supported for PDF creation!
      cfJP2K     = 7  // PDF or JPEG2000 output
   }

   public enum TCompressionLevel
   {
      clNone,
      clDefault,
      clFastest,
      clMax
   }

   public enum TDecodeFilter
   {
      dfNone,
      dfASCII85Decode,   // No parameters
      dfASCIIHexDecode,  // No parameters
      dfCCITTFaxDecode,  // Optional Parameters
      dfDCTDecode,       // Optional Parameters
      dfFlateDecode,     // Optional Parameters
      dfJBIG2Decode,     // Optional Parameters
      dfJPXDecode,       // No parameters
      dfLZWDecode,       // Optional Parameters
      dfRunLengthDecode  // No parameters
   }

   public enum TDecSeparator
   {  /* thousand separator, decimal separator */
      dsCommaDot,
      dsNoneDot,
      dsDotComma,
      dsNoneComma
   }

   public enum TDocumentInfo
   {
      diAuthor,
      diCreator,
      diKeywords,
      diProducer,
      diSubject,
      diTitle,
      diCompany,
      diPDFX_Ver, // GetInDocInfo() only -> The PDF/X version is set by SetPDFVersion()!
      diCustom    // User defined key
   }

   public enum TDrawDirection
   {
      ddCounterClockwise,
      ddClockwise
   }

   public enum TDrawMode
   {
      dmNormal         = 0,
      dmStroke         = 1,
      dmFillStroke     = 2,
      dmInvisible      = 3,
      dmFillClip       = 4,
      dmStrokeClip     = 5,
      dmFillStrokeClip = 6,
      dmClipping       = 7,
      dmNone           = -1 // Internal, do not use this mode!!!
   }

   public enum TDuplexMode
   {
      dpmNone,          // Default
      dpmSimplex,
      dpmFlipShortEdge,
      dpmFlipLongEdge
   }

   public enum TErrMode : uint
   {
      emIgnoreAll    = 0x00000000,
      emSyntaxError  = 0x00000001,
      emValueError   = 0x00000002,
      emWarning      = 0x00000004,
      emFileError    = 0x00000008,
      emFontError    = 0x00000010,
      emAllErrors    = 0x0000FFFF,
      emNoFuncNames  = 0x10000000 // Do not print function names in error messages
   }

   // Note that error codes are negative; to determine the error type use the absolute value:
   // if (-errCode & DynaPDF.TErrType.E_FATAL_ERROR) Console.Write("A fatal error occurred!");
   public enum TErrType : uint
   {
      E_WARNING      = 0x02000000,
      E_SYNTAX_ERROR = 0x04000000,
      E_VALUE_ERROR  = 0x08000000,
      E_FONT_ERROR   = 0x10000000,
      E_FATAL_ERROR  = 0x20000000,
      E_FILE_ERROR   = 0x40000000,

      E_NEED_PWD        = 0xb2 | E_FILE_ERROR,
      E_WRONG_OPEN_PWD  = 0xb3 | E_FILE_ERROR,
      E_WRONG_OWNER_PWD = 0xb4 | E_FILE_ERROR,
      E_WRONG_PWD       = 0xb5 | E_FILE_ERROR
   }

   public enum TExtColorSpace
   {
      esDeviceRGB  = 0,  // Device color space
      esDeviceCMYK = 1,  // Device color space
      esDeviceGray = 2,  // Device color space
      esCalGray    = 3,  // CIE-based color space
      esCalRGB     = 4,  // CIE-based color space
      esLab        = 5,  // CIE-based color space
      esICCBased   = 6,  // CIE-based color space -> contains an ICC profile
      esPattern    = 7,  // Special color space
      esIndexed    = 8,  // Special color space
      esSeparation = 9,  // Special color space
      esDeviceN    = 10, // Special color space
      esNChannel   = 11, // Special color space
      esInitSpace  = -1  // Internal -> This value must not be used!
   }

   public enum TFileAttachIcon
   {
      faiGraph,
      faiPaperClip,
      faiPushPin,
      faiTag
   }

   public enum TFileOP
   {
      foOpen,
      foPrint
   }

   public enum TFieldColor
   {
      fcBackColor,
      fcBorderColor,
      fcTextColor
   }

   public enum TFieldFlags : uint
   {
      ffReadOnly         = 0x00000001,
      ffRequired         = 0x00000002,
      ffNoExport         = 0x00000004,

      ffInvisible        = 0x00000008,
      ffHidden           = 0x00000010,
      ffPrint            = 0x00000020,
      ffNoZoom           = 0x00000040,
      ffNoRotate         = 0x00000080,
      ffNoView           = 0x00000100,

      ffMultiline        = 0x00001000, /* Text fields only */
      ffPassword         = 0x00002000, /* Text fields only */
      ffNoToggleToOff    = 0x00004000, /* Radio buttons    */
      ffRadioIsUnion     = 0x02000000, /* PDF-1.5 check boxes of a radio button field only */
      ffCommitOnSelCh    = 0x04000000, /* PDF-1.5 combo boxes, list boxes */

      ffEdit             = 0x00040000, /* Combo boxes only */
      ffSorted           = 0x00080000, /* Use this flag to change the sort flag of combo boxes or list boxes */
      ffFileSelect       = 0x00100000, /* PDF 1.4 Text fields only */
      ffMultiSelect      = 0x00200000, /* PDF 1.4 List boxes only  */
      ffDoNotSpellCheck  = 0x00400000, /* PDF 1.4 Text fields, combo boxes. If the field is a combo box, this flag is meaningful only if ffEdit is also set. */
      ffDoNotScroll      = 0x00800000, /* PDF 1.4 Text fields only */
      ffComb             = 0x01000000  /* PDF 1.5 Text fields only */
   }

   public enum TFieldType
   {
      ftButton,
      ftCheckBox,
      ftRadioBtn,
      ftComboBox,
      ftListBox,
      ftText,
      ftSignature,
      ftGroup // this is not a real field type, it is just an array of fields
   }

   public enum TFontBaseType
   {
      fbtTrueType,
      fbtType1,
      fbtOpenType, // OpenType font with Postscript outlines
      fbtStdFont,  // PDF Standard font
      fbtDisabled  // This value can be used to disable a specific font format. See SetFontSearchOrder() for further information.
   }

   public enum TFontFileSubtype
   {
      ffsType1C        = 0, // CFF based Type1 font
      ffsCIDFontType0C = 1, // CFF based Type1 CID font
      ffsOpenType      = 2, // TrueType based OpenType font
      ffsOpenTypeC     = 3, // CFF based OpenType font
      ffsCIDFontType2  = 4, // TrueType based CID Font
      ffsNoSubtype     = 9  // The font file is in the format of FontType
   }

   // If font selection mode is set to smPostScriptName the font styles fsItalic or fsBold are ignored.
   // The PostScriptName is already a unique font name incl. style information.
   public enum TFontSelMode
   {
      smFamilyName,
      smPostScriptName
   }

   public enum TFontType
   {
      ftMMType1,
      ftTrueType,
      ftType0,    // Check the font file type to determine the font sub type
      ftType1,
      ftType3
   }

/*
   The font search run works as follows:

      - DynaPDF tries always to find the exact weight, if it cannot be found then a font with
        the next smaller weight is selected (if available).
      - Italic styles can always be emulated but it is not possible to emulate thinner weights or
        regular styles with an italic font.
      - If the specified weight is larger as the font weight the remaining weight will be emulated
        if the difference to the requested weight is larger than 200.
        With SetFontWeight() it is possible to control whether a missing weight should be emulated.
        If FontWeight is smaller or equal to the requested font weight then emulation will be disabled.

   TFStyle is a bitmask that is defined as follows:

      - Bits 0..7   // Style bits fsItalic, fsUnderlined, fsStriked
      - Bits 8..19  // Width class -> Defined for future use.
      - Bits 20..31 // Font Weight

   - A width class can be converted to a style constant by multiplying it with 256 (width << 8).
   - A font weight can be converted to a style constant by multiplying it with 1048576 (weight << 20).
   - Additional attributes can be added with a binary or operator (e.g. style or fsItalic).
   - Only one width class and one font weight can be set at time.

   - WidthFromStyle() extracts the width class.
   - WeightFromStyle() extracts the font weight.

   The following functions extract the width class or font weight from a style variable:

      widthClass  = WidthFromStyle(style);
      weightClass = WeightFromStyle(style);
*/

   public enum TFStyle
   {
      fsNone           = 0x00000000, // Regular weight (400)
      fsItalic         = 0x00000001,
      fsUnderlined     = 0x00000004,
      fsStriked        = 0x00000008,
      fsVerticalMode   = 0x00000010, // Not considered at this time
      // Width class
      fsUltraCondensed = 0x00000100, // 1
      fsExtraCondensed = 0x00000200, // 2
      fsCondensed      = 0x00000300, // 3
      fsSemiCondensed  = 0x00000400, // 4
      fsNormal         = 0x00000500, // 5
      fsSemiExpanded   = 0x00000600, // 6
      fsExpanded       = 0x00000700, // 7
      fsExtraExpanded  = 0x00000800, // 8
      fsUltraExpanded  = 0x00000900, // 9
      // Weight class
      fsThin           = 0x06400000, // 100
      fsExtraLight     = 0x0c800000, // 200
      fsLight          = 0x12C00000, // 300
      fsRegular        = 0x19000000, // 400 -> Same as fsNone
      fsMedium         = 0x1F400000, // 500
      fsDemiBold       = 0x25800000, // 600
      fsBold           = 0x2BC00000, // 700 -> The old constant 2 is still supported to preserve backward compatibility
      fsExtraBold      = 0x32000000, // 800
      fsBlack          = 0x38400000, // 900
      fsUltraBlack     = 0x3E800000  // 1000
   }

   public enum TGStateFlags
   {
      gfCompatible         = 0x00000000, // Compatible graphics state to earlier DynaPDF versions -> default
      gfRestorePageCoords  = 0x00000001, // Restore the coordinate system with the graphics state (the value of PageCoords, see SetPageCoords())
      gfRealTopDownCoords  = 0x00000002, // If set, the page coordinate system is not reset to bottom-up when transforming
                                         // the coordinate system. However, real top-down coordinates require a large internal
                                         // overhead and where never fully implemented. The usage of this flag should be avoided
                                         // if possible.
      gfNativeBlackWhite   = 0x00000004, // Do not convert RGB black or white to DeviceGray
      gfUseImageColorSpace = 0x00000008, // If set, the active color space is ignored when inserting an image. The color space is taken
                                         // from the image file instead.
      gfIgnoreICCProfiles  = 0x00000010  // If set, embedded ICC profiles in images are ignored. The meaning of this flag depends on
                                         // whether the flag gfUseImageColorSpace is set too. If set, an embedded profile is not used to
                                         // create an ICCBased color space for the image. The image is inserted in the corresponding
                                         // device color space instead. If the flag gfUseImageColorSpace is absent an embedded ICC profile
                                         // is not used to convert CMYK images to DeviceRGB if necessary. This conversion is only applied
                                         // if the current color space is DeviceRGB and if the image color space is CMYK.
   }

   public enum THashType
   {
      htDetached, // CloseAndSignFileExt() returns the byte ranges of the finish PDF buffer to create a detached signature
      htSHA1      // CloseAndSignFileExt() returns the SHA1 hash of the PDF file so that it can be signed
   }

   public enum THighlightMode
   {
      hmNone,
      hmInvert,
      hmOutline,
      hmPush,
      hmPushUpd  // Update appereance stream on changes
   }

   public enum TICCProfileType
   {
      ictGray,
      ictRGB,
      ictCMYK,
      ictLab
   }

   public enum TImageConversionFlags
   {
      icNone      = 0, // Default
      icUseCCITT4 = 1  // Use CCITT Fax 4 compression instead of Flate for dithered images.
   }

   /*
      TIFF is the only format that supports different compression filters. The Filter parameter of the function
      AddImage() is ignored if the image format supports only one specific compression filter.
      Note that images are automatically converted to the nearest supported color space if the image format does
      not support the color space of the image.
   */
   public enum TImageFormat
   {
      ifmTIFF,     // DeviceRGB, DeviceCMYK, DeviceGray, Black & White -> CCITT Fax Group 3/4, JPEG, Flate, LZW.
      ifmJPEG,     // DeviceRGB, DeviceCMYK, DeviceGray    -> JPEG compression.
      ifmPNG,      // DeviceGray, DeviceRGB, Black & White -> Flate compression.
      ifmReserved, // Reserved for future extensions.
      ifmBMP,      // DeviceGray, DeviceRGB, Black & White -> Uncompressed.
      ifmJPC       // DeviceRGB, DeviceCMYK, DeviceGray    -> JPEG2000 compression.
   }

   public enum TImportFlags : uint
   {
      ifImportAll        = 0x0FFFFFFE, // default
      ifContentOnly      = 0x00000000,
      // If this flag is set, only interactive objects are imported if any, Otherwise only empty pages are imported.
      // This flag can be used to copy an Interactive Form, bookmarks or annotations to another PDF file.
      ifNoContent        = 0x00000001,
      // The imported page is not converted to a template if ifImportAsPage is set.
      // Note that this flag can cause resource conflicts. Use this flag carefully!
      ifImportAsPage     = 0x80000000,
      // base objects
      ifCatalogAction    = 0x00000002, // Open action, Catalog actions
      ifPageActions      = 0x00000004,
      ifBookmarks        = 0x00000008,
      ifArticles         = 0x00000010,
      ifPageLabels       = 0x00000020,
      ifThumbs           = 0x00000040,
      ifTranspGroups     = 0x00000080, // This flag is not longer considered.
      ifSeparationInfo   = 0x00000100,
      ifBoxColorInfo     = 0x00000200,
      ifStructureTree    = 0x00000400,
      ifTransition       = 0x00000800,
      ifSearchIndex      = 0x00001000,
      ifJavaScript       = 0x00002000, // Global JavaScripts
      ifJSActions        = 0x00004000, // JavaScript actions
      ifDocInfo          = 0x00008000, // Document info entries
      ifEmbeddedFiles    = 0x00200000, // File attachments
      ifFileCollections  = 0x00400000, // File collections (PDF 1.7)
      // Annotations -> Only the most important annotation types can be selected directly.
      // Note that all annotation types can be deleted with DeleteAnnotation.
      ifAllAnnots        = 0x009F0000,
         ifFreeText      = 0x00010000,
         ifTextAnnot     = 0x00020000,
         ifLink          = 0x00040000,
         ifStamp         = 0x00080000,
         if3DAnnot       = 0x00100000,
         ifOtherAnnots   = 0x00800000,
      // Interactive Form Fields are also annotations but we handle this type separately!
      ifFormFields       = 0x01000000,
      /* -------------------- Special flags -------------------- */
      ifPrepareForPDFA   = 0x10000000, // Replace LZW compression with Flate, set the Interpolate key of images to false, do not import embedded files.
      ifEnumFonts        = 0x20000000, // Import fonts for EnumDocFonts(). The document must be deleted when this flag is set!!!
      ifAllPageObjects   = 0x40000000  // Import links when using ImportPageEx() within an open page. The entire document should be imported in this case.
   }

   public enum TImportFlags2 : uint
   {
      if2MergeLayers     = 0x00000001, // If set, layers with identical name are merged. If this flag is absent DynaPDF
                                       // imports such layers separately so that each layer refers still to the pages
                                       // where it was orignally used.
      if2Normalize       = 0x00000002  // Replace LZW compression with Flate, apply limit checks, repair errors if possible
   }

   public enum TKeyLen
   {
      kl40bit,    // RC4 Encryption -> Acrobat 3 or higher
      kl128bit,   // RC4 Encryption -> Acrobat 5 or higher
      kl128bitEx, // RC4 Encryption -> Acrobat 6 or higher
      klAES128,   // AES Encryption -> Acrobat 7 or higher
      klAES256    // AES Encryption -> Acrobat 9 or higher
   }

   public enum TLineCapStyle
   {
      csButtCap,
      csRoundCap,
      csSquareCap
   }

   public enum TLineJoinStyle
   {
      jsMiterJoin,
      jsRoundJoin,
      jsBevelJoin
   }

   public enum TMetaFlags : uint
   {
      mfDefault           = 0x00000000, // Default conversion
      mfDebug             = 0x00000001, // Write debug information into the content stream
      mfShowBounds        = 0x00000002, // Show the bounding boxes of text strings
      mfNoTextScaling     = 0x00000004, // Do not scale text
      mfClipView          = 0x00000008, // Draw the file into a clipping rectangle
      mfUseRclBounds      = 0x00000010, // Use rclBounds instead of rclFrame
      mfNoClippingRgn     = 0x00000040, // Disables SelectClippingRegion and IntersectClipRect
      mfNoFontEmbedding   = 0x00000080, // Do not embed fonts -> Fonts should be embedded!!!
      mfNoImages          = 0x00000100, // Ignore image records
      mfNoStdPatterns     = 0x00000200, // Ignore standard patterns
      mfNoBmpPatterns     = 0x00000400, // Ignore bitmap patterns
      mfNoText            = 0x00000800, // Ignore text records
      mfUseUnicode        = 0x00001000, // Ignore ANSI_CHARSET
      mfUseTextScaling    = 0x00004000, // Scale text instead of using the intercharacter spacing array
      mfNoUnicode         = 0x00008000, // Avoid usage of Unicode fonts -> recommended to enable PDF 1.2 compability
      mfFullScale         = 0x00010000, // Recommended if 32 bit coordinates are used
      mfUseRclFrame       = 0x00020000, // This flag should be set if the rclFrame rectangle is properly set
      mfDefBkModeTransp   = 0x00040000, // Initialize the background mode to transparent (SetBkMode() overrides this state).
      mfApplyBidiAlgo     = 0x00080000, // Apply the bidirectional algorithm on Unicode strings
      mfGDIFontSelection  = 0x00100000, // Use the GDI to select fonts
      mfRclFrameEx        = 0x00200000, // If set, and if the rclBounds rectangle is larger than rclFrame, the function
                                        // extends the output rectangle according to rclBounds and uses the resulting
                                        // bounding box to calculate the image size (rclBounds represents the unscaled
                                        // image size). This is probably the correct way to calculate the image size.
                                        // However, to preserve backward compatibility the default calculation cannot
                                        // be changed.
      mfNoTextClipping    = 0x00400000, // If set, the ETO_CLIPPED flag in text records is ignored.
      mfSrcCopy_Only      = 0x00800000, // If set, images which use a ROP code other than SRCCOPY are ignored. This is useful when processing Excel 2007 spool files.
      mfClipRclBounds     = 0x01000000, // If set, the graphic is drawn into a clipping path with the size of rclBounds.
                                        // This flag is useful if the graphic contains content outside of its bounding box.
      mfDisableRasterEMF  = 0x02000000, // If set, EMF files which use unsupported ROP codes are not rastered.
      // Obsolete flags -> these flags are ignored, do no longer use them!
      mfUseSpacingArray   = 0x00000020, // enabled by default -> can be disabled with mfUseTextScaling
      mfIntersectClipRect = 0x00002000  // enabled by default -> can be disabled with mfNoClippingRgn
   }

   public enum TNamedAction
   {
      naFirstPage,
      naLastPage,
      naNextPage,
      naPrevPage,
      naGoBack,
      naOpenDlg,
      naPrintDlg,
      naGeneralInfo,
      naFontsInfo,
      naSaveAs,
      naSecurityInfo,
      naFitPage,
      naFullScreen,
      naDeletePages,
      naQuit
   }

   public enum TNegativeStyle
   {
      nsMinusBlack,
      nsRed,
      nsParensBlack,
      nsParensRed
   }

   public enum TObjEvent
   { /* All actions which should be applied to an event (except OnMouseUp) must be a JavaScript action! */
      oeNoEvent,           /* Internal use only -> DO NOT USE THIS VALUE!!! */
      oeOnOpen,            /* Catalog, Pages */
      oeOnClose,           /* Pages only */
      oeOnMouseUp,         /* All fields, page link annotations, bookmarks */
      oeOnMouseEnter,      /* Form fields only */
      oeOnMouseExit,       /* Form fields only */
      oeOnMouseDown,       /* Form fields only */
      oeOnFocus,           /* Form fields only */
      oeOnBlur,            /* Form fields only */
      oeOnKeyStroke,       /* Text fields only */
      oeOnFormat,          /* Text fields only */
      oeOnCalc,            /* Text fields, combo boxes, list boxes */
      oeOnValidate,        /* All form fields, except buttons  */
      oeOnPageVisible,     /* PDF 1.5 -> Form fields only */
      oeOnPageInVisible,   /* PDF 1.5 -> Form fields only */
      oeOnPageOpen,        /* PDF 1.5 -> Form fields only */
      oeOnPageClose,       /* PDF 1.5 -> Form fields only */
      oeOnBeforeClosing,   /* PDF 1.4 -> Catalog only */
      oeOnBeforeSaving,    /* PDF 1.4 -> Catalog only */
      oeOnAfterSaving,     /* PDF 1.4 -> Catalog only */
      oeOnBeforePrinting,  /* PDF 1.4 -> Catalog only */
      oeOnAfterPrinting    /* PDF 1.4 -> Catalog only */
   }

   public enum TObjType
   {
      otAction,
      otAnnotation,
      otBookmark,
      otCatalog,    // PDF 1.4
      otField,
      otPage,
      otPageLink
   }

   public enum TOCObject
   {
      ooAnnotation,
      ooField,
      ooImage,
      ooTemplate
   }

   public enum TOCGIntent
   {
      oiDesign = 2,
      oiView   = 4, // Default
      oiAll    = 8,
      oiEmpty  = 16 // Internal
   }

   public enum TOCVisibility
   {
      ovAllOff,
      ovAllOn,
      ovAnyOff,
      ovAnyOn,
      ovNotSet  // Internal
   }

   public enum TOrigin
   {
      orDownLeft,
      orTopLeft
   }

   public enum TPageBoundary
   {
      pbArtBox,
      pbBleedBox,
      pbCropBox,
      pbTrimBox,
      pbMediaBox
   }

   public enum TPageCoord
   {
      pcBottomUp,
      pcTopDown
   }

   public enum TPageFormat
   {
      pfDIN_A3,
      pfDIN_A4,
      pfDIN_A5,
      pfDIN_B4,
      pfDIN_B5,
      pfDIN_B6,
      pfDIN_C3,
      pfDIN_C4,
      pfDIN_C5,
      pfDIN_C6,
      pfDIN_C65,
      pfDIN_DL,
      pfDIN_E4,
      pfDIN_E5,
      pfDIN_E6,
      pfDIN_E65,
      pfDIN_M5,
      pfDIN_M65,
      pfUS_Legal,
      pfUS_Letter
   }

   public enum TPageLabelFormat
   {
      plfDecimalArabic,    // 1,2,3,4...
      plfUppercaseRoman,   // I,II,III,IV...
      plfLowercaseRoman,   // i,ii,iii,iv...
      plfUppercaseLetters, // A,B,C,D...
      plfLowercaseLetters, // a,b,c,d...
      plfNone
   }

   public enum TPageLayout
   {
      plSinglePage,
      plOneColumn,
      plTwoColumnLeft,
      plTwoColumnRight,
      plTwoPageLeft,    // PDF 1.5
      plTwoPageRight,   // PDF 1.5
      plDefault         // Use viewer's default settings
   }

   public enum TPageMode
   {
      pmUseNone,
      pmUseOutlines,
      pmUseThumbs,
      pmFullScreen,
      pmUseOC,         // PDF 1.5
      pmUseAttachments // PDF 1.6
   }

   public enum TParseFlags
   {
      pfNone              = 0x00000000,
      pfDecomprAllImages  = 0x00000002, // This flag causes that all image formats will be decompressed
                                        // with the exception of JBIG2 compressed images. If this flag is
                                        // not set, images which are already stored in a valid image file
                                        // format are returned as is. This is the case for Gray and RGB JPEG
                                        // images and for JPEG2000 images. If you want to extract the images
                                        // of a PDF file this flag should NOT be set!

      pfNoJPXDecode       = 0x00000004, // Meaningful only if the flag pfDecomprAllImages is set. If set,
                                        // JPEG2000 images are returned as is so that you can use your own
                                        // library to decompress such images since the the entire JPEG2000
                                        // codec is still marked as experimental. If we find an alternative
                                        // to the currently used Jasper library then we will replace it
                                        // immediatly with another one...

      // The following flags are ignored if an image is not decompressed. Note that only one flag must be set
      // at time. If no color space conversion flag is set images are returned in their native or alternate
      // device color space. Note that these flags do not convert colors of vector graphics and so on.
      // Use the function ConvColor() to convert colors of the graphics state into a device space.
      pfDitherImagesToBW  = 0x00000008, // Floyd-Steinberg dithering.
      pfConvImagesToGray  = 0x00000010,
      pfConvImagesToRGB   = 0x00000020,
      pfConvImagesToCMYK  = 0x00000040,
      pfImageInfoOnly     = 0x00000080  // If set, images are not decompressed. This flag is useful if you want
                                        // to enumerate the images of a PDF file or if you want to determine how
                                        // many images are stored in it.
                                        // Note that images can be compressed with multiple filters. The member
                                        // Filter of the structure TPDFImage contains only the last filter with
                                        // which the image was compressed. There is no indication whether multiple
                                        // decode filters are required to decompress the image buffer. So, it
                                        // makes no sense to set this flag if you want to try to decompress the
                                        // image buffer manually with your own decode filters.
   }

   public enum TPathFillMode
   {
      fmFillNoClose,
      fmStrokeNoClose,
      fmFillStrokeNoClose,
      fmFill,
      fmStroke,
      fmFillStroke,
      fmFillEvOdd,
      fmFillStrokeEvOdd,
      fmFillEvOddNoClose,
      fmFillStrokeEvOddNoClose,
      fmNoFill,
      fmClose  // Close the path, but do not fill or stroke it. More vertices are required or stroke the path in a separate step.
   }

   public enum TPatternType
   {
      ptColored,
      ptUnColored,
      ptShadingPattern
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFAnnotation
   {
      public TAnnotType      Type;
      public bool            Deleted;
      public TPDFRect        BBox;
      public double          BorderWidth;
      public uint            BorderColor;
      public TBorderStyle    BorderStyle;
      public uint            BackColor;
      public int             Handle;
      public String          Author;
      public String          Content;
      public String          Name;
      public String          Subject;
      public int             PageNum;
      public THighlightMode  HighlightMode;
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFAnnotationEx
   {
      public TAnnotType      Type;
      public bool            Deleted;
      public TPDFRect        BBox;
      public float           BorderWidth;
      public uint            BorderColor;
      public TBorderStyle    BorderStyle;
      public uint            BackColor;
      public int             Handle;
      public String          Author;
      public String          Content;
      public String          Name;
      public String          Subject;
      public int             PageNum;
      public THighlightMode  HighlightMode;
      public int             DestPage;
      public TPDFRect        DestPos;
      public TDestType       DestType;
      public String          DestFile;     // File link or web link annotations
      public int             Icon;         // The Icon type depends on the annotation type. If the annotation type is atText then the Icon
                                           // is of type TAnnotIcon. If the annotation type is atFileAttach then it is of type
                                           // TFileAttachIcon. If the annotation type is atStamp then the Icon is the stamp type (TRubberStamp).
                                           // For any other annotation type this value is not set (-1).
      public String          StampName;    // Set only, if Icon == rsUserDefined
      public uint            AnnotFlags;   // See TAnnotFlags for available flags

      public String          CreateDate;   // Creation Date -> Optional
      public String          ModDate;      // Modification Date -> Optional
      public bool            Grouped;      // (Reply type) Meaningful only if Parent != -1 and Type != atPopUp. If true,
                                           // the annotation is part of an annotation group. Properties like Content, CreateDate,
                                           // ModDate, BackColor, Subject, and Open must be taken from the parent annotation.
      public bool            Open;         // Meaningful only for annotations which have a corresponding PopUp annotation.
      public int             Parent;       // Parent annotation handle of a PopUp Annotation or the parent annotation if
                                           // this annotation represents a state of a base annotation. In this case,
                                           // the annotation type is always atText and only the following members should
                                           // be considered:
                                           //    State      // The current state
                                           //    StateModel // Marked, Review, and so on
                                           //    CreateDate // Creation Date
                                           //    ModDate    // Modification Date
                                           //    Author     // The user who has set the state
                                           //    Content    // Not displayed in Adobe's Acrobat...
                                           //    Subject    // Not displayed in Adobe's Acrobat...
                                           // The PopUp annotation of a text annotation which represent an Annotation State
                                           // must be ignored.
      public int             PopUp;        // Handle of the corresponding PopUp annotation if any.
      public String          State;        // The state of the annotation.
      public String          StateModel;   // The state model (Marked, Review, and so on).
      public int             EmbeddedFile; // FileAttach annotations only. A handle of an embedded file -> GetEmbeddedFile().
      public String          Subtype;      // Set only, if Type == atUnknownAnnot
   }

   // The tags have the same meaning as the corresponding HTML tags.
   // See PDF Reference for further information.
   public enum TPDFBaseTag
   {
      btArt,
      btArtifact,
      btAnnot,
      btBibEntry,      // BibEntry -> Bibliography entry
      btBlockQuote,
      btCaption,
      btCode,
      btDiv,
      btDocument,
      btFigure,
      btForm,
      btFormula,
      btH,
      btH1,
      btH2,
      btH3,
      btH4,
      btH5,
      btH6,
      btIndex,
      btLink,
      btList,          // L
      btListElem,      // LI
      btListText,      // LBody
      btNote,
      btP,
      btPart,
      btQuote,
      btReference,
      btSection,       // Sect
      btSpan,
      btTable,
      btTableDataCell, // TD
      btTableHeader,   // TH
      btTableRow,      // TR
      btTOC,
      btTOCEntry       // TOCI
   }

   /*
      Notice:
      When using a bidirectional 8 bit code page the bidi algorithm is applied by default in Left to Right mode
      also if the bidi mode is set to bmNone (default). This mode produces identical results in comparison to
      applications like Edit or WordPad.

      The Right to Left mode is available in applications which use Microsoft's Uniscribe, e.g. BabelPad. This
      mode works very well with the Reference Bidi Algorithm which is used by DynaPDF.

      However, Uniscribe's Left to Right mode produces different results in comparison to the Reference Bidi
      Algorithm. Because the bidi algorithm that is used in Uniscribe is not published it is practically
      impossible to get the same result in Left to Right mode without using this library.
   */
   public enum TPDFBidiMode
   {
      bmLeftToRight, // Apply the bidi algorithm on Unicode strings in Left to Right layout.
      bmRightToLeft, // Apply the bidi algorithm on Unicode strings in Right to Left layout.
      bmNone         // Default -> not apply the bidi algorithm
   }

   public struct TPDFChoiceValue
   {
      public String ExpValue;
      public String Value;
      public bool   Selected;
   }

   /*
      The structure contains several duplicate fields because CMap files contain usually a DSC comment
      section which provides Postscript specific initialization code. With exception of DSCResName the
      strings in the DSC section should not differ from their CMap counterparts. The Identity mapping
      of a character collection should contain the DSC comment "%%BeginResource: CMap (Identity)".
      Otherwise the string should be set to the CMap name.
   */
   [StructLayout(LayoutKind.Sequential, Pack = 0)]
   public struct TPDFCMap
   {
      public int    StructSize;     // Must be set to sizeof(TPDFCMap) before calling GetCMap()!
      [MarshalAs(UnmanagedType.LPStr)]
      public String BaseCMap;       // If set, this base cmap is required when loading the cmap.
      public uint   CIDCount;       // 0 if not set.
      [MarshalAs(UnmanagedType.LPStr)]
      public String CMapName;       // The CMap name.
      public uint   CMapType;       // Should be 1!
      public float  CMapVersion;    // The CMap version.
      [MarshalAs(UnmanagedType.LPStr)]
      public String DSCBaseCMap;    // DSC comment.
      public float  DSCCMapVersion; // DSC comment.
      [MarshalAs(UnmanagedType.LPStr)]
      public String DSCResName;     // DSC comment. If the CMap uses an Identity mapping this string should be set to Identity.
      [MarshalAs(UnmanagedType.LPStr)]
      public String DSCTitle;       // DSC comment -> DSC CMap name + Registry + Ordering + Supplement, e.g. "GB-EUC-H Adobe GB1 0"
      [MarshalAs(UnmanagedType.LPStr)]
      public String FileNameA;      // The file name and CMap name should be identical!
      [MarshalAs(UnmanagedType.LPWStr)]
      public String FileNameW;      // The file name and CMap name should be identical!
      [MarshalAs(UnmanagedType.LPStr)]
      public String FilePathA;      // The Ansi string is set if the Ansi version of SetCMapDir() was used.
      [MarshalAs(UnmanagedType.LPWStr)]
      public String FilePathW;      // The Unicode string is set if the Unicode version of SetCMapDir() was used.
      [MarshalAs(UnmanagedType.LPStr)]
      public String Ordering;       // CIDSystemInfo -> The Character Collection, e.g. Japan1.
      [MarshalAs(UnmanagedType.LPStr)]
      public String Registry;       // CIDSystemInfo -> The registrant of the Character Collection is usually Adobe.
      public uint   Supplement;     // CIDSystemInfo -> The Supplement number should be supported in the used PDF Version.
      public uint   WritingMode;    // 0 == Horizontal, 1 == Vertical
   }

   public enum TPDFColorSpace
   {
      csDeviceRGB,
      csDeviceCMYK,
      csDeviceGray
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFColorSpaceObj
   {
      public TExtColorSpace Type;
      public TExtColorSpace Alternate;        // Alternate color space or base space of an Indexed or Pattern color space.
      public IntPtr         IAlternate;       // Only set if the color space contains an alternate or base color space -> GetColorSpaceObjEx().
      public IntPtr         Buffer;           // Contains either an ICC profile or the color table of an Indexed color space.
      public int            BufSize;          // Buffer length in bytes.
      public float[]        BlackPoint;       // CIE blackpoint. If set, the array contains exactly 3 values.
      public float[]        WhitePoint;       // CIE whitepoint. If set, the array contains exactly 3 values.
      public float[]        Gamma;            // If set, one value per component.
      public float[]        Range;            // min/max for each component or for the .a and .b components of a Lab color space.
      public float[]        Matrix;           // XYZ matrix. If set, the array contains exactly 9 values.
      public int            NumInComponents;  // Number of input components.
      public int            NumOutComponents; // Number of output components.
      public int            NumColors;        // HiVal + 1 as specified in the color space. Indexed color space only.
      public String[]       Colorants;        // Colorant names (Separation, DeviceN, and NChannel only).
      public IntPtr         Metadata;         // Optional XMP metadata stream -> ICCBased only.
      public int            MetadataSize;     // Metadata length in bytes.
      public IntPtr         IFunction;        // Pointer to function object -> Separation, DeviceN, and NChannel only.
      public IntPtr         IAttributes;      // Optional attributes of DeviceN or NChannel color spaces -> GetNChannelAttributes().
      public IntPtr         IColorSpaceObj;   // Pointer of the corresponding color space object
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TDeviceNAttributes
   {
      public IntPtr   IProcessColorSpace;  // Pointer to process color space or NULL -> GetColorSpaceEx().
      public String[] ProcessColorants;    // Colorant names
      public IntPtr[] Separations;         // Optional pointers to Separation color spaces -> GetColorSpaceEx().
      public IntPtr   IMixingHints;        // Optional pointer to mixing hints. There is no API function at this time to access mixing hints.
   };

   public enum TPDFDateTime
   {
      dfMM_D,
      dfM_D_YY,
      dfMM_DD_YY,
      dfMM_YY,
      dfD_MMM,
      dfD_MMM_YY,
      dfDD_MMM_YY,
      dfYY_MM_DD,
      dfMMM_YY,
      dfMMMM_YY,
      dfMMM_D_YYYY,
      dfMMMM_D_YYYY,
      dfM_D_YY_H_MM_TT,
      dfM_D_YY_HH_MM,
      /* time formats */
      df24HR_MM,
      df12HR_MM,
      df24HR_MM_SS,
      df12HR_MM_SS
   }

   // It is not possible to set all available graphic state parameters with DynaPDF, such as black generation functions,
   // halftones and so on. The structure will be extended when further objects are supported.
   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFExtGState
   {
      public int              AutoStrokeAdjust; // PDF_MAX_INT if not set
      public TBlendMode       BlendMode;        // Default bmNotSet
      public float            FlatnessTol;      // -1.0 if not set
      public int              OverPrintFill;    // PDF_MAX_INT if not set
      public int              OverPrintStroke;  // PDF_MAX_INT if not set
      public int              OverPrintMode;    // PDF_MAX_INT if not set
      public TRenderingIntent RenderingIntent;  // riNone if not set
      public float            SmoothnessTol;    // -1.0 if not set
      public float            FillAlpha;        // -1.0 if not set
      public float            StrokeAlpha;      // -1.0 if not set
      public int              AlphaIsShape;     // PDF_MAX_INT if not set
      public int              TextKnockout;     // PDF_MAX_INT if not set
      public IntPtr           Reserved1;
      public IntPtr           Reserved2;
      public IntPtr           Reserved3;
      public IntPtr           Reserved4;
      public IntPtr           Reserved5;
      public IntPtr           Reserved6;
      public IntPtr           Reserved7;
      public IntPtr           Reserved8;
      public IntPtr           Reserved9;
   }

   // Extended graphics state dictionary
   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFExtGState2
   {
      public lbool            AlphaIsShape;       // PDF_MAX_INT if not set
      public lbool            AutoStrokeAdjust;   // PDF_MAX_INT if not set
      public IntPtr           BlackGen;           // Function handle or NULL -> GetFunction()
      public IntPtr           BlackGen2;          // Function handle or NULL -> GetFunction()
      public IntPtr           BlendMode;          // Array of blend modes
      public int              BlendModeCount;     // Number of blend modes
      public float            FillAlpha;          // -1.0 if not set
      public float            FlatnessTol;        // -1.0 if not set
      public IntPtr           Halftone;           // Halftone handle or NULL -> GetHalftoneDict()
      public lbool            OverPrintFill;      // PDF_MAX_INT if not set
      public lbool            OverPrintStroke;    // PDF_MAX_INT if not set
      public lbool            OverPrintMode;      // PDF_MAX_INT if not set
      public TRenderingIntent RenderingIntent;    // riNone if not set
      public float            SmoothnessTol;      // -1.0 if not set
      public IntPtr           SoftMask;           // Soft mask handle or NULL
      public float            StrokeAlpha;        // -1.0 if not set
      public lbool            TextKnockout;       // PDF_MAX_INT if not set
      public IntPtr           TransferFunc;       // Array of functions -> GetFunction()
      public int              TransferFuncCount;  // Number of transfer functions
      public IntPtr           TransferFunc2;      // Array of functions -> GetFunction()
      public int              TransferFunc2Count; // Number of transfer functions
      public IntPtr           UnderColorRem;      // Function handle or NULL -> GetFunction()
      public IntPtr           UnderColorRem2;     // Function handle or NULL -> GetFunction()
      public IntPtr           Reserved1;
      public IntPtr           Reserved2;
      public IntPtr           Reserved3;
      public IntPtr           Reserved4;
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFField
   {
      public TFieldType     FieldType;
      public bool           Deleted;
      public TPDFRect       BBox;
      public int            Handle;
      public String         FieldName;
      public TPDFColorSpace BackCS;
      public TPDFColorSpace TextCS;
      public uint           BackColor;
      public uint           BorderColor;
      public uint           TextColor;
      public bool           Checked;
      public int            Parent;
      public int            KidCount;
      public String         Font;
      public double         FontSize;
      public String         Value;
      public String         ToolTip;
   }

   public struct TPDFFieldEx
   {
      public bool            Deleted;       // If true, the field was marked as deleted by DeleteField()
      public TPDFRect        BBox;          // Bounding box of the field in bottom-up coordinates
      public TFieldType      FieldType;     // Field type
      public TFieldType      GroupType;     // If GroupType != FieldType the field is a terminal field of a field group
      public int             Handle;        // Field handle
      public uint            BackColor;     // Background color
      public TExtColorSpace  BackColorSP;   // Color space of the background color
      public uint            BorderColor;   // Border color
      public TExtColorSpace  BorderColorSP; // Color space of the border color
      public TBorderStyle    BorderStyle;   // Border style
      public float           BorderWidth;   // Border width
      public float           CharSpacing;   // Text fields only
      public bool            Checked;       // Check boxes only
      public int             CheckBoxChar;  // ZapfDingbats character that is used to display the on state
      public TCheckBoxState  DefState;      // Check boxes only
      public String          DefValue;      // Optional default value
      public IntPtr          IEditFont;     // Pointer to default editing font
      public String          EditFont;      // Postscript name of the editing font
      public int             ExpValCount;   // Combo and list boxes only. The values can be accessed with GetFieldExpValueEx()
      public String          ExpValue;      // Check boxes only
      public TFieldFlags     FieldFlags;    // Field flags
      public IntPtr          IFieldFont;    // Pointer to the font that is used by the field
      public String          FieldFont;     // Postscript name of the font
      public double          FontSize;      // Font size. 0.0 means auto font size
      public String          FieldName;     // Note that children of a field group or radio button have no name
      public THighlightMode  HighlightMode; // Highlight mode
      public bool            IsCalcField;   // If true, the OnCalc event of the field is connected with a JavaScript action
      public String          MapName;       // Optional unique mapping name of the field
      public int             MaxLen;        // Text fields only -> zero means not restricted
      public IntPtr[]        Kids;          // Array of child fields -> GetFieldEx2()
      public IntPtr          Parent;        // Pointer to parent field or NULL
      public int             PageNum;       // Page on which the field is used or -1
      public int             Rotate;        // Rotation angle in degrees
      public TTextAlign      TextAlign;     // Text fields only
      public uint            TextColor;     // Text color
      public TExtColorSpace  TextColorSP;   // Color space of the field's text
      public float           TextScaling;   // Text fields only
      public String          ToolTip;       // Optional tool tip
      public String          UniqueName;    // Optional unique name (NM key)
      public String          Value;         // Field value
      public float           WordSpacing;   // Text fields only
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFFileSpec
   {
      public IntPtr Buffer;      // Buffer of an embedded file
      public int    BufSize;     // Buffer size
      public bool   Compressed;  // Should be false if Decompress was true in the GetEmbeddedFile() call, otherwise usually true.
                                 // DynaPDF decompresses Flate encoded streams only. Other filters can occur but this is very unusual.
      public IntPtr ColItem;     // If != IntPtr.Zero the embedded file contains a collection item with user defined data. This entry
                                 // can occur in PDF Collections only (PDF 1.7). See "PDF Collections" in the help file for further
                                 // information.
      public String Name;        // Name of the file specification in the name tree. This value is always present.
      public String FileName;    // File name as 7 bit ASCII string.
      public bool   IsURL;       // If true, FileName contains a URL.
      public String UF;          // PDF 1.7. Same as FileName but Unicode is allowed.
      public String Desc;        // Description
      public int    FileSize;    // Size of the decompressed stream or zero if not known. Note: this is either the Size key of
                                 // the Params dictionary if present or the DL key in the file stream. Whether this value is
                                 // correct depends on the file creator! The parameter is definitely correct if the file was
                                 // decompressed.
      public String MIMEType;    // MIME media type name as defined in Internet RFC 2046.
      public String CreateDate;  // Creation date as string. See help file "The standard date format".
      public String ModDate;     // Modification date as string. See help file "The standard date format".
      public byte[] CheckSum;    // 16 byte MD5 digest.
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFFontObj
   {
      public float     Ascent;        // Ascent
      public String    BaseFont;      // PostScript Name or Family Name
      public float     CapHeight;     // Cap height
      public float     Descent;       // Descent
      public char[]    Encoding;      // Unicode mapping 0..255 -> not set if a CID font is selected
      public int       FirstChar;     // First char
      public uint      Flags;         // Font flags -> font descriptor
      public String    FontFamily;    // Optional Font Family (Family Name)
      public String    FontName;      // Font name -> font descriptor
      public TFontType FontType;      // If ftType0 the font is a CID font. The Encoding is not set in this case.
      public float     ItalicAngle;   // Italic angle
      public int       LastChar;      // Last char
      public float     SpaceWidth;    // Space width in font units. A default value is set if the font contains no space character.
      public float[]   Widths;        // Glyph widths -> 0..WidthsCount -1
      public float     XHeight;       // x-height
      public float     DefWidth;      // Default character widths -> CID fonts only
      public IntPtr    FontFile;      // Font file buffer -> only imported fonts are returned.
      public int       Length1;       // Length of the clear text portion of the Type1 font, or the length of the entire font program if FontType != ffType1.
      public int       Length2;       // Length of the encrypted portion of the Type1 font program (Type1 fonts only).
      public int       Length3;       // Length of the fixed-content portion of the Type1 font program or zero if not present.
      public TFontFileSubtype FontFileType;  // See description in the help file for further information.
   };

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFImage
   {
      public IntPtr              Buffer;           // Each scanline is aligned to a full byte.
      public int                 BufSize;          // The size of the image buffer in bytes.
      public TDecodeFilter       Filter;           // Required decode filter if the image is compressed.
                                                   // Possible values are dfDCTDecode (JPEG), dfJPXDecode (JPEG2000),
                                                   // and dfJBIG2Decode. Other filters are always removed by DynaPDF since
                                                   // a conversion to a native file format is then always required.
      public TDecodeFilter       OrgFilter;        // The image was compressed with this filter in the PDF file. This info is
                                                   // useful to determine which compression filter should be used when creating
                                                   // a new image file from the image buffer.
      public IntPtr              JBIG2Globals;     // Optional global page 0 segment (dfJBIG2Decode filter only).
      public int                 JBIG2GlobalsSize; // The size of the bit stream in bytes.
      public int                 BitsPerPixel;     // Bit depth of the image buffer. Possible values are 1, 2, 4, 8, 24, 32, and 64.
      public TExtColorSpace      ColorSpace;       // The color space refers either to the image buffer or to the color table if set.
                                                   // Note that 1 bit images can occur with and without a color table.
      public int                 NumComponents;    // The number of components stored in the image buffer.
      public lbool               MinIsWhite;       // If true, the colors of 1 bit images are reversed.
      public IntPtr              IColorSpaceObj;   // Pointer to the original color space.
      public IntPtr              ColorTable;       // The color table or NULL.
      public int                 ColorCount;       // The number of colors in the color table.
      public int                 Width;            // Image width in pixel.
      public int                 Height;           // Image height in pixel.
      public int                 ScanLineLength;   // The length of a scanline in bytes.
      public lbool               InlineImage;      // If true, the image is an inline image.
      public lbool               Interpolate;      // If true, image interpolation should be performed.
      public lbool               Transparent;      // The meaning is different depending on the bit depth and whether a color
                                                   // table is available. If the image is a 1 bit image and if no color table is available,
                                                   // black pixels must be drawn with the current fill color.
                                                   // If the image contains a color table ColorMask contains the range of indexes
                                                   // in the form min/max index which appears transparent. If no color table is
                                                   // present ColorMask contains the transparent ranges in the form min/max for
                                                   // each component.
      public IntPtr              ColorMask;        // The array contains ranges in the form min/max (2 values per component) for each
                                                   // component before decoding.
      public IntPtr              IMaskImage;       // If set, a 1 bit image is used as a transparency mask. Call GetImage() to decode the image.
      public IntPtr              ISoftMask;        // If set, a grayscale image is used as alpha channel. Call GetImage() to decode the image.
      public float[]             Decode;           // If set, samples must be decoded. The array contains 2 * NumComponents values.
                                                   // The decode array is never set if the image is returned decompressed since
                                                   // it is already applied during decompression.
      public TRenderingIntent    Intent;           // Default riNone.
      public int                 SMaskInData;      // JPXDecode only, PDF_MAX_INT if not set. See PDF Reference for further information.
      public IntPtr              OCG;              // Pointer to Optional Content Group if any.
      public IntPtr              Metadata;         // Optional XML Metadata stream.
      public int                 MetadataSize;     // Length of Metadata in bytes.
      public IntPtr              ObjectPtr;        // Internal pointer to the image class.
      public float               ResolutionX;      // Image resolution on the x-axis.
      public float               ResolutionY;      // Image resolution on the y-axis.
      // Reserved fields for future extensions
      int Reserved1;
      int Reserved2;
      int Reserved3;
      int Reserved4;
      int Reserved5;
      int Reserved6;
      int Reserved7;
      int Reserved8;
      int Reserved9;
      int Reserved10;
      int Reserved11;
      int Reserved12;
      int Reserved13;
      int Reserved14;
      int Reserved15;
      int Reserved16;
      int Reserved17;
      int Reserved18;
   }

   // Named Destination
   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFNamedDest
   {
      public String    Name;
      public int       DestPage;
      public TPDFRect  DestPos;
      public TDestType DestType;
   };


   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFPageLabel
   {
      public int              StartRange;   // Number of the first page in the range. If no further label follows, the last
                                            // page in the range is pdfGetPageCount(). The first page is denoted by 1.
      public TPageLabelFormat Format;       // Number format to be used.
      public int              FirstPageNum; // First page number to be displayed in the page label. Subsequent pages are
                                            // numbered sequentially from this value.
      public String           Prefix;       // Optional prefix
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFPrintSettings
   {
      public TDuplexMode   DuplexMode;
      public int           NumCopies;         // -1 means not set. Values larger than 5 are ignored in viewer applications.
      public int           PickTrayByPDFSize; // -1 means not set. 0 == false, 1 == true.
      public int[]         PrintRanges;       // If set, the array contains PrintRangesCount * 2 values. Each pair consists
                                              // of the first and last pages in the sub-range. The first page in the PDF file
                                              // is denoted by 0.
      public TPrintScaling PrintScaling;      // dpmNone means not set.
   };

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFRect
   {
      public double Left;
      public double Bottom;
      public double Right;
      public double Top;
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFSigParms
   {
      public int           PKCS7ObjLen; // The maximum length of the signed PKCS#7 object
      public THashType     HashType;    // If set to htDetached the bytes ranges of the PDF file will be returned.
      public byte[]        Range1;      // Out -> Contains either the hash or the first byte range to create a detached signature
      public byte[]        Range2;      // Out -> Set only if HashType == htDetached
      public String        ContactInfo; // Optional, e.g. an email address
      public String        Location;    // Optional location of the signer
      public String        Reason;      // Optional reason why the file was signed
      public String        Signer;      // Optional, the issuer of the certificate takes precedence
      public bool          Encrypt;     // If true, the file will be encrypted
      // These members will be ignored if Encrypt is set to false
      public String        OpenPwd;     // Open password (Ansi String)
      public String        OwnerPwd;    // Owner password to change the security settings (Ansi String)
      public TKeyLen       KeyLen;      // Key length to be used to encrypt the file
      public TRestrictions Restrict;    // What should be restricted?
   };

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFStack
   {
      public TCTM           ctm;          // Pre-multiplied global transformation matrix
      public TCTM           tm;           // Pre-multiplied text transformation matrix
      public double         x;            // Unused -> always 0.0
      public double         y;            // Unused -> always 0.0
      public double         FontSize;     // Font size
      public double         CharSP;       // Character spacing
      public double         WordSP;       // Word spacing
      public double         HScale;       // Horizontal text scaling
      public double         TextRise;     // Text rise -> always 0.0 because it is already included in the text transformation matrix
      public double         Leading;      // Leading
      public double         LineWidth;    // Line width
      public TDrawMode      DrawMode;     // Text draw mode
      public TPDFColorSpace FillCS;       // Fill color space
      public TPDFColorSpace StrokeCS;     // Stroke color space
      public uint           FillColor;    // Fill color
      public uint           StrokeColor;  // Stroke color
      public IntPtr         BaseObject;   // Internal
      public lbool          CIDFont;      // If true, ReplacePageText() can only be used to delete a string
      public IntPtr         Text;         // Raw text without kerning space
      public int            TextLen;      // Raw text length
      public IntPtr         RawKern;      // Raw kerning array (type TTextArrayA[])
      public IntPtr         Kerning;      // Already translated Unicode kerning array (type TTextArrayW[])
      public int            KerningCount; // Number of kerning records
      public float          TextWidth;    // The width of the entire text record measured in text space
      public IntPtr         IFont;        // Font object used to print the string -> fntGetFont() can be used to return the font properties
      public lbool          Embedded;     // If true, the font is embedded
      public float          SpaceWidth;   // Measured in text space
      /*
         These members can be modified after the structure has been initialized with InitStack().
         If the destination color space should be DeviceCMYK initialize FillColor and StrokeColor
         with PDF_CMYK(0,0,0,255); which represents black.
      */
      public lbool                ConvColors;   // If set to true (default), all colors are converted to the specified destination color space
      public TPDFColorSpace       DestSpace;    // Destination color space -> default == csDeviceRGB

      // This member can be used in combination with ReplacePageText() to preserve a number
      // of kerning records from deletion. All records above this value will be deleted.
      // Take a look into the file examples/util/pdf_edit_text.cs to determine how this member
      // can be used.
      public int                  DeleteKerningAt;
      public uint                 FontFlags;
      // ------------------------------- Reserved fields -------------------------------
      private int                  Reserved1;
      private int                  Reserved2;
      private int                  Reserved3;
      private int                  Reserved4;
      private int                  Reserved5;
      private int                  Reserved6;
      private int                  Reserved7;
      private int                  Reserved8;
      private int                  Reserved9;
      private int                  Reserved10;
      private int                  Reserved11;
      private IntPtr               ContentPtr;
   }

   public enum TPDFVersion
   {
      pvPDF_1_0,
      pvPDF_1_1,
      pvPDF_1_2,
      pvPDF_1_3,
      pvPDF_1_4,
      pvPDF_1_5,
      pvPDF_1_6,
      pvPDF_1_7,
      pvPDF_1_8,     // reserved for future use
      pvPDF_1_9,     // reserved for future use
      pvPDFX1a_2001, // PDF/X-1a:2001
      pvPDFX1a_2003, // PDF/X-1a:2002
      pvPDFX3_2002,  // PDF/X-3:2002
      pvPDFX3_2003,  // PDF/X-3:2003
      pvPDFA_2005    // PDF/A-1b 2005
   }

   public enum TPrintScaling
   {
      psAppDefault, // Default
      psNone
   }

   public enum TPwdType
   {
      ptOpen,
      ptOwner
   }

   public enum TRawImageFlags
   {
      rifByteAligned  = 0x1000,
      rifRGBData      = 0x2000,
      rifCMYKData     = 0x4000
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TRectL
   {
      public int Left;
      public int Top;
      public int Right;
      public int Bottom;
   }

   public enum TRenderingIntent
   {
      riAbsoluteColorimetric,
      riPerceptual,
      riRelativeColorimetric,
      riSaturation,
      riNone // Internal
   }

   public enum TRestrictions
   {
      rsDenyNothing      = 0x00000000,
      rsDenyAll          = 0x00000F3C,
      rsPrint            = 0x00000004,
      rsModify           = 0x00000008,
      rsCopyObj          = 0x00000010,
      rsAddObj           = 0x00000020,
      /* 128/256 bit encryption only -> these flags are ignored if 40 bit encryption is used */
      rsFillInFormFields = 0x00000100,
      rsExtractObj       = 0x00000200,
      rsAssemble         = 0x00000400,
      rsPrintHighRes     = 0x00000800,
      rsExlMetadata      = 0x00001000, // PDF 1.5 Exclude metadata streams -> 128/256 bit encryption bit only.
      rsEmbFilesOnly     = 0x00002000  // PDF 1.6 Encrypt embedded files only -> Requires AES encryption.
   }

   public enum TRubberStamp
   {
     rsApproved,
     rsAsIs,
     rsConfidential,
     rsDepartmental,
     rsDraft,
     rsExperimental,
     rsExpired,
     rsFinal,
     rsForComment,
     rsForPublicRelease,
     rsNotApproved,
     rsNotForPublicRelease,
     rsSold,
     rsTopSecred,
     rsUserDefined
   }

   public enum TShadingType
   {
      stUnknown, // cannot occur -> internal use
      stFunctionBased,
      stAxial,
      stRadial,
      stFreeFormGouraud,
      stLatticeFormGouraud,
      stCoonsPatch,
      stTensorProduct
   }

   public enum TStdPattern
   {
      spHorizontal, /* ----- */
      spVertical,   /* ||||| */
      spRDiagonal,  /* \\\\\ */
      spLDiagonal,  /* ///// */
      spCross,      /* +++++ */
      spDiaCross    /* xxxxx */
   }

   public enum TSubmitFlags
   {
      sfNone             = 0x00000000,
      sfExlude           = 0x00000001, // if set, the fields in a submit form action are excluded if any
      sfInclNoValFields  = 0x00000002,
      sfHTML             = 0x00000004,
      sfGetMethod        = 0x00000008,
      sfSubmCoords       = 0x00000010,
      sfXML              = 0x00000024,
      sfInclAppSaves     = 0x00000040,
      sfInclAnnots       = 0x00000080,
      sfPDF              = 0x00000100,
      sfCanonicalFormat  = 0x00000200,
      sfExlNonUserAnnots = 0x00000400,
      sfExlFKey          = 0x00000800,
      sfEmbedForm        = 0x00002000 // PDF 1.5 embed the entire form into a file stream inside the FDF file
   }

   public enum TTabOrder
   {
      toRow,
      toColumn,
      toStructure,
      toNone
   }

   public enum TTextAlign
   {
      taLeft      = 0,
      taCenter    = 1,
      taRight     = 2,
      taJustify   = 3,
      taPlainText = 0x10000000 // If this flag is set alignment and command tags are interpreted as plain text.
                               // See WriteFText() in the help file for further information.
   }

   public enum TTilingType
   {
      ttConstSpacing,
      ttNoDistortion,
      ttFastConstSpacing
   }

   public enum TViewerPreference
   {
      vpUseNone                = 0x00000000,
      vpHideToolBar            = 0x00000001,
      vpHideMenuBar            = 0x00000002,
      vpHideWindowUI           = 0x00000004,
      vpFitWindow              = 0x00000008,
      vpCenterWindow           = 0x00000010,
      vpDisplayDocTitle        = 0x00000020,
      vpNonFullScrPageMode     = 0x00000040,
      vpDirection              = 0x00000080,
      vpViewArea               = 0x00000100,
      vpViewClip               = 0x00000200,
      vpPrintArea              = 0x00000400,
      vpPrintClip              = 0x00000800
   }

   public enum TViewPrefAddVal
   {
      avNone                   = 0x00000000,
      avNonFullScrUseNone      = 0x00000001,
      avNonFullScrUseOutlines  = 0x00000002,
      avNonFullScrUseThumbs    = 0x00000004,
      avNonFullScrUseOC        = 0x00000400, // PDF 1.6
      avDirectionL2R           = 0x00000008,
      avDirectionR2L           = 0x00000010,
      avViewPrintArtBox        = 0x00000020,
      avViewPrintBleedBox      = 0x00000040,
      avViewPrintCropBox       = 0x00000080,
      avViewPrintMediaBox      = 0x00000100,
      avViewPrintTrimBox       = 0x00000200,

      // Use these masks to determine which value is defined.
      AV_NON_FULL_SRC_MASK     = 0x00000005,
      AV_DIRECTION_MASK        = 0x00000018,
      AV_VIEW_PRINT_MASK       = 0x000003E0
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TTextRecordA
   {
      public float  Advance; // Negative values move the cursor to right, positive to left. The value is measured in text space!
      public IntPtr Text;    // Raw text (not null-terminated)
      public int    Length;  // Raw text length in bytes
   }

   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TTextRecordW
   {
      public float  Advance; // Negative values move the cursor to right, positive to left. The value is measured in text space!
      public IntPtr Text;    // Already translated Unicode string (not null-terminated)
      public int    Length;  // Length in characters
      public float  Width;   // String width measured in text space
   };

   /* ---------------------------------------------- Parser Interface ---------------------------------------------- */

   public delegate int  TApplyPattern(IntPtr Data, IntPtr Object, TPatternType Type, IntPtr Pattern);
   public delegate int  TBeginPattern(IntPtr Data, lbool Fill, int Handle, TPatternType Type, ref TPDFRect BBox, IntPtr Matrix /* type TCTM */, IntPtr XStep /* type double */, IntPtr YStep /* type double */);
   public delegate int  TBeginTemplate(IntPtr Data, IntPtr Object, int Handle, ref TPDFRect BBox, IntPtr Matrix /* type TCTM */);
   public delegate int  TBezierTo1(IntPtr Data, IntPtr Object, double x1, double y1, double x3, double y3);
   public delegate int  TBezierTo2(IntPtr Data, IntPtr Object, double x2, double y2, double x3, double y3);
   public delegate int  TBezierTo3(IntPtr Data, IntPtr Object, double x1, double y1, double x2, double y2, double x3, double y3);
   public delegate int  TClipPath(IntPtr Data, IntPtr Object, lbool EvenOdd, TPathFillMode Mode);
   public delegate int  TClosePath(IntPtr Data, IntPtr Object, TPathFillMode Mode);
   public delegate int  TDrawShading(IntPtr Data, IntPtr Object, TShadingType Type, IntPtr Shading);
   public delegate void TEndPattern(IntPtr Data);
   public delegate void TEndTemplate(IntPtr Data);
   public delegate int  TInsertImage(IntPtr Data, ref TPDFImage Image);
   public delegate int  TLineTo(IntPtr Data, IntPtr Object, double x, double y);
   public delegate int  TMoveTo(IntPtr Data, IntPtr Object, double x ,double y);
   public delegate void TMulMatrix(IntPtr Data, IntPtr Object, ref TCTM Matrix);
   public delegate int  TRectangle(IntPtr Data, IntPtr Object, double x, double y, double w, double h);
   public delegate int  TRestoreGraphicState(IntPtr Data);
   public delegate int  TSaveGraphicState(IntPtr Data);
   public delegate void TSetCharSpacing(IntPtr Data, IntPtr Object, double Value);
   public delegate void TSetExtGState(IntPtr Data, IntPtr Object, ref TPDFExtGState2 GS);
   public delegate void TSetFillColor(IntPtr Data, IntPtr Object, IntPtr Color, int NumComps, TExtColorSpace CS, IntPtr IColorSpace);
   public delegate void TSetFont(IntPtr Data, IntPtr Object, TFontType Type, lbool Embedded, IntPtr FontName, TFStyle Style, double FontSize, IntPtr IFont);
   public delegate void TSetLeading(IntPtr Data, IntPtr Object, double Value);
   public delegate void TSetLineCapStyle(IntPtr Data, IntPtr Object, TLineCapStyle Style);
   public delegate void TSetLineDashPattern(IntPtr Data, IntPtr Object, IntPtr Dash, int NumValues, int Phase);
   public delegate void TSetLineJoinStyle(IntPtr Data, IntPtr Object, TLineJoinStyle Style);
   public delegate void TSetLineWidth(IntPtr Data, IntPtr Object, double Value);
   public delegate void TSetMiterLimit(IntPtr Data, IntPtr Object, double Value);
   public delegate void TSetStrokeColor(IntPtr Data, IntPtr Object, IntPtr Color, int NumComps, TExtColorSpace CS, IntPtr IColorSpace);
   public delegate void TSetTextDrawMode(IntPtr Data, IntPtr Object, TDrawMode Mode);
   public delegate void TSetTextScale(IntPtr Data, IntPtr Object, double Value);
   public delegate void TSetWordSpacing(IntPtr Data, IntPtr Object, double Value);
   // It is not required to declare the marshalling attributes in the real callback function.
   public delegate int  TShowTextArrayA(IntPtr Data, IntPtr Object, ref TCTM Matrix, [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 4)] TTextRecordA[] Source, int Count, double Width);
   public delegate int  TShowTextArrayW(IntPtr Data, [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 4)] TTextRecordA[] Source, ref TCTM Matrix, [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 4)]TTextRecordW[] Kerning, int Count, double Width, lbool Decoded);
   //public delegate int TShowTextArrayA(IntPtr Data, IntPtr Object, ref TCTM Matrix, IntPtr Source, int Count, double Width);
   //public delegate int TShowTextArrayW(IntPtr Data, IntPtr Source, ref TCTM M, IntPtr Kerning, int Count, double Width, lbool Decoded);
   
   // Unnecessary callback functions can be set to null -> already default when creating a new instance of the structure.
   [StructLayout(LayoutKind.Sequential, Pack=0)]
   public struct TPDFParseInterface
   {
      public TApplyPattern          ApplyPattern;
      public TBeginPattern          BeginPattern;
      public TBeginTemplate         BeginTemplate;
      public TBezierTo1             BezierTo1;
      public TBezierTo2             BezierTo2;
      public TBezierTo3             BezierTo3;
      public TClipPath              ClipPath;
      public TClosePath             ClosePath;
      public TDrawShading           DrawShading;
      public TEndPattern            EndPattern;
      public TEndTemplate           EndTemplate;
      public TLineTo                LineTo;
      public TMoveTo                MoveTo;
      public TMulMatrix             MulMatrix;
      public TRectangle             Rectangle;
      public TRestoreGraphicState   RestoreGraphicState;
      public TSaveGraphicState      SaveGraphicState;
      public TSetCharSpacing        SetCharSpacing;
      public TSetExtGState          SetExtGState;
      public TSetFillColor          SetFillColor;
      public TSetFont               SetFont;
      public TSetLeading            SetLeading;
      public TSetLineCapStyle       SetLineCapStyle;
      public TSetLineDashPattern    SetLineDashPattern;
      public TSetLineJoinStyle      SetLineJoinStyle;
      public TSetLineWidth          SetLineWidth;
      public TSetMiterLimit         SetMiterLimit;
      public TSetStrokeColor        SetStrokeColor;
      public TSetTextDrawMode       SetTextDrawMode;
      public TSetTextScale          SetTextScale;
      public TSetWordSpacing        SetWordSpacing;
      private IntPtr                Reserved001;
      private IntPtr                Reserved002;
      public TShowTextArrayW        ShowTextArrayW;
      public TInsertImage           InsertImage;
      public TShowTextArrayA        ShowTextArrayA;
      private IntPtr                 Reserved01;
      private IntPtr                 Reserved02;
      private IntPtr                 Reserved03;
      private IntPtr                 Reserved04;
      private IntPtr                 Reserved05;
      private IntPtr                 Reserved06;
      private IntPtr                 Reserved07;
      private IntPtr                 Reserved08;
      private IntPtr                 Reserved09;
      private IntPtr                 Reserved10;
      private IntPtr                 Reserved11;
      private IntPtr                 Reserved12;
      private IntPtr                 Reserved13;
      private IntPtr                 Reserved14;
      private IntPtr                 Reserved15;
      private IntPtr                 Reserved16;
      private IntPtr                 Reserved17;
      private IntPtr                 Reserved18;
      private IntPtr                 Reserved19;
      private IntPtr                 Reserved20;
      private IntPtr                 Reserved21;
      private IntPtr                 Reserved22;
      private IntPtr                 Reserved23;
      private IntPtr                 Reserved24;
      private IntPtr                 Reserved25;
      private IntPtr                 Reserved26;
      private IntPtr                 Reserved27;
   }

   /* ---------------------------------------------- Callback functions ---------------------------------------------- */

   // Data is always a user defined pointer which is passed unchanged to the callback funtion.
   // However, this pointer is usually set to IntPtr.Zero because no pointers are available in
   // .Net languages. However, because anything is declared in classes there is normally no need
   // to use it. The pointer Data is required in other programming languages like C/C++ or Delphi.

   // Note: The data type lbool is an alias for int.
   public delegate int TErrorProc(IntPtr Data, int ErrCode, IntPtr ErrMessage, int ErrType);
   public delegate int TEnumFontProc(IntPtr Data, IntPtr FamilyName, IntPtr PostScriptName, int Style);
   public delegate int TEnumFontProc2(IntPtr Data, IntPtr PDFFont, TFontType Type, IntPtr BaseFont, IntPtr FontName, lbool Embedded, lbool IsFormFont, uint Flags);
   public delegate int TEnumFontProcEx(IntPtr Data, IntPtr FamilyName, IntPtr PostScriptName, int Style, TFontBaseType BaseType, lbool Embeddable, IntPtr FilePath);
   public delegate int TOnFontNotFoundProc(IntPtr Data, IntPtr PDFFont, IntPtr FontName, int Style, int StdFontIndex, lbool IsSymbolFont);
   public delegate int TOnPageBreakProc(IntPtr Data, double LastPosX, double LastPosY, lbool PageBreak);
   public delegate int TOnReplaceICCProfile(IntPtr Data, TICCProfileType Type, int ColorSpace);

   /* ------------------------------- Callback functions to control a progress bar ---------------------------------- */

   public enum TProgType
   {
      ptImportPage, // Start page import
      ptWritePage   // Start writing a page to file or buffer
   }

   // TInitProgressProc is called before TProgressProc is called the first time.
   public delegate void TInitProgressProc(IntPtr Data, TProgType ProgType, int MaxCount);
   public delegate int  TProgressProc(IntPtr Data, int ActivePage);

   internal class CPDF : IDisposable
   {
      public const int  PDF_MAX_INT = 0x7FFFFFFF;
      // Basic RGB colors
      public const uint PDF_AQUA    = 0x00FFFF00;
      public const uint PDF_BLACK   = 0x00000000;
      public const uint PDF_BLUE    = 0x00FF0000;
      public const uint PDF_CREAM   = 0x00F0FBFF;
      public const uint PDF_DKGRAY  = 0x00808080;
      public const uint PDF_FUCHSIA = 0x00FF00FF;
      public const uint PDF_GRAY    = 0x00808080;
      public const uint PDF_GREEN   = 0x00008000;
      public const uint PDF_LIME    = 0x0000FF00;
      public const uint PDF_LTGRAY  = 0x00C0C0C0;
      public const uint PDF_MAROON  = 0x00000080;
      public const uint PDF_MEDGRAY = 0x00A4A0A0;
      public const uint PDF_MOGREEN = 0x00C0DCC0;
      public const uint PDF_NAVY    = 0x00800000;
      public const uint PDF_OLIVE   = 0x00008080;
      public const uint PDF_PURPLE  = 0x00800080;
      public const uint PDF_RED     = 0x000000FF;
      public const uint PDF_SILVER  = 0x00C0C0C0;
      public const uint PDF_SKYBLUE = 0x00F0CAA6;
      public const uint PDF_WHITE   = 0x00FFFFFF;
      public const uint PDF_TEAL    = 0x00808000;
      public const uint PDF_YELLOW  = 0x0000FFFF;
      // This value can be used for form fields, annotations and standard patterns to determine that the specific color is not set.
      // For example, when a form field should have a transparent background, set the field background color
      // to NO_COLOR. The background appears then transparent.
      public const uint NO_COLOR    = 0xFFFFFFF1;

      public CPDF()
      {
         m_Instance = pdfNewPDF();
      }

      ~CPDF()
      {
         DoDispose();
      }

      public void Dispose()
      {
         // do dispose code
         DoDispose();

         //suppress finalize
         GC.SuppressFinalize(this);
      }

      private void DoDispose()
      {
         //prevent this code from being called when it is already being called in another thread.
         lock(this)
         {
            if (!IntPtr.Zero.Equals(m_Instance))
            {
               pdfDeletePDF(m_Instance);
               m_Instance = IntPtr.Zero;
            }
         }
      }

      public bool AddActionToObj(TObjType ObjType, TObjEvent Event, int ActHandle, int ObjHandle)
      {
         return pdfAddActionToObj(m_Instance, ObjType, Event, ActHandle, ObjHandle) != 0;
      }

      public int AddArticle(double PosX, double PosY, double Width, double Height)
      {
         return pdfAddArticle(m_Instance, PosX, PosY, Width, Height);
      }

      public int AddBookmarkA(String Title, int Parent, int DestPage, bool Open)
      {
         return pdfAddBookmarkA(m_Instance, Title, Parent, DestPage, Convert.ToInt32(Open));
      }

      public int AddBookmarkW(String Title, int Parent, int DestPage, bool Open)
      {
         return pdfAddBookmarkW(m_Instance, Title, Parent, DestPage, Convert.ToInt32(Open));
      }

      public int AddBookmarkExA(String Title, int Parent, int NamedDest, bool Open)
      {
         return pdfAddBookmarkExA(m_Instance, Title, Parent, NamedDest, Convert.ToInt32(Open));
      }

      public int AddBookmarkExW(String Title, int Parent, int NamedDest, bool Open)
      {
         return pdfAddBookmarkExW(m_Instance, Title, Parent, NamedDest, Convert.ToInt32(Open));
      }

      public bool AddButtonImage(int BtnHandle, TButtonState State, String Caption, String ImgFile)
      {
         return pdfAddButtonImage(m_Instance, BtnHandle, State, Caption, ImgFile) != 0;
      }

      public bool AddButtonImageEx(int BtnHandle, TButtonState State, String Caption, IntPtr hBitmap)
      {
         return pdfAddButtonImageEx(m_Instance, BtnHandle, State, Caption, hBitmap) != 0;
      }

      public bool AddContinueTextA(String AText)
      {
         return pdfAddContinueTextA(m_Instance, AText) != 0;
      }

      public bool AddContinueTextW(String AText)
      {
         return pdfAddContinueTextW(m_Instance, AText) != 0;
      }

      public bool AddDeviceNProcessColorants(int DeviceNCS, String[] Colorants, int NumColorants, TExtColorSpace ProcessCS, int Handle)
      {
         return pdfAddDeviceNProcessColorants(m_Instance, DeviceNCS, Colorants, NumColorants, ProcessCS, Handle) != 0;
      }

      public bool AddDeviceNSeparations(int DeviceNCS, String[] Colorants, int[] SeparationCS, int NumColorants)
      {
         return pdfAddDeviceNSeparations(m_Instance, DeviceNCS, Colorants, SeparationCS, NumColorants) != 0;
      }

      public bool AddFieldToFormAction(int Action, int Field, bool Include)
      {
         return pdfAddFieldToFormAction(m_Instance, Action, Field, Convert.ToInt32(Include)) != 0;
      }

      public bool AddFieldToHideAction(int HideAct, int Field)
      {
         return pdfAddFieldToHideAction(m_Instance, HideAct, Field) != 0;
      }

      public bool AddFileCommentA(String AText)
      {
         return pdfAddFileCommentA(m_Instance, AText) != 0;
      }

      public bool AddFileCommentW(String AText)
      {
         return pdfAddFileCommentW(m_Instance, AText) != 0;
      }

      public int AddFontSearchPathA(String APath, bool Recursive)
      {
         return pdfAddFontSearchPathA(m_Instance, APath, Convert.ToInt32(Recursive));
      }

      public int AddFontSearchPathW(String APath, bool Recursive)
      {
         return pdfAddFontSearchPathW(m_Instance, APath, Convert.ToInt32(Recursive));
      }

      public bool AddImage(TCompressionFilter Filter, TImageConversionFlags Flags, TPDFImage Image)
      {
         return pdfAddImage(m_Instance, Filter, Flags, ref Image) != 0;
      }

      public int AddJavaScriptA(String Name, String Script)
      {
         return pdfAddJavaScriptA(m_Instance, Name, Script);
      }

      public int AddJavaScriptW(String Name, String Script)
      {
         return pdfAddJavaScriptW(m_Instance, Name, Script);
      }

      public bool AddObjectToLayer(int OCG, TOCObject ObjType, int Handle)
      {
         return pdfAddObjectToLayer(m_Instance, OCG, ObjType, Handle) != 0;
      }

      public int AddPageLabelA(int StartRange, TPageLabelFormat Format, String Prefix, int AddNum)
      {
         return pdfAddPageLabelA(m_Instance, StartRange, Format, Prefix, AddNum);
      }

      public int AddPageLabelW(int StartRange, TPageLabelFormat Format, String Prefix, int AddNum)
      {
         return pdfAddPageLabelW(m_Instance, StartRange, Format, Prefix, AddNum);
      }

      public int AddRenderingIntentA(String ICCFile)
      {
         return pdfAddRenderingIntentA(m_Instance, ICCFile);
      }

      public int AddRenderingIntentW(String ICCFile)
      {
         return pdfAddRenderingIntentW(m_Instance, ICCFile);
      }

      public int AddRenderingIntentEx(byte[] Buffer)
      {
         return pdfAddRenderingIntentEx(m_Instance, Buffer, Buffer.Length);
      }

      public bool AddValToChoiceFieldA(int Field, String ExpValue, String Value, bool Selected)
      {
         return pdfAddValToChoiceFieldA(m_Instance, Field, ExpValue, Value, Convert.ToInt32(Selected)) != 0;
      }

      public bool AddValToChoiceFieldW(int Field, String ExpValue, String Value, bool Selected)
      {
         return pdfAddValToChoiceFieldW(m_Instance, Field, ExpValue, Value, Convert.ToInt32(Selected)) != 0;
      }

      public bool Append()
      {
         return pdfAppend(m_Instance) != 0;
      }

      public bool ApplyPattern(int PattHandle, TColorMode ColorMode, uint Color)
      {
         return pdfApplyPattern(m_Instance, PattHandle, ColorMode, Color) != 0;
      }

      public bool ApplyShading(int ShadHandle)
      {
         return pdfApplyShading(m_Instance, ShadHandle) != 0;
      }

      public int AttachFileA(String FilePath, String Description, bool Compress)
      {
         return pdfAttachFileA(m_Instance, FilePath, Description, Convert.ToInt32(Compress));
      }

      public int AttachFileW(String FilePath, String Description, bool Compress)
      {
         return pdfAttachFileW(m_Instance, FilePath, Description, Convert.ToInt32(Compress));
      }

      public int AttachFileExA(byte[] Buffer, int BufSize, String FileName, String Description, bool Compress)
      {
         return pdfAttachFileExA(m_Instance, Buffer, BufSize, FileName, Description, Convert.ToInt32(Compress));
      }

      public int AttachFileExW(byte[] Buffer, int BufSize, String FileName, String Description, bool Compress)
      {
         return pdfAttachFileExW(m_Instance, Buffer, BufSize, FileName, Description, Convert.ToInt32(Compress));
      }

      public bool AutoTemplate(int Templ, double PosX, double PosY, double Width, double Height)
      {
         return pdfAutoTemplate(m_Instance, Templ, PosX, PosY, Width, Height) != 0;
      }

      public bool BeginClipPath()
      {
         return pdfBeginClipPath(m_Instance) != 0;
      }

      public bool BeginContinueText(double PosX, double PosY)
      {
         return pdfBeginContinueText(m_Instance, PosX, PosY) != 0;
      }

      public bool BeginLayer(int OCG)
      {
         return pdfBeginLayer(m_Instance, OCG) != 0;
      }

      public int BeginPattern(TPatternType PatternType, TTilingType TilingType, double Width, double Height)
      {
         return pdfBeginPattern(m_Instance, PatternType, TilingType, Width, Height);
      }

      public int BeginTemplate(double Width, double Height)
      {
         return pdfBeginTemplate(m_Instance, Width, Height);
      }

      public int BeginTemplateEx(ref TPDFRect BBox, ref TCTM Matrix)
      {
         return pdfBeginTemplateEx(m_Instance, ref BBox, ref Matrix);
      }

      public bool Bezier_1_2_3(double x1, double y1, double x2, double y2, double x3, double y3)
      {
         return pdfBezier_1_2_3(m_Instance, x1, y1, x2, y2, x3, y3) != 0;
      }

      public bool Bezier_1_3(double x1, double y1, double x3, double y3)
      {
         return pdfBezier_1_3(m_Instance, x1, y1, x3, y3) != 0;
      }

      public bool Bezier_2_3(double x2, double y2, double x3, double y3)
      {
         return pdfBezier_2_3(m_Instance, x2, y2, x3, y3) != 0;
      }

      public bool BuildFamilyNameAndStyle(IntPtr IFont, System.Text.StringBuilder Name, ref TFStyle Style)
      {
         if (Name.Capacity < 128) Name.Capacity = 128;
         Name.Length = 0;
         return fntBuildFamilyNameAndStyle(IFont, Name, ref Style) != 0;
      }

      public double CalcWidthHeight(double OrgWidth, double OrgHeight, double ScaledWidth, double ScaledHeight)
      {
         return pdfCalcWidthHeight(m_Instance, OrgWidth, OrgHeight, ScaledWidth, ScaledHeight);
      }

      public bool ChangeAnnotPos(int Handle, double PosX, double PosY, double Width, double Height)
      {
         return pdfChangeAnnotPos(m_Instance, Handle, PosX, PosY, Width, Height) != 0;
      }

      public bool ChangeAnnotNameA(int Handle, String Name)
      {
         return pdfChangeAnnotNameA(m_Instance, Handle, Name) != 0;
      }

      public bool ChangeAnnotNameW(int Handle, String Name)
      {
         return pdfChangeAnnotNameW(m_Instance, Handle, Name) != 0;
      }

      public bool ChangeBookmarkA(int ABmk, String Title, int Parent, int DestPage, bool Open)
      {
         return pdfChangeBookmarkA(m_Instance, ABmk, Title, Parent, DestPage, Convert.ToInt32(Open)) != 0;
      }

      public bool ChangeBookmarkW(int ABmk, String Title, int Parent, int DestPage, bool Open)
      {
         return pdfChangeBookmarkW(m_Instance, ABmk, Title, Parent, DestPage, Convert.ToInt32(Open)) != 0;
      }

      public bool ChangeFont(int Handle)
      {
         return pdfChangeFont(m_Instance, Handle) != 0;
      }

      public bool ChangeFontSize(double Size)
      {
         return pdfChangeFontSize(m_Instance, Size) != 0;
      }

      public bool ChangeFontStyle(TFStyle Style)
      {
         return pdfChangeFontStyle(m_Instance, Style) != 0;
      }

      public bool ChangeFontStyleEx(TFStyle Style)
      {
         return pdfChangeFontStyleEx(m_Instance, Style) != 0;
      }

      public bool ChangeJavaScriptA(int Handle, String NewScript)
      {
         return pdfChangeJavaScriptA(m_Instance, Handle, NewScript) != 0;
      }

      public bool ChangeJavaScriptW(int Handle, String NewScript)
      {
         return pdfChangeJavaScriptW(m_Instance, Handle, NewScript) != 0;
      }

      public bool ChangeJavaScriptActionA(int Handle, String NewScript)
      {
         return pdfChangeJavaScriptActionA(m_Instance, Handle, NewScript) != 0;
      }

      public bool ChangeJavaScriptActionW(int Handle, String NewScript)
      {
         return pdfChangeJavaScriptActionW(m_Instance, Handle, NewScript) != 0;
      }

      public bool ChangeJavaScriptNameA(int Handle, String Name)
      {
         return pdfChangeJavaScriptNameA(m_Instance, Handle, Name) != 0;
      }

      public bool ChangeJavaScriptNameW(int Handle, String Name)
      {
         return pdfChangeJavaScriptNameW(m_Instance, Handle, Name) != 0;
      }

      public bool ChangeLinkAnnot(int Handle, String URL)
      {
         return pdfChangeLinkAnnot(m_Instance, Handle, URL) != 0;
      }

      public int CheckAndRepairFonts()
      {
         return pdfCheckAndRepairFonts(m_Instance);
      }

      public bool CheckCollection()
      {
         return pdfCheckCollection(m_Instance) != 0;
      }

      public int CheckConformance(TConformanceType Type, TCheckOptions Options, IntPtr UserData, TOnFontNotFoundProc OnFontNotFound, TOnReplaceICCProfile OnReplaceICCProfile)
      {
         m_FontNotFoundProc    = OnFontNotFound;
         m_OnReplaceICCProfile = OnReplaceICCProfile;
         return pdfCheckConformance(m_Instance, Type, Options, UserData, m_FontNotFoundProc, m_OnReplaceICCProfile);
      }

      public int CheckFieldNames()
      {
         return pdfCheckFieldNames(m_Instance);
      }

      public bool ClearAutoTemplates()
      {
         return pdfClearAutoTemplates(m_Instance) != 0;
      }

      public bool ClearHostFonts()
      {
         return pdfClearHostFonts(m_Instance) != 0;
      }

      public bool ClipPath(TClippingMode ClipMode, TPathFillMode FillMode)
      {
         return pdfClipPath(m_Instance, ClipMode, FillMode) != 0;
      }

      public bool CloseAndSignFile(String CertFile, String Password, String Reason, String Location)
      {
         return pdfCloseAndSignFile(m_Instance, CertFile, Password, Reason, Location) != 0;
      }

      public bool CloseAndSignFileEx(String OpenPwd, String OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict, String CertFile, String Password, String Reason, String Location)
      {
         return pdfCloseAndSignFileEx(m_Instance, OpenPwd, OwnerPwd, KeyLen, Restrict, CertFile, Password, Reason, Location) != 0;
      }

      public bool CloseAndSignFileExt(TPDFSigParms SigParms)
      {
         TPDFSigParms_I p = new TPDFSigParms_I();
         p.StructSize   = Marshal.SizeOf(p);
         p.ContactInfoW = SigParms.ContactInfo;
         p.Encrypt      = Convert.ToInt32(SigParms.Encrypt);
         p.HashType     = SigParms.HashType;
         p.KeyLen       = SigParms.KeyLen;
         p.LocationW    = SigParms.Location;
         p.OpenPwd      = SigParms.OpenPwd;
         p.OwnerPwd     = SigParms.OwnerPwd;
         p.PKCS7ObjLen  = SigParms.PKCS7ObjLen;
         p.ReasonW      = SigParms.Reason;
         p.SignerW      = SigParms.Signer;
         p.Restrict     = SigParms.Restrict;
         if (pdfCloseAndSignFileExt(m_Instance, ref p) == 0) return false;
         SigParms.Range1 = new byte[p.Range1Len];
         Marshal.Copy(p.Range1, SigParms.Range1, 0, p.Range1Len);
         if (p.Range2Len > 0)
         {
            SigParms.Range2 = new byte[p.Range2Len];
            Marshal.Copy(p.Range2, SigParms.Range2, 0, p.Range2Len);
         }else
         {
            SigParms.Range2 = null;
         }
         return true;
      }

      public bool CloseFile()
      {
         return pdfCloseFile(m_Instance) != 0;
      }

      public bool CloseFileEx(String OpenPwd, String OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict)
      {
         return pdfCloseFileEx(m_Instance, OpenPwd, OwnerPwd, KeyLen, Restrict) != 0;
      }

      public bool CloseImage()
      {
         return pdfCloseImage(m_Instance) != 0;
      }

      public bool CloseImportFile()
      {
         return pdfCloseImportFile(m_Instance) != 0;
      }

      public bool ClosePath(TPathFillMode FillMode)
      {
         return pdfClosePath(m_Instance, FillMode) != 0;
      }

      public bool CloseTag()
      {
         return pdfCloseTag(m_Instance) != 0;
      }

      public uint CMYK(byte c, byte m, byte y, byte k)
      {
         return (uint)(k|(y<<8)|(m<<16)|(c<<24));
      }

      public bool ComputeBBox(ref TPDFRect BBox, TCompBBoxFlags Flags)
      {
         return pdfComputeBBox(m_Instance, ref BBox, Flags) != 0;
      }

      public uint ConvColor(IntPtr Color, int NumComps, TExtColorSpace SourceCS, IntPtr IColorSpace, TExtColorSpace DestCS)
      {
         return pdfConvColor(Color, NumComps, SourceCS, IColorSpace, DestCS);
      }

      public bool ConvertColors(TColorConvFlags Flags)
      {
         return pdfConvertColors(m_Instance, Flags, IntPtr.Zero) != 0;
      }

      public String ConvToUnicode(String AString, TCodepage CP)
      {
         IntPtr retval = pdfConvToUnicode(m_Instance, AString, CP);
         return ToString(retval, true);
      }

      public IntPtr[] CopyIntPtrArray(IntPtr Source, int Count)
      {
         IntPtr[] retval = new IntPtr[Count];
         pdfCopyMemIntPtr(Source, retval, Count * IntPtr.Size);
         return retval;
      }

      public int Create3DAnnot(double PosX, double PosY, double Width, double Height, String Author, String Name, String U3DFile, String Image)
      {
         return pdfCreate3DAnnot(m_Instance, PosX, PosY, Width, Height, Author, Name, U3DFile, Image);
      }

      public bool Create3DBackground(IntPtr IView, uint BackColor)
      {
         return pdfCreate3DBackground(m_Instance, IView, BackColor) != 0;
      }

      public int Create3DGotoViewAction(int Base3DAnnot, IntPtr IView, T3DNamedAction Named)
      {
         return pdfCreate3DGotoViewAction(m_Instance, Base3DAnnot, IView, Named);
      }

      public bool Create3DProjection(IntPtr IView, T3DProjType ProjType, T3DScaleType ScaleType, double Diameter, double FOV)
      {
         return pdfCreate3DProjection(m_Instance, IView, ProjType, ScaleType, Diameter, FOV) != 0;
      }

      public IntPtr Create3DView(int Base3DAnnot, String Name, bool SetAsDefault, ref double[] Matrix, double CamDistance, T3DRenderingMode RM, T3DLightingSheme LS)
      {
         return pdfCreate3DView(m_Instance, Base3DAnnot, Name, Convert.ToInt32(SetAsDefault), ref Matrix, CamDistance, RM, LS);
      }

      public int CreateAnnotAP(int Annot)
      {
         return pdfCreateAnnotAP(m_Instance, Annot);
      }

      public int CreateArticleThreadA(String ThreadName)
      {
         return pdfCreateArticleThreadA(m_Instance, ThreadName);
      }

      public int CreateArticleThreadW(String ThreadName)
      {
         return pdfCreateArticleThreadW(m_Instance, ThreadName);
      }

      public int CreateAxialShading(double sX, double sY, double eX, double eY, double SCenter, uint SColor, uint EColor, bool Extend1, bool Extend2)
      {
         return pdfCreateAxialShading(m_Instance, sX, sY, eX, eY, SCenter, SColor, EColor, Convert.ToInt32(Extend1), Convert.ToInt32(Extend2));
      }

      public int CreateButtonA(String Name, String Caption, int Parent, double PosX, double PosY, double Width, double Height)
      {
         return pdfCreateButtonA(m_Instance, Name, Caption, Parent, PosX, PosY, Width, Height);
      }

      public int CreateButtonW(String Name, String Caption, int Parent, double PosX, double PosY, double Width, double Height)
      {
         return pdfCreateButtonW(m_Instance, Name, Caption, Parent, PosX, PosY, Width, Height);
      }

      public int CreateCheckBox(String Name, String ExpValue, bool Checked, int Parent, double PosX, double PosY, double Width, double Height)
      {
         return pdfCreateCheckBox(m_Instance, Name, ExpValue, Convert.ToInt32(Checked), Parent, PosX, PosY, Width, Height);
      }

      public int CreateCIEColorSpace(TExtColorSpace Base, float[] WhitePoint, float[] BlackPoint, float[] Gamma, float[] Matrix)
      {
         return pdfCreateCIEColorSpace(m_Instance, Base, WhitePoint, BlackPoint, Gamma, Matrix);
      }

      public bool CreateCollection(TColView View)
      {
         return pdfCreateCollection(m_Instance, View) != 0;
      }

      public int CreateCollectionFieldA(TColColumnType ColType, int Column, String Name, String Key, bool Visible, bool Editable)
      {
         return pdfCreateCollectionFieldA(m_Instance, ColType, Column, Name, Key, Convert.ToInt32(Visible), Convert.ToInt32(Editable));
      }

      public int CreateCollectionFieldW(TColColumnType ColType, int Column, String Name, String Key, bool Visible, bool Editable)
      {
         return pdfCreateCollectionFieldW(m_Instance, ColType, Column, Name, Key, Convert.ToInt32(Visible), Convert.ToInt32(Editable));
      }

      public bool CreateColItemDate(int EmbFile, String Key, long Date, String Prefix)
      {
         return pdfCreateColItemDate(m_Instance, EmbFile, Key, Date, Prefix) != 0;
      }

      public bool CreateColItemNumber(int EmbFile, String Key, double Value, String Prefix)
      {
         return pdfCreateColItemNumber(m_Instance, EmbFile, Key, Value, Prefix) != 0;
      }

      public bool CreateColItemStringA(int EmbFile, String Key, String Value, String Prefix)
      {
         return pdfCreateColItemStringA(m_Instance, EmbFile, Key, Value, Prefix) != 0;
      }

      public bool CreateColItemStringW(int EmbFile, String Key, String Value, String Prefix)
      {
         return pdfCreateColItemStringW(m_Instance, EmbFile, Key, Value, Prefix) != 0;
      }

      public int CreateComboBox(String Name, bool Sort, int Parent, double PosX, double PosY, double Width, double Height)
      {
         return pdfCreateComboBox(m_Instance, Name, Convert.ToInt32(Sort), Parent, PosX, PosY, Width, Height);
      }

      public int CreateDeviceNColorSpace(String[] Colorants, int NumColorants, String PostScriptFunc, TExtColorSpace Alternate, int Handle)
      {
         return pdfCreateDeviceNColorSpace(m_Instance, Colorants, NumColorants, PostScriptFunc, Alternate, Handle);
      }

      public int CreateExtGState(ref TPDFExtGState GS)
      {
         return pdfCreateExtGState(m_Instance, ref GS);
      }

      public int CreateGoToAction(TDestType DestType, int PageNum, double a, double b, double c, double d)
      {
         return pdfCreateGoToAction(m_Instance, DestType, PageNum, a, b, c, d);
      }

      public int CreateGoToActionEx(int NamedDest)
      {
         return pdfCreateGoToActionEx(m_Instance, NamedDest);
      }

      public int CreateGoToRAction(String FileName, int PageNum)
      {
         return pdfCreateGoToRAction(m_Instance, FileName, PageNum);
      }

      public int CreateGoToRActionExA(String FileName, String DestName, bool NewWindow)
      {
         return pdfCreateGoToRActionExA(m_Instance, FileName, DestName, Convert.ToInt32(NewWindow));
      }

      public int CreateGoToRActionExW(String FileName, String DestName, bool NewWindow)
      {
         return pdfCreateGoToRActionExW(m_Instance, FileName, DestName, Convert.ToInt32(NewWindow));
      }

      public int CreateGroupField(String Name, int Parent)
      {
         return pdfCreateGroupField(m_Instance, Name, Parent);
      }

      public int CreateHideAction(int Field, bool Hide)
      {
         return pdfCreateHideAction(m_Instance, Field, Convert.ToInt32(Hide));
      }

      public int CreateICCBasedColorSpaceA(String ICCProfile)
      {
         return pdfCreateICCBasedColorSpaceA(m_Instance, ICCProfile);
      }

      public int CreateICCBasedColorSpaceW(String ICCProfile)
      {
         return pdfCreateICCBasedColorSpaceW(m_Instance, ICCProfile);
      }

      public bool CreateImageA(String FileName, TImageFormat Format)
      {
         return pdfCreateImageA(m_Instance, FileName, Format) != 0;
      }

      public bool CreateImageW(String FileName, TImageFormat Format)
      {
         return pdfCreateImageW(m_Instance, FileName, Format) != 0;
      }

      public int CreateImpDataActionA(String DataFile)
      {
         return pdfCreateImpDataActionA(m_Instance, DataFile);
      }

      public int CreateImpDataActionW(String DataFile)
      {
         return pdfCreateImpDataActionW(m_Instance, DataFile);
      }

      public int CreateIndexedColorSpace(TExtColorSpace Base, int Handle, byte[] ColorTable, int NumColors)
      {
         return pdfCreateIndexedColorSpace(m_Instance, Base, Handle, ColorTable, NumColors);
      }

      public int CreateJSActionA(String Script)
      {
         return pdfCreateJSActionA(m_Instance, Script);
      }

      public int CreateJSActionW(String Script)
      {
         return pdfCreateJSActionW(m_Instance, Script);
      }

      public int CreateLaunchAction(TFileOP OP, String FileName, String DefDir, String Param, bool NewWindow)
      {
         return pdfCreateLaunchAction(m_Instance, OP, FileName, DefDir, Param, Convert.ToInt32(NewWindow));
      }

      public int CreateListBox(String Name, bool Sort, int Parent, double PosX, double PosY, double Width, double Height)
      {
         return pdfCreateListBox(m_Instance, Name, Convert.ToInt32(Sort), Parent, PosX, PosY, Width, Height);
      }

      public int CreateNamedAction(TNamedAction Action)
      {
         return pdfCreateNamedAction(m_Instance, Action);
      }

      public int CreateNamedDestA(String Name, int DestPage, TDestType DestType, double a, double b, double c, double d)
      {
         return pdfCreateNamedDestA(m_Instance, Name, DestPage, DestType, a, b, c, d);
      }

      public int CreateNamedDestW(String Name, int DestPage, TDestType DestType, double a, double b, double c, double d)
      {
         return pdfCreateNamedDestW(m_Instance, Name, DestPage, DestType, a, b, c, d);
      }

      public bool CreateNewPDF(String OutPDF)
      {
         return pdfCreateNewPDF(m_Instance, OutPDF) != 0;
      }

      public int CreateOCGA(String Name, bool DisplayInUI, bool Visible, TOCGIntent Intent)
      {
         return pdfCreateOCGA(m_Instance, Name, Convert.ToInt32(DisplayInUI), Convert.ToInt32(Visible), Intent);
      }

      public int CreateOCGW(String Name, bool DisplayInUI, bool Visible, TOCGIntent Intent)
      {
         return pdfCreateOCGW(m_Instance, Name, Convert.ToInt32(DisplayInUI), Convert.ToInt32(Visible), Intent);
      }

      public int CreateOCMD(TOCVisibility Visibility, int[] OCGs)
      {
         return pdfCreateOCMD(m_Instance, Visibility, OCGs, OCGs.Length);
      }

      public int CreateRadialShading(double sX, double sY, double R1, double eX, double eY, double R2, double SCenter, uint SColor, uint EColor, bool Extend1, bool Extend2)
      {
         return pdfCreateRadialShading(m_Instance, sX, sY, R1, eX, eY, R2, SCenter, SColor, EColor, Convert.ToInt32(Extend1), Convert.ToInt32(Extend2));
      }

      public int CreateRadioButton(String Name, String ExpValue, bool Checked, int Parent, double PosX, double PosY, double Width, double Height)
      {
         return pdfCreateRadioButton(m_Instance, Name, ExpValue, Convert.ToInt32(Checked), Parent, PosX, PosY, Width, Height);
      }

      public int CreateResetAction()
      {
         return pdfCreateResetAction(m_Instance);
      }

      public int CreateSeparationCS(String Colorant, TExtColorSpace Alternate, int Handle, uint Color)
      {
         return pdfCreateSeparationCS(m_Instance, Colorant, Alternate, Handle, Color);
      }

      public int CreateSetOCGStateAction(int[] On, int[] Off, int[] Toggle, bool PreserveRB)
      {
         int numOn = 0, numOff = 0, numToggle = 0;
         if (On     != null) numOn     = On.Length;
         if (Off    != null) numOff    = Off.Length;
         if (Toggle != null) numToggle = Toggle.Length;
         return pdfCreateSetOCGStateAction(m_Instance, On, numOn, Off, numOff, Toggle, numToggle, Convert.ToInt32(PreserveRB));
      }

      public int CreateSigField(String Name, int Parent, double PosX, double PosY, double Width, double Height)
      {
         return pdfCreateSigField(m_Instance, Name, Parent, PosX, PosY, Width, Height);
      }

      public int CreateSigFieldAP(int SigField)
      {
         return pdfCreateSigFieldAP(m_Instance, SigField);
      }

      public int CreateStdPattern(TStdPattern Pattern, double LineWidth, double Distance, uint LineColor, uint BackColor)
      {
         return pdfCreateStdPattern(m_Instance, Pattern, LineWidth, Distance, LineColor, BackColor);
      }

      public bool CreateStructureTree()
      {
         return pdfCreateStructureTree(m_Instance) != 0;
      }

      public int CreateSubmitAction(TSubmitFlags Flags, String URL)
      {
         return pdfCreateSubmitAction(m_Instance, Flags, URL);
      }

      public int CreateTextField(String Name, int Parent, bool Multiline, int MaxLen, double PosX, double PosY, double Width, double Height)
      {
         return pdfCreateTextField(m_Instance, Name, Parent, Convert.ToInt32(Multiline), MaxLen, PosX, PosY, Width, Height);
      }

      public int CreateURIAction(String URL)
      {
         return pdfCreateURIAction(m_Instance, URL);
      }

      public int DecryptPDFA(String FileName, TPwdType PwdType, String Password)
      {
         return pdfDecryptPDFA(m_Instance, FileName, PwdType, Password);
      }

      public int DecryptPDFW(String FileName, TPwdType PwdType, String Password)
      {
         return pdfDecryptPDFW(m_Instance, FileName, PwdType, Password);
      }

      public void DeleteAcroForm()
      {
         pdfDeleteAcroForm(m_Instance);
      }

      public bool DeleteActionFromObj(TObjType ObjType, int ActHandle, int ObjHandle)
      {
         return pdfDeleteActionFromObj(m_Instance, ObjType, ActHandle, ObjHandle) != 0;
      }

      public bool DeleteActionFromObjEx(TObjType ObjType, int ObjHandle, int ActIndex)
      {
         return pdfDeleteActionFromObjEx(m_Instance, ObjType, ObjHandle, ActIndex) != 0;
      }

      public bool DeleteAnnotation(int Handle)
      {
         return pdfDeleteAnnotation(m_Instance, Handle) != 0;
      }

      public int DeleteBookmark(int ABmk)
      {
         return pdfDeleteBookmark(m_Instance, ABmk);
      }

      public bool DeleteEmbeddedFile(int Handle)
      {
         return pdfDeleteEmbeddedFile(m_Instance, Handle) != 0;
      }

      public bool DeleteField(int Field)
      {
         return pdfDeleteField(m_Instance, Field) != 0;
      }

      public bool DeleteFieldEx(String Name)
      {
         return pdfDeleteFieldEx(m_Instance, Name) != 0;
      }

      public bool DeletePage(int PageNum)
      {
         return pdfDeletePage(m_Instance, PageNum) != 0;
      }

      public void DeleteJavaScripts(bool DelJavaScriptActions)
      {
         pdfDeleteJavaScripts(m_Instance, Convert.ToInt32(DelJavaScriptActions));
      }

      public void DeletePageLabels()
      {
         pdfDeletePageLabels(m_Instance);
      }

      public bool DeleteSeparationInfo(bool AllPages)
      {
         return pdfDeleteSeparationInfo(m_Instance, Convert.ToInt32(AllPages)) != 0;
      }

      public bool DeleteTemplate(int Handle)
      {
         return pdfDeleteTemplate(m_Instance, Handle) != 0;
      }

      public bool DeleteTemplateEx(int Index)
      {
         return pdfDeleteTemplateEx(m_Instance, Index) != 0;
      }

      public void DeleteXFAForm()
      {
         pdfDeleteXFAForm(m_Instance);
      }

      public bool DrawArc(double PosX, double PosY, double Radius, double StartAngle, double EndAngle)
      {
         return pdfDrawArc(m_Instance, PosX, PosY, Radius, StartAngle, EndAngle) != 0;
      }

      public bool DrawArcEx(double PosX, double PosY, double Width, double Height, double StartAngle, double EndAngle)
      {
         return pdfDrawArcEx(m_Instance, PosX, PosY, Width, Height, StartAngle, EndAngle) != 0;
      }

      public bool DrawChord(double PosX, double PosY, double Width, double Height, double StartAngle, double EndAngle, TPathFillMode FillMode)
      {
         return pdfDrawChord(m_Instance, PosX, PosY, Width, Height, StartAngle, EndAngle, FillMode) != 0;
      }

      public bool DrawCircle(double PosX, double PosY, double Radius, TPathFillMode FillMode)
      {
         return pdfDrawCircle(m_Instance, PosX, PosY, Radius, FillMode) != 0;
      }

      public bool DrawPie(double PosX, double PosY, double Width, double Height, double StartAngle, double EndAngle, TPathFillMode FillMode)
      {
         return pdfDrawPie(m_Instance, PosX, PosY, Width, Height, StartAngle, EndAngle, FillMode) != 0;
      }

      public bool EditPage(int PageNum)
      {
         return pdfEditPage(m_Instance, PageNum) != 0;
      }

      public bool EditTemplate(int Index)
      {
         return pdfEditTemplate(m_Instance, Index) != 0;
      }

      public bool EditTemplate2(int Handle)
      {
         return pdfEditTemplate2(m_Instance, Handle) != 0;
      }

      public bool Ellipse(double PosX, double PosY, double Width, double Height, TPathFillMode FillMode)
      {
         return pdfEllipse(m_Instance, PosX, PosY, Width, Height, FillMode) != 0;
      }

      public int EncryptPDFA(String FileName, String OpenPwd, String OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict)
      {
         return pdfEncryptPDFA(m_Instance, FileName, OpenPwd, OwnerPwd, KeyLen, Restrict);
      }

      public int EncryptPDFW(String FileName, String OpenPwd, String OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict)
      {
         return pdfEncryptPDFW(m_Instance, FileName, OpenPwd, OwnerPwd, KeyLen, Restrict);
      }

      public bool EndContinueText()
      {
         return pdfEndContinueText(m_Instance) != 0;
      }

      public bool EndLayer()
      {
         return pdfEndLayer(m_Instance) != 0;
      }

      public bool EndPage()
      {
         return pdfEndPage(m_Instance) != 0;
      }

      public bool EndPattern()
      {
         return pdfEndPattern(m_Instance) != 0;
      }

      public bool EndTemplate()
      {
         return pdfEndTemplate(m_Instance) != 0;
      }

      public int EnumDocFonts(IntPtr Data, TEnumFontProc2 EnumProc)
      {
         m_EnumFontProc2 = EnumProc;
         return pdfEnumDocFonts(m_Instance, Data, m_EnumFontProc2);
      }

      public int EnumHostFonts(IntPtr Data, TEnumFontProc EnumProc)
      {
         m_EnumFontProc = EnumProc;
         return pdfEnumHostFonts(m_Instance, Data, m_EnumFontProc);
      }

      public int EnumHostFontsEx(IntPtr Data, TEnumFontProcEx EnumProc)
      {
         m_EnumFontProcEx = EnumProc;
         return pdfEnumHostFontsEx(m_Instance, Data, m_EnumFontProcEx);
      }

      public bool ExchangeBookmarks(int Bmk1, int Bmk2)
      {
         return pdfExchangeBookmarks(m_Instance, Bmk1, Bmk2) != 0;
      }

      public bool ExchangePages(int First, int Second)
      {
         return pdfExchangePages(m_Instance, First, Second) != 0;
      }

      public int FileAttachAnnotA(double PosX, double PosY, TFileAttachIcon Icon, String Author, String Desc, String AFile, bool Compress)
      {
         return pdfFileAttachAnnotA(m_Instance, PosX, PosY, Icon, Author, Desc, AFile, Convert.ToInt32(Compress));
      }

      public int FileAttachAnnotW(double PosX, double PosY, TFileAttachIcon Icon, String Author, String Desc, String AFile, bool Compress)
      {
         return pdfFileAttachAnnotW(m_Instance, PosX, PosY, Icon, Author, Desc, AFile, Convert.ToInt32(Compress));
      }

      public int FileLinkA(double PosX, double PosY, double Width, double Height, String AFilePath)
      {
         return pdfFileLinkA(m_Instance, PosX, PosY, Width, Height, AFilePath);
      }

      public int FileLinkW(double PosX, double PosY, double Width, double Height, String AFilePath)
      {
         return pdfFileLinkW(m_Instance, PosX, PosY, Width, Height, AFilePath);
      }

      public int FindBookmarkA(int DestPage, String Title)
      {
         return pdfFindBookmarkA(m_Instance, DestPage, Title);
      }

      public int FindBookmarkW(int DestPage, String Title)
      {
         return pdfFindBookmarkW(m_Instance, DestPage, Title);
      }

      public int FindFieldA(String Name)
      {
         return pdfFindFieldA(m_Instance, Name);
      }

      public int FindFieldW(String Name)
      {
         return pdfFindFieldW(m_Instance, Name);
      }

      public bool FinishSignature(byte[] PKCS7Obj)
      {
         return pdfFinishSignature(m_Instance, PKCS7Obj, PKCS7Obj.Length) != 0;
      }

      public int FindLinkAnnot(String URL)
      {
         return pdfFindLinkAnnot(m_Instance, URL);
      }

      public int FindNextBookmark()
      {
         return pdfFindNextBookmark(m_Instance);
      }

      public int FlattenAnnots(TAnnotFlattenFlags Flags)
      {
         return pdfFlattenAnnots(m_Instance, Flags);
      }

      public bool FlattenForm()
      {
         return pdfFlattenForm(m_Instance) != 0;
      }

      public bool FlushPageContent(ref TPDFStack Stack)
      {
         return pdfFlushPageContent(m_Instance, ref Stack) != 0;
      }

      public void FreeImageBuffer()
      {
         pdfFreeImageBuffer(m_Instance);
      }

      public bool FreePDF()
      {
         return pdfFreePDF(m_Instance) != 0;
      }

      public int FreeTextAnnotA(double PosX, double PosY, double Width, double Height, String Author, String AText, TTextAlign Align)
      {
         return pdfFreeTextAnnotA(m_Instance, PosX, PosY, Width, Height, Author, AText, Align);
      }

      public int FreeTextAnnotW(double PosX, double PosY, double Width, double Height, String Author, String AText, TTextAlign Align)
      {
         return pdfFreeTextAnnotW(m_Instance, PosX, PosY, Width, Height, Author, AText, Align);
      }

      public bool FreeUniBuf()
      {
         return pdfFreeUniBuf(m_Instance) != 0;
      }

      public int GetActionCount()
      {
         return pdfGetActionCount(m_Instance);
      }

      public int GetActionType(int ActHandle)
      {
         return pdfGetActionType(m_Instance, ActHandle);
      }

      public int GetActionTypeEx(TObjType ObjType, int ObjHandle, int ActIndex)
      {
         return pdfGetActionTypeEx(m_Instance, ObjType, ObjHandle, ActIndex);
      }

      public int GetActiveFont()
      {
         return pdfGetActiveFont(m_Instance);
      }

      public int GetAllocBy()
      {
         return pdfGetAllocBy(m_Instance);
      }

      public bool GetAnnot(int Handle, ref TPDFAnnotation Annot)
      {
         TPDFAnnotation_I f = new TPDFAnnotation_I();
         if (pdfGetAnnot(m_Instance, Handle, ref f) == 0) return false;
         GetIntAnnot(f, ref Annot);
         return true;
      }

      public bool GetAnnotBBox(int Handle, ref TPDFRect BBox)
      {
         return pdfGetAnnotBBox(m_Instance, Handle, ref BBox) != 0;
      }

      public int GetAnnotCount()
      {
         return pdfGetAnnotCount(m_Instance);
      }

      public bool GetAnnotEx(int Handle, ref TPDFAnnotationEx Annot)
      {
         TPDFAnnotationEx_I f = new TPDFAnnotationEx_I();
         if (pdfGetAnnotEx(m_Instance, Handle, ref f) == 0) return false;
         GetIntAnnotEx(f, ref Annot);
         return true;
      }

      public int GetAnnotFlags()
      {
         return pdfGetAnnotFlags(m_Instance);
      }

      public String GetAnnotLink(int Handle)
      {
         IntPtr retval = pdfGetAnnotLink(m_Instance, Handle);
         return ToString(retval, IntPtr.Zero);
      }

      public int GetAnnotType(int Handle)
      {
         return pdfGetAnnotType(m_Instance, Handle);
      }

      public double GetAscent()
      {
         return pdfGetAscent(m_Instance);
      }

      public bool GetBBox(TPageBoundary Boundary, ref TPDFRect BBox)
      {
         return pdfGetBBox(m_Instance, Boundary, ref BBox) != 0;
      }

      public int GetBidiMode()
      {
         return pdfGetBidiMode(m_Instance);
      }

      public bool GetBookmark(int Handle, ref TBookmark Bmk)
      {
         TBookmark_I b = new TBookmark_I();
         if (pdfGetBookmark(m_Instance, Handle, ref b) == 0) return false;
         Bmk.Color    = b.Color;
         Bmk.DestPage = b.DestPage;
         Bmk.DestPos  = b.DestPos;
         Bmk.DestType = b.DestType;
         Bmk.Open     = b.Open != 0;
         Bmk.Parent   = b.Parent;
         Bmk.Style    = b.Style;
         Bmk.Title    = ToString(b.Title, b.TitleLen, b.Unicode != 0);
         return true;
      }

      public int GetBookmarkCount()
      {
         return pdfGetBookmarkCount(m_Instance);
      }

      public int GetBorderStyle()
      {
         return pdfGetBorderStyle(m_Instance);
      }

      public IntPtr GetBuffer(ref int BufSize)
      {
         // The function returns a pointer to the original buffer.
         return pdfGetBuffer(m_Instance, ref BufSize);
      }

      public byte[] GetBuffer()
      {
         int bufSize = 0;
         IntPtr buffer = GetBuffer(ref bufSize);
         if (IntPtr.Zero.Equals(buffer) || bufSize == 0) return null;
         byte[] retval = new byte[bufSize];
         System.Runtime.InteropServices.Marshal.Copy(buffer, retval, 0, bufSize);
         return retval;
      }

      public double GetCapHeight()
      {
         return pdfGetCapHeight(m_Instance);
      }

      public double GetCharacterSpacing()
      {
         return pdfGetCharacterSpacing(m_Instance);
      }

      public int GetCheckBoxChar()
      {
         return pdfGetCheckBoxChar(m_Instance);
      }

      public int GetCheckBoxCharEx(int Field)
      {
         return pdfGetCheckBoxCharEx(m_Instance, Field);
      }

      public int GetCheckBoxDefState(int Field)
      {
         return pdfGetCheckBoxDefState(m_Instance, Field);
      }

      public bool GetCMap(int Index, ref TPDFCMap CMap)
      {
         CMap.StructSize = Marshal.SizeOf(CMap);
         return pdfGetCMap(m_Instance, Index, ref CMap) != 0;
      }
      
      public int GetCMapCount()
      {
         return pdfGetCMapCount(m_Instance);
      }

      public int GetColorSpace()
      {
         return pdfGetColorSpace(m_Instance);
      }

      public int GetColorSpaceCount()
      {
         return pdfGetColorSpaceCount(m_Instance);
      }

      public bool GetColorSpaceObj(int Handle, ref TPDFColorSpaceObj CS)
      {
         TPDFColorSpaceObj_I ics = new TPDFColorSpaceObj_I();
         if (pdfGetColorSpaceObj(m_Instance, Handle, ref ics) == 0) return false;
         GetIntColorSpaceObj(ics, ref CS);
         return true;
      }

      public bool GetColorSpaceObjEx(IntPtr IColorSpace, ref TPDFColorSpaceObj CS)
      {
         TPDFColorSpaceObj_I ics = new TPDFColorSpaceObj_I();
         if (pdfGetColorSpaceObjEx(IColorSpace, ref ics) == 0) return false;
         GetIntColorSpaceObj(ics, ref CS);
         return true;
      }

      public int GetCompressionFilter()
      {
         return pdfGetCompressionFilter(m_Instance);
      }

      public int GetCompressionLevel()
      {
         return pdfGetCompressionLevel(m_Instance);
      }

      public int GetContent(ref byte[] Buffer)
      {
         IntPtr buf = IntPtr.Zero;
         int retval = pdfGetContent(m_Instance, ref buf);
         MemCopy(buf, ref Buffer, retval);
         return retval;
      }

      public int GetDefBitsPerPixel()
      {
         return pdfGetDefBitsPerPixel(m_Instance);
      }

      public double GetDescent()
      {
         return pdfGetDescent(m_Instance);
      }

      public bool GetDeviceNAttributes(IntPtr IAttributes, ref TDeviceNAttributes Attributes)
      {
         int i;
         TDeviceNAttributes_I attr = new TDeviceNAttributes_I();
         if (pdfGetDeviceNAttributes(IAttributes, ref attr) == 0) return false;
         Attributes.IProcessColorSpace = attr.IProcessColorSpace;
         Attributes.IMixingHints       = attr.IMixingHints;
         if (attr.ProcessColorantsCount > 0)
         {
            byte[] bytes;
            Attributes.ProcessColorants = new String[attr.ProcessColorantsCount];
            for (i = 0; i < attr.ProcessColorantsCount; i++)
            {
               bytes = System.Text.Encoding.Unicode.GetBytes(Marshal.PtrToStringAnsi(attr.ProcessColorants[i]));
               Attributes.ProcessColorants[i] = System.Text.Encoding.UTF8.GetString(bytes);
            }
         }else
         {
            Attributes.ProcessColorants = null;
         }
         if (attr.SeparationsCount > 0)
         {
            Attributes.Separations = new IntPtr[attr.SeparationsCount];
            for (i = 0; i < attr.SeparationsCount; i++)
            {
               Attributes.Separations[i] = attr.Separations[i];
            }
         }
         return true;
      }

      public int GetDocInfo(TDocumentInfo DInfo, ref String Value)
      {
         IntPtr val = IntPtr.Zero;
         int retval = pdfGetDocInfo(m_Instance, DInfo, ref val);
         Value = ToString(val, retval, true);
         return retval;
      }

      public int GetDocInfoCount()
      {
         return pdfGetDocInfoCount(m_Instance);
      }

      public int GetDocInfoEx(int Index, ref TDocumentInfo DInfo, ref String Key, ref String Value)
      {
         int retval, uni = 0;
         IntPtr key = IntPtr.Zero;
         IntPtr val = IntPtr.Zero;
         retval = pdfGetDocInfoEx(m_Instance, Index, ref DInfo, ref key, ref val, ref uni);
         if (retval > 0)
         {
            Key   = ToString(key, false);
            Value = ToString(val, retval, uni != 0);
         }
         return retval;
      }

      public int GetDrawDirection()
      {
         return pdfGetDrawDirection(m_Instance);
      }

      public String GetDynaPDFVersion()
      {
         IntPtr retval = pdfGetDynaPDFVersion();
         return ToString(retval, false);
      }

      public bool GetEmbeddedFile(int Handle, ref TPDFFileSpec FileSpec, bool Decompress)
      {
         TPDFFileSpec_I f = new TPDFFileSpec_I();
         if (pdfGetEmbeddedFile(m_Instance, Handle, ref f, Convert.ToInt32(Decompress)) == 0) return false;
         /*
            We do not copy the buffer here. This can be done if you really need it.
            Copy the buffer into a byte array if you want to write it into a .Net file stream.
            A better solution would be to write the buffer directly into a file. Take a look into
            the CBinaryStream() class in utils/binary_stream...
         */
         FileSpec.Buffer     = f.Buffer;
         FileSpec.BufSize    = f.BufSize;
         FileSpec.ColItem    = f.ColItem;
         FileSpec.Compressed = f.Compressed != 0;
         FileSpec.FileSize   = f.FileSize;
         FileSpec.IsURL      = f.IsURL != 0;
         if (!IntPtr.Zero.Equals(f.CheckSum))
            MemCopy(f.CheckSum, ref FileSpec.CheckSum, 16);
         else
            FileSpec.CheckSum = null;
         FileSpec.CreateDate = ToString(f.CreateDate, false);
         FileSpec.Desc       = ToString(f.Desc, f.DescUnicode != 0);
         FileSpec.FileName   = ToString(f.FileName, false);
         FileSpec.MIMEType   = ToString(f.MIMEType, false);
         FileSpec.ModDate    = ToString(f.ModDate, false);
         FileSpec.Name       = ToString(f.Name, f.NameUnicode != 0);
         FileSpec.UF         = ToString(f.Name, f.UFUnicode != 0);
         return true;
      }

      public int GetEmbeddedFileCount()
      {
         return pdfGetEmbeddedFileCount(m_Instance);
      }

      public double GetEMFPatternDistance()
      {
         return pdfGetEMFPatternDistance(m_Instance);
      }

      public String GetErrorMessage()
      {
         IntPtr retval = pdfGetErrorMessage(m_Instance);
         return ToString(retval, false);
      }

      public int GetErrorMode()
      {
         return pdfGetErrorMode(m_Instance);
      }

      public bool GetField(int Handle, ref TPDFField Field)
      {
         TPDFField_I f = new TPDFField_I();
         if (pdfGetField(m_Instance, Handle, ref f) == 0) return false;
         GetIntField(f, ref Field);
         return true;
      }

      public uint GetFieldBackColor()
      {
         return pdfGetFieldBackColor(m_Instance);
      }

      public uint GetFieldBorderColor()
      {
         return pdfGetFieldBorderColor(m_Instance);
      }

      public int GetFieldBorderStyle(int Field)
      {
         return pdfGetFieldBorderStyle(m_Instance, Field);
      }

      public double GetFieldBorderWidth(int Field)
      {
         return pdfGetFieldBorderWidth(m_Instance, Field);
      }

      public bool GetFieldChoiceValue(int AField, int ValIndex, ref TPDFChoiceValue Value)
      {
         TPDFChoiceValue_I v = new TPDFChoiceValue_I();
         v.StructSize = Marshal.SizeOf(v);
         if (pdfGetFieldChoiceValue(m_Instance, AField, ValIndex, ref v) == 0) return false;
         Value.ExpValue = ToString(v.ExpValueA, v.ExpValueW, v.ExpValueLen);
         Value.Value    = ToString(v.ValueA, v.ValueW, v.ValueLen);
         Value.Selected = v.Selected != 0;
         return true;
      }

      public bool GetFieldColor(int Field, TFieldColor ColorType, ref TPDFColorSpace ColorSpace, ref uint Color)
      {
         return pdfGetFieldColor(m_Instance, Field, ColorType, ref ColorSpace, ref Color) != 0;
      }

      public int GetFieldCount()
      {
         return pdfGetFieldCount(m_Instance);
      }

      public bool GetFieldEx(int Handle, ref TPDFFieldEx Field)
      {
         TPDFFieldEx_I f = new TPDFFieldEx_I();
         f.StructSize = Marshal.SizeOf(f);
         if (pdfGetFieldEx(m_Instance, Handle, ref f) == 0) return false;
         GetIntFieldEx(f, ref Field);
         return true;
      }

      public bool GetFieldEx2(IntPtr IField, ref TPDFFieldEx Field)
      {
         TPDFFieldEx_I f = new TPDFFieldEx_I();
         f.StructSize = Marshal.SizeOf(f);
         if (pdfGetFieldEx2(IField, ref f) == 0) return false;
         GetIntFieldEx(f, ref Field);
         return true;
      }

      public int GetFieldExpValCount(int Field)
      {
         return pdfGetFieldExpValCount(m_Instance, Field);
      }

      public int GetFieldExpValue(int Field, ref String Value)
      {
         IntPtr val = IntPtr.Zero;
         int retval = pdfGetFieldExpValue(m_Instance, Field, ref val);
         Value = ToString(val, retval, false);
         return retval;
      }

      public bool GetFieldExpValueEx(int Field, int ValIndex, ref String Value, ref String ExpValue, ref bool Selected)
      {
         int sel = 0;
         IntPtr expVal = IntPtr.Zero;
         IntPtr val = IntPtr.Zero;
         if (pdfGetFieldExpValueEx(m_Instance, Field, ValIndex, ref val, ref expVal, ref sel) != 0)
         {
            Value    = ToString(val, false);
            ExpValue = ToString(expVal, false);
            Selected = sel != 0;
            return true;
         }else
         {
            Value    = null;
            ExpValue = null;
            Selected = false;
         }
         return false;
      }

      public int GetFieldFlags(int Field)
      {
         return pdfGetFieldFlags(m_Instance, Field);
      }

      public int GetFieldGroupType(int Field)
      {
         return pdfGetFieldGroupType(m_Instance, Field);
      }

      public int GetFieldHighlightMode(int Field)
      {
         return pdfGetFieldHighlightMode(m_Instance, Field);
      }

      public int GetFieldIndex(int Field)
      {
         return pdfGetFieldIndex(m_Instance, Field);
      }

      public int GetFieldMapName(int Field, ref String Value)
      {
         int retval, uni = 0;
         IntPtr val = IntPtr.Zero;
         retval = pdfGetFieldMapName(m_Instance, Field, ref val, ref uni);
         Value  = ToString(val, retval, uni != 0);
         return retval;
      }

      public String GetFieldName(int Field)
      {
         IntPtr val = IntPtr.Zero;
         int retval = pdfGetFieldName(m_Instance, Field, ref val);
         int len    = pdfStrLenA(val);
         return ToString(val, retval, len != retval);
      }

      public int GetFieldOrientation(int Field)
      {
         return pdfGetFieldOrientation(m_Instance, Field);
      }

      public int GetFieldTextAlign(int Field)
      {
         return pdfGetFieldTextAlign(m_Instance, Field);
      }

      public uint GetFieldTextColor()
      {
         return pdfGetFieldTextColor(m_Instance);
      }

      public int GetFieldToolTip(int Field, ref String Value)
      {
         int retval, uni = 0;
         IntPtr val = IntPtr.Zero;
         retval = pdfGetFieldToolTip(m_Instance, Field, ref val, ref uni);
         Value  = ToString(val, retval, uni != 0);
         return retval;
      }

      public int GetFieldType(int Field)
      {
         return pdfGetFieldType(m_Instance, Field);
      }

      public uint GetFillColor()
      {
         return pdfGetFillColor(m_Instance);
      }

      public bool GetFont(IntPtr IFont, ref TPDFFontObj F)
      {
         TPDFFontObj_I font = new TPDFFontObj_I();
         if (fntGetFont(IFont, ref font) == 0) return false;
         GetIntFont(font, ref F);
         return true;
      }

      public int GetFontCount()
      {
         return pdfGetFontCount(m_Instance);
      }

      public bool GetFontEx(int Handle, ref TPDFFontObj F)
      {
         TPDFFontObj_I font = new TPDFFontObj_I();
         if (pdfGetFontEx(m_Instance, Handle, ref font) == 0) return false;
         GetIntFont(font, ref F);
         return true;
      }

      public int GetFontOrigin()
      {
         return pdfGetFontOrigin(m_Instance);
      }

      public void GetFontSearchOrder(TFontBaseType[] Order)
      {
         pdfGetFontSearchOrder(m_Instance, Order);
      }

      public int GetFontSelMode()
      {
         return pdfGetFontSelMode(m_Instance);
      }

      public int GetFontWeight()
      {
         return pdfGetFontWeight(m_Instance);
      }

      public double GetFTextHeightA(TTextAlign Align, String AText)
      {
         return pdfGetFTextHeightA(m_Instance, Align, AText);
      }

      public double GetFTextHeightW(TTextAlign Align, String AText)
      {
         return pdfGetFTextHeightW(m_Instance, Align, AText);
      }

      public double GetFTextHeightExA(double Width, TTextAlign Align, String AText)
      {
         return pdfGetFTextHeightExA(m_Instance, Width, Align, AText);
      }

      public double GetFTextHeightExW(double Width, TTextAlign Align, String AText)
      {
         return pdfGetFTextHeightExW(m_Instance, Width, Align, AText);
      }

      public uint GetGStateFlags()
      {
         return pdfGetGStateFlags(m_Instance);
      }

      public uint GetIconColor()
      {
         return pdfGetIconColor(m_Instance);
      }

      public IntPtr GetImageBuffer(ref int BufSize)
      {
         return pdfGetImageBuffer(m_Instance, ref BufSize);
      }

      public int GetImageCount(String FileName)
      {
         return pdfGetImageCount(m_Instance, FileName);
      }

      public int GetImageCountEx(byte[] Buffer)
      {
         return pdfGetImageCountEx(m_Instance, Buffer, Buffer.Length);
      }

      public int GetImageHeight(int Handle)
      {
         return pdfGetImageHeight(m_Instance, Handle);
      }

      public int GetImageWidth(int Handle)
      {
         return pdfGetImageWidth(m_Instance, Handle);
      }

      public uint GetImportFlags()
      {
         return pdfGetImportFlags(m_Instance);
      }

      public bool GetInBBox(int PageNum, TPageBoundary Boundary, ref TPDFRect BBox)
      {
         return pdfGetInBBox(m_Instance, PageNum, Boundary, ref BBox) != 0;
      }

      public int GetInDocInfo(TDocumentInfo DInfo, ref String Value)
      {
         int retval;
         IntPtr val = IntPtr.Zero;
         retval = pdfGetInDocInfo(m_Instance, DInfo, ref val);
         Value  = ToString(val, retval, true);
         return retval;
      }

      public int GetInDocInfoCount()
      {
         return pdfGetInDocInfoCount(m_Instance);
      }

      public int GetInDocInfoEx(int Index, ref TDocumentInfo DInfo, ref String Key, ref String Value)
      {
         int retval = 0, uni = 0;
         IntPtr key = IntPtr.Zero;
         IntPtr val = IntPtr.Zero;
         retval = pdfGetInDocInfoEx(m_Instance, Index, ref DInfo, ref key, ref val, ref uni);
         if (retval > 0)
         {
            Key   = ToString(key, false);
            Value = ToString(val, retval, uni != 0);
         }else
         {
            Key   = null;
            Value = null;
         }
         return retval;
      }

      public int GetInEncryptionFlags()
      {
         return pdfGetInEncryptionFlags(m_Instance);
      }

      public bool GetInIsCollection()
      {
         int retval;
         retval = pdfGetInIsCollection(m_Instance);
         if (retval < 0) return false;
         return (retval != 0);
      }

      public bool GetInIsEncrypted()
      {
         int retval;
         retval = pdfGetInIsEncrypted(m_Instance);
         if (retval < 0) return false;
         return (retval != 0);
      }

      public bool GetInIsSigned()
      {
         int retval;
         retval = pdfGetInIsSigned(m_Instance);
         if (retval < 0) return false;
         return (retval != 0);
      }

      public bool GetInIsTrapped()
      {
         int retval;
         retval = pdfGetInIsTrapped(m_Instance);
         if (retval < 0) return false;
         return (retval != 0);
      }

      public bool GetInIsXFAForm()
      {
         int retval;
         retval = pdfGetInIsXFAForm(m_Instance);
         if (retval < 0) return false;
         return (retval != 0);
      }

      public bool GetInNamedDest(int Index, ref TPDFNamedDest Dest)
      {
         TPDFNamedDest_I d = new TPDFNamedDest_I();
         if (pdfGetInNamedDest(m_Instance, Index, ref d) == 0) return false;
         Dest.Name     = ToString(d.NameA, d.NameW, d.NameLen);
         Dest.DestPage = d.DestPage;
         Dest.DestPos  = d.DestPos;
         Dest.DestType = d.DestType;
         return true;
      }

      public int GetInNamedDestCount()
      {
         return pdfGetInNamedDestCount(m_Instance);
      }

      public int GetInPageCount()
      {
         return pdfGetInPageCount(m_Instance);
      }

      public int GetInOrientation(int PageNum)
      {
         return pdfGetInOrientation(m_Instance, PageNum);
      }

      public int GetInPDFVersion()
      {
         return pdfGetInPDFVersion(m_Instance);
      }

      public bool GetInPrintSettings(ref TPDFPrintSettings Settings)
      {
         TPDFPrintSettings_I s = new TPDFPrintSettings_I();
         if (pdfGetInPrintSettings(m_Instance, ref s) == 0) return false;
         Settings.DuplexMode        = s.DuplexMode;
         Settings.NumCopies         = s.NumCopies;
         Settings.PickTrayByPDFSize = s.PickTrayByPDFSize;
         Settings.PrintScaling      = s.PrintScaling;
         if (s.PrintRangesCount > 0)
         {
            Settings.PrintRanges = new int[s.PrintRangesCount<<1];
            Marshal.Copy(s.PrintRanges, Settings.PrintRanges, 0, s.PrintRangesCount<<1);
         }
         return true;
      }

      public bool GetIsFixedPitch()
      {
         return pdfGetIsFixedPitch(m_Instance) != 0;
      }

      public double GetItalicAngle()
      {
         return pdfGetItalicAngle(m_Instance);
      }

      public bool GetJavaScript(int Handle, ref String Script)
      {
         int len = 0;
         int uni = 0;
         IntPtr retval = pdfGetJavaScript(m_Instance, Handle, ref len, ref uni);
         Script = ToString(retval, len, uni != 0);
         return (len > 0);
      }

      public bool GetJavaScriptAction(int Handle, ref String Script)
      {
         int len = 0;
         int uni = 0;
         IntPtr retval = pdfGetJavaScriptAction(m_Instance, Handle, ref len, ref uni);
         Script = ToString(retval, len, uni != 0);
         return (len > 0);
      }
      
      public bool GetJavaScriptAction2(TObjType ObjType, int ObjHandle, int ActIndex, ref String Script, ref TObjEvent Event)
      {
         int len = 0;
         int uni = 0;
         IntPtr retval = pdfGetJavaScriptAction2(m_Instance, ObjType, ObjHandle, ActIndex, ref len, ref uni, ref Event);
         Script = ToString(retval, len, uni != 0);
         return (len > 0);
      }

      public int GetJavaScriptCount()
      {
         return pdfGetJavaScriptCount(m_Instance);
      }

      public bool GetJavaScriptEx(String Name, ref String Script)
      {
         int len = 0;
         int uni = 0;
         IntPtr retval = pdfGetJavaScriptEx(m_Instance, Name, ref len, ref uni);
         Script = ToString(retval, len, uni != 0);
         return (len > 0);
      }

      public String GetJavaScriptName(int Handle)
      {
         int len = 0;
         int uni = 0;
         IntPtr retval = pdfGetJavaScriptName(m_Instance, Handle, ref len, ref uni);
         return ToString(retval, len, uni != 0);
      }

      public int GetJPEGQuality()
      {
         return pdfGetJPEGQuality(m_Instance);
      }

      public String GetLanguage()
      {
         IntPtr retval = pdfGetLanguage(m_Instance);
         return ToString(retval, false);
      }

      public double GetLastTextPosX()
      {
         return pdfGetLastTextPosX(m_Instance);
      }

      public double GetLastTextPosY()
      {
         return pdfGetLastTextPosY(m_Instance);
      }

      public double GetLeading()
      {
         return pdfGetLeading(m_Instance);
      }

      public int GetLineCapStyle()
      {
         return pdfGetLineCapStyle(m_Instance);
      }

      public int GetLineJoinStyle()
      {
         return pdfGetLineJoinStyle(m_Instance);
      }

      public double GetLineWidth()
      {
         return pdfGetLineWidth(m_Instance);
      }

      public int GetLinkHighlightMode()
      {
         return pdfGetLinkHighlightMode(m_Instance);
      }

      public bool GetLogMetafileSize(String FileName, ref TRectL R)
      {
         return pdfGetLogMetafileSize(m_Instance, FileName, ref R) != 0;
      }

      public bool GetLogMetafileSizeEx(byte[] Buffer, ref TRectL R)
      {
         return pdfGetLogMetafileSizeEx(m_Instance, Buffer, Buffer.Length, ref R) != 0;
      }

      public bool GetMatrix(ref TCTM M)
      {
         return pdfGetMatrix(m_Instance, ref M) != 0;
      }

      public int GetMaxFieldLen(int TxtField)
      {
         return pdfGetMaxFieldLen(m_Instance, TxtField);
      }

      public uint GetMetaConvFlags()
      {
         return pdfGetMetaConvFlags(m_Instance);
      }

      public uint[] GetMissingGlyphs()
      {
         int count = 0;
         IntPtr ptr = pdfGetMissingGlyphs(m_Instance, ref count);
         if (IntPtr.Zero.Equals(ptr))
            return null;
         else
         {
            uint[] retval = new uint[count];
            // Why does the marshaller not provide a copy function for uint[]?
            pdfCopyMemUInt(ptr, retval, count << 2);
            return retval;
         }
      }

      public double GetMiterLimit()
      {
         return pdfGetMiterLimit(m_Instance);
      }

      public bool GetNeedAppearance()
      {
         return pdfGetNeedAppearance(m_Instance) != 0;
      }

      public int GetObjActionCount(TObjType ObjType, int ObjHandle)
      {
         return pdfGetObjActionCount(m_Instance, ObjType, ObjHandle);
      }

      public double GetOpacity()
      {
         return pdfGetOpacity(m_Instance);
      }

      public int GetOrientation()
      {
         return pdfGetOrientation(m_Instance);
      }

      public bool GetPageAnnot(int Index, ref TPDFAnnotation Annot)
      {
         TPDFAnnotation_I f = new TPDFAnnotation_I();
         if (pdfGetPageAnnot(m_Instance, Index, ref f) == 0) return false;
         GetIntAnnot(f, ref Annot);
         return true;
      }

      public int GetPageAnnotCount()
      {
         return pdfGetPageAnnotCount(m_Instance);
      }

      public bool GetPageAnnotEx(int Index, ref TPDFAnnotationEx Annot)
      {
         TPDFAnnotationEx_I f = new TPDFAnnotationEx_I();
         if (pdfGetPageAnnotEx(m_Instance, Index, ref f) == 0) return false;
         GetIntAnnotEx(f, ref Annot);
         return true;
      }

      public int GetPageCoords()
      {
         return pdfGetPageCoords(m_Instance);
      }

      public int GetPageCount()
      {
         return pdfGetPageCount(m_Instance);
      }

      public bool GetPageField(int Index, ref TPDFField Field)
      {
          TPDFField_I f = new TPDFField_I();
          if (pdfGetPageField(m_Instance, Index, ref f) == 0) return false;
          GetIntField(f, ref Field);
          return true;
      }

      public int GetPageFieldCount()
      {
         return pdfGetPageFieldCount(m_Instance);
      }

      public bool GetPageFieldEx(int Index, ref TPDFFieldEx Field)
      {
         TPDFFieldEx_I f = new TPDFFieldEx_I();
         f.StructSize = Marshal.SizeOf(f);
         if (pdfGetPageFieldEx(m_Instance, Index, ref f) == 0) return false;
         GetIntFieldEx(f, ref Field);
         return true;
      }

      public double GetPageHeight()
      {
         return pdfGetPageHeight(m_Instance);
      }

      public bool GetPageLabel(int Index, ref TPDFPageLabel Label)
      {
         TPDFPageLabel_I lbl = new TPDFPageLabel_I();
         if (pdfGetPageLabel(m_Instance, Index, ref lbl) == 0) return false;
         Label.StartRange   = lbl.StartRange;
         Label.Format       = lbl.Format;
         Label.FirstPageNum = lbl.FirstPageNum;
         Label.Prefix       = ToString(lbl.Prefix, lbl.PrefixLen, lbl.PrefixUni != 0);
         return true;
      }

      public int GetPageLabelCount()
      {
         return pdfGetPageLabelCount(m_Instance);
      }

      public int GetPageLayout()
      {
         return pdfGetPageLayout(m_Instance);
      }

      public int GetPageMode()
      {
         return pdfGetPageMode(m_Instance);
      }

      public int GetPageNum()
      {
         return pdfGetPageNum(m_Instance);
      }

      public bool GetPageText(ref TPDFStack Stack)
      {
         if (pdfGetPageText(m_Instance, ref Stack) == 0) return false;
         return true;
      }

      public double GetPageWidth()
      {
         return pdfGetPageWidth(m_Instance);
      }

      public int GetPDFVersion()
      {
         return pdfGetPDFVersion(m_Instance);
      }

      public bool GetPrintSettings(ref TPDFPrintSettings Settings)
      {
         TPDFPrintSettings_I s = new TPDFPrintSettings_I();
         if (pdfGetPrintSettings(m_Instance, ref s) == 0) return false;
         Settings.DuplexMode        = s.DuplexMode;
         Settings.NumCopies         = s.NumCopies;
         Settings.PickTrayByPDFSize = s.PickTrayByPDFSize;
         Settings.PrintScaling      = s.PrintScaling;
         if (s.PrintRangesCount > 0)
         {
            Settings.PrintRanges = new int[s.PrintRangesCount<<1];
            Marshal.Copy(s.PrintRanges, Settings.PrintRanges, 0, s.PrintRangesCount<<1);
         }
         return true;
      }

      public int GetResolution()
      {
         return pdfGetResolution(m_Instance);
      }

      public bool GetSaveNewImageFormat()
      {
         return pdfGetSaveNewImageFormat(m_Instance) != 0;
      }

      public bool GetSeparationInfo(ref String Colorant, ref TExtColorSpace CS)
      {
         IntPtr clt = IntPtr.Zero;
         if (pdfGetSeparationInfo(m_Instance, ref clt, ref CS) == 0) return false;
         Colorant = ToString(clt, false);
         return true;
      }

      public double GetSpaceWidth(IntPtr IFont, double FontSize)
      {
         return fntGetSpaceWidth(IFont, FontSize);
      }

      public uint GetStrokeColor()
      {
         return pdfGetStrokeColor(m_Instance);
      }

      public int GetTabLen()
      {
         return pdfGetTabLen(m_Instance);
      }

      public int GetTemplCount()
      {
         return pdfGetTemplCount(m_Instance);
      }

      public int GetTemplHandle()
      {
         return pdfGetTemplHandle(m_Instance);
      }

      public double GetTemplHeight(int Handle)
      {
         return pdfGetTemplHeight(m_Instance, Handle);
      }

      public double GetTemplWidth(int Handle)
      {
         return pdfGetTemplWidth(m_Instance, Handle);
      }

      public int GetTextDrawMode()
      {
         return pdfGetTextDrawMode(m_Instance);
      }

      public bool GetTextFieldValue(int Field, ref String Value, ref String DefValue)
      {
         IntPtr val = IntPtr.Zero, defVal = IntPtr.Zero;
         lbool valUni = 0, defValUni = 0;
         if (pdfGetTextFieldValue(m_Instance, Field, ref val, ref valUni, ref defVal, ref defValUni) == 0)
         {
            Value = null;
            DefValue = null;
            return false;
         }
         Value    = ToString(val, valUni != 0);
         DefValue = ToString(defVal, defValUni != 0);
         return true;
      }

      public bool GetTextRect(ref double PosX, ref double PosY, ref double Width, ref double Height)
      {
         return pdfGetTextRect(m_Instance, ref PosX, ref PosY, ref Width, ref Height) != 0;
      }

      public double GetTextRise()
      {
         return pdfGetTextRise(m_Instance);
      }

      public double GetTextScaling()
      {
         return pdfGetTextScaling(m_Instance);
      }

      public double GetTextWidth(IntPtr IFont, IntPtr Text, int Len, float CharSpacing, float WordSpacing, float TextScale)
      {
         return fntGetTextWidth(IFont, Text, Len, CharSpacing, WordSpacing, TextScale);
      }      

      public double GetTextWidthA(String AText)
      {
         return pdfGetTextWidthA(m_Instance, AText);
      }

      public double GetTextWidthW(String AText)
      {
         return pdfGetTextWidthW(m_Instance, AText);
      }

      public double GetTextWidthExA(String AText, int Len)
      {
         return pdfGetTextWidthExA(m_Instance, AText, Len);
      }

      public double GetTextWidthExW(String AText, int Len)
      {
         return pdfGetTextWidthExW(m_Instance, AText, Len);
      }

      public uint GetTransparentColor()
      {
         return pdfGetTransparentColor(m_Instance);
      }

      public bool GetTrapped()
      {
         return pdfGetTrapped(m_Instance) != 0;
      }

      public bool GetUseExactPwd()
      {
         return pdfGetUseExactPwd(m_Instance) != 0;
      }

      public bool GetUseGlobalImpFiles()
      {
         return pdfGetUseGlobalImpFiles(m_Instance) != 0;
      }

      public int GetUserRights()
      {
         return pdfGetUserRights(m_Instance);
      }

      public bool GetUseStdFonts()
      {
         return pdfGetUseStdFonts(m_Instance) != 0;
      }

      public bool GetUseSystemFonts()
      {
         return pdfGetUseSystemFonts(m_Instance) != 0;
      }

      public bool GetUseTransparency()
      {
         return pdfGetUseTransparency(m_Instance) != 0;
      }

      public bool GetUseVisibleCoords()
      {
         return pdfGetUseVisibleCoords(m_Instance) != 0;
      }

      public bool GetViewerPrefrences(ref int Preference, ref int AddVal)
      {
         return pdfGetViewerPrefrences(m_Instance, ref Preference, ref AddVal) != 0;
      }

      public bool GetWMFDefExtent(ref int Width, ref int Height)
      {
         return pdfGetWMFDefExtent(m_Instance, ref Width, ref Height) != 0;
      }

      public int GetWMFPixelPerInch()
      {
         return pdfGetWMFPixelPerInch(m_Instance);
      }

      public double GetWordSpacing()
      {
         return pdfGetWordSpacing(m_Instance);
      }

      public bool HaveOpenDoc()
      {
         return pdfHaveOpenDoc(m_Instance) != 0;
      }

      public int HighlightAnnotA(TAnnotType SubType, double PosX, double PosY, double Width, double Height, uint Color, String Author, String Subject, String Comment)
      {
         return pdfHighlightAnnotA(m_Instance, SubType, PosX, PosY, Width, Height, Color, Author, Subject, Comment);
      }

      public int HighlightAnnotW(TAnnotType SubType, double PosX, double PosY, double Width, double Height, uint Color, String Author, String Subject, String Comment)
      {
         return pdfHighlightAnnotW(m_Instance, SubType, PosX, PosY, Width, Height, Color, Author, Subject, Comment);
      }

      public int ImportBookmarks()
      {
         return pdfImportBookmarks(m_Instance);
      }

      public bool ImportCatalogObjects()
      {
         return pdfImportCatalogObjects(m_Instance) != 0;
      }

      public bool ImportDocInfo()
      {
         return pdfImportDocInfo(m_Instance) != 0;
      }

      public int ImportPage(int PageNum)
      {
         return pdfImportPage(m_Instance, PageNum);
      }

      public int ImportPageEx(int PageNum, double ScaleX, double ScaleY)
      {
         return pdfImportPageEx(m_Instance, PageNum, ScaleX, ScaleY);
      }

      public int ImportPDFFile(int DestPage, double ScaleX, double ScaleY)
      {
         return pdfImportPDFFile(m_Instance, DestPage, ScaleX, ScaleY);
      }

      public bool InitExtGState(ref TPDFExtGState GS)
      {
         return pdfInitExtGState(ref GS) != 0;
      }

      public bool InitStack(ref TPDFStack Stack)
      {
         if (pdfInitStack(m_Instance, ref Stack) == 0) return false;
         return true;
      }

      public int InsertBMPFromBuffer(double PosX, double PosY, double ScaleWidth, double ScaleHeight, byte[] Buffer)
      {
         return pdfInsertImageFromBuffer(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, Buffer, Buffer.Length, 0);
      }

      public int InsertBMPFromHandle(double PosX, double PosY, double ScaleWidth, double ScaleHeight, IntPtr hBitmap)
      {
         return pdfInsertBMPFromHandle(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, hBitmap);
      }

      public int InsertImage(double PosX, double PosY, double ScaleWidth, double ScaleHeight, String AFile)
      {
         return pdfInsertImage(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, AFile);
      }

      public int InsertImageEx(double PosX, double PosY, double ScaleWidth, double ScaleHeight, String AFile, int Index)
      {
         return pdfInsertImageEx(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, AFile, Index);
      }

      public int InsertImageFromBuffer(double PosX, double PosY, double ScaleWidth, double ScaleHeight, byte[] Buffer, int Index)
      {
         return pdfInsertImageFromBuffer(m_Instance, PosX, PosY, ScaleWidth, ScaleHeight, Buffer, Buffer.Length, Index);
      }

      public bool InsertMetafile(String FileName, double PosX, double PosY, double Width, double Height)
      {
         return pdfInsertMetafile(m_Instance, FileName, PosX, PosY, Width, Height) != 0;
      }

      public bool InsertMetafileEx(byte[] Buffer, double PosX, double PosY, double Width, double Height)
      {
         return pdfInsertMetafileEx(m_Instance, Buffer, Buffer.Length, PosX, PosY, Width, Height) != 0;
      }

      public bool InsertMetafileExt(String FileName, ref TRectL View, double PosX, double PosY, double Width, double Height)
      {
         return pdfInsertMetafileExt(m_Instance, FileName, ref View, PosX, PosY, Width, Height) != 0;
      }

      public bool InsertMetafileExtEx(byte[] Buffer, ref TRectL View, double PosX, double PosY, double Width, double Height)
      {
         return pdfInsertMetafileExtEx(m_Instance, Buffer, Buffer.Length, ref View, PosX, PosY, Width, Height) != 0;
      }

      public bool InsertMetafileFromHandle(IntPtr hEnhMetafile, double PosX, double PosY, double Width, double Height)
      {
         return pdfInsertMetafileFromHandle(m_Instance, hEnhMetafile, PosX, PosY, Width, Height) != 0;
      }

      public bool InsertMetafileFromHandleEx(IntPtr hEnhMetafile, ref TRectL View, double PosX, double PosY, double Width, double Height)
      {
         return pdfInsertMetafileFromHandleEx(m_Instance, hEnhMetafile, ref View, PosX, PosY, Width, Height) != 0;
      }

      public int InsertRawImage(byte[] Buffer, int BitsPerPixel, int ColorCount, int ImgWidth, int ImgHeight, double PosX, double PosY, double ScaleWidth, double ScaleHeight)
      {
         return pdfInsertRawImage(m_Instance, Buffer, BitsPerPixel, ColorCount, ImgWidth, ImgHeight, PosX, PosY, ScaleWidth, ScaleHeight);
      }

      public int IsBidiText(String AText)
      {
         return pdfIsBidiText(m_Instance, AText);
      }

      public bool IsColorPage(bool GrayIsColor)
      {
         return pdfIsColorPage(m_Instance, Convert.ToInt32(GrayIsColor)) != 0;
      }

      public bool IsEmptyPage()
      {
         return pdfIsEmptyPage(m_Instance) != 0;
      }

      public bool IsWrongPwd(int ErrCode)
      {
         switch (-ErrCode)
         {
            case (int)TErrType.E_NEED_PWD:
            case (int)TErrType.E_WRONG_OPEN_PWD:
            case (int)TErrType.E_WRONG_OWNER_PWD:
            case (int)TErrType.E_WRONG_PWD:
               return true;
         }
         return false;
      }

      public bool LineTo(double PosX, double PosY)
      {
         return pdfLineTo(m_Instance, PosX, PosY) != 0;
      }

      public int LoadCMap(String CMapName, bool Embed)
      {
         return pdfLoadCMap(m_Instance, CMapName, Convert.ToInt32(Embed));
      }

      public bool LoadFDFDataA(IntPtr IPDF, String FileName, String Password, int Flags)
      {
         return pdfLoadFDFDataA(m_Instance, FileName, Password, Flags) != 0;
      }

      public bool LoadFDFDataW(IntPtr IPDF, String FileName, String Password, int Flags)
      {
         return pdfLoadFDFDataW(m_Instance, FileName, Password, Flags) != 0;
      }

      public bool LoadFDFDataEx(IntPtr IPDF, byte[] Buffer, String Password, int Flags)
      {
         return pdfLoadFDFDataEx(m_Instance, Buffer, Buffer.Length, Password, Flags) != 0;
      }

      public int LoadFont(byte[] Buffer, TFStyle Style, double Size, bool Embed, TCodepage CP)
      {
         return pdfLoadFont(m_Instance, Buffer, Buffer.Length, Style, Size, Convert.ToInt32(Embed), CP);
      }

      public void MemCopy(IntPtr Source, ref byte[] Dest, int Len)
      {
         if (Len <= 0)
         {
            Dest = null;
            return;
         }
         Dest = new Byte[Len];
         System.Runtime.InteropServices.Marshal.Copy(Source, Dest, 0, Len);
      }

      public bool MovePage(int Source, int Dest)
      {
         return pdfMovePage(m_Instance, Source, Dest) != 0;
      }

      public bool MoveTo(double PosX, double PosY)
      {
         return pdfMoveTo(m_Instance, PosX, PosY) != 0;
      }

      public int MultiplyMatrix(ref TCTM M1, ref TCTM M2, ref TCTM NewMatrix)
      {
         return pdfMultiplyMatrix(ref M1, ref M2, ref NewMatrix);
      }

      public int OpenImportBuffer(byte[] Buffer, TPwdType PwdType, String Password)
      {
         return pdfOpenImportBuffer(m_Instance, Buffer, Buffer.Length, PwdType, Password);
      }

      public int OpenImportFileA(String FileName, TPwdType PwdType, String Password)
      {
         return pdfOpenImportFileA(m_Instance, FileName, PwdType, Password);
      }

      public int OpenImportFileW(String FileName, TPwdType PwdType, String Password)
      {
         return pdfOpenImportFileW(m_Instance, FileName, PwdType, Password);
      }

      public bool OpenOutputFile(String OutPDF)
      {
         return pdfOpenOutputFile(m_Instance, OutPDF) != 0;
      }

      public bool OpenTagA(TPDFBaseTag Tag, String Lang, String AltText, String Expansion)
      {
         return pdfOpenTagA(m_Instance, Tag, Lang, AltText, Expansion) != 0;
      }

      public bool OpenTagW(TPDFBaseTag Tag, String Lang, String AltText, String Expansion)
      {
         return pdfOpenTagW(m_Instance, Tag, Lang, AltText, Expansion) != 0;
      }

      public int PageLink(double PosX, double PosY, double Width, double Height, int DestPage)
      {
         return pdfPageLink(m_Instance, PosX, PosY, Width, Height, DestPage);
      }

      public int PageLink2(double PosX, double PosY, double Width, double Height, int NamedDest)
      {
         return pdfPageLink2(m_Instance, PosX, PosY, Width, Height, NamedDest);
      }

      public int PageLinkEx(double PosX, double PosY, double Width, double Height, TDestType DestType, int DestPage, double a, double b, double c, double d)
      {
         return pdfPageLinkEx(m_Instance, PosX, PosY, Width, Height, DestType, DestPage, a, b, c, d);
      }

      public bool ParseContent(IntPtr Data, ref TPDFParseInterface Stack, TParseFlags Flags)
      {
         return pdfParseContent(m_Instance, Data, ref Stack, Flags) != 0;
      }

      public bool PlaceImage(int ImgHandle, double PosX, double PosY, double ScaleWidth, double ScaleHeight)
      {
         return pdfPlaceImage(m_Instance, ImgHandle, PosX, PosY, ScaleWidth, ScaleHeight) != 0;
      }

      public bool PlaceSigFieldValidateIcon(int SigField, double PosX, double PosY, double Width, double Height)
      {
         return pdfPlaceSigFieldValidateIcon(m_Instance, SigField, PosX, PosY, Width, Height) != 0;
      }

      public bool PlaceTemplate(int TmplHandle, double PosX, double PosY, double ScaleWidth, double ScaleHeight)
      {
         return pdfPlaceTemplate(m_Instance, TmplHandle, PosX, PosY, ScaleWidth, ScaleHeight) != 0;
      }

      public bool PlaceTemplateEx(int TmplHandle, double PosX, double PosY, double ScaleWidth, double ScaleHeight)
      {
         return pdfPlaceTemplateEx(m_Instance, TmplHandle, PosX, PosY, ScaleWidth, ScaleHeight) != 0;
      }

      public bool PlaceTemplByMatrix(int TmplHandle)
      {
         return pdfPlaceTemplByMatrix(m_Instance, TmplHandle) != 0;
      }

      public bool ReadImageFormat2(String FileName, int Index, ref int Width, ref int Height, ref int BitsPerPixel, ref int UseZip)
      {
         return pdfReadImageFormat2(m_Instance, FileName, Index, ref Width, ref Height, ref BitsPerPixel, ref UseZip) != 0;
      }

      public bool ReadImageFormatEx(IntPtr hBitmap, ref int Width, ref int Height, ref int BitsPerPixel, ref int UseZip)
      {
         return pdfReadImageFormatEx(m_Instance, hBitmap, ref Width, ref Height, ref BitsPerPixel, ref UseZip) != 0;
      }

      public bool ReadImageFormatFromBuffer(byte[] Buffer, int Index, ref int Width, ref int Height, ref int BitsPerPixel, ref int UseZip)
      {
         return pdfReadImageFormatFromBuffer(m_Instance, Buffer, Buffer.Length, Index, ref Width, ref Height, ref BitsPerPixel, ref UseZip) != 0;
      }

      public bool ReadImageResolution(String FileName, int Index, ref int ResX, ref int ResY)
      {
         return pdfReadImageResolution(m_Instance, FileName, Index, ref ResX, ref ResY) != 0;
      }

      public bool ReadImageResolutionEx(byte[] Buffer, int Index, ref int ResX, ref int ResY)
      {
         return pdfReadImageResolutionEx(m_Instance, Buffer, Buffer.Length, Index, ref ResX, ref ResY) != 0;
      }

      public bool Rectangle(double PosX, double PosY, double Width, double Height, TPathFillMode FillMode)
      {
         return pdfRectangle(m_Instance, PosX, PosY, Width, Height, FillMode) != 0;
      }

      public int ReEncryptPDFA(String FileName, TPwdType PwdType, String InPwd, String NewOpenPwd, String NewOwnerPwd, TKeyLen NewKeyLen, TRestrictions Restrict)
      {
         return pdfReEncryptPDFA(m_Instance, FileName, PwdType, InPwd, NewOpenPwd, NewOwnerPwd, NewKeyLen, Restrict);
      }

      public int ReEncryptPDFW(String FileName, TPwdType PwdType, String InPwd, String NewOpenPwd, String NewOwnerPwd, TKeyLen NewKeyLen, TRestrictions Restrict)
      {
         return pdfReEncryptPDFW(m_Instance, FileName, PwdType, InPwd, NewOpenPwd, NewOwnerPwd, NewKeyLen, Restrict);
      }

      public int RenameSpotColor(IntPtr IPDF, String Colorant, String NewName)
      {
         return pdfRenameSpotColor(m_Instance, Colorant, NewName);
      }

      public int ReplaceFontA(IntPtr PDFFont, String Name, TFStyle Style, bool Embed)
      {
         return pdfReplaceFontA(m_Instance, PDFFont, Name, Style, Convert.ToInt32(Embed));
      }

      public int ReplaceFontW(IntPtr PDFFont, String Name, TFStyle Style, bool Embed)
      {
         return pdfReplaceFontW(m_Instance, PDFFont, Name, Style, Convert.ToInt32(Embed));
      }

      public int ReplaceFontExA(IntPtr PDFFont, String FontFile, bool Embed)
      {
         return pdfReplaceFontExA(m_Instance, PDFFont, FontFile, Convert.ToInt32(Embed));
      }

      public int ReplaceFontExW(IntPtr PDFFont, String FontFile, bool Embed)
      {
         return pdfReplaceFontExW(m_Instance, PDFFont, FontFile, Convert.ToInt32(Embed));
      }

      public int ReplaceICCProfileA(int ColorSpace, String ICCFile)
      {
         return pdfReplaceICCProfileA(m_Instance, ColorSpace, ICCFile);
      }

      public int ReplaceICCProfileW(int ColorSpace, String ICCFile)
      {
         return pdfReplaceICCProfileW(m_Instance, ColorSpace, ICCFile);
      }

      public bool ReplacePageTextA(String NewText, ref TPDFStack Stack)
      {
         return pdfReplacePageTextA(m_Instance, NewText, ref Stack) != 0;
      }

      public bool ReplacePageTextExA(String NewText, ref TPDFStack Stack)
      {
         return pdfReplacePageTextExA(m_Instance, NewText, ref Stack) != 0;
      }

      public bool ReplacePageTextExW(String NewText, ref TPDFStack Stack)
      {
         return pdfReplacePageTextExW(m_Instance, NewText, ref Stack) != 0;
      }

      public bool ResetLineDashPattern()
      {
         return pdfResetLineDashPattern(m_Instance) != 0;
      }

      public bool RestoreGraphicState()
      {
         return pdfRestoreGraphicState(m_Instance) != 0;
      }

      public bool RotateCoords(double alpha, double OriginX, double OriginY)
      {
         return pdfRotateCoords(m_Instance, alpha, OriginX, OriginY) != 0;
      }

      public uint RGB(byte r, byte g, byte b)
      {
         return (uint)((r|(g<<8))|(b<<16));
      }

      public bool RoundRect(double PosX, double PosY, double Width, double Height, double Radius, TPathFillMode FillMode)
      {
         return pdfRoundRect(m_Instance, PosX, PosY, Width, Height, Radius, FillMode) != 0;
      }

      public bool RoundRectEx(double PosX, double PosY, double Width, double Height, double rWidth, double rHeight, TPathFillMode FillMode)
      {
         return pdfRoundRectEx(m_Instance, PosX, PosY, Width, Height, rWidth, rHeight, FillMode) != 0;
      }

      public bool SaveGraphicState()
      {
         return pdfSaveGraphicState(m_Instance) != 0;
      }

      public bool ScaleCoords(double sx, double sy)
      {
         return pdfScaleCoords(m_Instance, sx, sy) != 0;
      }

      public bool SelfTest()
      {
         return pdfSelfTest(m_Instance) != 0;
      }

      public bool Set3DAnnotProps(int Annot, T3DActivationType ActType, T3DDeActivateType DeActType, T3DInstanceType InstType, T3DDeActInstance DeInstType, bool DisplToolbar, bool DisplModelTree)
      {
         return pdfSet3DAnnotProps(m_Instance, Annot, ActType, DeActType, InstType, DeInstType, Convert.ToInt32(DisplToolbar), Convert.ToInt32(DisplModelTree)) != 0;
      }

      public bool Set3DAnnotScriptA(int Annot, String Value)
      {
         return pdfSet3DAnnotScriptA(m_Instance, Annot, Value, Value.Length) != 0;
      }

      public int SetAllocBy(int Value)
      {
         return pdfSetAllocBy(m_Instance, Value);
      }

      public bool SetAnnotBorderStyle(int Handle, TBorderStyle Style)
      {
         return pdfSetAnnotBorderStyle(m_Instance, Handle, Style) != 0;
      }

      public bool SetAnnotBorderWidth(int Handle, double LineWidth)
      {
         return pdfSetAnnotBorderWidth(m_Instance, Handle, LineWidth) != 0;
      }

      public bool SetAnnotColor(int Handle, TAnnotColor ColorType, TPDFColorSpace CS, uint Color)
      {
         return pdfSetAnnotColor(m_Instance, Handle, ColorType, CS, Color) != 0;
      }

      public bool SetAnnotFlags(TAnnotFlags Flags)
      {
         return pdfSetAnnotFlags(m_Instance, Flags) != 0;
      }

      public bool SetAnnotFlagsEx(int Handle, int Flags)
      {
         return pdfSetAnnotFlagsEx(m_Instance, Handle, Flags) != 0;
      }

      public bool SetAnnotHighlightMode(int Handle, THighlightMode Mode)
      {
         return pdfSetAnnotHighlightMode(m_Instance, Handle, Mode) != 0;
      }

      public bool SetAnnotIcon(int Handle, TAnnotIcon Icon)
      {
         return pdfSetAnnotIcon(m_Instance, Handle, Icon) != 0;
      }

      public bool SetAnnotOpenState(int Handle, bool Open)
      {
         return pdfSetAnnotOpenState(m_Instance, Handle, Open) != 0;
      }

      public bool SetAnnotStringA(int Handle, TAnnotString StringType, String Value)
      {
         return pdfSetAnnotStringA(m_Instance, Handle, StringType, Value) != 0;
      }

      public bool SetAnnotStringW(int Handle, TAnnotString StringType, String Value)
      {
         return pdfSetAnnotStringW(m_Instance, Handle, StringType, Value) != 0;
      }

      public bool SetAnnotSubjectA(int Handle, String Value)
      {
         return pdfSetAnnotSubjectA(m_Instance, Handle, Value) != 0;
      }

      public bool SetAnnotSubjectW(int Handle, String Value)
      {
         return pdfSetAnnotSubjectW(m_Instance, Handle, Value) != 0;
      }

      public bool SetBBox(TPageBoundary Boundary, double LeftX, double LeftY, double RightX, double RightY)
      {
         return pdfSetBBox(m_Instance, Boundary, LeftX, LeftY, RightX, RightY) != 0;
      }

      public bool SetBidiMode(TPDFBidiMode Mode)
      {
         return pdfSetBidiMode(m_Instance, Mode) != 0;
      }

      public bool SetBookmarkDest(int ABmk, TDestType DestType, double a, double b, double c, double d)
      {
         return pdfSetBookmarkDest(m_Instance, ABmk, DestType, a, b, c, d) != 0;
      }

      public bool SetBookmarkStyle(int ABmk, TFStyle Style, uint RGBColor)
      {
         return pdfSetBookmarkStyle(m_Instance, ABmk, Style, RGBColor) != 0;
      }

      public bool SetBorderStyle(TBorderStyle Style)
      {
         return pdfSetBorderStyle(m_Instance, Style) != 0;
      }

      public bool SetCharacterSpacing(double Value)
      {
         return pdfSetCharacterSpacing(m_Instance, Value) != 0;
      }

      public bool SetCheckBoxChar(TCheckBoxChar CheckBoxChar)
      {
         return pdfSetCheckBoxChar(m_Instance, CheckBoxChar) != 0;
      }

      public bool SetCheckBoxDefState(int Field, bool Checked)
      {
         return pdfSetCheckBoxDefState(m_Instance, Field, Convert.ToInt32(Checked)) != 0;
      }

      public bool SetCheckBoxState(int Field, bool Checked)
      {
         return pdfSetCheckBoxState(m_Instance, Field, Convert.ToInt32(Checked)) != 0;
      }

      public int SetCIDFontA(int CMapHandle, String Name, TFStyle Style, double Size, bool Embed)
      {
         return pdfSetCIDFontA(m_Instance, CMapHandle, Name, Style, Size, Convert.ToInt32(Embed));
      }

      public int SetCIDFontW(int CMapHandle, String Name, TFStyle Style, double Size, bool Embed)
      {
         return pdfSetCIDFontW(m_Instance, CMapHandle, Name, Style, Size, Convert.ToInt32(Embed));
      }

      public int SetCMapDirA(String Path, bool Recursive)
      {
         return pdfSetCMapDirA(m_Instance, Path, Convert.ToInt32(Recursive));
      }

      public int SetCMapDirW(String Path, bool Recursive)
      {
         return pdfSetCMapDirW(m_Instance, Path, Convert.ToInt32(Recursive));
      }

      public bool SetColDefFile(int EmbFile)
      {
         return pdfSetColDefFile(m_Instance, EmbFile) != 0;
      }

      public bool SetColors(uint Color)
      {
         return pdfSetColors(m_Instance, Color) != 0;
      }

      public bool SetColorSpace(TPDFColorSpace ColorSpace)
      {
         return pdfSetColorSpace(m_Instance, ColorSpace) != 0;
      }

      public bool SetColSortField(int ColField, bool AscendingOrder)
      {
         return pdfSetColSortField(m_Instance, ColField, Convert.ToInt32(AscendingOrder)) != 0;
      }

      public bool SetCompressionFilter(TCompressionFilter Filter)
      {
         return pdfSetCompressionFilter(m_Instance, Filter) != 0;
      }

      public bool SetCompressionLevel(TCompressionLevel CompressLevel)
      {
         return pdfSetCompressionLevel(m_Instance, CompressLevel) != 0;
      }

      public bool SetContent(String Buffer, int BufSize)
      {
         return pdfSetContent(m_Instance, Buffer, BufSize) != 0;
      }

      public bool SetDateTimeFormat(int TxtField, TPDFDateTime Fmt)
      {
         return pdfSetDateTimeFormat(m_Instance, TxtField, Fmt) != 0;
      }

      public bool SetDefBitsPerPixel(int Value)
      {
         return pdfSetDefBitsPerPixel(m_Instance, Value) != 0;
      }

      public bool SetDocInfoA(TDocumentInfo DInfo, String Value)
      {
         return pdfSetDocInfoA(m_Instance, DInfo, Value) != 0;
      }

      public bool SetDocInfoW(TDocumentInfo DInfo, String Value)
      {
         return pdfSetDocInfoW(m_Instance, DInfo, Value) != 0;
      }

      public bool SetDocInfoExA(TDocumentInfo DInfo, String Key, String Value)
      {
         return pdfSetDocInfoExA(m_Instance, DInfo, Key, Value) != 0;
      }

      public bool SetDocInfoExW(TDocumentInfo DInfo, String Key, String Value)
      {
         return pdfSetDocInfoExW(m_Instance, DInfo, Key, Value) != 0;
      }

      public bool SetDrawDirection(TDrawDirection Direction)
      {
         return pdfSetDrawDirection(m_Instance, Direction) != 0;
      }

      public bool SetEMFFrameDPI(int DPIX, int DPIY)
      {
         return pdfSetEMFFrameDPI(m_Instance, DPIX, DPIY) != 0;
      }

      public bool SetEMFPatternDistance(double Value)
      {
         return pdfSetEMFPatternDistance(m_Instance, Value) != 0;
      }

      public bool SetErrorMode(TErrMode ErrMode)
      {
         return pdfSetErrorMode(m_Instance, ErrMode) != 0;
      }

      public bool SetExtColorSpace(int Handle)
      {
         return pdfSetExtColorSpace(m_Instance, Handle) != 0;
      }

      public bool SetExtGState(int Handle)
      {
         return pdfSetExtGState(m_Instance, Handle) != 0;
      }

      public bool SetFieldBBox(int Field, ref TPDFRect BBox)
      {
         return pdfSetFieldBBox(m_Instance, Field, ref BBox) != 0;
      }

      public bool SetFieldBackColor(uint AColor)
      {
         return pdfSetFieldBackColor(m_Instance, AColor) != 0;
      }

      public bool SetFieldBorderColor(uint AColor)
      {
         return pdfSetFieldBorderColor(m_Instance, AColor) != 0;
      }

      public bool SetFieldBorderWidth(int Field, double LineWidth)
      {
         return pdfSetFieldBorderWidth(m_Instance, Field, LineWidth) != 0;
      }

      public bool SetFieldColor(int Field, TFieldColor ColorType, TPDFColorSpace CS, uint Color)
      {
         return pdfSetFieldColor(m_Instance, Field, ColorType, CS, Color) != 0;
      }

      public bool SetFieldBorderStyle(int Field, TBorderStyle Style)
      {
         return pdfSetFieldBorderStyle(m_Instance, Field, Style) != 0;
      }

      public bool SetFieldExpValue(int Field, int ValIndex, String Value, String ExpValue, bool Selected)
      {
         return pdfSetFieldExpValue(m_Instance, Field, ValIndex, Value, ExpValue, Convert.ToInt32(Selected)) != 0;
      }

      public bool SetFieldFlags(int Field, TFieldFlags Flags, bool Reset)
      {
         return pdfSetFieldFlags(m_Instance, Field, Flags, Convert.ToInt32(Reset)) != 0;
      }

      public bool SetFieldFontSize(int Field, double FontSize)
      {
         return pdfSetFieldFontSize(m_Instance, Field, FontSize) != 0;
      }

      public bool SetFieldHighlightMode(int Field, THighlightMode Mode)
      {
         return pdfSetFieldHighlightMode(m_Instance, Field, Mode) != 0;
      }

      public bool SetFieldIndex(int Field, int Index)
      {
         return pdfSetFieldIndex(m_Instance, Field, Index) != 0;
      }

      public bool SetFieldMapName(int Field, String Name)
      {
         return pdfSetFieldMapName(m_Instance, Field, Name) != 0;
      }

      public bool SetFieldName(int Field, String NewName)
      {
         return pdfSetFieldName(m_Instance, Field, NewName) != 0;
      }

      public bool SetFieldOrientation(int Field, int Value)
      {
         return pdfSetFieldOrientation(m_Instance, Field, Value) != 0;
      }

      public bool SetFieldTextAlign(int Field, TTextAlign Align)
      {
         return pdfSetFieldTextAlign(m_Instance, Field, Align) != 0;
      }

      public bool SetFieldTextColor(uint Color)
      {
         return pdfSetFieldTextColor(m_Instance, Color) != 0;
      }

      public bool SetFieldToolTipA(int Field, String Value)
      {
         return pdfSetFieldToolTipA(m_Instance, Field, Value) != 0;
      }

      public bool SetFieldToolTipW(int Field, String Value)
      {
         return pdfSetFieldToolTipW(m_Instance, Field, Value) != 0;
      }

      public bool SetFillColor(uint Color)
      {
         return pdfSetFillColor(m_Instance, Color) != 0;
      }

      public bool SetFillColorEx(byte[] Color, int NumComponents)
      {
         return pdfSetFillColorEx(m_Instance, Color, NumComponents) != 0;
      }

      public bool SetFillColorSpace(TPDFColorSpace CS)
      {
         return pdfSetFillColorSpace(m_Instance, CS) != 0;
      }

      public int SetFont(String Name, TFStyle Style, double Size, bool Embed, TCodepage CP)
      {
         return pdfSetFont(m_Instance, Name, Style, Size, Convert.ToInt32(Embed), CP);
      }

      public int SetFontEx(String Name, TFStyle Style, double Size, bool Embed, TCodepage CP)
      {
         return pdfSetFontEx(m_Instance, Name, Style, Size, Convert.ToInt32(Embed), CP);
      }

      public bool SetFontOrigin(TOrigin Origin)
      {
         return pdfSetFontOrigin(m_Instance, Origin) != 0;
      }

      public void SetFontSearchOrder(TFontBaseType[] Order)
      {
         pdfSetFontSearchOrder(m_Instance, Order);
      }

      public bool SetFontSelMode(TFontSelMode Mode)
      {
         return pdfSetFontSelMode(m_Instance, Mode) != 0;
      }

      public bool SetFontWeight(int Weight)
      {
         return pdfSetFontWeight(m_Instance, Weight) != 0;
      }

      public void SetGStateFlags(TGStateFlags Flags, bool Reset)
      {
         pdfSetGStateFlags(m_Instance, Flags, Convert.ToInt32(Reset));
      }

      public bool SetIconColor(uint Color)
      {
         return pdfSetIconColor(m_Instance, Color) != 0;
      }

      public bool SetImportFlags(TImportFlags Flags)
      {
         return pdfSetImportFlags(m_Instance, Flags) != 0;
      }

      public bool SetImportFlags2(TImportFlags2 Flags)
      {
         return pdfSetImportFlags2(m_Instance, Flags) != 0;
      }

      public bool SetItalicAngle(double Angle)
      {
         return pdfSetItalicAngle(m_Instance, Angle) != 0;
      }

      public bool SetJPEGQuality(int Value)
      {
         return pdfSetJPEGQuality(m_Instance, Value) != 0;
      }

      public bool SetLanguage(String ISOTag)
      {
         return pdfSetLanguage(m_Instance, ISOTag) != 0;
      }

      public bool SetLeading(double Value)
      {
         return pdfSetLeading(m_Instance, Value) != 0;
      }

      public bool SetLicenseKey(String Value)
      {
         return pdfSetLicenseKey(m_Instance, Value) != 0;
      }

      public bool SetLineCapStyle(TLineCapStyle Style)
      {
         return pdfSetLineCapStyle(m_Instance, Style) != 0;
      }

      public bool SetLineDashPatternEx(ref double[] Dash, int NumValues, int Phase)
      {
         return pdfSetLineDashPatternEx(m_Instance, ref Dash, NumValues, Phase) != 0;
      }

      public bool SetLineJoinStyle(TLineJoinStyle Style)
      {
         return pdfSetLineJoinStyle(m_Instance, Style) != 0;
      }

      public bool SetLineWidth(double Value)
      {
         return pdfSetLineWidth(m_Instance, Value) != 0;
      }

      public bool SetLinkHighlightMode(THighlightMode Mode)
      {
         return pdfSetLinkHighlightMode(m_Instance, Mode) != 0;
      }

      public bool SetListFont(int Handle)
      {
         return pdfSetListFont(m_Instance, Handle) != 0;
      }

      public bool SetMatrix(ref TCTM Matrix)
      {
         return pdfSetMatrix(m_Instance, ref Matrix) != 0;
      }

      public bool SetMaxFieldLen(int TxtField, int MaxLen)
      {
         return pdfSetMaxFieldLen(m_Instance, TxtField, MaxLen) != 0;
      }

      public bool SetMetaConvFlags(TMetaFlags Flags)
      {
         return pdfSetMetaConvFlags(m_Instance, Flags) != 0;
      }

      public bool SetMiterLimit(double Value)
      {
         return pdfSetMiterLimit(m_Instance, Value) != 0;
      }

      public bool SetNeedAppearance(bool Value)
      {
         return pdfSetNeedAppearance(m_Instance, Convert.ToInt32(Value)) != 0;
      }

      public bool SetNumberFormat(int TxtField, TDecSeparator Sep, int DecPlaces, TNegativeStyle NegStyle, String CurrStr, bool Prepend)
      {
         return pdfSetNumberFormat(m_Instance, TxtField, Sep, DecPlaces, NegStyle, CurrStr, Convert.ToInt32(Prepend)) != 0;
      }

      public bool SetOnErrorProc(IntPtr Data, TErrorProc ErrProc)
      {
         m_ErrorProc = ErrProc;
         return pdfSetOnErrorProc(m_Instance, Data, m_ErrorProc) != 0;
      }

      public bool SetOnPageBreakProc(IntPtr Data, TOnPageBreakProc OnPageBreakProc)
      {
         m_OnPageBreakProc = OnPageBreakProc;
         return pdfSetOnPageBreakProc(m_Instance, Data, m_OnPageBreakProc) != 0;
      }

      public bool SetOpacity(double Value)
      {
         return pdfSetOpacity(m_Instance, Value) != 0;
      }

      public bool SetOrientation(int Value)
      {
         return pdfSetOrientation(m_Instance, Value) != 0;
      }

      public bool SetOrientationEx(int Value)
      {
         return pdfSetOrientationEx(m_Instance, Value) != 0;
      }

      public bool SetPageCoords(TPageCoord PageCoords)
      {
         return pdfSetPageCoords(m_Instance, PageCoords) != 0;
      }

      public bool SetPageFormat(TPageFormat Value)
      {
         return pdfSetPageFormat(m_Instance, Value) != 0;
      }

      public bool SetPageHeight(double Value)
      {
         return pdfSetPageHeight(m_Instance, Value) != 0;
      }

      public bool SetPageLayout(TPageLayout Layout)
      {
         return pdfSetPageLayout(m_Instance, Layout) != 0;
      }

      public bool SetPageMode(TPageMode Mode)
      {
         return pdfSetPageMode(m_Instance, Mode) != 0;
      }

      public bool SetPageWidth(double Value)
      {
         return pdfSetPageWidth(m_Instance, Value) != 0;
      }

      public bool SetPDFVersion(TPDFVersion Version)
      {
         return pdfSetPDFVersion(m_Instance, Version) != 0;
      }

      public bool SetPrintSettings(TDuplexMode Mode, int PickTrayByPDFSize, int NumCopies, TPrintScaling PrintScaling, uint[] PrintRanges, int NumRanges)
      {
         return pdfSetPrintSettings(m_Instance, Mode, PickTrayByPDFSize, NumCopies, PrintScaling, PrintRanges, NumRanges) != 0;
      }

      public bool SetProgressProc(IntPtr Data, TInitProgressProc InitProgress, TProgressProc Progress)
      {
         m_InitProgressProc = InitProgress;
         m_ProgressProc     = Progress;
         return pdfSetProgressProc(m_Instance, Data, m_InitProgressProc, m_ProgressProc) != 0;
      }

      public bool SetResolution(int Value)
      {
         return pdfSetResolution(m_Instance, Value) != 0;
      }

      public bool SetSaveNewImageFormat(bool Value)
      {
         return pdfSetSaveNewImageFormat(m_Instance, Convert.ToInt32(Value)) != 0;
      }

      public bool SetSeparationInfo(int Handle)
      {
         // Handle must be a handle of a separation color space
         return pdfSetSeparationInfo(m_Instance, Handle) != 0;
      }

      public bool SetStrokeColor(uint Color)
      {
         return pdfSetStrokeColor(m_Instance, Color) != 0;
      }

      public bool SetStrokeColorEx(byte[] Color, int NumComponents)
      {
         return pdfSetStrokeColorEx(m_Instance, Color, NumComponents) != 0;
      }

      public bool SetStrokeColorSpace(TPDFColorSpace CS)
      {
         return pdfSetStrokeColorSpace(m_Instance, CS) != 0;
      }

      public bool SetTabLen(int TabLen)
      {
         return pdfSetTabLen(m_Instance, TabLen) != 0;
      }

      public bool SetTextDrawMode(TDrawMode Mode)
      {
         return pdfSetTextDrawMode(m_Instance, Mode) != 0;
      }

      public bool SetTextFieldValueA(int Field, String Value, String DefValue, TTextAlign Align)
      {
         return pdfSetTextFieldValueA(m_Instance, Field, Value, DefValue, Align) != 0;
      }

      public bool SetTextFieldValueW(int Field, String Value, String DefValue, TTextAlign Align)
      {
         return pdfSetTextFieldValueW(m_Instance, Field, Value, DefValue, Align) != 0;
      }

      public bool SetTextFieldValueExA(int Field, String Value)
      {
         return pdfSetTextFieldValueExA(m_Instance, Field, Value) != 0;
      }

      public bool SetTextFieldValueExW(int Field, String Value)
      {
         return pdfSetTextFieldValueExW(m_Instance, Field, Value) != 0;
      }

      public bool SetTextRect(double PosX, double PosY, double Width, double Height)
      {
         return pdfSetTextRect(m_Instance, PosX, PosY, Width, Height) != 0;
      }

      public bool SetTextRise(double Value)
      {
         return pdfSetTextRise(m_Instance, Value) != 0;
      }

      public bool SetTextScaling(double Value)
      {
         return pdfSetTextScaling(m_Instance, Value) != 0;
      }

      public bool SetTransparentColor(uint AColor)
      {
         return pdfSetTransparentColor(m_Instance, AColor) != 0;
      }

      public void SetTrapped(bool Value)
      {
         pdfSetTrapped(m_Instance, Convert.ToInt32(Value));
      }

      public bool SetUseExactPwd(bool Value)
      {
         return pdfSetUseExactPwd(m_Instance, Convert.ToInt32(Value)) != 0;
      }

      public bool SetUseGlobalImpFiles(bool Value)
      {
         return pdfSetUseGlobalImpFiles(m_Instance, Convert.ToInt32(Value)) != 0;
      }

      public bool SetUseStdFonts(bool Value)
      {
         return pdfSetUseStdFonts(m_Instance, Convert.ToInt32(Value)) != 0;
      }

      public bool SetUseSwapFile(bool SwapContents, int SwapLimit)
      {
         return pdfSetUseSwapFile(m_Instance, Convert.ToInt32(SwapContents), SwapLimit) != 0;
      }

      public bool SetUseSwapFileEx(bool SwapContents, int SwapLimit, String SwapDir)
      {
         return pdfSetUseSwapFileEx(m_Instance, Convert.ToInt32(SwapContents), SwapLimit, SwapDir) != 0;
      }

      public bool SetUseSystemFonts(bool Value)
      {
         return pdfSetUseSystemFonts(m_Instance, Convert.ToInt32(Value)) != 0;
      }

      public bool SetUseTransparency(bool Value)
      {
         return pdfSetUseTransparency(m_Instance, Convert.ToInt32(Value)) != 0;
      }

      public bool SetUseVisibleCoords(bool Value)
      {
         return pdfSetUseVisibleCoords(m_Instance, Convert.ToInt32(Value)) != 0;
      }

      public bool SetViewerPreferences(TViewerPreference Value, TViewPrefAddVal AddVal)
      {
         return pdfSetViewerPreferences(m_Instance, Value, AddVal) != 0;
      }

      public bool SetWMFDefExtent(int Width, int Height)
      {
         return pdfSetWMFDefExtent(m_Instance, Width, Height) != 0;
      }

      public bool SetWMFPixelPerInch(int Value)
      {
         return pdfSetWMFPixelPerInch(m_Instance, Value) != 0;
      }

      public bool SetWordSpacing(double Value)
      {
         return pdfSetWordSpacing(m_Instance, Value) != 0;
      }

      public bool SkewCoords(double alpha, double beta, double OriginX, double OriginY)
      {
         return pdfSkewCoords(m_Instance, alpha, beta, OriginX, OriginY) != 0;
      }

      public bool SortFieldsByIndex()
      {
         return pdfSortFieldsByIndex(m_Instance) != 0;
      }

      public bool SortFieldsByName()
      {
         return pdfSortFieldsByName(m_Instance) != 0;
      }

      public int StampAnnotA(TRubberStamp SubType, double PosX, double PosY, double Width, double Height, String Author, String Subject, String Comment)
      {
         return pdfStampAnnotA(m_Instance, SubType, PosX, PosY, Width, Height, Author, Subject, Comment);
      }

      public int StampAnnotW(TRubberStamp SubType, double PosX, double PosY, double Width, double Height, String Author, String Subject, String Comment)
      {
         return pdfStampAnnotW(m_Instance, SubType, PosX, PosY, Width, Height, Author, Subject, Comment);
      }

      public bool StrokePath()
      {
         return pdfStrokePath(m_Instance) != 0;
      }

      public int TextAnnotA(double PosX, double PosY, double Width, double Height, String Author, String Text, TAnnotIcon Icon, bool Open)
      {
         return pdfTextAnnotA(m_Instance, PosX, PosY, Width, Height, Author, Text, Icon, Convert.ToInt32(Open));
      }

      public int TextAnnotW(double PosX, double PosY, double Width, double Height, String Author, String Text, TAnnotIcon Icon, bool Open)
      {
         return pdfTextAnnotW(m_Instance, PosX, PosY, Width, Height, Author, Text, Icon, Convert.ToInt32(Open));
      }

      public bool TranslateCoords(double OriginX, double OriginY)
      {
         return pdfTranslateCoords(m_Instance, OriginX, OriginY) != 0;
      }

      public int TranslateRawCode(IntPtr IFont, IntPtr Text, int Len, ref double Width, byte[] OutText, ref int OutLen, ref lbool Decoded, float CharSpacing, float WordSpacing, float TextScale)
      {
         return fntTranslateRawCode(IFont, Text, Len, ref Width, OutText, ref OutLen, ref Decoded, CharSpacing, WordSpacing, TextScale);
      }

      public int TranslateString(TPDFStack Stack, ref String OutText, int Flags)
      {
         byte[] buf = new byte[(Stack.TextLen * 16 / 10) * 2 + 64];
         int retval = fntTranslateString(ref Stack, buf, buf.Length >> 1, Flags);
         OutText = System.Text.UnicodeEncoding.Unicode.GetString(buf, 0, retval << 1);
         return retval;
      }

      public int TranslateString2(IntPtr IFont, IntPtr Text, int Len, ref String OutText, int Size, int Flags)
      {
         byte[] buf = new byte[(Len * 16 / 10) * 2 + 64];
         int retval = fntTranslateString2(IFont, Text, Len, buf, buf.Length >> 1, Flags);
         OutText = System.Text.UnicodeEncoding.Unicode.GetString(buf, 0, retval << 1);
         return retval;
      }

      public bool Triangle(double x1, double y1, double x2, double y2, double x3, double y3, TPathFillMode FillMode)
      {
         return pdfTriangle(m_Instance, x1, y1, x2, y2, x3, y3, FillMode) != 0;
      }

      public int WebLinkA(double PosX, double PosY, double Width, double Height, String URL)
      {
         return pdfWebLinkA(m_Instance, PosX, PosY, Width, Height, URL);
      }

      public int WebLinkW(double PosX, double PosY, double Width, double Height, String URL)
      {
         return pdfWebLinkW(m_Instance, PosX, PosY, Width, Height, URL);
      }

      public bool WriteAngleTextA(String AText, double Angle, double PosX, double PosY, double Radius, double YOrigin)
      {
         return pdfWriteAngleTextA(m_Instance, AText, Angle, PosX, PosY, Radius, YOrigin) != 0;
      }

      public uint WeightFromStyle(TFStyle Style)
      {
         return (((uint)Style & ~0xC00FFFFF) >> 20);
      }

      public TFStyle WeightToStyle(uint Weight)
      {
         return (TFStyle)(Weight << 20);
      }

      public uint WidthFromStyle(TFStyle Style)
      {
         return (((uint)Style & ~0xFFF2D00F) >> 8);
      }

      public TFStyle WidthToStyle(uint Width)
      {
         return (TFStyle)(Width << 8);
      }

      public bool WriteAngleTextW(String AText, double Angle, double PosX, double PosY, double Radius, double YOrigin)
      {
         return pdfWriteAngleTextW(m_Instance, AText, Angle, PosX, PosY, Radius, YOrigin) != 0;
      }

      public bool WriteFTextA(TTextAlign Align, String AText)
      {
         return pdfWriteFTextA(m_Instance, Align, AText) != 0;
      }

      public bool WriteFTextW(TTextAlign Align, String AText)
      {
         return pdfWriteFTextW(m_Instance, Align, AText) != 0;
      }

      public bool WriteFTextExA(double PosX, double PosY, double Width, double Height, TTextAlign Align, String AText)
      {
         return pdfWriteFTextExA(m_Instance, PosX, PosY, Width, Height, Align, AText) != 0;
      }

      public bool WriteFTextExW(double PosX, double PosY, double Width, double Height, TTextAlign Align, String AText)
      {
         return pdfWriteFTextExW(m_Instance, PosX, PosY, Width, Height, Align, AText) != 0;
      }

      public bool WriteTextA(double PosX, double PosY, String AText)
      {
         return pdfWriteTextA(m_Instance, PosX, PosY, AText) != 0;
      }

      public bool WriteTextW(double PosX, double PosY, String AText)
      {
         return pdfWriteTextW(m_Instance, PosX, PosY, AText) != 0;
      }

      public bool WriteTextExA(double PosX, double PosY, String AText, int Len)
      {
         return pdfWriteTextExA(m_Instance, PosX, PosY, AText, Len) != 0;
      }

      public bool WriteTextExW(double PosX, double PosY, String AText, int Len)
      {
         return pdfWriteTextExW(m_Instance, PosX, PosY, AText, Len) != 0;
      }

      public bool WriteTextMatrixA(ref TCTM M, String AText)
      {
         return pdfWriteTextMatrixA(m_Instance, ref M, AText) != 0;
      }

      public bool WriteTextMatrixW(ref TCTM M, String AText)
      {
         return pdfWriteTextMatrixW(m_Instance, ref M, AText) != 0;
      }

      public bool WriteTextMatrixExA(ref TCTM M, String AText, int Len)
      {
         return pdfWriteTextMatrixExA(m_Instance, ref M, AText, Len) != 0;
      }

      public bool WriteTextMatrixExW(ref TCTM M, String AText, int Len)
      {
         return pdfWriteTextMatrixExW(m_Instance, ref M, AText, Len) != 0;
      }

      /* ------------------------------------- Private functions ------------------------------------- */

      private void GetIntAnnot(TPDFAnnotation_I IAnnot, ref TPDFAnnotation Annot)
      {
         Annot.BackColor     = IAnnot.BackColor;
         Annot.BBox          = IAnnot.BBox;
         Annot.BorderColor   = IAnnot.BorderColor;
         Annot.BorderStyle   = IAnnot.BorderStyle;
         Annot.BorderWidth   = IAnnot.BorderWidth;
         Annot.Deleted       = IAnnot.Deleted != 0;
         Annot.Handle        = IAnnot.Handle;
         Annot.HighlightMode = IAnnot.HighlightMode;
         Annot.PageNum       = IAnnot.PageNum;
         Annot.Type          = IAnnot.Type;
         Annot.Author        = ToString(IAnnot.AuthorA, IAnnot.AuthorW);
         Annot.Content       = ToString(IAnnot.ContentA, IAnnot.ContentW);
         Annot.Name          = ToString(IAnnot.NameA, IAnnot.NameW);
         Annot.Subject       = ToString(IAnnot.SubjectA, IAnnot.SubjectW);
      }
      
      private void GetIntAnnotEx(TPDFAnnotationEx_I IAnnot, ref TPDFAnnotationEx Annot)
      {
         Annot.BackColor     = IAnnot.BackColor;
         Annot.BBox          = IAnnot.BBox;
         Annot.BorderColor   = IAnnot.BorderColor;
         Annot.BorderStyle   = IAnnot.BorderStyle;
         Annot.BorderWidth   = IAnnot.BorderWidth;
         Annot.Deleted       = IAnnot.Deleted != 0;
         Annot.Handle        = IAnnot.Handle;
         Annot.HighlightMode = IAnnot.HighlightMode;
         Annot.PageNum       = IAnnot.PageNum;
         Annot.Type          = IAnnot.Type;
         Annot.Author        = ToString(IAnnot.AuthorA, IAnnot.AuthorW);
         Annot.Content       = ToString(IAnnot.ContentA, IAnnot.ContentW);
         Annot.Name          = ToString(IAnnot.NameA, IAnnot.NameW);
         Annot.Subject       = ToString(IAnnot.SubjectA, IAnnot.SubjectW);
         Annot.DestPage      = IAnnot.DestPage;
         Annot.DestPos       = IAnnot.DestPos;
         Annot.DestType      = IAnnot.DestType;
         Annot.DestFile      = ToString(IAnnot.DestFile, false);
         Annot.Icon          = IAnnot.Icon;
         Annot.StampName     = ToString(IAnnot.StampName, false);
         Annot.AnnotFlags    = IAnnot.AnnotFlags;
         Annot.CreateDate    = ToString(IAnnot.CreateDate, false);
         Annot.ModDate       = ToString(IAnnot.ModDate, false);
         Annot.Grouped       = IAnnot.Grouped != 0;
         Annot.Open          = IAnnot.Open != 0;
         Annot.Parent        = IAnnot.Parent;
         Annot.PopUp         = IAnnot.PopUp;
         Annot.State         = ToString(IAnnot.State, false);
         Annot.StateModel    = ToString(IAnnot.StateModel, false);
         Annot.EmbeddedFile  = IAnnot.EmbeddedFile;
         Annot.Subtype       = ToString(IAnnot.Subtype, false);
      }

      private void GetIntColorSpaceObj(TPDFColorSpaceObj_I ICS, ref TPDFColorSpaceObj CS)
      {
         int i;
         CS.Type             = ICS.Type;
         CS.Alternate        = ICS.Alternate;
         CS.IAlternate       = ICS.IAlternate;
         CS.Buffer           = ICS.Buffer;
         CS.BufSize          = ICS.BufSize;
         CS.NumInComponents  = ICS.NumInComponents;
         CS.NumOutComponents = ICS.NumOutComponents;
         CS.NumColors        = ICS.NumColors;
         CS.Metadata         = ICS.Metadata;
         CS.MetadataSize     = ICS.MetadataSize;
         CS.IFunction        = ICS.IFunction;
         CS.IAttributes      = ICS.IAttributes;
         CS.IColorSpaceObj   = ICS.IColorSpaceObj;
         if (!IntPtr.Zero.Equals(ICS.BlackPoint))
         {
            CS.BlackPoint = new float[3];
            Marshal.Copy(ICS.BlackPoint, CS.BlackPoint, 0, 3);
         }else
         {
            CS.BlackPoint = null;
         }
         if (!IntPtr.Zero.Equals(ICS.WhitePoint))
         {
            CS.WhitePoint = new float[3];
            Marshal.Copy(ICS.WhitePoint, CS.WhitePoint, 0, 3);
         }else
         {
            CS.WhitePoint = null;
         }
         if (!IntPtr.Zero.Equals(ICS.Gamma))
         {
            CS.Gamma = new float[ICS.NumInComponents];
            Marshal.Copy(ICS.Gamma, CS.Gamma, 0, ICS.NumInComponents);
         }else
         {
            CS.Gamma = null;
         }
         if (!IntPtr.Zero.Equals(ICS.Range))
         {
            if (ICS.Type == TExtColorSpace.esLab)
            {
               CS.Range = new float[4];
               Marshal.Copy(ICS.Range, CS.Range, 0, 4);
            }else
            {
               CS.Range = new float[ICS.NumInComponents * 2];
               Marshal.Copy(ICS.Range, CS.Range, 0, ICS.NumInComponents * 2);
            }
         }else
         {
            CS.Range = null;
         }
         if (!IntPtr.Zero.Equals(ICS.Matrix))
         {
            CS.Matrix = new float[9];
            Marshal.Copy(ICS.Matrix, CS.Matrix, 0, 9);
         }else
         {
            CS.Matrix = null;
         }
         if (ICS.ColorantsCount == 0)
            CS.Colorants = null;
         else
         {
            int len;
            byte[] bytes;
            CS.Colorants = new String[ICS.ColorantsCount];
            // Convert the incoming UTF-8 strings to Unicode
            for(i = 0; i < ICS.ColorantsCount; i++)
            {
               len = pdfStrLenA(ICS.Colorants[i]);
               bytes = new byte[len];
               Marshal.Copy(ICS.Colorants[i], bytes, 0, len);
               bytes = System.Text.Encoding.Convert(System.Text.Encoding.UTF8, System.Text.Encoding.Unicode, bytes);
               CS.Colorants[i] = System.Text.Encoding.Unicode.GetString(bytes);
            }
         }
      }

      private void GetIntField(TPDFField_I IField, ref TPDFField Field)
      {
         Field.BackColor   = IField.BackColor;
         Field.BackCS      = IField.BackCS;
         Field.BBox        = IField.BBox;
         Field.BorderColor = IField.BorderColor;
         Field.Checked     = IField.Checked != 0;
         Field.Deleted     = IField.Deleted != 0;
         Field.FieldType   = IField.FieldType;
         Field.Font        = ToString(IField.Font, false);
         Field.FontSize    = IField.FontSize;
         Field.Handle      = IField.Handle;
         Field.KidCount    = IField.KidCount;
         Field.Parent      = IField.Parent;
         Field.TextColor   = IField.TextColor;
         Field.TextCS      = IField.TextCS;
         Field.ToolTip     = ToString(IField.ToolTip, IField.ToolTipLen, IField.UniToolTip != 0);
         Field.Value       = ToString(IField.Value, IField.ValLen, IField.UniVal != 0);
         int len = pdfStrLenA(IField.FieldName);
         Field.FieldName = ToString(IField.FieldName, IField.FieldNameLen, len != IField.FieldNameLen);
      }

      private void GetIntFieldEx(TPDFFieldEx_I IField, ref TPDFFieldEx Field)
      {
         Field.BackColor     = IField.BackColor;
         Field.BackColorSP   = IField.BackColorSP;
         Field.BBox          = IField.BBox;
         Field.BorderColor   = IField.BorderColor;
         Field.BorderColorSP = IField.BorderColorSP;
         Field.BorderStyle   = IField.BorderStyle;
         Field.BorderWidth   = IField.BorderWidth;
         Field.CharSpacing   = IField.CharSpacing;
         Field.CheckBoxChar  = IField.CheckBoxChar;
         Field.Checked       = IField.Checked != 0;
         Field.DefState      = IField.DefState;
         Field.DefValue      = ToString(IField.DefValueA, IField.DefValueW);
         Field.Deleted       = IField.Deleted != 0;
         Field.EditFont      = ToString(IField.EditFont, false);
         Field.ExpValCount   = IField.ExpValCount;
         Field.ExpValue      = ToString(IField.ExpValueA, IField.ExpValueW);
         Field.FieldFlags    = IField.FieldFlags;
         Field.FieldFont     = ToString(IField.FieldFont, false);
         Field.FieldName     = ToString(IField.FieldNameA, IField.FieldNameW);
         Field.FieldType     = IField.FieldType;
         Field.FontSize      = IField.FontSize;
         Field.GroupType     = IField.GroupType;
         Field.Handle        = IField.Handle;
         Field.HighlightMode = IField.HighlightMode;
         Field.IEditFont     = IField.IEditFont;
         Field.IFieldFont    = IField.IFieldFont;
         Field.IsCalcField   = IField.IsCalcField != 0;
         Field.Kids          = CopyIntPtrArray(IField.Kids, IField.KidCount);
         Field.MapName       = ToString(IField.MapNameA, IField.MapNameW);
         Field.MaxLen        = IField.MaxLen;
         Field.PageNum       = IField.PageNum;
         Field.Parent        = IField.Parent;
         Field.Rotate        = IField.Rotate;
         Field.TextAlign     = IField.TextAlign;
         Field.TextColor     = IField.TextColor;
         Field.TextColorSP   = IField.TextColorSP;
         Field.TextScaling   = IField.TextScaling;
         Field.ToolTip       = ToString(IField.ToolTipA, IField.ToolTipW);
         Field.UniqueName    = ToString(IField.UniqueNameA, IField.UniqueNameW);
         Field.Value         = ToString(IField.ValueA, IField.ValueW);
         Field.WordSpacing   = IField.WordSpacing;
      }

      private void GetIntFont(TPDFFontObj_I IFont, ref TPDFFontObj F)
      {
         F.Ascent       = IFont.Ascent;
         F.BaseFont     = ToString(IFont.BaseFont, false);
         F.CapHeight    = IFont.CapHeight;
         F.DefWidth     = IFont.DefWidth;
         F.Descent      = IFont.Descent;
         F.FirstChar    = IFont.FirstChar;
         F.Flags        = IFont.Flags;
         F.FontFamily   = ToString(IFont.FontFamily, IFont.FontFamilyUni != 0);
         F.FontName     = ToString(IFont.FontName, false);
         F.FontType     = IFont.FontType;
         F.ItalicAngle  = IFont.ItalicAngle;
         F.LastChar     = IFont.LastChar;
         F.SpaceWidth   = IFont.SpaceWidth;
         F.XHeight      = IFont.XHeight;
         // We don't copy the buffer here! Copy the buffer manually with MemCopy() if you need it!
         // However, if F.Length1 == 0 and if !IntPtr.Zero.Equals(F.FontFile) then the variable contains
         // the file path to the used IFont file. This string is always an Ansi string at this time.
         F.FontFile     = IFont.FontFile;
         F.Length1      = IFont.Length1;
         F.Length2      = IFont.Length2;
         F.Length3      = IFont.Length3;
         F.FontFileType = IFont.FontFileType;
         if (!IntPtr.Zero.Equals(IFont.Encoding))
         {
            String s = ToString(IFont.Encoding, 256, true);
            F.Encoding = s.ToCharArray();
         }else
            F.Encoding = null;
         if (IFont.WidthsCount > 0)
         {
            F.Widths = new float[IFont.WidthsCount];
            Marshal.Copy(IFont.Widths, F.Widths, 0, IFont.WidthsCount);
         }else
            F.Widths = null;
      }

      private String ToString(IntPtr Ptr, bool Unicode)
      {
         if (IntPtr.Zero.Equals(Ptr)) return null;
         if (!Unicode)
            return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(Ptr);
         else
            return System.Runtime.InteropServices.Marshal.PtrToStringUni(Ptr);
      }

      private String ToString(IntPtr Ptr, int Len, bool Unicode)
      {
         if (IntPtr.Zero.Equals(Ptr) || Len <= 0) return null;
         if (!Unicode)
            return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(Ptr, Len);
         else
            return System.Runtime.InteropServices.Marshal.PtrToStringUni(Ptr, Len);
      }

      private String ToString(IntPtr PtrA, IntPtr PtrW)
      {
         if (!IntPtr.Zero.Equals(PtrA))
            return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(PtrA);
         else if (!IntPtr.Zero.Equals(PtrW))
            return System.Runtime.InteropServices.Marshal.PtrToStringUni(PtrW);
         else
            return null;
      }

      private String ToString(IntPtr PtrA, IntPtr PtrW, int Len)
      {
         if (!IntPtr.Zero.Equals(PtrA))
            return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(PtrA, Len);
         else if (!IntPtr.Zero.Equals(PtrW))
            return System.Runtime.InteropServices.Marshal.PtrToStringUni(PtrW, Len);
         else
            return null;
      }

      private IntPtr               m_Instance;
      // We must save the pointer of a callback function because the .Net framework changes the pointer otherwise
      // when it reallocates memory the next time.
      private TErrorProc           m_ErrorProc;
      private TEnumFontProc        m_EnumFontProc;
      private TEnumFontProc2       m_EnumFontProc2;
      private TEnumFontProcEx      m_EnumFontProcEx;
      private TOnFontNotFoundProc  m_FontNotFoundProc;
      private TInitProgressProc    m_InitProgressProc;
      private TOnPageBreakProc     m_OnPageBreakProc;
      private TOnReplaceICCProfile m_OnReplaceICCProfile;
      private TProgressProc        m_ProgressProc;

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TBookmark_I
      {
         public uint      Color;
         public int       DestPage;
         public TPDFRect  DestPos;
         public TDestType DestType;
         public lbool     Open;
         public int       Parent;
         public TFStyle   Style;
         public IntPtr    Title;
         public int       TitleLen;
         public lbool     Unicode;
      }

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFAnnotation_I
      {
         public TAnnotType      Type;
         public lbool           Deleted;
         public TPDFRect        BBox;
         public double          BorderWidth;
         public uint            BorderColor;
         public TBorderStyle    BorderStyle;
         public uint            BackColor;
         public int             Handle;
         public IntPtr          AuthorA;
         public IntPtr          AuthorW;
         public IntPtr          ContentA;
         public IntPtr          ContentW;
         public IntPtr          NameA;
         public IntPtr          NameW;
         public IntPtr          SubjectA;
         public IntPtr          SubjectW;
         public int             PageNum;
         public THighlightMode  HighlightMode;
      }

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFAnnotationEx_I
      {
         public TAnnotType      Type;
         public lbool           Deleted;
         public TPDFRect        BBox;
         public float           BorderWidth;
         public uint            BorderColor;
         public TBorderStyle    BorderStyle;
         public uint            BackColor;
         public int             Handle;
         public IntPtr          AuthorA;
         public IntPtr          AuthorW;
         public IntPtr          ContentA;
         public IntPtr          ContentW;
         public IntPtr          NameA;
         public IntPtr          NameW;
         public IntPtr          SubjectA;
         public IntPtr          SubjectW;
         public int             PageNum;
         public THighlightMode  HighlightMode;
         // Page link annotations only
         public int             DestPage;
         public TPDFRect        DestPos;
         public TDestType       DestType;
         // File link or web link annotations
         public IntPtr          DestFile;
         public int             Icon;        // The Icon type depends on the annotation type. If the annotation type is atText then the Icon
                                             // is of type TAnnotIcon. If the annotation type is atFileAttach then it is of type
                                             // TFileAttachIcon. If the annotation type is atStamp then the Icon is the stamp type (TRubberStamp).
                                             // For any other annotation type this value is not set (-1).
         public IntPtr          StampName;   // Set only, if Icon == rsUserDefined
         public uint            AnnotFlags;  // See TAnnotFlags for available flags
         public IntPtr          CreateDate;  // Creation Date -> Optional
         public IntPtr          ModDate;     // Modification Date -> Optional
         public lbool           Grouped;     // (Reply type) Meaningful only if Parent != -1 and Type != atPopUp. If true,
                                             // the annotation is part of an annotation group. Properties like Content, CreateDate,
                                             // ModDate, BackColor, Subject, and Open must be taken from the parent annotation.
         public lbool           Open;        // Meaningful only for annotations which have a corresponding PopUp annotation.
         public int             Parent;      // Parent annotation handle of a PopUp Annotation or the parent annotation if
                                             // this annotation represents a state of a base annotation. In this case,
                                             // the annotation type is always atText and only the following members should
                                             // be considered:
                                             //    State      // The current state
                                             //    StateModel // Marked, Review, and so on
                                             //    CreateDate // Creation Date
                                             //    ModDate    // Modification Date
                                             //    Author     // The user who has set the state
                                             //    Content    // Not displayed in Adobe's Acrobat...
                                             //    Subject    // Not displayed in Adobe's Acrobat...
                                             // The PopUp annotation of a text annotation which represent an Annotation State
                                             // must be ignored.
         public int             PopUp;       // Handle of the corresponding PopUp annotation if any.
         public IntPtr          State;       // The state of the annotation.
         public IntPtr          StateModel;  // The state model (Marked, Review, and so on).
         public int             EmbeddedFile;// FileAttach annotations only. A handle of an embedded file -> GetEmbeddedFile().
         public IntPtr          Subtype;     // Set only, if Type == atUnknownAnnot

         // Reserved fields for future extensions
         public IntPtr          Reserved01;
         public IntPtr          Reserved02;
         public IntPtr          Reserved03;
         public IntPtr          Reserved04;
         public IntPtr          Reserved05;
         public IntPtr          Reserved06;
         public IntPtr          Reserved07;
         public IntPtr          Reserved08;
         public IntPtr          Reserved09;
         public IntPtr          Reserved10;
         public IntPtr          Reserved11;
         public IntPtr          Reserved12;
         public IntPtr          Reserved13;
         public IntPtr          Reserved14;
         public IntPtr          Reserved15;
         public IntPtr          Reserved16;
         public IntPtr          Reserved17;
         public IntPtr          Reserved18;
         public IntPtr          Reserved19;
         public IntPtr          Reserved20;
         public IntPtr          Reserved21;
         public IntPtr          Reserved22;
         public IntPtr          Reserved23;
         public IntPtr          Reserved24;
         public IntPtr          Reserved25;
         public IntPtr          Reserved26;
         public IntPtr          Reserved27;
      }

      [StructLayout(LayoutKind.Sequential, Pack = 0)]
      private struct TPDFChoiceValue_I
      {
         public int    StructSize; // Must be set to sizeof(TPDFChoiceValue)
         public IntPtr ExpValueA;
         public IntPtr ExpValueW;
         public int    ExpValueLen;
         public IntPtr ValueA;
         public IntPtr ValueW;
         public int    ValueLen;
         public lbool  Selected;
      }

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFColorSpaceObj_I
      {
         public TExtColorSpace Type;
         public TExtColorSpace Alternate;        // Alternate color space or base space of an Indexed or Pattern color space.
         public IntPtr         IAlternate;       // Only set if the color space contains an alternate or base color space -> GetColorSpaceObjEx().
         public IntPtr         Buffer;           // Contains either an ICC profile or the color table of an Indexed color space.
         public int            BufSize;          // Buffer length in bytes.
         public IntPtr         BlackPoint;       // CIE blackpoint. If set, the array contains exactly 3 values.
         public IntPtr         WhitePoint;       // CIE whitepoint. If set, the array contains exactly 3 values.
         public IntPtr         Gamma;            // If set, one value per component.
         public IntPtr         Range;            // min/max for each component or for the .a and .b components of a Lab color space.
         public IntPtr         Matrix;           // XYZ matrix. If set, the array contains exactly 9 values.
         public int            NumInComponents;  // Number of input components.
         public int            NumOutComponents; // Number of output components.
         public int            NumColors;        // HiVal + 1 as specified in the color space. Indexed color space only.
         [MarshalAs(UnmanagedType.ByValArray, SizeConst=32)]
         public IntPtr[]       Colorants;        // UTF-8 Colorant names (Separation, DeviceN, and NChannel only).
         public int            ColorantsCount;   // The number of colorants in the array.
         public IntPtr         Metadata;         // Optional XMP metadata stream -> ICCBased only.
         public int            MetadataSize;     // Metadata length in bytes.
         public IntPtr         IFunction;        // Pointer to function object -> Separation, DeviceN, and NChannel only.
         public IntPtr         IAttributes;      // Optional attributes of DeviceN or NChannel color spaces -> GetNChannelAttributes().
         public IntPtr         IColorSpaceObj;   // Pointer of the corresponding color space object
         public IntPtr         Reserved01;
         public IntPtr         Reserved02;
         public IntPtr         Reserved03;
         public IntPtr         Reserved04;
         public IntPtr         Reserved05;
         public IntPtr         Reserved06;
         public IntPtr         Reserved07;
         public IntPtr         Reserved08;
         public IntPtr         Reserved09;
      };

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TDeviceNAttributes_I
      {
         public IntPtr IProcessColorSpace;    // Pointer to process color space or NULL -> GetColorSpaceEx().
                                              // Does a process color space with more than 8 components exist? 6 components
                                              // is the maximum so far I know. However, 8 components should be large enough for
                                              // the next years...
         [MarshalAs(UnmanagedType.ByValArray, SizeConst=8)]
         public IntPtr[] ProcessColorants; // UTF-8 strings
         public int    ProcessColorantsCount; // Number of process colorants in the array or zero if not set.
         [MarshalAs(UnmanagedType.ByValArray, SizeConst=32)]
         public IntPtr[] Separations; // Optional pointers to Separation color spaces -> GetColorSpaceEx().
         public int    SeparationsCount;      // Number of color spaces in the array.
         public IntPtr IMixingHints;          // Optional mixing hints. There is no API function at this time to access mixing hints.
         public IntPtr Reserved01;
         public IntPtr Reserved02;
         public IntPtr Reserved03;
         public IntPtr Reserved04;
      };

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFField_I
      {
         public TFieldType     FieldType;
         public lbool          Deleted;
         public TPDFRect       BBox;
         public int            Handle;
         public IntPtr         FieldName;
         public int            FieldNameLen;
         public TPDFColorSpace BackCS;
         public TPDFColorSpace TextCS;
         public uint           BackColor;
         public uint           BorderColor;
         public uint           TextColor;
         public lbool          Checked;
         public int            Parent;
         public int            KidCount;
         public IntPtr         Font;
         public double         FontSize;
         public IntPtr         Value;
         public lbool          UniVal;
         public int            ValLen;
         public IntPtr         ToolTip;
         public lbool          UniToolTip;
         public int            ToolTipLen;
      }

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFFieldEx_I
      {
         public int             StructSize;    // Must be set to sizeof(TPDFFieldEx)
         public lbool           Deleted;       // If true, the field was marked as deleted by DeleteField()
         public TPDFRect        BBox;          // Bounding box of the field in bottom-up coordinates
         public TFieldType      FieldType;     // Field type
         public TFieldType      GroupType;     // If GroupType != FieldType the field is a terminal field of a field group
         public int             Handle;        // Field handle
         public uint            BackColor;     // Background color
         public TExtColorSpace  BackColorSP;   // Color space of the background color
         public uint            BorderColor;   // Border color
         public TExtColorSpace  BorderColorSP; // Color space of the border color
         public TBorderStyle    BorderStyle;   // Border style
         public float           BorderWidth;   // Border width
         public float           CharSpacing;   // Text fields only
         public lbool           Checked;       // Check boxes only
         public int             CheckBoxChar;  // ZapfDingbats character that is used to display the on state
         public TCheckBoxState  DefState;      // Check boxes only
         public IntPtr          DefValueA;     // Optional default value
         public IntPtr          DefValueW;     // Optional default value
         public IntPtr          IEditFont;     // Pointer to default editing font
         public IntPtr          EditFont;      // Postscript name of the editing font
         public int             ExpValCount;   // Combo and list boxes only. The values can be accessed with GetFieldExpValueEx()
         public IntPtr          ExpValueA;     // Check boxes only
         public IntPtr          ExpValueW;     // Check boxes only
         public TFieldFlags     FieldFlags;    // Field flags
         public IntPtr          IFieldFont;    // Pointer to the font that is used by the field
         public IntPtr          FieldFont;     // Postscript name of the font
         private int            Reserved;
         public double          FontSize;      // Font size. 0.0 means auto font size
         public IntPtr          FieldNameA;    // Note that children of a field group or radio button have no name
         public IntPtr          FieldNameW;    // Field name length in characters
         public THighlightMode  HighlightMode; // Highlight mode
         public lbool           IsCalcField;   // If true, the OnCalc event of the field is connected with a JavaScript action
         public IntPtr          MapNameA;      // Optional unique mapping name of the field
         public IntPtr          MapNameW;      // MapName length in characters
         public int             MaxLen;        // Text fields only -> zero means not restricted
         public IntPtr          Kids;          // Array of child fields -> GetFieldEx2()
         public int             KidCount;      // Number of fields in the array
         public IntPtr          Parent;        // Pointer to parent field or NULL
         public int             PageNum;       // Page on which the field is used or -1
         public int             Rotate;        // Rotation angle in degrees
         public TTextAlign      TextAlign;     // Text fields only
         public uint            TextColor;     // Text color
         public TExtColorSpace  TextColorSP;   // Color space of the field's text
         public float           TextScaling;   // Text fields only
         public IntPtr          ToolTipA;      // Optional tool tip
         public IntPtr          ToolTipW;      // Optional tool tip
         public IntPtr          UniqueNameA;   // Optional unique name (NM key)
         public IntPtr          UniqueNameW;   // Optional unique name (NM key)
         public IntPtr          ValueA;        // Field value
         public IntPtr          ValueW;        // Field value
         public float           WordSpacing;   // Text fields only
      }

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFFileSpec_I
      {
         public IntPtr Buffer;      // Buffer of an embedded file
         public int    BufSize;     // Buffer size in bytes.
         public lbool  Compressed;  // Should be false if Decompress was true in the GetEmbeddedFile() call, otherwise usually true.
                                    // DynaPDF decompresses Flate encoded streams only. Other filters can occur but this is very unusual.
         public IntPtr ColItem;     // If != IntPtr.Zero the embedded file contains a collection item with user defined data. This entry
                                    // can occur in PDF Collections only (PDF 1.7). See "PDF Collections" in the help file for further
                                    // information.
         public IntPtr Name;        // Name of the file specification in the name tree. This value is always present.
         public lbool  NameUnicode; // Is Name in Unicode format?
         public IntPtr FileName;    // File name as 7 bit ASCII string.
         public lbool  IsURL;       // If true, FileName contains a URL.
         public IntPtr UF;          // PDF 1.7. Same as FileName but Unicode is allowed.
         public lbool  UFUnicode;   // Is UF in Unicode format?
         public IntPtr Desc;        // Description
         public lbool  DescUnicode; // Is Desc in Unicode format?
         public int    FileSize;    // Size of the decompressed stream or zero if not known. Note: this is either the Size key of
                                    // the Params dictionary if present or the DL key in the file stream. Whether this value is
                                    // correct depends on the file creator! The parameter is definitely correct if the file was
                                    // decompressed.
         public IntPtr MIMEType;    // MIME media type name as defined in Internet RFC 2046.
         public IntPtr CreateDate;  // Creation date as string. See help file "The standard date format".
         public IntPtr ModDate;     // Modification date as string. See help file "The standard date format".
         public IntPtr CheckSum;    // 16 byte MD5 digest. Note that this is a binary string. It is exactly 16 bytes long if set!
      }

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFFontObj_I
      {
         public float     Ascent;        // Ascent
         public IntPtr    BaseFont;      // PostScript Name or Family Name
         public float     CapHeight;     // Cap height
         public float     Descent;       // Descent
         public IntPtr    Encoding;      // Unicode mapping 0..255 -> not set if a CID font is selected
         public int       FirstChar;     // First char
         public uint      Flags;         // Font flags -> font descriptor
         public IntPtr    FontFamily;    // Optional Font Family (Family Name)
         public lbool     FontFamilyUni; // Is FontFamily in Unicode format?
         public IntPtr    FontName;      // Font name -> font descriptor
         public TFontType FontType;      // If ftType0 the font is a CID font. The Encoding is not set in this case.
         public float     ItalicAngle;   // Italic angle
         public int       LastChar;      // Last char
         public float     SpaceWidth;    // Space width in font units. A default value is set if the font contains no space character.
         public IntPtr    Widths;        // Glyph widths -> 0..WidthsCount -1
         public int       WidthsCount;   // Number of widths in the array
         public float     XHeight;       // x-height
         public float     DefWidth;      // Default character widths -> CID fonts only
         public IntPtr    FontFile;      // Font file buffer -> only imported fonts are returned.
         public int       Length1;       // Length of the clear text portion of the Type1 font, or the length of the entire font program if FontType != ffType1.
         public int       Length2;       // Length of the encrypted portion of the Type1 font program (Type1 fonts only).
         public int       Length3;       // Length of the fixed-content portion of the Type1 font program or zero if not present.
         public TFontFileSubtype FontFileType;  // See description in the help file for further information.
      };

      // Named Destination
      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFNamedDest_I
      {
         public IntPtr    NameA;
         public IntPtr    NameW;
         public int       NameLen;
         public int       DestPage;
         public TPDFRect  DestPos;
         public TDestType DestType;
      };

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFPageLabel_I
      {
         public int              StartRange;
         public TPageLabelFormat Format;
         public int              FirstPageNum;
         public IntPtr           Prefix;
         public int              PrefixLen;
         public lbool            PrefixUni;
      };

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFPrintSettings_I
      {
         public TDuplexMode   DuplexMode;
         public int           NumCopies;         // -1 means not set. Values larger than 5 are ignored in viewer applications.
         public int           PickTrayByPDFSize; // -1 means not set. 0 == false, 1 == true.
         public IntPtr        PrintRanges;       // If set, the array contains PrintRangesCount * 2 values. Each pair consists
                                                 // of the first and last pages in the sub-range. The first page in the PDF file
                                                 // is denoted by 0.
         public int           PrintRangesCount;  // Number of ranges available in PrintRanges.
         public TPrintScaling PrintScaling;      // dpmNone means not set.
         /* Reserved fields for future extensions*/
         int Reserved0;
         int Reserved1;
         int Reserved2;
         int Reserved3;
         int Reserved4;
         int Reserved5;
         int Reserved6;
         int Reserved7;
         int Reserved8;
         int Reserved9;
      };

      [StructLayout(LayoutKind.Sequential, Pack=0)]
      private struct TPDFSigParms_I
      {
         public int           StructSize;   // Must be set to sizeof(TSignParms)
         public int           PKCS7ObjLen;  // The maximum length of the signed PKCS#7 object
         public THashType     HashType;     // If set to htDetached, the bytes ranges of the PDF file will be returned.
         public IntPtr        Range1;       // Out -> Contains either the hash or the first byte range to create a detached signature
         public int           Range1Len;    // Out -> Length of the buffer
         public IntPtr        Range2;       // Out -> Set only if HashType == htDetached
         public int           Range2Len;    // Out -> Length of the buffer
         [MarshalAs(UnmanagedType.LPStr)]
         public String        ContactInfoA; // Optional, e.g. an email address
         [MarshalAs(UnmanagedType.LPWStr)]
         public String        ContactInfoW; // Optional, e.g. an email address
         [MarshalAs(UnmanagedType.LPStr)]
         public String        LocationA;    // Optional location of the signer
         [MarshalAs(UnmanagedType.LPWStr)]
         public String        LocationW;    // Optional location of the signer
         [MarshalAs(UnmanagedType.LPStr)]
         public String        ReasonA;      // Optional reason why the file was signed
         [MarshalAs(UnmanagedType.LPWStr)]
         public String        ReasonW;      // Optional reason why the file was signed
         [MarshalAs(UnmanagedType.LPStr)]
         public String        SignerA;      // Optional, the issuer of the certificate takes precedence
         [MarshalAs(UnmanagedType.LPWStr)]
         public String        SignerW;      // Optional, the issuer of the certificate takes precedence
         public lbool         Encrypt;      // If true, the file will be encrypted
         // These members will be ignored if Encrypt is set to false
         [MarshalAs(UnmanagedType.LPStr)]
         public String        OpenPwd;      // Open password
         [MarshalAs(UnmanagedType.LPStr)]
         public String        OwnerPwd;     // Owner password to change the security settings
         public TKeyLen       KeyLen;       // Key length to be used to encrypt the file
         public TRestrictions Restrict;     // What should be restricted?
      };

      // The Unicode version of a function takes precedence over the Ansi version in the .Net framework.
      // This behaviour makes it impossible to declare overloaded string functions as "Auto" because only
      // the Unicode version would be called.
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  fntBuildFamilyNameAndStyle(IntPtr IFont, System.Text.StringBuilder Name, ref TFStyle Style);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  fntGetFont(IntPtr IFont, ref TPDFFontObj_I F);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double fntGetSpaceWidth(IntPtr IFont, double FontSize);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double fntGetTextWidth(IntPtr IFont, IntPtr Text, int Len, float CharSpacing, float WordSpacing, float TextScale);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    fntTranslateRawCode(IntPtr IFont, IntPtr Text, int Len, ref double Width, byte[] OutText, ref int OutLen, ref lbool Decoded, float CharSpacing, float WordSpacing, float TextScale);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    fntTranslateString(ref TPDFStack Stack, byte[] Buffer, int Size, int Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    fntTranslateString2(IntPtr IFont, IntPtr Text, int Len, byte[] OutText, int Size, int Flags);

      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddActionToObj(IntPtr IPDF, TObjType ObjType, TObjEvent Event, int ActHandle, int ObjHandle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfAddArticle(IntPtr IPDF, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfAddBookmarkA(IntPtr IPDF, String Title, int Parent, int DestPage, lbool Open);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfAddBookmarkW(IntPtr IPDF, String Title, int Parent, int DestPage, lbool Open);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfAddBookmarkExA(IntPtr IPDF, String Title, int Parent, int NamedDest, lbool Open);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfAddBookmarkExW(IntPtr IPDF, String Title, int Parent, int NamedDest, lbool Open);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddButtonImage(IntPtr IPDF, int BtnHandle, TButtonState State, String Caption, String ImgFile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddButtonImageEx(IntPtr IPDF, int BtnHandle, TButtonState State, String Caption, IntPtr hBitmap);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddContinueTextA(IntPtr IPDF, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfAddContinueTextW(IntPtr IPDF, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddDeviceNProcessColorants(IntPtr IPDF, int DeviceNCS, String[] Colorants, int NumColorants, TExtColorSpace ProcessCS, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddDeviceNSeparations(IntPtr IPDF, int DeviceNCS, String[] Colorants, int[] SeparationCS, int NumColorants);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddFieldToFormAction(IntPtr IPDF, int Action, int Field, lbool Include);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddFieldToHideAction(IntPtr IPDF, int HideAct, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddFileCommentA(IntPtr IPDF, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfAddFileCommentW(IntPtr IPDF, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfAddFontSearchPathA(IntPtr IPDF, String APath, lbool Recursive);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfAddFontSearchPathW(IntPtr IPDF, String APath, lbool Recursive);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddImage(IntPtr IPDF, TCompressionFilter Filter, TImageConversionFlags Flags, ref TPDFImage Image);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfAddJavaScriptA(IntPtr IPDF, String Name, String Script);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfAddJavaScriptW(IntPtr IPDF, [MarshalAs(UnmanagedType.LPStr)]String Name, [MarshalAs(UnmanagedType.LPWStr)]String Script);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddObjectToLayer(IntPtr IPDF, int OCG, TOCObject ObjType, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfAddPageLabelA(IntPtr IPDF, int StartRange, TPageLabelFormat Format, String Prefix, int AddNum);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfAddPageLabelW(IntPtr IPDF, int StartRange, TPageLabelFormat Format, String Prefix, int AddNum);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfAddRenderingIntentA(IntPtr IPDF, String ICCFile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfAddRenderingIntentW(IntPtr IPDF, String ICCFile);
      [DllImport("dynapdf.dll", CharSet = CharSet.Ansi)] private static extern int    pdfAddRenderingIntentEx(IntPtr IPDF, byte[] Buffer, int BufSize);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAddValToChoiceFieldA(IntPtr IPDF, int Field, String ExpValue, String Value, lbool Selected);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfAddValToChoiceFieldW(IntPtr IPDF, int Field, [MarshalAs(UnmanagedType.LPStr)]String ExpValue, [MarshalAs(UnmanagedType.LPWStr)]String Value, lbool Selected);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAppend(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfApplyPattern(IntPtr IPDF, int PattHandle, TColorMode ColorMode, uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfApplyShading(IntPtr IPDF, int ShadHandle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfAttachFileA(IntPtr IPDF, String FilePath, String Description, lbool Compress);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfAttachFileW(IntPtr IPDF, String FilePath, String Description, lbool Compress);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfAttachFileExA(IntPtr IPDF, byte[] Buffer, int BufSize, String FileName, String Description, lbool Compress);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfAttachFileExW(IntPtr IPDF, byte[] Buffer, int BufSize, String FileName, String Description, lbool Compress);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfAutoTemplate(IntPtr IPDF, int Templ, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfBeginClipPath(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfBeginContinueText(IntPtr IPDF, double PosX, double PosY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfBeginLayer(IntPtr IPDF, int OCG);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfBeginPattern(IntPtr IPDF, TPatternType PatternType, TTilingType TilingType, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfBeginTemplate(IntPtr IPDF, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfBeginTemplateEx(IntPtr IPDF, ref TPDFRect BBox, ref TCTM Matrix);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfBezier_1_2_3(IntPtr IPDF, double x1, double y1, double x2, double y2, double x3, double y3);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfBezier_1_3(IntPtr IPDF, double x1, double y1, double x3, double y3);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfBezier_2_3(IntPtr IPDF, double x2, double y2, double x3, double y3);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfCalcWidthHeight(IntPtr IPDF, double OrgWidth, double OrgHeight, double ScaledWidth, double ScaledHeight);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeAnnotPos(IntPtr IPDF, int Handle, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeAnnotNameA(IntPtr IPDF, int Handle, String Name);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfChangeAnnotNameW(IntPtr IPDF, int Handle, String Name);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeBookmarkA(IntPtr IPDF, int ABmk, String Title, int Parent, int DestPage, lbool Open);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfChangeBookmarkW(IntPtr IPDF, int ABmk, String Title, int Parent, int DestPage, lbool Open);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeFont(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeFontSize(IntPtr IPDF, double Size);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeFontStyle(IntPtr IPDF, TFStyle Style);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeFontStyleEx(IntPtr IPDF, TFStyle Style);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeJavaScriptA(IntPtr IPDF, int Handle, String NewScript);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfChangeJavaScriptW(IntPtr IPDF, int Handle, String NewScript);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeJavaScriptActionA(IntPtr IPDF, int Handle, String NewScript);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfChangeJavaScriptActionW(IntPtr IPDF, int Handle, String NewScript);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeJavaScriptNameA(IntPtr IPDF, int Handle, String Name);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfChangeJavaScriptNameW(IntPtr IPDF, int Handle, String Name);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfChangeLinkAnnot(IntPtr IPDF, int Handle, String URL);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCheckAndRepairFonts(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCheckCollection(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCheckConformance(IntPtr IPDF, TConformanceType Type, TCheckOptions Options, IntPtr UserData, TOnFontNotFoundProc OnFontNotFound, TOnReplaceICCProfile OnReplaceICCProfile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCheckFieldNames(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfClearAutoTemplates(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfClearHostFonts(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfClipPath(IntPtr IPDF, TClippingMode ClipMode, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCloseAndSignFile(IntPtr IPDF, String CertFile, String Password, String Reason, String Location);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCloseAndSignFileEx(IntPtr IPDF, String OpenPwd, String OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict, String CertFile, String Password, String Reason, String Location);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCloseAndSignFileExt(IntPtr IPDF, ref TPDFSigParms_I SigParms);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCloseFile(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCloseFileEx(IntPtr IPDF, String OpenPwd, String OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCloseImage(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCloseImportFile(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfClosePath(IntPtr IPDF, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCloseTag(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfComputeBBox(IntPtr IPDF, ref TPDFRect BBox, TCompBBoxFlags Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfConvColor(IntPtr Color, int NumComps, TExtColorSpace SourceCS, IntPtr IColorSpace, TExtColorSpace DestCS);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfConvertColors(IntPtr IPDF, TColorConvFlags Flags, IntPtr Reserved);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfConvToUnicode(IntPtr IPDF, String AString, TCodepage CP);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreate3DAnnot(IntPtr IPDF, double PosX, double PosY, double Width, double Height, String Author, String Name, String U3DFile, String Image);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCreate3DBackground(IntPtr IPDF, IntPtr IView, uint BackColor);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreate3DGotoViewAction(IntPtr IPDF, int Base3DAnnot, IntPtr IView, T3DNamedAction Named);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCreate3DProjection(IntPtr IPDF, IntPtr IView, T3DProjType ProjType, T3DScaleType ScaleType, double Diameter, double FOV);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfCreate3DView(IntPtr IPDF, int Base3DAnnot, String Name, lbool SetAsDefault, ref double[] Matrix, double CamDistance, T3DRenderingMode RM, T3DLightingSheme LS);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateAnnotAP(IntPtr IPDF, int Annot);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateArticleThreadA(IntPtr IPDF, String ThreadName);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfCreateArticleThreadW(IntPtr IPDF, String ThreadName);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateAxialShading(IntPtr IPDF, double sX, double sY, double eX, double eY, double SCenter, uint SColor, uint EColor, lbool Extend1, lbool Extend2);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateButtonA(IntPtr IPDF, String Name, String Caption, int Parent, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfCreateButtonW(IntPtr IPDF, [MarshalAs(UnmanagedType.LPStr)]String Name, [MarshalAs(UnmanagedType.LPWStr)]String Caption, int Parent, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateCheckBox(IntPtr IPDF, String Name, String ExpValue, lbool Checked, int Parent, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateCIEColorSpace(IntPtr IPDF, TExtColorSpace Base, float[] WhitePoint, float[] BlackPoint, float[] Gamma, float[] Matrix);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCreateCollection(IntPtr IPDF, TColView View);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateCollectionFieldA(IntPtr IPDF, TColColumnType ColType, int Column, String Name, String Key, lbool Visible, lbool Editable);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfCreateCollectionFieldW(IntPtr IPDF, TColColumnType ColType, int Column, [MarshalAs(UnmanagedType.LPWStr)]String Name, [MarshalAs(UnmanagedType.LPStr)]String Key, lbool Visible, lbool Editable);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCreateColItemDate(IntPtr IPDF, int EmbFile, String Key, long Date, String Prefix);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCreateColItemNumber(IntPtr IPDF, int EmbFile, String Key, double Value, String Prefix);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCreateColItemStringA(IntPtr IPDF, int EmbFile, String Key, String Value, String Prefix);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfCreateColItemStringW(IntPtr IPDF, int EmbFile, [MarshalAs(UnmanagedType.LPStr)]String Key, [MarshalAs(UnmanagedType.LPWStr)]String Value, [MarshalAs(UnmanagedType.LPWStr)]String Prefix);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateComboBox(IntPtr IPDF, String Name, lbool Sort, int Parent, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateDeviceNColorSpace(IntPtr IPDF, String[] Colorants, int NumColorants, String PostScriptFunc, TExtColorSpace Alternate, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateExtGState(IntPtr IPDF, ref TPDFExtGState GS);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateGoToAction(IntPtr IPDF, TDestType DestType, int PageNum, double a, double b, double c, double d);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateGoToActionEx(IntPtr IPDF, int NamedDest);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateGoToRAction(IntPtr IPDF, String FileName, int PageNum);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateGoToRActionExA(IntPtr IPDF, String FileName, String DestName, lbool NewWindow);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfCreateGoToRActionExW(IntPtr IPDF, [MarshalAs(UnmanagedType.LPStr)]String FileName, [MarshalAs(UnmanagedType.LPWStr)]String DestName, lbool NewWindow);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateGroupField(IntPtr IPDF, String Name, int Parent);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateHideAction(IntPtr IPDF, int Field, lbool Hide);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateICCBasedColorSpaceA(IntPtr IPDF, String ICCProfile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfCreateICCBasedColorSpaceW(IntPtr IPDF, String ICCProfile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCreateImageA(IntPtr IPDF, String FileName, TImageFormat Format);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfCreateImageW(IntPtr IPDF, String FileName, TImageFormat Format);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateImpDataActionA(IntPtr IPDF, String DataFile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfCreateImpDataActionW(IntPtr IPDF, String DataFile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateIndexedColorSpace(IntPtr IPDF, TExtColorSpace Base, int Handle, byte[] ColorTable, int NumColors);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateJSActionA(IntPtr IPDF, String Script);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfCreateJSActionW(IntPtr IPDF, String Script);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateLaunchAction(IntPtr IPDF, TFileOP OP, String FileName, String DefDir, String Param, lbool NewWindow);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateListBox(IntPtr IPDF, String Name, lbool Sort, int Parent, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateNamedAction(IntPtr IPDF, TNamedAction Action);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateNamedDestA(IntPtr IPDF, String Name, int DestPage, TDestType DestType, double a, double b, double c, double d);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfCreateNamedDestW(IntPtr IPDF, String Name, int DestPage, TDestType DestType, double a, double b, double c, double d);
      [DllImport("dynapdf.dll", CharSet=CharSet.Auto)]private static extern    lbool  pdfCreateNewPDF(IntPtr IPDF, String OutPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateOCGA(IntPtr IPDF, String Name, lbool DisplayInUI, lbool Visible, TOCGIntent Intent);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfCreateOCGW(IntPtr IPDF, String Name, lbool DisplayInUI, lbool Visible, TOCGIntent Intent);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateOCMD(IntPtr IPDF, TOCVisibility Visibility, int[] OCGs, int Count);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateRadialShading(IntPtr IPDF, double sX, double sY, double R1, double eX, double eY, double R2, double SCenter, uint SColor, uint EColor, lbool Extend1, lbool Extend2);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateRadioButton(IntPtr IPDF, String Name, String ExpValue, int Checked, int Parent, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateResetAction(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateSeparationCS(IntPtr IPDF, String Colorant, TExtColorSpace Alternate, int Handle, uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateSetOCGStateAction(IntPtr IPDF, int[] On, int OnCount, int[] Off, int OffCount, int[] Toggle, int ToggleCount, lbool PreserveRB);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateSigField(IntPtr IPDF, String Name, int Parent, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateSigFieldAP(IntPtr IPDF, int SigField);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateStdPattern(IntPtr IPDF, TStdPattern Pattern, double LineWidth, double Distance, uint LineColor, uint BackColor);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfCreateStructureTree(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateSubmitAction(IntPtr IPDF, TSubmitFlags Flags, String URL);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateTextField(IntPtr IPDF, String Name, int Parent, lbool Multiline, int MaxLen, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfCreateURIAction(IntPtr IPDF, String URL);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfDecryptPDFA(IntPtr IPDF, String FileName, TPwdType PwdType, String Password);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfDecryptPDFW(IntPtr IPDF, [MarshalAs(UnmanagedType.LPWStr)]String FileName, TPwdType PwdType, [MarshalAs(UnmanagedType.LPStr)]String Password);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    void   pdfDeleteAcroForm(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeleteActionFromObj(IntPtr IPDF, TObjType ObjType, int ActHandle, int ObjHandle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeleteActionFromObjEx(IntPtr IPDF, TObjType ObjType, int ObjHandle, int ActIndex);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeleteAnnotation(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfDeleteBookmark(IntPtr IPDF, int ABmk);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeleteEmbeddedFile(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeleteField(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeleteFieldEx(IntPtr IPDF, String Name);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    void   pdfDeleteJavaScripts(IntPtr IPDF, lbool DelJavaScriptActions);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeletePage(IntPtr IPDF, int PageNum);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    void   pdfDeletePageLabels(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfDeletePDF(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeleteSeparationInfo(IntPtr IPDF, lbool AllPages);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeleteTemplate(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDeleteTemplateEx(IntPtr IPDF, int Index);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    void   pdfDeleteXFAForm(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDrawArc(IntPtr IPDF, double PosX, double PosY, double Radius, double StartAngle, double EndAngle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDrawArcEx(IntPtr IPDF, double PosX, double PosY, double Width, double Height, double StartAngle, double EndAngle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDrawChord(IntPtr IPDF, double PosX, double PosY, double Width, double Height, double StartAngle, double EndAngle, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDrawCircle(IntPtr IPDF, double PosX, double PosY, double Radius, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfDrawPie(IntPtr IPDF, double PosX, double PosY, double Width, double Height, double StartAngle, double EndAngle, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfEditPage(IntPtr IPDF, int PageNum);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfEditTemplate(IntPtr IPDF, int Index);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfEditTemplate2(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfEllipse(IntPtr IPDF, double PosX, double PosY, double Width, double Height, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfEncryptPDFA(IntPtr IPDF, String FileName, String OpenPwd, String OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfEncryptPDFW(IntPtr IPDF, [MarshalAs(UnmanagedType.LPWStr)]String FileName, [MarshalAs(UnmanagedType.LPStr)]String OpenPwd, [MarshalAs(UnmanagedType.LPStr)]String OwnerPwd, TKeyLen KeyLen, TRestrictions Restrict);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfEndContinueText(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfEndLayer(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfEndPage(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfEndPattern(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfEndTemplate(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfEnumDocFonts(IntPtr IPDF, IntPtr Data, TEnumFontProc2 EnumProc);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfEnumHostFonts(IntPtr IPDF, IntPtr Data, TEnumFontProc EnumProc);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfEnumHostFontsEx(IntPtr IPDF, IntPtr Data, TEnumFontProcEx EnumProc);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfExchangeBookmarks(IntPtr IPDF, int Bmk1, int Bmk2);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfExchangePages(IntPtr IPDF, int First, int Second);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfFileAttachAnnotA(IntPtr IPDF, double PosX, double PosY, TFileAttachIcon Icon, String Author, String Desc, String AFile, lbool Compress);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfFileAttachAnnotW(IntPtr IPDF, double PosX, double PosY, TFileAttachIcon Icon, String Author, String Desc, String AFile, lbool Compress);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfFileLinkA(IntPtr IPDF, double PosX, double PosY, double Width, double Height, String AFilePath);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfFileLinkW(IntPtr IPDF, double PosX, double PosY, double Width, double Height, String AFilePath);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfFindBookmarkA(IntPtr IPDF, int DestPage, String Title);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfFindBookmarkW(IntPtr IPDF, int DestPage, String Title);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfFindFieldA(IntPtr IPDF, String Name);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfFindFieldW(IntPtr IPDF, String Name);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfFinishSignature(IntPtr IPDF, byte[] PKCS7Obj, int Length);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfFindLinkAnnot(IntPtr IPDF, String URL);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfFindNextBookmark(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfFlattenAnnots(IntPtr IPDF, TAnnotFlattenFlags Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfFlattenForm(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfFlushPageContent(IntPtr IPDF, ref TPDFStack Stack);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    void   pdfFreeImageBuffer(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfFreePDF(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfFreeTextAnnotA(IntPtr IPDF, double PosX, double PosY, double Width, double Height, String Author, String AText, TTextAlign Align);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfFreeTextAnnotW(IntPtr IPDF, double PosX, double PosY, double Width, double Height, String Author, String AText, TTextAlign Align);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfFreeUniBuf(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetActionCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetActionType(IntPtr IPDF, int ActHandle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetActionTypeEx(IntPtr IPDF, TObjType ObjType, int ObjHandle, int ActIndex);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetActiveFont(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetAllocBy(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetAnnot(IntPtr IPDF, int Handle, ref TPDFAnnotation_I Annot);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetAnnotBBox(IntPtr IPDF, int Handle, ref TPDFRect BBox);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetAnnotCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetAnnotEx(IntPtr IPDF, int Handle, ref TPDFAnnotationEx_I Annot);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetAnnotFlags(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetAnnotLink(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetAnnotType(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetAscent(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetBBox(IntPtr IPDF, TPageBoundary Boundary, ref TPDFRect BBox);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetBidiMode(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetBookmark(IntPtr IPDF, int Handle, ref TBookmark_I Bmk);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetBookmarkCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetBorderStyle(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetBuffer(IntPtr IPDF, ref int BufSize);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetCapHeight(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetCharacterSpacing(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetCheckBoxChar(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetCheckBoxCharEx(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetCheckBoxDefState(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetCMap(IntPtr IPDF, int Index, ref TPDFCMap CMap);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetCMapCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetColorSpace(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetColorSpaceCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetColorSpaceObj(IntPtr IPDF, int Handle, ref TPDFColorSpaceObj_I CS);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetColorSpaceObjEx(IntPtr IColorSpace, ref TPDFColorSpaceObj_I CS);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetCompressionFilter(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetCompressionLevel(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetContent(IntPtr IPDF, ref IntPtr Buffer);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetDefBitsPerPixel(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetDescent(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetDeviceNAttributes(IntPtr IAttributes, ref TDeviceNAttributes_I Attributes);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetDocInfo(IntPtr IPDF, TDocumentInfo DInfo, ref IntPtr Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetDocInfoCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetDocInfoEx(IntPtr IPDF, int Index, ref TDocumentInfo DInfo, ref IntPtr Key, ref IntPtr Value, ref lbool Unicode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetDrawDirection(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetDynaPDFVersion();
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetEmbeddedFile(IntPtr IPDF, int Handle, ref TPDFFileSpec_I FileSpec, lbool Decompress);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetEmbeddedFileCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetEMFPatternDistance(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetErrorMessage(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetErrorMode(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetField(IntPtr IPDF, int Handle, ref TPDFField_I Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetFieldBackColor(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetFieldBorderColor(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldBorderStyle(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetFieldBorderWidth(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetFieldChoiceValue(IntPtr IPDF, int AField, int ValIndex, ref TPDFChoiceValue_I Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetFieldColor(IntPtr IPDF, int Field, TFieldColor ColorType, ref TPDFColorSpace ColorSpace, ref uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetFieldEx(IntPtr IPDF, int Handle, ref TPDFFieldEx_I Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetFieldEx2(IntPtr IField, ref TPDFFieldEx_I Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldExpValCount(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldExpValue(IntPtr IPDF, int Field, ref IntPtr Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetFieldExpValueEx(IntPtr IPDF, int Field, int ValIndex, ref IntPtr Value, ref IntPtr ExpValue, ref lbool Selected);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldFlags(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldGroupType(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldHighlightMode(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldIndex(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldMapName(IntPtr IPDF, int Field, ref IntPtr Value, ref lbool Unicode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldName(IntPtr IPDF, int Field, ref IntPtr Name);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldOrientation(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldTextAlign(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetFieldTextColor(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldToolTip(IntPtr IPDF, int Field, ref IntPtr Value, ref lbool Unicode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFieldType(IntPtr IPDF, int Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetFillColor(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFontCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetFontEx(IntPtr IPDF, int Handle, ref TPDFFontObj_I F);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFontOrigin(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    void   pdfGetFontSearchOrder(IntPtr IPDF, TFontBaseType[] Order);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFontSelMode(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetFontWeight(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetFTextHeightA(IntPtr IPDF, TTextAlign Align, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern double pdfGetFTextHeightW(IntPtr IPDF, TTextAlign Align, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetFTextHeightExA(IntPtr IPDF, double Width, TTextAlign Align, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern double pdfGetFTextHeightExW(IntPtr IPDF, double Width, TTextAlign Align, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetGStateFlags(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetIconColor(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetImageBuffer(IntPtr IPDF, ref int BufSize);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetImageCount(IntPtr IPDF, String FileName);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetImageCountEx(IntPtr IPDF, byte[] Buffer, int BufSize);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetImageHeight(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetImageWidth(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetImportFlags(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetInBBox(IntPtr IPDF, int PageNum, TPageBoundary Boundary, ref TPDFRect BBox);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetInDocInfo(IntPtr IPDF, TDocumentInfo DInfo, ref IntPtr Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetInDocInfoCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetInDocInfoEx(IntPtr IPDF, int Index, ref TDocumentInfo DInfo, ref IntPtr Key, ref IntPtr Value, ref lbool Unicode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetInEncryptionFlags(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetInIsCollection(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetInIsEncrypted(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetInIsSigned(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetInIsTrapped(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetInIsXFAForm(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetInNamedDest(IntPtr IPDF, int Index, ref TPDFNamedDest_I Dest);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetInNamedDestCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetInPageCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetInOrientation(IntPtr IPDF, int PageNum);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetInPDFVersion(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetInPrintSettings(IntPtr IPDF, ref TPDFPrintSettings_I Settings);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetIsFixedPitch(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetItalicAngle(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetJavaScript(IntPtr IPDF, int Handle, ref int Len, ref lbool Unicode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetJavaScriptAction2(IntPtr IPDF, TObjType ObjType, int ObjHandle, int ActIndex, ref int Len, ref lbool Unicode, ref TObjEvent Event);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetJavaScriptAction(IntPtr IPDF, int Handle, ref int Len, ref lbool Unicode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetJavaScriptCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetJavaScriptEx(IntPtr IPDF, String Name, ref int Len, ref lbool Unicode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetJavaScriptName(IntPtr IPDF, int Handle, ref int Len, ref lbool Unicode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetJPEGQuality(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetLanguage(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetLastTextPosX(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetLastTextPosY(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetLeading(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetLineCapStyle(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetLineJoinStyle(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetLineWidth(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetLinkHighlightMode(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetLogMetafileSize(IntPtr IPDF, String FileName, ref TRectL R);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetLogMetafileSizeEx(IntPtr IPDF, byte[] Buffer, int BufSize, ref TRectL R);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetMatrix(IntPtr IPDF, ref TCTM M);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetMaxFieldLen(IntPtr IPDF, int TxtField);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetMetaConvFlags(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfGetMissingGlyphs(IntPtr IPDF, ref int Count);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetMiterLimit(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetNeedAppearance(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetObjActionCount(IntPtr IPDF, TObjType ObjType, int ObjHandle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetOpacity(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetOrientation(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetPageAnnot(IntPtr IPDF, int Handle, ref TPDFAnnotation_I Annot);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetPageAnnotCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetPageAnnotEx(IntPtr IPDF, int Handle, ref TPDFAnnotationEx_I Annot);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetPageCoords(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetPageCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetPageField(IntPtr IPDF, int Index, ref TPDFField_I Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetPageFieldCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetPageFieldEx(IntPtr IPDF, int Index, ref TPDFFieldEx_I Field);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetPageHeight(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetPageLabel(IntPtr IPDF, int Index, ref TPDFPageLabel_I Label);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetPageLabelCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetPageLayout(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetPageMode(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetPageNum(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetPageText(IntPtr IPDF, ref TPDFStack Stack);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetPageWidth(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetPDFVersion(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetPrintSettings(IntPtr IPDF, ref TPDFPrintSettings_I Settings);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetResolution(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetSaveNewImageFormat(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetSeparationInfo(IntPtr IPDF, ref IntPtr Colorant, ref TExtColorSpace CS);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetStrokeColor(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetTabLen(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetTemplCount(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetTemplHandle(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetTemplHeight(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetTemplWidth(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetTextDrawMode(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetTextFieldValue(IntPtr IPDF, int Field, ref IntPtr Value, ref lbool ValIsUnicode, ref IntPtr DefValue, ref lbool DefValIsUnicode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetTextRect(IntPtr IPDF, ref double PosX, ref double PosY, ref double Width, ref double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetTextRise(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetTextScaling(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetTextWidthA(IntPtr IPDF, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern double pdfGetTextWidthW(IntPtr IPDF, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetTextWidthExA(IntPtr IPDF, String AText, int Len);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern double pdfGetTextWidthExW(IntPtr IPDF, String AText, int Len);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    uint   pdfGetTransparentColor(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetTrapped(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetUseExactPwd(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetUseGlobalImpFiles(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetUserRights(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetUseStdFonts(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetUseSystemFonts(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetUseTransparency(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetUseVisibleCoords(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetViewerPrefrences(IntPtr IPDF, ref int Preference, ref int AddVal);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfGetWMFDefExtent(IntPtr IPDF, ref int Width, ref int Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfGetWMFPixelPerInch(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    double pdfGetWordSpacing(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfHaveOpenDoc(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfHighlightAnnotA(IntPtr IPDF, TAnnotType SubType, double PosX, double PosY, double Width, double Height, uint Color, String Author, String Subject, String Comment);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfHighlightAnnotW(IntPtr IPDF, TAnnotType SubType, double PosX, double PosY, double Width, double Height, uint Color, String Author, String Subject, String Comment);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfImportBookmarks(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfImportCatalogObjects(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfImportDocInfo(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfImportPage(IntPtr IPDF, int PageNum);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfImportPageEx(IntPtr IPDF, int PageNum, double ScaleX, double ScaleY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfImportPDFFile(IntPtr IPDF, int DestPage, double ScaleX, double ScaleY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfInitExtGState(ref TPDFExtGState GS);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfInitStack(IntPtr IPDF, ref TPDFStack Stack);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfInsertBMPFromHandle(IntPtr IPDF, double PosX, double PosY, double ScaleWidth, double ScaleHeight, IntPtr hBitmap);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfInsertImage(IntPtr IPDF, double PosX, double PosY, double ScaleWidth, double ScaleHeight, String AFile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfInsertImageEx(IntPtr IPDF, double PosX, double PosY, double ScaleWidth, double ScaleHeight, String AFile, int Index);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfInsertImageFromBuffer(IntPtr IPDF, double PosX, double PosY, double ScaleWidth, double ScaleHeight, byte[] Buffer, int BufSize, int Index);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfInsertMetafile(IntPtr IPDF, String FileName, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfInsertMetafileEx(IntPtr IPDF, byte[] Buffer, int BufSize, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfInsertMetafileExt(IntPtr IPDF, String FileName, ref TRectL View, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfInsertMetafileExtEx(IntPtr IPDF, byte[] Buffer, int BufSize, ref TRectL View, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfInsertMetafileFromHandle(IntPtr IPDF, IntPtr hEnhMetafile, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfInsertMetafileFromHandleEx(IntPtr IPDF, IntPtr hEnhMetafile, ref TRectL View, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfInsertRawImage(IntPtr IPDF, byte[] Buffer, int BitsPerPixel, int ColorCount, int ImgWidth, int ImgHeight, double PosX, double PosY, double ScaleWidth, double ScaleHeight);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfIsBidiText(IntPtr IPDF, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfIsColorPage(IntPtr IPDF, lbool GrayIsColor);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfIsEmptyPage(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfLineTo(IntPtr IPDF, double PosX, double PosY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfLoadCMap(IntPtr IPDF, String CMapName, lbool Embed);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfLoadFDFDataA(IntPtr IPDF, String FileName, String Password, int Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfLoadFDFDataW(IntPtr IPDF, String FileName, [MarshalAs(UnmanagedType.LPStr)]String Password, int Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfLoadFDFDataEx(IntPtr IPDF, byte[] Buffer, int BufSize, String Password, int Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfLoadFont(IntPtr IPDF, byte[] Buffer, int BufSize, TFStyle Style, double Size, lbool Embed, TCodepage CP);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfMovePage(IntPtr IPDF, int Source, int Dest);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfMoveTo(IntPtr IPDF, double PosX, double PosY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfMultiplyMatrix(ref TCTM M1, ref TCTM M2, ref TCTM NewMatrix);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    IntPtr pdfNewPDF();
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfOpenImportBuffer(IntPtr IPDF, byte[] Buffer, int BufSize, TPwdType PwdType, String Password);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfOpenImportFileA(IntPtr IPDF, String FileName, TPwdType PwdType, String Password);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfOpenImportFileW(IntPtr IPDF, [MarshalAs(UnmanagedType.LPWStr)]String FileName, TPwdType PwdType, [MarshalAs(UnmanagedType.LPStr)]String Password);
      [DllImport("dynapdf.dll", CharSet=CharSet.Auto)]private static extern    lbool  pdfOpenOutputFile(IntPtr IPDF, String OutPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfOpenTagA(IntPtr IPDF, TPDFBaseTag Tag, String Lang, String AltText, String Expansion);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfOpenTagW(IntPtr IPDF, TPDFBaseTag Tag, [MarshalAs(UnmanagedType.LPStr)]String Lang, String AltText, String Expansion);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfPageLink(IntPtr IPDF, double PosX, double PosY, double Width, double Height, int DestPage);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfPageLink2(IntPtr IPDF, double PosX, double PosY, double Width, double Height, int NamedDest);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfPageLinkEx(IntPtr IPDF, double PosX, double PosY, double Width, double Height, TDestType DestType, int DestPage, double a, double b, double c, double d);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfParseContent(IntPtr IPDF, IntPtr Data, ref TPDFParseInterface Stack, TParseFlags Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfPlaceImage(IntPtr IPDF, int ImgHandle, double PosX, double PosY, double ScaleWidth, double ScaleHeight);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfPlaceSigFieldValidateIcon(IntPtr IPDF, int SigField, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfPlaceTemplate(IntPtr IPDF, int TmplHandle, double PosX, double PosY, double ScaleWidth, double ScaleHeight);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfPlaceTemplateEx(IntPtr IPDF, int TmplHandle, double PosX, double PosY, double ScaleWidth, double ScaleHeight);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfPlaceTemplByMatrix(IntPtr IPDF, int TmplHandle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfReadImageFormat2(IntPtr IPDF, String FileName, int Index, ref int Width, ref int Height, ref int BitsPerPixel, ref int UseZip);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfReadImageFormatEx(IntPtr IPDF, IntPtr hBitmap, ref int Width, ref int Height, ref int BitsPerPixel, ref int UseZip);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfReadImageFormatFromBuffer(IntPtr IPDF, byte[] Buffer, int BufSize, int Index, ref int Width, ref int Height, ref int BitsPerPixel, ref int UseZip);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfReadImageResolution(IntPtr IPDF, String FileName, int Index, ref int ResX, ref int ResY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfReadImageResolutionEx(IntPtr IPDF, byte[] Buffer, int BufSize, int Index, ref int ResX, ref int ResY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfRectangle(IntPtr IPDF, double PosX, double PosY, double Width, double Height, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfReEncryptPDFA(IntPtr IPDF, String FileName, TPwdType PwdType, String InPwd, String NewOpenPwd, String NewOwnerPwd, TKeyLen NewKeyLen, TRestrictions Restrict);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfReEncryptPDFW(IntPtr IPDF, [MarshalAs(UnmanagedType.LPWStr)]String FileName, TPwdType PwdType, [MarshalAs(UnmanagedType.LPStr)]String InPwd, [MarshalAs(UnmanagedType.LPStr)]String NewOpenPwd, [MarshalAs(UnmanagedType.LPStr)]String NewOwnerPwd, TKeyLen NewKeyLen, TRestrictions Restrict);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfRenameSpotColor(IntPtr IPDF, String Colorant, String NewName);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfReplaceFontA(IntPtr IPDF, IntPtr PDFFont, String Name, TFStyle Style, lbool Embed);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfReplaceFontW(IntPtr IPDF, IntPtr PDFFont, String Name, TFStyle Style, lbool Embed);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfReplaceFontExA(IntPtr IPDF, IntPtr PDFFont, String FontFile, lbool Embed);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfReplaceFontExW(IntPtr IPDF, IntPtr PDFFont, String FontFile, lbool Embed);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfReplaceICCProfileA(IntPtr IPDF, int ColorSpace, String ICCFile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfReplaceICCProfileW(IntPtr IPDF, int ColorSpace, String ICCFile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfReplacePageTextA(IntPtr IPDF, String NewText, ref TPDFStack Stack);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfReplacePageTextExA(IntPtr IPDF, String NewText, ref TPDFStack Stack);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfReplacePageTextExW(IntPtr IPDF, String NewText, ref TPDFStack Stack);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfResetLineDashPattern(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfRestoreGraphicState(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfRotateCoords(IntPtr IPDF, double alpha, double OriginX, double OriginY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfRoundRect(IntPtr IPDF, double PosX, double PosY, double Width, double Height, double Radius, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfRoundRectEx(IntPtr IPDF, double PosX, double PosY, double Width, double Height, double rWidth, double rHeight, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSaveGraphicState(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfScaleCoords(IntPtr IPDF, double sx, double sy);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSelfTest(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSet3DAnnotProps(IntPtr IPDF, int Annot, T3DActivationType ActType, T3DDeActivateType DeActType, T3DInstanceType InstType, T3DDeActInstance DeInstType, lbool DisplToolbar, lbool DisplModelTree);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSet3DAnnotScriptA(IntPtr IPDF, int Annot, String Value, int Len);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfSetAllocBy(IntPtr IPDF, int Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotBorderStyle(IntPtr IPDF, int Handle, TBorderStyle Style);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotBorderWidth(IntPtr IPDF, int Handle, double LineWidth);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotColor(IntPtr IPDF, int Handle, TAnnotColor ColorType, TPDFColorSpace CS, uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotFlags(IntPtr IPDF, TAnnotFlags Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotFlagsEx(IntPtr IPDF, int Handle, int Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotHighlightMode(IntPtr IPDF, int Handle, THighlightMode Mode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotIcon(IntPtr IPDF, int Handle, TAnnotIcon Icon);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotOpenState(IntPtr IPDF, int Handle, bool Open);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotStringA(IntPtr IPDF, int Handle, TAnnotString StringType, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfSetAnnotStringW(IntPtr IPDF, int Handle, TAnnotString StringType, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetAnnotSubjectA(IntPtr IPDF, int Handle, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfSetAnnotSubjectW(IntPtr IPDF, int Handle, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetBBox(IntPtr IPDF, TPageBoundary Boundary, double LeftX, double LeftY, double RightX, double RightY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetBidiMode(IntPtr IPDF, TPDFBidiMode Mode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetBookmarkDest(IntPtr IPDF, int ABmk, TDestType DestType, double a, double b, double c, double d);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetBookmarkStyle(IntPtr IPDF, int ABmk, TFStyle Style, uint RGBColor);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetBorderStyle(IntPtr IPDF, TBorderStyle Style);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetCharacterSpacing(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetCheckBoxChar(IntPtr IPDF, TCheckBoxChar CheckBoxChar);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetCheckBoxDefState(IntPtr IPDF, int Field, lbool Checked);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetCheckBoxState(IntPtr IPDF, int Field, lbool Checked);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfSetCIDFontA(IntPtr IPDF, int CMapHandle, String Name, TFStyle Style, double Size, lbool Embed);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfSetCIDFontW(IntPtr IPDF, int CMapHandle, String Name, TFStyle Style, double Size, lbool Embed);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfSetCMapDirA(IntPtr IPDF, String Path, lbool Recursive);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfSetCMapDirW(IntPtr IPDF, String Path, lbool Recursive);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetColDefFile(IntPtr IPDF, int EmbFile);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetColors(IntPtr IPDF, uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetColorSpace(IntPtr IPDF, TPDFColorSpace ColorSpace);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetColSortField(IntPtr IPDF, int ColField, lbool AscendingOrder);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetCompressionFilter(IntPtr IPDF, TCompressionFilter Filter);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetCompressionLevel(IntPtr IPDF, TCompressionLevel CompressLevel);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetContent(IntPtr IPDF, String Buffer, int BufSize);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetDateTimeFormat(IntPtr IPDF, int TxtField, TPDFDateTime Fmt);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetDefBitsPerPixel(IntPtr IPDF, int Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetDocInfoA(IntPtr IPDF, TDocumentInfo DInfo, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfSetDocInfoW(IntPtr IPDF, TDocumentInfo DInfo, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetDocInfoExA(IntPtr IPDF, TDocumentInfo DInfo, String Key, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfSetDocInfoExW(IntPtr IPDF, TDocumentInfo DInfo, [MarshalAs(UnmanagedType.LPStr)]String Key, [MarshalAs(UnmanagedType.LPWStr)]String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetDrawDirection(IntPtr IPDF, TDrawDirection Direction);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetEMFFrameDPI(IntPtr IPDF, int DPIX, int DPIY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetEMFPatternDistance(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetErrorMode(IntPtr IPDF, TErrMode ErrMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetExtColorSpace(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetExtGState(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldBBox(IntPtr IPDF, int Field, ref TPDFRect BBox);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldBackColor(IntPtr IPDF, uint AColor);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldBorderColor(IntPtr IPDF, uint AColor);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldBorderWidth(IntPtr IPDF, int Field, double LineWidth);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldColor(IntPtr IPDF, int Field, TFieldColor ColorType, TPDFColorSpace CS, uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldBorderStyle(IntPtr IPDF, int Field, TBorderStyle Style);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldExpValue(IntPtr IPDF, int Field, int ValIndex, String Value, String ExpValue, lbool Selected);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldFlags(IntPtr IPDF, int Field, TFieldFlags Flags, lbool Reset);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldFontSize(IntPtr IPDF, int Field, double FontSize);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldHighlightMode(IntPtr IPDF, int Field, THighlightMode Mode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldIndex(IntPtr IPDF, int Field, int Index);
      [DllImport("dynapdf.dll", CharSet=CharSet.Auto)]private static extern    lbool  pdfSetFieldMapName(IntPtr IPDF, int Field, String Name);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldName(IntPtr IPDF, int Field, String NewName);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldOrientation(IntPtr IPDF, int Field, int Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldTextAlign(IntPtr IPDF, int Field, TTextAlign Align);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldTextColor(IntPtr IPDF, uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFieldToolTipA(IntPtr IPDF, int Field, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfSetFieldToolTipW(IntPtr IPDF, int Field, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFillColor(IntPtr IPDF, uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFillColorEx(IntPtr IPDF, byte[] Color, int NumComponents);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFillColorSpace(IntPtr IPDF, TPDFColorSpace CS);
      [DllImport("dynapdf.dll", CharSet=CharSet.Auto)]private static extern    int    pdfSetFont(IntPtr IPDF, String Name, TFStyle Style, double Size, lbool Embed, TCodepage CP);
      [DllImport("dynapdf.dll", CharSet=CharSet.Auto)]private static extern    int    pdfSetFontEx(IntPtr IPDF, String Name, TFStyle Style, double Size, lbool Embed, TCodepage CP);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFontOrigin(IntPtr IPDF, TOrigin Origin);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    void   pdfSetFontSearchOrder(IntPtr IPDF, TFontBaseType[] Order);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFontSelMode(IntPtr IPDF, TFontSelMode Mode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetFontWeight(IntPtr IPDF, int Weight);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    void   pdfSetGStateFlags(IntPtr IPDF, TGStateFlags Flags, lbool Reset);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetIconColor(IntPtr IPDF, uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetImportFlags(IntPtr IPDF, TImportFlags Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetImportFlags2(IntPtr IPDF, TImportFlags2 Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetItalicAngle(IntPtr IPDF, double Angle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetJPEGQuality(IntPtr IPDF, int Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetLanguage(IntPtr IPDF, String ISOTag);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetLeading(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetLicenseKey(IntPtr IPDF, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetLineCapStyle(IntPtr IPDF, TLineCapStyle Style);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetLineDashPatternEx(IntPtr IPDF, ref double[] Dash, int NumValues, int Phase);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetLineJoinStyle(IntPtr IPDF, TLineJoinStyle Style);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetLineWidth(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetLinkHighlightMode(IntPtr IPDF, THighlightMode Mode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetListFont(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetMatrix(IntPtr IPDF, ref TCTM Matrix);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetMaxFieldLen(IntPtr IPDF, int TxtField, int MaxLen);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetMetaConvFlags(IntPtr IPDF, TMetaFlags Flags);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetMiterLimit(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetNeedAppearance(IntPtr IPDF, lbool Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetNumberFormat(IntPtr IPDF, int TxtField, TDecSeparator Sep, int DecPlaces, TNegativeStyle NegStyle, String CurrStr, lbool Prepend);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetOnErrorProc(IntPtr IPDF, IntPtr Data, TErrorProc ErrProc);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetOnPageBreakProc(IntPtr IPDF, IntPtr Data, TOnPageBreakProc OnBreakProc);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetOpacity(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetOrientation(IntPtr IPDF, int Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetOrientationEx(IntPtr IPDF, int Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetPageCoords(IntPtr IPDF, TPageCoord PageCoords);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetPageFormat(IntPtr IPDF, TPageFormat Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetPageHeight(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetPageLayout(IntPtr IPDF, TPageLayout Layout);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetPageMode(IntPtr IPDF, TPageMode Mode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetPageWidth(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetPDFVersion(IntPtr IPDF, TPDFVersion Version);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetPrintSettings(IntPtr IPDF, TDuplexMode Mode, int PickTrayByPDFSize, int NumCopies, TPrintScaling PrintScaling, uint[] PrintRanges, int NumRanges);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetProgressProc(IntPtr IPDF, IntPtr Data, TInitProgressProc InitProgress, TProgressProc Progress);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetResolution(IntPtr IPDF, int Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetSaveNewImageFormat(IntPtr IPDF, lbool Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetSeparationInfo(IntPtr IPDF, int Handle);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetStrokeColor(IntPtr IPDF, uint Color);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetStrokeColorEx(IntPtr IPDF, byte[] Color, int NumComponents);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetStrokeColorSpace(IntPtr IPDF, TPDFColorSpace CS);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetTabLen(IntPtr IPDF, int TabLen);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetTextDrawMode(IntPtr IPDF, TDrawMode Mode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetTextFieldValueA(IntPtr IPDF, int Field, String Value, String DefValue, TTextAlign Align);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfSetTextFieldValueW(IntPtr IPDF, int Field, String Value, String DefValue, TTextAlign Align);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetTextFieldValueExA(IntPtr IPDF, int Field, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfSetTextFieldValueExW(IntPtr IPDF, int Field, String Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetTextRect(IntPtr IPDF, double PosX, double PosY, double Width, double Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetTextRise(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetTextScaling(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetTransparentColor(IntPtr IPDF, uint AColor);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    void   pdfSetTrapped(IntPtr IPDF, lbool Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetUseExactPwd(IntPtr IPDF, lbool Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetUseGlobalImpFiles(IntPtr IPDF, lbool Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetUseStdFonts(IntPtr IPDF, lbool Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetUseSwapFile(IntPtr IPDF, lbool SwapContents, int SwapLimit);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetUseSwapFileEx(IntPtr IPDF, lbool SwapContents, int SwapLimit, String SwapDir);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetUseSystemFonts(IntPtr IPDF, lbool Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetUseTransparency(IntPtr IPDF, lbool Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetUseVisibleCoords(IntPtr IPDF, lbool Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetViewerPreferences(IntPtr IPDF, TViewerPreference Value, TViewPrefAddVal AddVal);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetWMFDefExtent(IntPtr IPDF, int Width, int Height);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetWMFPixelPerInch(IntPtr IPDF, int Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSetWordSpacing(IntPtr IPDF, double Value);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSkewCoords(IntPtr IPDF, double alpha, double beta, double OriginX, double OriginY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSortFieldsByIndex(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfSortFieldsByName(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfStampAnnotA(IntPtr IPDF, TRubberStamp SubType, double PosX, double PosY, double Width, double Height, String Author, String Subject, String Comment);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfStampAnnotW(IntPtr IPDF, TRubberStamp SubType, double PosX, double PosY, double Width, double Height, String Author, String Subject, String Comment);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfStrLenA(IntPtr AStr);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfStrLenW(IntPtr AStr);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfStrokePath(IntPtr IPDF);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfTextAnnotA(IntPtr IPDF, double PosX, double PosY, double Width, double Height, String Author, String Text, TAnnotIcon Icon, lbool Open);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfTextAnnotW(IntPtr IPDF, double PosX, double PosY, double Width, double Height, String Author, String Text, TAnnotIcon Icon, lbool Open);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfTranslateCoords(IntPtr IPDF, double OriginX, double OriginY);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfTriangle(IntPtr IPDF, double x1, double y1, double x2, double y2, double x3, double y3, TPathFillMode FillMode);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    int    pdfWebLinkA(IntPtr IPDF, double PosX, double PosY, double Width, double Height, String URL);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern int    pdfWebLinkW(IntPtr IPDF, double PosX, double PosY, double Width, double Height, String URL);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfWriteAngleTextA(IntPtr IPDF, String AText, double Angle, double PosX, double PosY, double Radius, double YOrigin);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfWriteAngleTextW(IntPtr IPDF, String AText, double Angle, double PosX, double PosY, double Radius, double YOrigin);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfWriteFTextA(IntPtr IPDF, TTextAlign Align, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfWriteFTextW(IntPtr IPDF, TTextAlign Align, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfWriteFTextExA(IntPtr IPDF, double PosX, double PosY, double Width, double Height, TTextAlign Align, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfWriteFTextExW(IntPtr IPDF, double PosX, double PosY, double Width, double Height, TTextAlign Align, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfWriteTextA(IntPtr IPDF, double PosX, double PosY, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfWriteTextW(IntPtr IPDF, double PosX, double PosY, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfWriteTextExA(IntPtr IPDF, double PosX, double PosY, String AText, int Len);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfWriteTextExW(IntPtr IPDF, double PosX, double PosY, String AText, int Len);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfWriteTextMatrixA(IntPtr IPDF, ref TCTM M, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfWriteTextMatrixW(IntPtr IPDF, ref TCTM M, String AText);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi)]private static extern    lbool  pdfWriteTextMatrixExA(IntPtr IPDF, ref TCTM M, String AText, int Len);
      [DllImport("dynapdf.dll", CharSet=CharSet.Unicode)]private static extern lbool  pdfWriteTextMatrixExW(IntPtr IPDF, ref TCTM M, String AText, int Len);

      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi, EntryPoint="pdfCopyMem")]private static extern int pdfCopyMemUInt(IntPtr Source, uint[] Dest, int Len);
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi, EntryPoint="pdfCopyMem")]private static extern int pdfCopyMemIntPtr(IntPtr Source, IntPtr[] Dest, int Len);
      // Helper function to copy a kerning record into the managed type TTextRecordW.
      [DllImport("dynapdf.dll", CharSet=CharSet.Ansi, EntryPoint="pdfCopyMem")]public static extern int CopyKernRecord(IntPtr Source, ref TTextRecordW Record, int Len);
   }
}
